import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.LineNumberReader;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Stack;
import java.util.StringTokenizer;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.ScheduledFuture;

import L2jFT.Config;
import L2jFT.extensions.scripts.Functions;
import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.ThreadPoolManager;
import L2jFT.game.cache.Msg;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.network.serverpackets.Ride;

public class Transport extends Functions implements ScriptFile
{
	public static L2Object self;
	public static L2Object npc;

	public void onLoad()
	{
		loadWyvernPath();
		_riders = new ConcurrentLinkedQueue<Rider>();
		thread = ThreadPoolManager.getInstance().scheduleGeneralAtFixedRate(new MoveWyverns(), 10, 10);
		System.out.println("Transport Loaded");
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	private void loadWyvernPath()
	{
		LineNumberReader lnr = null;
		wyverns = new HashMap<String, Wyvern>();
		try
		{
			File wyvernData = new File(Config.DATAPACK_ROOT, "data/wyvernpath.csv");
			lnr = new LineNumberReader(new BufferedReader(new FileReader(wyvernData)));
			String line = null;
			while((line = lnr.readLine()) != null)
			{
				if(line.trim().length() == 0 || line.startsWith("#"))
					continue;
				Wyvern W = new Wyvern();
				W.parseLine(line);
				wyverns.put(W.name, W);
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			try
			{
				if(lnr != null)
					lnr.close();
			}
			catch(Exception e1)
			{ /* ignore problems */}
		}
	}

	public static HashMap<String, Wyvern> wyverns;
	public static ConcurrentLinkedQueue<Rider> _riders;
	public static ScheduledFuture thread;

	public class MoveWyverns extends Thread
	{
		@Override
		public void run()
		{
			try
			{
				for(Rider r : _riders)
					if(!r.P.isMoving)
						if(!r.way.empty())
						{// летим в следующую точку
							Integer[] next = r.way.firstElement();
							r.way.remove(0);
							r.P.moveToLocation(next[0], next[1], next[2], 0, false);
						}
						else
						{// прилетели
							r.P.broadcastPacket(new Ride(r.P, Ride.ACTION_DISMOUNT, 0));
							r.P.setMountType(0);
							r.P.setIsInvul(false);
							r.P.unblock();
							_riders.remove(r);
						}
			}
			catch(Exception e)
			{
				System.out.println("Wyvern moving thread problem: " + e.getMessage());
				e.printStackTrace();
			}
		}
	}

	public class Wyvern
	{
		public ArrayList<Integer[]> path;
		public String name;

		public void parseLine(String line)
		{
			path = new ArrayList<Integer[]>();
			StringTokenizer st = new StringTokenizer(line, " ");
			name = st.nextToken();
			while(st.hasMoreTokens())
			{
				StringTokenizer points = new StringTokenizer(st.nextToken(), ";");
				Integer[] point = {
						Integer.parseInt(points.nextToken()),
						Integer.parseInt(points.nextToken()),
						Integer.parseInt(points.nextToken()) };
				path.add(point);
			}
		}
	}

	public class Rider
	{
		public Wyvern W;
		public L2Player P;
		public Stack<Integer[]> way;
	}

	public void HireWyvern(String[] param)
	{
		if(param.length < 2)
			throw new IllegalArgumentException();

		//loadWyvernPath();

		L2Player player = (L2Player) self;

		int price = Integer.parseInt(param[1]);

		int day = Calendar.getInstance().get(Calendar.DAY_OF_WEEK);
		int hour = Calendar.getInstance().get(Calendar.HOUR_OF_DAY);
		if(day != 1 && day != 7 && (hour <= 12 || hour >= 22))
			price /= 2;

		if(player.isActionsDisabled() || player.isSitting() || player.isMounted() || player.getLastNpc().getDistance(player) > 300)
			return;

		if(player.getAdena() < price)
		{
			player.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ADENA);
			return;
		}

		if(price > 0)
			player.reduceAdena(price);

		Wyvern W = wyverns.get(param[0]);

		Ride ride = new Ride(player, Ride.ACTION_MOUNT, 12621);
		player.broadcastPacket(ride);
		player.setMountType(2);
		player.block();
		player.setIsInvul(true);

		Rider r = new Rider();
		r.P = player;
		r.W = W;
		r.way = new Stack<Integer[]>();
		r.way.addAll(W.path);
		_riders.add(r);
	}
}