import L2jFT.extensions.scripts.Functions;
import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.cache.Msg;
import L2jFT.game.geodata.GeoData;
import L2jFT.game.instancemanager.TownManager;
import L2jFT.game.instancemanager.ZoneManager;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Zone;
import L2jFT.game.model.L2Zone.ZoneType;
import L2jFT.game.model.entity.Castle;
import L2jFT.game.model.entity.SevenSigns;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.util.CharPosition;
import L2jFT.util.Rnd;

import java.util.Calendar;

public class Util extends Functions implements ScriptFile
{
	public static L2Object self;
	public static L2Object npc;

	public void onLoad()
	{
		System.out.println("Utilites Loaded");
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	/**
	 * Перемещает за плату в аденах
	 * 
	 * @param x
	 * @param y
	 * @param z
	 * @param price
	 */
	public void Gatekeeper(String[] param)
	{
		if(param.length < 4)
			throw new IllegalArgumentException();

		L2Player player = (L2Player) self;

		int price = Integer.parseInt(param[3]);

		int day = Calendar.getInstance().get(Calendar.DAY_OF_WEEK);
		int hour = Calendar.getInstance().get(Calendar.HOUR_OF_DAY);
		if(day != 1 && day != 7 && (hour <= 12 || hour >= 22))
			price /= 2;

		if(player.isActionsDisabled() || player.isSitting() || player.getLastNpc().getDistance(player) > 300)
			return;

		if(player.getAdena() < price)
		{
			player.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ADENA);
			return;
		}

		int x = Integer.parseInt(param[0]);
		int y = Integer.parseInt(param[1]);
		int z = Integer.parseInt(param[2]);

		// Нельзя телепортироваться в города, где идет осада
		// Узнаем, идет ли осада в ближайшем замке к точке телепортации
		Castle castle = TownManager.getInstance().getClosestTown(x, y).getCastle();
		if(castle != null && castle.getSiege().isInProgress())
		{
			// Определяем, в город ли телепортируется чар
			boolean teleToTown = false;
			int townId = 0;
			for(L2Zone town : ZoneManager.getInstance().getZoneByType(ZoneType.Town))
				if(town.checkIfInZone(x, y))
				{
					teleToTown = true;
					townId = town.getIndex();
					break;
				}

			if(teleToTown && townId == castle.getTown())
			{
				player.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_TELEPORT_TO_A_VILLAGE_THAT_IS_IN_A_SIEGE));
				return;
			}
		}

		// TODO: вынести это в отдельную функцию
		CharPosition pos = new CharPosition(0, 0, 0, 0);
		for(int i = 0; i < 100; i++)
		{
			pos = Rnd.coordsRandomize(x, y, z, 0, 50, 100);
			if(GeoData.getInstance().canMoveToCoord(pos.x, pos.y, pos.z, x, y, z))
				break;
		}
		if(pos.x == 0)
		{
			pos.x = x;
			pos.y = y;
			pos.z = z;
		}

		if(price > 0)
			player.reduceAdena(price);
		player.teleToLocation(pos.x, pos.y, pos.z);
	}

	public void SSGatekeeper(String[] param)
	{
		if(param.length < 4)
			throw new IllegalArgumentException();

		L2Player player = (L2Player) self;
		int type = Integer.parseInt(param[3]);

		if(player.isActionsDisabled() || player.isSitting() || player.getLastNpc().getDistance(player) > 300)
			return;

		if(SevenSigns.getInstance().getCurrentPeriod() == SevenSigns.PERIOD_COMPETITION && SevenSigns.getInstance().getPlayerCabal(player) == SevenSigns.CABAL_NULL)
			return;

		if(SevenSigns.getInstance().getCurrentPeriod() == SevenSigns.PERIOD_SEAL_VALIDATION)
		{
			if(type == 1 && SevenSigns.getInstance().getSealOwner(SevenSigns.SEAL_AVARICE) == SevenSigns.CABAL_DAWN && SevenSigns.getInstance().getPlayerCabal(player) != SevenSigns.CABAL_DAWN)
				return;
			if(type == 2 && SevenSigns.getInstance().getSealOwner(SevenSigns.SEAL_GNOSIS) == SevenSigns.CABAL_DUSK && SevenSigns.getInstance().getPlayerCabal(player) != SevenSigns.CABAL_DUSK)
				return;
		}

		int x = Integer.parseInt(param[0]);
		int y = Integer.parseInt(param[1]);
		int z = Integer.parseInt(param[2]);
		CharPosition pos = Rnd.coordsRandomize(x, y, z, 0, 20, 70);

		player.teleToLocation(pos.x, pos.y, pos.z);
	}

	/**
	 * Перемещает за определенный предмет
	 * 
	 * @param x
	 * @param y
	 * @param z
	 * @param count
	 * @param item
	 */
	public void QuestGatekeeper(String[] param)
	{
		if(param.length < 5)
			throw new IllegalArgumentException();

		L2Player player = (L2Player) self;

		int count = Integer.parseInt(param[3]);
		int item = Integer.parseInt(param[4]);
		L2ItemInstance ii = player.getInventory().getItemByItemId(item);

		if(player.isActionsDisabled() || player.isSitting() || player.getLastNpc().getDistance(player) > 300)
			return;

		if(ii == null || ii.getCount() < count)
		{
			player.sendPacket(new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_ENOUGH_REQUIRED_ITEMS));
			return;
		}

		int x = Integer.parseInt(param[0]);
		int y = Integer.parseInt(param[1]);
		int z = Integer.parseInt(param[2]);
		CharPosition pos = Rnd.coordsRandomize(x, y, z, 0, 20, 70);

		player.getInventory().destroyItem(ii, count, true, true);

		if(count > 1)
			player.sendPacket(new SystemMessage(SystemMessage.S2_S1_HAS_DISAPPEARED).addItemName(item).addNumber(count));
		else
			player.sendPacket(new SystemMessage(SystemMessage.S1_HAS_DISAPPEARED).addItemName(item));

		player.teleToLocation(pos.x, pos.y, pos.z);
	}

	/**
	 * Используется для телепортации за Newbie Token, проверяет уровень и передает
	 * параметры в QuestGatekeeper
	 */
	public void TokenJump(String[] param)
	{
		L2Player player = (L2Player) self;
		if(player.getLevel() <= 19)
			QuestGatekeeper(param);
		else
			show("Only for newbies", player);
	}

	public void NoblessTeleport()
	{
		L2Player player = (L2Player) self;
		if(player.isNoble())
			show("data/scripts/noble.htm", player);
		else
			show("data/scripts/nobleteleporter-no.htm", player);
	}
}