package ai;

import L2jFT.Config;
import L2jFT.game.ThreadPoolManager;
import L2jFT.game.ai.Fighter;
import L2jFT.game.geodata.GeoData;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Spawn;
import L2jFT.game.network.serverpackets.MagicSkillUse;
import L2jFT.game.tables.TerritoryTable;
import L2jFT.util.Location;
import L2jFT.util.Rnd;

/**
 * Моб использует телепортацию вместо рандом валка.
 * @author SYS
 */
public class RndTeleportFighter extends Fighter
{
	private long _lastTeleport;

	public RndTeleportFighter(L2Character actor)
	{
		super(actor);
	}

	@Override
	protected boolean maybeMoveToHome()
	{
		if(System.currentTimeMillis() - _lastTeleport < 10000)
			return false;

		boolean randomWalk = _actor.hasRandomWalk();
		if(_thisActor.getSpawnedX() == 0)
			return false;

		// Random walk or not?
		if(randomWalk && (!Config.RND_WALK || Rnd.chance(Config.RND_WALK_RATE)))
			return false;

		int spawnX = _thisActor.getSpawnedX();
		int spawnY = _thisActor.getSpawnedY();
		int spawnZ = _thisActor.getSpawnedZ();

		if(!randomWalk && _thisActor.isInRangeZ(spawnX, spawnY, spawnZ, Config.MAX_DRIFT_RANGE))
			return false;

		int x = spawnX + Rnd.get(2 * Config.MAX_DRIFT_RANGE) - Config.MAX_DRIFT_RANGE;
		int y = spawnY + Rnd.get(2 * Config.MAX_DRIFT_RANGE) - Config.MAX_DRIFT_RANGE;
		int z = GeoData.getInstance().getHeight(x, y, spawnZ);

		L2Spawn spawn = _thisActor.getSpawn();
		if(spawn != null && spawn.getLocation() != 0 && !TerritoryTable.getInstance().getLocation(spawn.getLocation()).isInside(x, y))
			return false;

		_thisActor.broadcastPacketToOthers(new MagicSkillUse(_thisActor, _thisActor, 4671, 1, 500, 0));
		ThreadPoolManager.getInstance().scheduleAi(new Teleport(new Location(x, y, z)), 500);
		_lastTeleport = System.currentTimeMillis();

		return true;
	}
}