package commands.admin;

import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.Announcements;
import L2jFT.game.handler.AdminCommandHandler;
import L2jFT.game.handler.IAdminCommandHandler;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;

/**
 * This class handles following admin commands: - announce text = announces text
 * to all players - list_announcements = show menu - reload_announcements =
 * reloads announcements from txt file - announce_announcements = announce all
 * stored announcements to all players - add_announcement text = adds text to
 * startup announcements - del_announcement id = deletes announcement with
 * respective id
 */
public class AdminAnnouncements implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = {
			"admin_list_announcements", "admin_announce_announcements",
			"admin_add_announcement", "admin_del_announcement", "admin_announce",
			"admin_a", "admin_announce_menu" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().CanAnnounce)
			return false;

		if(command.equals("admin_list_announcements"))
			Announcements.getInstance().listAnnouncements(activeChar);
		else if(command.startsWith("admin_announce_menu"))
		{
			Announcements sys = new Announcements();
			sys.handleAnnounce(command, 20);
			Announcements.getInstance().listAnnouncements(activeChar);
		}
		else if(command.equals("admin_announce_announcements"))
		{
			for(L2Player player : L2World.getAllPlayers())
				Announcements.getInstance().showAnnouncements(player);
			Announcements.getInstance().listAnnouncements(activeChar);
		}
		else if(command.startsWith("admin_add_announcement"))
		{
			// FIXME the player can send only 16 chars (if you try to send more it
			// sends null), remove this function or not?
			if(!command.equals("admin_add_announcement"))
				try
				{
					String val = command.substring(23);
					Announcements.getInstance().addAnnouncement(val);
					Announcements.getInstance().listAnnouncements(activeChar);
				}
				catch(StringIndexOutOfBoundsException e)
				{}
		}
		else if(command.startsWith("admin_del_announcement"))
			try
			{
				int val = new Integer(command.substring(23)).intValue();
				Announcements.getInstance().delAnnouncement(val);
				Announcements.getInstance().listAnnouncements(activeChar);
			}
			catch(StringIndexOutOfBoundsException e)
			{}

		// Command is admin announce
		else if(command.startsWith("admin_announce"))
		{
			// Call method from another class
			Announcements sys = new Announcements();
			sys.handleAnnounce(command, 15);
		}
		else if(command.startsWith("admin_a "))
		{
			// Call method from another class
			Announcements sys = new Announcements();
			sys.handleAnnounce(command, 8);
		}

		return true;
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] {
				"file://com_list_announcements.htm",
				"file://com_announce_announcements.htm",
				"file://com_add_announcement.htm", "file://com_del_announcement.htm",
				"file://com_announce.htm", "file://com_a.htm",
				"file://com_announce_menu.htm" };

	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_MANAGESERVER,
				AdminCommandHandler.GROUP_MANAGESERVER,
				AdminCommandHandler.GROUP_MANAGESERVER,
				AdminCommandHandler.GROUP_MANAGESERVER,
				AdminCommandHandler.GROUP_MANAGESERVER,
				AdminCommandHandler.GROUP_MANAGESERVER,
				AdminCommandHandler.GROUP_MANAGESERVER };
	}
}
