package commands.admin;

import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.cache.Msg;
import L2jFT.game.handler.AdminCommandHandler;
import L2jFT.game.handler.IAdminCommandHandler;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Effect;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;
import L2jFT.game.network.serverpackets.Revive;
import L2jFT.game.network.serverpackets.SocialAction;
import L2jFT.util.Log;

import jsr166x.ConcurrentSkipListSet;

public class AdminCancel implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = { "admin_cancel" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().CanEditChar)
			return false;
		if(command.startsWith("admin_cancel "))
			handleCancel(activeChar, command.split(" ")[1]);
		if(command.equals("admin_cancel"))
			handleCancel(activeChar);
		return true;
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	private void handleCancel(L2Player activeChar)
	{
		handleCancel(activeChar, null);
	}

	private void handleCancel(L2Player activeChar, String player)
	{
		L2Object obj = activeChar.getTarget();
		if(player != null)
		{
			L2Player plyr = L2World.getPlayer(player);
			if(plyr != null)
				obj = plyr;
			else
				try
				{
					int radius = Math.max(Integer.parseInt(player), 100);
					for(L2Character character : activeChar.getKnownCharacters(radius))
					{
						ConcurrentSkipListSet<L2Effect> effects = character.getAllEffects();

						for(L2Effect e : effects)
							e.exit();

						character.broadcastPacket(new SocialAction(character.getObjectId(), 15));
						character.broadcastPacket(new Revive(character));
					}
					activeChar.sendMessage("Apply Cancel within " + radius + " unit radius.");
					return;
				}
				catch(NumberFormatException e)
				{
					activeChar.sendMessage("Enter valid player name or radius");
					return;
				}
		}

		if(obj == null)
			obj = activeChar;
		if(obj.isCharacter)
		{
			L2Character target = (L2Character) obj;
			ConcurrentSkipListSet<L2Effect> effects = target.getAllEffects();

			for(L2Effect e : effects)
				e.exit();

			target.broadcastPacket(new SocialAction(target.getObjectId(), 15));
			target.broadcastPacket(new Revive(target));
			
			Log.LogCommand(activeChar, Log.Adm_DelSkill, "admin_cancel", 1);
		}
		else
			activeChar.sendPacket(Msg.INCORRECT_TARGET);
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] { "file://com_cancel.htm" };
	}

	public String[] getCommandGroup()
	{
		return new String[] { AdminCommandHandler.GROUP_EDITCHAR };
	}
}