package commands.admin;

import java.util.StringTokenizer;

import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.handler.AdminCommandHandler;
import L2jFT.game.handler.IAdminCommandHandler;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.network.serverpackets.ItemList;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.ItemTable;
import L2jFT.util.Log;

/**
 * This class handles following admin commands: - itemcreate = show menu -
 * create_item id [num] = creates num items with respective id, if num is not
 * specified num = 1
 */
public class AdminCreateItem implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = {
			"admin_itemcreate", "admin_create_item" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().UseGMShop)
			return false;

		if(command.equals("admin_itemcreate"))
			AdminHelpPage.showHelpPage(activeChar, "itemcreation.htm");
		else if(command.startsWith("admin_create_item"))
		{
			try
			{
				String val = command.substring(17);
				StringTokenizer st = new StringTokenizer(val);
				if(st.countTokens() == 2)
				{
					String id = st.nextToken();
					short idval = Short.parseShort(id);
					String num = st.nextToken();
					int numval = Integer.parseInt(num);
					createItem(activeChar, idval, numval);
				}
				else if(st.countTokens() == 1)
				{
					String id = st.nextToken();
					short idval = Short.parseShort(id);
					createItem(activeChar, idval, 1);
				}
			}
			catch(NumberFormatException nfe)
			{
				activeChar.sendMessage("Specify a valid number.");
			}
			catch(StringIndexOutOfBoundsException e)
			{
				activeChar.sendMessage("Can't create this item.", "Невозможно создать этот предмет.");
			}
			AdminHelpPage.showHelpPage(activeChar, "itemcreation.htm");
		}
		return true;
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	private void createItem(L2Player activeChar, short id, int num)
	{
		L2ItemInstance createditem = ItemTable.getInstance().createItem(id, activeChar.getObjectId(), 0, "AdminCreateItem.createItem[74]");
		createditem.setCount(num);
		activeChar.getInventory().addItem(createditem);
		Log.LogItem(activeChar, Log.Adm_AddItem, createditem);
		activeChar.sendPacket(new ItemList(activeChar, true));
		if(id == 57)
			activeChar.sendPacket(new SystemMessage(SystemMessage.EARNED_S1_ADENA).addNumber(num));
		else if(num > 1)
			activeChar.sendPacket(new SystemMessage(SystemMessage.EARNED_S2_S1_s).addItemName(id).addNumber(num));
		else
			activeChar.sendPacket(new SystemMessage(SystemMessage.EARNED_S1).addItemName(id));

		Log.add("spawned " + num + " item(s) number " + id + " in inventory", "gm_ext_actions", activeChar);
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] {
				"file://com_itemcreate.htm", "file://com_create_item.htm" };
	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_MANAGESERVER1,
				AdminCommandHandler.GROUP_MANAGESERVER1 };
	}
}