package commands.admin;

import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.ThreadPoolManager;
import L2jFT.game.cache.Msg;
import L2jFT.game.handler.AdminCommandHandler;
import L2jFT.game.handler.IAdminCommandHandler;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;
import L2jFT.game.network.serverpackets.SystemMessage;

/**
 * This class handles following admin commands: - disconnect, kick = disconnects
 * player by name or target
 */
public class AdminDisconnect implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = { "admin_disconnect", "admin_kick" };

	private enum CommandEnum
	{
		admin_disconnect,
		admin_kick
	}

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().CanKick)
			return false;

		String[] wordList = command.split(" ");
		CommandEnum cmd;

		try
		{
			cmd = CommandEnum.valueOf(wordList[0]);
		}
		catch(Exception e)
		{
			return false;
		}
		switch(cmd)
		{
			case admin_disconnect:
			case admin_kick:
				final L2Player player;
				if(wordList.length == 1)
				{
					// Обработка по таргету
					L2Object target = activeChar.getTarget();
					if(target == null)
					{
						activeChar.sendMessage("Select character or specify player name.", "Выделите игрока или укажите его ник.");
						break;
					}
					if(!target.isPlayer)
					{
						activeChar.sendPacket(Msg.INCORRECT_TARGET);
						break;
					}
					player = (L2Player) target;
				}
				else
				{
					// Обработка по нику
					player = L2World.getPlayer(wordList[1]);
					if(player == null)
					{
						activeChar.sendMessage("Character " + player + " not found in game.", "Персонаж " + player + " не найден.");
						break;
					}
				}

				if(player.getObjectId() == activeChar.getObjectId())
				{
					activeChar.sendMessage("You can't logout your character.", "Вы не можете выкинуть из игры самого себя.");
					break;
				}

				activeChar.sendMessage("Character " + player.getName() + " disconnected from server.", "Персонаж " + player.getName() + " выкинут из игры.");

				player.sendMessage("You are kicked by GM.", "ГМ выкинул вас из игры.");
				player.sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_BEEN_DISCONNECTED_FROM_THE_SERVER_PLEASE_LOGIN_AGAIN));
				ThreadPoolManager.getInstance().scheduleGeneral(new Runnable(){
					public void run()
					{
						player.kick(false);
					}
				}, 500);
				break;
		}
		return true;
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] {
				"file://com_disconnect.htm", "file://com_kick.htm" };
	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_BANS, AdminCommandHandler.GROUP_BANS };
	}
}