package commands.admin;

import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.cache.Msg;
import L2jFT.game.handler.AdminCommandHandler;
import L2jFT.game.handler.IAdminCommandHandler;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.instances.L2DoorInstance;
import L2jFT.game.tables.DoorTable;

import java.util.HashMap;
import java.util.Map;

/**
 * This class handles following admin commands: - open1 = open coloseum door
 * 24190001 - open2 = open coloseum door 24190002 - open3 = open coloseum door
 * 24190003 - open4 = open coloseum door 24190004 - openall = open all coloseum
 * door - close1 = close coloseum door 24190001 - close2 = close coloseum door
 * 24190002 - close3 = close coloseum door 24190003 - close4 = close coloseum
 * door 24190004 - closeall = close all coloseum door
 *  - open = open selected door - close = close selected door
 * @version $Revision: 1.2.4.5 $ $Date: 2005/04/11 10:06:06 $
 */
public class AdminDoorControl implements IAdminCommandHandler, ScriptFile
{
	private static DoorTable _doorTable;
	private static String[] _adminCommands = {
			"admin_open", "admin_close", "admin_openall", "admin_closeall" };
	private static final Map<String, Integer> doorMap = new HashMap<String, Integer>();

	static
	{
		doorMap.put("1", 24190001);
		doorMap.put("2", 24190002);
		doorMap.put("3", 24190003);
		doorMap.put("4", 24190004);
	}

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().Door)
			return false;
		_doorTable = DoorTable.getInstance();

		try
		{
			if(command.startsWith("admin_open "))
			{
				String doorName = command.split(" ")[1];
				_doorTable.getDoor(doorMap.get(doorName)).openMe();
			}
			else if(command.equals("admin_close "))
			{
				String doorName = command.split(" ")[1];
				_doorTable.getDoor(doorMap.get(doorName)).closeMe();
			}
			if(command.equals("admin_closeall"))
				for(L2DoorInstance door : _doorTable.getDoors())
					door.closeMe();
			if(command.equals("admin_openall"))
				for(L2DoorInstance door : _doorTable.getDoors())
					door.openMe();
			if(command.equals("admin_open"))
			{
				L2Object target = activeChar.getTarget();
				if(target instanceof L2DoorInstance)
					((L2DoorInstance) target).openMe();
				else
					activeChar.sendPacket(Msg.INCORRECT_TARGET);
			}

			if(command.equals("admin_close"))
			{
				L2Object target = activeChar.getTarget();
				if(target instanceof L2DoorInstance)
					((L2DoorInstance) target).closeMe();
				else
					activeChar.sendPacket(Msg.INCORRECT_TARGET);
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		return true;
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] {
				"file://com_open.htm", "file://com_close.htm",
				"file://com_openall.htm", "file://com_closeall.htm" };
	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_MANAGESERVER1,
				AdminCommandHandler.GROUP_MANAGESERVER1,
				AdminCommandHandler.GROUP_MANAGESERVER1,
				AdminCommandHandler.GROUP_MANAGESERVER1 };
	}
}