package commands.admin;

import javolution.util.FastList;
import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.cache.Msg;
import L2jFT.game.handler.AdminCommandHandler;
import L2jFT.game.handler.IAdminCommandHandler;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;
import L2jFT.game.model.L2WorldRegion;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.network.serverpackets.Earthquake;
import L2jFT.game.network.serverpackets.NpcInfo;
import L2jFT.game.network.serverpackets.SocialAction;
import L2jFT.game.network.serverpackets.StopMove;
import L2jFT.game.network.serverpackets.UserInfo;
import L2jFT.game.tables.SkillTable;
import L2jFT.util.Rnd;

import java.util.StringTokenizer;

@SuppressWarnings( { "nls", "unqualified-field-access", "boxing" })
public class AdminEffects implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = {
			"admin_invis", "admin_vis", "admin_earthquake", "admin_bighead",
			"admin_shrinkhead", "admin_unpara_all", "admin_para_all", "admin_unpara",
			"admin_para", "admin_polyself", "admin_unpolyself", "admin_changename",
			"admin_gmspeed", "admin_invul", "admin_setinvul", "admin_social",
			"admin_abnormal" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().GodMode)
			return false;

		if(command.startsWith("admin_social"))
		{
			String[] wordList = command.split(" ");
			int val;
			if(wordList.length == 1)
				val = Rnd.get(1, 7);
			else
				try
				{
					val = Integer.parseInt(wordList[1]);
				}
				catch(NumberFormatException nfe)
				{
					activeChar.sendMessage("Specify a valid social action number.");
					return false;
				}
			L2Character target = (L2Character) activeChar.getTarget();
			if(target == null || target == activeChar)
				activeChar.broadcastPacket(new SocialAction(activeChar.getObjectId(), val));
			else
				target.broadcastPacket(new SocialAction(target.getObjectId(), val));
		}
		else if(command.startsWith("admin_gmspeed"))
			try
			{
				int val = Integer.parseInt(command.substring(14));
				if(activeChar.getEffect((short) 7029) != null)
					activeChar.stopEffect((short) 7029);
				else if(val >= 1 && val <= 4)
					activeChar.doCast(SkillTable.getInstance().getInfo(7029, val), activeChar, true);
			}
			catch(Exception e)
			{
				activeChar.sendMessage("Use //gmspeed value = [0...4].");
			}
			finally
			{
				activeChar.updateEffectIcons();
			}
		else if(command.equalsIgnoreCase("admin_invis") || command.equalsIgnoreCase("admin_vis"))
		{
			if(activeChar.isInvisible())
			{
				activeChar.setInvisible(false);
				activeChar.broadcastUserInfo();
				if(activeChar.getPet() != null)
					activeChar.getPet().broadcastPetInfo();
			}
			else
			{
				activeChar.setInvisible(true);
				activeChar.sendPacket(new UserInfo(activeChar));
				
				if(activeChar.getCurrentRegion() != null)
					for(L2WorldRegion neighbor : activeChar.getCurrentRegion().getNeighbors())
						if(neighbor != null)
							neighbor.removePlayerFromOtherPlayers(activeChar);
			}
		}
		else if(command.equalsIgnoreCase("admin_earthquake"))
			try
			{
				String val = command.substring(17);
				StringTokenizer st = new StringTokenizer(val);
				String val1 = st.nextToken();
				int intensity = Integer.parseInt(val1);
				String val2 = st.nextToken();
				int duration = Integer.parseInt(val2);
				activeChar.broadcastPacket(new Earthquake(activeChar.getX(), activeChar.getY(), activeChar.getZ(), intensity, duration));
			}
			catch(Exception e)
			{}
		else if(command.equalsIgnoreCase("admin_para"))
		{
			String type = "1";
			StringTokenizer st = new StringTokenizer(command);
			try
			{
				st.nextToken();
				type = st.nextToken();
			}
			catch(Exception e)
			{}
			try
			{
				L2Object target = activeChar.getTarget();
				L2Character player;
				if(target.isCharacter)
				{
					player = (L2Character) target;
					if(type.equals("1"))
						player.startAbnormalEffect((short) L2Character.ABNORMAL_EFFECT_HOLD_1);
					else
						player.startAbnormalEffect((short) L2Character.ABNORMAL_EFFECT_HOLD_2);
					player.setParalyzed(true);

					player.broadcastPacket(new StopMove(player));
				}
			}
			catch(Exception e)
			{}
		}
		else if(command.equalsIgnoreCase("admin_unpara"))
			try
			{
				L2Object target = activeChar.getTarget();
				L2Character player = null;
				if(target.isCharacter)
				{
					player = (L2Character) target;
					player.stopAbnormalEffect((short) L2Character.ABNORMAL_EFFECT_HOLD_1);
					player.setParalyzed(false);
				}
			}
			catch(Exception e)
			{}
		else if(command.equalsIgnoreCase("admin_para_all"))
			try
			{
				FastList<L2Player> players = activeChar.getAroundPlayers(1250);
				for(FastList.Node<L2Player> n = players.head(), end = players.tail(); (n = n.getNext()) != end;)
				{
					L2Player player = n.getValue();
					if(!player.isGM())
					{
						player.startAbnormalEffect((short) L2Character.ABNORMAL_EFFECT_HOLD_1);
						player.setParalyzed(true);

						player.broadcastPacket(new StopMove(player));
					}
				}
			}
			catch(Exception e)
			{}
		else if(command.equalsIgnoreCase("admin_unpara_all"))
			try
			{
				for(L2Player player : activeChar.getAroundPlayers(2500))
				{
					player.stopAbnormalEffect((short) L2Character.ABNORMAL_EFFECT_HOLD_1);
					player.setParalyzed(false);
				}
			}
			catch(Exception e)
			{}
		else if(command.equalsIgnoreCase("admin_bighead") || command.equalsIgnoreCase("admin_shrinkhead"))
			try
			{
				L2Character target = (L2Character) activeChar.getTarget();
				if(target == null)
					activeChar.sendPacket(Msg.INCORRECT_TARGET);
				else if(target.isCharacter)
				{
					if(target.getAbnormalEffect() == L2Character.ABNORMAL_EFFECT_BIG_HEAD)
						target.setBigHead(false);
					else
						target.setBigHead(true);
					if(target.isPlayer)
						target.sendMessage("Admin changed your head type.");
				}
			}
			catch(Exception e)
			{}
		else if(command.equalsIgnoreCase("admin_polyself"))
		{
			StringTokenizer st = new StringTokenizer(command);
			try
			{
				st.nextToken();
				String id = st.nextToken();
				activeChar.setPolyInfo("npc", id);
				activeChar.teleToLocation(activeChar.getX(), activeChar.getY(), activeChar.getZ());
				activeChar.broadcastUserInfo();
			}
			catch(Exception e)
			{}
		}
		else if(command.equalsIgnoreCase("admin_unpolyself"))
		{
			activeChar.setPolyInfo(null, "1");
			activeChar.decayMe();
			activeChar.spawnMe(activeChar.getX(), activeChar.getY(), activeChar.getZ());
			activeChar.broadcastUserInfo();
		}
		else if(command.equalsIgnoreCase("admin_changename"))
			try
			{
				String name = command.substring(17);
				String oldName = "null";
				try
				{
					L2Object target = activeChar.getTarget();
					L2Character player = null;
					if(target == null)
					{
						player = activeChar;
						oldName = activeChar.getName();
					}
					else if(target.isCharacter)
					{
						player = (L2Character) target;
						oldName = player.getName();
					}
					else
						return false;
					if(player.isPlayer)
					{
						L2World.removeObject(player);
						player.decayMe();
					}
					player.setName(name);
					if(player.isPlayer)
						player.spawnMe();

					if(player.isPlayer)
						player.broadcastUserInfo();
					else if(player instanceof L2NpcInstance)
						player.broadcastPacket(new NpcInfo((L2NpcInstance) player, null));

					activeChar.sendMessage("Changed name from " + oldName + " to " + name + ".");
				}
				catch(Exception e)
				{}
			}
			catch(StringIndexOutOfBoundsException e)
			{}
		else if(command.equalsIgnoreCase("admin_invul"))
			handleInvul(activeChar);
		else if(command.equalsIgnoreCase("admin_setinvul"))
		{
			L2Object target = activeChar.getTarget();
			if(target.isPlayer)
				handleInvul((L2Player) target);
		}
		else if(command.startsWith("admin_abnormal"))
		{
			String[] wordList = command.split(" ");

			int val;
				try
				{
					val = Integer.parseInt(wordList[1], 2);
				}
				catch(Exception e)
				{
					activeChar.sendMessage("Specify a valid abnormal mask.");
					return false;
				}
				activeChar.setCustomEffect(val);
				activeChar.updateAbnormalEffect();
		}

		return true;
	}

	private void handleInvul(L2Player activeChar)
	{
		if(activeChar.isInvul())
		{
			activeChar.setIsInvul(false);
			if(activeChar.getPet() != null)
				activeChar.getPet().setIsInvul(false);
			activeChar.sendMessage(activeChar.getName() + " is now mortal.");
		}
		else
		{
			activeChar.setIsInvul(true);
			if(activeChar.getPet() != null)
				activeChar.getPet().setIsInvul(true);
			activeChar.sendMessage(activeChar.getName() + " is now immortal.");
		}
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] {
				"Описание команды invis", "Описание команды vis",
				"Описание команды earthquake", "Описание команды bighead",
				"Описание команды shrinkhead", "Описание команды unpara_all",
				"Описание команды para_all", "Описание команды unpara",
				"Описание команды para", "Описание команды polyself",
				"Описание команды unpolyself", "Описание команды changename",
				"Описание команды gmspeed", "Описание команды invul",
				"Описание команды setinvul", "Описание команды social",
				"Описание команды abnormal"
		};

	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_MANAGESERVER1,
				AdminCommandHandler.GROUP_MANAGESERVER1,
				AdminCommandHandler.GROUP_MANAGESERVER1,
				AdminCommandHandler.GROUP_MANAGESERVER1,
				AdminCommandHandler.GROUP_MANAGESERVER1,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2 };
	}
}
