package commands.admin;

import javolution.text.TextBuilder;
import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.handler.AdminCommandHandler;
import L2jFT.game.handler.IAdminCommandHandler;
import L2jFT.game.model.Inventory;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.network.serverpackets.InventoryUpdate;
import L2jFT.game.network.serverpackets.NpcHtmlMessage;
import L2jFT.util.Log;

public class AdminEnchant implements IAdminCommandHandler, ScriptFile
{
	private static final String[] ADMIN_COMMANDS = { "admin_seteh",// 6
			"admin_setec",// 10
			"admin_seteg",// 9
			"admin_setel",// 11
			"admin_seteb",// 12
			"admin_setew",// 7
			"admin_setes",// 8
			"admin_setle",// 1
			"admin_setre",// 2
			"admin_setlf",// 4
			"admin_setrf",// 5
			"admin_seten",// 3
			"admin_setun",// 0
			"admin_setba",// 13
			"admin_enchant" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().CanEditChar)
			return false;

		if(command.equals("admin_enchant"))
			showMainPage(activeChar);
		else
		{
			int armorType = -1;

			if(command.startsWith("admin_seteh"))
				armorType = Inventory.PAPERDOLL_HEAD;
			else if(command.startsWith("admin_setec"))
				armorType = Inventory.PAPERDOLL_CHEST;
			else if(command.startsWith("admin_seteg"))
				armorType = Inventory.PAPERDOLL_GLOVES;
			else if(command.startsWith("admin_seteb"))
				armorType = Inventory.PAPERDOLL_FEET;
			else if(command.startsWith("admin_setel"))
				armorType = Inventory.PAPERDOLL_LEGS;
			else if(command.startsWith("admin_setew"))
				armorType = Inventory.PAPERDOLL_RHAND;
			else if(command.startsWith("admin_setes"))
				armorType = Inventory.PAPERDOLL_LHAND;
			else if(command.startsWith("admin_setle"))
				armorType = Inventory.PAPERDOLL_LEAR;
			else if(command.startsWith("admin_setre"))
				armorType = Inventory.PAPERDOLL_REAR;
			else if(command.startsWith("admin_setlf"))
				armorType = Inventory.PAPERDOLL_LFINGER;
			else if(command.startsWith("admin_setrf"))
				armorType = Inventory.PAPERDOLL_RFINGER;
			else if(command.startsWith("admin_seten"))
				armorType = Inventory.PAPERDOLL_NECK;
			else if(command.startsWith("admin_setun"))
				armorType = Inventory.PAPERDOLL_UNDER;
			else if(command.startsWith("admin_setba"))
				armorType = Inventory.PAPERDOLL_BACK;

			if(armorType != -1)
				try
				{
					int ench = Integer.parseInt(command.substring(12));

					// check value
					if(ench < 0 || ench > 65535)
						activeChar.sendMessage("You must set the enchant level to be between 0-65535.");
					else
						setEnchant(activeChar, ench, armorType);
				}
				catch(StringIndexOutOfBoundsException e)
				{
					activeChar.sendMessage("Please specify a new enchant value.");
				}
				catch(NumberFormatException e)
				{
					activeChar.sendMessage("Please specify a valid new enchant value.");
				}

			// show the enchant menu after an action
			showMainPage(activeChar);
		}

		return true;
	}

	private void setEnchant(L2Player activeChar, int ench, int armorType)
	{
		// get the target
		L2Object target = activeChar.getTarget();
		if(target == null)
			target = activeChar;
		L2Player player = null;
		if(target instanceof L2Player)
			player = (L2Player) target;
		else
		{
			activeChar.sendMessage("Wrong target type.");
			return;
		}

		// now we need to find the equipped weapon of the targeted character...
		int curEnchant = 0; // display purposes only
		L2ItemInstance itemInstance = null;

		// only attempt to enchant if there is a weapon equipped
		L2ItemInstance parmorInstance = player.getInventory().getPaperdollItem(armorType);
		if(parmorInstance != null && parmorInstance.getEquipSlot() == armorType)
			itemInstance = parmorInstance;
		else
		{
			// for bows and double handed weapons
			parmorInstance = player.getInventory().getPaperdollItem(Inventory.PAPERDOLL_LRHAND);
			if(parmorInstance != null && parmorInstance.getEquipSlot() == Inventory.PAPERDOLL_LRHAND)
				itemInstance = parmorInstance;
		}

		if(itemInstance != null)
		{
			curEnchant = itemInstance.getEnchantLevel();

			// set enchant value
			player.getInventory().unEquipItemInSlotAndRecord((short) armorType);
			itemInstance.setEnchantLevel(ench);
			player.getInventory().equipItemAndRecord(itemInstance);

			// send packets
			InventoryUpdate iu = new InventoryUpdate();
			iu.addModifiedItem(itemInstance);
			player.sendPacket(iu);
			player.broadcastUserInfo();

			// informations
			activeChar.sendMessage("Changed enchantment of " + player.getName() + "'s " + itemInstance.getItem().getName() + " from " + curEnchant + " to " + ench + ".");
			player.sendMessage("Admin has changed the enchantment of your " + itemInstance.getItem().getName() + " from " + curEnchant + " to " + ench + ".");

			Log.add(activeChar + " change the enchantment of " + player.getName() + "'s " + itemInstance.getItem().getName() + " from " + curEnchant + " to " + ench + ".", "gm_ext_actions", activeChar);
		}
	}

	public void showMainPage(L2Player activeChar)
	{
		NpcHtmlMessage adminReply = new NpcHtmlMessage(5);

		TextBuilder replyMSG = new TextBuilder("<html><body>");
		replyMSG.append("<center><table width=260><tr><td width=40>");
		replyMSG.append("<button value=\"Main\" action=\"bypass -h admin_admin\" width=45 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\">");
		replyMSG.append("</td><td width=180>");
		replyMSG.append("<center>Enchant Equip</center>");
		replyMSG.append("</td><td width=40>");
		replyMSG.append("</td></tr></table></center><br>");
		replyMSG.append("<center><table width=270><tr><td>");
		replyMSG.append("<button value=\"Underwear\" action=\"bypass -h admin_setun $menu_command\" width=50 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td><td>");
		replyMSG.append("<button value=\"Helmet\" action=\"bypass -h admin_seteh $menu_command\" width=50 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td><td>");
		replyMSG.append("<button value=\"Cloak\" action=\"bypass -h admin_setba $menu_command\" width=50 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td><td>");
		replyMSG.append("<button value=\"Mask\" width=50 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td><td>");
		replyMSG.append("<button value=\"Necklace\" action=\"bypass -h admin_seten $menu_command\" width=50 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td></tr></table>");
		replyMSG.append("</center><center><table width=270><tr><td>");
		replyMSG.append("<button value=\"Weapon\" action=\"bypass -h admin_setew $menu_command\" width=50 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td><td>");
		replyMSG.append("<button value=\"Chest\" action=\"bypass -h admin_setec $menu_command\" width=50 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td><td>");
		replyMSG.append("<button value=\"Shield\" action=\"bypass -h admin_setes $menu_command\" width=50 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td><td>");
		replyMSG.append("<button value=\"Earring\" action=\"bypass -h admin_setre $menu_command\" width=50 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td><td>");
		replyMSG.append("<button value=\"Earring\" action=\"bypass -h admin_setle $menu_command\" width=50 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td></tr></table>");
		replyMSG.append("</center><center><table width=270><tr><td>");
		replyMSG.append("<button value=\"Gloves\" action=\"bypass -h admin_seteg $menu_command\" width=50 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td><td>");
		replyMSG.append("<button value=\"Leggings\" action=\"bypass -h admin_setel $menu_command\" width=50 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td><td>");
		replyMSG.append("<button value=\"Boots\" action=\"bypass -h admin_seteb $menu_command\" width=50 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td><td>");
		replyMSG.append("<button value=\"Ring\" action=\"bypass -h admin_setrf $menu_command\" width=50 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td><td>");
		replyMSG.append("<button value=\"Ring\" action=\"bypass -h admin_setlf $menu_command\" width=50 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td></tr></table>");
		replyMSG.append("</center><br>");
		replyMSG.append("<center>[Enchant 0-65535]</center>");
		replyMSG.append("<center><edit var=\"menu_command\" width=100 height=15></center><br>");
		replyMSG.append("</body></html>");

		adminReply.setHtml(replyMSG.toString());
		activeChar.sendPacket(adminReply);
	}

	public String[] getAdminCommandList()
	{
		return ADMIN_COMMANDS;
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] { "Описание команды seteh",// 6
				"Описание команды setec",// 10
				"Описание команды seteg",// 9
				"Описание команды setel",// 11
				"Описание команды seteb",// 12
				"Описание команды setew",// 7
				"Описание команды setes",// 8
				"Описание команды setle",// 1
				"Описание команды setre",// 2
				"Описание команды setlf",// 4
				"Описание команды setrf",// 5
				"Описание команды seten",// 3
				"Описание команды setun",// 0
				"Описание команды setba",// 13
				"Описание команды enchant" };

	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_EDITCHAR, AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR, AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR, AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR, AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR, AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR, AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR, AdminCommandHandler.GROUP_EDITCHAR,
				AdminCommandHandler.GROUP_EDITCHAR };
	}
}