package commands.admin;

import L2jFT.Config;
import L2jFT.extensions.multilang.CustomMessage;
import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.geodata.GeoData;
import L2jFT.game.geodata.GeoEngine;
import L2jFT.game.geodata.geoeditorcon.GeoEditorConnector;
import L2jFT.game.handler.AdminCommandHandler;
import L2jFT.game.handler.IAdminCommandHandler;
import L2jFT.game.model.L2Player;

public class AdminGeodata implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = {
			"admin_geo_z", "admin_geo_type", "admin_geo_nswe", "admin_geo_los",
			"admin_geo_bug", "admin_geo_load", "admin_geo_unload",
			"admin_geo_enabled", "admin_geo_disabled", "admin_geo_waterlevel",
			"admin_geo_info", "admin_geoeditor_connect", "admin_geoeditor_join",
			"admin_geoeditor_leave" };

	private enum CommandEnum
	{
		admin_geo_z,
		admin_geo_type,
		admin_geo_nswe,
		admin_geo_los,
		admin_geo_load,
		admin_geo_unload,
		admin_geo_bug,
		admin_geo_enabled,
		admin_geo_disabled,
		admin_geo_waterlevel,
		admin_geo_info,
		admin_geoeditor_connect,
		admin_geoeditor_join,
		admin_geoeditor_leave
	}

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().CanReload)
			return false;

		String[] wordList = command.split(" ");
		CommandEnum comm;

		try
		{
			comm = CommandEnum.valueOf(wordList[0]);
		}
		catch(Exception e)
		{
			return false;
		}

		CommandEnum commandEnum = comm;
		switch(commandEnum)
		{
			case admin_geo_z:
			case admin_geo_type:
			case admin_geo_nswe:
			case admin_geo_los:
			case admin_geo_load:
			case admin_geo_unload:
			case admin_geo_bug:
			case admin_geo_waterlevel:
			case admin_geo_info:
			case admin_geoeditor_connect:
			case admin_geoeditor_join:
			case admin_geoeditor_leave:
				if(!Config.GEODATA)
				{
					activeChar.sendMessage("Геодата неактивна.");
					return true;
				}
				break;
		}

		switch(commandEnum)
		{
			case admin_geo_z:
				activeChar.sendMessage("GeoEngine: Geo_Z = " + GeoData.getInstance().getHeight(activeChar.getX(), activeChar.getY(), activeChar.getZ()) + " Loc_Z = " + activeChar.getZ());
				break;
			case admin_geo_type:
				int type = GeoData.getInstance().getType(activeChar.getX(), activeChar.getY());
				activeChar.sendMessage("GeoEngine: Geo_Type = " + type);
				break;
			case admin_geo_nswe:
				String result = "";
				int nswe = GeoData.getInstance().getNSWE(activeChar.getX(), activeChar.getY(), activeChar.getZ());
				if((nswe & 8) == 0)
					result += " N";
				if((nswe & 4) == 0)
					result += " S";
				if((nswe & 2) == 0)
					result += " W";
				if((nswe & 1) == 0)
					result += " E";
				activeChar.sendMessage("GeoEngine: Geo_NSWE -> " + nswe + "->" + result);
				break;
			case admin_geo_los:
				if(activeChar.getTarget() != null)
					if(GeoData.getInstance().canSeeTarget(activeChar, activeChar.getTarget()))
						activeChar.sendMessage("GeoEngine: Can See Target");
					else
						activeChar.sendMessage("GeoEngine: Can't See Target");
				else
					activeChar.sendMessage("None Target!");
				break;
			case admin_geo_load:
				if(wordList.length != 3)
					activeChar.sendMessage("Usage: //admin_geo_load <regionX> <regionY>");
				else
					try
					{
						byte rx = Byte.parseByte(wordList[1]);
						byte ry = Byte.parseByte(wordList[2]);
						if(GeoEngine.LoadGeodataFile(rx, ry))
							activeChar.sendMessage("GeoEngine: Регион [" + rx + "," + ry + "] успешно загружен");
						else
							activeChar.sendMessage("GeoEngine: Регион [" + rx + "," + ry + "] не загрузился");
					}
					catch(Exception e)
					{
						activeChar.sendMessage(new CustomMessage("common.Error", activeChar));
					}
				break;
			case admin_geo_unload:
				if(wordList.length != 3)
					activeChar.sendMessage("Формат: //admin_geo_unload <regionX> <regionY>");
				else
					try
					{
						byte rx = Byte.parseByte(wordList[1]);
						byte ry = Byte.parseByte(wordList[2]);
						GeoData.unloadGeodataFile(rx, ry);
						activeChar.sendMessage("GeoEngine: Регион [" + rx + "," + ry + "] выгружен.");
					}
					catch(Exception e)
					{
						activeChar.sendMessage(new CustomMessage("common.Error", activeChar));
					}
				break;
			case admin_geo_bug:
				if(wordList.length != 3)
					activeChar.sendMessage("Формат: //admin_geo_bug комментарии");
				else
					try
					{
						GeoData.getInstance().addGeoDataBug(activeChar, wordList[1]);
					}
					catch(StringIndexOutOfBoundsException e)
					{
						activeChar.sendMessage(new CustomMessage("common.Error", activeChar));
					}
				break;
			case admin_geo_enabled:
				Config.GEODATA = true;
				activeChar.sendMessage("Геодата включена.");
				break;
			case admin_geo_disabled:
				Config.GEODATA = false;
				activeChar.sendMessage("Геодата выключена.");
				break;
			case admin_geo_waterlevel:
				int geo_type = GeoData.getInstance().getWaterLevel(activeChar.getX(), activeChar.getY(), activeChar.getZ());
				activeChar.sendMessage("GeoEngine: Geo_Type = " + geo_type);
				break;
			case admin_geo_info:
				GeoData.getInstance().getInfo(activeChar, activeChar.getX(), activeChar.getY(), activeChar.getZ());
				break;
			case admin_geoeditor_connect:
				try
				{
					int ticks = Integer.parseInt(command.substring(24));
					GeoEditorConnector.getInstance().connect(activeChar, ticks);
				}
				catch(Exception e)
				{
					activeChar.sendMessage("Usage: //geoeditor_connect <number>");
				}
				break;
			case admin_geoeditor_join:
				GeoEditorConnector.getInstance().join(activeChar);
				break;
			case admin_geoeditor_leave:
				GeoEditorConnector.getInstance().leave(activeChar);
				break;
		}

		return true;
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] {
				"Описание команды geo_z", "Описание команды geo_type",
				"Описание команды geo_nswe", "Описание команды geo_los",
				"Описание команды geo_bug", "Описание команды geo_load",
				"Описание команды geo_unload", "Описание команды geo_enabled",
				"Описание команды geo_disabled", "Описание команды geo_waterlevel",
				"Описание команды geo_info", "Описание команды geoeditor_connect",
				"Описание команды geoeditor_join", "Описание команды geoeditor_leave" };
	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER2,
				AdminCommandHandler.GROUP_MANAGESERVER3,
				AdminCommandHandler.GROUP_MANAGESERVER3,
				AdminCommandHandler.GROUP_MANAGESERVER3,
				AdminCommandHandler.GROUP_MANAGESERVER3,
				AdminCommandHandler.GROUP_MANAGESERVER3,
				AdminCommandHandler.GROUP_MANAGESERVER3,
				AdminCommandHandler.GROUP_MANAGESERVER3,
				AdminCommandHandler.GROUP_MANAGESERVER3 };
	}
}