package commands.admin;

import javolution.text.TextBuilder;
import javolution.util.FastList;
import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.handler.AdminCommandHandler;
import L2jFT.game.handler.IAdminCommandHandler;
import L2jFT.game.model.L2Player;
import L2jFT.game.network.serverpackets.NpcHtmlMessage;
import L2jFT.util.Files;

import java.io.File;
import java.util.Collections;
import java.util.HashMap;
import java.util.Vector;

/**
 * This class handles following admin commands: - help path = shows
 * /data/html/admin/path file to char, should not be used by GM's directly
 */
public class AdminHelpPage implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = { "admin_help", "admin_showhtml" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().Menu)
			return false;

		String args = null;

		if(command.startsWith("admin_help"))
		{
			if(command.length() > _adminCommands[0].length() + 1)
				args = command.substring(_adminCommands[0].length()).trim();

			if(args == null || args.equals("groups"))
			{
				HashMap<String, Vector<String>> map = getGrouppedMap();

				TextBuilder tb = TextBuilder.newInstance();
				tb.append("<html><body>");
				tb.append("<center>");
				tb.append("<font color=\"LEVEL\">L2J Rebellion - GM Help</font>");
				tb.append("</center><br><br>");

				FastList<String> groups = FastList.newInstance();
				for(String group : map.keySet())
					groups.add(group);

				Collections.sort(groups);

				for(String group : groups)
				{
					tb.append("<center>");
					tb.append("<a action=\"bypass -h admin_help group " + group + "\">" + group + "</a>");
					tb.append("</center>");
					tb.append("<br1>");
				}

				//tb.append("<br>");
				//tb.append("<a action=\"bypass -h admin_help all\">All commands</a>");

				tb.append("</body></html>");

				NpcHtmlMessage html = new NpcHtmlMessage(0);
				html.setHtml(tb.toString());
				activeChar.sendPacket(html);
				TextBuilder.recycle(tb);
				FastList.recycle(groups);
			}
			/*else if(args.equals("all")) // закоментрировано т.к. слишком длинная html и команда не используется.
			   {
				TextBuilder tb = TextBuilder.newInstance();
				tb.append("<html><body>");
				tb.append("<center>");
				tb.append("L2J Rebellion - GM Help");
				tb.append("</center><br><br>");

				for(String cmd : AdminCommandHandler.getInstance().getAllCommands())
				{
					tb.append("<font color=\"LEVEL\">");
					tb.append(cmd.startsWith("admin_") ? cmd.substring(6) : cmd);
					tb.append("</font>");
					tb.append(" - ");
					tb.append(AdminCommandHandler.getInstance().getDescriptionForCommand(cmd));
					tb.append("<br1>");
				}

				tb.append("</body></html>");

				NpcHtmlMessage html = new NpcHtmlMessage(0);
				html.setHtml(tb.toString());
				activeChar.sendPacket(html);
				TextBuilder.recycle(tb);
			   }*/
			else if(args.startsWith("group"))
			{
				args = args.substring(5).trim();
				HashMap<String, Vector<String>> map = getGrouppedMap();

				Vector<String> group = map.get(args);

				if(group == null)
				{
					NpcHtmlMessage html = new NpcHtmlMessage(0);
					html.setHtml("<html><body>Sorry, but group \"" + args + "\" does not exists</body></html>");
					activeChar.sendPacket(html);
					return false;
				}

				Collections.sort(group);

				TextBuilder tb = TextBuilder.newInstance();
				tb.append("<html><body>");
				tb.append("<center>");
				tb.append("<font color=\"LEVEL\">L2J Rebellion - GM Help</font>");
				tb.append("<br>");
				tb.append("Current group: " + args);
				tb.append("</center><br><br>");

				for(String cmd : group)
				{
					String desk = AdminCommandHandler.getInstance().getDescriptionForCommand(cmd);
					if(desk.startsWith("file://"))
					{
						cmd = cmd.startsWith("admin_") ? cmd.substring(6) : cmd;

						String[] split = desk.split("\\|");
						if(split.length == 2)
							desk = "<a action=\"bypass -h admin_showhtml help/" + split[0].substring(7) + "\">" + cmd + " - "+split[1]+"</a>";
						else
							desk = "<a action=\"bypass -h admin_showhtml help/" + desk.substring(7) + "\">" + cmd + "</a>";
						tb.append(desk);
					}
					else
					{
						tb.append("<font color=\"LEVEL\">");
						tb.append(cmd.startsWith("admin_") ? cmd.substring(6) : cmd);
						tb.append("</font>");
						tb.append(" - ");
						tb.append(desk);
					}

					tb.append("<br1>");
				}

				tb.append("<center><a action=\"bypass -h admin_help\">Back</a></center>");
				tb.append("</body></html>");

				NpcHtmlMessage html = new NpcHtmlMessage(0);
				html.setHtml(tb.toString());
				activeChar.sendPacket(html);
				TextBuilder.recycle(tb);
			}
		}
		else if(command.startsWith("admin_showhtml"))
		{
			if(command.length() > _adminCommands[1].length() + 1)
				args = command.substring(_adminCommands[1].length()).trim();

			if(args == null)
			{
				activeChar.sendMessage("Usage: //showhtml <file>");
				return false;
			}

			showHelpPage(activeChar, args);
		}

		return true;
	}

	/**
	 * Создает мап со всем командам/группами.
	 *
	 * @return отсортированные каманды по группам
	 */
	private HashMap<String, Vector<String>> getGrouppedMap()
	{
		HashMap<String, Vector<String>> map = new HashMap<String, Vector<String>>();

		for(String cmd : AdminCommandHandler.getInstance().getAllCommands())
		{
			String group = AdminCommandHandler.getInstance().getGroupForCommand(cmd);

			Vector<String> groupped = map.get(group);

			if(groupped == null)
			{
				groupped = new Vector<String>();
				map.put(group, groupped);
			}

			groupped.add(cmd);
		}
		return map;
	}

	/**
	 * FIXME: implement method to send html to player in L2Player directly
	 * PUBLIC & STATIC so other classes from package can include it directly
	 */
	public static void showHelpPage(L2Player targetChar, String filename)
	{
		String content = Files.read("data/html/admin/" + filename, targetChar);

		if(filename.startsWith("help/")) // Шаблон для Help'а
		{
			File f = new File(filename);
			String command = f.getName();
			command = command.substring(4, command.length() - 4); // Убираем com_ и .htm

			content = content.replaceFirst("<html><body>", "");
			content = content.replaceFirst("</body></html>", "");

			IAdminCommandHandler handler = AdminCommandHandler.getInstance().getAdminCommandHandler(command);
			if(handler == null)
				command = "admin_" + command;

			TextBuilder tb = TextBuilder.newInstance();
			tb.append("<html><body><center>");
			tb.append("<font color=\"LEVEL\">L2J Rebellion Help Page</font>");
			tb.append("<img src=\"L2UI.SquareBlank\" width=260 height=4>");
			tb.append("<img src=\"L2UI.SquareWhite\" width=260 height=1>");
			tb.append("<img src=\"L2UI.SquareBlank\" width=260 height=4></center>");
			tb.append("<br>");
			tb.append("<br>");
			tb.append(content);
			tb.append("<br>");
			tb.append("<br>");
			tb.append("<center><a action=\"bypass -h admin_help group " + AdminCommandHandler.getInstance().getGroupForCommand(command) + "\">Back</a>");
			tb.append("<br1>");
			tb.append("<img src=\"L2UI.SquareBlank\" width=260 height=4>");
			tb.append("<img src=\"L2UI.SquareWhite\" width=260 height=1>");
			tb.append("<img src=\"L2UI.SquareBlank\" width=260 height=4>");
			tb.append("Server software running L2J Rebellion.<br1>Created by L2J Rebellion team.");
			tb.append("</center></body></html>");
			content = tb.toString();
			TextBuilder.recycle(tb);
		}

		NpcHtmlMessage adminReply = new NpcHtmlMessage(5);

		adminReply.setHtml(content);
		targetChar.sendPacket(adminReply);
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] {
				"Справка по админ командам", "Открытие файла из data/html/admin" };
	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_HELP, AdminCommandHandler.GROUP_HELP };
	}
}