package commands.admin;

import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.cache.Msg;
import L2jFT.game.handler.AdminCommandHandler;
import L2jFT.game.handler.IAdminCommandHandler;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;
import L2jFT.util.Log;

public class AdminKill implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = { "admin_kill" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().CanEditNPC)
			return false;
		if(command.startsWith("admin_kill "))
			handleKill(activeChar, command.split(" ")[1]);
		if(command.equals("admin_kill"))
			handleKill(activeChar);
		return true;
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	private void handleKill(L2Player activeChar)
	{
		handleKill(activeChar, null);
	}

	private void handleKill(L2Player activeChar, String player)
	{
		L2Object obj = activeChar.getTarget();
		if(player != null)
		{
			L2Player plyr = L2World.getPlayer(player);
			if(plyr != null)
				obj = plyr;
			else
			{
				int radius = Math.max(Integer.parseInt(player), 100);
				for(L2Character character : activeChar.getKnownCharacters(radius))
					character.reduceCurrentHp(character.getMaxHp() + character.getMaxCp() + 1, character);
				activeChar.sendMessage("Killed within " + radius + " unit radius.");
				return;
			}
		}

		if(obj != null && obj.isCharacter)
		{
			L2Character target = (L2Character) obj;
			target.reduceCurrentHp(target.getMaxHp() + target.getMaxCp() + 1, activeChar);
			Log.add("kill character " + target.getObjectId() + " " + target.getName(), "gm_ext_actions", activeChar);
		}
		else
			activeChar.sendPacket(Msg.INCORRECT_TARGET);
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] { "Описание команды kill" };
	}

	public String[] getCommandGroup()
	{
		return new String[] { AdminCommandHandler.GROUP_EDITCHAR };
	}
}
