package commands.admin;

import java.util.StringTokenizer;

import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.cache.Msg;
import L2jFT.game.handler.AdminCommandHandler;
import L2jFT.game.handler.IAdminCommandHandler;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.base.Experience;

public class AdminLevel implements IAdminCommandHandler, ScriptFile
{
	private static final String[] ADMIN_COMMANDS = {
			"admin_addLevel", "admin_setLevel" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(activeChar == null)
			return false;

		if(!activeChar.getPlayerAccess().CanEditChar)
			return false;

		if(activeChar.getTarget() == null || !activeChar.getTarget().isPlayer)
		{
			activeChar.sendPacket(Msg.INCORRECT_TARGET);
			return false;
		}

		L2Player target = (L2Player) activeChar.getTarget();
		StringTokenizer st = new StringTokenizer(command, " ");
		String actualCommand = st.nextToken();

		String val = "";
		if(st.countTokens() >= 1)
			val = st.nextToken();

		if(actualCommand.equalsIgnoreCase("admin_addLevel"))
			try
			{
				Byte level = (byte) (target.getLevel() + Byte.parseByte(val));
				Long exp_add = Experience.LEVEL[level] - target.getExp();
				target.addExpAndSp(exp_add, 0);
				while(target.getExp() < Experience.LEVEL[level] && level > 1)
				{
					target.decreaseLevel();
					level = target.getLevel();
				}
			}
			catch(NumberFormatException e)
			{
				activeChar.sendMessage("Wrong Number Format");
			}
		else if(actualCommand.equalsIgnoreCase("admin_setLevel"))
			try
			{
				byte level = Byte.parseByte(val);
				if(level >= 1 && level <= Experience.MAX_LEVEL)
				{
					Long exp_add = Experience.LEVEL[level] - target.getExp();
					target.addExpAndSp(exp_add, 0);
					while(target.getExp() < Experience.LEVEL[level] && level > 1)
					{
						target.decreaseLevel();
						level = target.getLevel();
					}
				}
				else
				{
					activeChar.sendMessage("You must specify level between 1 and " + Experience.MAX_LEVEL + ".");
					return false;
				}
			}
			catch(NumberFormatException e)
			{
				activeChar.sendMessage("You must specify level between 1 and " + Experience.MAX_LEVEL + ".");
				return false;
			}
		return true;
	}

	public String[] getAdminCommandList()
	{
		return ADMIN_COMMANDS;
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] { "admin_addLevel", "admin_setLevel" };
	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_EDITCHAR, AdminCommandHandler.GROUP_EDITCHAR };
	}
}
