package commands.admin;

import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.handler.AdminCommandHandler;
import L2jFT.game.handler.IAdminCommandHandler;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.network.serverpackets.SystemMessage;

import java.util.ArrayList;

/**
 * Admin Command Handler for Mammon NPCs
 */
public class AdminMammon implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = {
			"admin_find_mammon", "admin_show_mammon", "admin_hide_mammon",
			"admin_list_spawns" };

	ArrayList<Integer> npcIds = new ArrayList<Integer>();

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		npcIds.clear();

		if(!activeChar.getPlayerAccess().Menu)
			return false;

		else if(command.startsWith("admin_find_mammon"))
		{
			npcIds.add(31113);
			npcIds.add(31126);
			npcIds.add(31092); // Add the Marketeer of Mammon also
			int teleportIndex = -1;

			try
			{
				if(command.length() > 16)
					teleportIndex = Integer.parseInt(command.substring(18));
			}
			catch(Exception NumberFormatException)
			{
				// activeChar.sendPacket(SystemMessage.sendString("Command format is
				// //find_mammon <teleportIndex>"));
			}

			findAdminNPCs(activeChar, npcIds, teleportIndex, -1);
		}

		else if(command.equals("admin_show_mammon"))
		{
			npcIds.add(31113);
			npcIds.add(31126);

			findAdminNPCs(activeChar, npcIds, -1, 1);
		}

		else if(command.equals("admin_hide_mammon"))
		{
			npcIds.add(31113);
			npcIds.add(31126);

			findAdminNPCs(activeChar, npcIds, -1, 0);
		}

		else if(command.startsWith("admin_list_spawns"))
		{
			int npcId = 0;

			try
			{
				npcId = Integer.parseInt(command.substring(18).trim());
			}
			catch(Exception NumberFormatException)
			{
				activeChar.sendMessage("Command format is //list_spawns <NPC_ID>");
			}

			npcIds.add(npcId);
			findAdminNPCs(activeChar, npcIds, -1, -1);
		}

		// Used for testing SystemMessage IDs - Use //msg <ID>
		else if(command.startsWith("admin_msg"))
			activeChar.sendPacket(new SystemMessage(Integer.parseInt(command.substring(10).trim())));

		return true;
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	public void findAdminNPCs(L2Player activeChar, ArrayList<Integer> npcIdList, int teleportIndex, int makeVisible)
	{
		int index = 0;

		for(L2Object obj : L2World.getAllObjects().values())
			if(obj instanceof L2NpcInstance)
			{
				L2NpcInstance npcInst = (L2NpcInstance) obj;
				int npcId = npcInst.getNpcId();

				if(npcIdList.contains(npcId))
				{
					if(makeVisible == 1)
						npcInst.spawnMe();
					else if(makeVisible == 0)
						npcInst.decayMe();

					if(npcInst.isVisible())
					{
						index++;

						if(teleportIndex > -1)
						{
							if(teleportIndex == index)
								activeChar.teleToLocation(npcInst.getX(), npcInst.getY(), npcInst.getZ());
						}
						else
							activeChar.sendMessage(index + " - " + npcInst.getName() + " (" + npcInst.getObjectId() + "): " + npcInst.getX() + " " + npcInst.getY() + " " + npcInst.getZ());
					}
				}
			}
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] {
				"admin_find_mammon", "admin_show_mammon", "admin_hide_mammon",
				"admin_list_spawns" };
	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_MANAGESERVER3,
				AdminCommandHandler.GROUP_MANAGESERVER3,
				AdminCommandHandler.GROUP_MANAGESERVER3,
				AdminCommandHandler.GROUP_MANAGESERVER3 };
	}
}
