package commands.admin;

import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.LoginServerThread;
import L2jFT.game.cache.Msg;
import L2jFT.game.handler.AdminCommandHandler;
import L2jFT.game.handler.IAdminCommandHandler;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;

import java.util.StringTokenizer;

/**
 * This class handles following admin commands: - handles ever admin menu
 * command
 * 
 * @version $Revision: 1.3.2.6.2.4 $ $Date: 2005/04/11 10:06:06 $
 */
public class AdminMenu implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = {
			"admin_char_manage", "admin_teleport_character_to_menu",
			"admin_recall_char_menu", "admin_goto_char_menu", "admin_kick_menu",
			"admin_kill_menu", "admin_ban_menu", "admin_unban_menu" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().Menu)
			return false;

		if(command.equals("admin_char_manage"))
			AdminHelpPage.showHelpPage(activeChar, "charmanage.htm");
		else if(command.startsWith("admin_teleport_character_to_menu"))
		{
			String[] data = command.split(" ");
			if(data.length == 5)
			{
				String playerName = data[1];
				int x = Integer.parseInt(data[2]);
				int y = Integer.parseInt(data[3]);
				int z = Integer.parseInt(data[4]);
				L2Player player = L2World.getPlayer(playerName);
				if(player != null)
					teleportCharacter(player, x, y, z, activeChar);
			}
			AdminHelpPage.showHelpPage(activeChar, "charmanage.htm");
		}
		else if(command.startsWith("admin_recall_char_menu"))
			try
			{
				String targetName = command.substring(23);
				L2Player player = L2World.getPlayer(targetName);
				int x = activeChar.getX();
				int y = activeChar.getY();
				int z = activeChar.getZ();
				teleportCharacter(player, x, y, z, activeChar);
			}
			catch(StringIndexOutOfBoundsException e)
			{}
		else if(command.startsWith("admin_goto_char_menu"))
			try
			{
				String targetName = command.substring(21);
				L2Player player = L2World.getPlayer(targetName);
				teleportToCharacter(activeChar, player);
			}
			catch(StringIndexOutOfBoundsException e)
			{}
		else if(command.equals("admin_kill_menu"))
			handleKill(activeChar);
		else if(command.startsWith("admin_kick_menu"))
		{
			StringTokenizer st = new StringTokenizer(command);
			if(st.countTokens() > 1)
			{
				st.nextToken();
				String player = st.nextToken();
				L2Player plyr = L2World.getPlayer(player);
				if(plyr != null)
					plyr.kick(false);
				if(plyr != null)
					activeChar.sendMessage("You kicked " + plyr.getName() + " from the game.");
				else
					activeChar.sendMessage("Player " + player + " not found in game.");
			}
			AdminHelpPage.showHelpPage(activeChar, "charmanage.htm");
		}
		else if(command.startsWith("admin_ban_menu"))
		{
			StringTokenizer st = new StringTokenizer(command);
			if(st.countTokens() > 1)
			{
				st.nextToken();
				String player = st.nextToken();
				L2Player plyr = L2World.getPlayer(player);
				if(plyr != null)
				{
					LoginServerThread.getInstance().sendAccessLevel(plyr.getAccountName(), -100, "admin_ban_menu", -1);
					plyr.kick(false);
				}
			}
			AdminHelpPage.showHelpPage(activeChar, "charmanage.htm");
		}
		else if(command.startsWith("admin_unban_menu"))
		{
			StringTokenizer st = new StringTokenizer(command);
			if(st.countTokens() > 1)
			{
				st.nextToken();
				String player = st.nextToken();
				L2Player plyr = L2World.getPlayer(player);
				if(plyr != null)
					LoginServerThread.getInstance().sendAccessLevel(plyr.getAccountName(), 0, "admin_unban_menu", 0);
			}
			AdminHelpPage.showHelpPage(activeChar, "charmanage.htm");
		}
		return true;
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	private void handleKill(L2Player activeChar)
	{
		handleKill(activeChar, null);
	}

	private void handleKill(L2Player activeChar, String player)
	{
		L2Object obj = activeChar.getTarget();
		if(player != null)
		{
			L2Player plyr = L2World.getPlayer(player);
			if(plyr != null)
				obj = plyr;
			if(plyr != null)
				activeChar.sendMessage("You kicked " + plyr.getName() + " from the game.");
			else
				activeChar.sendMessage("Player " + player + " not found in game.");
		}

		if(obj != null && obj.isCharacter)
		{
			L2Character target = (L2Character) obj;
			target.reduceCurrentHp(target.getMaxHp() + 1, activeChar);
		}
		else
			activeChar.sendPacket(Msg.INCORRECT_TARGET);
		AdminHelpPage.showHelpPage(activeChar, "charmanage.htm");
	}

	private void teleportCharacter(L2Player player, int x, int y, int z, L2Player activeChar)
	{
		if(player != null)
		{
			player.sendMessage("Admin is teleporting you.");
			player.teleToLocation(x, y, z);
		}
		AdminHelpPage.showHelpPage(activeChar, "charmanage.htm");
	}

	private void teleportToCharacter(L2Player activeChar, L2Object target)
	{
		L2Player player = null;
		if(target != null && target.isPlayer)
			player = (L2Player) target;
		else
		{
			activeChar.sendPacket(Msg.INCORRECT_TARGET);
			return;
		}

		if(player.getObjectId() == activeChar.getObjectId())
			activeChar.sendMessage("You cannot self teleport.");
		else
		{
			int x = player.getX();
			int y = player.getY();
			int z = player.getZ();

			activeChar.teleToLocation(x, y, z);

			activeChar.sendMessage("You have teleported to character " + player.getName() + ".");
		}
		AdminHelpPage.showHelpPage(activeChar, "charmanage.htm");
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] {
				"Описание команды char_manage",
				"Описание команды teleport_character_to_menu",
				"Описание команды recall_char_menu", "Описание команды goto_char_menu",
				"Описание команды kick_menu", "Описание команды kill_menu",
				"Описание команды ban_menu", "Описание команды unban_menu" };
	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_NONE, AdminCommandHandler.GROUP_NONE,
				AdminCommandHandler.GROUP_NONE, AdminCommandHandler.GROUP_NONE,
				AdminCommandHandler.GROUP_NONE, AdminCommandHandler.GROUP_NONE,
				AdminCommandHandler.GROUP_NONE, AdminCommandHandler.GROUP_NONE };
	}
}