package commands.admin;

import L2jFT.database.DatabaseUtils;
import L2jFT.database.FiltredPreparedStatement;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.database.ThreadConnection;
import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.handler.AdminCommandHandler;
import L2jFT.game.handler.IAdminCommandHandler;
import L2jFT.game.instancemanager.SiegeManager;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2Player;
import L2jFT.game.network.serverpackets.PledgeShowInfoUpdate;
import L2jFT.game.network.serverpackets.PledgeStatusChanged;
import L2jFT.game.network.serverpackets.SocialAction;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.network.serverpackets.UserInfo;
import L2jFT.game.tables.ClanTable;

import java.util.StringTokenizer;

/**
 * Pledge Manipulation //pledge <create|dismiss|setlevel>
 */
public class AdminPledge implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = { "admin_pledge" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(activeChar.getPlayerAccess() == null || !activeChar.getPlayerAccess().CanEditChar || activeChar.getTarget() == null || !activeChar.getTarget().isPlayer)
			return false;

		L2Player target = (L2Player) activeChar.getTarget();
		if(!(activeChar == target || activeChar.getPlayerAccess().CanEditCharAll))
			return false;
		if(command.startsWith("admin_pledge"))
		{
			StringTokenizer st = new StringTokenizer(command);
			st.nextToken();

			String action = st.nextToken(); // create|dismiss|setlevel

			if(action.equals("create"))
				try
				{
					String pledgeName = st.nextToken();
					L2Clan clan = ClanTable.getInstance().createClan(target, pledgeName);
					if(clan != null)
					{
						target.sendPacket(new PledgeShowInfoUpdate(clan));
						target.sendPacket(new UserInfo(target));
						target.sendPacket(new SystemMessage(SystemMessage.CLAN_HAS_BEEN_CREATED));
						return true;
					}
				}
				catch(Exception e)
				{}
			else if(action.equals("dismiss"))
			{
				if(target.getClan() == null || target.getObjectId() != target.getClan().getLeaderId())
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.ONLY_THE_CLAN_LEADER_IS_ENABLED));
					return false;
				}

				SiegeManager.getInstance().removeSiegeSkills(target);
				SystemMessage sm = new SystemMessage(SystemMessage.CLAN_HAS_DISPERSED);
				for(L2Player clanMember : target.getClan().getOnlineMembers(null))
				{
					clanMember.setClan(null);
					clanMember.setTitle(null);
					clanMember.sendPacket(sm);
					clanMember.broadcastUserInfo();
				}

				ThreadConnection con = null;
				FiltredPreparedStatement statement = null;
				try
				{
					con = L2DatabaseFactory.getInstance().getConnection();
					statement = con.prepareStatement("UPDATE characters SET clanid = 0 WHERE clanid=?");
					statement.setInt(1, target.getClanId());
					statement.execute();
					DatabaseUtils.closeStatement(statement);

					statement = con.prepareStatement("DELETE FROM clan_data WHERE clan_id=?");
					statement.setInt(1, target.getClanId());
					statement.execute();
					DatabaseUtils.closeStatement(statement);
					statement = null;
					target.sendPacket(sm);
					target.broadcastUserInfo();
				}
				catch(Exception e)
				{}
				finally
				{
					DatabaseUtils.closeDatabaseCS(con, statement);
				}
				return true;
			}
			else if(action.equals("setlevel"))
			{
				if(target.getClan() == null || target.getObjectId() != target.getClan().getLeaderId())
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.ONLY_THE_CLAN_LEADER_IS_ENABLED));
					return false;
				}

				try
				{
					byte level = Byte.parseByte(st.nextToken());

					ThreadConnection con = null;
					FiltredPreparedStatement statement = null;
					try
					{
						con = L2DatabaseFactory.getInstance().getConnection();
						statement = con.prepareStatement("UPDATE clan_data SET clan_level = ? WHERE clan_id = ?");
						statement.setInt(1, level);
						statement.setInt(2, target.getClanId());
						statement.execute();
					}
					catch(Exception e)
					{}
					finally
					{
						DatabaseUtils.closeDatabaseCS(con, statement);
					}

					activeChar.sendMessage("You set level " + level + " for clan " + target.getClan().getName());
					target.getClan().setLevel(level);
					target.broadcastPacket(new SocialAction(target.getObjectId(), 16));

					if(level < 4)
						SiegeManager.getInstance().removeSiegeSkills(target);
					else if(level > 3)
						SiegeManager.getInstance().addSiegeSkills(target);
					
					if(level == 5) 
						target.sendPacket(new SystemMessage(SystemMessage.NOW_THAT_YOUR_CLAN_LEVEL_IS_ABOVE_LEVEL_5_IT_CAN_ACCUMULATE_CLAN_REPUTATION_POINTS));

					SystemMessage sm = new SystemMessage(SystemMessage.CLANS_SKILL_LEVEL_HAS_INCREASED);
					PledgeShowInfoUpdate pu = new PledgeShowInfoUpdate(target.getClan());
					
					for(L2Player member : target.getClan().getOnlineMembers(null))
					{
						member.updatePledgeClass();
						member.sendPacket(sm);
						member.sendPacket(pu);
						member.broadcastUserInfo();
					}

					target.getClan().broadcastToOnlineMembers(new PledgeStatusChanged(target.getClan()));
					target.getClan().broadcastClanStatus();

					return true;
				}
				catch(Exception e)
				{}
			}
			else if(action.equals("resetcreate"))
			{
				if(target.getClan() == null)
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.INCORRECT_TARGET));
					return false;
				}
				target.getClan().setExpelledMemberTime(0);
				activeChar.sendMessage("The penalty for creating a clan has been lifted for" + target.getName());
			}
			else if(action.equals("resetwait"))
			{
				target.setLeaveClanTime(0);
				activeChar.sendMessage("The penalty for leaving a clan has been lifted for " + target.getName());
			}
		}

		return false;
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] { "file://com_pledge.htm" };
	}

	public String[] getCommandGroup()
	{
		return new String[] { AdminCommandHandler.GROUP_MANAGESERVER4 };
	}
}
