package commands.admin;

import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.cache.Msg;
import L2jFT.game.handler.AdminCommandHandler;
import L2jFT.game.handler.IAdminCommandHandler;
import L2jFT.game.instancemanager.CastleManager;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.entity.Castle;
import L2jFT.game.network.serverpackets.NpcHtmlMessage;
import L2jFT.game.tables.ClanTable;

import java.util.StringTokenizer;

public class AdminSiege implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = {
			"admin_siege", "admin_add_attacker", "admin_add_defender",
			"admin_add_guard", "admin_list_siege_clans", "admin_clear_siege_list",
			"admin_move_defenders", "admin_spawn_doors", "admin_endsiege",
			"admin_startsiege", "admin_setcastle", "admin_castledel" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().CanEditNPC)
			return false;

		StringTokenizer st = new StringTokenizer(command, " ");
		command = st.nextToken();

		Castle castle = null;
		if(st.hasMoreTokens())
			castle = CastleManager.getInstance().getCastleByName(st.nextToken());

		if(castle == null || castle.getCastleId() < 0)
			showCastleSelectPage(activeChar);
		else
		{
			L2Object target = activeChar.getTarget();
			L2Player player = null;
			if(target != null)
			{
				if(target.isPlayer)
					player = (L2Player) target;
			}
			else
				player = activeChar;

			if(command.equalsIgnoreCase("admin_add_attacker"))
			{
				if(player == null)
					activeChar.sendPacket(Msg.TARGET_IS_INCORRECT);
				else
					castle.getSiege().registerAttacker(player, true);
			}
			else if(command.equalsIgnoreCase("admin_add_defender"))
			{
				if(player == null)
					activeChar.sendPacket(Msg.TARGET_IS_INCORRECT);
				else
					castle.getSiege().registerDefender(player, true);
			}
			else if(command.equalsIgnoreCase("admin_add_guard"))
			{
				// Get value
				String val = "";
				if(st.hasMoreTokens())
					val = st.nextToken();

				if(!val.equals(""))
					try
					{
						int npcId = Integer.parseInt(val);
						castle.getSiege().getSiegeGuardManager().addSiegeGuard(activeChar, npcId);
					}
					catch(Exception e)
					{
						activeChar.sendMessage("Value entered for Npc Id wasn't an integer");
					}
				else
					activeChar.sendMessage("Missing Npc Id");
			}
			else if(command.equalsIgnoreCase("admin_clear_siege_list"))
				castle.getSiege().getDatabase().clearSiegeClan();
			else if(command.equalsIgnoreCase("admin_endsiege"))
				castle.getSiege().endSiege();
			else if(command.equalsIgnoreCase("admin_list_siege_clans"))
			{
				castle.getSiege().listRegisterClan(activeChar);
				return true;
			}
			else if(command.equalsIgnoreCase("admin_move_defenders"))
				activeChar.sendPacket(Msg.NOT_WORKING_PLEASE_TRY_AGAIN_LATER);
			else if(command.equalsIgnoreCase("admin_setcastle"))
			{
				if(player == null)
					activeChar.sendPacket(Msg.TARGET_IS_INCORRECT);
				else
				{
					castle.setOwner(player.getClan());
					System.out.println("Castle " + castle.getName() + " owned by clan " + player.getClan().getName());
				}
			}
			else if(command.equalsIgnoreCase("admin_castledel"))
				castle.setOwner(null);
			else if(command.equalsIgnoreCase("admin_spawn_doors"))
				castle.spawnDoor();
			else if(command.equalsIgnoreCase("admin_startsiege"))
				castle.getSiege().startSiege();

			showSiegePage(activeChar, castle.getName());
		}

		return true;
	}

	public void showCastleSelectPage(L2Player activeChar)
	{
		NpcHtmlMessage adminReply = new NpcHtmlMessage(5);

		StringBuffer replyMSG = new StringBuffer("<html><body>");
		replyMSG.append("<table width=260><tr>");
		replyMSG.append("<td width=40><button value=\"Main\" action=\"bypass -h admin_admin\" width=40 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td>");
		replyMSG.append("<td width=180><center><font color=\"LEVEL\">Siege Castles</font></center></td>");
		replyMSG.append("<td width=40><button value=\"Back\" action=\"bypass -h admin_admin\" width=40 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td>");
		replyMSG.append("</tr></table><br>");

		replyMSG.append("<table width=260>");
		replyMSG.append("<tr><td>Castle Name</td><td>Owner</td></tr>");

		for(Castle castle : CastleManager.getInstance().getCastles().values())
			if(castle != null)
			{
				replyMSG.append("<tr><td>");
				replyMSG.append("<a action=\"bypass -h admin_siege " + castle.getName() + "\">" + castle.getName() + "</a>");
				replyMSG.append("</td><td>");

				L2Clan owner = castle.getOwnerId() == 0 ? null : ClanTable.getInstance().getClan(castle.getOwnerId());
				if(owner == null)
					replyMSG.append("NPC");
				else
					replyMSG.append(owner.getName());

				replyMSG.append("</td></tr>");
			}

		replyMSG.append("</table>");
		replyMSG.append("</body></html>");

		adminReply.setHtml(replyMSG.toString());
		activeChar.sendPacket(adminReply);
	}

	public void showSiegePage(L2Player activeChar, String castleName)
	{
		NpcHtmlMessage adminReply = new NpcHtmlMessage(5);
		StringBuffer replyMSG = new StringBuffer("<html><body>");
		replyMSG.append("<table width=260><tr>");
		replyMSG.append("<td width=40><button value=\"Main\" action=\"bypass -h admin_admin\" width=40 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td>");
		replyMSG.append("<td width=180><center>Siege Menu</center></td>");
		replyMSG.append("<td width=40><button value=\"Back\" action=\"bypass -h admin_siege\" width=40 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td>");
		replyMSG.append("</tr></table>");
		replyMSG.append("<center>");
		replyMSG.append("<br><br><br>Castle: " + castleName + "<br><br>");
		replyMSG.append("Castle Owner: ");
		Castle castle = CastleManager.getInstance().getCastleByName(castleName);

		L2Clan owner = castle.getOwnerId() == 0 ? null : ClanTable.getInstance().getClan(castle.getOwnerId());
		if(owner == null)
			replyMSG.append("NPC");
		else
			replyMSG.append(owner.getName());

		replyMSG.append("<br><br><table>");
		replyMSG.append("<tr><td><button value=\"Add Attacker\" action=\"bypass -h admin_add_attacker " + castleName + "\" width=80 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td>");
		replyMSG.append("<td><button value=\"Add Defender\" action=\"bypass -h admin_add_defender " + castleName + "\" width=80 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td></tr>");
		replyMSG.append("<tr><td><button value=\"List Clans\" action=\"bypass -h admin_list_siege_clans " + castleName + "\" width=80 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td>");
		replyMSG.append("<td><button value=\"Clear List\" action=\"bypass -h admin_clear_siege_list " + castleName + "\" width=80 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td></tr>");
		replyMSG.append("</table>");
		replyMSG.append("<br>");
		replyMSG.append("<table>");
		replyMSG.append("<tr><td><button value=\"Move Defenders\" action=\"bypass -h admin_move_defenders " + castleName + "\" width=80 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td>");
		replyMSG.append("<td><button value=\"Spawn Doors\" action=\"bypass -h admin_spawn_doors " + castleName + "\" width=80 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td></tr>");
		replyMSG.append("</table>");
		replyMSG.append("<br>");
		replyMSG.append("<table>");
		replyMSG.append("<tr><td><button value=\"Start Siege\" action=\"bypass -h admin_startsiege " + castleName + "\" width=80 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td>");
		replyMSG.append("<td><button value=\"End Siege\" action=\"bypass -h admin_endsiege " + castleName + "\" width=80 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td></tr>");
		replyMSG.append("</table>");
		replyMSG.append("<br>");
		replyMSG.append("<table>");
		replyMSG.append("<tr><td><button value=\"Give Castle\" action=\"bypass -h admin_setcastle " + castleName + "\" width=80 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td>");
		replyMSG.append("<td><button value=\"Take Castle\" action=\"bypass -h admin_castledel " + castleName + "\" width=80 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td></tr>");
		replyMSG.append("</table>");
		replyMSG.append("<br>");
		replyMSG.append("<table>");
		replyMSG.append("<tr><td>NpcId: <edit var=\"value\" width=40>");
		replyMSG.append("<td><button value=\"Add Guard\" action=\"bypass -h admin_add_guard " + castleName + " $value\" width=80 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td></tr>");
		replyMSG.append("</table>");
		replyMSG.append("</center>");
		replyMSG.append("</body></html>");

		adminReply.setHtml(replyMSG.toString());
		activeChar.sendPacket(adminReply);
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] {
				"Описание команды siege", "Описание команды add_attacker",
				"Описание команды add_defender", "Описание команды add_guard",
				"Описание команды list_siege_clans",
				"Описание команды clear_siege_list", "Описание команды move_defenders",
				"Описание команды spawn_doors", "Описание команды endsiege",
				"Описание команды startsiege", "Описание команды setcastle",
				"Описание команды castledel" };
	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_MANAGESERVER5,
				AdminCommandHandler.GROUP_MANAGESERVER5,
				AdminCommandHandler.GROUP_MANAGESERVER5,
				AdminCommandHandler.GROUP_MANAGESERVER5,
				AdminCommandHandler.GROUP_MANAGESERVER5,
				AdminCommandHandler.GROUP_MANAGESERVER5,
				AdminCommandHandler.GROUP_MANAGESERVER5,
				AdminCommandHandler.GROUP_MANAGESERVER5,
				AdminCommandHandler.GROUP_MANAGESERVER5,
				AdminCommandHandler.GROUP_MANAGESERVER5,
				AdminCommandHandler.GROUP_MANAGESERVER5,
				AdminCommandHandler.GROUP_MANAGESERVER5 };
	}
}