package commands.admin;

import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.handler.AdminCommandHandler;
import L2jFT.game.handler.IAdminCommandHandler;
import L2jFT.game.instancemanager.DayNightSpawnManager;
import L2jFT.game.instancemanager.RaidBossSpawnManager;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Spawn;
import L2jFT.game.model.L2World;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.GmListTable;
import L2jFT.game.tables.NpcTable;
import L2jFT.game.tables.SpawnTable;
import L2jFT.game.templates.L2NpcTemplate;
import L2jFT.util.Log;

import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * This class handles following admin commands: - show_spawns = shows menu -
 * spawn_index lvl = shows menu for monsters with respective level -
 * spawn_monster id = spawns monster id on target spawn1 id count - заспавнить
 * count мобов только 1 раз
 */
public class AdminSpawn implements IAdminCommandHandler, ScriptFile
{

	private static String[] _adminCommands = {
			"admin_show_spawns", "admin_spawn", "admin_spawn_monster",
			"admin_spawn_index", "admin_unspawnall", "admin_spawn1" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().CanEditNPC)
			return false;

		if(command.equals("admin_show_spawns"))
			AdminHelpPage.showHelpPage(activeChar, "spawns.htm");
		else if(command.startsWith("admin_spawn_index"))
			try
			{
				String val = command.substring(18);
				AdminHelpPage.showHelpPage(activeChar, "spawns/" + val + ".htm");
			}
			catch(StringIndexOutOfBoundsException e)
			{}
		else if(command.startsWith("admin_spawn1"))
		{
			StringTokenizer st = new StringTokenizer(command, " ");
			try
			{
				st.nextToken();
				String id = st.nextToken();
				int mobCount = 1;
				if(st.hasMoreTokens())
					mobCount = Integer.parseInt(st.nextToken());
				spawnMonster(activeChar, id, 0, mobCount);
			}
			catch(Exception e)
			{
				// Case of wrong monster data
			}
		}
		else if(command.startsWith("admin_spawn") || command.startsWith("admin_spawn_monster"))
		{
			StringTokenizer st = new StringTokenizer(command, " ");
			try
			{
				st.nextToken();
				String id = st.nextToken();
				int respawnTime = 30;
				int mobCount = 1;
				if(st.hasMoreTokens())
					mobCount = Integer.parseInt(st.nextToken());
				if(st.hasMoreTokens())
					respawnTime = Integer.parseInt(st.nextToken());
				spawnMonster(activeChar, id, respawnTime, mobCount);
			}
			catch(Exception e)
			{
				// Case of wrong monster data
			}
		}
		else if(command.startsWith("admin_unspawnall"))
		{
			for(L2Player player : L2World.getAllPlayers())
				player.sendPacket(new SystemMessage(SystemMessage.THE_NPC_SERVER_IS_NOT_OPERATING));

			RaidBossSpawnManager.getInstance().cleanUp();
			DayNightSpawnManager.getInstance().cleanUp();
			L2World.deleteVisibleNpcSpawns();
			GmListTable.broadcastMessageToGMs("NPC Unspawn completed!");
		}

		return true;
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	private void spawnMonster(L2Player activeChar, String monsterId, int respawnTime, int mobCount)
	{
		L2Object target = activeChar.getTarget();
		if(target == null)
			target = activeChar;

		Pattern pattern = Pattern.compile("[0-9]*");
		Matcher regexp = pattern.matcher(monsterId);
		L2NpcTemplate template;
		if(regexp.matches())
		{
			// First parameter was an ID number
			int monsterTemplate = Integer.parseInt(monsterId);
			template = NpcTable.getTemplate(monsterTemplate);
		}
		else
		{
			// First parameter wasn't just numbers so go by name not ID
			monsterId = monsterId.replace('_', ' ');
			template = NpcTable.getTemplateByName(monsterId);
		}

		if(template == null)
		{
			activeChar.sendMessage("Incorrect monster template.");
			return;
		}

		try
		{
			L2Spawn spawn = new L2Spawn(template);
			spawn.setLocx(target.getX());
			spawn.setLocy(target.getY());
			spawn.setLocz(target.getZ());
			spawn.setLocation(0);
			spawn.setAmount(mobCount);
			spawn.setHeading(activeChar.getHeading());
			spawn.setRespawnDelay(respawnTime);

			if(RaidBossSpawnManager.getInstance().isDefined(spawn.getNpcid()))
				activeChar.sendMessage("Raid Boss " + template.name + " already spawned.");
			else
			{
				if(RaidBossSpawnManager.getInstance().getValidTemplate(spawn.getNpcid()) != null)
				{
					if(respawnTime != 0)
						RaidBossSpawnManager.getInstance().addNewSpawn(spawn, true);
				}
				else
					SpawnTable.getInstance().addNewSpawn(spawn, respawnTime != 0, activeChar);

				spawn.init();
				if(respawnTime == 0)
					spawn.stopRespawn();

				activeChar.sendMessage("Created " + template.name + " on " + target.getObjectId() + ".");

				Log.add("Created " + template.name + " on " + target.getObjectId(), "gm_ext_actions", activeChar);
			}
		}
		catch(Exception e)
		{
			activeChar.sendMessage("Target is not ingame.", "Цель не в игре.");
		}
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] {
				"admin_show_spawns", "admin_spawn", "admin_spawn_monster",
				"admin_spawn_index", "admin_unspawnall", "admin_spawn1" };
	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_MANAGESERVER6,
				AdminCommandHandler.GROUP_MANAGESERVER6,
				AdminCommandHandler.GROUP_MANAGESERVER6,
				AdminCommandHandler.GROUP_MANAGESERVER6,
				AdminCommandHandler.GROUP_MANAGESERVER6,
				AdminCommandHandler.GROUP_MANAGESERVER6 };
	}
}
