package commands.admin;

import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.ai.CtrlIntention;
import L2jFT.game.cache.Msg;
import L2jFT.game.handler.AdminCommandHandler;
import L2jFT.game.handler.IAdminCommandHandler;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Spawn;
import L2jFT.game.model.L2World;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.network.serverpackets.NpcHtmlMessage;
import L2jFT.game.tables.NpcTable;
import L2jFT.game.tables.SpawnTable;
import L2jFT.game.templates.L2NpcTemplate;
import L2jFT.util.CharPosition;
import L2jFT.util.Log;

import java.util.StringTokenizer;

/**
 * This class handles following admin commands: - show_moves - show_teleport -
 * teleport_to_character - move_to - teleport_character
 * 
 * @version $Revision: 1.3.2.6.2.4 $ $Date: 2005/04/11 10:06:06 $
 */
public class AdminTeleport implements IAdminCommandHandler, ScriptFile
{
	private static String[] _adminCommands = {
			"admin_show_moves", "admin_show_moves_other", "admin_show_teleport",
			"admin_teleport_to_character", "admin_teleportto", "admin_move_to",
			"admin_teleport_character", "admin_recall", "admin_walk",
			"admin_recall_npc", "admin_gonorth", "admin_gosouth", "admin_goeast",
			"admin_gowest", "admin_goup", "admin_godown", "admin_tele",
			"admin_teleto", "admin_failed", "admin_tonpc" };

	public boolean useAdminCommand(String command, L2Player activeChar)
	{
		if(!activeChar.getPlayerAccess().CanTeleport)
			return false;

		if(command.equals("admin_teleto"))
			activeChar.setTeleMode(1);
		if(command.equals("admin_teleto r"))
			activeChar.setTeleMode(2);
		if(command.equals("admin_teleto end"))
			activeChar.setTeleMode(0);
		if(command.equals("admin_show_moves"))
			AdminHelpPage.showHelpPage(activeChar, "teleports.htm");
		if(command.equals("admin_show_moves_other"))
			AdminHelpPage.showHelpPage(activeChar, "tele/other.htm");
		else if(command.equals("admin_show_teleport"))
			showTeleportCharWindow(activeChar);
		else if(command.equals("admin_recall_npc"))
			recallNPC(activeChar);
		else if(command.equals("admin_teleport_to_character"))
			teleportToCharacter(activeChar, activeChar.getTarget());
		else if(command.startsWith("admin_walk"))
			try
			{
				String val = command.substring(11);
				StringTokenizer st = new StringTokenizer(val);
				String x1 = st.nextToken();
				int x = Integer.parseInt(x1);
				String y1 = st.nextToken();
				int y = Integer.parseInt(y1);
				String z1 = st.nextToken();
				int z = Integer.parseInt(z1);
				CharPosition pos = new CharPosition(x, y, z, 0);
				activeChar.moveToLocation(pos.x, pos.y, pos.z, 0, true);
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
		else if(command.startsWith("admin_move_to"))
			try
			{
				String val = command.substring(14);
				teleportTo(activeChar, val);
			}
			catch(StringIndexOutOfBoundsException e)
			{ // Case of empty coordinates
				activeChar.sendMessage("Wrong or no Coordinates given.");
			}
		else if(command.startsWith("admin_teleport_character"))
			try
			{
				String val = command.substring(25);

				teleportCharacter(activeChar, val);
			}
			catch(StringIndexOutOfBoundsException e)
			{
				// Case of empty coordinates
				activeChar.sendMessage("Wrong or no Coordinates given.");
				showTeleportCharWindow(activeChar); // back to character teleport
			}
		else if(command.startsWith("admin_teleportto "))
			try
			{
				String targetName = command.substring(17);
				L2Player player = L2World.getPlayer(targetName);
				teleportToCharacter(activeChar, player);
			}
			catch(StringIndexOutOfBoundsException e)
			{}
		else if(command.startsWith("admin_recall "))
			try
			{
				String targetName = command.substring(13);
				L2Player player = L2World.getPlayer(targetName);
				if(player == null)
					activeChar.sendMessage("->" + targetName + "<- is incorrect.");
				else
				{
					teleportCharacter(player, activeChar.getX(), activeChar.getY(), activeChar.getZ());
					
					Log.add("teleport character " + player.getName() + " to " + activeChar.getX() + "," + activeChar.getY() + "," + activeChar.getZ(), "gm_ext_actions", activeChar);
				}
			}
			catch(StringIndexOutOfBoundsException e)
			{}
		else if(command.startsWith("admin_failed"))
		{
			activeChar.sendMessage("Trying ActionFailed...");
			activeChar.sendActionFailed();
		}
		else if(command.equals("admin_tele"))
			showTeleportWindow(activeChar);
		else if(command.equals("admin_goup"))
		{
			int x = activeChar.getX();
			int y = activeChar.getY();
			int z = activeChar.getZ() + 150;
			activeChar.teleToLocation(x, y, z);
			showTeleportWindow(activeChar);
		}
		else if(command.startsWith("admin_goup"))
			try
			{
				String val = command.substring(11);
				int intVal = Integer.parseInt(val);
				int x = activeChar.getX();
				int y = activeChar.getY();
				int z = activeChar.getZ() + intVal;
				activeChar.teleToLocation(x, y, z);
				showTeleportWindow(activeChar);
			}
			catch(StringIndexOutOfBoundsException e)
			{}
		else if(command.equals("admin_godown"))
		{
			int x = activeChar.getX();
			int y = activeChar.getY();
			int z = activeChar.getZ();
			activeChar.teleToLocation(x, y, z - 150);
			showTeleportWindow(activeChar);
		}
		else if(command.startsWith("admin_godown"))
			try
			{
				String val = command.substring(13);
				int intVal = Integer.parseInt(val);
				int x = activeChar.getX();
				int y = activeChar.getY();
				int z = activeChar.getZ() - intVal;
				activeChar.teleToLocation(x, y, z);
				showTeleportWindow(activeChar);
			}
			catch(StringIndexOutOfBoundsException e)
			{}
		else if(command.equals("admin_goeast"))
		{
			int x = activeChar.getX();
			int y = activeChar.getY();
			int z = activeChar.getZ();
			activeChar.teleToLocation(x + 150, y, z);
			showTeleportWindow(activeChar);
		}
		else if(command.startsWith("admin_goeast"))
			try
			{
				String val = command.substring(13);
				int intVal = Integer.parseInt(val);
				int x = activeChar.getX() + intVal;
				int y = activeChar.getY();
				int z = activeChar.getZ();
				activeChar.teleToLocation(x, y, z);
				showTeleportWindow(activeChar);
			}
			catch(StringIndexOutOfBoundsException e)
			{}
		else if(command.equals("admin_gowest"))
		{
			int x = activeChar.getX();
			int y = activeChar.getY();
			int z = activeChar.getZ();
			activeChar.teleToLocation(x - 150, y, z);
			showTeleportWindow(activeChar);
		}
		else if(command.startsWith("admin_gowest"))
			try
			{
				String val = command.substring(13);
				int intVal = Integer.parseInt(val);
				int x = activeChar.getX() - intVal;
				int y = activeChar.getY();
				int z = activeChar.getZ();
				activeChar.teleToLocation(x, y, z);
				showTeleportWindow(activeChar);
			}
			catch(StringIndexOutOfBoundsException e)
			{}
		else if(command.equals("admin_gosouth"))
		{
			int x = activeChar.getX();
			int y = activeChar.getY() + 150;
			int z = activeChar.getZ();
			activeChar.teleToLocation(x, y, z);
			showTeleportWindow(activeChar);
		}
		else if(command.startsWith("admin_gosouth"))
			try
			{
				String val = command.substring(14);
				int intVal = Integer.parseInt(val);
				int x = activeChar.getX();
				int y = activeChar.getY() + intVal;
				int z = activeChar.getZ();
				activeChar.teleToLocation(x, y, z);
				showTeleportWindow(activeChar);
			}
			catch(StringIndexOutOfBoundsException e)
			{}
		else if(command.equals("admin_gonorth"))
		{
			int x = activeChar.getX();
			int y = activeChar.getY();
			int z = activeChar.getZ();
			activeChar.teleToLocation(x, y - 150, z);
			showTeleportWindow(activeChar);
		}
		else if(command.startsWith("admin_gonorth"))
			try
			{
				String val = command.substring(14);
				int intVal = Integer.parseInt(val);
				int x = activeChar.getX();
				int y = activeChar.getY() - intVal;
				int z = activeChar.getZ();
				activeChar.teleToLocation(x, y, z);
				showTeleportWindow(activeChar);
			}
			catch(StringIndexOutOfBoundsException e)
			{}
		else if(command.startsWith("admin_tonpc"))
			try
			{
				Integer npc_id = Integer.parseInt(command.substring(12));
				L2NpcInstance npc = L2World.findNpcByNpcId(npc_id);
				teleportToCharacter(activeChar, npc);
			}
			catch(Exception e)
			{
				try
				{
					String npc_name = command.substring(12);
					L2NpcInstance npc = L2World.findNpcByName(npc_name);
					teleportToCharacter(activeChar, npc);
				}
				catch(Exception e2)
				{
					activeChar.sendMessage("Npc not found");
				}
			}

		return true;
	}

	public String[] getAdminCommandList()
	{
		return _adminCommands;
	}

	private void teleportTo(L2Player activeChar, String Cords)
	{
		StringTokenizer st = new StringTokenizer(Cords);
		if(st.countTokens() < 3)
		{
			activeChar.sendMessage("3 coordinates required to teleport");
			return;
		}

		String x1 = st.nextToken();
		int x = Integer.parseInt(x1);
		String y1 = st.nextToken();
		int y = Integer.parseInt(y1);
		String z1 = st.nextToken();
		int z = Integer.parseInt(z1);

		activeChar.getAI().setIntention(CtrlIntention.AI_INTENTION_IDLE, null, null);
		activeChar.teleToLocation(x, y, z);

		activeChar.sendMessage("You have been teleported to " + Cords);

		Log.add("teleported to " + Cords, "gm_ext_actions", activeChar);
	}

	private void showTeleportWindow(L2Player activeChar)
	{
		NpcHtmlMessage adminReply = new NpcHtmlMessage(5);

		StringBuffer replyMSG = new StringBuffer("<html><title>Teleport Menu</title>");
		replyMSG.append("<body>");

		replyMSG.append("<br>");
		replyMSG.append("<center><table>");

		replyMSG.append("<tr><td><button value=\"  \" action=\"bypass -h admin_tele\" width=70 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td>");
		replyMSG.append("<td><button value=\"North\" action=\"bypass -h admin_gonorth\" width=70 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td>");
		replyMSG.append("<td><button value=\"Up\" action=\"bypass -h admin_goup\" width=70 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td></tr>");
		replyMSG.append("<tr><td><button value=\"West\" action=\"bypass -h admin_gowest\" width=70 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td>");
		replyMSG.append("<td><button value=\"  \" action=\"bypass -h admin_tele\" width=70 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td>");
		replyMSG.append("<td><button value=\"East\" action=\"bypass -h admin_goeast\" width=70 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td></tr>");
		replyMSG.append("<tr><td><button value=\"  \" action=\"bypass -h admin_tele\" width=70 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td>");
		replyMSG.append("<td><button value=\"South\" action=\"bypass -h admin_gosouth\" width=70 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td>");
		replyMSG.append("<td><button value=\"Down\" action=\"bypass -h admin_godown\" width=70 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></td></tr>");

		replyMSG.append("</table></center>");
		replyMSG.append("</body></html>");

		adminReply.setHtml(replyMSG.toString());
		activeChar.sendPacket(adminReply);
	}

	private void showTeleportCharWindow(L2Player activeChar)
	{
		L2Object target = activeChar.getTarget();
		L2Player player = null;
		if(target.isPlayer)
			player = (L2Player) target;
		else
		{
			activeChar.sendPacket(Msg.INCORRECT_TARGET);
			return;
		}
		NpcHtmlMessage adminReply = new NpcHtmlMessage(5);

		StringBuffer replyMSG = new StringBuffer("<html><title>Teleport Character</title>");
		replyMSG.append("<body>");
		replyMSG.append("The character you will teleport is " + player.getName() + ".");
		replyMSG.append("<br>");

		replyMSG.append("Co-ordinate x");
		replyMSG.append("<edit var=\"char_cord_x\" width=110>");
		replyMSG.append("Co-ordinate y");
		replyMSG.append("<edit var=\"char_cord_y\" width=110>");
		replyMSG.append("Co-ordinate z");
		replyMSG.append("<edit var=\"char_cord_z\" width=110>");
		replyMSG.append("<button value=\"Teleport\" action=\"bypass -h admin_teleport_character $char_cord_x $char_cord_y $char_cord_z\" width=60 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\">");
		replyMSG.append("<button value=\"Teleport near you\" action=\"bypass -h admin_teleport_character " + activeChar.getX() + " " + activeChar.getY() + " " + activeChar.getZ() + "\" width=115 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\">");
		replyMSG.append("<center><button value=\"Back\" action=\"bypass -h admin_current_player\" width=40 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></center>");
		replyMSG.append("</body></html>");

		adminReply.setHtml(replyMSG.toString());
		activeChar.sendPacket(adminReply);
	}

	private void teleportCharacter(L2Player activeChar, String Cords)
	{
		L2Object target = activeChar.getTarget();
		L2Player player = null;
		if(target.isPlayer)
			player = (L2Player) target;
		else
		{
			activeChar.sendPacket(Msg.INCORRECT_TARGET);
			return;
		}

		if(player.getObjectId() == activeChar.getObjectId())
			player.sendMessage("You cannot teleport your character.");
		else
		{
			StringTokenizer st = new StringTokenizer(Cords);
			String x1 = st.nextToken();
			int x = Integer.parseInt(x1);
			String y1 = st.nextToken();
			int y = Integer.parseInt(y1);
			String z1 = st.nextToken();
			int z = Integer.parseInt(z1);
			teleportCharacter(player, x, y, z);
		}
	}

	/**
	 * @param player
	 * @param x
	 * @param y
	 * @param z
	 */
	private void teleportCharacter(L2Player player, int x, int y, int z)
	{
		if(player != null)
		{
			// Common character information
			player.sendMessage("Admin is teleporting you.");

			player.getAI().setIntention(CtrlIntention.AI_INTENTION_IDLE, null, null);
			player.teleToLocation(x, y, z);
		}
	}

	private void teleportToCharacter(L2Player activeChar, L2Object target)
	{
		if(target == null)
			return;

		int x = target.getX();
		int y = target.getY();
		int z = target.getZ();

		activeChar.getAI().setIntention(CtrlIntention.AI_INTENTION_IDLE, null, null);
		activeChar.teleToLocation(x, y, z);

		activeChar.sendMessage("You have teleported to " + target);
	}

	private void recallNPC(L2Player activeChar)
	{
		L2Object obj = activeChar.getTarget();
		if(obj != null && obj instanceof L2NpcInstance)
		{
			L2NpcInstance target = (L2NpcInstance) obj;
			L2Spawn spawn = target.getSpawn();

			int monsterTemplate = target.getTemplate().npcId;

			L2NpcTemplate template1 = NpcTable.getTemplate(monsterTemplate);

			if(template1 == null)
			{
				activeChar.sendMessage("Incorrect monster template.");
				return;
			}

			int respawnTime = spawn.getRespawnDelay();

			target.deleteMe();
			spawn.stopRespawn();
			SpawnTable.getInstance().deleteSpawn(spawn, true);

			try
			{
				// L2MonsterInstance mob = new L2MonsterInstance(monsterTemplate,
				// template1);

				spawn = new L2Spawn(template1);
				spawn.setLocx(activeChar.getX());
				spawn.setLocy(activeChar.getY());
				spawn.setLocz(activeChar.getZ());
				spawn.setAmount(1);
				spawn.setHeading(activeChar.getHeading());
				spawn.setRespawnDelay(respawnTime);
				SpawnTable.getInstance().addNewSpawn(spawn, true, activeChar);
				spawn.init();

				activeChar.sendMessage("Created " + template1.name + " on " + target.getObjectId() + ".");

				Log.add("GM: " + activeChar.getName() + "(" + activeChar.getObjectId() + ") moved NPC" + target.getObjectId(), "gm_ext_actions", activeChar);
			}
			catch(Exception e)
			{
				activeChar.sendMessage("Target is not in game.");
			}

		}
		else
			activeChar.sendPacket(Msg.INCORRECT_TARGET);
	}

	public void onLoad()
	{
		AdminCommandHandler.getInstance().registerAdminCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public String[] getDescription()
	{
		return new String[] {
				"Описание команды show_moves", "Описание команды show_moves_other",
				"Описание команды show_teleport",
				"Описание команды teleport_to_character",
				"Описание команды teleportto", "Описание команды move_to",
				"Описание команды teleport_character", "Описание команды recall",
				"Описание команды walk", "Описание команды recall_npc",
				"Описание команды gonorth", "Описание команды gosouth",
				"Описание команды goeast", "Описание команды gowest",
				"Описание команды goup", "Описание команды godown",
				"Описание команды tele", "Описание команды teleto",
				"Описание команды failed", "Описание команды tonpc" };
	}

	public String[] getCommandGroup()
	{
		return new String[] {
				AdminCommandHandler.GROUP_TELEPORT, AdminCommandHandler.GROUP_TELEPORT,
				AdminCommandHandler.GROUP_TELEPORT, AdminCommandHandler.GROUP_TELEPORT,
				AdminCommandHandler.GROUP_TELEPORT, AdminCommandHandler.GROUP_TELEPORT,
				AdminCommandHandler.GROUP_TELEPORT, AdminCommandHandler.GROUP_TELEPORT,
				AdminCommandHandler.GROUP_TELEPORT, AdminCommandHandler.GROUP_TELEPORT,
				AdminCommandHandler.GROUP_TELEPORT, AdminCommandHandler.GROUP_TELEPORT,
				AdminCommandHandler.GROUP_TELEPORT, AdminCommandHandler.GROUP_TELEPORT,
				AdminCommandHandler.GROUP_TELEPORT, AdminCommandHandler.GROUP_TELEPORT,
				AdminCommandHandler.GROUP_TELEPORT, AdminCommandHandler.GROUP_TELEPORT,
				AdminCommandHandler.GROUP_TELEPORT, AdminCommandHandler.GROUP_TELEPORT };
	}
}