package commands.voiced;

import static L2jFT.game.model.L2Zone.ZoneType.OlympiadStadia;
import static L2jFT.game.model.L2Zone.ZoneType.Siege;
import static L2jFT.game.model.L2Zone.ZoneType.no_restart;
import L2jFT.extensions.scripts.Functions;
import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.LoginServerThread;
import L2jFT.game.cache.Msg;
import L2jFT.game.geodata.GeoData;
import L2jFT.game.handler.IVoicedCommandHandler;
import L2jFT.game.handler.VoicedCommandHandler;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.network.serverpackets.ExRestartClient;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.util.CharPosition;
import L2jFT.util.Rnd;

public class Relocate extends Functions implements IVoicedCommandHandler, ScriptFile
{
	public static L2Object self;
	public static L2NpcInstance npc;
	public static int SUMMON_PRICE = 200000;

	private final String[] _commandList = new String[] {
			"relocate", "km-all-to-me", "ghost" };

	public String[] getVoicedCommandList()
	{
		return _commandList;
	}

	public boolean useVoicedCommand(String command, L2Player activeChar, String target)
	{
		if(command.equalsIgnoreCase("relocate"))
		{
			if(activeChar.isParalyzed())
			{
				activeChar.sendMessage("You are paralyzed!");
				return true;
			}
			activeChar.setParalyzed(true);
			if(activeChar.getPlayerAccess().CanTeleport)
			{
				show("You are paralyzed for 3 sec.", activeChar);
				executeTask(activeChar, "commands.voiced.Relocate", "relocate", new Object[0], 3000);
			}
			else
			{
				show("You are paralyzed for 30 sec.", activeChar);
				executeTask(activeChar, "commands.voiced.Relocate", "relocate", new Object[0], 30000);
			}
			return true;
		}
		else if(command.equalsIgnoreCase("ghost"))
		{
			if(!activeChar.isInStoreMode())
				return false;
			activeChar.sendPacket(new ExRestartClient());
			LoginServerThread.getInstance().sendLogout(activeChar.getNetConnection().getLoginName());
			try
			{
				Thread.sleep(100);
			}
			catch(InterruptedException e)
			{
				e.printStackTrace();
			}
			activeChar.setConnected(false);
			return true;
		}
		else if(command.equalsIgnoreCase("km-all-to-me"))
		{
			if(!activeChar.isClanLeader())
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.ONLY_THE_CLAN_LEADER_IS_ENABLED));
				return false;
			}

			if(activeChar.isAlikeDead())
			{
				activeChar.sendMessage("You cannot use this command while dead.", "Вы не можете использовать эту команду мертвым.");
				return false;
			}

			// "Нельзя вызывать персонажей в/из зоны свободного PvP"
			// "в зоны осад"
			// "на Олимпийский стадион"
			// "в зоны определенных рейд-боссов и эпик-боссов"
			if(activeChar.isInZoneBattle() || activeChar.isInZone(Siege) || activeChar.isInZoneIncludeZ(no_restart))
				return false;

			boolean stop = false;
			L2Player[] clan = activeChar.getClan().getOnlineMembers(activeChar.getName());
			int price = 0;

			if(activeChar.getAdena() < SUMMON_PRICE * clan.length)
			{
				activeChar.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ADENA);
				return false;
			}

			for(L2Player pl : clan)
			{
				if(stop)
					break;

				if(pl == null)
					continue;

				if(pl.isInZoneBattle() || pl.isInZone(Siege) || pl.isInZone(OlympiadStadia) || pl.isInZoneIncludeZ(no_restart))
					continue;

				// Нельзя призывать торгующих, мертвых персонажей
				if(pl.getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE || pl.isTransactionInProgress() || pl.isAlikeDead())
					continue;

				// Нельзя призывать персонажей, которые находятся в режиме PvP
				if(pl.getPvpFlag() != 0)
					continue;

				// Снимаем адену за каждого призываемого члена клана
				price += SUMMON_PRICE;

				// Пытаемся найти координаты с разбросом, если не получается - берем без разброса 
				int[] coords = new int[3];
				for(int i = 0; i < 100; i++)
				{
					CharPosition cp = Rnd.coordsRandomize(activeChar, 150);
					if(GeoData.getInstance().canMoveToCoord(cp.x, cp.y, cp.z, activeChar.getX(), activeChar.getY(), activeChar.getZ()))
					{
						coords[0] = cp.x;
						coords[1] = cp.y;
						coords[2] = cp.z;
						break;
					}
				}
				if(coords[0] == 0)
				{
					coords[0] = activeChar.getX();
					coords[1] = activeChar.getY();
					coords[2] = activeChar.getZ();
				}

				// Спрашиваем, согласие на призыв
				pl.summonCharacterRequest(activeChar.getName(), coords);
			}

			if(price != 0)
			{
				activeChar.reduceAdena(price);
				activeChar.sendPacket(new SystemMessage(SystemMessage.S1_ADENA_DISAPPEARED).addNumber(price));
			}

			return true;
		}
		return false;
	}

	public void relocate()
	{
		((L2Player) self).teleToLocation(self.getX(), self.getY(), self.getZ());
		((L2Character) self).setParalyzed(false);
	}

	public void onLoad()
	{
		VoicedCommandHandler.getInstance().registerVoicedCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}
}