package commands.voiced;

import L2jFT.Config;
import L2jFT.database.DatabaseUtils;
import L2jFT.database.FiltredPreparedStatement;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.database.ThreadConnection;
import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.ThreadPoolManager;
import L2jFT.game.ai.CtrlIntention;
import L2jFT.game.cache.Msg;
import L2jFT.game.handler.IVoicedCommandHandler;
import L2jFT.game.handler.VoicedCommandHandler;
import L2jFT.game.instancemanager.CoupleManager;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.L2World;
import L2jFT.game.model.entity.Couple;
import L2jFT.game.network.serverpackets.ConfirmDlg;
import L2jFT.game.network.serverpackets.MagicSkillUse;
import L2jFT.game.network.serverpackets.SetupGauge;
import L2jFT.game.network.serverpackets.StopMove;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.SkillTable;

import java.sql.ResultSet;

public class Wedding implements IVoicedCommandHandler, ScriptFile
{
	private static String[] _voicedCommands = { "divorce", "engage", "gotolove" };

	/*
	 * (non-Javadoc)
	 * 
	 * @see L2jFT.game.handler.IUserCommandHandler#useUserCommand(int,
	 *      L2jFT.game.model.L2Player)
	 */
	public boolean useVoicedCommand(String command, L2Player activeChar, String target)
	{
		if(command.startsWith("engage"))
			return engage(activeChar);
		else if(command.startsWith("divorce"))
			return divorce(activeChar);
		else if(command.startsWith("gotolove"))
			return goToLove(activeChar);
		return false;
	}

	public boolean divorce(L2Player activeChar)
	{
		if(activeChar.getPartnerId() == 0)
			return false;

		int _partnerId = activeChar.getPartnerId();
		int AdenaAmount = 0;

		if(activeChar.isMaried())
		{
			activeChar.sendMessage("You are divorced now.", "Вы развелись.");
			AdenaAmount = Math.abs(activeChar.getAdena() / 100 * Config.WEDDING_DIVORCE_COSTS - 10);
			activeChar.reduceAdena(AdenaAmount);
		}
		else
			activeChar.sendMessage("You are disengaged now.", "Помолвка разорвана.");

		activeChar.setMaried(false);
		activeChar.setPartnerId(0);
		Couple couple = CoupleManager.getInstance().getCouple(activeChar.getCoupleId());
		couple.divorce();
		couple = null;

		L2Player partner;
		partner = (L2Player) L2World.findObject(_partnerId);

		if(partner != null)
		{
			partner.setPartnerId(0);
			if(partner.isMaried())
				partner.sendMessage("Your Partner has decided to divorce from you.", "Ваш партнер решил развестись.");
			else
				partner.sendMessage("Your Partner has decided to disengage.", "Ваш партнер решил разорвать помолвку.");
			partner.setMaried(false);

			// give adena
			if(AdenaAmount > 0)
				partner.addAdena(AdenaAmount);
		}
		return true;
	}

	public boolean engage(L2Player activeChar)
	{
		// check target
		if(activeChar.getTarget() == null)
		{
			activeChar.sendMessage("You have none targeted.", "Не выбран игрок.");
			return false;
		}
		// check if target is a L2Player
		if(!activeChar.getTarget().isPlayer)
		{
			activeChar.sendMessage("You can only ask another Player for partnership.", "Вы можете спрашивать только игроков.");
			return false;
		}
		// check if player is already engaged
		if(activeChar.getPartnerId() != 0)
		{
			activeChar.sendMessage("You are already engaged.", "Вы уже обручены!");
			if(Config.WEDDING_PUNISH_INFIDELITY)
			{
				activeChar.startAbnormalEffect(0x2000); // give player a Big
				// Head
				// lets recycle the sevensigns debuffs
				short skillId;

				int skillLevel = 1;

				if(activeChar.getLevel() > 40)
					skillLevel = 2;

				if(activeChar.isMageClass())
					skillId = 4361;
				else
					skillId = 4362;

				L2Skill skill = SkillTable.getInstance().getInfo(skillId, skillLevel);

				if(activeChar.getEffect(skill) == null)
				{
					skill.getEffects(activeChar, activeChar, false);
					SystemMessage sm = new SystemMessage(SystemMessage.YOU_CAN_FEEL_S1S_EFFECT);
					sm.addSkillName(skillId, (short) skillLevel);
					activeChar.sendPacket(sm);
				}
			}
			return false;
		}

		L2Player ptarget = (L2Player) activeChar.getTarget();

		// check if player target himself
		if(ptarget.getObjectId() == activeChar.getObjectId())
		{
			activeChar.sendMessage("Engaging with yourself?", "Самому с собой нельзя.");
			return false;
		}

		if(ptarget.isMaried())
		{
			activeChar.sendMessage("Already married.", "Игрок уже женат.");
			return false;
		}

		if(ptarget.getPartnerId() != 0)
		{
			activeChar.sendMessage("Already engaged.", "Игрок уже помолвлен.");
			return false;
		}

		if(ptarget.isEngageRequest())
		{
			activeChar.sendMessage("Already asked by someone else.", "Игрок уже получил предложение от другого.");
			return false;
		}

		if(ptarget.getPartnerId() != 0)
		{
			activeChar.sendMessage("Is already engaged with someone else.", "Игрок уже помолвлен с кем-то другим.");
			return false;
		}

		if(ptarget.getSex() == activeChar.getSex() && !Config.WEDDING_SAMESEX)
		{
			activeChar.sendMessage("You cant ask partners of same sex.", "Однополые браки запрещены.");
			return false;
		}

		// check if target has player on friendlist
		boolean FoundOnFriendList = false;
		int objectId;
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			statement = con.prepareStatement("SELECT friend_id FROM character_friends WHERE char_id=?");
			statement.setInt(1, ptarget.getObjectId());
			rset = statement.executeQuery();

			while(rset.next())
			{
				objectId = rset.getInt("friend_id");
				if(objectId == activeChar.getObjectId())
				{
					FoundOnFriendList = true;
					break;
				}
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}

		if(!FoundOnFriendList)
		{
			activeChar.sendMessage("The Person you wanna ask hasn't added you on the friendlist.", "Игрок которому Вы делаете предложение не находится в Вашем списке друзей.");
			return false;
		}

		ptarget.setEngageRequest(true, activeChar.getObjectId());
		// ptarget.sendMessage("Player "+activeChar.getName()+" wants to engage with you.");
		ptarget.sendPacket(new ConfirmDlg(SystemMessage.S1_S2, "Player " + activeChar.getName() + " asking you to engage. Do you want to start new relationship?", 0, null, 4));
		return true;
	}

	public boolean goToLove(L2Player activeChar)
	{
		if(!activeChar.isMaried())
		{
			activeChar.sendMessage("You are not married.", "Вы не женаты (за мужем).");
			return false;
		}

		if(activeChar.getPartnerId() == 0)
		{
			activeChar.sendMessage("Couldn't find your Partner in Database - Inform a Gamemaster.", "Не удаеться найти вашу вторую половинку в базе, пожалуйста информируйте об этом ГМ.");
			return false;
		}

		L2Player partner;
		partner = (L2Player) L2World.findObject(activeChar.getPartnerId());
		if(partner == null)
		{
			activeChar.sendMessage("Youre Partner is not online.", "Ваш партнер не в игре.");
			return false;
		}

		if(partner.isInOlympiadMode() || partner.isFestivalParticipant() || activeChar.isMovementDisabled() || activeChar.isMuted() || activeChar.isInOlympiadMode() || activeChar.getDuel() != null || activeChar.isFestivalParticipant())
		{
			activeChar.sendMessage("Try later.", "Попробуйте позже.");
			return false;
		}

		if(activeChar.isInParty() && activeChar.getParty().isInDimensionalRift() || partner.isInParty() && partner.getParty().isInDimensionalRift())
		{
			activeChar.sendMessage("Try later.", "Попробуйте позже.");
			return false;
		}

		if(activeChar.getTeleMode() != 0 || activeChar.getUnstuck() != 0)
		{
			activeChar.sendMessage("Try later.", "Попробуйте позже.");
			return false;
		}

		/*
		 if(activeChar.getVarB("jailed?"))
		 {
		 activeChar.sendMessage("You are jailed.", "Вы не можете это сделать, находясь в тюрьме.");
		 return false;
		 }
		 */

		activeChar.abortCast();
		activeChar.abortAttack();
		activeChar.sendActionFailed();
		activeChar.broadcastPacket(new StopMove(activeChar));
		activeChar.setParalyzed(true);
		activeChar.setUnstuck(1);

		int teleportTimer = Config.WEDDING_TELEPORT_INTERVAL * 1000;

		if(activeChar.getInventory().getAdena() < Config.WEDDING_TELEPORT_PRICE)
		{
			activeChar.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ADENA);
			return false;
		}

		activeChar.reduceAdena(Config.WEDDING_TELEPORT_PRICE);

		activeChar.sendMessage("After " + teleportTimer / 60000 + " min. you will be teleported to your Love.", "Через " + teleportTimer / 60000 + " мин. Вы будете телепортированы к Вашей половинке.");

		activeChar.getAI().setIntention(CtrlIntention.AI_INTENTION_IDLE, null, null);

		// SoE Animation section
		activeChar.broadcastPacket(new MagicSkillUse(activeChar, activeChar, 1050, 1, teleportTimer, 0));
		activeChar.sendPacket(new SetupGauge(0, teleportTimer));
		// End SoE Animation section

		// continue execution later
		ThreadPoolManager.getInstance().scheduleGeneral(new EscapeFinalizer(activeChar, partner.getX(), partner.getY(), partner.getZ()), teleportTimer);
		return true;
	}

	static class EscapeFinalizer implements Runnable
	{
		private L2Player _activeChar;
		private int _partnerx;
		private int _partnery;
		private int _partnerz;

		EscapeFinalizer(L2Player activeChar, int x, int y, int z)
		{
			_activeChar = activeChar;
			_partnerx = x;
			_partnery = y;
			_partnerz = z;
		}

		public void run()
		{
			if(_activeChar.isDead() || _activeChar.getUnstuck() == 0)
				return;
			_activeChar.setParalyzed(false);
			_activeChar.setUnstuck(0);
			_activeChar.teleToLocation(_partnerx, _partnery, _partnerz);
		}
	}

	public String[] getVoicedCommandList()
	{
		return _voicedCommands;
	}

	public void onLoad()
	{
		if(Config.ALLOW_WEDDING)
			VoicedCommandHandler.getInstance().registerVoicedCommandHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}
}