package events;

import java.util.ArrayList;

import L2jFT.Config;
import L2jFT.database.mysql;
import L2jFT.extensions.multilang.CustomMessage;
import L2jFT.extensions.scripts.Functions;
import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.Announcements;
import L2jFT.game.cache.Msg;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Spawn;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.network.serverpackets.ItemList;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.ItemTable;
import L2jFT.game.tables.NpcTable;
import L2jFT.game.templates.L2NpcTemplate;
import L2jFT.util.Files;
import L2jFT.util.Util;

// Эвент Coffer of Shadows
public class CofferofShadows extends Functions implements ScriptFile
{
	public static L2Object self;
	public static L2Object npc;
	private static int COFFER_PRICE = 50000; // 50.000 adena at x1 servers
	private static int COFFER_ID = 8659;
	private static int EVENT_MANAGER_ID = 32091;
	private static ArrayList<L2Spawn> _spawns = new ArrayList<L2Spawn>();

	private static boolean _active = false;

	/**
	 * Спавнит эвент менеджеров
	 */
	private void spawnEventManagers()
	{
		final int EVENT_MANAGERS[][] = { { -14823, 123567, -3143, 8192 }, // Gludio
				{ -83159, 150914, -3155, 49152 }, // Gludin
				{ 18600, 145971, -3095, 7400 }, // Dion
				{ 82158, 148609, -3493, 60 }, // Giran
				{ 110992, 218753, -3568, 0 }, // Hiene
				{ 116339, 75424, -2738, 0 }, // Hunter Village
				{ 81140, 55218, -1551, 32768 }, // Oren
				{ 147148, 27401, -2231, 2300 }, // Aden
				{ 43532, -46807, -823, 57344 }, // Rune
				{ 87765, -141947, -1367, 6500 }, // Schuttgart
				{ 147154, -55527, -2807, 61300 } // Goddard
		};

		L2NpcTemplate template = NpcTable.getTemplate(EVENT_MANAGER_ID);
		for(int[] element : EVENT_MANAGERS)
			try
			{
				L2Spawn sp = new L2Spawn(template);
				sp.setLocx(element[0]);
				sp.setLocy(element[1]);
				sp.setLocz(element[2]);
				sp.setAmount(1);
				sp.setHeading(element[3]);
				sp.setRespawnDelay(0);
				sp.init();
				_spawns.add(sp);
			}
			catch(ClassNotFoundException e)
			{
				e.printStackTrace();
			}
	}

	/**
	 * Удаляет спавн эвент менеджеров
	 */
	private void unSpawnEventManagers()
	{
		for(L2Spawn sp : _spawns)
		{
			sp.stopRespawn();
			sp.getLastSpawn().deleteMe();
		}
		_spawns.clear();
	}

	/**
	 * Читает статус эвента из базы.
	 * @return
	 */
	private static boolean isActive()
	{
		Object eventStatus = mysql.get("SELECT value FROM `server_variables` WHERE `name`=\"CofferofShadows\"");
		return eventStatus != null && eventStatus.toString().equalsIgnoreCase("on");
	}

	/**
	* Запускает эвент
	*/
	public void startEvent()
	{
		L2Player player = (L2Player) self;
		if(!player.getPlayerAccess().IsEventGm)
			return;

		if(!isActive())
		{
			if(!mysql.set("INSERT INTO `server_variables` (`name`, `type`, `value`) VALUES (\"CofferofShadows\", \"event\", \"on\")"))
			{
				player.sendMessage("Can't start event. See server log files for details.");
				return;
			}
			spawnEventManagers();
			System.out.println("Event: Coffer of Shadows started.");
			Announcements.getInstance().announceByCustomMessage("scripts.events.CofferofShadows.AnnounceEventStarted", null);
		}
		else
			player.sendMessage("Event 'Coffer of Shadows' already started.");

		_active = true;
		show(Files.read("data/html/admin/events.htm", player), player);
	}

	/**
	* Останавливает эвент
	*/
	public void stopEvent()
	{
		L2Player player = (L2Player) self;
		if(!player.getPlayerAccess().IsEventGm)
			return;
		if(isActive())
		{
			if(!mysql.set("DELETE FROM `server_variables` WHERE `name`=\"CofferofShadows\""))
			{
				player.sendMessage("Can't stop event. See server log files for details.");
				return;
			}
			unSpawnEventManagers();
			System.out.println("Event: Coffer of Shadows stopped.");
			Announcements.getInstance().announceByCustomMessage("scripts.events.CofferofShadows.AnnounceEventStoped", null);
		}
		else
			player.sendMessage("Event 'Coffer of Shadows' not started.");

		_active = false;
		show(Files.read("data/html/admin/events.htm", player), player);
	}

	/**
	* Продает 1 сундук игроку
	*/
	public static void buycoffer()
	{
		L2Player player = (L2Player) self;

		if(!player.isQuestContinuationPossible())
			return;

		if(player.isActionsDisabled() || player.isSitting() || player.getLastNpc().getDistance(player) > 300)
			return;

		if(player.getAdena() < COFFER_PRICE * Config.EVENT_CofferOfShadowsPriceRate)
		{
			player.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ADENA);
			return;
		}

		player.reduceAdena(COFFER_PRICE * Config.EVENT_CofferOfShadowsPriceRate);
		L2ItemInstance createditem = ItemTable.getInstance().createItem(COFFER_ID, player.getObjectId(), 0, "CofferofShadows");
		createditem.setCount(1);
		player.getInventory().addItem(createditem);
		player.sendPacket(new ItemList(player, true));
		player.sendPacket(new SystemMessage(SystemMessage.EARNED_S1).addItemName(COFFER_ID));
	}

	/**
	 * Добавляет в диалоги эвент менеджеров строчку с байпасом для покупки сундука
	 */
	public static String DialogAppend_32091(Integer val)
	{
		if(val != 0)
			return "";

		String price = Util.formatAdena(COFFER_PRICE * Config.EVENT_CofferOfShadowsPriceRate);

		String append = "<a action=\"bypass -h scripts_events.CofferofShadows:buycoffer\">";
		append += new CustomMessage("scripts.events.CofferofShadows.buycoffer", self).addString(price);
		append += "</a>";

		return append;
	}

	public void onLoad()
	{
		if(isActive())
		{
			_active = true;
			spawnEventManagers();
			System.out.println("Loaded Event: Coffer of Shadows [state: activated]");
		}
		else
			System.out.println("Loaded Event: Coffer of Shadows [state: deactivated]");
	}

	public void onReload()
	{
		unSpawnEventManagers();
	}

	public void onShutdown()
	{
		unSpawnEventManagers();
	}

	public static void onPlayerEnter(L2Player player)
	{
		if(_active)
			Announcements.getInstance().announceToPlayerByCustomMessage(player, "scripts.events.CofferofShadows.AnnounceEventStarted", null);
	}
}