package events.clanhall;

import events.clanhall.devastatedcastle.DCSiege;
import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.model.instances.L2DoorInstance;
import L2jFT.game.model.L2Clan;
import L2jFT.game.tables.DoorTable;
import L2jFT.game.tables.ClanTable;
import L2jFT.database.ThreadConnection;
import L2jFT.database.FiltredPreparedStatement;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.database.DatabaseUtils;
import npc.model.L2CHSiegeDoorInstance;

import java.util.HashMap;
import java.util.ArrayList;
import java.util.Calendar;
import java.sql.ResultSet;

public class CHSiegeManager implements ScriptFile
{
	private HashMap<Integer, CHSiege> _chsieges = new HashMap<Integer, CHSiege>();

	public void onLoad()
	{
		instance = new CHSiegeManager();
		load();
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public static class MonsterLocation
	{
		private int _id;
		private int _x;
		private int _y;
		private int _z;
		private int _countMin;
		private int _countMax;
		private int _heading;

		public MonsterLocation(int id, int x, int y, int z)
		{
			_id = id;
			_x = x;
			_y = y;
			_z = z;
			_heading = 0;
			_countMin = _countMax = 1;
		}

		public MonsterLocation(int id, int x, int y, int z, int heading)
		{
			_id = id;
			_x = x;
			_y = y;
			_z = z;
			_heading = heading;
			_countMin = _countMax = 1;
		}

		public MonsterLocation(int id, int x, int y, int z, int heading, int count)
		{
			_id = id;
			_x = x;
			_y = y;
			_z = z;
			_heading = heading;
			_countMin = _countMax = count;
		}

		public MonsterLocation(int id, int x, int y, int z, int heading, int countMin, int countMax)
		{
			_id = id;
			_x = x;
			_y = y;
			_z = z;
			_heading = heading;
			_countMin = countMin;
			_countMax = countMax;
		}

		public int getId()
		{
			return _id;
		}

		public int getX()
		{
			return _x;
		}

		public int getY()
		{
			return _y;
		}

		public int getZ()
		{
			return _z;
		}

		public int getCountMin()
		{
			return _countMin;
		}

		public int getCountMax()
		{
			return _countMax;
		}

		public int getHeading()
		{
			return _heading;
		}
	}

	public static class CHSiege
	{
		private int _id;
		private int _startTime;
		private boolean _progress;
		private static ArrayList<L2Clan> _attackers = new ArrayList<L2Clan>();
		private boolean _registration;
		private Calendar startDate;

		public CHSiege(int id, Calendar c)
		{
			_id = id;
			startDate = c;
		}

		public int getStartTime()
		{
			return _startTime;
		}

		public void setStartTime(int startTime)
		{
			_startTime = startTime;
		}

		public int getId()
		{
			return _id;
		}

		public boolean isInProgress()
		{
			return _progress;
		}

		public void setInProgress(boolean progress)
		{
			_progress = progress;
		}

		public void addAttacker(L2Clan clan, boolean dbupdate)
		{
			_attackers.add(clan);
			if(!dbupdate)
				return;
			ThreadConnection con = null;
			FiltredPreparedStatement statement = null;
			ResultSet rset = null;
			try
			{
				con = L2DatabaseFactory.getInstance().getConnection();
				statement = con.prepareStatement("INSERT INTO chs_clans(ch_id,clan_id) VALUES(?,?)");
				statement.setInt(1, _id);
				statement.setInt(2, clan.getClanId());
				statement.executeUpdate();
			}
			catch(Exception e)
			{
				System.out.println("Exception: CHSiege:AddAttacker(): " + e.getMessage());
				e.printStackTrace();
			}
			finally
			{
				DatabaseUtils.closeDatabaseCSR(con, statement, rset);
			}
		}

		public void removeAttacker(L2Clan clan)
		{
			_attackers.remove(clan);
			ThreadConnection con = null;
			FiltredPreparedStatement statement = null;
			ResultSet rset = null;
			try
			{
				con = L2DatabaseFactory.getInstance().getConnection();
				statement = con.prepareStatement("DELETE FROM chs_clans WHERE ch_id = ? AND clan_id = ?");
				statement.setInt(1, _id);
				statement.setInt(2, clan.getClanId());
				statement.executeUpdate();
			}
			catch(Exception e)
			{
				System.out.println("Exception: CHSiege:RemoveAttacker(): " + e.getMessage());
				e.printStackTrace();
			}
			finally
			{
				DatabaseUtils.closeDatabaseCSR(con, statement, rset);
			}
		}

		public ArrayList<L2Clan> getAttackers()
		{
			return _attackers;
		}

		public boolean isRegistered(L2Clan clan)
		{
			return _attackers.contains(clan);
		}

		public void clearAttackers()
		{
			ThreadConnection con = null;
			FiltredPreparedStatement statement = null;
			ResultSet rset = null;
			try
			{
				con = L2DatabaseFactory.getInstance().getConnection();
				statement = con.prepareStatement("DELETE FROM chs_clans WHERE ch_id = ?");
				statement.setInt(1, _id);
				statement.executeUpdate();
			}
			catch(Exception e)
			{
				System.out.println("Exception: DCSiege:clearAttackers(): " + e.getMessage());
				e.printStackTrace();
			}
			finally
			{
				DatabaseUtils.closeDatabaseCSR(con, statement, rset);
			}
			_attackers.clear();
		}

		public boolean isRegistrationAvaliable()
		{
			return _registration;
		}

		public void setRegistrationAvaliable(boolean registration)
		{
			_registration = registration;
		}

		public int setNewStartTime()
		{
			while(startDate.getTimeInMillis() < System.currentTimeMillis())
				startDate.add(Calendar.DAY_OF_MONTH, 7);
			_startTime = (int) (startDate.getTimeInMillis() / 1000);
			ThreadConnection con = null;
			FiltredPreparedStatement statement = null;
			ResultSet rset = null;
			try
			{
				con = L2DatabaseFactory.getInstance().getConnection();
				statement = con.prepareStatement("UPDATE ch_sieges SET starttime=? where ch_id=?");
				statement.setLong(1, _startTime);
				statement.setInt(2, _id);
				statement.executeUpdate();
				con = L2DatabaseFactory.getInstance().getConnection();
				statement = con.prepareStatement("DELETE FROM chs_clans where ch_id=?");
				statement.setInt(1, _id);
				statement.executeUpdate();
			}
			catch(Exception e)
			{
				System.out.println("Exception: DCSiege:stopSiege(): " + e.getMessage());
				e.printStackTrace();
			}
			finally
			{
				DatabaseUtils.closeDatabaseCSR(con, statement, rset);
			}
			return _startTime;
		}

		public void setStartDate(Calendar c)
		{
			startDate = c;
		}
	}

	private static CHSiegeManager instance;

	public static CHSiegeManager getInstance()
	{
		return instance;
	}

	public CHSiegeManager()
	{}

	private void load()
	{
		DCSiege.getInstance().load();
	}

	public void spawnDoors(int[] doors, boolean siege)
	{
		for(int element : doors)
		{
			L2DoorInstance door = DoorTable.getInstance().getDoor(element);
			if(siege)
			{
				door.decayMe();
				L2CHSiegeDoorInstance sdoor = new L2CHSiegeDoorInstance(door.getObjectId(), door.getTemplate(), door.getDoorId(), door.getDoorName(), false, true);
				sdoor.setDestroyable(true);
				DoorTable.getInstance().putDoor(sdoor.getDoorId(), sdoor);
				sdoor.spawnMe(door.getX(), door.getY(), door.getZ());
			}
			else
			{
				door.decayMe();
				L2DoorInstance sdoor = new L2CHSiegeDoorInstance(door.getObjectId(), door.getTemplate(), door.getDoorId(), door.getDoorName(), false, true);
				sdoor.setDestroyable(true);
				DoorTable.getInstance().putDoor(sdoor.getDoorId(), sdoor);
				sdoor.spawnMe(door.getX(), door.getY(), door.getZ());
			}
		}
	}

	public void registerCHSiege(int id, Calendar c)
	{
		if(!_chsieges.containsKey(id))
		{
			_chsieges.put(id, new CHSiege(id, c));
			loadAttackers(id);
			loadSiegeTime(id);
		}
		else
		{
			getSiege(id).setStartDate(c);
			getSiege(id).setNewStartTime();
		}
	}

	public boolean isRegistered(L2Clan clan)
	{
		for(CHSiege siege : _chsieges.values())
			if(siege.isRegistered(clan))
				return true;
		return false;
	}

	public CHSiege getSiege(int id)
	{
		return _chsieges.get(id);
	}

	public void loadAttackers(int id)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT clan_id FROM chs_clans where ch_id=?");
			statement.setInt(1, id);
			rset = statement.executeQuery();
			while(rset.next())
				getSiege(id).addAttacker(ClanTable.getInstance().getClan(rset.getInt("clan_id")), false);
		}
		catch(Exception e)
		{
			System.out.println("Exception: CHSiege:loadAttackers() Id:" + id + " : " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	public void loadSiegeTime(int id)
	{
		long _starttime = 0;
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT starttime FROM ch_sieges where ch_id=?");
			statement.setInt(1, id);
			rset = statement.executeQuery();
			if(rset.next())
				_starttime = rset.getLong("starttime");
			DatabaseUtils.closeStatement(statement);
			if(_starttime < System.currentTimeMillis() / 1000)
			{
				_starttime = 0;
				con = L2DatabaseFactory.getInstance().getConnection();
				statement = con.prepareStatement("DELETE FROM chs_clans where ch_id=?");
				statement.setInt(1, id);
				statement.executeUpdate();
				DatabaseUtils.closeStatement(statement);
				CHSiegeManager.getInstance().getSiege(id).clearAttackers();
				statement = con.prepareStatement("DELETE FROM ch_sieges where ch_id=?");
				statement.setInt(1, id);
				statement.executeUpdate();
				DatabaseUtils.closeStatement(statement);
			}
			if(_starttime == 0)
			{
				_starttime = getSiege(id).setNewStartTime();
				statement = con.prepareStatement("INSERT INTO ch_sieges(ch_id, starttime) VALUES(?, ?)");
				statement.setInt(1, id);
				statement.setLong(2, _starttime);
				statement.executeUpdate();
			}
		}
		catch(Exception e)
		{
			System.out.println("Exception: CHSiege:loadSiegeTime() Id:" + id + " : " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
		getSiege(id).setRegistrationAvaliable(true);
		getSiege(id).setStartTime((int) _starttime);
	}

	public static ArrayList<L2Clan> getAttackersForSiege(Integer id)
	{
		return CHSiegeManager.getInstance().getSiege(id).getAttackers();
	}

	public static void addAttackerForSiege(Integer id, L2Clan clan)
	{
		CHSiegeManager.getInstance().getSiege(id).addAttacker(clan, true);
	}

	public static void removeAttackerForSiege(Integer id, L2Clan clan)
	{
		CHSiegeManager.getInstance().getSiege(id).removeAttacker(clan);
	}
}
