package events.l2day;

import L2jFT.Config;
import L2jFT.database.mysql;
import L2jFT.extensions.scripts.Functions;
import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.Announcements;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Spawn;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.ItemTable;
import L2jFT.game.tables.NpcTable;
import L2jFT.game.templates.L2NpcTemplate;
import L2jFT.util.Files;
import L2jFT.util.Rnd;

import java.util.ArrayList;
import java.util.HashMap;

public class l2day extends Functions implements ScriptFile
{
	public static L2Object self;
	public static L2NpcInstance npc;
	private static int EVENT_MANAGER_ID = 31230;
	private static int LETTER_CHANCE = 2; // 2%

	// Буквы
	private static int A = 3875;
	private static int C = 3876;
	private static int E = 3877;
	private static int F = 3878;
	private static int G = 3879;
	private static int H = 3880;
	private static int I = 3881;
	private static int L = 3882;
	private static int N = 3883;
	private static int O = 3884;
	private static int R = 3885;
	private static int S = 3886;
	private static int T = 3887;
	private static int II = 3888;

	// Награды
	private static int BSOR = 3959;
	private static int BSOE = 3958;
	private static int ACUMEN = 3929;
	private static int EMPOWER = 3932;
	private static int GUIDANCE = 3926;
	private static int WHISPER = 3927;
	private static int FOCUS = 3928;
	private static int WINDWALK = 3934;

	private static HashMap<String, Integer[]> _words = new HashMap<String, Integer[]>();

	static
	{
		_words.put("LineageII", new Integer[] {
				BSOR, 3, BSOE, 3, ACUMEN, 3, EMPOWER, 3, // награда
				L, 1, I, 1, N, 1, E, 2, A, 1, G, 1, II, 1, // нужные буквы 
				C, 0, F, 0 // ненужные буквы
		});
		_words.put("NCSOFT", new Integer[] {
				BSOE, 1, BSOR, 1, GUIDANCE, 1, WHISPER, 1, // награда
				N, 1, C, 1, S, 1, O, 1, F, 1, T, 1, // нужные буквы 
				A, 0, E, 0, G, 0 // ненужные буквы
		});
		_words.put("CHRONICLE", new Integer[] {
				WINDWALK, 2, BSOR, 2, BSOE, 2, FOCUS, 2, // награда 
				C, 2, H, 1, R, 1, O, 1, N, 1, I, 1, L, 1, E, 1, // нужные буквы  
				T, 0 // ненужные буквы 
		});
	}

	private static ArrayList<L2Spawn> _spawns = new ArrayList<L2Spawn>();

	private static boolean _active = false;

	public void onLoad()
	{
		if(isActive())
		{
			_active = true;
			spawnEventManagers();
			System.out.println("Loaded Event: l2day [state: activated]");
		}
		else
			System.out.println("Loaded Event: l2day [state: deactivated]");
	}

	/**
	 * Читает статус эвента из базы.
	 * @return
	 */
	private static boolean isActive()
	{
		Object eventStatus = mysql.get("SELECT value FROM `server_variables` WHERE `name`=\"l2day\"");
		return eventStatus != null && eventStatus.toString().equalsIgnoreCase("on");
	}

	/**
	* Запускает эвент
	*/
	public void startEvent()
	{
		L2Player player = (L2Player) self;
		if(!player.getPlayerAccess().IsEventGm)
			return;

		if(!isActive())
		{
			if(!mysql.set("INSERT INTO `server_variables` (`name`, `type`, `value`) VALUES (\"l2day\", \"event\", \"on\")"))
			{
				player.sendMessage("Can't start event. See server log files for details.");
				return;
			}
			spawnEventManagers();
			System.out.println("Event 'l2day' started.");
			Announcements.getInstance().announceByCustomMessage("scripts.events.l2day.AnnounceEventStarted", null);
		}
		else
			player.sendMessage("Event 'l2day' already started.");

		_active = true;

		show(Files.read("data/html/admin/events.htm", player), player);
	}

	/**
	* Останавливает эвент
	*/
	public void stopEvent()
	{
		L2Player player = (L2Player) self;
		if(!player.getPlayerAccess().IsEventGm)
			return;
		if(isActive())
		{
			if(!mysql.set("DELETE FROM `server_variables` WHERE `name`=\"l2day\""))
			{
				player.sendMessage("Can't stop event. See server log files for details.");
				return;
			}
			unSpawnEventManagers();
			System.out.println("Event 'l2day' stopped.");
			Announcements.getInstance().announceByCustomMessage("scripts.events.l2day.AnnounceEventStoped", null);
		}
		else
			player.sendMessage("Event 'l2day' not started.");

		_active = false;

		show(Files.read("data/html/admin/events.htm", player), player);
	}

	/**
	 * Спавнит эвент менеджеров
	 */
	private void spawnEventManagers()
	{
		final int EVENT_MANAGERS[][] = {
				{ 19541, 145419, -3103, 30419 }, { 147485, -59049, -2980, 9138 },
				{ 109947, 218176, -3543, 63079 }, { -81363, 151611, -3121, 42910 },
				{ 144741, 28846, -2453, 2059 }, { 44192, -48481, -796, 23331 },
				{ -13889, 122999, -3109, 40099 }, { 116278, 75498, -2713, 12022 },
				{ 82029, 55936, -1519, 58708 }, { 147142, 28555, -2261, 59402 },
				{ 82153, 148390, -3466, 57344 }, };

		L2NpcTemplate template = NpcTable.getTemplate(EVENT_MANAGER_ID);
		for(int[] element : EVENT_MANAGERS)
			try
			{
				L2Spawn sp = new L2Spawn(template);
				sp.setLocx(element[0]);
				sp.setLocy(element[1]);
				sp.setLocz(element[2]);
				sp.setAmount(1);
				sp.setHeading(element[3]);
				sp.setRespawnDelay(0);
				sp.init();
				_spawns.add(sp);
			}
			catch(ClassNotFoundException e)
			{
				e.printStackTrace();
			}
	}

	/**
	 * Удаляет спавн эвент менеджеров
	 */
	private void unSpawnEventManagers()
	{
		for(L2Spawn sp : _spawns)
		{
			sp.stopRespawn();
			sp.getLastSpawn().deleteMe();
		}
		_spawns.clear();
	}

	public void onReload()
	{
		unSpawnEventManagers();
	}

	public void onShutdown()
	{
		unSpawnEventManagers();
	}

	/**
	 * Обработчик смерти мобов, управляющий эвентовым дропом
	 */
	public static void onDie(L2Character cha, L2Character killer)
	{
		if(_active && cha.isMonster && killer != null && killer.getPlayer() != null && Math.abs(cha.getLevel() - killer.getLevel()) < 10 && Rnd.chance((int) (LETTER_CHANCE * Config.RATE_DROP_ITEMS)))
		{
			L2ItemInstance item = ItemTable.getInstance().createItem(Rnd.get(A, II), killer.getPlayer().getObjectId(), 0, "l2day");
			((L2NpcInstance) cha).dropItem(killer.getPlayer(), item);
		}
	}

	/**
	* Обмен эвентовых вещей, где var - слово.
	*/
	public static void exchange(String[] var)
	{
		L2Player player = (L2Player) self;

		if(!player.isQuestContinuationPossible())
			return;

		if(player.isActionsDisabled() || player.isSitting() || player.getLastNpc().getDistance(player) > 300)
			return;

		Integer[] mss = _words.get(var[0]);

		if(getItemCount(player, mss[8]) < mss[9] || getItemCount(player, mss[10]) < mss[11] || getItemCount(player, mss[14]) < mss[15] || getItemCount(player, mss[16]) < mss[17] || getItemCount(player, mss[18]) < mss[19] || getItemCount(player, mss[20]) < mss[21] || getItemCount(player, mss[22]) < mss[23] || getItemCount(player, mss[24]) < mss[25])
		{
			player.sendPacket(new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_ENOUGH_REQUIRED_ITEMS));
			return;
		}

		if(mss[9] > 0)
			removeItem(player, mss[8], mss[9]);
		if(mss[11] > 0)
			removeItem(player, mss[10], mss[11]);
		if(mss[13] > 0)
			removeItem(player, mss[12], mss[13]);
		if(mss[15] > 0)
			removeItem(player, mss[14], mss[15]);
		if(mss[17] > 0)
			removeItem(player, mss[16], mss[17]);
		if(mss[19] > 0)
			removeItem(player, mss[18], mss[19]);
		if(mss[21] > 0)
			removeItem(player, mss[20], mss[21]);
		if(mss[23] > 0)
			removeItem(player, mss[22], mss[23]);
		if(mss[25] > 0)
			removeItem(player, mss[24], mss[25]);
		if(mss[1] > 0)
			addItem(player, mss[0], mss[1]);
		if(mss[3] > 0)
			addItem(player, mss[2], mss[3]);
		if(mss[5] > 0)
			addItem(player, mss[4], mss[5]);
		if(mss[7] > 0)
			addItem(player, mss[6], mss[7]);
	}
	
	public static void onPlayerEnter(L2Player player)
	{
		if(_active)
			Announcements.getInstance().announceToPlayerByCustomMessage(player, "scripts.events.l2day.AnnounceEventStarted", null);
	}
}