package events.lastHero;

import java.util.ArrayList;

import L2jFT.extensions.listeners.L2ZoneEnterLeaveListener;
import L2jFT.extensions.scripts.Functions;
import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.Announcements;
import L2jFT.game.instancemanager.ZoneManager;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Playable;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.L2World;
import L2jFT.game.model.L2Zone;
import L2jFT.game.model.L2Zone.ZoneType;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.network.serverpackets.Revive;
import L2jFT.game.network.serverpackets.SocialAction;
import L2jFT.game.tables.SkillTable;
import L2jFT.util.CharPosition;
import L2jFT.util.Rnd;

public class LastHero extends Functions implements ScriptFile
{
	public static L2Object self;
	public static L2NpcInstance npc;

	private static ArrayList<L2Player> players_list = new ArrayList<L2Player>();
	private static ArrayList<L2Player> live_list = new ArrayList<L2Player>();

	private static Integer _status = 0;
	private static Integer _time_to_start = 5;

	private static L2Zone _zone = ZoneManager.getInstance().getZoneByIndex(ZoneType.battle_zone, 4, true);
	ZoneListener _zoneListener = new ZoneListener();

	public void onLoad()
	{
		_zone.getListenerEngine().addMethodInvokedListener(_zoneListener);
		System.out.println("Loaded Event: Last Hero");
	}

	public void onReload()
	{
		_zone.getListenerEngine().removeMethodInvokedListener(_zoneListener);
	}

	public void onShutdown()
	{
		onReload();
	}

	public static void start()
	{
		if(!((L2Player) self).getPlayerAccess().IsEventGm)
			return;

		_time_to_start = 5;
		_status = 0;

		players_list = new ArrayList<L2Player>();
		live_list = new ArrayList<L2Player>();

		Announcements.getInstance().announceToAll("Запущен эвент 'Last Hero'. Все зарегистрированные участники будут телепортированы в Coliseum через 5 минут.");
		Announcements.getInstance().announceToAll("Игроки, отказавшиеся от участия в эвенте смогут наблюдать за битвой через обзорные кристаллы.");
		executeTask("events.lastHero.LastHero", "question", new Object[0], 10000);
		executeTask("events.lastHero.LastHero", "announce", new Object[0], 60000);
	}

	public static void question()
	{
		for(L2Player player : L2World.getAllPlayers())
			player.scriptRequest("Вы хотите принять участие в эвенте?", "events.lastHero.LastHero:addPlayer", new Object[0]);
	}

	public static void announce()
	{
		Announcements a = Announcements.getInstance();
		if(players_list.size() < 2)
		{
			a.announceToAll("Эвент отменен, слишком мало участников.");
			return;
		}

		if(_time_to_start > 1)
		{
			_time_to_start--;
			a.announceToAll("Начало эвента через " + _time_to_start + " мин.");
			executeTask("events.lastHero.LastHero", "announce", new Object[0], 60000);
		}
		else
		{
			a.announceToAll("Телепортация игроков...");
			executeTask("events.lastHero.LastHero", "prepare", new Object[0], 5000);
		}
	}

	public void addPlayer()
	{
		players_list.add((L2Player) self);
		live_list.add((L2Player) self);
	}

	public static void prepare()
	{
		cleanPlayers();
		cleanLive();
		executeTask("events.lastHero.LastHero", "ressurectPlayers", new Object[0], 5000);
		executeTask("events.lastHero.LastHero", "healPlayers", new Object[0], 10000);
		executeTask("events.lastHero.LastHero", "saveBackCoords", new Object[0], 15000);
		executeTask("events.lastHero.LastHero", "teleportPlayersToColiseum", new Object[0], 20000);
		executeTask("events.lastHero.LastHero", "paralyzePlayers", new Object[0], 25000);
		executeTask("events.lastHero.LastHero", "go", new Object[0], 120000);

		Announcements a = Announcements.getInstance();
		a.announceToAll("Правила эвента 'Last Hero':");
		a.announceToAll("Эвент длится 5 минут.");
		a.announceToAll("Цель - убить как можно больше игроков.");
		a.announceToAll("За каждого убитого вам дают 5к * уровень убитого аден.");
		a.announceToAll("Старт через 2 минуты, по команде 'FIGHT!!!'.");
	}

	public static void go()
	{
		_status = 1;
		upParalyzePlayers();
		checkLive();
		Announcements.getInstance().announceToAll(">>> FIGHT!!! <<<");
		executeTask("events.lastHero.LastHero", "endBattle", new Object[0], 300000);
	}

	public static void endBattle()
	{
		_status = 0;
		Announcements a = Announcements.getInstance();
		a.announceToAll("Эвент 'Last Hero' окончен.");
		a.announceToAll("Через 1 минуту все участники будут вылечены и телепортированы обратно.");
		executeTask("events.lastHero.LastHero", "end", new Object[0], 60000);
	}

	public static void end()
	{
		players_list = L2World.getAllPlayers();
		executeTask("events.lastHero.LastHero", "ressurectPlayers", new Object[0], 5000);
		executeTask("events.lastHero.LastHero", "healPlayers", new Object[0], 10000);
		executeTask("events.lastHero.LastHero", "teleportPlayersToSavedCoords", new Object[0], 15000);
	}

	public static void saveBackCoords()
	{
		synchronized (players_list)
		{
			for(L2Player player : players_list)
				if(player != null)
					player.setVar("LastHero_backCoords", player.getX() + " " + player.getY() + " " + player.getZ());
		}
	}

	public static void teleportPlayersToColiseum()
	{
		synchronized (players_list)
		{
			for(L2Player player : players_list)
				if(player != null)
				{
					CharPosition pos = Rnd.coordsRandomize(149505, 46719, -3417, 0, 0, 500);
					player.teleToLocation(pos.x, pos.y, pos.z);
				}
		}
	}

	public static void teleportPlayersToSavedCoords()
	{
		synchronized (players_list)
		{
			for(L2Player player : players_list)
				if(player != null)
				{
					String var = player.getVar("LastHero_backCoords");
					if(var == null || var.equals(""))
						continue;
					String[] coords = var.split(" ");
					if(coords.length != 3)
						continue;
					CharPosition pos = new CharPosition(Integer.parseInt(coords[0]), Integer.parseInt(coords[1]), Integer.parseInt(coords[2]), 0);
					player.teleToLocation(pos.x, pos.y, pos.z);
					player.setVar("LastHero_backCoords", "");
				}
		}
	}

	public static void paralyzePlayers()
	{
		L2Skill revengeSkill = SkillTable.getInstance().getInfo(4515, 1);
		synchronized (players_list)
		{
			for(L2Player player : players_list)
				if(player != null)
				{
					revengeSkill.getEffects(player, player, false);
					if(player.getPet() != null)
						revengeSkill.getEffects(player, player.getPet(), false);
				}
		}
	}

	public static void upParalyzePlayers()
	{
		synchronized (players_list)
		{
			for(L2Player player : players_list)
				if(player != null)
				{
					player.stopEffect((short) 4515);
					if(player.getPet() != null)
						player.getPet().stopEffect((short) 4515);
				}
		}
	}

	public static void ressurectPlayers()
	{
		synchronized (players_list)
		{
			for(L2Player player : players_list)
				if(player != null)
				{
					player.setCurrentHpMp(player.getMaxHp(), player.getMaxMp());
					player.setCurrentCp(player.getMaxCp());
					player.restoreExp();
					player.broadcastPacket(new SocialAction(player.getObjectId(), 15));
					player.broadcastPacket(new Revive(player));
					player.doRevive();
				}
		}
	}

	public static void healPlayers()
	{
		synchronized (players_list)
		{
			for(L2Player player : players_list)
				if(player != null)
				{
					player.setCurrentHpMp(player.getMaxHp(), player.getMaxMp());
					player.setCurrentCp(player.getMaxCp());
				}
		}
	}

	public static void cleanPlayers()
	{
		ArrayList<L2Player> temp = new ArrayList<L2Player>();
		synchronized (players_list)
		{
			for(L2Player player : players_list)
				if(player != null)
					temp.add(player);
		}
		players_list = temp;
	}

	public static void cleanLive()
	{
		ArrayList<L2Player> temp = new ArrayList<L2Player>();
		synchronized (live_list)
		{
			for(L2Player player : live_list)
				if(player != null)
					temp.add(player);
		}
		live_list = temp;
	}

	public static void checkLive()
	{
		ArrayList<L2Player> temp = new ArrayList<L2Player>();
		synchronized (live_list)
		{
			for(L2Player player : live_list)
				if(player != null && player.isInZone(ZoneType.battle_zone) && !player.isDead())
				{
					temp.add(player);
					player.setTeam(2);
				}
				else if(player != null)
				{
					player.setTeam(0);
					player.sendMessage("Вы проиграли! Дождитесь окончания эвента для обратной телепортации.");
				}
		}
		live_list = temp;
	}

	public static void onDie(L2Object self, L2Character killer)
	{
		if(_status > 0 && self != null && self.isPlayer && live_list.contains(self))
		{
			live_list.remove(self);
			((L2Player) self).setTeam(0);
			((L2Player) self).sendMessage("Вы проиграли! Дождитесь окончания эвента для обратной телепортации.");
			checkLive();
			if(killer != null && killer.isPlayer)
				((L2Player) killer).addAdena(((L2Player) self).getLevel() * 5000);
		}
	}

	public static void OnPlayerExit(@SuppressWarnings("unused")
	L2Player player)
	{
		if(_status > 0 && self != null && self.isPlayer && live_list.contains(self))
		{
			live_list.remove(self);
			((L2Player) self).setTeam(0);
			checkLive();
		}
	}

	private class ZoneListener extends L2ZoneEnterLeaveListener
	{
		@Override
		public void objectEntered(L2Zone zone, L2Object object)
		{
			if(_status > 0 && object != null && object.getPlayer() != null && !live_list.contains(object.getPlayer()))
				((L2Character) object).teleToLocation(147451, 46728, -3410);
		}

		@Override
		public void objectLeaved(L2Zone zone, L2Object object)
		{
			L2Player player = object.getPlayer();
			if(_status > 0 && player != null && live_list.contains(player))
			{
				L2Playable playable = (L2Playable) object;
				Integer heading = playable.getHeading();
				if(heading <= 16384)
					playable.teleToLocation(playable.getX() - 50, playable.getY() - 50, playable.getZ());
				else if(heading > 16384 && heading <= 32768)
					playable.teleToLocation(playable.getX() + 50, playable.getY() - 50, playable.getZ());
				else if(heading > 32768 && heading <= 49152)
					playable.teleToLocation(playable.getX() + 50, playable.getY() + 50, playable.getZ());
				else if(heading > 49152)
					playable.teleToLocation(playable.getX() - 50, playable.getY() + 50, playable.getZ());
			}
		}
	}
}
