package events.rename;

import java.sql.ResultSet;

import L2jFT.Config;
import L2jFT.database.DatabaseUtils;
import L2jFT.database.FiltredPreparedStatement;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.database.ThreadConnection;
import L2jFT.database.mysql;
import L2jFT.extensions.multilang.CustomMessage;
import L2jFT.extensions.scripts.Functions;
import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.cache.Msg;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;
import L2jFT.util.Files;
import L2jFT.util.Log;
import L2jFT.util.Util;

public class Rename extends Functions implements ScriptFile
{
	public static L2Object self;
	public static L2Object npc;
	private static int RENAME_ITEM = Config.EVENT_RenameItemID;
	private static int RENAME_PRICE = Config.EVENT_RenamePrice;
	private static boolean _active = false;

	private static boolean isActive()
	{
		Object eventStatus = mysql.get("SELECT value FROM `server_variables` WHERE `name`=\"Rename_event\"");
		return eventStatus != null && eventStatus.toString().equalsIgnoreCase("on");
	}

	public void startEvent()
	{
		L2Player player = (L2Player) self;
		if(!player.getPlayerAccess().IsEventGm)
			return;

		if(!isActive())
		{
			if(!mysql.set("INSERT INTO `server_variables` (`name`, `type`, `value`) VALUES (\"Rename_event\", \"event\", \"on\")"))
			{
				player.sendMessage("Can't start event. See server log files for details.");
				return;
			}
			System.out.println("Event: Rename started.");
		}
		else
			player.sendMessage("Event 'Rename' already started.");

		_active = true;

		show(Files.read("data/html/admin/events.htm", player), player);
	}

	public void stopEvent()
	{
		L2Player player = (L2Player) self;
		if(!player.getPlayerAccess().IsEventGm)
			return;
		if(isActive())
		{
			if(!mysql.set("DELETE FROM `server_variables` WHERE `name`=\"Rename_event\""))
			{
				player.sendMessage("Can't stop event. See server log files for details.");
				return;
			}
			System.out.println("Event: Rename stopped.");
		}
		else
			player.sendMessage("Event 'Rename' not started.");

		_active = false;

		show(Files.read("data/html/admin/events.htm", player), player);
	}

	public static String DialogAppend_30031(Integer val)
	{
		if(val == 0 && _active)
		{
			String append = "[scripts_events.rename.Rename:rename_page|";
			append += new CustomMessage("scripts.events.rename.Rename.Rename", self);
			append += "]";
			return append;
		}
		return "";
	}

	public static String rename_page()
	{
		String append = "!Rename";
		append += "<br>";
		append += "<font color=\"LEVEL\">" + (new CustomMessage("scripts.events.rename.Rename.RenameFor", self).addString(Util.formatAdena(RENAME_PRICE)).addItemName(RENAME_ITEM)) + "</font>";
		append += "<table>";
		append += "<tr><td>" + (new CustomMessage("scripts.events.rename.Rename.NewName", self)) + ": <edit var=\"new_name\" width=80></td></tr>";
		append += "<tr><td><button value=\"" + (new CustomMessage("scripts.events.rename.Rename.RenameButton", self)) + "\" action=\"bypass -h scripts_events.rename.Rename:rename $new_name\" width=80 height=15></td></tr>";
		append += "</table>";
		return append;
	}

	public static void rename(String[] args)
	{
		L2Player player = (L2Player) self;
		if(args.length != 1)
		{
			show(new CustomMessage("scripts.events.rename.Rename.incorrectinput", player), player);
			return;
		}

		String name = args[0];
		if(!Util.isMatchingRegexp(name, Config.CNAME_TEMPLATE))
		{
			show(new CustomMessage("scripts.events.rename.Rename.incorrectinput", player), player);
			return;
		}

		if(getItemCount(player, RENAME_ITEM) < RENAME_PRICE)
		{
			if(RENAME_ITEM == 57)
				player.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ADENA);
			else
				player.sendPacket(Msg.INCORRECT_ITEM_COUNT);
			return;
		}

		ThreadConnection con = null;
		FiltredPreparedStatement offline = null;
		ResultSet rs = null;

		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			offline = con.prepareStatement("SELECT char_name FROM characters WHERE char_name LIKE ?");
			offline.setString(1, name);
			rs = offline.executeQuery();
			if(rs.next())
			{
				show(new CustomMessage("scripts.events.rename.Rename.Thisnamealreadyexists", player), player);
				return;
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
			show(new CustomMessage("common.Error", player), player);
			return;
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, offline, rs);
		}

		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			offline = con.prepareStatement("UPDATE characters SET char_name = ? WHERE obj_Id = ?");
			offline.setString(1, name);
			offline.setInt(2, player.getObjectId());
			offline.executeUpdate();
		}
		catch(Exception e)
		{
			e.printStackTrace();
			show(new CustomMessage("common.Error", player), player);
			return;
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, offline);
		}

		removeItem(player, RENAME_ITEM, RENAME_PRICE);

		String oldName = player.getName();
		L2World.removeObject(player);
		player.decayMe();
		player.setName(name);
		player.spawnMe();
		player.broadcastUserInfo();
		Log.add("Character " + oldName + " renamed to " + name, "renames");
		show(new CustomMessage("scripts.events.rename.Rename.changedname", player).addString(oldName).addString(name), player);
	}

	public void onLoad()
	{
		if(isActive())
		{
			_active = true;
			System.out.println("Loaded Event: Rename [state: activated]");
		}
		else
			System.out.println("Loaded Event: Rename [state: deactivated]");
	}

	public void onReload()
	{}

	public void onShutdown()
	{}
}