package items;

import L2jFT.Config;
import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.handler.IItemHandler;
import L2jFT.game.handler.ItemHandler;
import L2jFT.util.Rnd;
import L2jFT.game.model.FishDropData;
import L2jFT.game.model.L2Playable;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.network.serverpackets.InventoryUpdate;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.FishTable;
import L2jFT.game.tables.ItemTable;

import java.util.List;

public class FishItem implements IItemHandler, ScriptFile
{
	private static short[] _itemIds = null;

	public FishItem()
	{
		FishTable ft = FishTable.getInstance();
		_itemIds = new short[ft.GetFishItemCount()];
		for(int i = 0; i < ft.GetFishItemCount(); i++)
			_itemIds[i] = ft.getFishIdfromList(i);
	}

	public synchronized void useItem(L2Playable playable, L2ItemInstance item)
	{
		if(playable == null || !playable.isPlayer)
			return;
		L2Player player = (L2Player) playable;

		List<FishDropData> rewards = FishTable.getInstance().getFishReward(item.getItemId());
		int count = 0;
		player.getInventory().destroyItem(item, 1, true, true);
		for(FishDropData d : rewards)
			if(Rnd.chance(d.getChance()))
			{
				giveItems(player, d.getRewardItemId(), Rnd.get((int) (d.getMinCount() * Config.RATE_FISH_DROP_COUNT), (int) (d.getMaxCount() * Config.RATE_FISH_DROP_COUNT)));
				count++;
				continue;
			}
		if(count == 0)
			player.sendMessage("Got nothing.", "Рыба пуста.");
	}

	public void giveItems(L2Player activeChar, short itemId, int count)
	{
		InventoryUpdate iu = new InventoryUpdate();
		L2ItemInstance item = ItemTable.getInstance().createItem(itemId, activeChar.getObjectId(), 0, "FishItem_add");
		item.setCount(count);
		iu.addItem(activeChar.getInventory().addItem(item));
		if(count > 1)
			activeChar.sendPacket(new SystemMessage(SystemMessage.EARNED_S2_S1_s).addItemName(itemId).addNumber(count));
		else
			activeChar.sendPacket(new SystemMessage(SystemMessage.EARNED_S1).addItemName(itemId));
		activeChar.sendPacket(iu);
	}

	public short[] getItemIds()
	{
		return _itemIds;
	}

	public void onLoad()
	{
		ItemHandler.getInstance().registerItemHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}
}