package items;

import javolution.util.FastList;
import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.ThreadPoolManager;
import L2jFT.game.cache.Msg;
import L2jFT.game.geodata.GeoData;
import L2jFT.game.handler.IItemHandler;
import L2jFT.game.handler.ItemHandler;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2PetData;
import L2jFT.game.model.L2Playable;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.L2World;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.instances.L2PetBabyInstance;
import L2jFT.game.model.instances.L2PetInstance;
import L2jFT.game.network.serverpackets.MagicSkillLaunched;
import L2jFT.game.network.serverpackets.MagicSkillUse;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.NpcTable;
import L2jFT.game.tables.PetDataTable;
import L2jFT.game.tables.SkillTable;
import L2jFT.game.templates.L2NpcTemplate;
import L2jFT.util.CharPosition;
import L2jFT.util.Rnd;

import java.util.logging.Logger;

public class PetSummon implements IItemHandler, ScriptFile
{
	protected static Logger _log = Logger.getLogger(PetSummon.class.getName());

	// all the items ids that this handler knowns
	private static final short[] _itemIds = PetDataTable._itemControlIds;
	private static final int _skillId = 2046;

	static final SystemMessage YOU_MAY_NOT_USE_MULTIPLE_PETS_OR_SERVITORS_AT_THE_SAME_TIME = new SystemMessage(SystemMessage.YOU_MAY_NOT_USE_MULTIPLE_PETS_OR_SERVITORS_AT_THE_SAME_TIME);
	static final SystemMessage A_STRIDER_CANNOT_BE_RIDDEN_WHILE_IN_BATTLE = new SystemMessage(SystemMessage.A_STRIDER_CANNOT_BE_RIDDEN_WHILE_IN_BATTLE);
	static final SystemMessage SUMMON_A_PET = new SystemMessage(SystemMessage.SUMMON_A_PET);

	public void useItem(L2Playable playable, L2ItemInstance item)
	{
		if(playable == null || !playable.isPlayer)
			return;
		L2Player player = (L2Player) playable;

		if(player.isSitting())
		{
			player.sendPacket(Msg.YOU_CANNOT_MOVE_WHILE_SITTING);
			return;
		}

		if(player.isInOlympiadMode())
		{
			player.sendPacket(Msg.THIS_ITEM_IS_NOT_AVAILABLE_FOR_THE_OLYMPIAD_EVENT);
			return;
		}

		if(player.isCastingNow() || player.isActionsDisabled())
			return;

		L2Skill skill = SkillTable.getInstance().getInfo(_skillId, 1);
		if(!skill.checkCondition(player, player, false, true, true))
			return;

		if(player.getPet() != null)
		{
			player.sendPacket(YOU_MAY_NOT_USE_MULTIPLE_PETS_OR_SERVITORS_AT_THE_SAME_TIME);
			return;
		}

		if(player.isMounted() || player.isInBoat())
			return;

		if(player.isCursedWeaponEquipped())
		{
			// You can't mount while weilding a cursed weapon
			player.sendPacket(A_STRIDER_CANNOT_BE_RIDDEN_WHILE_IN_BATTLE);
			return;
		}

		int npcId = PetDataTable.getSummonId(item);
		if(npcId == 0)
			return;

		L2NpcTemplate petTemplate = NpcTable.getTemplate(npcId);

		L2PetInstance newpet;
		if(L2PetData.isBaby(npcId))
			newpet = L2PetBabyInstance.spawnPet(petTemplate, player, item);
		else
			newpet = L2PetInstance.spawnPet(petTemplate, player, item);

		if(newpet == null)
			return;

		newpet.setTitle(player.getName());

		if(!newpet.isRespawned())
			try
			{
				newpet.setCurrentHp(newpet.getMaxHp());
				newpet.setCurrentMp(newpet.getMaxMp());
				newpet.setExp(newpet.getExpForThisLevel());
				newpet.setCurrentFed(newpet.getMaxFed());
			}
			catch(NullPointerException e)
			{
				_log.warning("PetSummon: failed set stats for summon " + npcId + ".");
				return;
			}

		newpet.setRunning();

		if(!newpet.isRespawned())
			newpet.store();

		player.setPet(newpet);

		player.broadcastPacket(new MagicSkillUse(player, player, _skillId, 1, 1000, 600000));
		player.sendPacket(SUMMON_A_PET);
		L2World.addObject(newpet);

		// Пытаемся найти координаты с разбросом, если не получается - берем без разброса 
		int[] coords = new int[3];
		for(int i = 0; i < 100; i++)
		{
			CharPosition cp = Rnd.coordsRandomize(player, 150);
			if(GeoData.getInstance().canMoveToCoord(cp.x, cp.y, cp.z, player.getX(), player.getY(), player.getZ()))
			{
				coords[0] = cp.x;
				coords[1] = cp.y;
				coords[2] = cp.z;
				break;
			}
		}
		if(coords[0] == 0)
		{
			coords[0] = player.getX();
			coords[1] = player.getY();
			coords[2] = player.getZ();
		}
		newpet.spawnMe(coords[0], coords[1], coords[2]);

		newpet.broadcastPetInfo();

		newpet.startFeed(false);

		if(newpet instanceof L2PetBabyInstance)
			((L2PetBabyInstance) newpet).startHealTask();

		// continue execution in 1 seconds
		ThreadPoolManager.getInstance().scheduleGeneral(new SummonFinalizer(player, newpet), 900);
	}

	static class SummonFinalizer implements Runnable
	{
		private L2Player _activeChar;
		private L2PetInstance _newpet;

		SummonFinalizer(L2Player activeChar, L2PetInstance newpet)
		{
			_activeChar = activeChar;
			_newpet = newpet;
		}

		public void run()
		{
			try
			{
				FastList<L2Character> temp = new FastList<L2Character>();
				temp.add(_newpet);
				_activeChar.sendPacket(new MagicSkillLaunched(_activeChar.getObjectId(), 2046, 1, temp));

				_newpet.setFollowStatus(true);
				// addVisibleObject created the info packets with summon animation
				// if someone comes into range now, the animation shouldnt show any more
				_newpet.setShowSummonAnimation(false);
			}
			catch(Throwable e)
			{
				_log.severe(e.toString());
			}
		}
	}

	public final short[] getItemIds()
	{
		return _itemIds;
	}

	public void onLoad()
	{
		ItemHandler.getInstance().registerItemHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}
}
