package items;

import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.RecipeController;
import L2jFT.game.handler.IItemHandler;
import L2jFT.game.handler.ItemHandler;
import L2jFT.game.model.L2Playable;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2RecipeList;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.network.serverpackets.SystemMessage;

public class Recipes implements IItemHandler, ScriptFile
{
	private static short[] _itemIds = null;

	static final SystemMessage NO_FURTHER_RECIPES_MAY_BE_REGISTERED = new SystemMessage(SystemMessage.NO_FURTHER_RECIPES_MAY_BE_REGISTERED);
	static final SystemMessage CREATE_ITEM_LEVEL_IS_TOO_LOW_TO_REGISTER_THIS_RECIPE = new SystemMessage(SystemMessage.CREATE_ITEM_LEVEL_IS_TOO_LOW_TO_REGISTER_THIS_RECIPE);
	static final SystemMessage THAT_RECIPE_IS_ALREADY_REGISTERED = new SystemMessage(SystemMessage.THAT_RECIPE_IS_ALREADY_REGISTERED);
	static final SystemMessage YOU_ARE_NOT_AUTHORIZED_TO_REGISTER_A_RECIPE = new SystemMessage(SystemMessage.YOU_ARE_NOT_AUTHORIZED_TO_REGISTER_A_RECIPE);

	public Recipes()
	{
		RecipeController rc = RecipeController.getInstance();
		_itemIds = new short[rc.getRecipesCount()];
		for(int i = 0; i < rc.getRecipesCount(); i++)
			_itemIds[i] = rc.getRecipeList(i).getRecipeId();
	}

	public void useItem(L2Playable playable, L2ItemInstance item)
	{
		if(playable == null || !playable.isPlayer)
			return;
		L2Player player = (L2Player) playable;

		L2RecipeList rp = RecipeController.getInstance().getRecipeByItemId(item.getItemId());
		if(rp.isDwarvenRecipe())
		{
			if(player.getDwarvenRecipeLimit() > 0)
			{
				if(player.getDwarvenRecipeBook().length >= player.getDwarvenRecipeLimit())
				{
					player.sendPacket(NO_FURTHER_RECIPES_MAY_BE_REGISTERED);
					return;
				}

				if(rp.getLevel() > player.getSkillLevel(L2Skill.SKILL_CRAFTING))
				{
					player.sendPacket(CREATE_ITEM_LEVEL_IS_TOO_LOW_TO_REGISTER_THIS_RECIPE);
					return;
				}
				if(player.findRecipe(rp))
				{
					player.sendPacket(THAT_RECIPE_IS_ALREADY_REGISTERED);
					return;
				}
				// add recipe to recipebook
				player.registerRecipeList(rp);
				player.sendPacket(new SystemMessage(SystemMessage.S1_HAS_BEEN_ADDED).addString(item.getItem().getName()));
				player.getInventory().destroyItem(item, 1, true, true);
			}
			else
				player.sendPacket(YOU_ARE_NOT_AUTHORIZED_TO_REGISTER_A_RECIPE);
			return;
		}

		if(player.getCommonRecipeLimit() > 0)
		{
			if(player.getCommonRecipeBook().length >= player.getCommonRecipeLimit())
			{
				player.sendPacket(NO_FURTHER_RECIPES_MAY_BE_REGISTERED);
				return;
			}
			if(player.findRecipe(rp))
			{
				player.sendPacket(THAT_RECIPE_IS_ALREADY_REGISTERED);
				return;
			}
			player.registerRecipeList(rp);
			player.sendPacket(new SystemMessage(SystemMessage.S1_HAS_BEEN_ADDED).addString(item.getItem().getName()));
			player.getInventory().destroyItem(item, 1, true, true);
		}
		else
			player.sendPacket(YOU_ARE_NOT_AUTHORIZED_TO_REGISTER_A_RECIPE);
	}

	public short[] getItemIds()
	{
		return _itemIds;
	}

	public void onLoad()
	{
		ItemHandler.getInstance().registerItemHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}
}