package items;

import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.GameTimeController;
import L2jFT.game.ThreadPoolManager;
import L2jFT.game.cache.Msg;
import L2jFT.game.handler.IItemHandler;
import L2jFT.game.handler.ItemHandler;
import L2jFT.game.model.L2Playable;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.instances.L2MonsterInstance;
import L2jFT.game.network.serverpackets.MagicSkillUse;
import L2jFT.game.network.serverpackets.SetupGauge;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.skills.Formulas;
import L2jFT.game.tables.SkillTable;

public class SoulCrystals implements IItemHandler, ScriptFile
{
	// First line is for Red Soul Crystals, second is Green and third is Blue Soul
	// Crystals,
	// ordered by ascending level, from 0 to 13
	public static final short[] _itemIds = {
			4629, 4640, 4651, 4630, 4641, 4652, 4631, 4642, 4653, 4632, 4643, 4654,
			4633, 4644, 4655, 4634, 4645, 4656, 4635, 4646, 4657, 4636, 4647, 4658,
			4637, 4648, 4659, 4638, 4649, 4660, 4639, 4650, 4661, 5577, 5578, 5579,
			5580, 5581, 5582, 5908, 5911, 5914 };

	// Our main method, where everything goes on
	public void useItem(L2Playable playable, L2ItemInstance item)
	{
		if(playable == null || !playable.isPlayer)
			return;
		L2Player player = (L2Player) playable;

		if(player.getTarget() == null || !player.getTarget().isMonster)
		{
			player.sendPacket(Msg.INCORRECT_TARGET);
			player.sendActionFailed();
			return;
		}

		L2MonsterInstance target = (L2MonsterInstance) player.getTarget();

		// u can use soul crystal only when target hp goest to <50%
		if(target.getCurrentHp() > target.getMaxHp() / 2)
		{
			player.sendPacket(new SystemMessage(SystemMessage.THE_SOUL_CRYSTAL_WAS_NOT_ABLE_TO_ABSORB_A_SOUL));
			player.sendActionFailed();
			return;
		}

		// Soul Crystal Casting section
		L2Skill skill = SkillTable.getInstance().getInfo(2096, 1);
		int skillTime = skill.getSkillTime();
		int castEndTime = Formulas.calcMAtkSpd(player, player, skill, skillTime);
		int castInterruptTime = 1000;
		player._castEndTime = 10 + GameTimeController.getGameTicks() + castEndTime / GameTimeController.MILLIS_IN_TICK;
		player._castInterruptTime = GameTimeController.getGameTicks() + castInterruptTime / GameTimeController.MILLIS_IN_TICK;
		player.broadcastPacket(new MagicSkillUse(player, 2096, 1, skillTime, 0));
		player.sendPacket(new SetupGauge(0, skillTime));
		// End Soul Crystal Casting section

		// Continue execution later
		player._skillTask = ThreadPoolManager.getInstance().scheduleGeneral(new CrystalFinalizer(player, target, item.getItemId()), skill.getSkillTime());
	}

	static class CrystalFinalizer implements Runnable
	{
		private L2Player _activeChar;
		private L2MonsterInstance _target;
		private int _crystalId;

		CrystalFinalizer(L2Player activeChar, L2MonsterInstance target, int crystalId)
		{
			_activeChar = activeChar;
			_target = target;
			_crystalId = crystalId;
		}

		public void run()
		{
			if(_activeChar.isDead() || _target.isDead())
				return;
			try
			{
				_target.addAbsorber(_activeChar, _crystalId);
				_activeChar.setTarget(_target);
			}
			catch(Throwable e)
			{
				e.printStackTrace();
			}
		}
	}

	public final short[] getItemIds()
	{
		return _itemIds;
	}

	public void onLoad()
	{
		ItemHandler.getInstance().registerItemHandler(this);
	}

	public void onReload()
	{}

	public void onShutdown()
	{}
}
