package quests.ProofOfClanAlliance;

import L2jFT.extensions.scripts.Functions;
import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.util.CharPosition;
import L2jFT.util.Rnd;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.model.quest.Quest;
import L2jFT.game.model.quest.QuestState;
import L2jFT.game.model.quest.QuestTimer;
import L2jFT.game.model.quest.State;
import L2jFT.game.tables.SkillTable;

import java.util.ArrayList;
import java.util.Arrays;

public class ProofOfClanAlliance extends Quest implements ScriptFile
{
	// Quest Npcs
	public final int SIR_KRISTOF_RODEMAI = 30756;
	public final int STATUE_OF_OFFERING = 30757;
	public final int WITCH_ATHREA = 30758;
	public final int WITCH_KALIS = 30759;

	// Quest Items
	public final short HERB_OF_HARIT = 3832;
	public final short HERB_OF_VANOR = 3833;
	public final short HERB_OF_OEL_MAHUM = 3834;
	public final short BLOOD_OF_EVA = 3835;
	public final short SYMBOL_OF_LOYALTY = 3837;
	public final short PROOF_OF_ALLIANCE = 3874;
	public final short VOUCHER_OF_FAITH = 3873;
	public final short ANTIDOTE_RECIPE = 3872;
	public final short POTION_OF_RECOVERY = 3889;

	// Quest mobs, drop, rates and prices
	public final int[] CHESTS = { 27173, 27174, 27175, 27176, 27177 };
	public final int[][] MOBS = {
			{ 20685, HERB_OF_VANOR }, { 20644, HERB_OF_HARIT },
			{ 20576, HERB_OF_OEL_MAHUM } };

	public final short RATE = 35;
	// stackable items paid to retry chest game: (default 10k adena)
	public final short RETRY_ITEMS = 57;
	public final short RETRY_PRICE = 10000;

	public State CREATED = new State("Start", this);
	public State PART2 = new State("Part2", this);
	public State PART3 = new State("Part3", this);
	public State PART4 = new State("Part4", this);
	public State COMPLETED = new State("Completed", this);

	public void onLoad()
	{
		System.out.println("Loaded Quest: 501: Proof Of Clan Alliance");
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public ProofOfClanAlliance()
	{
		super(501, "ProofOfClanAlliance", "Proof Of Clan Alliance", false);
		setInitialState(CREATED);

		for(int i = SIR_KRISTOF_RODEMAI; i <= WITCH_ATHREA; i++)
		{
			addStartNpc(i);
			addTalkId(i);
		}

		addTalkId(WITCH_KALIS);

		for(int i = SIR_KRISTOF_RODEMAI; i <= WITCH_KALIS; i++)
		{
			addTalkId(i);
			addTalkId(i);
			addTalkId(i);
		}

		CREATED.addQuestDrop(STATUE_OF_OFFERING, SYMBOL_OF_LOYALTY, 1);
		PART3.addQuestDrop(WITCH_KALIS, ANTIDOTE_RECIPE, 1);
		PART4.addQuestDrop(WITCH_KALIS, VOUCHER_OF_FAITH, 1);
		PART4.addQuestDrop(WITCH_KALIS, POTION_OF_RECOVERY, 1);
		PART4.addQuestDrop(WITCH_KALIS, ANTIDOTE_RECIPE, 1);

		for(int[] i : MOBS)
		{
			addKillId(i[0]);
			CREATED.addQuestDrop(i[0], i[1], 1);
		}

		for(int i : CHESTS)
			addKillId(i);
	}

	public QuestState leader(QuestState st)
	{
		L2Clan clan = st.getPlayer().getClan();
		QuestState leader = null;
		if(clan != null && clan.getLeader() != null && clan.getLeader().getPlayer() != null)
			leader = clan.getLeader().getPlayer().getQuestState("ProofOfClanAlliance");
		return leader;
	}

	public void members_finnish(QuestState st)
	{
		if(leader(st) == null)
			return;
		L2Player leader = leader(st).getPlayer();
		if(leader != null)
			for(L2Player player : leader.getClan().getOnlineMembers(""))
				if(player != null && !player.equals(leader) && player.getQuestState("ProofOfClanAlliance") != null)
					player.getQuestState("ProofOfClanAlliance").exitCurrentQuest(true);
	}

	@Override
	public String onEvent(String event, QuestState st)
	{
		String htmltext = event;

		/* ##### Leaders area ###### */

		// SIR_KRISTOF_RODEMAI
		if(event.equalsIgnoreCase("30756-03.htm"))
		{
			st.setState(PART2);
			st.set("cond", "1");
			st.playSound("ItemSound.quest_accept");
		}

		// WITCH_KALIS
		else if(event.equalsIgnoreCase("30759-03.htm"))
		{
			st.setState(PART3);
			st.set("cond", "2");
			st.set("dead_list", " ");
		}
		else if(event.equalsIgnoreCase("30759-07.htm"))
		{
			for(int i = 0; i < 3; i++)
				st.takeItems(SYMBOL_OF_LOYALTY, 1);
			st.giveItems(ANTIDOTE_RECIPE, 1);
			st.addNotifyOfDeath(st.getPlayer());
			st.setState(PART4);
			st.set("cond", "3");
			st.set("chest_count", "0");
			st.set("chest_game", "0");
			st.set("chest_try", "0");
			st.startQuestTimer("poison_timer", 3600000);
			st.getPlayer().altUseSkill(SkillTable.getInstance().getInfo(4082, 1), st.getPlayer());
			htmltext = "30759-07.htm";
		}

		// Timers
		else if(event.equalsIgnoreCase("poison_timer"))
		{
			members_finnish(st);
			st.exitCurrentQuest(true);
			htmltext = "30759-09.htm";
		}
		else if(event.equalsIgnoreCase("chest_timer"))
		{
			htmltext = "";
			if(leader(st).getInt("chest_game") < 2)
				stop_chest_game(st);
		}

		/* ##### Members area ###### */

		// STATUE_OF_OFFERING
		else if(event.equalsIgnoreCase("30757-04.htm"))
		{
			ArrayList<String> deadlist = new ArrayList<String>();
			deadlist.addAll(Arrays.asList(((String) leader(st).get("dead_list")).split(" ")));
			deadlist.add(st.getPlayer().getName());
			String deadstr = "";
			for(String s : deadlist)
				deadstr += s + " ";
			leader(st).set("dead_list", deadstr);
			st.addNotifyOfDeath(st.getPlayer().getClan().getLeader().getPlayer());
			st.getPlayer().reduceCurrentHp(st.getPlayer().getCurrentHp() * 8, st.getPlayer());
			st.giveItems(SYMBOL_OF_LOYALTY, 1);
			st.playSound("ItemSound.quest_accept");
		}
		else if(event.equalsIgnoreCase("30757-05.htm"))
			st.exitCurrentQuest(true);

		// WITCH_ATHREA
		else if(event.equalsIgnoreCase("30758-03.htm"))
			start_chest_game(st);
		else if(event.equalsIgnoreCase("30758-07.htm"))
			if(st.getQuestItemsCount(RETRY_ITEMS) < RETRY_PRICE)
				htmltext = "30758-06.htm";
			else
				st.takeItems(RETRY_ITEMS, RETRY_PRICE);
		return htmltext;
	}

	@Override
	public String onTalk(L2NpcInstance npc, QuestState st)
	{
		int npcId = npc.getNpcId();
		String htmltext = "no_quest.htm";
		if(npcId == SIR_KRISTOF_RODEMAI)
		{
			if(st.getPlayer().getClan() == null || !st.getPlayer().isClanLeader())
			{
				st.exitCurrentQuest(true);
				htmltext = "30756-10.htm";
			}
			else if(st.getPlayer().getClan().getLevel() <= 2)
			{
				st.exitCurrentQuest(true);
				htmltext = "30756-08.htm";
			}
			else if(st.getPlayer().getClan().getLevel() >= 4)
			{
				st.exitCurrentQuest(true);
				htmltext = "30756-09.htm";
			}
			else if(st.getState() == PART4 && st.getQuestItemsCount(VOUCHER_OF_FAITH) > 0)
			{
				st.playSound("ItemSound.quest_fanfare_2");
				st.takeItems(VOUCHER_OF_FAITH, 1);
				st.giveItems(PROOF_OF_ALLIANCE, 1);
				st.addExpAndSp(0, 120000);
				htmltext = "30756-07.htm";
				st.exitCurrentQuest(true);
			}
			else if(st.getState() == PART2 || st.getState() == PART3)
				htmltext = "30756-06.htm";
			else if(st.getQuestItemsCount(PROOF_OF_ALLIANCE) == 0)
			{
				st.set("cond", "0");
				htmltext = "30756-01.htm";
			}
			else
				st.exitCurrentQuest(true);
		}
		else if(npcId == WITCH_KALIS)
		{
			if(st.getPlayer().getClan() == null)
				st.exitCurrentQuest(true);
			else if(st.getPlayer().isClanLeader())
				if(st.getState() == PART2)
					htmltext = "30759-01.htm";
				else if(st.getState() == PART3)
				{
					htmltext = "30759-05.htm";
					if(st.getQuestItemsCount(SYMBOL_OF_LOYALTY) == 3)
					{
						int deads = 0;
						try
						{
							deads = ((String) st.get("dead_list")).split(" ").length;
						}
						finally
						{
							if(deads == 3)
								htmltext = "30759-06.htm";
						}
					}
				}
				else if(st.getState() == PART4)
				{
					if(st.getQuestItemsCount(HERB_OF_HARIT) > 0 && st.getQuestItemsCount(HERB_OF_VANOR) > 0 && st.getQuestItemsCount(HERB_OF_OEL_MAHUM) > 0 && st.getQuestItemsCount(BLOOD_OF_EVA) > 0 && st.getQuestItemsCount(ANTIDOTE_RECIPE) > 0 && st.getInt("chest_game") == 3)
					{
						st.takeItems(ANTIDOTE_RECIPE, 1);
						st.takeItems(HERB_OF_HARIT, 1);
						st.takeItems(HERB_OF_VANOR, 1);
						st.takeItems(HERB_OF_OEL_MAHUM, 1);
						st.takeItems(BLOOD_OF_EVA, 1);
						st.giveItems(POTION_OF_RECOVERY, 1);
						st.giveItems(VOUCHER_OF_FAITH, 1);
						QuestTimer timer = leader(st).getQuestTimer("poison_timer");
						if(timer != null)
							timer.cancel();
						members_finnish(st);
						st.getPlayer().stopEffect((short) 4082);
						htmltext = "30759-08.htm";
						st.playSound("ItemSound.quest_finish");
					}
					else if(st.getQuestItemsCount(VOUCHER_OF_FAITH) == 0)
						htmltext = "30759-10.htm";
					else
						st.exitCurrentQuest(true);
				}
				else
					try
					{
						if(leader(st).getState() == PART4)
							htmltext = "30759-11.htm";
					}
					catch(Exception e)
					{
						st.exitCurrentQuest(true);
					}
		}
		else if(npcId == STATUE_OF_OFFERING)
		{
			if(st.getPlayer().getClan() == null)
				st.exitCurrentQuest(true);
			else if(st.getPlayer().isClanLeader())
			{
				if(st.getState() == PART2 || st.getState() == PART3 || st.getState() == PART4)
					htmltext = "30757-03.htm";
				else
					st.exitCurrentQuest(true);
			}
			else if(st.getPlayer().getLevel() <= 39)
			{
				st.exitCurrentQuest(true);
				htmltext = "30757-02.htm";
			}
			else
			{
				String[] dlist = null;
				int deads = 3;
				try
				{
					dlist = ((String) leader(st).get("dead_list")).split(" ");
					deads = dlist.length;
				}
				catch(Exception e)
				{
					st.exitCurrentQuest(true);
				}
				if(deads < 3)
				{
					Boolean flag = false;
					if(dlist != null)
						for(String str : dlist)
							if(st.getPlayer().getName().equalsIgnoreCase(str))
								flag = true;
					if(!flag)
					{
						if(st.getQuestItemsCount(SYMBOL_OF_LOYALTY) <= 0)
							htmltext = "30757-01.htm";
						else
						{
							htmltext = "30757-06.htm";
							st.exitCurrentQuest(true);
						}
					}
					else
						htmltext = "you cannot die again!";
				}
			}
		}
		else if(npcId == WITCH_ATHREA)
			if(st.getPlayer().getClan() == null)
				st.exitCurrentQuest(true);
			else if(st.getPlayer().isClanLeader())
				st.exitCurrentQuest(true);
			else if(leader(st) != null)
			{
				// Проверяем, участвует ли в квесте
				String[] dlist;
				try
				{
					dlist = ((String) leader(st).get("dead_list")).split(" ");
				}
				catch(Exception e)
				{
					st.exitCurrentQuest(true);
					return "Who are you?";
				}
				Boolean flag = false;
				if(dlist != null)
					for(String str : dlist)
						if(st.getPlayer().getName().equalsIgnoreCase(str))
							flag = true;
				if(!flag)
				{
					st.exitCurrentQuest(true);
					return "Who are you?";
				}

				int game_state = leader(st).getInt("chest_game");
				if(game_state == 0)
				{
					if(leader(st).getInt("chest_try") == 0)
						htmltext = "30758-01.htm";
					else
						htmltext = "30758-05.htm";
				}
				else if(game_state == 1)
					htmltext = "30758-09.htm";
				else if(game_state == 2)
				{
					htmltext = "30758-08.htm";
					st.playSound("ItemSound.quest_finish");
					st.giveItems(BLOOD_OF_EVA, 1);
					QuestTimer timer = leader(st).getQuestTimer("chest_timer");
					if(timer != null)
						timer.cancel();
					stop_chest_game(st);
					leader(st).set("chest_game", "3");
				}
			}
			else
				st.exitCurrentQuest(true);
		return htmltext;
	}

	@Override
	public String onKill(L2NpcInstance npc, QuestState st)
	{
		// first part, general checking
		int npcId = npc.getNpcId();
		if(leader(st) == null)
		{
			st.exitCurrentQuest(true);
			return "Quest Failed";
		}

		QuestTimer timer = leader(st).getQuestTimer("poison_timer");
		if(timer == null)
		{
			stop_chest_game(st);
			return "Quest Failed";
		}

		// second part, herbs gathering
		for(int[] m : MOBS)
			if(npcId == m[0] && st.getInt(String.valueOf(m[1])) == 0)
				if(Rnd.chance(RATE))
				{
					st.giveItems(m[1], 1);
					leader(st).set(String.valueOf(m[1]), "1");
					st.playSound("ItemSound.quest_middle");
					return null;
				}
		// third part, chest game
		for(int i : CHESTS)
			if(npcId == i)
			{
				timer = leader(st).getQuestTimer("chest_timer");
				if(timer == null)
				{
					stop_chest_game(st);
					return "Time is up!";
				}
				if(Rnd.chance(25))
				{
					Functions.npcSayInRange(npc, "###### BINGO! ######", 1000);
					int count = leader(st).getInt("chest_count");
					if(count < 4)
					{
						count += 1;
						leader(st).set("chest_count", String.valueOf(count));
					}
					if(count >= 4)
					{
						stop_chest_game(st);
						leader(st).set("chest_game", "2");
						timer.cancel();
						st.playSound("ItemSound.quest_middle");
					}
					else
						st.playSound("ItemSound.quest_itemget");
				}
				return null;
			}
		return null;
	}

	public void start_chest_game(QuestState st)
	{
		leader(st).set("chest_game", "1");
		leader(st).set("chest_count", "0");
		int attempts = leader(st).getInt("chest_try");
		leader(st).set("chest_try", String.valueOf(attempts + 1));
		// randomize_chests(st);
		for(int n = 1; n <= 5; n++)
			for(int i : CHESTS)
			{
				CharPosition pos = Rnd.coordsRandomize(102100, 103450, -3400, 0, 100, 100);
				leader(st).getPcSpawn().addSpawn(i, pos.x, pos.y, pos.z, 60000);
			}
		leader(st).startQuestTimer("chest_timer", 60000);
	}

	public void stop_chest_game(QuestState st)
	{
		leader(st).getPcSpawn().removeAllSpawn();
		leader(st).set("chest_game", "0");
	}

	@Override
	public String onDeath(L2NpcInstance npc, L2Character pc, QuestState st)
	{
		if(st.getPlayer() == pc)
		{
			if(leader(st) != null)
			{
				QuestTimer timer1 = leader(st).getQuestTimer("poison_timer");
				QuestTimer timer2 = leader(st).getQuestTimer("chest_timer");
				if(timer1 != null)
					timer1.cancel();
				if(timer2 != null)
					timer2.cancel();
			}
			st.exitCurrentQuest(true);
		}
		return null;
	}
}