package quests.TheClansReputation;

import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.model.quest.Quest;
import L2jFT.game.model.quest.QuestState;
import L2jFT.game.model.quest.State;
import L2jFT.game.network.serverpackets.PledgeShowInfoUpdate;
import L2jFT.game.network.serverpackets.SystemMessage;

public class TheClansReputation extends Quest implements ScriptFile
{
	public State CREATED = new State("Start", this);
	public State STARTED = new State("Started", this);

	// Quest NPC
	static final int SIR_ERIC_RODEMAI = 30868;

	// Quest Items
	static final int NUCLEUS_OF_FLAMESTONE_GIANT = 8494; // Nucleus of Flamestone Giant : Nucleus obtained by defeating Flamestone Giant
	static final int THEMIS_SCALE = 8277; // Themis' Scale : Obtain this scale by defeating Palibati Queen Themis.
	static final int NUCLEUS_OF_HEKATON_PRIME = 8279; // Nucleus of Hekaton Prime : Nucleus obtained by defeating Hekaton Prime
	static final int TIPHON_SHARD = 8280; // Tiphon Shard : Debris obtained by defeating Tiphon, Gargoyle Lord.
	static final int GLAKIS_NECLEUS = 8281; // Glaki's Necleus : Nucleus obtained by defeating Glaki, the last lesser Giant.
	static final int RAHHAS_FANG = 8282; // Rahha's Fang : Fangs obtained by defeating Rahha.

	// Quest Raid Bosses
	static final int FLAMESTONE_GIANT = 25524;
	static final int PALIBATI_QUEEN_THEMIS = 25252;
	static final int HEKATON_PRIME = 25140;
	static final int GARGOYLE_LORD_TIPHON = 25255;
	static final int LAST_LESSER_GIANT_GLAKI = 25245;
	static final int RAHHA = 25051;

	// Reward 100-300 points per boss (off-like)
	static final int CLAN_POINTS_REWARD = 250; // middle

	// id:[RaidBossNpcId,questItemId]
	static final int[][] REWARDS_LIST = {
			{ 0, 0 }, { PALIBATI_QUEEN_THEMIS, THEMIS_SCALE },
			{ HEKATON_PRIME, NUCLEUS_OF_HEKATON_PRIME },
			{ GARGOYLE_LORD_TIPHON, TIPHON_SHARD },
			{ LAST_LESSER_GIANT_GLAKI, GLAKIS_NECLEUS }, { RAHHA, RAHHAS_FANG },
			{ FLAMESTONE_GIANT, NUCLEUS_OF_FLAMESTONE_GIANT } };

	static final int[][] RADAR = {
			{ 0, 0, 0 }, { 192346, 21528, -3648 }, { 191979, 54902, -7658 },
			{ 170038, -26236, -3824 }, { 171762, 55028, -5992 },
			{ 117232, -9476, -3320 }, { 144218, -5816, -4722 }, };

	public void onLoad()
	{
		System.out.println("Loaded Quest: 508: The Clans Reputation");
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public TheClansReputation()
	{
		super(508, "TheClansReputation", "The Clans Reputation", true);
		setInitialState(CREATED);

		addStartNpc(SIR_ERIC_RODEMAI);
		addTalkId(SIR_ERIC_RODEMAI);
		addTalkId(SIR_ERIC_RODEMAI);

		for(int[] i : REWARDS_LIST)
		{
			addKillId(i[0]);
			STARTED.addQuestDrop(i[0], i[1], 1);
		}
	}

	@Override
	public String onEvent(String event, QuestState st)
	{
		int cond = st.getInt("cond");
		String htmltext = event;
		if(event.equalsIgnoreCase("30868-0.htm") && cond == 0)
		{
			st.set("cond", "1");
			st.setState(STARTED);
		}
		else if(isdigit(event))
		{
			int evt = Integer.parseInt(event);
			st.set("raid", event);
			htmltext = "30868-" + event + ".htm";
			int x = RADAR[evt][0];
			int y = RADAR[evt][1];
			int z = RADAR[evt][2];
			if(x + y + z > 0)
				st.addRadar(x, y, z);
			st.playSound("ItemSound.quest_accept");
		}
		else if(event.equalsIgnoreCase("30868-7.htm"))
		{
			st.playSound("ItemSound.quest_finish");
			st.exitCurrentQuest(true);
		}
		return htmltext;
	}

	private boolean isdigit(String s)
	{
		try
		{
			Integer.parseInt(s);
		}
		catch(NumberFormatException e)
		{
			return false;
		}
		return true;
	}

	@Override
	public String onTalk(L2NpcInstance npc, QuestState st)
	{
		String htmltext = "<html><body>I have nothing to say you</body></html>";
		L2Clan clan = st.getPlayer().getClan();

		if(clan == null)
		{
			st.exitCurrentQuest(true);
			htmltext = "30868-0a.htm";
		}
		else if(clan.getLeader().getPlayer() != st.getPlayer())
		{
			st.exitCurrentQuest(true);
			htmltext = "30868-0a.htm";
		}
		else if(clan.getLevel() < 5)
		{
			st.exitCurrentQuest(true);
			htmltext = "30868-0b.htm";
		}
		else
		{
			int cond = st.getInt("cond");
			int raid = st.getInt("raid");
			State id = st.getState();
			if(id.equals(CREATED) && cond == 0)
				htmltext = "30868-0c.htm";
			else if(id.equals(STARTED) && cond == 1)
			{
				int item = REWARDS_LIST[raid][1];
				int count = st.getQuestItemsCount(item);
				if(count == 0)
					htmltext = "30868-" + raid + "a.htm";
				else if(count == 1)
				{
					htmltext = "30868-" + raid + "b.htm";
					clan.incReputation(CLAN_POINTS_REWARD, true);
					clan.broadcastToOnlineMembers(new PledgeShowInfoUpdate(clan));
					st.getPlayer().sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_SUCCESSFULLY_COMPLETED_A_CLAN_QUEST_S1_POINTS_HAVE_BEEN_ADDED_TO_YOUR_CLAN_REPUTATION_SCORE).addNumber(CLAN_POINTS_REWARD));
					st.takeItems(item, 1);
				}
			}
		}
		return htmltext;
	}

	@Override
	public String onKill(L2NpcInstance npc, QuestState st)
	{
		QuestState id = null;
		if(st.getPlayer().getClan().getLeader().getPlayer() == st.getPlayer())
			id = st.getPlayer().getQuestState("TheClansReputation");
		else if(st.getPlayer().getClan().getLeader().getPlayer().getDistance(npc) <= 1600)
			id = st.getPlayer().getClan().getLeader().getPlayer().getQuestState("TheClansReputation");
		if(id == null)
			return null;
		if(st.getInt("cond") == 1 && st.getState() == STARTED)
		{
			int raid = REWARDS_LIST[st.getInt("raid")][0];
			int item = REWARDS_LIST[st.getInt("raid")][1];
			int npcId = npc.getNpcId();
			if(npcId == raid && st.getQuestItemsCount(item) == 0)
			{
				st.giveItems(item, 1);
				st.playSound("ItemSound.quest_middle");
			}
		}
		return null;
	}
}