package quests.YokeOfThePast;

import L2jFT.Config;
import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.model.base.Experience;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.model.quest.QuestJython;
import L2jFT.game.model.quest.QuestState;
import L2jFT.game.model.quest.State;
import L2jFT.util.Rnd;

public class YokeOfThePast extends QuestJython implements ScriptFile
{
	final short ANCIENT_SCROLL = 5902;
	final short BLANK_SCROLL = 5965;

	public State CREATED = new State("Start", this);
	public State STARTED = new State("Started", this);

	public void onLoad()
	{
		System.out.println("Loaded Quest: 385: Yoke of the Past");
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public YokeOfThePast()
	{
		super(385, "YokeOfThePast", "Yoke of the Past", true);
		setInitialState(CREATED);

		for(int npcId = 31095; npcId <= 31126; npcId++)
			if(npcId != 31111 && npcId != 31112 && npcId != 31113)
			{
				addStartNpc(npcId);
				addTalkId(npcId);
				addTalkId(npcId);
			}

		for(int mobs = 21208; mobs <= 21256; mobs++)
		{
			addKillId(mobs);
			STARTED.addQuestDrop(mobs, ANCIENT_SCROLL, 1);
		}
	}

	public boolean checkNPC(int npc)
	{
		if(npc >= 31095 && npc <= 31126)
			if(npc != 31100 && npc != 31111 && npc != 31112 && npc != 31113)
				return true;
		return false;
	}

	@Override
	public String onEvent(String event, QuestState st)
	{
		String htmltext = event;
		if(event.equalsIgnoreCase("14.htm"))
		{
			st.setState(STARTED);
			st.playSound("ItemSound.quest_accept");
			st.set("cond", "1");
		}
		else if(event.equalsIgnoreCase("16.htm"))
		{
			htmltext = "quit.htm";
			st.playSound("ItemSound.quest_finish");
			st.exitCurrentQuest(true);
		}
		return htmltext;
	}

	@Override
	public String onKill(L2NpcInstance npc, QuestState st)
	{
		int rand = 50;
		int diff = calculateLevelDiffForDrop(npc.getLevel(), st.getPlayer().getLevel());
		if(diff > 0)
			rand *= Experience.penaltyModifier(diff - 1, 9);
		if(rand != 0 && Rnd.chance(rand))
		{
			st.giveItems(ANCIENT_SCROLL, Math.round(Config.RATE_QUESTS_REWARD));
			st.playSound("ItemSound.quest_itemget");
		}
		return null;
	}

	private int calculateLevelDiffForDrop(int mobLevel, int player)
	{
		if(Config.DEEPBLUE_DROP_RULES && player - mobLevel >= 9)
			return player - mobLevel - 8;
		return 0;
	}

	@Override
	public String onTalk(L2NpcInstance npc, QuestState st)
	{
		int npcId = npc.getNpcId();
		String htmltext = "<html><body>I have nothing to say you</body></html>";
		if(checkNPC(npcId) && st.getInt("cond") == 0)
			if(st.getPlayer().getLevel() < 20)
			{
				htmltext = "00.htm";
				st.exitCurrentQuest(true);
			}
			else
				htmltext = "10.htm";
		else if(st.getInt("cond") == 1 && st.getQuestItemsCount(ANCIENT_SCROLL) == 0)
			htmltext = "17.htm";
		else if(st.getInt("cond") == 1 && st.getQuestItemsCount(ANCIENT_SCROLL) > 0)
		{
			htmltext = "16.htm";
			st.giveItems(BLANK_SCROLL, st.getQuestItemsCount(ANCIENT_SCROLL));
			st.takeItems(ANCIENT_SCROLL, -1);
		}
		else
			st.exitCurrentQuest(true);
		return htmltext;
	}
}