package quests._403_PathToRogue;

import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.model.quest.Quest;
import L2jFT.game.model.quest.QuestState;
import L2jFT.game.model.quest.State;
import L2jFT.util.Rnd;

public class _403_PathToRogue extends Quest implements ScriptFile
{
	//npc
	public final int BEZIQUE = 30379;
	public final int NETI = 30425;
	//mobs
	public final int TRACKER_SKELETON = 20035;
	public final int TRACKER_SKELETON_LEADER = 20042;
	public final int SKELETON_SCOUT = 20045;
	public final int SKELETON_BOWMAN = 20051;
	public final int RUIN_SPARTOI = 20054;
	public final int RAGING_SPARTOI = 20060;
	public final int CATS_EYE_BANDIT = 27038;
	//items
	public final int BEZIQUES_LETTER_ID = 1180;
	public final int SPATOIS_BONES_ID = 1183;
	public final int HORSESHOE_OF_LIGHT_ID = 1184;
	public final int WANTED_BILL_ID = 1185;
	public final int STOLEN_JEWELRY_ID = 1186;
	public final int STOLEN_TOMES_ID = 1187;
	public final int STOLEN_RING_ID = 1188;
	public final int STOLEN_NECKLACE_ID = 1189;
	public final int BEZIQUES_RECOMMENDATION_ID = 1190;
	public final int NETIS_BOW_ID = 1181;
	public final int NETIS_DAGGER_ID = 1182;
	//MobsTable {MOB_ID,CHANCE}
	public final int[][] MobsTable = {
			{ TRACKER_SKELETON, 2 }, { TRACKER_SKELETON_LEADER, 3 },
			{ SKELETON_SCOUT, 2 }, { SKELETON_BOWMAN, 2 }, { RUIN_SPARTOI, 8 },
			{ RAGING_SPARTOI, 8 } };

	public final int[] STOLEN_ITEM = {
			STOLEN_JEWELRY_ID, STOLEN_TOMES_ID, STOLEN_RING_ID, STOLEN_NECKLACE_ID };

	public State CREATED = new State("Start", this);
	public State STARTED = new State("Started", this);
	public State COMPLETED = new State("Completed", this);

	public void onLoad()
	{
		System.out.println("Loaded Quest: 403: Path to Rogue");
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public _403_PathToRogue()
	{
		super(403, "_403_PathToRogue", "Path to Rogue", false);
		setInitialState(CREATED);
		addStartNpc(BEZIQUE);

		addTalkId(BEZIQUE);
		addTalkId(NETI);

		addKillId(CATS_EYE_BANDIT);

		for(int[] element : MobsTable)
		{
			addKillId(element[0]);
			STARTED.addQuestDrop(element[0], SPATOIS_BONES_ID, 1);
		}
		for(int i : STOLEN_ITEM)
			STARTED.addQuestDrop(CATS_EYE_BANDIT, i, 1);
		STARTED.addQuestDrop(30425, NETIS_BOW_ID, 1);
		STARTED.addQuestDrop(30425, NETIS_DAGGER_ID, 1);
		STARTED.addQuestDrop(30379, WANTED_BILL_ID, 1);
		STARTED.addQuestDrop(30425, HORSESHOE_OF_LIGHT_ID, 1);
		STARTED.addQuestDrop(30379, BEZIQUES_LETTER_ID, 1);
	}

	@Override
	public String onEvent(String event, QuestState st)
	{
		String htmltext = event;
		if(event.equalsIgnoreCase("30379_2"))
		{
			if(st.getPlayer().getClassId().getId() == 0x00)
			{
				if(st.getPlayer().getLevel() >= 19)
				{
					if(st.getQuestItemsCount(BEZIQUES_RECOMMENDATION_ID) > 0)
						htmltext = "30379-04.htm";
					else
						htmltext = "30379-05.htm";
				}
				else
					htmltext = "30379-03.htm";
			}
			else if(st.getPlayer().getClassId().getId() == 0x07)
				htmltext = "30379-02a.htm";
			else
				htmltext = "30379-02.htm";
		}
		else if(event.equalsIgnoreCase("1"))
		{
			st.set("cond", "1");
			st.setState(STARTED);
			st.giveItems(BEZIQUES_LETTER_ID, 1);
			htmltext = "30379-06.htm";
			st.playSound("ItemSound.quest_accept");
		}
		else if(event.equalsIgnoreCase("30425_1"))
		{
			st.takeItems(BEZIQUES_LETTER_ID, 1);
			if(st.getQuestItemsCount(NETIS_BOW_ID) < 1)
				st.giveItems(NETIS_BOW_ID, 1);
			if(st.getQuestItemsCount(NETIS_DAGGER_ID) < 1)
				st.giveItems(NETIS_DAGGER_ID, 1);
			st.set("cond", "2");
			htmltext = "30425-05.htm";
		}
		return htmltext;
	}

	@Override
	public String onTalk(L2NpcInstance npc, QuestState st)
	{
		String htmltext = "<html><body>I have nothing to say you</body></html>";
		int npcId = npc.getNpcId();
		int cond = st.getInt("cond");
		if(npcId == BEZIQUE)
		{
			if(cond == 6 && st.getQuestItemsCount(HORSESHOE_OF_LIGHT_ID) < 1 && st.getQuestItemsCount(STOLEN_JEWELRY_ID) + st.getQuestItemsCount(STOLEN_TOMES_ID) + st.getQuestItemsCount(STOLEN_RING_ID) + st.getQuestItemsCount(STOLEN_NECKLACE_ID) == 4)
			{
				htmltext = "30379-09.htm";
				st.giveItems(BEZIQUES_RECOMMENDATION_ID, 1);
				st.takeItems(NETIS_BOW_ID, 1);
				st.takeItems(NETIS_DAGGER_ID, 1);
				st.takeItems(WANTED_BILL_ID, 1);
				for(int i : STOLEN_ITEM)
					st.takeItems(i, -1);
				st.unset("cond");
				st.exitCurrentQuest(true);
				st.playSound("ItemSound.quest_finish");
			}
			else if(cond == 1 && st.getQuestItemsCount(HORSESHOE_OF_LIGHT_ID) < 1 && st.getQuestItemsCount(BEZIQUES_LETTER_ID) > 0)
				htmltext = "30379-07.htm";
			else if(cond == 4 && st.getQuestItemsCount(HORSESHOE_OF_LIGHT_ID) > 0)
			{
				htmltext = "30379-08.htm";
				st.takeItems(HORSESHOE_OF_LIGHT_ID, 1);
				st.giveItems(WANTED_BILL_ID, 1);
				st.set("cond", "5");
			}
			else if(cond > 1 && st.getQuestItemsCount(NETIS_BOW_ID) > 0 && st.getQuestItemsCount(NETIS_DAGGER_ID) > 0 && st.getQuestItemsCount(WANTED_BILL_ID) < 1)
				htmltext = "30379-10.htm";
			else if(cond == 5 && st.getQuestItemsCount(WANTED_BILL_ID) > 0)
				htmltext = "30379-11.htm";
			else
				htmltext = "30379-01.htm";
		}
		else if(npcId == NETI)
			if(cond == 1 && st.getQuestItemsCount(BEZIQUES_LETTER_ID) > 0)
				htmltext = "30425-01.htm";
			else if(cond == 2 | cond == 3 && st.getQuestItemsCount(SPATOIS_BONES_ID) < 10)
			{
				htmltext = "30425-06.htm";
				st.set("cond", "2");
			}
			else if(cond == 3 && st.getQuestItemsCount(SPATOIS_BONES_ID) > 9)
			{
				htmltext = "30425-07.htm";
				st.takeItems(SPATOIS_BONES_ID, -1);
				st.giveItems(HORSESHOE_OF_LIGHT_ID, 1);
				st.set("cond", "4");
			}
			else if(cond == 4 && st.getQuestItemsCount(HORSESHOE_OF_LIGHT_ID) > 0)
				htmltext = "30425-08.htm";
		return htmltext;
	}

	@Override
	public String onKill(L2NpcInstance npc, QuestState st)
	{
		int npcId = npc.getNpcId();
		int cond = st.getInt("cond");
		if(st.getItemEquipped(7) == NETIS_BOW_ID | st.getItemEquipped(7) == NETIS_DAGGER_ID)
		{
			for(int[] element : MobsTable)
				if(cond == 2 && npcId == element[0] && Rnd.chance(10 * element[1]) && st.getQuestItemsCount(SPATOIS_BONES_ID) < 10)
				{
					st.giveItems(SPATOIS_BONES_ID, 1);
					if(st.getQuestItemsCount(SPATOIS_BONES_ID) == 10)
					{
						st.playSound("ItemSound.quest_middle");
						st.set("cond", "3");
					}
					else
						st.playSound("ItemSound.quest_itemget");
				}
			if(cond == 5 && npcId == CATS_EYE_BANDIT)
				if(st.getQuestItemsCount(WANTED_BILL_ID) > 0)
				{
					int n = Rnd.get(4);
					if(st.getQuestItemsCount(STOLEN_ITEM[n]) == 0)
					{
						st.giveItems(STOLEN_ITEM[n], 1);
						if(st.getQuestItemsCount(STOLEN_JEWELRY_ID) + st.getQuestItemsCount(STOLEN_TOMES_ID) + st.getQuestItemsCount(STOLEN_RING_ID) + st.getQuestItemsCount(STOLEN_NECKLACE_ID) < 4)
							st.playSound("ItemSound.quest_itemget");
						else
						{
							st.playSound("ItemSound.quest_middle");
							st.set("cond", "6");
						}
					}
				}
		}
		return null;
	}
}