package quests._406_PathToElvenKnight;

import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.model.quest.Quest;
import L2jFT.game.model.quest.QuestState;
import L2jFT.game.model.quest.State;
import L2jFT.util.Rnd;

public class _406_PathToElvenKnight extends Quest implements ScriptFile
{
	public final int TRACKER_SKELETON = 20035;
	public final int TRACKER_SKELETON_LEADER = 20042;
	public final int OL_MAHUM_NOVICE = 20782;
	public final int SORIUS = 30327;
	public final int KLUTO = 30317;

	public final short SORIUS_LETTER1 = 1202;
	public final short KLUTO_BOX = 1203;
	public final short ELVEN_KNIGHT_BROOCH = 1204;
	public final short TOPAZ_PIECE = 1205;
	public final short EMERALD_PIECE = 1206;
	public final short KLUTO_MEMO = 1276;

	public State CREATED = new State("Start", this);
	public State STARTED = new State("Started", this);
	public State COMPLETED = new State("Completed", this);

	public void onLoad()
	{
		System.out.println("Loaded Quest: 406: Path to Elven Knight");
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public _406_PathToElvenKnight()
	{
		super(406, "_406_PathToElvenKnight", "Path To Elven Knight", false);
		setInitialState(CREATED);
		addStartNpc(SORIUS);
		addTalkId(SORIUS);
		addTalkId(KLUTO);
		addTalkId(SORIUS);
		addTalkId(SORIUS);

		addKillId(OL_MAHUM_NOVICE);
		addKillId(TRACKER_SKELETON);
		addKillId(TRACKER_SKELETON_LEADER);
	}

	@Override
	public String onEvent(String event, QuestState st)
	{
		String htmltext = event;
		if(event.equalsIgnoreCase("30327-05.htm"))
		{
			if(st.getPlayer().getClassId().getId() == 0x12)
			{
				if(st.getQuestItemsCount(ELVEN_KNIGHT_BROOCH) > 0)
				{
					htmltext = "30327-04.htm";
					st.exitCurrentQuest(true);
				}
				else if(st.getPlayer().getLevel() < 19)
				{
					htmltext = "30327-03.htm";
					st.exitCurrentQuest(true);
				}
			}
			else if(st.getPlayer().getClassId().getId() == 0x13)
			{
				htmltext = "30327-02a.htm";
				st.exitCurrentQuest(true);
			}
			else
			{
				htmltext = "30327-02.htm";
				st.exitCurrentQuest(true);
			}
		}
		else if(event.equalsIgnoreCase("30327-06.htm"))
		{
			st.set("cond", "1");
			st.setState(STARTED);
			st.playSound("ItemSound.quest_accept");
		}

		else if(event.equalsIgnoreCase("30317-02.htm"))
		{
			if(st.getInt("cond") == 3)
			{
				st.takeItems(SORIUS_LETTER1, -1);
				if(st.getQuestItemsCount(KLUTO_MEMO) == 0)
				{
					st.giveItems(KLUTO_MEMO, 1);
					st.set("cond", "4");
				}
				else
					htmltext = "no_quest.htm";
			}
		}
		else
			htmltext = "no_quest.htm";

		return htmltext;
	}

	@Override
	public String onTalk(L2NpcInstance npc, QuestState st)
	{
		int npcId = npc.getNpcId();
		String htmltext = "no_quest.htm";
		State id = st.getState();
		int cond = 0;
		if(!id.equals(CREATED))
			cond = st.getInt("cond");
		if(npcId == SORIUS)
		{
			if(cond == 0)
				htmltext = "30327-01.htm";
			else if(cond == 1)
			{
				if(st.getQuestItemsCount(TOPAZ_PIECE) == 0)
					htmltext = "30327-07.htm";
				else
					htmltext = "30327-08.htm";
			}
			else if(cond == 2)
			{
				if(st.getQuestItemsCount(SORIUS_LETTER1) == 0)
					st.giveItems(SORIUS_LETTER1, 1);
				st.set("cond", "3");
				htmltext = "30327-09.htm";
			}
			else if(cond == 3 || cond == 4 || cond == 5)
				htmltext = "30327-11.htm";
			else if(cond == 6)
			{
				st.takeItems(KLUTO_BOX, -1);
				if(st.getQuestItemsCount(ELVEN_KNIGHT_BROOCH) == 0)
				{
					st.addExpAndSp(3200, 2280);
					st.giveItems(ELVEN_KNIGHT_BROOCH, 1);
				}
				st.exitCurrentQuest(true);
				st.playSound("ItemSound.quest_finish");
				htmltext = "30327-10.htm";
			}
		}
		else if(npcId == KLUTO)
			if(cond == 3)
				htmltext = "30317-01.htm";
			else if(cond == 4)
			{
				if(st.getQuestItemsCount(EMERALD_PIECE) == 0)
					htmltext = "30317-03.htm";
				else
					htmltext = "30317-04.htm";
			}
			else if(cond == 5)
			{
				st.takeItems(EMERALD_PIECE, -1);
				st.takeItems(TOPAZ_PIECE, -1);
				if(st.getQuestItemsCount(KLUTO_BOX) == 0)
					st.giveItems(KLUTO_BOX, 1);
				st.takeItems(KLUTO_MEMO, -1);
				st.set("cond", "6");
				htmltext = "30317-05.htm";
			}
			else if(cond == 6)
				htmltext = "30317-06.htm";
		return htmltext;
	}

	@Override
	public String onKill(L2NpcInstance npc, QuestState st)
	{
		int npcId = npc.getNpcId();
		int cond = st.getInt("cond");
		if(npcId == TRACKER_SKELETON || npcId == TRACKER_SKELETON_LEADER)
		{
			if(cond == 1 && st.getQuestItemsCount(TOPAZ_PIECE) < 20 && Rnd.chance(70))
			{
				st.giveItems(TOPAZ_PIECE, 1);
				if(st.getQuestItemsCount(TOPAZ_PIECE) == 20)
				{
					st.playSound("ItemSound.quest_middle");
					st.set("cond", "2");
				}
				else
					st.playSound("ItemSound.quest_itemget");
			}
		}
		else if(npcId == OL_MAHUM_NOVICE)
			if(cond == 4 && st.getQuestItemsCount(EMERALD_PIECE) < 20 && Rnd.chance(50))
			{
				st.giveItems(EMERALD_PIECE, 1);
				if(st.getQuestItemsCount(EMERALD_PIECE) == 20)
				{
					st.playSound("ItemSound.quest_middle");
					st.set("cond", "5");
				}
				else
					st.playSound("ItemSound.quest_itemget");
			}
		return null;
	}
}
