package quests._999_C4Tutorial;

import java.util.Arrays;
import java.util.HashMap;

import L2jFT.extensions.scripts.ScriptFile;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.instances.L2MonsterInstance;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.model.quest.Quest;
import L2jFT.game.model.quest.QuestState;
import L2jFT.game.model.quest.State;
import L2jFT.util.Rnd;

public class _999_C4Tutorial extends Quest implements ScriptFile
{
	public State CREATED = new State("Start", this);
	public State COMPLETED = new State("Completed", this);

	static final HashMap<Integer, Integer[]> NewbieHelper = new HashMap<Integer, Integer[]>();
	static
	{
		NewbieHelper.put(30009, new Integer[] { 0, 0, 1067 });
		NewbieHelper.put(30019, new Integer[] { 10, 0, 1068 });
		NewbieHelper.put(30400, new Integer[] { -1, 1, 1069 });
		NewbieHelper.put(30131, new Integer[] { -1, 2, 1070 });
		NewbieHelper.put(30575, new Integer[] { -1, 3, 1496 });
		NewbieHelper.put(30530, new Integer[] { -1, 4, 1498 });
	}

	static final HashMap<Integer, Integer[]> GrandMaster = new HashMap<Integer, Integer[]>();
	static
	{
		GrandMaster.put(30008, new Integer[] { 1067, -84081, 243277, -3723 });
		GrandMaster.put(30017, new Integer[] { 1068, -84081, 243277, -3723 });
		GrandMaster.put(30370, new Integer[] { 1069, 45475, 48359, -3060 });
		GrandMaster.put(30129, new Integer[] { 1070, 17024, 13296, -3744 });
		GrandMaster.put(30573, new Integer[] { 1496, -45032, -113598, -192 });
		GrandMaster.put(30528, new Integer[] { 1498, 115632, -177996, -905 });
	}

	static final int BEGINER_TRAVEL_TOKEN = 8542;
	int[] NewbieGuide = { 30598, 30599, 30600, 30602, 30601 };

	public void onLoad()
	{
		System.out.println("Loaded Quest: 999: C4 Tutorial Quest");
	}

	public void onReload()
	{}

	public void onShutdown()
	{}

	public _999_C4Tutorial()
	{
		super(999, "_999_C4Tutorial", "Tutorial", false);

		setInitialState(CREATED);

		for(int npcId : NewbieHelper.keySet())
		{
			setInitialState(CREATED);
			addStartNpc(npcId);
			addTalkId(npcId);
		}

		for(int npcId : GrandMaster.keySet())
			addTalkId(npcId);

		for(int npcId : NewbieGuide)
			addTalkId(npcId);

		addKillId(18342);
	}

	@Override
	public String onEvent(String event, QuestState st)
	{
		String htmltext = event;
		QuestState qs = st.getPlayer().getQuestState("_999_C4Tutorial");
		if(qs != null)
		{
			int Ex = qs.getInt("Ex");
			int classId = st.getPlayer().getClassId().getId();
			if(event.equalsIgnoreCase("TimerEx_NewbieHelper"))
			{
				if(Ex == 0)
				{
					st.playTutorialVoice("tutorial_voice_009a");
					qs.set("Ex", "1");
				}
				else if(Ex == 3)
				{
					st.playTutorialVoice("tutorial_voice_010a");
					qs.set("Ex", "4");
				}
				return null;
			}
			else if(event.equalsIgnoreCase("TimerEx_GrandMaster"))
			{
				if(Ex >= 4)
				{
					st.showQuestionMark(7);
					st.playSound("ItemSound.quest_tutorial");
					st.playTutorialVoice("tutorial_voice_025");
				}
				return null;
			}
			else if(GrandMaster.containsKey(event))
			{
				Integer[] GM = GrandMaster.get(event);
				int itemId = GM[0];
				int radarX = GM[1];
				int radarY = GM[2];
				int radarZ = GM[3];
				if(st.getQuestItemsCount(itemId) > 0)
				{
					if(st.getPlayer().getClassId().isMage())
					{
						if(classId == 49)
						{
							st.giveItems(5789, 200);
							st.playTutorialVoice("tutorial_voice_026");
						}
						else
						{
							st.giveItems(5790, 100);
							st.playTutorialVoice("tutorial_voice_027");
						}
					}
					else
					{
						st.giveItems(5789, 200);
						st.playTutorialVoice("tutorial_voice_026");
						st.giveItems(BEGINER_TRAVEL_TOKEN, 12);
						st.addExpAndSp(0, 50);
						htmltext = event + ".htm";
						st.startQuestTimer("TimerEx_GrandMaster", 60000);
						st.addRadar(radarX, radarY, radarZ);
						if(Ex <= 3)
							qs.set("Ex", "4");
					}
					st.takeItems(itemId, 1);
				}
			}
			else if(Arrays.asList(NewbieGuide).contains(event))
			{
				if(Ex < 5)
				{
					qs.set("Ex", "5");
					if(st.getPlayer().getClassId().isMage())
					{
						if(st.getQuestItemsCount(5789) <= 400 && st.getQuestItemsCount(5790) <= 200)
							if(classId == 49)
							{
								st.playTutorialVoice("tutorial_voice_026");
								st.giveItems(5789, 200);

							}
							else
							{
								st.playTutorialVoice("tutorial_voice_027");
								st.giveItems(5790, 100);
							}
					}
					else if(st.getQuestItemsCount(5789) <= 400)
					{
						st.playTutorialVoice("tutorial_voice_026");
						st.giveItems(5789, 200);
					}
				}
				return null;
			}
		}
		return htmltext;
	}

	@Override
	public String onTalk(L2NpcInstance npc, QuestState st)
	{
		String htmltext = "<html><body>I have nothing to say you</body></html>";
		int npcId = npc.getNpcId();
		QuestState qs = st.getPlayer().getQuestState("_999_C4Tutorial");
		if(qs != null)
		{
			int Ex = qs.getInt("Ex");
			int classId = st.getPlayer().getClassId().getId();
			int raceId = st.getPlayer().getRace().ordinal();
			if(NewbieHelper.containsKey(npcId))
			{
				Integer[] nh = NewbieHelper.get(npcId);
				int occupation = nh[0];
				int race = nh[1];
				int itemId = nh[2];
				if(Ex < 0)
					if(raceId == race)
						if(occupation < 0)
						{
							st.startQuestTimer("TimerEx_NewbieHelper", 30000);
							qs.set("Ex", "0");
							st.onTutorialClientEvent(1048576);
							if(st.getPlayer().getClassId().isMage())
								htmltext = npcId + "-01m.htm";
							else
								htmltext = npcId + "-01f.htm";
						}
						else if(classId == occupation)
						{
							st.startQuestTimer("TimerEx_NewbieHelper", 30000);
							htmltext = npcId + "-01.htm";
							qs.set("Ex", "0");
							st.onTutorialClientEvent(1048576);
						}
						else
							htmltext = npcId + "-06.htm";
					else
						htmltext = npcId + "-06.htm";
				else if(Ex <= 2)
				{
					if(st.getQuestItemsCount(6353) > 0)
					{
						st.takeItems(6353, -1);
						qs.set("Ex", "3");
						st.giveItems(itemId, 1);
						st.startQuestTimer("TimerEx_NewbieHelper", 30000);
						qs.set("ucMemo", "3");
						if(st.getPlayer().getClassId().isMage())
						{
							if(st.getQuestItemsCount(5789) <= 0 && st.getQuestItemsCount(5790) <= 0)
								if(classId == 49)
								{
									st.playTutorialVoice("tutorial_voice_026");
									st.giveItems(5789, 200);
								}
								else
								{
									st.playTutorialVoice("tutorial_voice_027");
									st.giveItems(5790, 100);
								}
						}
						else if(st.getQuestItemsCount(5789) <= 0)
						{
							st.giveItems(5789, 200);
							st.playTutorialVoice("tutorial_voice_026");
						}
						if(occupation < 0)
						{
							if(st.getPlayer().getClassId().isMage())
								htmltext = npcId + "-03m.htm";
							else
								htmltext = npcId + "-03f.htm";
						}
						else
							htmltext = npcId + "-03.htm";
					}
					else if(occupation < 0)
						if(st.getPlayer().getClassId().isMage())
							htmltext = npcId + "-02m.htm";
						else
							htmltext = npcId + "-02f.htm";
					else
						htmltext = npcId + "-02.htm";
				}
				else if(Ex == 3)
					htmltext = npcId + "-04.htm";
				else if(Ex > 3)
					htmltext = npcId + "-05.htm";
			}
			else if(GrandMaster.containsKey(npcId))
			{
				Integer[] GM = GrandMaster.get(npcId);
				int itemId = GM[0];
				int radarX = GM[1];
				int radarY = GM[2];
				int radarZ = GM[3];
				if(st.getQuestItemsCount(itemId) > 0)
				{
					if(st.getPlayer().getClassId().isMage())
					{
						if(classId == 49)
						{
							st.giveItems(5789, 200);
							st.playTutorialVoice("tutorial_voice_026");
						}
						else
						{
							st.giveItems(5790, 100);
							st.playTutorialVoice("tutorial_voice_027");
						}
					}
					else
					{
						st.giveItems(5789, 200);
						st.playTutorialVoice("tutorial_voice_026");
						st.giveItems(BEGINER_TRAVEL_TOKEN, 12);
						st.addExpAndSp(0, 50);
						htmltext = npcId + ".htm";
						st.startQuestTimer("TimerEx_GrandMaster", 60000);
						st.addRadar(radarX, radarY, radarZ);
						if(Ex <= 3)
							qs.set("Ex", "4");
					}
					st.takeItems(itemId, 1);
				}
			}
		}
		return htmltext;
	}

	@Override
	public String onKill(L2NpcInstance npc, QuestState st)
	{
		QuestState qs = st.getPlayer().getQuestState("_999_C4Tutorial");
		if(qs != null)
		{
			int Ex = qs.getInt("Ex");
			if(Ex <= 1)
			{
				st.playTutorialVoice("tutorial_voice_011");
				st.showQuestionMark(3);
				qs.set("Ex", "2");
			}
			if(Ex <= 2 && st.getQuestItemsCount(6353) < 1)
				if(Rnd.chance(50))
				{
					L2Player player = st.getPlayer();
					st.dropItem((L2MonsterInstance) npc, player, 6353, 1);
					st.playSound("ItemSound.quest_tutorial");
				}
		}
		return null;
	}
}