package L2jFT;

import javolution.util.FastList;
import L2jFT.game.network.gameserverpackets.AuthRequest;
import L2jFT.game.network.gameserverpackets.AuthRequest.AdvIP;
import L2jFT.game.model.L2Player;
import static L2jFT.game.model.L2Zone.ZoneType.peace_zone;
import L2jFT.game.model.base.PlayerAccess;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

import javax.xml.parsers.DocumentBuilderFactory;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.OutputStream;
import java.lang.reflect.Field;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.logging.Logger;

/**
 * This class contain global server configuration.
 * It has static final fields initialized from configuration files.
 * It's initialized at the very begin of startup, and later JIT will optimize
 * away debug/unused code.
 */
public class Config
{
	protected static Logger _log = Logger.getLogger(Config.class.getName());
	
	// network	
	public static final String CONFIGURATION_FILE         = "./config/network/server.ini";
	public static final String LOGIN_CONFIGURATION_FILE   = "./config/network/loginserver.ini";
	
	// functions
	public static final String AI_CONFIG_FILE             = "./config/functions/ai.ini";	
	public static final String ALT_SETTINGS_FILE          = "./config/functions/altsettings.ini";	
	public static final String CLANHALL_CONFIG_FILE       = "./config/functions/clanhall.ini";
	public static final String OTHER_CONFIG_FILE          = "./config/functions/other.ini";
	public static final String PVP_CONFIG_FILE            = "./config/functions/pvp.ini";
	public static final String RATE_CONFIG_FILE           = "./config/functions/rate.ini";	
	public static final String SIEGE_CONFIGURATION_FILE   = "./config/functions/siege.ini";	
	public static final String SPOIL_CONFIG_FILE          = "./config/functions/spoil.ini";	
	public static final String WEB_CONFIG_FILE            = "./config/functions/web.ini";
	
	// fun
	public static final String ADV_IP_FILE                = "./config/fun/advipsystem.ini";
	public static final String EVENTS                     = "./config/fun/events.ini";	
	
	// Config
	public static final String GM_PERSONAL_ACCESS_FILE    = "./config/GMAccess.xml";
	public static final String TELNET_FILE                = "./config/telnet.ini";
	public static final String LOGIN_TELNET_FILE          = "./config/login_telnet.ini";
	public static final String VERSION_FILE               = "L2jFT-version.ini";
	public static final String BANNED_IP_XML              = "./config/banned.xml";
	public static final String HEXID_FILE                 = "./config/hexid.txt";
	public static final String MAT_CONFIG_FILE            = "./config/mats.ini";




	/** Debug/release mode */
	public static boolean DEBUG;
	public static boolean LOGIN_DEBUG;

	public static boolean COMBO_MODE;
	public static boolean TEST_SERVER;

	/** Game/Login Server ports */
	public static int PORT_GAME;
	public static int PORT_LOGIN;
	public static int LOGIN_TRY_BEFORE_BAN;
	public static String GAMESERVER_HOSTNAME;
	public static boolean ADVIPSYSTEM;
	public static ArrayList<AdvIP> GAMEIPS = new ArrayList<AdvIP>();
	public static int IP_UPDATE_TIME;
	public static boolean FLOOD_PROTECTION;
	public static int FAST_CONNECTION_LIMIT;
	public static int NORMAL_CONNECTION_TIME;
	public static int FAST_CONNECTION_TIME;
	public static int MAX_CONNECTION_PER_IP;
	public static boolean LOGIN_PING;
	public static int LOGIN_PING_TIME;

	public static int WebServerPort;
	public static String WebServerRoot;

	public static String DATABASE_DRIVER;
	public static String DATABASE_URL;
	public static String DATABASE_LOGIN;
	public static String DATABASE_PASSWORD;
	public static int DATABASE_MAX_CONNECTIONS;
	public static int DATABASE_MAX_IDLE_TIMEOUT;
	public static int DATABASE_IDLE_TEST_PERIOD;

	// Database additional options
	/** lazy item update */
	public static boolean LAZY_ITEM_UPDATE;
	/** lazy item update time */
	public static int LAZY_ITEM_UPDATE_TIME;

	public static int MAXIMUM_ONLINE_USERS;
	public static boolean AUTO_CREATE_ACCOUNTS;
	public static boolean ALLOW_OLD_AUTH;

	public static boolean SERVER_LIST_BRACKET;
	public static boolean SERVER_LIST_CLOCK;
	public static boolean SERVER_LIST_TESTSERVER;
	public static boolean SERVER_GMONLY;
	public static boolean PING;
	public static int PING_TIME;

	public static boolean CHECK_LANG_FILES_MODIFY;
	public static boolean USE_FILE_CACHE;

	public static int LINEAR_CELL_SIZE;
	public static int LINEAR_TERRITORY_CELL_SIZE;
	public static boolean DONTLOADSPAWN;
	public static boolean DONTLOADQUEST;

	/** ChatBan */
	public static Boolean MAT_BANCHAT;
	public static Boolean MAT_BANCHATG;
	public static Boolean MAT_REPLACE;
	public static String MAT_REPLACE_STRING;
	public static int UNCHATBANTIME;
	public static ArrayList<String> MAT_LIST = new ArrayList<String>();

	public static Boolean SAVING_SPS;
	public static Boolean MANAHEAL_SPS_BONUS;

	public static int ALT_ADD_RECIPES;

	public static int PARTY_DIFF_LEVEL_PENALTY;

	/** Count kills? */
	public static Boolean KILL_COUNTER;
	public static Boolean KILL_COUNTER_PRELOAD;

	/** Count dropped? */
	public static Boolean DROP_COUNTER;

	/** Count crafted? */
	public static Boolean CRAFT_COUNTER;

	/** Thread pools size */
	public static int THREAD_P_EFFECTS;
	public static int THREAD_P_GENERAL;
	public static int GENERAL_PACKET_THREAD_CORE_SIZE;
	public static int URGENT_PACKET_THREAD_CORE_SIZE;
	public static int AI_MAX_THREAD;

	/** Auto-loot? */
	public static Boolean AUTO_LOOT;

	/** Auto-loot for/from players with karma also? */
	public static Boolean AUTO_LOOT_PK;

	/** Auto-loot herbs? */
	public static Boolean AUTO_LOOT_HERBS;

	/** Account name template */
	public static String ANAME_TEMPLATE;

	/** Account password template */
	public static String APASSWD_TEMPLATE;

	/** Character name template */
	public static String CNAME_TEMPLATE;

	/** Clan name template */
	public static String CLAN_NAME_TEMPLATE;

	/** Clan title template */
	public static String CLAN_TITLE_TEMPLATE;

	/** Ally name template */
	public static String ALLY_NAME_TEMPLATE;

	/** Global chat state */
	public static int GLOBAL_CHAT;
	public static int GLOBAL_TRADE_CHAT;

	/** For test servers - evrybody has admin rights */
	public static boolean EVERYBODY_HAS_ADMIN_RIGHTS;
	public static boolean ALLOW_SPECIAL_COMMANDS;

	/** For small servers */
	public static boolean ALT_GAME_MATHERIALSDROP;

	/** Все мобы не являющиеся рейдами спавнятся в двойном количестве */
	public static boolean ALT_DOUBLE_SPAWN;

	public static boolean ALT_ALLOW_AUGMENT_ALL;

	/** Give exp and sp for craft */
	public static boolean ALT_GAME_EXP_FOR_CRAFT;

	public static boolean ALT_GAME_UNREGISTER_RECIPE;

	/** Delay for announce SS period (in minutes) */
	public static int SS_ANNOUNCE_PERIOD;

	/** Show mob stats/droplist to players? */
	public static boolean ALT_GAME_SHOW_DROPLIST;
	public static boolean ALT_FULL_NPC_STATS_PAGE;

	public static boolean ALT_SKILLS_CHANCES;
	public static double SKILLS_CHANCE_CAP;
	public static boolean SKILLS_SHOW_CHANCE;
	public static boolean ALT_SKILLS_UNSAVABLE_SAVE;
	public static boolean ALT_SHOW_REUSE_MSG;

	/** Система наподобии той, что в WoW */
	public static boolean ALT_HONOR_SYSTEM;

	/** Таймаут на использование social action */
	public static boolean ALT_SOCIAL_ACTION_REUSE;

	/** Fast or slow multiply coefficient for skill hit time */
	public static float ALT_GAME_SKILL_HIT_RATE;

	/** Alternative gameing - loss of XP on death */
	public static boolean ALT_GAME_DELEVEL;

	public static boolean ALT_GAME_SUBCLASS_WITHOUT_QUESTS;
	public static int ALT_GAME_LEVEL_TO_GET_SUBCLASS;

	public static boolean ALT_SIMPLE_SIGNS;
	public static int ALT_MAMMOTH_EXCHANGE;
	public static int ALT_MAMMOTH_UPGRADE;
	public static boolean ALT_ALLOW_TATTOO;

	public static boolean ALT_ALLOW_OTHERS_WITHDRAW_FROM_CLAN_WAREHOUSE;
	public static boolean ALT_GAME_REQUIRE_CLAN_CASTLE;
	public static boolean ALT_GAME_REQUIRE_CASTLE_DAWN;

	/** Olympiad Compitition Starting time */
	public static int ALT_OLY_START_TIME;
	/** Olympiad Compition Min */
	public static int ALT_OLY_MIN;
	/** Olympaid Comptetition Period */
	public static long ALT_OLY_CPERIOD;
	/** Olympiad Battle Period */
	public static long ALT_OLY_BATTLE;
	/** Olympiad Battle Wait */
	public static long ALT_OLY_BWAIT;
	/** Olympiad Inital Wait */
	public static long ALT_OLY_IWAIT;
	/** Olympaid Weekly Period */
	public static long ALT_OLY_WPERIOD;
	/** Olympaid Validation Period */
	public static long ALT_OLY_VPERIOD;

	public static boolean ENABLE_OLYMPIAD;

	public static int CORPSE_MOB_TIME;

	public static float ALT_TRUE_CHESTS;

	public static int NONOWNER_ITEM_PICKUP_DELAY;

	/** Enable Rate Hp  */
	public static boolean ENABLE_RATE_HP;

	/** Logging Chat Window */
	public static boolean LOG_CHAT;
	public static boolean LOG_ITEMS;
	public static boolean LOG_KILLS;
	public static Boolean LOG_TELNET;
	public static boolean SQL_LOG;

	public static boolean ALT_PRIVILEGES_SECURE_CHECK;
	public static int ALT_PRIVILEGES_DEFAULT_LEVEL;

	public static HashMap<Integer, PlayerAccess> gmlist = new HashMap<Integer, PlayerAccess>();

	/** Castle Guards stuff */
	public static Integer CG_HIREPRICE;
	public static Integer CG_HIRE;
	public static Integer CG_KILL;
	public static Integer CG_MAXHIREDGUARDS;
	public static Integer CG_VIEWSTATS;
	/**
	 * this is not a clasical probability number but the higher the value is,
	 * the more the probablity of stats up is
	 */
	public static Integer CG_STATUPPROB;
	public static Integer CG_STATUPGAIN;
	public static Integer CG_MOVEMENT;
	public static Integer CG_SET;
	public static Integer CG_CASTLE;
	public static Boolean CG_SPAWNSIEGEGUARDS;

	/** Rate control */
	public static float RATE_XP;
	public static float RATE_SP;
	public static float RATE_PARTY_XP;
	public static float RATE_PARTY_SP;
	public static float RATE_QUESTS_REWARD;
	public static float RATE_CLAN_REP_SCORE;
	public static Integer RATE_CLAN_REP_SCORE_MAX_AFFECTED;
	public static float RATE_DROP_ADENA;
	public static float RATE_CONSUMABLE_COST;
	public static float RATE_DROP_ITEMS;
	public static float RATE_DROP_SPOIL;
	public static Integer RATE_MANOR;
	public static float RATE_FISH_DROP_COUNT;
	public static float RATE_SIEGE_GUARDS_PRICE;

	/** Player Drop Rate control */
	public static Boolean KARMA_DROP_EQUIPPED_ITEMS;
	public static Boolean KARMA_DROP_EQUIPPED_WEAPONS;
	public static Boolean KARMA_DROP_GM;

	public static Integer KARMA_DROP_ITEM_LIMIT;

	public static Integer KARMA_RANDOM_DROP_LOCATION_LIMIT;

	public static Integer KARMA_DROPCHANCE_MINIMUM;
	public static Integer KARMA_DROPCHANCE_MULTIPLIER;
	public static Integer KARMA_DROPCHANCE_EQUIPMENT;
	public static Integer KARMA_DROPCHANCE_EQUIPPED_WEAPON;

	public static int AUTODESTROY_ITEM_AFTER;

	public static int COORD_SYNCHRONIZE;

	public static int DELETE_DAYS;

	/** Datapack root directory */
	public static File DATAPACK_ROOT;

	public static String AttributeBonusFile;

	public static Boolean WEAR_TEST_ENABLED;
	public static boolean USE_DATABASE_LAYER;

	/** Allow L2Walker client */
	public static enum L2WalkerAllowed
	{
		True,
		False,
		Peace
	}

	public static L2WalkerAllowed ALLOW_L2WALKER_CLIENT;
	public static int MAX_L2WALKER_LEVEL;
	public static boolean ALLOW_CRAFT_BOT;
	public static boolean ALLOW_BOT_CAST;
	public static boolean KILL_BOTS_WITH_SKILLCOOLTIME;
	public static int L2WALKER_PUNISHMENT;
	public static int BUGUSER_PUNISH;

	public static boolean ALLOW_DISCARDITEM;
	public static boolean ALLOW_FREIGHT;
	public static boolean ALLOW_WAREHOUSE;
	public static boolean ALLOW_LOTTERY;
	public static boolean ALLOW_RACE;
	public static boolean ALLOW_WATER;
	public static boolean ALLOW_BOAT;
	public static boolean ALLOW_CURSED_WEAPONS;

	public static int PACKET_LIFETIME;

	/** Pets */
	public static int WYVERN_SPEED;
	public static int STRIDER_SPEED;
	public static int SWIMING_SPEED;
	public static boolean ALLOW_WYVERN_UPGRADER;

	/** protocol revision */
	public static int MIN_PROTOCOL_REVISION;
	public static int MAX_PROTOCOL_REVISION;

	/** random animation interval */
	public static int MIN_NPC_ANIMATION;
	public static int MAX_NPC_ANIMATION;

	public static boolean ACTIVATE_POSITION_RECORDER;
	public static boolean USE_3D_MAP;

	public static Boolean ENABLE_TRANSLIT_ENGINE;
	public static Boolean ENABLE_TEXT_FILTER;

	public static Integer RESTART_TIME;
	public static Integer LRESTART_TIME;

	public static boolean CHECK_KNOWN;

	public static String LOGIN_HOST;
	public static int GAME_SERVER_LOGIN_PORT;
	public static String GAME_SERVER_LOGIN_HOST;
	public static String INTERNAL_HOSTNAME;
	public static String EXTERNAL_HOSTNAME;

	public static boolean SERVER_SIDE_NPC_NAME;
	public static boolean SERVER_SIDE_NPC_TITLE;
	public static boolean SERVER_SIDE_NPC_TITLE_WITH_LVL;
	public static boolean SERVER_SIDE_NPC_TITLE_WITH_SOCIAL;
	public static boolean ALLOW_GUARDS;
	public static String ALLOW_CLASS_MASTERS;
	public static String CLASS_MASTERS_PRICE;
	public static ArrayList<Integer> ALLOW_CLASS_MASTERS_LIST = new ArrayList<Integer>();
	public static int[] CLASS_MASTERS_PRICE_LIST = new int[4];

	public static String SERVER_VERSION;
	public static String SERVER_BUILD_DATE;

	/** Crafting Enabled? */
	public static boolean IS_CRAFTING_ENABLED;

	/** Inventory slots limits */
	public static int INVENTORY_MAXIMUM_NO_DWARF;
	public static int INVENTORY_MAXIMUM_DWARF;
	public static int INVENTORY_MAXIMUM_GM;

	/** Warehouse slots limits */
	public static int WAREHOUSE_SLOTS_NO_DWARF;
	public static int WAREHOUSE_SLOTS_DWARF;
	public static int WAREHOUSE_SLOTS_CLAN;

	/** Spoil Rates */
	public static float BASE_SPOIL_RATE;
	public static float MINIMUM_SPOIL_RATE;
	public static boolean ALT_SPOIL_FORMULA;

	/** Manor Config */
	public static int MANOR_SOWING_BASIC_SUCCESS;
	public static int MANOR_SOWING_ALT_BASIC_SUCCESS;
	public static int MANOR_HARVESTING_BASIC_SUCCESS;
	public static int MANOR_DIFF_PLAYER_TARGET;
	public static int MANOR_DIFF_PLAYER_TARGET_PENALTY;
	public static int MANOR_DIFF_SEED_TARGET;
	public static int MANOR_DIFF_SEED_TARGET_PENALTY;

	/** Karma System Variables */
	public static int KARMA_MIN_KARMA;
	public static int KARMA_SP_DIVIDER;
	public static int KARMA_LOST_BASE;
	public static boolean KARMA_AWARD_PK_KILL;

	public static String KARMA_NONDROPPABLE_PET_ITEMS;
	public static String KARMA_NONDROPPABLE_ITEMS;
	public static ArrayList<Integer> KARMA_LIST_NONDROPPABLE_PET_ITEMS = new ArrayList<Integer>();
	public static ArrayList<Integer> KARMA_LIST_NONDROPPABLE_ITEMS = new ArrayList<Integer>();

	public static String NONDROPPABLE_ITEMS;
	public static ArrayList<Integer> LIST_NONDROPPABLE_ITEMS = new ArrayList<Integer>();

	public static String PET_RENT_NPC;
	public static ArrayList<Integer> LIST_PET_RENT_NPC = new ArrayList<Integer>();

	public static int PVP_TIME;

	/** Karma Punishment */
	public static Boolean ALT_GAME_KARMAPLAYERCANBEKILLEDINPEACEZONE;
	public static boolean ALT_GAME_KARMA_PLAYER_CAN_SHOP;

	/** Day/Night Status **/
	public static boolean DAY_STATUS_FORCE_CLIENT_UPDATE;
	public static int DAY_STATUS_SUN_RISE_AT;
	public static int DAY_STATUS_SUN_SET_AT;

	/**
	 * Counting of amount of packets per minute
	 */
	public static boolean COUNT_PACKETS = false;
	public static boolean DUMP_PACKET_COUNTS = false;
	public static int DUMP_INTERVAL_SECONDS = 60;

	public static int EXECUTOR_PRI_LOW;
	public static int EXECUTOR_PRI_MED;
	public static int EXECUTOR_PRI_HIGH;

	public static enum IdFactoryType
	{
		Compaction,
		BitSet,
		Stack
	}

	public static boolean BAD_ID_CHECKING;

	public static Integer ADMIN_TEST;

	public static enum ObjectMapType
	{
		L2ObjectHashMap,
		WorldObjectMap
	}

	public static enum ObjectSetType
	{
		L2ObjectHashSet,
		WorldObjectSet
	}

	public static ObjectMapType MAP_TYPE;
	public static ObjectSetType SET_TYPE;

	public static boolean EFFECT_CANCELING;

	public static boolean AUTODELETE_INVALID_QUEST_DATA;
	public static boolean HARD_DB_CLEANUP_ON_START;

	/** Chance that an item will succesfully be enchanted */
	public static int ENCHANT_CHANCE_WEAPON;
	public static int ENCHANT_CHANCE_ARMOR;
	public static byte ENCHANT_MAX;

	public static double HP_REGEN_MULTIPLIER;
	public static double MP_REGEN_MULTIPLIER;
	public static double CP_REGEN_MULTIPLIER;
	public static boolean REGEN_SIT_WAIT;

	/** Apply rates on RaidBosses */
	public static boolean APPLY_RATES_ON_RBS;

	public static float RATE_RAID_REGEN;
	public static int RAID_MAX_LEVEL_DIFF;

	public static float ALT_PK_DEATH_RATE;
	public static int STARTING_ADENA;

	/** Deep Blue Mobs' Drop Rules Enabled */
	public static boolean DEEPBLUE_DROP_RULES;
	public static boolean UNSTUCK_SKILL;

	/** telnet enabled */
	public static boolean IS_TELNET_ENABLED;
	public static boolean IS_LOGIN_TELNET_ENABLED;

	/** Player Protection control */
	public static int PLAYER_SPAWN_PROTECTION;

	/** Percent CP is restore on respawn */
	public static double RESPAWN_RESTORE_CP;
	/** Percent HP is restore on respawn */
	public static double RESPAWN_RESTORE_HP;
	/** Percent MP is restore on respawn */
	public static double RESPAWN_RESTORE_MP;

	/** Maximum number of available slots for pvt stores (sell/buy) - Dwarves */
	public static int MAX_PVTSTORE_SLOTS_DWARF;
	/** Maximum number of available slots for pvt stores (sell/buy) - Others */
	public static int MAX_PVTSTORE_SLOTS_OTHER;

	public static boolean ALLOW_CH_DOOR_OPEN_ON_CLICK;

	/** Clan Hall function related configs*/
	public static long CH_TELE_FEE_RATIO;
	public static int CH_TELE1_FEE;
	public static int CH_TELE2_FEE;
	public static int CH_TELE3_FEE;
	public static long CH_ITEM_FEE_RATIO;
	public static int CH_ITEM1_FEE;
	public static int CH_ITEM2_FEE;
	public static int CH_ITEM3_FEE;
	public static long CH_MPREG_FEE_RATIO;
	public static int CH_MPREG1_FEE;
	public static int CH_MPREG2_FEE;
	public static int CH_MPREG3_FEE;
	public static int CH_MPREG4_FEE;
	public static int CH_MPREG5_FEE;
	public static int CH_MPREG6_FEE;
	public static long CH_HPREG_FEE_RATIO;
	public static int CH_HPREG1_FEE;
	public static int CH_HPREG2_FEE;
	public static int CH_HPREG3_FEE;
	public static int CH_HPREG4_FEE;
	public static int CH_HPREG5_FEE;
	public static int CH_HPREG6_FEE;
	public static int CH_HPREG7_FEE;
	public static int CH_HPREG8_FEE;
	public static int CH_HPREG9_FEE;
	public static int CH_HPREG10_FEE;
	public static int CH_HPREG11_FEE;
	public static int CH_HPREG12_FEE;
	public static long CH_EXPREG_FEE_RATIO;
	public static int CH_EXPREG1_FEE;
	public static int CH_EXPREG2_FEE;
	public static int CH_EXPREG3_FEE;
	public static int CH_EXPREG4_FEE;
	public static int CH_EXPREG5_FEE;
	public static int CH_EXPREG6_FEE;
	public static int CH_EXPREG7_FEE;
	public static int CH_EXPREG8_FEE;
	public static long CH_SUPPORT_FEE_RATIO;
	public static int CH_SUPPORT1_FEE;
	public static int CH_SUPPORT2_FEE;
	public static int CH_SUPPORT3_FEE;
	public static int CH_SUPPORT4_FEE;
	public static int CH_SUPPORT5_FEE;
	public static int CH_SUPPORT7_FEE;
	public static int CH_SUPPORT8_FEE;
	public static long CH_CURTAIN_FEE_RATIO;
	public static int CH_CURTAIN1_FEE;
	public static int CH_CURTAIN2_FEE;
	public static long CH_PLATFORM_FEE_RATIO;
	public static int CH_PLATFORM1_FEE;
	public static int CH_PLATFORM2_FEE;

	/** Show licence or not just after login (if false, will directly go to the Server List */
	public static boolean SHOW_LICENCE;

	/** Deafult punishment for illegal actions */
	public static int DEFAULT_PUNISH;

	public static boolean ACCEPT_NEW_GAMESERVER;
	public static byte[] HEX_ID;
	public static boolean ACCEPT_ALTERNATE_ID;
	public static int REQUEST_ID;
	public static boolean RESERVE_HOST_ON_LOGIN = false;

	public static int MINIMUM_UPDATE_DISTANCE;
	public static int KNOWNLIST_FORGET_DELAY;
	public static int MINIMUN_UPDATE_TIME;

	public static boolean ANNOUNCE_MAMMON_SPAWN;
	public static boolean LAZY_CACHE;

	public static int GM_NAME_COLOUR;
	public static boolean GM_HERO_AURA;
	public static int NORMAL_NAME_COLOUR;
	public static int CLANLEADER_NAME_COLOUR;
	public static int BOT_NAME_COLOUR;
	public static String BOT_NAME_HEX_COLOUR;

	public static boolean OVER_LEVEL_PENALTY;
	public static int DIFF_LEVEL_PENALTY;

	/** AI */
	public static boolean RND_WALK;
	public static int RND_WALK_RATE;
	public static int RND_ANIMATION_RATE;

	/** Maximum range mobs can randomly go from spawn point */
	public static int MAX_DRIFT_RANGE;

	/** Maximum range mobs can pursue agressor from spawn point */
	public static int MAX_PURSUE_RANGE;

	public static int FF_STUN_RATE;
	public static int FR_STUN_RATE;
	public static int FM_STUN_RATE;
	public static int FP_STUN_RATE;
	public static int FF_DEBUFF_RATE;
	public static int FR_DEBUFF_RATE;
	public static int FM_DEBUFF_RATE;
	public static int FP_DEBUFF_RATE;
	public static int FF_DOT_RATE;
	public static int FR_DOT_RATE;
	public static int FM_DOT_RATE;
	public static int FP_DOT_RATE;
	public static int FF_MDAM_RATE;
	public static int FR_MDAM_RATE;
	public static int FM_MDAM_RATE;
	public static int FP_MDAM_RATE;
	public static int FF_PDAM_RATE;
	public static int FR_PDAM_RATE;
	public static int FM_PDAM_RATE;
	public static int FP_PDAM_RATE;
	public static int FF_PHYS_RATE;
	public static int FR_PHYS_RATE;
	public static int FM_PHYS_RATE;
	public static int FP_PHYS_RATE;

	public static int RF_STUN_RATE;
	public static int RR_STUN_RATE;
	public static int RM_STUN_RATE;
	public static int RP_STUN_RATE;
	public static int RF_DEBUFF_RATE;
	public static int RR_DEBUFF_RATE;
	public static int RM_DEBUFF_RATE;
	public static int RP_DEBUFF_RATE;
	public static int RF_DOT_RATE;
	public static int RR_DOT_RATE;
	public static int RM_DOT_RATE;
	public static int RP_DOT_RATE;
	public static int RF_MDAM_RATE;
	public static int RR_MDAM_RATE;
	public static int RM_MDAM_RATE;
	public static int RP_MDAM_RATE;
	public static int RF_PDAM_RATE;
	public static int RR_PDAM_RATE;
	public static int RM_PDAM_RATE;
	public static int RP_PDAM_RATE;
	public static int RF_PHYS_RATE;
	public static int RR_PHYS_RATE;
	public static int RM_PHYS_RATE;
	public static int RP_PHYS_RATE;

	public static int MF_STUN_RATE;
	public static int MR_STUN_RATE;
	public static int MM_STUN_RATE;
	public static int MP_STUN_RATE;
	public static int MF_DEBUFF_RATE;
	public static int MR_DEBUFF_RATE;
	public static int MM_DEBUFF_RATE;
	public static int MP_DEBUFF_RATE;
	public static int MF_DOT_RATE;
	public static int MR_DOT_RATE;
	public static int MM_DOT_RATE;
	public static int MP_DOT_RATE;
	public static int MF_MDAM_RATE;
	public static int MR_MDAM_RATE;
	public static int MM_MDAM_RATE;
	public static int MP_MDAM_RATE;
	public static int MF_PDAM_RATE;
	public static int MR_PDAM_RATE;
	public static int MM_PDAM_RATE;
	public static int MP_PDAM_RATE;
	public static int MF_PHYS_RATE;
	public static int MR_PHYS_RATE;
	public static int MM_PHYS_RATE;
	public static int MP_PHYS_RATE;

	public static int PF_STUN_RATE;
	public static int PR_STUN_RATE;
	public static int PM_STUN_RATE;
	public static int PP_STUN_RATE;
	public static int PF_DEBUFF_RATE;
	public static int PR_DEBUFF_RATE;
	public static int PM_DEBUFF_RATE;
	public static int PP_DEBUFF_RATE;
	public static int PF_DOT_RATE;
	public static int PR_DOT_RATE;
	public static int PM_DOT_RATE;
	public static int PP_DOT_RATE;
	public static int PF_MDAM_RATE;
	public static int PR_MDAM_RATE;
	public static int PM_MDAM_RATE;
	public static int PP_MDAM_RATE;
	public static int PF_PDAM_RATE;
	public static int PR_PDAM_RATE;
	public static int PM_PDAM_RATE;
	public static int PP_PDAM_RATE;
	public static int PF_PHYS_RATE;
	public static int PR_PHYS_RATE;
	public static int PM_PHYS_RATE;
	public static int PP_PHYS_RATE;

	public static int ONE_FACTION_CALL_CHANCE;

	/** It's EVIL :) */
	public static boolean MOBSLOOTERS;

	/** Bow-reuse modifier constant */
	public static float BOW_REUSE_MODIFIER;

	public static boolean RRD_ENABLED;
	public static boolean RRD_EXTENDED;
	public static boolean RRD_GRAPH_AA;
	public static String RRD_PATH;
	public static String RRD_EXT_PATH;
	public static String RRD_GRAPH_PATH;
	public static String RRD_GRAPH_FORMAT;
	public static long RRD_REDRAW_TIME;
	public static long RRD_UPDATE_TIME;
	public static int RRD_GRAPH_HEIGHT;
	public static int RRD_GRAPH_WIDTH;

	public static float BASE_EXP_LOST_ON_DEATH;
	public static boolean ALT_DEATH_PENALTY;
	public static boolean ALLOW_DEATH_PENALTY_C5;
	public static int ALT_DEATH_PENALTY_C5_CHANCE;
	public static boolean ALT_DEATH_PENALTY_C5_CHAOTIC_RECOVERY;
	public static int ALT_DEATH_PENALTY_C5_EXPERIENCE_PENALTY;
	public static int ALT_DEATH_PENALTY_C5_KARMA_PENALTY;

	public static boolean BYPASS_VALIDATION;

	public static boolean HIDE_GM_STATUS;
	public static boolean SHOW_GM_LOGIN;

	/** security options */
	public static String DISABLE_CREATION_IDs;
	public static ArrayList<Integer> DISABLE_CREATION_ID_LIST = new ArrayList<Integer>();
	public static String LOG_MULTISELL_IDs;
	public static ArrayList<Integer> LOG_MULTISELL_ID_LIST = new ArrayList<Integer>();

	public static boolean ENABLE_FISHING_CHECK_WATER;
	
	public static boolean AUTO_LEARN_SKILLS;

	/** Geodata */
	public static boolean GEODATA;
	/** Pattern used to autocreate geo_index.txt if not present */
	public static String GEOFILES_PATTERN;
	/** Debug, only for GM */
	public static boolean GEODATA_DEBUG;

	public static boolean PATH_CLEAN;
	public static boolean PATHFINDING_BOOST;

	public static Boolean DAMAGE_FROM_FALLING;

	public static String GEOEDITOR_HOST;

	/** Community Board */
	public static boolean ALLOW_COMMUNITYBOARD;
	public static String COMMUNITY_TYPE;
	public static String BBS_DEFAULT;
	public static boolean SHOW_LEVEL_COMMUNITYBOARD;
	public static boolean SHOW_STATUS_COMMUNITYBOARD;
	public static int NAME_PAGE_SIZE_COMMUNITYBOARD;
	public static int NAME_PER_ROW_COMMUNITYBOARD;

	/** Wedding Options */
	public static boolean ALLOW_WEDDING;
	public static int WEDDING_PRICE;
	public static Boolean WEDDING_PUNISH_INFIDELITY;
	public static Boolean WEDDING_TELEPORT;
	public static int WEDDING_TELEPORT_PRICE;
	public static int WEDDING_TELEPORT_INTERVAL;
	public static Boolean WEDDING_SAMESEX;
	public static boolean WEDDING_FORMALWEAR;
	public static int WEDDING_DIVORCE_COSTS;

	public static boolean FORCE_STATUSUPDATE;

	/** Castle siege options **/
	public static boolean SIEGE_OPERATE_DOORS;
	public static boolean SIEGE_OPERATE_DOORS_LORD_ONLY;

	/**limits of stats **/
	public static int LIM_MATK_SPD;
	public static int LIM_PATK_SPD;
	public static int LIM_CRIT;

	/** Enchant Config **/
	public static int SAFE_ENCHANT_COMMON;
	public static int SAFE_ENCHANT_FULL_BODY;

	/** Dimensional Rift Config **/
	public static int RIFT_MIN_PARTY_SIZE;
	public static int RIFT_MAX_JUMPS;
	public static int RIFT_AUTO_JUMPS_TIME;
	public static int RIFT_AUTO_JUMPS_TIME_RAND;
	public static int RIFT_ENTER_COST_RECRUIT;
	public static int RIFT_ENTER_COST_SOLDIER;
	public static int RIFT_ENTER_COST_OFFICER;
	public static int RIFT_ENTER_COST_CAPTAIN;
	public static int RIFT_ENTER_COST_COMMANDER;
	public static int RIFT_ENTER_COST_HERO;
	public static float RIFT_BOSS_ROOM_TIME_MUTIPLY;

	/** Some more LS Settings */
	public static FastList<String> INTERNAL_IP = new FastList<String>();
	public static String internalIpList;
	public static boolean SESSIONKEY_IGNORE;

	public static boolean ALLOW_TALK_WHILE_SITTING;

	/** Party options **/
	public static int PARTY_EXP_CALCULATIONS;
	public static boolean PARTY_LEADER_ONLY_CAN_INVITE;

	/** Разрешены ли клановые скилы? **/
	public static boolean ALLOW_CLANSKILLS;

	/** Player visibility **/
	public static int PLAYER_VISIBILITY;
	public static int PLAYER_VISIBILITY_Z;

	public static boolean LOGIN_GG_CHECK;
	public static boolean GG_CHECK;
	public static boolean CHECK_SESSIONKEY;

	public static boolean TUTORIAL_ON_OFF;

	/** Allow Manor system */
	public static boolean ALLOW_MANOR;

	/** Manor Refresh Starting time */
	public static int MANOR_REFRESH_TIME;

	/** Manor Refresh Min */
	public static int MANOR_REFRESH_MIN;

	/** Manor Next Period Approve Starting time */
	public static int MANOR_APPROVE_TIME;

	/** Manor Next Period Approve Min */
	public static int MANOR_APPROVE_MIN;

	/** Manor Maintenance Time */
	public static int MANOR_MAINTENANCE_PERIOD;

	/** Manor Save All Actions */
	public static boolean MANOR_SAVE_ALL_ACTIONS;

	/** Упрощенная версия манора, без управления */
	public static boolean ALT_SIMPLE_MANOR;

	/** Manor Save Period Rate */
	public static int MANOR_SAVE_PERIOD_RATE;

	public static int SHIFT_BY;
	public static int SHIFT_BY_FOR_Z;

	/** NPC Buffer config */
	public static int EVENT_Min_lvl;
	public static int EVENT_Max_lvl;
	public static int EVENT_dp;
	public static int EVENT_dpay;
	public static int EVENT_type_buffer;
	public static boolean EVENT_Buffer_Siege;

	/** Rename event config */
	public static int EVENT_RenameItemID;
	public static int EVENT_RenamePrice;

	public static int EVENT_CofferOfShadowsPriceRate;

	public static void load()
	{
		if(Server.SERVER_MODE == Server.MODE_GAMESERVER || Server.SERVER_MODE == Server.MODE_COMBOSERVER)
		{
			_log.info("Loading gameserver config.");
			try
			{
				Properties serverSettings = new Properties();
				InputStream is = new FileInputStream(new File(CONFIGURATION_FILE));
				serverSettings.load(is);
				is.close();

				GAME_SERVER_LOGIN_HOST = serverSettings.getProperty("LoginHost", "127.0.0.1");
				GAME_SERVER_LOGIN_PORT = Integer.parseInt(serverSettings.getProperty("LoginPort", "9013"));
				DEBUG = Boolean.parseBoolean(serverSettings.getProperty("Debug", "false"));
				TEST_SERVER = Boolean.parseBoolean(serverSettings.getProperty("TestServer", "false"));
				SERVER_LIST_TESTSERVER = Boolean.parseBoolean(serverSettings.getProperty("TestServer", "false"));
				ADVIPSYSTEM = Boolean.parseBoolean(serverSettings.getProperty("AdvIPSystem", "false"));
				HIDE_GM_STATUS = Boolean.parseBoolean(serverSettings.getProperty("HideGMStatus", "false"));
				SHOW_GM_LOGIN = Boolean.parseBoolean(serverSettings.getProperty("ShowGMLogin", "true"));
				PING = Boolean.parseBoolean(serverSettings.getProperty("PingServer", "True"));
				PING_TIME = Integer.parseInt(serverSettings.getProperty("WaitPingTime", "5"));

				REQUEST_ID = Integer.parseInt(serverSettings.getProperty("RequestServerID", "0"));
				ACCEPT_ALTERNATE_ID = Boolean.parseBoolean(serverSettings.getProperty("AcceptAlternateID", "True"));

				PORT_GAME = Integer.parseInt(serverSettings.getProperty("GameserverPort", "7777"));
				PORT_LOGIN = Integer.parseInt(serverSettings.getProperty("LoginserverPort", "2106"));
				CNAME_TEMPLATE = serverSettings.getProperty("CnameTemplate", "[A-Za-z0-9]{3,16}");
				CLAN_NAME_TEMPLATE = serverSettings.getProperty("ClanNameTemplate", "[A-Za-z0-9]{3,16}");
				CLAN_TITLE_TEMPLATE = serverSettings.getProperty("ClanTitleTemplate", "[A-Za-z0-9 \\p{Punct}]{1,16}");
				ALLY_NAME_TEMPLATE = serverSettings.getProperty("AllyNameTemplate", "[A-Za-z0-9]{3,16}");
				ANAME_TEMPLATE = serverSettings.getProperty("AnameTemplate", "[A-Za-z0-9]{3,14}");
				APASSWD_TEMPLATE = serverSettings.getProperty("ApasswdTemplate", "[A-Za-z0-9]{5,16}");
				LOGIN_TRY_BEFORE_BAN = Integer.parseInt(serverSettings.getProperty("LoginTryBeforeBan", "10"));
				GAMESERVER_HOSTNAME = serverSettings.getProperty("GameserverHostname");

				GLOBAL_CHAT = Integer.parseInt(serverSettings.getProperty("GlobalChat", "0"));
				GLOBAL_TRADE_CHAT = Integer.parseInt(serverSettings.getProperty("GlobalTradeChat", "0"));
				EVERYBODY_HAS_ADMIN_RIGHTS = Boolean.parseBoolean(serverSettings.getProperty("EverybodyHasAdminRights", "false"));
				ALLOW_SPECIAL_COMMANDS = Boolean.parseBoolean(serverSettings.getProperty("AllowSpecialCommands", "false"));
				LOG_CHAT = Boolean.valueOf(serverSettings.getProperty("LogChat", "false"));
				LOG_ITEMS = Boolean.valueOf(serverSettings.getProperty("LogItems", "false"));
				LOG_KILLS = Boolean.valueOf(serverSettings.getProperty("LogKills", "false"));
				LOG_TELNET = Boolean.valueOf(serverSettings.getProperty("LogTelnet", "false"));
				SQL_LOG = Boolean.valueOf(serverSettings.getProperty("SqlLog", "false"));

				ADMIN_TEST = Integer.parseInt(serverSettings.getProperty("ADMINTest", "100"));

				RATE_CONSUMABLE_COST = Float.parseFloat(serverSettings.getProperty("RateConsumableCost", "1."));
				RATE_SIEGE_GUARDS_PRICE = Float.parseFloat(serverSettings.getProperty("RateSiegeGuardsPrice", "1."));

				CG_HIREPRICE = Integer.parseInt(serverSettings.getProperty("AncientAdena", "100"));
				CG_HIRE = Integer.parseInt(serverSettings.getProperty("CGHire", "110"));
				CG_MAXHIREDGUARDS = Integer.parseInt(serverSettings.getProperty("CGMaxHiredGuards", "50"));
				CG_KILL = Integer.parseInt(serverSettings.getProperty("CGKill", "70"));
				CG_VIEWSTATS = Integer.parseInt(serverSettings.getProperty("CGViewStats", "80"));
				CG_STATUPGAIN = Integer.parseInt(serverSettings.getProperty("CGStatUpGain", "1"));
				CG_STATUPPROB = Integer.parseInt(serverSettings.getProperty("CGStatUpProb", "1"));
				CG_MOVEMENT = Integer.parseInt(serverSettings.getProperty("CGMovement", "40"));
				CG_CASTLE = Integer.parseInt(serverSettings.getProperty("CGCastle", "120"));
				CG_SET = Integer.parseInt(serverSettings.getProperty("CGSet", "100"));
				CG_SPAWNSIEGEGUARDS = Boolean.parseBoolean(serverSettings.getProperty("CGSpawnSiegeGuards", "false"));

				COORD_SYNCHRONIZE = Integer.parseInt(serverSettings.getProperty("CoordSynchronize", "-1"));
				DELETE_DAYS = Integer.parseInt(serverSettings.getProperty("DeleteCharAfterDays", "7"));

				DATAPACK_ROOT = new File(serverSettings.getProperty("DatapackRoot", ".")).getCanonicalFile();
				AttributeBonusFile = serverSettings.getProperty("AttributeBonusFile", "data/attribute_bonus.xml");

				ALLOW_L2WALKER_CLIENT = L2WalkerAllowed.valueOf(serverSettings.getProperty("AllowL2Walker", "False"));
				MAX_L2WALKER_LEVEL = Integer.parseInt(serverSettings.getProperty("MaxL2WalkerLevel", "5"));

				ALLOW_CRAFT_BOT = Boolean.parseBoolean(serverSettings.getProperty("AllowCraftBot", "True"));
				ALLOW_BOT_CAST = Boolean.parseBoolean(serverSettings.getProperty("AllowBotCast", "False"));
				KILL_BOTS_WITH_SKILLCOOLTIME = Boolean.parseBoolean(serverSettings.getProperty("KillBotsWithSkillCoolTime", "True"));
				L2WALKER_PUNISHMENT = Integer.parseInt(serverSettings.getProperty("L2WalkerPunishment", "1"));
				BUGUSER_PUNISH = Integer.parseInt(serverSettings.getProperty("BugUserPunishment", "2"));
				DEFAULT_PUNISH = Integer.parseInt(serverSettings.getProperty("IllegalActionPunishment", "1"));

				ALLOW_DISCARDITEM = Boolean.valueOf(serverSettings.getProperty("AllowDiscardItem", "True"));
				ALLOW_FREIGHT = Boolean.valueOf(serverSettings.getProperty("AllowFreight", "True"));
				ALLOW_WAREHOUSE = Boolean.valueOf(serverSettings.getProperty("AllowWarehouse", "True"));
				ALLOW_LOTTERY = Boolean.valueOf(serverSettings.getProperty("AllowLottery", "False"));
				ALLOW_RACE = Boolean.valueOf(serverSettings.getProperty("AllowRace", "False"));
				ALLOW_WATER = Boolean.valueOf(serverSettings.getProperty("AllowWater", "True"));
				ALLOW_BOAT = Boolean.valueOf(serverSettings.getProperty("AllowBoat", "False"));
				ALLOW_CURSED_WEAPONS = Boolean.valueOf(serverSettings.getProperty("AllowCursedWeapons", "False"));

				PACKET_LIFETIME = Integer.parseInt(serverSettings.getProperty("PacketLifeTime", "0"));

				MIN_PROTOCOL_REVISION = Integer.parseInt(serverSettings.getProperty("MinProtocolRevision", "709"));
				MAX_PROTOCOL_REVISION = Integer.parseInt(serverSettings.getProperty("MaxProtocolRevision", "746"));

				if(MIN_PROTOCOL_REVISION > MAX_PROTOCOL_REVISION)
					throw new Error("MinProtocolRevision is bigger than MaxProtocolRevision in server configuration file.");

				ACTIVATE_POSITION_RECORDER = Boolean.valueOf(serverSettings.getProperty("ActivatePositionRecorder", "False"));
				USE_3D_MAP = Boolean.valueOf(serverSettings.getProperty("Use3DMap", "False"));
				MIN_NPC_ANIMATION = Integer.parseInt(serverSettings.getProperty("MinNPCAnimation", "5"));
				MAX_NPC_ANIMATION = Integer.parseInt(serverSettings.getProperty("MaxNPCAnimation", "90"));

				ALLOW_COMMUNITYBOARD = Boolean.valueOf(serverSettings.getProperty("AllowCommunityBoard", "True"));
				COMMUNITY_TYPE = serverSettings.getProperty("CommunityType", "old").toLowerCase();
				BBS_DEFAULT = serverSettings.getProperty("BBSDefault", "_bbshome");
				SHOW_LEVEL_COMMUNITYBOARD = Boolean.valueOf(serverSettings.getProperty("ShowLevelOnCommunityBoard", "False"));
				SHOW_STATUS_COMMUNITYBOARD = Boolean.valueOf(serverSettings.getProperty("ShowStatusOnCommunityBoard", "True"));
				NAME_PAGE_SIZE_COMMUNITYBOARD = Integer.parseInt(serverSettings.getProperty("NamePageSizeOnCommunityBoard", "25"));
				NAME_PER_ROW_COMMUNITYBOARD = Integer.parseInt(serverSettings.getProperty("NamePerRowOnCommunityBoard", "5"));

				INTERNAL_HOSTNAME = serverSettings.getProperty("InternalHostname", "*");
				EXTERNAL_HOSTNAME = serverSettings.getProperty("ExternalHostname", "*");

				SERVER_SIDE_NPC_NAME = Boolean.valueOf(serverSettings.getProperty("ServerSideNpcName", "False"));
				SERVER_SIDE_NPC_TITLE_WITH_LVL = Boolean.valueOf(serverSettings.getProperty("ServerSideNpcTitleWithLvl", "False"));
				SERVER_SIDE_NPC_TITLE_WITH_SOCIAL = Boolean.valueOf(serverSettings.getProperty("ServerSideNpcTitleWithSocial", "False"));
				SERVER_SIDE_NPC_TITLE = Boolean.valueOf(serverSettings.getProperty("ServerSideNpcTitle", "False"));
				AUTODELETE_INVALID_QUEST_DATA = Boolean.valueOf(serverSettings.getProperty("AutoDeleteInvalidQuestData", "False"));

				HARD_DB_CLEANUP_ON_START = Boolean.valueOf(serverSettings.getProperty("HardDbCleanUpOnStart", "False"));

				MAXIMUM_ONLINE_USERS = Integer.parseInt(serverSettings.getProperty("MaximumOnlineUsers", "100"));

				DAY_STATUS_FORCE_CLIENT_UPDATE = Boolean.valueOf(serverSettings.getProperty("DayStatusForceClientUpdate", "True"));
				DAY_STATUS_SUN_RISE_AT = Integer.parseInt(serverSettings.getProperty("DayStatusSunRiseAt", "6"));
				DAY_STATUS_SUN_SET_AT = Integer.parseInt(serverSettings.getProperty("DayStatusSunSetAt", "18"));

				DATABASE_DRIVER = serverSettings.getProperty("Driver", "com.mysql.jdbc.Driver");
				DATABASE_URL = serverSettings.getProperty("URL", "jdbc:mysql://localhost/l2rdb");
				DATABASE_LOGIN = serverSettings.getProperty("Login", "root");
				DATABASE_PASSWORD = serverSettings.getProperty("Password", "");
				DATABASE_MAX_CONNECTIONS = Integer.parseInt(serverSettings.getProperty("MaximumDbConnections", "10"));
				DATABASE_MAX_IDLE_TIMEOUT = Integer.parseInt(serverSettings.getProperty("MaxIdleConnectionTimeout", "600"));
				DATABASE_IDLE_TEST_PERIOD = Integer.parseInt(serverSettings.getProperty("IdleConnectionTestPeriod", "60"));
				USE_DATABASE_LAYER = Boolean.parseBoolean(serverSettings.getProperty("UseDatabaseLayer", "true"));

				LAZY_ITEM_UPDATE = Boolean.parseBoolean(serverSettings.getProperty("LazyItemUpdate", "False"));
				LAZY_ITEM_UPDATE_TIME = Integer.parseInt(serverSettings.getProperty("LazyItemUpdateTime", "60000"));

				COUNT_PACKETS = Boolean.valueOf(serverSettings.getProperty("CountPacket", "False"));
				DUMP_PACKET_COUNTS = Boolean.valueOf(serverSettings.getProperty("DumpPacketCounts", "False"));
				DUMP_INTERVAL_SECONDS = Integer.parseInt(serverSettings.getProperty("PacketDumpInterval", "60"));

				SERVER_LIST_BRACKET = Boolean.valueOf(serverSettings.getProperty("ServerListBrackets", "false"));
				SERVER_LIST_CLOCK = Boolean.valueOf(serverSettings.getProperty("ServerListClock", "false"));
				SERVER_GMONLY = Boolean.valueOf(serverSettings.getProperty("ServerGMOnly", "false"));

				MINIMUM_UPDATE_DISTANCE = Integer.parseInt(serverSettings.getProperty("MaximumUpdateDistance", "50"));
				MINIMUN_UPDATE_TIME = Integer.parseInt(serverSettings.getProperty("MinimumUpdateTime", "500"));
				CHECK_KNOWN = Boolean.valueOf(serverSettings.getProperty("CheckKnownList", "false"));
				KNOWNLIST_FORGET_DELAY = Integer.parseInt(serverSettings.getProperty("KnownListForgetDelay", "10000"));

				LAZY_CACHE = Boolean.valueOf(serverSettings.getProperty("LazyCache", "False"));

				EXECUTOR_PRI_LOW = Integer.parseInt(serverSettings.getProperty("ExecutorPriLow", "-1"));
				EXECUTOR_PRI_MED = Integer.parseInt(serverSettings.getProperty("ExecutorPriMed", "0"));
				EXECUTOR_PRI_HIGH = Integer.parseInt(serverSettings.getProperty("ExecutorPriHigh", "1"));

				THREAD_P_EFFECTS = Integer.parseInt(serverSettings.getProperty("ThreadPoolSizeEffects", "10"));
				THREAD_P_GENERAL = Integer.parseInt(serverSettings.getProperty("ThreadPoolSizeGeneral", "15"));
				GENERAL_PACKET_THREAD_CORE_SIZE = Integer.parseInt(serverSettings.getProperty("GeneralPacketThreadCoreSize", "4"));
				URGENT_PACKET_THREAD_CORE_SIZE = Integer.parseInt(serverSettings.getProperty("UrgentPacketThreadCoreSize", "2"));
				AI_MAX_THREAD = Integer.parseInt(serverSettings.getProperty("AiMaxThread", "10"));

				MAT_BANCHAT = Boolean.valueOf(serverSettings.getProperty("MAT_BANCHAT", "False"));
				MAT_BANCHATG = Boolean.valueOf(serverSettings.getProperty("MAT_BANCHATG", "False"));
				MAT_REPLACE = Boolean.valueOf(serverSettings.getProperty("MAT_REPLACE", "False"));
				MAT_REPLACE_STRING = serverSettings.getProperty("MAT_REPLACE_STRING", "[censored]");
				UNCHATBANTIME = Integer.parseInt(serverSettings.getProperty("Timer_to_UnBan", "30"));
				ENABLE_TRANSLIT_ENGINE = Boolean.valueOf(serverSettings.getProperty("EnableTranslitEngine", "False"));
				ENABLE_TEXT_FILTER = Boolean.valueOf(serverSettings.getProperty("EnableTextFilter", "False"));
				RESTART_TIME = Integer.parseInt(serverSettings.getProperty("AutoRestart", "4"));
				CHECK_LANG_FILES_MODIFY = Boolean.valueOf(serverSettings.getProperty("checkLangFilesModify", "false"));

				LINEAR_CELL_SIZE = Integer.parseInt(serverSettings.getProperty("LinearCellSize", "8"));
				LINEAR_TERRITORY_CELL_SIZE = Integer.parseInt(serverSettings.getProperty("LinearTerritoryCellSize", "32"));

				DISABLE_CREATION_IDs = serverSettings.getProperty("DisableCreateItems", "");
				if(DISABLE_CREATION_IDs.length() != 0)
					for(String id : DISABLE_CREATION_IDs.split(","))
						DISABLE_CREATION_ID_LIST.add(Integer.parseInt(id));

				LOG_MULTISELL_IDs = serverSettings.getProperty("LogMultisellId", "");
				if(LOG_MULTISELL_IDs.length() != 0)
					for(String id : LOG_MULTISELL_IDs.split(","))
						LOG_MULTISELL_ID_LIST.add(Integer.parseInt(id));

				GEODATA = Boolean.parseBoolean(serverSettings.getProperty("GeoData", "False"));
				GEOFILES_PATTERN = serverSettings.getProperty("GeoFilesPattern", "(\\d{2}_\\d{2})\\.l2j");
				GEODATA_DEBUG = Boolean.parseBoolean(serverSettings.getProperty("GeodataDebug", "false"));
				PATH_CLEAN = Boolean.parseBoolean(serverSettings.getProperty("PathClean", "True"));
				PATHFINDING_BOOST = Boolean.parseBoolean(serverSettings.getProperty("PathFindingBoost", "False"));

				DAMAGE_FROM_FALLING = Boolean.parseBoolean(serverSettings.getProperty("DamageFromFalling", "True"));

				GEOEDITOR_HOST = serverSettings.getProperty("DamageFromFalling", "127.0.0.1");

				ALLOW_WEDDING = Boolean.valueOf(serverSettings.getProperty("AllowWedding", "False"));
				WEDDING_PRICE = Integer.parseInt(serverSettings.getProperty("WeddingPrice", "500000"));
				WEDDING_PUNISH_INFIDELITY = Boolean.parseBoolean(serverSettings.getProperty("WeddingPunishInfidelity", "true"));
				WEDDING_TELEPORT = Boolean.parseBoolean(serverSettings.getProperty("WeddingTeleport", "true"));
				WEDDING_TELEPORT_PRICE = Integer.parseInt(serverSettings.getProperty("WeddingTeleportPrice", "500000"));
				WEDDING_TELEPORT_INTERVAL = Integer.parseInt(serverSettings.getProperty("WeddingTeleportInterval", "120"));
				WEDDING_SAMESEX = Boolean.parseBoolean(serverSettings.getProperty("WeddingAllowSameSex", "true"));
				WEDDING_FORMALWEAR = Boolean.parseBoolean(serverSettings.getProperty("WeddingFormalWear", "true"));
				WEDDING_DIVORCE_COSTS = Integer.parseInt(serverSettings.getProperty("WeddingDivorceCosts", "20"));

				FORCE_STATUSUPDATE = Boolean.parseBoolean(serverSettings.getProperty("ForceStatusUpdate", "false"));
				DONTLOADSPAWN = Boolean.parseBoolean(serverSettings.getProperty("StartWhisoutSpawn", "False"));
				DONTLOADQUEST = Boolean.parseBoolean(serverSettings.getProperty("StartWhisoutQuest", "False"));
				TUTORIAL_ON_OFF = Boolean.parseBoolean(serverSettings.getProperty("TutorialEnabled", "True"));

				PLAYER_VISIBILITY = Integer.parseInt(serverSettings.getProperty("PlayerVisibilityHorizontal", "4500"));
				PLAYER_VISIBILITY_Z = Integer.parseInt(serverSettings.getProperty("PlayerVisibilityVertical", "2000"));
				if(PLAYER_VISIBILITY > 7000 || PLAYER_VISIBILITY < 1500)
					PLAYER_VISIBILITY = 4500;
				if(PLAYER_VISIBILITY_Z > 3000 || PLAYER_VISIBILITY_Z < 500)
					PLAYER_VISIBILITY_Z = 2000;
				GG_CHECK = Boolean.parseBoolean(serverSettings.getProperty("GGCheck", "True"));
				CHECK_SESSIONKEY = Boolean.parseBoolean(serverSettings.getProperty("CheckSessionKey", "True"));

				SHIFT_BY = Integer.parseInt(serverSettings.getProperty("ShiftBy", "12"));
				SHIFT_BY_FOR_Z = Integer.parseInt(serverSettings.getProperty("ShiftByForZ", "10"));
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + CONFIGURATION_FILE + " File.");
			}

			/*
			 * Load L2jFT Version Properties file (if exists)
			 */
			try
			{
				Properties serverVersion = new Properties();
				InputStream is = Config.class.getResourceAsStream(Config.VERSION_FILE);
				serverVersion.load(is);
				is.close();

				SERVER_VERSION = serverVersion.getProperty("version", "Unsupported Custom Version.");
				SERVER_BUILD_DATE = serverVersion.getProperty("builddate", "Undefined Date.");
				_log.info("Gameserver Version: " + SERVER_VERSION + ", build date: " + SERVER_BUILD_DATE);
			}
			catch(Exception e)
			{
				//Ignore Properties file if it doesn't exist
				SERVER_VERSION = "Unsupported Custom Version.";
				SERVER_BUILD_DATE = "Undefined Date.";
			}

			// telnet
			try
			{
				Properties telnetSettings = new Properties();
				InputStream is = new FileInputStream(new File(TELNET_FILE));
				telnetSettings.load(is);
				is.close();

				IS_TELNET_ENABLED = Boolean.valueOf(telnetSettings.getProperty("EnableTelnet", "false"));
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + TELNET_FILE + " File.");
			}

			// clanhall settings
			try
			{
				Properties clanhallSettings = new Properties();
				InputStream is = new FileInputStream(new File(CLANHALL_CONFIG_FILE));
				clanhallSettings.load(is);
				is.close();

				ALLOW_CH_DOOR_OPEN_ON_CLICK = Boolean.valueOf(clanhallSettings.getProperty("AllowChDoorOpenOnClick", "true"));

				CH_TELE_FEE_RATIO = Long.valueOf(clanhallSettings.getProperty("ClanHallTeleportFunctionFeeRation", "86400000"));
				CH_TELE1_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallTeleportFunctionFeeLvl1", "86400000"));
				CH_TELE2_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallTeleportFunctionFeeLvl2", "86400000"));
				CH_TELE3_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallTeleportFunctionFeeLvl3", "86400000"));
				CH_SUPPORT_FEE_RATIO = Long.valueOf(clanhallSettings.getProperty("ClanHallSupportFunctionFeeRation", "86400000"));
				CH_SUPPORT1_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallSupportFeeLvl1", "86400000"));
				CH_SUPPORT2_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallSupportFeeLvl2", "86400000"));
				CH_SUPPORT3_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallSupportFeeLvl3", "86400000"));
				CH_SUPPORT4_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallSupportFeeLvl4", "86400000"));
				CH_SUPPORT5_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallSupportFeeLvl5", "86400000"));
				CH_SUPPORT7_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallSupportFeeLvl7", "86400000"));
				CH_SUPPORT8_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallSupportFeeLvl8", "86400000"));
				CH_MPREG_FEE_RATIO = Long.valueOf(clanhallSettings.getProperty("ClanHallMpRegenerationFunctionFeeRation", "86400000"));
				CH_MPREG1_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallMpRegenerationFeeLvl1", "86400000"));
				CH_MPREG2_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallMpRegenerationFeeLvl2", "86400000"));
				CH_MPREG3_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallMpRegenerationFeeLvl3", "86400000"));
				CH_MPREG4_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallMpRegenerationFeeLvl4", "86400000"));
				CH_MPREG5_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallMpRegenerationFeeLvl5", "86400000"));
				CH_MPREG6_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallMpRegenerationFeeLvl6", "86400000"));
				CH_HPREG_FEE_RATIO = Long.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFunctionFeeRation", "86400000"));
				CH_HPREG1_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl1", "86400000"));
				CH_HPREG2_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl2", "86400000"));
				CH_HPREG3_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl3", "86400000"));
				CH_HPREG4_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl4", "86400000"));
				CH_HPREG5_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl5", "86400000"));
				CH_HPREG6_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl6", "86400000"));
				CH_HPREG7_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl7", "86400000"));
				CH_HPREG8_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl8", "86400000"));
				CH_HPREG9_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl9", "86400000"));
				CH_HPREG10_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl10", "86400000"));
				CH_HPREG11_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl11", "86400000"));
				CH_HPREG12_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallHpRegenerationFeeLvl12", "86400000"));
				CH_EXPREG_FEE_RATIO = Long.valueOf(clanhallSettings.getProperty("ClanHallExpRegenerationFunctionFeeRation", "86400000"));
				CH_EXPREG1_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallExpRegenerationFeeLvl1", "86400000"));
				CH_EXPREG2_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallExpRegenerationFeeLvl2", "86400000"));
				CH_EXPREG3_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallExpRegenerationFeeLvl3", "86400000"));
				CH_EXPREG4_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallExpRegenerationFeeLvl4", "86400000"));
				CH_EXPREG5_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallExpRegenerationFeeLvl5", "86400000"));
				CH_EXPREG6_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallExpRegenerationFeeLvl6", "86400000"));
				CH_EXPREG7_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallExpRegenerationFeeLvl7", "86400000"));
				CH_EXPREG8_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallExpRegenerationFeeLvl8", "86400000"));
				CH_ITEM_FEE_RATIO = Long.valueOf(clanhallSettings.getProperty("ClanHallItemCreationFunctionFeeRation", "86400000"));
				CH_ITEM1_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallItemCreationFunctionFeeLvl1", "86400000"));
				CH_ITEM2_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallItemCreationFunctionFeeLvl2", "86400000"));
				CH_ITEM3_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallItemCreationFunctionFeeLvl3", "86400000"));
				CH_CURTAIN_FEE_RATIO = Long.valueOf(clanhallSettings.getProperty("ClanHallCurtainFunctionFeeRation", "86400000"));
				CH_CURTAIN1_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallCurtainFunctionFeeLvl1", "86400000"));
				CH_CURTAIN2_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallCurtainFunctionFeeLvl2", "86400000"));
				CH_PLATFORM_FEE_RATIO = Long.valueOf(clanhallSettings.getProperty("ClanHallFrontPlatformFunctionFeeRation", "86400000"));
				CH_PLATFORM1_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallFrontPlatformFunctionFeeLvl1", "86400000"));
				CH_PLATFORM2_FEE = Integer.valueOf(clanhallSettings.getProperty("ClanHallFrontPlatformFunctionFeeLvl2", "86400000"));

			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + CLANHALL_CONFIG_FILE + " File.");
			}

			//WEB
			try
			{
				Properties webSettings = new Properties();
				InputStream is = new FileInputStream(new File(WEB_CONFIG_FILE));
				webSettings.load(is);
				is.close();
				
				WebServerPort = Integer.parseInt(webSettings.getProperty("WebServerPort", "7778"));
				WebServerRoot = webSettings.getProperty("WebServerRoot", "./data/webserver");
				USE_FILE_CACHE = Boolean.valueOf(webSettings.getProperty("useFileCache", "true"));
				
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + WEB_CONFIG_FILE + " File.");
			}
			
			//RATE
			try
			{
				Properties rateSettings = new Properties();
				InputStream is = new FileInputStream(new File(RATE_CONFIG_FILE));
				rateSettings.load(is);
				is.close();
				
				RATE_XP = Float.parseFloat(rateSettings.getProperty("RateXp", "1."));
				RATE_SP = Float.parseFloat(rateSettings.getProperty("RateSp", "1."));
				RATE_PARTY_XP = Float.parseFloat(rateSettings.getProperty("RatePartyXp", "1."));
				RATE_PARTY_SP = Float.parseFloat(rateSettings.getProperty("RatePartySp", "1."));
				RATE_QUESTS_REWARD = Float.parseFloat(rateSettings.getProperty("RateQuestsReward", "1."));
				RATE_CLAN_REP_SCORE = Float.parseFloat(rateSettings.getProperty("RateClanRepScore", "1."));
				RATE_CLAN_REP_SCORE_MAX_AFFECTED = Integer.parseInt(rateSettings.getProperty("RateClanRepScoreMaxAffected", "2"));
				RATE_DROP_ADENA = Float.parseFloat(rateSettings.getProperty("RateDropAdena", "1."));
				RATE_DROP_ITEMS = Float.parseFloat(rateSettings.getProperty("RateDropItems", "1."));
				RATE_DROP_SPOIL = Float.parseFloat(rateSettings.getProperty("RateDropSpoil", "1."));
				RATE_MANOR = Integer.parseInt(rateSettings.getProperty("RateManor", "1"));
				RATE_RAID_REGEN = Float.parseFloat(rateSettings.getProperty("RateRaidRegen", "1."));
				RAID_MAX_LEVEL_DIFF = Integer.parseInt(rateSettings.getProperty("RaidMaxLevelDiff", "8"));
				APPLY_RATES_ON_RBS = Boolean.parseBoolean(rateSettings.getProperty("ApplyRatesOnRBs", "False"));
				RATE_FISH_DROP_COUNT = Float.parseFloat(rateSettings.getProperty("RateFishDropCount", "1."));
				AUTODESTROY_ITEM_AFTER = Integer.parseInt(rateSettings.getProperty("AutoDestroyDroppedItemAfter", "0"));
								
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + RATE_CONFIG_FILE + " File.");
			}
			
				// other
				try
				{
				Properties otherSettings = new Properties();
				InputStream is = new FileInputStream(new File(OTHER_CONFIG_FILE));
				otherSettings.load(is);
				is.close();

				DEEPBLUE_DROP_RULES = Boolean.parseBoolean(otherSettings.getProperty("UseDeepBlueDropRules", "True"));
				ALLOW_GUARDS = Boolean.valueOf(otherSettings.getProperty("AllowGuards", "False"));
				EFFECT_CANCELING = Boolean.valueOf(otherSettings.getProperty("CancelLesserEffect", "True"));
				WYVERN_SPEED = Integer.parseInt(otherSettings.getProperty("WyvernSpeed", "100"));
				STRIDER_SPEED = Integer.parseInt(otherSettings.getProperty("StriderSpeed", "80"));
				SWIMING_SPEED = Integer.parseInt(otherSettings.getProperty("SwimingSpeedTemplate", "50"));
				ALLOW_WYVERN_UPGRADER = Boolean.valueOf(otherSettings.getProperty("AllowWyvernUpgrader", "False"));

				/* Inventory slots limits */
				INVENTORY_MAXIMUM_NO_DWARF = Integer.parseInt(otherSettings.getProperty("MaximumSlotsForNoDwarf", "80"));
				INVENTORY_MAXIMUM_DWARF = Integer.parseInt(otherSettings.getProperty("MaximumSlotsForDwarf", "100"));
				INVENTORY_MAXIMUM_GM = Integer.parseInt(otherSettings.getProperty("MaximumSlotsForGMPlayer", "250"));

				/* Warehouse slots limits */
				WAREHOUSE_SLOTS_NO_DWARF = Integer.parseInt(otherSettings.getProperty("BaseWarehouseSlotsForNoDwarf", "100"));
				WAREHOUSE_SLOTS_DWARF = Integer.parseInt(otherSettings.getProperty("BaseWarehouseSlotsForDwarf", "120"));
				WAREHOUSE_SLOTS_CLAN = Integer.parseInt(otherSettings.getProperty("MaximumWarehouseSlotsForClan", "200"));

				/* chance to enchant an item over safe level */
				ENCHANT_CHANCE_WEAPON = Integer.parseInt(otherSettings.getProperty("EnchantChance", "66"));
				ENCHANT_CHANCE_ARMOR = Integer.parseInt(otherSettings.getProperty("EnchantChanceArmor", String.valueOf(ENCHANT_CHANCE_WEAPON)));
				/* limit on enchant */
				ENCHANT_MAX = Byte.parseByte(otherSettings.getProperty("EnchantMax", "20"));
				if(ENCHANT_MAX > 20)
					ENCHANT_MAX = 20;

				/* if different from 100 (ie 100%) heal rate is modified acordingly */
				HP_REGEN_MULTIPLIER = Double.parseDouble(otherSettings.getProperty("HpRegenMultiplier", "100"));
				MP_REGEN_MULTIPLIER = Double.parseDouble(otherSettings.getProperty("MpRegenMultiplier", "100"));
				CP_REGEN_MULTIPLIER = Double.parseDouble(otherSettings.getProperty("CpRegenMultiplier", "100"));
				REGEN_SIT_WAIT = Boolean.valueOf(otherSettings.getProperty("RegenSitWait", "False"));

				STARTING_ADENA = Integer.parseInt(otherSettings.getProperty("StartingAdena", "0"));
				UNSTUCK_SKILL = Boolean.parseBoolean(otherSettings.getProperty("UnstuckSkill", "True"));

				/* Player protection after teleport or login */
				PLAYER_SPAWN_PROTECTION = Integer.parseInt(otherSettings.getProperty("PlayerSpawnProtection", "0"));

				/* Amount of HP, MP, and CP is restored */
				RESPAWN_RESTORE_CP = Double.parseDouble(otherSettings.getProperty("RespawnRestoreCP", "0")) / 100;
				RESPAWN_RESTORE_HP = Double.parseDouble(otherSettings.getProperty("RespawnRestoreHP", "70")) / 100;
				RESPAWN_RESTORE_MP = Double.parseDouble(otherSettings.getProperty("RespawnRestoreMP", "70")) / 100;

				/* Maximum number of available slots for pvt stores */
				MAX_PVTSTORE_SLOTS_DWARF = Integer.parseInt(otherSettings.getProperty("MaxPvtStoreSlotsDwarf", "5"));
				MAX_PVTSTORE_SLOTS_OTHER = Integer.parseInt(otherSettings.getProperty("MaxPvtStoreSlotsOther", "4"));

				PET_RENT_NPC = otherSettings.getProperty("ListPetRentNpc", "7827");

				LIST_PET_RENT_NPC = new ArrayList<Integer>();
				for(String id : PET_RENT_NPC.split(","))
					LIST_PET_RENT_NPC.add(Integer.parseInt(id));
				NONDROPPABLE_ITEMS = otherSettings.getProperty("ListOfNonDroppableItems", "1147,425,1146,461,10,2368,7,6,2370,2369,5598");

				LIST_NONDROPPABLE_ITEMS = new ArrayList<Integer>();
				for(String id : NONDROPPABLE_ITEMS.split(","))
					LIST_NONDROPPABLE_ITEMS.add(Integer.parseInt(id));

				ANNOUNCE_MAMMON_SPAWN = Boolean.parseBoolean(otherSettings.getProperty("AnnounceMammonSpawn", "True"));

				ALT_PRIVILEGES_SECURE_CHECK = Boolean.parseBoolean(otherSettings.getProperty("AltPrivilegesSecureCheck", "True"));
				ALT_PRIVILEGES_DEFAULT_LEVEL = Integer.parseInt(otherSettings.getProperty("AltPrivilegesDefaultLevel", "100"));

				GM_NAME_COLOUR = Integer.decode("0x" + otherSettings.getProperty("GMNameColour", "FFFFFF"));
				GM_HERO_AURA = Boolean.parseBoolean(otherSettings.getProperty("GMHeroAura", "True"));
				NORMAL_NAME_COLOUR = Integer.decode("0x" + otherSettings.getProperty("NormalNameColour", "FFFFFF"));
				CLANLEADER_NAME_COLOUR = Integer.decode("0x" + otherSettings.getProperty("ClanleaderNameColour", "FFFFFF"));
				BOT_NAME_COLOUR = Integer.decode("0x" + otherSettings.getProperty("BotNameColour", "FFFFFF"));
				BOT_NAME_HEX_COLOUR = otherSettings.getProperty("BotNameHexColour", "FFFFFF");

				OVER_LEVEL_PENALTY = Boolean.valueOf(otherSettings.getProperty("OverLevelPenalty", "False"));
				DIFF_LEVEL_PENALTY = Integer.parseInt(otherSettings.getProperty("DiffLevelPenalty", "3"));

				BOW_REUSE_MODIFIER = Float.parseFloat(otherSettings.getProperty("BowReuseModifier", "1.5"));

				RRD_ENABLED = Boolean.valueOf(otherSettings.getProperty("UseRRD", "True"));
				RRD_EXTENDED = Boolean.valueOf(otherSettings.getProperty("UseExtendedRRD", "False"));
				RRD_GRAPH_AA = Boolean.valueOf(otherSettings.getProperty("GraphAntiAliasing", "False"));
				RRD_PATH = otherSettings.getProperty("RRDPath", "./config/");
				RRD_GRAPH_PATH = otherSettings.getProperty("GraphPath", "./data/webserver/");
				RRD_REDRAW_TIME = Long.parseLong(otherSettings.getProperty("GraphRedrawDelay", "300"));
				RRD_UPDATE_TIME = Long.parseLong(otherSettings.getProperty("UpdateDelay", "30"));
				RRD_GRAPH_FORMAT = otherSettings.getProperty("GraphFormat", "png");
				RRD_GRAPH_HEIGHT = Integer.parseInt(otherSettings.getProperty("GraphHeight", "378"));
				RRD_GRAPH_WIDTH = Integer.parseInt(otherSettings.getProperty("GraphWidth", "580"));
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + OTHER_CONFIG_FILE + " File.");
			}

			// spoil & manor
			try
			{
				Properties spoilSettings = new Properties();
				InputStream is = new FileInputStream(new File(SPOIL_CONFIG_FILE));
				spoilSettings.load(is);
				is.close();

				BASE_SPOIL_RATE = Float.parseFloat(spoilSettings.getProperty("BasePercentChanceOfSpoilSuccess", "78."));
				MINIMUM_SPOIL_RATE = Float.parseFloat(spoilSettings.getProperty("MinimumPercentChanceOfSpoilSuccess", "1."));
				ALT_SPOIL_FORMULA = Boolean.parseBoolean(spoilSettings.getProperty("AltFormula", "False"));
				MANOR_SOWING_BASIC_SUCCESS = Integer.parseInt(spoilSettings.getProperty("BasePercentChanceOfSowingSuccess", "100"));
				MANOR_SOWING_ALT_BASIC_SUCCESS = Integer.parseInt(spoilSettings.getProperty("BasePercentChanceOfSowingAltSuccess", "10"));
				MANOR_HARVESTING_BASIC_SUCCESS = Integer.parseInt(spoilSettings.getProperty("BasePercentChanceOfHarvestingSuccess", "90"));
				MANOR_DIFF_PLAYER_TARGET = Integer.parseInt(spoilSettings.getProperty("MinDiffPlayerMob", "5"));
				MANOR_DIFF_PLAYER_TARGET_PENALTY = Integer.parseInt(spoilSettings.getProperty("DiffPlayerMobPenalty", "5"));
				MANOR_DIFF_SEED_TARGET = Integer.parseInt(spoilSettings.getProperty("MinDiffSeedMob", "5"));
				MANOR_DIFF_SEED_TARGET_PENALTY = Integer.parseInt(spoilSettings.getProperty("DiffSeedMobPenalty", "5"));
				ALLOW_MANOR = Boolean.parseBoolean(spoilSettings.getProperty("AllowManor", "true"));
				ALT_SIMPLE_MANOR = Boolean.parseBoolean(spoilSettings.getProperty("SimpleManor", "false"));
				MANOR_REFRESH_TIME = Integer.parseInt(spoilSettings.getProperty("AltManorRefreshTime", "20"));
				MANOR_REFRESH_MIN = Integer.parseInt(spoilSettings.getProperty("AltManorRefreshMin", "00"));
				MANOR_APPROVE_TIME = Integer.parseInt(spoilSettings.getProperty("AltManorApproveTime", "6"));
				MANOR_APPROVE_MIN = Integer.parseInt(spoilSettings.getProperty("AltManorApproveMin", "00"));
				MANOR_MAINTENANCE_PERIOD = Integer.parseInt(spoilSettings.getProperty("AltManorMaintenancePeriod", "360000"));
				MANOR_SAVE_ALL_ACTIONS = Boolean.parseBoolean(spoilSettings.getProperty("AltManorSaveAllActions", "false"));
				MANOR_SAVE_PERIOD_RATE = Integer.parseInt(spoilSettings.getProperty("AltManorSavePeriodRate", "2"));
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + SPOIL_CONFIG_FILE + " File.");
			}

			// alternative settings
			try
			{
				Properties altSettings = new Properties();
				InputStream is = new FileInputStream(new File(ALT_SETTINGS_FILE));
				altSettings.load(is);
				is.close();

				ALT_GAME_DELEVEL = Boolean.parseBoolean(altSettings.getProperty("Delevel", "true"));

				ALT_SKILLS_CHANCES = Boolean.parseBoolean(altSettings.getProperty("AltSkillChances", "False"));
				SKILLS_CHANCE_CAP = Double.parseDouble(altSettings.getProperty("SkillsChanceCap", "95"));
				SKILLS_SHOW_CHANCE = Boolean.parseBoolean(altSettings.getProperty("SkillsShowChance", "True"));
				ALT_SKILLS_UNSAVABLE_SAVE = Boolean.parseBoolean(altSettings.getProperty("AltSkillsUnsaveableSave", "False"));
				ALT_SHOW_REUSE_MSG = Boolean.parseBoolean(altSettings.getProperty("AltShowSkillReuseMessage", "False"));
				ALLOW_CLASS_MASTERS = altSettings.getProperty("AllowClassMasters", "0");
				if(ALLOW_CLASS_MASTERS.length() != 0 && !ALLOW_CLASS_MASTERS.equals("0"))
					for(String id : ALLOW_CLASS_MASTERS.split(","))
						ALLOW_CLASS_MASTERS_LIST.add(Integer.parseInt(id));
				CLASS_MASTERS_PRICE = altSettings.getProperty("ClassMastersPrice", "0,0,0");
				if(CLASS_MASTERS_PRICE.length() >= 5)
				{
					int level = 1;
					for(String id : CLASS_MASTERS_PRICE.split(","))
					{
						CLASS_MASTERS_PRICE_LIST[level] = Integer.parseInt(id);
						level++;
					}
				}
				ALT_GAME_SKILL_HIT_RATE = Float.parseFloat(altSettings.getProperty("AltGameSkillHitRate", "1."));
				ENABLE_RATE_HP = Boolean.parseBoolean(altSettings.getProperty("EnableRateHp", "True"));
				WEAR_TEST_ENABLED = Boolean.valueOf(altSettings.getProperty("WearTestEnabled", "False"));
				IS_CRAFTING_ENABLED = Boolean.parseBoolean(altSettings.getProperty("CraftingEnabled", "True"));
				AUTO_LOOT = Boolean.parseBoolean(altSettings.getProperty("AutoLoot", "True"));
				AUTO_LOOT_PK = Boolean.parseBoolean(altSettings.getProperty("AutoLootPK", "True"));
				AUTO_LOOT_HERBS = Boolean.parseBoolean(altSettings.getProperty("AutoLootHerbs", "True"));
				ALT_GAME_KARMAPLAYERCANBEKILLEDINPEACEZONE = Boolean.valueOf(altSettings.getProperty("AltKarmaPlayerCanBeKilledInPeaceZone", "false"));
				ALT_GAME_KARMA_PLAYER_CAN_SHOP = Boolean.valueOf(altSettings.getProperty("AltKarmaPlayerCanShop", "false"));
				SAVING_SPS = Boolean.parseBoolean(altSettings.getProperty("SavingSpS", "false"));
				MANAHEAL_SPS_BONUS = Boolean.parseBoolean(altSettings.getProperty("ManahealSpSBonus", "false"));
				KILL_COUNTER = Boolean.parseBoolean(altSettings.getProperty("KillCounter", "true"));
				KILL_COUNTER_PRELOAD = Boolean.parseBoolean(altSettings.getProperty("KillCounterPreload", "true"));
				DROP_COUNTER = Boolean.parseBoolean(altSettings.getProperty("DropCounter", "true"));
				CRAFT_COUNTER = Boolean.parseBoolean(altSettings.getProperty("CraftCounter", "true"));
				CORPSE_MOB_TIME = Integer.parseInt(altSettings.getProperty("CorpseMobTime", "8500"));
				ALT_TRUE_CHESTS = Float.parseFloat(altSettings.getProperty("TrueChests", ".33"));
				ALT_GAME_MATHERIALSDROP = Boolean.parseBoolean(altSettings.getProperty("AltMatherialsDrop", "false"));
				ALT_DOUBLE_SPAWN = Boolean.parseBoolean(altSettings.getProperty("DoubleSpawn", "false"));
				ALT_ALLOW_AUGMENT_ALL = Boolean.parseBoolean(altSettings.getProperty("AugmentAll", "false"));
				ALT_GAME_EXP_FOR_CRAFT = Boolean.parseBoolean(altSettings.getProperty("AltExpForCraft", "true"));
				ALT_GAME_UNREGISTER_RECIPE = Boolean.parseBoolean(altSettings.getProperty("AltUnregisterRecipe", "true"));
				ALT_GAME_SHOW_DROPLIST = Boolean.parseBoolean(altSettings.getProperty("AltShowDroplist", "true"));
				ALT_FULL_NPC_STATS_PAGE = Boolean.parseBoolean(altSettings.getProperty("AltFullStatsPage", "False"));
				ALT_GAME_SUBCLASS_WITHOUT_QUESTS = Boolean.parseBoolean(altSettings.getProperty("AltAllowSubClassWithoutQuest", "false"));
				ALT_GAME_LEVEL_TO_GET_SUBCLASS = Integer.parseInt(altSettings.getProperty("AltLevelToGetSubclass", "75"));
				ALT_ALLOW_OTHERS_WITHDRAW_FROM_CLAN_WAREHOUSE = Boolean.parseBoolean(altSettings.getProperty("AltAllowOthersWithdrawFromClanWarehouse", "true"));
				ALT_GAME_REQUIRE_CLAN_CASTLE = Boolean.parseBoolean(altSettings.getProperty("AltRequireClanCastle", "false"));
				ALT_GAME_REQUIRE_CASTLE_DAWN = Boolean.parseBoolean(altSettings.getProperty("AltRequireCastleDawn", "true"));
				ALT_ADD_RECIPES = Integer.parseInt(altSettings.getProperty("AltAddRecipes", "0"));
				PARTY_DIFF_LEVEL_PENALTY = Integer.parseInt(altSettings.getProperty("AltPartyDiffLevelPenalty", "9"));
				SS_ANNOUNCE_PERIOD = Integer.parseInt(altSettings.getProperty("SSAnnouncePeriod", "0"));
				ENABLE_FISHING_CHECK_WATER = Boolean.parseBoolean(altSettings.getProperty("EnableFishingWaterCheck", "true"));
				AUTO_LEARN_SKILLS = Boolean.parseBoolean(altSettings.getProperty("AutoLearnSkills", "false"));
				ENABLE_OLYMPIAD = Boolean.parseBoolean(altSettings.getProperty("EnableOlympiad", "false"));
				ALT_OLY_START_TIME = Integer.parseInt(altSettings.getProperty("AltOlyStartTime", "18"));
				ALT_OLY_MIN = Integer.parseInt(altSettings.getProperty("AltOlyMin", "00"));
				ALT_OLY_CPERIOD = Long.parseLong(altSettings.getProperty("AltOlyCPeriod", "21600000"));
				ALT_OLY_BATTLE = Long.parseLong(altSettings.getProperty("AltOlyBattle", "360000"));
				ALT_OLY_BWAIT = Long.parseLong(altSettings.getProperty("AltOlyBWait", "600000"));
				ALT_OLY_IWAIT = Long.parseLong(altSettings.getProperty("AltOlyIWait", "300000"));
				ALT_OLY_WPERIOD = Long.parseLong(altSettings.getProperty("AltOlyWPeriod", "604800000"));
				ALT_OLY_VPERIOD = Long.parseLong(altSettings.getProperty("AltOlyVPeriod", "86400000"));
				ALT_HONOR_SYSTEM = Boolean.parseBoolean(altSettings.getProperty("HonorSystem", "False"));
				ALT_SOCIAL_ACTION_REUSE = Boolean.parseBoolean(altSettings.getProperty("AltSocialActionReuse", "False"));
				ALT_SIMPLE_SIGNS = Boolean.parseBoolean(altSettings.getProperty("PushkinSignsOptinos", "false"));
				ALT_MAMMOTH_UPGRADE = Integer.parseInt(altSettings.getProperty("MammonUpgrade", "2"));
				ALT_MAMMOTH_EXCHANGE = Integer.parseInt(altSettings.getProperty("MammonExchange", "3"));
				ALT_ALLOW_TATTOO = Boolean.parseBoolean(altSettings.getProperty("AllowTattoo", "false"));
				BASE_EXP_LOST_ON_DEATH = Float.parseFloat(altSettings.getProperty("BaseExpLostOnDeath", "4."));
				ALT_DEATH_PENALTY = Boolean.parseBoolean(altSettings.getProperty("EnableAltDeathPenalty", "false"));
				ALLOW_DEATH_PENALTY_C5 = Boolean.parseBoolean(altSettings.getProperty("EnableDeathPenaltyC5", "true"));
				ALT_DEATH_PENALTY_C5_CHANCE = Integer.parseInt(altSettings.getProperty("DeathPenaltyC5Chance", "10"));
				ALT_DEATH_PENALTY_C5_CHAOTIC_RECOVERY = Boolean.parseBoolean(altSettings.getProperty("ChaoticCanUseScrollOfRecovery", "false"));
				ALT_DEATH_PENALTY_C5_EXPERIENCE_PENALTY = Integer.parseInt(altSettings.getProperty("DeathPenaltyC5RateExpPenalty", "1"));
				ALT_DEATH_PENALTY_C5_KARMA_PENALTY = Integer.parseInt(altSettings.getProperty("DeathPenaltyC5RateKarma", "1"));
				ALT_PK_DEATH_RATE = Float.parseFloat(altSettings.getProperty("AltPKDeathRate", "0."));
				NONOWNER_ITEM_PICKUP_DELAY = Integer.parseInt(altSettings.getProperty("NonOwnerItemPickupDelay", "15"));
				LIM_PATK_SPD = Integer.parseInt(altSettings.getProperty("LimitPatkSpd", "0"));
				LIM_MATK_SPD = Integer.parseInt(altSettings.getProperty("LimitMAtkSpd", "0"));
				LIM_CRIT = Integer.parseInt(altSettings.getProperty("LimitCritical", "0"));
				SAFE_ENCHANT_COMMON = Integer.parseInt(altSettings.getProperty("SafeEnchantCommon", "3"));
				SAFE_ENCHANT_FULL_BODY = Integer.parseInt(altSettings.getProperty("SafeEnchantFullBody", "4"));

				RIFT_MIN_PARTY_SIZE = Integer.parseInt(altSettings.getProperty("RiftMinPartySize", "5"));
				RIFT_MAX_JUMPS = Integer.parseInt(altSettings.getProperty("MaxRiftJumps", "4"));
				RIFT_AUTO_JUMPS_TIME = Integer.parseInt(altSettings.getProperty("AutoJumpsDelay", "8"));
				RIFT_AUTO_JUMPS_TIME_RAND = Integer.parseInt(altSettings.getProperty("AutoJumpsDelayRandom", "120000"));

				RIFT_ENTER_COST_RECRUIT = Integer.parseInt(altSettings.getProperty("RecruitFC", "18"));
				RIFT_ENTER_COST_SOLDIER = Integer.parseInt(altSettings.getProperty("SoldierFC", "21"));
				RIFT_ENTER_COST_OFFICER = Integer.parseInt(altSettings.getProperty("OfficerFC", "24"));
				RIFT_ENTER_COST_CAPTAIN = Integer.parseInt(altSettings.getProperty("CaptainFC", "27"));
				RIFT_ENTER_COST_COMMANDER = Integer.parseInt(altSettings.getProperty("CommanderFC", "30"));
				RIFT_ENTER_COST_HERO = Integer.parseInt(altSettings.getProperty("HeroFC", "33"));
				RIFT_BOSS_ROOM_TIME_MUTIPLY = Float.parseFloat(altSettings.getProperty("BossRoomTimeMultiply", "1.5"));
				ALLOW_CLANSKILLS = Boolean.parseBoolean(altSettings.getProperty("AllowClanSkills", "false"));
				PARTY_EXP_CALCULATIONS = Integer.parseInt(altSettings.getProperty("PartyExpCalculations", "1"));
				PARTY_LEADER_ONLY_CAN_INVITE = Boolean.parseBoolean(altSettings.getProperty("PartyLeaderOnlyCanInvite", "True"));
				ALLOW_TALK_WHILE_SITTING = Boolean.parseBoolean(altSettings.getProperty("AllowTalkWhileSitting", "True"));
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + ALT_SETTINGS_FILE + " File.");
			}

			// pvp config
			try
			{
				Properties pvpSettings = new Properties();
				InputStream is = new FileInputStream(new File(PVP_CONFIG_FILE));
				pvpSettings.load(is);
				is.close();

				/* KARMA SYSTEM */
				KARMA_MIN_KARMA = Integer.parseInt(pvpSettings.getProperty("MinKarma", "240"));
				KARMA_SP_DIVIDER = Integer.parseInt(pvpSettings.getProperty("SPDivider", "7"));
				KARMA_LOST_BASE = Integer.parseInt(pvpSettings.getProperty("BaseKarmaLost", "0"));

				KARMA_DROP_GM = Boolean.parseBoolean(pvpSettings.getProperty("CanGMDropEquipment", "false"));
				KARMA_AWARD_PK_KILL = Boolean.parseBoolean(pvpSettings.getProperty("AwardPKKillPVPPoint", "true"));

				KARMA_NONDROPPABLE_PET_ITEMS = pvpSettings.getProperty("ListOfPetItems", "3500,3501,3502,4422,4423,4424,2375");
				KARMA_NONDROPPABLE_ITEMS = pvpSettings.getProperty("ListOfNonDroppableItems", "57,1147,425,1146,461,10,2368,7,6,2370,2369");

				KARMA_DROP_EQUIPPED_ITEMS = Boolean.parseBoolean(pvpSettings.getProperty("DroppableEquipment", "true"));
				KARMA_DROP_EQUIPPED_WEAPONS = Boolean.parseBoolean(pvpSettings.getProperty("DroppableWeapon", "true"));

				KARMA_DROP_ITEM_LIMIT = Integer.parseInt(pvpSettings.getProperty("MaxItemsDroppable", "10"));

				KARMA_RANDOM_DROP_LOCATION_LIMIT = Integer.parseInt(pvpSettings.getProperty("MaxDropThrowDistance", "70"));

				KARMA_DROPCHANCE_MINIMUM = Integer.parseInt(pvpSettings.getProperty("ChanceOfDropMinimum", "1"));
				KARMA_DROPCHANCE_MULTIPLIER = Integer.parseInt(pvpSettings.getProperty("ChanceOfDropMultiplier", "1"));
				KARMA_DROPCHANCE_EQUIPMENT = Integer.parseInt(pvpSettings.getProperty("ChanceOfDropEquipped", "20"));
				KARMA_DROPCHANCE_EQUIPPED_WEAPON = Integer.parseInt(pvpSettings.getProperty("ChanceOfDropEquippedWeapon", "25"));

				KARMA_LIST_NONDROPPABLE_PET_ITEMS = new ArrayList<Integer>();
				for(String id : KARMA_NONDROPPABLE_PET_ITEMS.split(","))
					KARMA_LIST_NONDROPPABLE_PET_ITEMS.add(Integer.parseInt(id));

				KARMA_LIST_NONDROPPABLE_ITEMS = new ArrayList<Integer>();
				for(String id : KARMA_NONDROPPABLE_ITEMS.split(","))
					KARMA_LIST_NONDROPPABLE_ITEMS.add(Integer.parseInt(id));

				PVP_TIME = Integer.parseInt(pvpSettings.getProperty("PvPTime", "15000"));
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + PVP_CONFIG_FILE + " File.");
			}

			// AI
			try
			{
				Properties aiSettings = new Properties();
				InputStream is = new FileInputStream(new File(AI_CONFIG_FILE));
				aiSettings.load(is);
				is.close();

				MOBSLOOTERS = Boolean.parseBoolean(aiSettings.getProperty("MonstersLooters", "true"));

				RND_WALK = Boolean.valueOf(aiSettings.getProperty("RndWalk", "False"));
				RND_WALK_RATE = Integer.parseInt(aiSettings.getProperty("RndWalkRate", "2"));
				RND_ANIMATION_RATE = Integer.parseInt(aiSettings.getProperty("RndAnimationRate", "3"));
				ONE_FACTION_CALL_CHANCE = Integer.parseInt(aiSettings.getProperty("OneFactionCallChance", "75"));

				MAX_DRIFT_RANGE = Integer.parseInt(aiSettings.getProperty("MaxDriftRange", "300"));
				MAX_PURSUE_RANGE = Integer.parseInt(aiSettings.getProperty("MaxPursueRange", "10000"));

				FF_STUN_RATE = Integer.parseInt(aiSettings.getProperty("FF_STUN_RATE", "20"));
				FR_STUN_RATE = Integer.parseInt(aiSettings.getProperty("FR_STUN_RATE", "30"));
				FM_STUN_RATE = Integer.parseInt(aiSettings.getProperty("FM_STUN_RATE", "50"));
				FP_STUN_RATE = Integer.parseInt(aiSettings.getProperty("FP_STUN_RATE", "40"));
				FF_DEBUFF_RATE = Integer.parseInt(aiSettings.getProperty("FF_DEBUFF_RATE", "30"));
				FR_DEBUFF_RATE = Integer.parseInt(aiSettings.getProperty("FR_DEBUFF_RATE", "30"));
				FM_DEBUFF_RATE = Integer.parseInt(aiSettings.getProperty("FM_DEBUFF_RATE", "20"));
				FP_DEBUFF_RATE = Integer.parseInt(aiSettings.getProperty("FP_DEBUFF_RATE", "20"));
				FF_DOT_RATE = Integer.parseInt(aiSettings.getProperty("FF_DOT_RATE", "20"));
				FR_DOT_RATE = Integer.parseInt(aiSettings.getProperty("FR_DOT_RATE", "30"));
				FM_DOT_RATE = Integer.parseInt(aiSettings.getProperty("FM_DOT_RATE", "50"));
				FP_DOT_RATE = Integer.parseInt(aiSettings.getProperty("FP_DOT_RATE", "40"));
				FF_MDAM_RATE = Integer.parseInt(aiSettings.getProperty("FF_MDAM_RATE", "10"));
				FR_MDAM_RATE = Integer.parseInt(aiSettings.getProperty("FR_MDAM_RATE", "10"));
				FM_MDAM_RATE = Integer.parseInt(aiSettings.getProperty("FM_MDAM_RATE", "10"));
				FP_MDAM_RATE = Integer.parseInt(aiSettings.getProperty("FP_MDAM_RATE", "10"));
				FF_PDAM_RATE = Integer.parseInt(aiSettings.getProperty("FF_PDAM_RATE", "30"));
				FR_PDAM_RATE = Integer.parseInt(aiSettings.getProperty("FR_PDAM_RATE", "30"));
				FM_PDAM_RATE = Integer.parseInt(aiSettings.getProperty("FM_PDAM_RATE", "50"));
				FP_PDAM_RATE = Integer.parseInt(aiSettings.getProperty("FP_PDAM_RATE", "40"));
				FF_PHYS_RATE = Integer.parseInt(aiSettings.getProperty("FF_PHYS_RATE", "50"));
				FR_PHYS_RATE = Integer.parseInt(aiSettings.getProperty("FR_PHYS_RATE", "50"));
				FM_PHYS_RATE = Integer.parseInt(aiSettings.getProperty("FM_PHYS_RATE", "50"));
				FP_PHYS_RATE = Integer.parseInt(aiSettings.getProperty("FP_PHYS_RATE", "50"));

				RF_STUN_RATE = Integer.parseInt(aiSettings.getProperty("FF_STUN_RATE", "20"));
				RR_STUN_RATE = Integer.parseInt(aiSettings.getProperty("FR_STUN_RATE", "30"));
				RM_STUN_RATE = Integer.parseInt(aiSettings.getProperty("FM_STUN_RATE", "50"));
				RP_STUN_RATE = Integer.parseInt(aiSettings.getProperty("FP_STUN_RATE", "40"));
				RF_DEBUFF_RATE = Integer.parseInt(aiSettings.getProperty("FF_DEBUFF_RATE", "30"));
				RR_DEBUFF_RATE = Integer.parseInt(aiSettings.getProperty("FR_DEBUFF_RATE", "30"));
				RM_DEBUFF_RATE = Integer.parseInt(aiSettings.getProperty("FM_DEBUFF_RATE", "20"));
				RP_DEBUFF_RATE = Integer.parseInt(aiSettings.getProperty("FP_DEBUFF_RATE", "20"));
				RF_DOT_RATE = Integer.parseInt(aiSettings.getProperty("FF_DOT_RATE", "20"));
				RR_DOT_RATE = Integer.parseInt(aiSettings.getProperty("FR_DOT_RATE", "30"));
				RM_DOT_RATE = Integer.parseInt(aiSettings.getProperty("FM_DOT_RATE", "50"));
				RP_DOT_RATE = Integer.parseInt(aiSettings.getProperty("FP_DOT_RATE", "40"));
				RF_MDAM_RATE = Integer.parseInt(aiSettings.getProperty("FF_MDAM_RATE", "10"));
				RR_MDAM_RATE = Integer.parseInt(aiSettings.getProperty("FR_MDAM_RATE", "10"));
				RM_MDAM_RATE = Integer.parseInt(aiSettings.getProperty("FM_MDAM_RATE", "10"));
				RP_MDAM_RATE = Integer.parseInt(aiSettings.getProperty("FP_MDAM_RATE", "10"));
				RF_PDAM_RATE = Integer.parseInt(aiSettings.getProperty("FF_PDAM_RATE", "50"));
				RR_PDAM_RATE = Integer.parseInt(aiSettings.getProperty("FR_PDAM_RATE", "50"));
				RM_PDAM_RATE = Integer.parseInt(aiSettings.getProperty("FM_PDAM_RATE", "50"));
				RP_PDAM_RATE = Integer.parseInt(aiSettings.getProperty("FP_PDAM_RATE", "50"));
				RF_PHYS_RATE = Integer.parseInt(aiSettings.getProperty("FF_PHYS_RATE", "50"));
				RR_PHYS_RATE = Integer.parseInt(aiSettings.getProperty("FR_PHYS_RATE", "50"));
				RM_PHYS_RATE = Integer.parseInt(aiSettings.getProperty("FM_PHYS_RATE", "50"));
				RP_PHYS_RATE = Integer.parseInt(aiSettings.getProperty("FP_PHYS_RATE", "50"));

				MF_STUN_RATE = Integer.parseInt(aiSettings.getProperty("FF_STUN_RATE", "10"));
				MR_STUN_RATE = Integer.parseInt(aiSettings.getProperty("FR_STUN_RATE", "10"));
				MM_STUN_RATE = Integer.parseInt(aiSettings.getProperty("FM_STUN_RATE", "10"));
				MP_STUN_RATE = Integer.parseInt(aiSettings.getProperty("FP_STUN_RATE", "10"));
				MF_DEBUFF_RATE = Integer.parseInt(aiSettings.getProperty("FF_DEBUFF_RATE", "30"));
				MR_DEBUFF_RATE = Integer.parseInt(aiSettings.getProperty("FR_DEBUFF_RATE", "30"));
				MM_DEBUFF_RATE = Integer.parseInt(aiSettings.getProperty("FM_DEBUFF_RATE", "30"));
				MP_DEBUFF_RATE = Integer.parseInt(aiSettings.getProperty("FP_DEBUFF_RATE", "30"));
				MF_DOT_RATE = Integer.parseInt(aiSettings.getProperty("FF_DOT_RATE", "20"));
				MR_DOT_RATE = Integer.parseInt(aiSettings.getProperty("FR_DOT_RATE", "30"));
				MM_DOT_RATE = Integer.parseInt(aiSettings.getProperty("FM_DOT_RATE", "50"));
				MP_DOT_RATE = Integer.parseInt(aiSettings.getProperty("FP_DOT_RATE", "40"));
				MF_MDAM_RATE = Integer.parseInt(aiSettings.getProperty("FF_MDAM_RATE", "50"));
				MR_MDAM_RATE = Integer.parseInt(aiSettings.getProperty("FR_MDAM_RATE", "50"));
				MM_MDAM_RATE = Integer.parseInt(aiSettings.getProperty("FM_MDAM_RATE", "50"));
				MP_MDAM_RATE = Integer.parseInt(aiSettings.getProperty("FP_MDAM_RATE", "50"));
				MF_PDAM_RATE = Integer.parseInt(aiSettings.getProperty("FF_PDAM_RATE", "10"));
				MR_PDAM_RATE = Integer.parseInt(aiSettings.getProperty("FR_PDAM_RATE", "10"));
				MM_PDAM_RATE = Integer.parseInt(aiSettings.getProperty("FM_PDAM_RATE", "10"));
				MP_PDAM_RATE = Integer.parseInt(aiSettings.getProperty("FP_PDAM_RATE", "10"));
				MF_PHYS_RATE = Integer.parseInt(aiSettings.getProperty("FF_PHYS_RATE", "0"));
				MR_PHYS_RATE = Integer.parseInt(aiSettings.getProperty("FR_PHYS_RATE", "0"));
				MM_PHYS_RATE = Integer.parseInt(aiSettings.getProperty("FM_PHYS_RATE", "0"));
				MP_PHYS_RATE = Integer.parseInt(aiSettings.getProperty("FP_PHYS_RATE", "0"));

				PF_STUN_RATE = Integer.parseInt(aiSettings.getProperty("FF_STUN_RATE", "10"));
				PR_STUN_RATE = Integer.parseInt(aiSettings.getProperty("FR_STUN_RATE", "10"));
				PM_STUN_RATE = Integer.parseInt(aiSettings.getProperty("FM_STUN_RATE", "20"));
				PP_STUN_RATE = Integer.parseInt(aiSettings.getProperty("FP_STUN_RATE", "20"));
				PF_DEBUFF_RATE = Integer.parseInt(aiSettings.getProperty("FF_DEBUFF_RATE", "40"));
				PR_DEBUFF_RATE = Integer.parseInt(aiSettings.getProperty("FR_DEBUFF_RATE", "40"));
				PM_DEBUFF_RATE = Integer.parseInt(aiSettings.getProperty("FM_DEBUFF_RATE", "40"));
				PP_DEBUFF_RATE = Integer.parseInt(aiSettings.getProperty("FP_DEBUFF_RATE", "40"));
				PF_DOT_RATE = Integer.parseInt(aiSettings.getProperty("FF_DOT_RATE", "20"));
				PR_DOT_RATE = Integer.parseInt(aiSettings.getProperty("FR_DOT_RATE", "30"));
				PM_DOT_RATE = Integer.parseInt(aiSettings.getProperty("FM_DOT_RATE", "40"));
				PP_DOT_RATE = Integer.parseInt(aiSettings.getProperty("FP_DOT_RATE", "50"));
				PF_MDAM_RATE = Integer.parseInt(aiSettings.getProperty("FF_MDAM_RATE", "40"));
				PR_MDAM_RATE = Integer.parseInt(aiSettings.getProperty("FR_MDAM_RATE", "30"));
				PM_MDAM_RATE = Integer.parseInt(aiSettings.getProperty("FM_MDAM_RATE", "30"));
				PP_MDAM_RATE = Integer.parseInt(aiSettings.getProperty("FP_MDAM_RATE", "30"));
				PF_PDAM_RATE = Integer.parseInt(aiSettings.getProperty("FF_PDAM_RATE", "20"));
				PR_PDAM_RATE = Integer.parseInt(aiSettings.getProperty("FR_PDAM_RATE", "30"));
				PM_PDAM_RATE = Integer.parseInt(aiSettings.getProperty("FM_PDAM_RATE", "50"));
				PP_PDAM_RATE = Integer.parseInt(aiSettings.getProperty("FP_PDAM_RATE", "40"));
				PF_PHYS_RATE = Integer.parseInt(aiSettings.getProperty("FF_PHYS_RATE", "0"));
				PR_PHYS_RATE = Integer.parseInt(aiSettings.getProperty("FR_PHYS_RATE", "0"));
				PM_PHYS_RATE = Integer.parseInt(aiSettings.getProperty("FM_PHYS_RATE", "0"));
				PP_PHYS_RATE = Integer.parseInt(aiSettings.getProperty("FP_PHYS_RATE", "0"));
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + AI_CONFIG_FILE + " File.");
			}

			// siege options
			try
			{
				Properties siegeSettings = new Properties();
				InputStream is = new FileInputStream(new File(SIEGE_CONFIGURATION_FILE));
				siegeSettings.load(is);
				is.close();

				SIEGE_OPERATE_DOORS = Boolean.parseBoolean(siegeSettings.getProperty("OperateDoors", "True"));
				SIEGE_OPERATE_DOORS_LORD_ONLY = Boolean.parseBoolean(siegeSettings.getProperty("OperateDoorsLordOnly", "True"));

			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + SIEGE_CONFIGURATION_FILE + " File.");
			}

			// hexid
			try
			{
				Properties Settings = new Properties();
				InputStream is = new FileInputStream(HEXID_FILE);
				Settings.load(is);
				is.close();
				HEX_ID = new BigInteger(Settings.getProperty("HexID"), 16).toByteArray();
			}
			catch(Exception e)
			{

			}

			// Events config
			try
			{
				Properties spoilSettings = new Properties();
				InputStream is = new FileInputStream(new File(EVENTS));
				spoilSettings.load(is);
				is.close();

				EVENT_Min_lvl = Integer.parseInt(spoilSettings.getProperty("Min_lvl", "1"));
				EVENT_Max_lvl = Integer.parseInt(spoilSettings.getProperty("Man_lvl", "80"));
				EVENT_dp = Integer.parseInt(spoilSettings.getProperty("dp", "5000"));
				EVENT_dpay = Integer.parseInt(spoilSettings.getProperty("dpay", "57"));
				EVENT_type_buffer = Integer.parseInt(spoilSettings.getProperty("type_buffer", "1"));
				EVENT_Buffer_Siege = Boolean.parseBoolean(spoilSettings.getProperty("Buffer_Siege", "False"));

				EVENT_CofferOfShadowsPriceRate = Integer.parseInt(spoilSettings.getProperty("CofferOfShadowsPriceRate", "1"));
				EVENT_RenameItemID = Integer.parseInt(spoilSettings.getProperty("RENAME_ITEM", "57"));
				EVENT_RenamePrice = Integer.parseInt(spoilSettings.getProperty("RENAME_PRICE", "500000"));
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + EVENTS + " File.");
			}

			abuseLoad();
			loadGMAccess();
			_log.info("loading xml GMAccess");
			if(ADVIPSYSTEM)
				ipsLoad();
		}
		else if(Server.SERVER_MODE == Server.MODE_LOGINSERVER)
		{
			_log.info("loading login config");
			try
			{
				Properties serverSettings = new Properties();
				InputStream is = new FileInputStream(new File(LOGIN_CONFIGURATION_FILE));
				serverSettings.load(is);
				is.close();

				LOGIN_HOST = serverSettings.getProperty("LoginserverHostname", "127.0.0.1");
				GAME_SERVER_LOGIN_PORT = Integer.parseInt(serverSettings.getProperty("LoginPort", "9013"));
				GAME_SERVER_LOGIN_HOST = serverSettings.getProperty("LoginHost", "127.0.0.1");
				PORT_LOGIN = Integer.parseInt(serverSettings.getProperty("LoginserverPort", "2106"));

				COMBO_MODE = Boolean.parseBoolean(serverSettings.getProperty("ComboMode", "false"));
				LOGIN_DEBUG = Boolean.parseBoolean(serverSettings.getProperty("Debug", "false"));

				ACCEPT_NEW_GAMESERVER = Boolean.parseBoolean(serverSettings.getProperty("AcceptNewGameServer", "True"));

				LOGIN_TRY_BEFORE_BAN = Integer.parseInt(serverSettings.getProperty("LoginTryBeforeBan", "10"));
				LOGIN_PING = Boolean.parseBoolean(serverSettings.getProperty("PingServer", "True"));
				LOGIN_PING_TIME = Integer.parseInt(serverSettings.getProperty("WaitPingTime", "5"));

				DATABASE_DRIVER = serverSettings.getProperty("Driver", "com.mysql.jdbc.Driver");
				DATABASE_URL = serverSettings.getProperty("URL", "jdbc:mysql://localhost/l2rdb");
				DATABASE_LOGIN = serverSettings.getProperty("Login", "root");
				DATABASE_PASSWORD = serverSettings.getProperty("Password", "");
				DATABASE_MAX_CONNECTIONS = Integer.parseInt(serverSettings.getProperty("MaximumDbConnections", "10"));
				DATABASE_MAX_IDLE_TIMEOUT = Integer.parseInt(serverSettings.getProperty("MaxIdleConnectionTimeout", "600"));
                DATABASE_IDLE_TEST_PERIOD = Integer.parseInt(serverSettings.getProperty("IdleConnectionTestPeriod", "60"));

				SHOW_LICENCE = Boolean.parseBoolean(serverSettings.getProperty("ShowLicence", "true"));
				SQL_LOG = Boolean.parseBoolean(serverSettings.getProperty("SqlLog", "false"));
				AUTO_CREATE_ACCOUNTS = Boolean.parseBoolean(serverSettings.getProperty("AutoCreateAccounts", "true"));
				ALLOW_OLD_AUTH = Boolean.parseBoolean(serverSettings.getProperty("AllowOldAuth", "false"));
				IP_UPDATE_TIME = Integer.parseInt(serverSettings.getProperty("IpUpdateTime", "15"));
				FLOOD_PROTECTION = Boolean.parseBoolean(serverSettings.getProperty("EnableFloodProtection", "True"));
				FAST_CONNECTION_LIMIT = Integer.parseInt(serverSettings.getProperty("FastConnectionLimit", "15"));
				NORMAL_CONNECTION_TIME = Integer.parseInt(serverSettings.getProperty("NormalConnectionTime", "700"));
				FAST_CONNECTION_TIME = Integer.parseInt(serverSettings.getProperty("FastConnectionTime", "350"));
				MAX_CONNECTION_PER_IP = Integer.parseInt(serverSettings.getProperty("MaxConnectionPerIP", "50"));
				ANAME_TEMPLATE = serverSettings.getProperty("AnameTemplate", "[A-Za-z0-9]{3,14}");
				APASSWD_TEMPLATE = serverSettings.getProperty("ApasswdTemplate", "[A-Za-z0-9]{5,16}");
				SESSIONKEY_IGNORE = Boolean.parseBoolean(serverSettings.getProperty("IgnoreSessionKey", "false"));
				internalIpList = serverSettings.getProperty("InternalIpList", "127.0.0.1,192.168.0.0-192.168.255.255,10.0.0.0-10.255.255.255,172.16.0.0-172.16.31.255");
				LOGIN_GG_CHECK = Boolean.parseBoolean(serverSettings.getProperty("GGCheck", "True"));
				LRESTART_TIME = Integer.parseInt(serverSettings.getProperty("AutoRestart", "-1"));

				for(String s : internalIpList.split(","))
					INTERNAL_IP.add(s);
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + CONFIGURATION_FILE + " File.");
			}

			//	      telnet
			try
			{
				Properties telnetSettings = new Properties();
				InputStream is = new FileInputStream(new File(LOGIN_TELNET_FILE));
				telnetSettings.load(is);
				is.close();

				IS_LOGIN_TELNET_ENABLED = Boolean.valueOf(telnetSettings.getProperty("EnableTelnet", "false"));
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new Error("Failed to Load " + TELNET_FILE + " File.");
			}
		}
		else
			_log.severe("Could not Load Config: server mode was not set");

	}

	public static boolean allowL2Walker(L2Player player)
	{
		return ALLOW_L2WALKER_CLIENT == L2WalkerAllowed.True || player.getPlayerAccess().AllowWalker || ALLOW_L2WALKER_CLIENT == L2WalkerAllowed.Peace && player.isInZone(peace_zone) && player.getLevel() <= MAX_L2WALKER_LEVEL;
	}

	// it has no instancies
	private Config()
	{}

	/**
	 * @param string
	 */
	public static void saveHexid(String string)
	{
		saveHexid(string, HEXID_FILE);
	}

	/**
	 * @param string
	 */
	public static void saveHexid(String string, String fileName)
	{
		try
		{
			Properties hexSetting = new Properties();
			File file = new File(fileName);
			file.createNewFile();
			OutputStream out = new FileOutputStream(file);
			hexSetting.setProperty("HexID", string);
			hexSetting.store(out, "the hexID to auth into login");
			out.close();
		}
		catch(Exception e)
		{
			System.out.println("Failed to save hex id to " + fileName + " File.");
			e.printStackTrace();
		}
	}

	public final static void abuseLoad()
	{
		MAT_LIST = new ArrayList<String>();

		LineNumberReader lnr = null;
		try
		{
			int i = 0;
			String line = null;

			lnr = new LineNumberReader(new InputStreamReader(new FileInputStream(Server.RebellionHomeDir + MAT_CONFIG_FILE), "UTF-8"));

			while((line = lnr.readLine()) != null)
			{
				StringTokenizer st = new StringTokenizer(line, "\n\r");
				if(st.hasMoreTokens())
				{
					String Mat = st.nextToken();
					MAT_LIST.add(Mat);

					i++;
				}
			}

			_log.info("Abuse: Loaded " + i + " abuse words.");

		}
		catch(IOException e1)
		{
			_log.warning("Error reading abuse: " + e1);
		}
		finally
		{
			try
			{
				if(lnr != null)
					lnr.close();
			}
			catch(Exception e2)
			{
				// nothing
			}
		}
	}

	private final static void ipsLoad()
	{
		try
		{
			Properties ipsSettings = new Properties();
			InputStream is = new FileInputStream(new File(ADV_IP_FILE));
			ipsSettings.load(is);
			is.close();

			String NetMask;
			String ip;
			for(int i = 0; i < ipsSettings.size() / 2; i++)
			{
				NetMask = ipsSettings.getProperty("NetMask" + (i + 1));
				ip = ipsSettings.getProperty("IPAdress" + (i + 1));
				for(String mask : NetMask.split(","))
				{
					AuthRequest ar = new AuthRequest();
					AdvIP advip = ar.new AdvIP();
					advip.ipadress = ip;
					advip.ipmask = mask.split("/")[0];
					advip.bitmask = mask.split("/")[1];
					GAMEIPS.add(advip);
				}
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new Error("Failed to Load " + ADV_IP_FILE + " File.");
		}
	}

	public final static void loadGMAccess()
	{
		gmlist.clear();
		loadGMAccess(new File(GM_PERSONAL_ACCESS_FILE));
		File dir = new File("./config/GMAccess.d/");
		if(!dir.exists())
		{
			_log.config("Dir " + dir.getAbsolutePath() + " not exists");
			return;
		}
		File[] files = dir.listFiles();
		for(File f : files)
			if(f.getName().endsWith(".xml"))
				loadGMAccess(f);
	}

	@SuppressWarnings("unchecked")
	public final static void loadGMAccess(File file)
	{
		try
		{
			Field fld = null;
			//File file = new File(filename);
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			factory.setValidating(false);
			factory.setIgnoringComments(true);
			Document doc = factory.newDocumentBuilder().parse(file);

			for(Node z = doc.getFirstChild(); z != null; z = z.getNextSibling())
				for(Node n = z.getFirstChild(); n != null; n = n.getNextSibling())
				{
					if(!n.getNodeName().equalsIgnoreCase("char"))
						continue;

					PlayerAccess pa = new PlayerAccess();
					for(Node d = n.getFirstChild(); d != null; d = d.getNextSibling())
					{
						Class cls = pa.getClass();
						String node = d.getNodeName();

						if(node.equalsIgnoreCase("#text"))
							continue;
						try
						{
							fld = cls.getField(node);
						}
						catch(NoSuchFieldException e)
						{
							_log.info("Not found desclarate ACCESS name: " + node + " in XML Player access Object");
							continue;
						}

						if(fld.getType().getName().equalsIgnoreCase("boolean"))
							fld.setBoolean(pa, Boolean.parseBoolean(d.getAttributes().getNamedItem("set").getNodeValue()));
						else if(fld.getType().getName().equalsIgnoreCase("int"))
							fld.setInt(pa, Integer.valueOf(d.getAttributes().getNamedItem("set").getNodeValue()));
					}
					gmlist.put(pa.PlayerID, pa);
				}
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new Error("Failed to Load " + GM_PERSONAL_ACCESS_FILE + " File.");
		}
	}
}
