package L2jFT.extensions.scripts;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ScheduledFuture;

import L2jFT.Config;
import L2jFT.extensions.multilang.CustomMessage;
import L2jFT.game.ThreadPoolManager;
import L2jFT.game.model.Inventory;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Playable;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.model.instances.L2PetInstance;
import L2jFT.game.network.serverpackets.InventoryUpdate;
import L2jFT.game.network.serverpackets.NpcHtmlMessage;
import L2jFT.game.network.serverpackets.NpcSay;
import L2jFT.game.network.serverpackets.PetItemList;
import L2jFT.game.network.serverpackets.Ride;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.ItemTable;

/**
 * @Author: Diamond
 * @Date: 7/6/2007
 * @Time: 5:22:23
 */
public class Functions
{
	public static L2Object self;
	public static L2NpcInstance npc;

	/**
	 * Вызывает метод с задержкой
	 * @param object - от чьего имени вызывать
	 * @param sClass - вызываемый класс
	 * @param sMethod - вызываемый метод
	 * @param args - массив аргуметов
	 * @param variables - список выставляемых переменных
	 * @param delay - задержка в миллисекундах
	 */
	@SuppressWarnings("unchecked")
	public static ScheduledFuture executeTask(final L2Object object, final String sClass, final String sMethod, final Object[] args, final HashMap<String, Object> variables, Integer delay)
	{
		return ThreadPoolManager.getInstance().scheduleGeneral(new Runnable(){
			public void run()
			{
				if(object != null)
					object.callScripts(sClass, sMethod, args, variables);
			}
		}, delay);
	}

	@SuppressWarnings("unchecked")
	public static ScheduledFuture executeTask(final String sClass, final String sMethod, final Object[] args, final HashMap<String, Object> variables, Integer delay)
	{
		return ThreadPoolManager.getInstance().scheduleGeneral(new Runnable(){
			public void run()
			{
				callScripts(sClass, sMethod, args, variables);
			}
		}, delay);
	}

	@SuppressWarnings("unchecked")
	public static ScheduledFuture executeTask(final L2Object object, final String sClass, final String sMethod, final Object[] args, Integer delay)
	{
		return executeTask(object, sClass, sMethod, args, null, delay);
	}

	@SuppressWarnings("unchecked")
	public static ScheduledFuture executeTask(final String sClass, final String sMethod, final Object[] args, Integer delay)
	{
		return executeTask(sClass, sMethod, args, null, delay);
	}

	public static void show(String text, L2Player self)
	{
		if(text == null || self == null)
			return;
		NpcHtmlMessage msg = new NpcHtmlMessage(self.getLastNpc() != null ? self.getLastNpc().getObjectId() : 5);

		// Не указываем явно язык
		if(text.endsWith(".html-ru") || text.endsWith(".htm-ru"))
			text = text.substring(0, text.length() - 3);

		// приводим нашу html-ку в нужный вид
		if(text.endsWith(".html") || text.endsWith(".htm"))
			msg.setFile(text);
		else
			msg.setHtml(text);
		self.sendPacket(msg);
	}

	public static void show(CustomMessage message, L2Player self)
	{
		show(message.toString(), self);
	}

	public static void sendMessage(String text, L2Player self)
	{
		self.sendMessage(text);
	}

	public static void sendMessage(CustomMessage message, L2Player self)
	{
		self.sendMessage(message);
	}

	public static void npcSayInRange(L2NpcInstance npc, String text, int range)
	{
		NpcSay cs;
		if(npc == null)
			return;
		cs = new NpcSay(npc, 0, text);
		for(L2Player player : npc.getAroundPlayers(range))
			if(player != null)
				player.sendPacket(cs);
	}

	public static void npcSayToAll(L2NpcInstance npc, String text)
	{
		NpcSay cs;
		if(npc == null)
			return;
		cs = new NpcSay(npc, 1, text);
		if(Config.GLOBAL_CHAT < 81)
		{
			for(L2Player player : L2World.getAllPlayers())
				if(player != null)
					player.sendPacket(cs);
		}
		else
			for(L2Player player : L2World.getAroundPlayers(npc))
				if(player != null)
					player.sendPacket(cs);
	}

	public static void npcSayToPlayer(L2NpcInstance npc, L2Player player, String text)
	{
		NpcSay cs;
		if(npc == null)
			return;
		cs = new NpcSay(npc, 0, text);
		player.sendPacket(cs);
	}

	public static void npcShout(L2NpcInstance npc, String text)
	{
		NpcSay cs;
		if(npc == null)
			return;
		cs = new NpcSay(npc, 0, text);
		for(L2Player player : L2World.getAroundPlayers(npc))
			if(player != null)
				player.sendPacket(cs);
	}

	public static void npcShoutCustomMessage(L2NpcInstance npc, String address, String[] replacements)
	{
		if(npc == null)
			return;

		for(L2Player player : L2World.getAroundPlayers(npc))
			if(player != null)
			{
				CustomMessage cm = new CustomMessage(address, player);

				if(replacements != null)
					for(String s : replacements)
						cm.addString(s);

				player.sendPacket(new NpcSay(npc, 0, cm.toString()));
			}
	}

	/**
	 * Добавляет предмет в инвентарь чара
	 * @param playable Владелец инвентаря
	 * @param item_id ID предмета
	 * @param count количество
	 */
	public static void addItem(L2Playable playable, int item_id, long count)
	{
		L2Playable player;
		if(playable.isSummon)
			player = playable.getOwner();
		else
			player = playable;
		if(count <= 0)
			return;
		L2ItemInstance item = ItemTable.getInstance().createItem(item_id, player.getObjectId(), 0, "Scripts.addItem");
		InventoryUpdate iu = new InventoryUpdate();
		if(!item.isStackable())
			for(int i = 0; i < count; i++)
				iu.addNewItem(player.getInventory().addItem(ItemTable.getInstance().createItem(item_id, player.getObjectId(), 0, "Scripts.addItem")));
		else
		{
			item.setCount(item.getCount() + count - 1);
			iu.addItem(player.getInventory().addItem(item));
		}
		if(player instanceof L2PetInstance)
			player.sendPacket(new PetItemList((L2PetInstance) player));
		else
			player.sendPacket(iu);
		if(item_id == 57)
			player.sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_OBTAINED_S1_ADENA).addNumber((int) count));
		else if(count > 1)
			player.sendPacket(new SystemMessage(SystemMessage.EARNED_S2_S1_s).addItemName(item_id).addNumber((int) count));
		else
			player.sendPacket(new SystemMessage(SystemMessage.EARNED_S1).addItemName(item_id));
	}

	/**
	 * Возвращает количество предметов в инвентаре чара.
	 * @param playable Владелец инвентаря
	 * @param item_id ID предмета
	 * @return количество
	 */
	public static int getItemCount(L2Playable playable, int item_id)
	{
		int count = 0;
		L2Playable player;
		if(playable.isSummon)
			player = playable.getOwner();
		else
			player = playable;
		Inventory inv = player.getInventory();
		if(inv == null)
			return 0;
		L2ItemInstance[] items = inv.getItems();
		synchronized (items)
		{
			for(L2ItemInstance item : items)
				if(item.getItemId() == item_id)
					count += item.getCount();
		}
		return count;
	}

	/**
	 * Удаляет предметы из инвентаря чара.
	 * @param playable Владелец инвентаря
	 * @param item_id ID предмета
	 * @param count количество
	 */
	public static void removeItem(L2Playable playable, int item_id, int count)
	{
		L2Playable player;
		if(playable.isSummon)
			player = playable.getOwner();
		else
			player = playable;
		if(count <= 0)
			return;
		Inventory inv = player.getInventory();
		if(inv == null)
			return;
		int removed = count;
		L2ItemInstance[] items = inv.getItems();
		InventoryUpdate iu = new InventoryUpdate();
		synchronized (items)
		{
			for(L2ItemInstance item : items)
				if(item.getItemId() == item_id && count > 0)
				{
					int item_count = item.getCount();
					long rem = count <= item_count ? count : item_count;
					iu.addItem(player.getInventory().destroyItemByItemId(item_id, rem, false, true));
					count -= rem;
				}
		}
		if(player.isPet)
			player.sendPacket(new PetItemList((L2PetInstance) player));
		else
			player.sendPacket(iu);
		removed -= count;
		if(item_id == 57)
			player.sendPacket(new SystemMessage(SystemMessage.S1_ADENA_DISAPPEARED).addNumber(removed));
		else if(removed > 1)
			player.sendPacket(new SystemMessage(SystemMessage.S2_S1_HAS_DISAPPEARED).addItemName(item_id).addNumber(removed));
		else
			player.sendPacket(new SystemMessage(SystemMessage.S1_HAS_DISAPPEARED).addItemName(item_id));
	}

	public static boolean ride(L2Player player, Integer pet)
	{
		if(player.isMounted())
		{
			player.broadcastPacket(new Ride(player, Ride.ACTION_DISMOUNT, 0));
			player.setMountType((short) 0);
		}

		if(player.getPet() != null)
		{
			player.sendPacket(new SystemMessage(SystemMessage.YOU_ALREADY_HAVE_A_PET));
			return false;
		}

		Ride mount = new Ride(player, Ride.ACTION_MOUNT, pet);
		player.broadcastPacket(mount);
		player.setMountType(mount.getMountType());
		return true;
	}

	public static void unRide(L2Player player)
	{
		player.broadcastPacket(new Ride(player, Ride.ACTION_DISMOUNT, 0));
		player.setMountType((short) 0);
	}

	public static Object callScripts(String _class, String method, Object[] args, HashMap<String, Object> variables)
	{
		if(L2jFT.extensions.scripts.Scripts.loading)
			return null;

		ScriptObject o;

		Script scriptClass = Scripts.getInstance().getClasses().get(_class);

		if(scriptClass == null)
			return null;

		try
		{
			o = scriptClass.newInstance();
		}
		catch(Exception e)
		{
			e.printStackTrace();
			return null;
		}

		if(variables != null)
			for(Map.Entry<String, Object> obj : variables.entrySet())
				try
				{
					o.setProperty(obj.getKey(), obj.getValue());
				}
				catch(Exception e)
				{}

		return o.invokeMethod(method, args);
	}
}