package L2jFT.extensions.scripts;

import com.sun.tools.javac.Main;
import L2jFT.Config;
import L2jFT.extensions.scripts.jarloader.JarClassLoader;
import L2jFT.game.GameServer;
import L2jFT.game.handler.AdminCommandHandler;
import L2jFT.game.instancemanager.QuestManager;
import L2jFT.game.model.quest.Quest;

import java.io.File;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.TreeMap;
import java.util.logging.Logger;

public class Scripts
{
	private static final Logger _log = Logger.getLogger(Scripts.class.getName());

	public static boolean JAR = false;

	private static Scripts _instance;
	private HashMap<String, Script> _classes = new HashMap<String, Script>();
	public static HashMap<Integer, ArrayList<String>> itemHandlers = new HashMap<Integer, ArrayList<String>>();
	public static HashMap<Integer, ArrayList<String>> dialogAppends = new HashMap<Integer, ArrayList<String>>();
	public static HashMap<String, String> onAction = new HashMap<String, String>();
	public static HashMap<String, String> onActionShift = new HashMap<String, String>();
	public static ArrayList<String> onPlayerExit = new ArrayList<String>();
	public static ArrayList<String> onPlayerEnter = new ArrayList<String>();
	private boolean error;
	private ArrayList<String> _scriptNames;
	private ArrayList<String> _scriptDirs;

	public static boolean loading;

	public static Scripts getInstance()
	{
		if(_instance == null)
			new Scripts();
		return _instance;
	}

	public Scripts()
	{
		_instance = this;
		load(false);
	}

	public boolean reload()
	{
		loading = true;

		for(ScriptObject go : GameServer.scriptsObjects.values())
			try
			{
				go.invokeMethod("onReload");
			}
			catch(Exception f)
			{
				f.printStackTrace();
			}

		//GameServer.scriptsObjects.clear();
		//_classes.clear();

		load(true);
		callOnLoad();

		loading = false;
		return error;
	}

	public void shutdown()
	{
		for(ScriptObject go : GameServer.scriptsObjects.values())
			try
			{
				go.invokeMethod("onShutdown");
			}
			catch(Exception f)
			{
				f.printStackTrace();
			}
	}

	private void load(boolean reload)
	{
		_log.info("Scripts loading...");
		error = false;
		Class c;

		JarClassLoader jcl;
		if(JAR)
		{
			try
			{
				jcl = new JarClassLoader("./scripts.jar");
				for(String name : jcl.getClassNames())
				{
					name = name.replace(".class","");
					c = jcl.loadClass(name);
					Script s = new Script(c);
					_classes.put(c.getName(), s);
				}
			}
			catch(Exception e)
			{
				error = true;
				e.printStackTrace();
			}
		}
		else
		{
			_scriptNames = new ArrayList<String>();
			_scriptDirs = new ArrayList<String>();

			parseClasses(new File("./data/scripts"), "");

			String[] files = new String[_scriptNames.size() + 2];
			files[0] = "-encoding";
			files[1] = "utf8";
			for(int i = 0; i < _scriptNames.size(); i++)
			{
				files[i + 2] = "./data/scripts/" + _scriptDirs.get(i) + _scriptNames.get(i);
				File f = new File(files[i + 2].replace(".java", ".class"));
				if(!reload && f.exists())
					f.delete();
			}

			try
			{
				Main.compile(files);
			}
			catch(Exception e)
			{
				error = true;
				e.printStackTrace();
			}

			URLClassLoader ucl;
			try
			{
				ucl = new URLClassLoader(new URL[] { new File("./data/scripts/").toURI().toURL() });
				for(int i = 0; i < _scriptNames.size(); i++)
				{
					String filename = _scriptNames.get(i).replace(".java", "");
					c = ucl.loadClass((_scriptDirs.get(i) + filename).replace("/", "."));
					Script s = new Script(c);
					_classes.put(c.getName(), s);
				}
			}
			catch(ClassNotFoundException e)
			{
				_log.warning("Can't load script class:" + e.getMessage());
			}
			catch(Exception e)
			{
				error = true;
				e.printStackTrace();
			}
		}

		if(error)
		{
			_log.info("Scripts loaded with errors. Loaded " + _classes.size() + " classes.");
			if(!reload)
				Runtime.getRuntime().halt(0);
		}
		else
			_log.info("Scripts successfully loaded. Loaded " + _classes.size() + " classes.");
	}

	public void callOnLoad()
	{
		itemHandlers.clear();
		dialogAppends.clear();

		TreeMap<String, Script> temp = new TreeMap<String, Script>(_classes);

		for(Script _class : temp.values())
			try
			{
				ScriptObject go = _class.newInstance();
				GameServer.scriptsObjects.put(_class.getName(), go);
				go.invokeMethod("onLoad");

				for(Method method : go.getRawClass().getMethods())
					if(method.getName().contains("ItemHandler_"))
					{
						Integer id = Integer.parseInt(method.getName().substring(12));
						ArrayList<String> handler = itemHandlers.get(id);
						if(handler == null)
							handler = new ArrayList<String>();
						handler.add(_class.getName() + ":" + method.getName());
						itemHandlers.put(id, handler);
					}

				for(Method method : go.getRawClass().getMethods())
					if(method.getName().contains("DialogAppend_"))
					{
						Integer id = Integer.parseInt(method.getName().substring(13));
						ArrayList<String> handler = dialogAppends.get(id);
						if(handler == null)
							handler = new ArrayList<String>();
						handler.add(_class.getName() + ":" + method.getName());
						dialogAppends.put(id, handler);
					}

				for(Method method : go.getRawClass().getMethods())
					if(method.getName().contains("OnAction_"))
						onAction.put(method.getName().substring(9), _class.getName() + ":" + method.getName());

				for(Method method : go.getRawClass().getMethods())
					if(method.getName().contains("OnActionShift_"))
						onActionShift.put(method.getName().substring(14), _class.getName() + ":" + method.getName());

				for(Method method : go.getRawClass().getMethods())
					if(method.getName().equals("OnPlayerExit"))
						onPlayerExit.add(_class.getName() + ":" + method.getName());
				
				for(Method method : go.getRawClass().getMethods())
					if(method.getName().equals("OnPlayerEnter"))
						onPlayerEnter.add(_class.getName() + ":" + method.getName());
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}

		AdminCommandHandler.getInstance().registerAdminCommandHandler(new AdminScripts());
	}

	public boolean reloadClass(String name)
	{
		if(name.contains("."))
			return reloadClassByName(name);
		return reloadClassByPath(name);
	}

	public boolean reloadClassByPath(String path)
	{
		boolean file = new File("./data/scripts/" + path + ".java").isFile();
		String directoryPath = "./data/scripts/" + path;
		boolean directory = path.endsWith("/") || new File(directoryPath).isDirectory();
		ArrayList<String> classNames = new ArrayList<String>();
		if(file)
			return reloadClassByName(path.replace("/", "."));
		if(directory)
		{
			File[] files = new File(directoryPath).listFiles();
			if(files == null)
			{
				_log.warning("Could't load files from directory " + directoryPath);
				return true;
			}
			for(File f : files)
				if(f.isFile() && f.getName().endsWith(".java"))
					classNames.add("./data/scripts/" + path + "/" + f.getName());
			return reloadClasses(classNames);
		}
		return true;
	}

	public boolean reloadClassByName(String name)
	{

		ScriptObject so = GameServer.scriptsObjects.get(name);
		if(so != null)
			so.invokeMethod("onReload");
		String[] files = new String[3];
		files[0] = "-encoding";
		files[1] = "utf8";
		files[2] = "./data/scripts/" + name.replace(".", "/") + ".java";
		try
		{
			Main.compile(files);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		try
		{
			URLClassLoader ucl = new URLClassLoader(new URL[] { new File("./data/scripts/").toURI().toURL() });
			Class c = ucl.loadClass(name);
			Script s = new Script(c);
			_classes.put(c.getName(), s);
			so = s.newInstance();
			GameServer.scriptsObjects.put(s.getName(), so);
			so.invokeMethod("onLoad");
		}
		catch(MalformedURLException e)
		{}
		catch(ClassNotFoundException e)
		{}
		return error;
	}

	public boolean reloadQuest(String name)
	{
		if(Config.DONTLOADQUEST)
			return true;
		Quest q = QuestManager.getInstance().getQuest(name);
		if(q != null)
		{
			String path = q.getClass().getPackage().getName().replace(".", "/");
			if(new File("./data/scripts/" + path + "/").isDirectory())
				return reloadClassByPath(path);
		}
		q = QuestManager.getInstance().getQuest(Integer.parseInt(name));
		if(q != null)
		{
			String path = q.getClass().getPackage().getName().replace(".", "/");
			if(new File("./data/scripts/" + path + "/").isDirectory())
				return reloadClassByPath(path);
		}
		return reloadClassByPath("quests/" + name + "/");
	}

	private boolean reloadClasses(ArrayList<String> classes)
	{
		for(String cl : classes)
		{
			ScriptObject so = GameServer.scriptsObjects.get(cl);
			if(so != null)
				so.invokeMethod("onReload");
		}
		String[] files = new String[classes.size() + 2];
		files[0] = "-encoding";
		files[1] = "utf8";
		int i = 2;
		for(String c : classes)
		{
			files[i] = c;
			i++;
		}
		try
		{
			Main.compile(files);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		ArrayList<ScriptObject> temp = new ArrayList<ScriptObject>();
		try
		{
			for(String cl : classes)
			{
				String name = cl.replace(".java", "").replace("./data/scripts/", "").replace("/", ".");
				URLClassLoader ucl = new URLClassLoader(new URL[] { new File("./data/scripts/").toURI().toURL() });
				Class c = ucl.loadClass(name);
				Script s = new Script(c);
				_classes.put(c.getName(), s);
				ScriptObject so = s.newInstance();
				temp.add(so);
				GameServer.scriptsObjects.put(s.getName(), so);
			}
		}
		catch(MalformedURLException e)
		{}
		catch(ClassNotFoundException e)
		{
			_log.warning("Can't load script class:" + e.getMessage());
			error = true;
		}

		for(ScriptObject so : temp)
			so.invokeMethod("onLoad");
		return error;
	}

	private void parseClasses(File f, String dir)
	{
		for(File z : f.listFiles())
			if(z.isDirectory())
			{
				if(z.isHidden() || z.getName().equals(".svn"))
					continue;

				if(Config.DONTLOADQUEST && z.getName().equals("quests") && z.getParentFile().getName().equals("scripts"))
					continue;

				String olddir = dir;
				dir = dir + z.getName() + "/";
				parseClasses(z, dir);
				dir = olddir;
			}
			else
			{
				if(z.isHidden() || !z.getName().contains(".java"))
					continue;

				_scriptDirs.add(dir);
				_scriptNames.add(z.getName());
			}
	}

	public HashMap<String, Script> getClasses()
	{
		return _classes;
	}
}
