package L2jFT.game;

import com.l2jserver.mmocore.network.SelectorServerConfig;
import com.l2jserver.mmocore.network.SelectorThread;
import javolution.util.FastMap;
import L2jFT.Config;
import L2jFT.Server;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.extensions.crypt.FilesCrypt;
import L2jFT.extensions.scripts.Events;
import L2jFT.extensions.scripts.ScriptObject;
import L2jFT.game.cache.CrestCache;
import L2jFT.game.geodata.GeoData;
import L2jFT.game.handler.AdminCommandHandler;
import L2jFT.game.handler.ItemHandler;
import L2jFT.game.handler.UserCommandHandler;
import L2jFT.game.handler.VoicedCommandHandler;
import L2jFT.game.idfactory.IdFactory;
import L2jFT.game.instancemanager.*;
import L2jFT.game.model.AutoChatHandler;
import L2jFT.game.model.AutoSpawnHandler;
import L2jFT.game.model.L2Manor;
import L2jFT.game.model.entity.*;
import L2jFT.game.network.L2GameClient;
import L2jFT.game.network.L2GamePacketHandler;
import L2jFT.game.tables.*;
import L2jFT.game.taskmanager.TaskManager;
import L2jFT.game.webserver.WebServer;
import L2jFT.status.Status;
import L2jFT.util.Log;
import L2jFT.util.RRDTools;
import L2jFT.util.Strings;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.util.logging.LogManager;
import java.util.logging.Logger;

@SuppressWarnings( { "nls", "unqualified-field-access", "boxing" })
public class GameServer
{
	private static final Logger _log = Logger.getLogger(GameServer.class.getName());

	private final SelectorThread<L2GameClient> _selectorThread;
	private final SkillTable _skillTable;
	private final ItemTable _itemTable;
	private final ArmorSetsTable _armorSetsTable;
	private final HennaTable _hennaTable;
	private final IdFactory _idFactory;
	public static GameServer gameServer;

	private final ItemHandler _itemHandler;
	private final AdminCommandHandler _adminCommandHandler;
	private final Shutdown _shutdownHandler;
	private final UserCommandHandler _userCommandHandler;
	private final VoicedCommandHandler _voicedCommandHandler;
	private static DoorTable _doorTable;
	private final SevenSigns _sevenSignsEngine;
	private final AutoChatHandler _autoChatHandler;
	private final AutoSpawnHandler _autoSpawnHandler;
	private LoginServerThread _loginThread;
	public static Status statusServer;
	public static WebServer webServer = null;
	@SuppressWarnings("unused")
	private final ThreadPoolManager _threadpools;

	//NEWAGE >>
	public static boolean _playerExpPenalty = true;
	public static boolean _playerDrop = true;
	//NEWAGE <<
	public static Events events;

	public static FastMap<String, ScriptObject> scriptsObjects = new FastMap<String, ScriptObject>();

	private static int _serverStarted;
	private static boolean _isLoading = true;
	public static final boolean ltvj = false;

	public long getUsedMemoryMB()
	{
		return (Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory()) / 1024 / 1024;
	}

	public SelectorThread<L2GameClient> getSelectorThread()
	{
		return _selectorThread;
	}

	public static final DoorTable getDoorsTable()
	{
		return _doorTable;
	}

	public static final int time()
	{
		return (int) (System.currentTimeMillis() / 1000);
	}

	public static final int uptime()
	{
		return time() - _serverStarted;
	}

	public static final boolean isLoading()
	{
		return _isLoading;
	}

	public GameServer() throws Exception
	{
		Server.gameServer = this;

		_serverStarted = time();
		_isLoading = true;

		_log.finest("used mem:" + getUsedMemoryMB() + "MB");

		Strings.reload();

		_idFactory = IdFactory.getInstance();
		if(!_idFactory.isInitialized())
		{
			_log.severe("Could not read object IDs from DB. Please Check Your Data.");
			throw new Exception("Could not initialize the ID factory");
		}

		_threadpools = ThreadPoolManager.getInstance();

		CrestCache.load();

		// start game time control early
		GameTimeController.getInstance();

		// keep the references of Singletons to prevent garbage collection
		CharNameTable.getInstance();

		_skillTable = SkillTable.getInstance();
		if(!_skillTable.isInitialized())
		{
			_log.severe("Could not find the Skills files. Please Check Your Data.");
			throw new Exception("Could not initialize the skill table");
		}

		_itemTable = ItemTable.getInstance();
		if(!_itemTable.isInitialized())
		{
			_log.severe("Could not find the >Items files. Please Check Your Data.");
			throw new Exception("Could not initialize the item table");
		}

		_armorSetsTable = ArmorSetsTable.getInstance();
		if(!_armorSetsTable.isInitialized())
		{
			_log.severe("Could not find the ArmorSets files. Please Check Your Data.");
			throw new Exception("Could not initialize the armorSets table");
		}

		if(Config.WebServerPort != 0)
			(webServer = new WebServer()).start();

		events = new Events();

		L2Manor.getInstance();
		CastleManorManager.getInstance();

		TradeController.getInstance();
		ItemCreationController.getInstance();

		RecipeController.getInstance();

		SkillTreeTable.getInstance();
		SkillSpellbookTable.getInstance();
		CharTemplateTable.getInstance();

		NpcTable.getInstance();
		if(!NpcTable.isInitialized())
		{
			_log.severe("Could not find the extraced files. Please Check Your Data.");
			throw new Exception("Could not initialize the npc table");
		}
		_hennaTable = HennaTable.getInstance();
		if(!_hennaTable.isInitialized())
			throw new Exception("Could not initialize the Henna Table");
		HennaTreeTable.getInstance();
		if(!_hennaTable.isInitialized())
			throw new Exception("Could not initialize the Henna Tree Table");

		LevelUpTable.getInstance();

		GeoData.getInstance();

		TownManager.init();

		SpawnTable.getInstance();

		for(Castle c : CastleManager.getInstance().getCastles().values())
		{
			c.getSiege().spawnArtifacts(c.getCastleId());
			c.getSiege().spawnControlTowers(c.getCastleId());
		}

		//CastleManager.getInstance().validateManorId();

		RaidBossSpawnManager.getInstance();

		DimensionalRiftManager.getInstance();

		Announcements.getInstance();

		ClanTable.getInstance();

		ClanHallManager.getInstance();

		MapRegionTable.getInstance();

		AugmentationData.getInstance();

		if(Config.AUTODESTROY_ITEM_AFTER > 0)
			ItemsAutoDestroy.getInstance();

		MonsterRace.getInstance();

		_doorTable = DoorTable.getInstance(); // needs ClanHallManager & ClanTable for CH doors! 

		StaticObjectsTable.getInstance();

		_sevenSignsEngine = SevenSigns.getInstance();
		SevenSignsFestival.getInstance();

		_autoSpawnHandler = AutoSpawnHandler.getInstance();
		_log.config("AutoSpawnHandler: Loaded " + _autoSpawnHandler.size() + " handlers in total.");

		_autoChatHandler = AutoChatHandler.getInstance();
		_log.config("AutoChatHandler: Loaded " + _autoChatHandler.size() + " handlers in total.");

		_sevenSignsEngine.spawnSevenSignsNPC();

		if(Config.ENABLE_OLYMPIAD)
		{
			Olympiad.getInstance();
			Hero.getInstance();
		}

		CursedWeaponsManager.getInstance();

		if(!Config.ALLOW_WEDDING)
		{
			CoupleManager.getInstance();
			_log.config("CoupleManager initialized");
		}

		_itemHandler = ItemHandler.getInstance();
		_log.config("ItemHandler: Loaded " + _itemHandler.size() + " handlers.");

		_adminCommandHandler = AdminCommandHandler.getInstance();
		_log.config("AdminCommandHandler: Loaded " + _adminCommandHandler.size() + " handlers.");

		_userCommandHandler = UserCommandHandler.getInstance();
		_log.config("UserCommandHandler: Loaded " + _userCommandHandler.size() + " handlers.");

		_voicedCommandHandler = VoicedCommandHandler.getInstance();
		_log.config("VoicedCommandHandler: Loaded " + _voicedCommandHandler.size() + " handlers.");

		QuestManager.getInstance();

		TaskManager.getInstance();

		MercTicketManager.getInstance();

		BoatManager.getInstance();

		_shutdownHandler = Shutdown.getInstance();
		Runtime.getRuntime().addShutdownHook(_shutdownHandler);

		try
		{
			// Colosseum doors
			_doorTable.getDoor(24190001).openMe();
			_doorTable.getDoor(24190002).openMe();
			_doorTable.getDoor(24190003).openMe();
			_doorTable.getDoor(24190004).openMe();

			// TOI doors
			_doorTable.getDoor(23180001).openMe();
			_doorTable.getDoor(23180002).openMe();
			_doorTable.getDoor(23180003).openMe();
			_doorTable.getDoor(23180004).openMe();
			_doorTable.getDoor(23180005).openMe();
			_doorTable.getDoor(23180006).openMe();

			// Эти двери, похоже выполняют декоративную функцию,
			// находятся во Frozen Labyrinth над мостом по пути к снежной королеве.
			_doorTable.getDoor(23140001).openMe();
			_doorTable.getDoor(23140002).openMe();

			_doorTable.checkAutoOpen();
		}
		catch(NullPointerException e)
		{
			_log.warning("Door.csv does not contain the right door info. Update door.csv");
			e.printStackTrace();
		}

		_log.config("IdFactory: Free ObjectID's remaining: " + IdFactory.getInstance().size());

		AuctionManager.getInstance();

		File file = new File("./log/game");
		file.mkdirs();

		Shutdown.getInstance().setAutoRestart(Config.RESTART_TIME * 60 * 60);

		_log.info("GameServer Started");
		_log.config("Maximum Numbers of Connected Players: " + Config.MAXIMUM_ONLINE_USERS);

		if(Config.RRD_ENABLED)
			RRDTools.init();

		if(ltvj)
		{
			FilesCrypt.getInstance();
			Config.MAXIMUM_ONLINE_USERS = (408 + 242) / 13;
		}
		
		_loginThread = LoginServerThread.getInstance();
		_loginThread.setMaxPlayer(Config.MAXIMUM_ONLINE_USERS);
		_loginThread.start();

		SelectorServerConfig ssc = new SelectorServerConfig(Config.PORT_GAME);
		L2GamePacketHandler gph = new L2GamePacketHandler();
		_selectorThread = new SelectorThread<L2GameClient>(ssc, gph, gph, gph);
		_selectorThread.openServerSocket();
		_selectorThread.start();
		_isLoading = false;
	}

	public final ItemTable getItemTable()
	{
		return _itemTable;
	}

	public static void main(String[] args) throws Exception
	{
		Server.SERVER_MODE = Server.MODE_GAMESERVER;
		// Local Constants
		final String LOG_FOLDER = "log"; // Name of folder for log file
		final String LOG_NAME = "./config/log.ini"; // Name of log file

		/*** Main ***/
		// Create log folder
		File logFolder = new File(Config.DATAPACK_ROOT, LOG_FOLDER);
		logFolder.mkdir();

		// Create input stream for log file -- or store file data into memory
		InputStream is = new FileInputStream(new File(LOG_NAME));
		LogManager.getLogManager().readConfiguration(is);
		is.close();

		// Initialize config 
		Config.load();
		checkFreePorts();
		L2DatabaseFactory.getInstance();
		gameServer = new GameServer();

		if(Config.IS_TELNET_ENABLED)
		{
			statusServer = new Status(Server.MODE_GAMESERVER);
			statusServer.start();
		}
		else
			_log.info("Telnet server is currently disabled.");

		System.gc();
		// maxMemory is the upper limit the jvm can use, totalMemory the size of the current allocation pool, freeMemory the unused memory in the allocation pool
		long freeMem = (Runtime.getRuntime().maxMemory() - Runtime.getRuntime().totalMemory() + Runtime.getRuntime().freeMemory()) / 1024 / 1024;
		long totalMem = Runtime.getRuntime().maxMemory() / 1024 / 1024;
		_log.info("Free memory " + freeMem + " Mb of " + totalMem + " Mb");
		Log.LogServ(Log.GS_started, (int) freeMem, (int) totalMem, IdFactory.getInstance().size(), 0);

		//Shutdown.getInstance().setAutoRestart(Config.RESTART_TIME * 60 * 60);
	}

	public static void checkFreePorts()
	{
		try
		{
			ServerSocket ss;
			if(Config.GAMESERVER_HOSTNAME.equalsIgnoreCase("*"))
				ss = new ServerSocket(Config.PORT_GAME);
			else
				ss = new ServerSocket(Config.PORT_GAME, 50, InetAddress.getByName(Config.GAMESERVER_HOSTNAME));
			ss.close();
		}
		catch(Exception e)
		{

			_log.warning("\nPort " + Config.PORT_GAME + " is allready binded. Please free it and restart server.");
			System.exit(0);
		}

	}
}
