package L2jFT.game;

import L2jFT.database.DatabaseUtils;
import L2jFT.database.FiltredPreparedStatement;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.database.ThreadConnection;
import L2jFT.game.model.L2TradeList;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.tables.ItemTable;
import L2jFT.util.Strings;

import java.sql.ResultSet;
import java.util.HashMap;
import java.util.logging.Logger;

public class ItemCreationController
{
	private static Logger _log = Logger.getLogger(ItemCreationController.class.getName());
	private static ItemCreationController _instance;
	private HashMap<Integer, L2TradeList> _lists;
	private int _nextListId;

	public static ItemCreationController getInstance()
	{
		if(_instance == null)
			_instance = new ItemCreationController();
		return _instance;
	}

	// npc shop markup id name count maxCount lastRechargeTime rechargeTime order

	private ItemCreationController()
	{
		_lists = new HashMap<Integer, L2TradeList>();
		ThreadConnection con = null;
		FiltredPreparedStatement statement1 = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset1 = null, rset = null;
		int ItemCount = 0;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT id, markup, shop, count, maxCount, lastRechargeTime, rechargeTime, `order` FROM item_creation WHERE shop=? ORDER BY `order` ASC");
			statement1 = con.prepareStatement("SELECT shop, npc FROM item_creation");
			rset1 = statement1.executeQuery();
			while(rset1.next())
			{
				String npc_id = rset1.getString("npc");
				statement.setString(1, String.valueOf(rset1.getInt("shop")));
				rset = statement.executeQuery();
				if(rset.next())
				{
					ItemCount++;
					int shop_id = rset1.getInt("shop");
					L2TradeList buy = new L2TradeList(shop_id);
					short itemId = rset.getShort("id");
					short count = rset.getShort("count");
					short maxCount = rset.getShort("maxCount");
					long lastRechargeTime = rset.getLong("lastRechargeTime");
					long rechargeTime = rset.getLong("rechargeTime");
					int price = Math.round(ItemTable.getInstance().getTemplate(itemId).getReferencePrice() * (1 + rset.getInt("markup") / 100));
					L2ItemInstance item = ItemTable.getInstance().createDummyItem(itemId, Strings.parseInt(npc_id), shop_id, "itemCreation[53]");
					item.setPriceToSell(price);
					item.setCountToSell(count);
					item.setMaxCountToSell(count);
					item.setLastRechargeTime(lastRechargeTime);
					item.setRechargeTime(rechargeTime);
					item.setItemCreation(true);
					buy.addItem(item);
					buy.setNpcId(npc_id);
					while(rset.next())
					{
						ItemCount++;
						itemId = rset.getShort("id");
						count = rset.getShort("count");
						maxCount = rset.getShort("maxCount");
						lastRechargeTime = rset.getLong("lastRechargeTime");
						rechargeTime = rset.getLong("rechargeTime");
						price = Math.round(ItemTable.getInstance().getTemplate(itemId).getReferencePrice() * (1 + rset.getInt("markup") / 100));
						L2ItemInstance item2 = ItemTable.getInstance().createDummyItem(itemId, Strings.parseInt(npc_id), shop_id, "itemCreation[88]");
						if(item2 != null)
						{
							item2.setPriceToSell(price);
							item2.setCountToSell(count);
							item2.setMaxCountToSell(maxCount);
							item2.setLastRechargeTime(lastRechargeTime);
							item2.setRechargeTime(rechargeTime);
							item2.setItemCreation(true);
							buy.addItem(item2);
						}
						else
							_log.warning("ItemCreationController[92]: null dummy item " + itemId);
					}

					_lists.put(new Integer(buy.getListId()), buy);
					_nextListId = Math.max(_nextListId, buy.getListId() + 1);
				}

			}

			_log.config("ItemCreationController: Loaded " + ItemCount + " Items.");
			_log.config("ItemCreationController: Loaded " + _lists.size() + " Lists.");

			for(Integer id : _lists.keySet())
				TradeController.getInstance().addToBuyList(id, _lists.get(id));

			_log.config("Add ItemCreation Lists to Trade Buylists.");
		}
		catch(Exception e)
		{
			_log.warning("ItemCreationController: ItemCreation could not be initialized.");
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseSR(statement1, rset1);
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	public L2TradeList getBuyList(int listId)
	{
		return _lists.get(new Integer(listId));
	}

	public synchronized int getNextId()
	{
		return _nextListId++;
	}
}