package L2jFT.game;

import L2jFT.Config;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.extensions.scripts.Scripts;
import L2jFT.game.instancemanager.CoupleManager;
import L2jFT.game.instancemanager.CursedWeaponsManager;
import L2jFT.game.instancemanager.RaidBossSpawnManager;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;
import L2jFT.game.model.entity.Olympiad;
import L2jFT.game.model.entity.SevenSigns;
import L2jFT.game.model.entity.SevenSignsFestival;
import L2jFT.game.network.serverpackets.ExRestartClient;
import L2jFT.util.Log;

import java.util.logging.Logger;

@SuppressWarnings( { "nls", "unqualified-field-access", "boxing" })
public class Shutdown extends Thread
{
	private static final Logger _log = Logger.getLogger(Shutdown.class.getName());

	private static Shutdown _instance;
	private static Shutdown _counterInstance = null;

	private int secondsShut;
	private int shutdownMode;

	public static final int SIGTERM = 0;
	public static final int GM_SHUTDOWN = 1;
	public static final int GM_RESTART = 2;
	public static final int ABORT = 3;
	private static String[] _modeText = {
			"brought down", "brought down", "restarting", "aborting" };

	public int get_seconds()
	{
		if(_counterInstance != null)
			return _counterInstance.secondsShut;
		return -1;
	}

	public int get_mode()
	{
		if(_counterInstance != null)
			return _counterInstance.shutdownMode;
		return -1;
	}

	/**
	 * This function starts a shutdown countdown from Telnet (Copied from Function startShutdown())
	 *
	 * @param ip		    IP Which Issued shutdown command
	 * @param seconds	   seconds untill shutdown
	 * @param restart	   true if the server will restart after shutdown
	 */
	public void startTelnetShutdown(String IP, int seconds, boolean restart)
	{
		Announcements _an = Announcements.getInstance();
		_log.warning("IP: " + IP + " issued shutdown command. " + _modeText[shutdownMode] + " in " + seconds + " seconds!");
		_an.announceToAll("This server will be " + _modeText[shutdownMode] + " in " + seconds + " seconds!");

		if(_counterInstance != null)
			_counterInstance._abort();
		_counterInstance = new Shutdown(seconds, restart);
		_counterInstance.start();
	}

	public void setAutoRestart(int seconds)
	{
		_log.warning("AutoRestart in " + seconds + " seconds.");
		if(_counterInstance != null)
			_counterInstance._abort();
		_counterInstance = new Shutdown(seconds, true);
		_counterInstance.start();
	}

	/**
	 * This function aborts a running countdown
	 *
	 * @param IP		    IP Which Issued shutdown command
	 */
	public void Telnetabort(String IP)
	{
		Announcements _an = Announcements.getInstance();
		_log.warning("IP: " + IP + " issued shutdown ABORT. " + _modeText[shutdownMode] + " has been stopped!");
		_an.announceToAll("This server aborts " + _modeText[shutdownMode] + " and continues normal operation!");

		if(_counterInstance != null)
			_counterInstance._abort();
	}

	/**
	 * Default constucter is only used internal to create the shutdown-hook instance
	 *
	 */
	public Shutdown()
	{
		secondsShut = -1;
		shutdownMode = SIGTERM;
	}

	/**
	 * This creates a countdown instance of Shutdown.
	 *
	 * @param seconds	how many seconds until shutdown
	 * @param restart	true is the server shall restart after shutdown
	 *
	 */
	public Shutdown(int seconds, boolean restart)
	{
		if(seconds < 0)
			seconds = 0;
		secondsShut = seconds;
		if(restart)
			shutdownMode = GM_RESTART;
		else
			shutdownMode = GM_SHUTDOWN;
	}

	/**
	 * get the shutdown-hook instance
	 * the shutdown-hook instance is created by the first call of this function,
	 * but it has to be registrered externaly.
	 *
	 * @return instance of Shutdown, to be used as shutdown hook
	 */
	public static Shutdown getInstance()
	{
		if(_instance == null)
			_instance = new Shutdown();
		return _instance;
	}

	/**
	 * this function is called, when a new thread starts
	 *
	 * if this thread is the thread of getInstance, then this is the shutdown hook
	 * and we save all data and disconnect all clients.
	 *
	 * after this thread ends, the server will completely exit
	 *
	 * if this is not the thread of getInstance, then this is a countdown thread.
	 * we start the countdown, and when we finished it, and it was not aborted,
	 * we tell the shutdown-hook why we call exit, and then call exit
	 *
	 * when the exit status of the server is 1, startServer.sh / startServer.bat
	 * will restart the server.
	 *
	 * Логгинг в этом методе не работает!!!
	 */
	@Override
	public void run()
	{
		if(this == _instance)
		{
			System.out.println("Shutting down scripts.");
			// Вызвать выключение у скриптов
			Scripts.getInstance().shutdown();

			// ensure all services are stopped
			// stop all scheduled tasks
			saveData();
			try
			{
				ThreadPoolManager.getInstance().shutdown();
			}
			catch(Throwable t)
			{}
			// last byebye, save all data and quit this server
			// logging doesn't works here :(
			try
			{
				LoginServerThread.getInstance().interrupt();
			}
			catch(Throwable t)
			{}
			// saveData sends messages to exit players, so shutdown selector after it
			try
			{
				System.out.println("Shutting down selector.");
				GameServer.gameServer.getSelectorThread().shutdown();
				GameServer.gameServer.getSelectorThread().setDaemon(true);
			}
			catch(Throwable t)
			{}
			// commit data, last chance
			try
			{
				System.out.println("Shutting down database communication.");
				L2DatabaseFactory.getInstance().shutdown();
			}
			catch(Throwable t)
			{}
			// server will quit, when this function ends.
			if(_instance.shutdownMode == GM_RESTART)
				Runtime.getRuntime().halt(2);
			else
				Runtime.getRuntime().halt(0);
		}
		else
		{
			// gm shutdown: send warnings and then call exit to start shutdown sequence
			countdown();
			// last point where logging is operational :(
			System.out.println("GM shutdown countdown is over. " + _modeText[shutdownMode] + " NOW!");
			switch(shutdownMode)
			{
				case GM_SHUTDOWN:
					_instance.setMode(GM_SHUTDOWN);
					System.exit(0);
					break;
				case GM_RESTART:
					_instance.setMode(GM_RESTART);
					System.exit(2);
					break;
			}
		}
	}

	/**
	 * This functions starts a shutdown countdown
	 *
	 * @param activeChar	GM who issued the shutdown command
	 * @param seconds		seconds until shutdown
	 * @param restart		true if the server will restart after shutdown
	 */
	public void startShutdown(L2Player activeChar, int seconds, boolean restart)
	{
		Announcements _an = Announcements.getInstance();
		_log.warning("GM: " + activeChar.getName() + "(" + activeChar.getObjectId() + ") issued shutdown command. " + _modeText[shutdownMode] + " in " + seconds + " seconds!");

		if(shutdownMode > 0)
			_an.announceToAll("This server will be brought down in " + seconds + " seconds");

		if(_counterInstance != null)
			_counterInstance._abort();

		//		 the main instance should only run for shutdown hook, so we start a new instance
		_counterInstance = new Shutdown(seconds, restart);
		_counterInstance.start();
	}

	/**
	 * This function aborts a running countdown
	 *
	 * @param activeChar	GM who issued the abort command
	 */
	public void abort(L2Player activeChar)
	{
		Announcements _an = Announcements.getInstance();
		_log.warning("GM: " + activeChar.getName() + "(" + activeChar.getObjectId() + ") issued shutdown ABORT. " + _modeText[shutdownMode] + " has been stopped!");
		_an.announceToAll("This server aborts " + _modeText[shutdownMode] + " and continues normal operation!");

		if(_counterInstance != null)
			_counterInstance._abort();
	}

	/**
	 * set the shutdown mode
	 * @param mode	what mode shall be set
	 */
	private void setMode(int mode)
	{
		shutdownMode = mode;
	}

	/**
	 * set shutdown mode to ABORT
	 */
	private void _abort()
	{
		shutdownMode = ABORT;
	}

	/**
	 * this counts the countdown and reports it to all players
	 * countdown is aborted if mode changes to ABORT
	 */
	private void countdown()
	{
		Announcements _an = Announcements.getInstance();

		try
		{
			while(secondsShut > 0)
			{

				switch(secondsShut)
				{
					case 600:
						_an.announceToAll("Attention Players!");
						_an.announceToAll("This server will be brought down in 10 minutes");
						_an.announceToAll("Please avoid using Gatekeepers or performing");
						_an.announceToAll("other character actions until after the downtime");
						_an.announceToAll("and find a safe place to log out at this time.");
						_an.announceToAll("Thank you!");
						break;
					case 300:
						_an.announceToAll("Attention Players!");
						_an.announceToAll("This server will be brought down in 5 minutes");
						_an.announceToAll("Please avoid using Gatekeepers or performing");
						_an.announceToAll("other character actions until after the downtime");
						_an.announceToAll("and find a safe place to log out at this time.");
						_an.announceToAll("Thank you!");
						break;
					case 60:
						_an.announceToAll("Attention Players!");
						_an.announceToAll("This server will be brought down in 1 minute");
						_an.announceToAll("Please avoid using Gatekeepers or performing");
						_an.announceToAll("other character actions until after the downtime");
						_an.announceToAll("and find a safe place to log out at this time.");
						_an.announceToAll("Thank you!");
						L2World.deleteVisibleNpcSpawns();
						break;
					case 30:
						_an.announceToAll("This server will be " + _modeText[shutdownMode] + " momentally!");
						break;
				}

				secondsShut--;

				int delay = 1000; //milliseconds
				Thread.sleep(delay);

				if(shutdownMode == ABORT)
					break;
			}
		}
		catch(InterruptedException e)
		{
			//this will never happen
		}
	}

	/**
	 * this sends a last byebye, disconnects all players and saves data
	 *
	 */
	private void saveData()
	{
		switch(shutdownMode)
		{
			case SIGTERM:
				System.err.println("SIGTERM received. Shutting down NOW!");
				Log.LogServ(Log.GS_SIGTERM, 0, 0, 0, 0);
				break;
			case GM_SHUTDOWN:
				System.err.println("GM shutdown received. Shutting down NOW!");
				Log.LogServ(Log.GS_shutdown, 0, 0, 0, 0);
				break;
			case GM_RESTART:
				System.err.println("GM restart received. Restarting NOW!");
				Log.LogServ(Log.GS_restart, 0, 0, 0, 0);
				break;
		}

		disconnectAllCharacters();
		// Seven Signs data is now saved along with Festival data.
		if(!SevenSigns.getInstance().isSealValidationPeriod())
		{
			SevenSignsFestival.getInstance().saveFestivalData(false);
			System.out.println("Seven Signs Festival data saved.");
		}

		// Save Seven Signs data before closing. :)
		SevenSigns.getInstance().saveSevenSignsData(null, true);
		System.out.println("Seven Signs data saved.");

		// Save all Raidboss status
		if(!Config.DONTLOADSPAWN)
		{
			RaidBossSpawnManager.getInstance().cleanUp();
			System.out.println("RaidBossSpawnManager: All raidboss info saved!");
		}

		try
		{
			if(Config.ENABLE_OLYMPIAD)
			{
				Olympiad.getInstance().save();
				System.out.println("Olympiad System: Data saved!");
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

		if(Config.ALLOW_WEDDING)
		{
			CoupleManager.getInstance().store();
			System.out.println("Couples: Data saved!");
		}

		if(Config.ALLOW_CURSED_WEAPONS)
		{
			CursedWeaponsManager.getInstance().saveData();
			System.out.println("CursedWeaponsManager: Data saved!");
		}

		System.out.println("All Data saved. All players disconnected, shutting down.");
		try
		{
			int delay = 5000;
			Thread.sleep(delay);
		}
		catch(InterruptedException e)
		{
			//never happens :p
		}
	}

	/**
	 * this disconnects all clients from the server
	 *
	 */
	private void disconnectAllCharacters()
	{
		ExRestartClient er = new ExRestartClient();
		for(L2Player player : L2World.getAllPlayers())
			//Logout Character
			try
			{
				player.sendPacket(er);
				player.logout(true, false);
			}
			catch(Throwable t)
			{
				System.out.println("Error while disconnect char: " + player.getName());
				t.printStackTrace();
			}
		try
		{
			Thread.sleep(1000);
		}
		catch(Throwable t)
		{
			System.out.println(t);
		}
	}
}