package L2jFT.game.ai;

import static L2jFT.game.ai.CtrlIntention.AI_INTENTION_IDLE;

import java.util.logging.Logger;

import L2jFT.Config;
import L2jFT.game.handler.IScriptHandler;
import L2jFT.game.handler.ScriptHandler;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.L2World;
import L2jFT.game.network.serverpackets.StopRotation;
import L2jFT.util.CharPosition;

public abstract class AbstractAI
{
	protected static final Logger _log = Logger.getLogger(AbstractAI.class.getName());

	/** Означает, что AI всегда включен, независимо от состояния региона */
	protected boolean _globalAI;

	protected final L2Character _actor;

	protected CtrlIntention _intention = AI_INTENTION_IDLE;
	protected Object _intention_arg0 = null;
	protected Object _intention_arg1 = null;

	protected L2Character _attack_target;
	protected L2Skill _skill;

	boolean _running = false;
	public final double HEADINGS_IN_PI = 10430.378350470452724949566316381;

	protected AbstractAI(L2Character actor)
	{
		_actor = actor;
		if(_globalAI)
			L2World.addCustomMoveObject(actor);
	}

	public synchronized void changeIntention(CtrlIntention intention, Object arg0, Object arg1)
	{
		if(Config.DEBUG)
			_log.info("AbstractAI: " + _actor + ".changeIntention(" + intention + ", " + arg0 + ", " + arg1 + ")");

		_intention = intention;
		_intention_arg0 = arg0;
		_intention_arg1 = arg1;
	}

	public final void setIntention(CtrlIntention intention)
	{
		setIntention(intention, null, null);
	}

	public final void setIntention(CtrlIntention intention, Object arg0)
	{
		setIntention(intention, arg0, null);
	}

	public void setIntention(CtrlIntention intention, Object arg0, Object arg1)
	{
		if(!_actor.hasAI())
			return;

		if(!_actor.isVisible())
		{
			if(_intention == AI_INTENTION_IDLE)
				return;

			intention = AI_INTENTION_IDLE;
		}

		IScriptHandler handler = ScriptHandler.getInstance().getScriptHandler();
		if(handler != null)
			handler.setIntention(intention, arg0, arg1);

		switch(intention)
		{
			case AI_INTENTION_IDLE:
				onIntentionIdle();
				break;
			case AI_INTENTION_ACTIVE:
				onIntentionActive();
				break;
			case AI_INTENTION_REST:
				onIntentionRest();
				break;
			case AI_INTENTION_ATTACK:
				onIntentionAttack((L2Character) arg0);
				break;
			case AI_INTENTION_CAST:
				onIntentionCast((L2Skill) arg0, (L2Character) arg1);
				break;
			case AI_INTENTION_PICK_UP:
				onIntentionPickUp((L2Object) arg0);
				break;
			case AI_INTENTION_INTERACT:
				onIntentionInteract((L2Object) arg0);
				break;
			case AI_INTENTION_FOLLOW:
				onIntentionFollow((L2Character) arg0, (Integer) arg1);
				break;
		}
	}

	public final void notifyEvent(CtrlEvent evt)
	{
		notifyEvent(evt, null, null);
	}

	public final void notifyEvent(CtrlEvent evt, Object arg0)
	{
		notifyEvent(evt, arg0, null);
	}

	public void notifyEvent(CtrlEvent evt, Object arg0, Object arg1)
	{
		if(!_actor.isVisible() || !_actor.hasAI())
			return;

		IScriptHandler handler = ScriptHandler.getInstance().getScriptHandler();
		if(handler != null)
			handler.notifyEvent(evt, arg0, arg1);

		switch(evt)
		{
			case EVT_THINK:
				onEvtThink();
				break;
			case EVT_ATTACKED:
				onEvtAttacked((L2Character) arg0, (Boolean) arg1);
				break;
			case EVT_AGGRESSION:
				onEvtAggression((L2Character) arg0, ((Number) arg1).intValue());
				break;
			case EVT_READY_TO_ACT:
				onEvtReadyToAct();
				break;
			case EVT_ARRIVED:
				onEvtArrived();
				break;
			case EVT_ARRIVED_BLOCKED:
				onEvtArrivedBlocked((CharPosition) arg0);
				break;
			case EVT_FORGET_OBJECT:
				onEvtForgetObject((L2Object) arg0);
				break;
			case EVT_CANCEL:
				onEvtCancel();
				break;
			case EVT_DEAD:
				onEvtDead();
				break;
			case EVT_FAKE_DEATH:
				onEvtFakeDeath();
				break;
			case EVT_FINISH_CASTING:
				onEvtFinishCasting();
				break;
			case EVT_SEE_SPELL:
				onEvtSeeSpell((L2Skill) arg0, (L2Character) arg1);
				break;
		}
	}

	protected void clientActionFailed()
	{
		if(_actor.isPlayer)
			_actor.sendActionFailed();
	}

	public void clientStopMoving(CharPosition pos)
	{
		if(_actor.isMoving || _actor.isFollow)
			_actor.stopMove(pos);

		if(pos != null)
			_actor.broadcastPacket(new StopRotation(_actor, pos.h));
	}

	public L2Character getActor()
	{
		return _actor;
	}

	public CtrlIntention getIntention()
	{
		return _intention;
	}

	public synchronized void setAttackTarget(L2Character target)
	{
		_attack_target = target;
	}

	public L2Character getAttackTarget()
	{
		return _attack_target;
	}

	public boolean isGlobalAI()
	{
		return _globalAI;
	}

	public void setGlobalAI(boolean value)
	{
		_globalAI = value;
	}

	@Override
	public String toString()
	{
		return getL2ClassShortName() + " for " + _actor;
	}

	public String getL2ClassShortName()
	{
		if(getClass() != null && getClass().getName() != null)
			return getClass().getName().replaceAll("^.*\\.(.*?)$", "$1");
		return "";
	}

	protected abstract void onIntentionIdle();

	protected abstract void onIntentionActive();

	protected abstract void onIntentionRest();

	protected abstract void onIntentionAttack(L2Character target);

	protected abstract void onIntentionCast(L2Skill skill, L2Character target);

	protected abstract void onIntentionPickUp(L2Object item);

	protected abstract void onIntentionInteract(L2Object object);

	protected abstract void onEvtThink();

	protected abstract void onEvtAttacked(L2Character attacker, Boolean canBeAttackedByMob);

	protected abstract void onEvtAggression(L2Character target, int aggro);

	protected abstract void onEvtReadyToAct();

	protected abstract void onEvtArrived();

	protected abstract void onEvtArrivedBlocked(CharPosition blocked_at_pos);

	protected abstract void onEvtForgetObject(L2Object object);

	protected abstract void onEvtCancel();

	protected abstract void onEvtDead();

	protected abstract void onEvtFakeDeath();

	protected abstract void onEvtFinishCasting();

	protected abstract void onEvtSeeSpell(L2Skill skill, L2Character caster);

	protected abstract void onIntentionFollow(L2Character target, Integer offset);
}
