package L2jFT.game.ai;

import static L2jFT.game.ai.CtrlIntention.AI_INTENTION_ACTIVE;
import static L2jFT.game.ai.CtrlIntention.AI_INTENTION_ATTACK;
import static L2jFT.game.ai.CtrlIntention.AI_INTENTION_CAST;
import static L2jFT.game.ai.CtrlIntention.AI_INTENTION_IDLE;
import static L2jFT.game.ai.CtrlIntention.AI_INTENTION_FOLLOW;
import L2jFT.game.ai.L2PlayableAI.nextAction;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Skill;
import L2jFT.game.network.serverpackets.AutoAttackStop;
import L2jFT.game.network.serverpackets.Die;
import L2jFT.util.CharPosition;

public class L2CharacterAI extends AbstractAI
{
	public L2CharacterAI(L2Character actor)
	{
		super(actor);
	}

	@Override
	protected void onIntentionIdle()
	{
		if(_intention != AI_INTENTION_IDLE)
		{
			changeIntention(AI_INTENTION_IDLE, null, null);
			_attack_target = null;
			clientStopMoving(null);
		}
	}

	@Override
	protected void onIntentionActive()
	{
		if(_intention != AI_INTENTION_ACTIVE)
		{
			changeIntention(AI_INTENTION_ACTIVE, null, null);
			_attack_target = null;
			clientStopMoving(null);
			onEvtThink();
		}
	}

	@Override
	protected void onIntentionRest()
	{
		setIntention(AI_INTENTION_IDLE, null, null);
	}

	@Override
	protected void onIntentionAttack(L2Character target)
	{
		_attack_target = target;
		changeIntention(AI_INTENTION_ATTACK, target, null);
		notifyEvent(CtrlEvent.EVT_THINK, null, null);
	}

	@Override
	protected void onIntentionCast(L2Skill skill, L2Character target)
	{
		_skill = skill;
		_attack_target = target;
		changeIntention(AI_INTENTION_CAST, skill, target);
		notifyEvent(CtrlEvent.EVT_THINK, null, null);
	}

	@Override
	protected void onEvtReadyToAct()
	{
		onEvtThink();
	}

	@Override
	protected void onEvtArrived()
	{
	//onEvtThink();
	}

	@Override
	protected void onEvtArrivedBlocked(CharPosition blocked_at_pos)
	{
		clientStopMoving(blocked_at_pos);
		onEvtThink();
	}

	@Override
	protected void onEvtForgetObject(L2Object object)
	{
		if(_attack_target == object || _actor.getTargetId() == object.getObjectId())
		{
			_attack_target = null;
			_actor.setTarget(null);
			setIntention(AI_INTENTION_ACTIVE, null, null);
			return;
		}
	}

	@Override
	protected void onEvtCancel()
	{
		_actor.stopMove(null);

		if(!_actor.isInCombat())
			_actor.broadcastPacket(new AutoAttackStop(_actor.getObjectId()));

		onEvtThink();
	}

	@Override
	protected void onEvtDead()
	{
		_actor.breakAttack();
		_actor.breakCast(true);
		_actor.stopMove(null);
		_actor.broadcastPacket(new Die(_actor));
		_intention = AI_INTENTION_IDLE;
		_attack_target = null;
	}

	@Override
	protected void onEvtFakeDeath()
	{
		clientStopMoving(null);
		_intention = AI_INTENTION_IDLE;
		_attack_target = null;
		_attack_target = null;
	}

	@Override
	protected void onEvtAttacked(L2Character attacker, Boolean canBeAttackedByMob)
	{
		_actor.startAttackStanceTask();
		if(attacker != null)
			attacker.startAttackStanceTask();
	}

	@SuppressWarnings("unused")
	public void Attack(L2Object target, boolean forceUse)
	{
		setIntention(AI_INTENTION_ATTACK, target);
	}

	public void Cast(L2Skill skill, L2Character target)
	{
		Cast(skill, target, false, false);
	}

	@SuppressWarnings("unused")
	public void Cast(L2Skill skill, L2Character target, boolean forceUse, boolean dontMove)
	{
		setIntention(AI_INTENTION_ATTACK, target);
	}

	@Override
	protected void onIntentionFollow(L2Character target, Integer offset)
	{
		changeIntention(AI_INTENTION_FOLLOW, target, offset);
		_actor.followToCharacter(target, offset, false);
	}

	@Override
	protected void onEvtThink()
	{}

	@Override
	protected void onEvtAggression(L2Character target, int aggro)
	{}

	@Override
	protected void onEvtFinishCasting()
	{}

	@Override
	protected void onIntentionInteract(L2Object object)
	{}

	@Override
	protected void onIntentionPickUp(L2Object item)
	{}

	@Override
	protected void onEvtSeeSpell(L2Skill skill, L2Character caster)
	{}

	@SuppressWarnings("unused")
	public void stopAITask()
	{}

	@SuppressWarnings("unused")
	public void startAITask()
	{}

	@SuppressWarnings("unused")
	public void setNextAction(nextAction action, Object arg0, Object arg1, boolean arg2, boolean arg3)
	{}

	@SuppressWarnings("unused")
	public void clearNextAction()
	{}

	@SuppressWarnings("unused")
	public void teleportHome()
	{}

	public boolean isActive()
	{
		return true;
	}
}