package L2jFT.game.ai;

import L2jFT.Config;
import L2jFT.game.ThreadPoolManager;
import static L2jFT.game.ai.CtrlIntention.AI_INTENTION_ACTIVE;
import static L2jFT.game.ai.CtrlIntention.AI_INTENTION_ATTACK;
import static L2jFT.game.ai.CtrlIntention.AI_INTENTION_CAST;
import static L2jFT.game.ai.CtrlIntention.AI_INTENTION_IDLE;
import static L2jFT.game.ai.CtrlIntention.AI_INTENTION_INTERACT;
import static L2jFT.game.ai.CtrlIntention.AI_INTENTION_PICK_UP;
import L2jFT.game.cache.Msg;
import L2jFT.game.geodata.GeoData;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Effect;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.L2Skill.NextAction;
import L2jFT.game.model.instances.L2PetInstance;
import L2jFT.game.network.serverpackets.MyTargetSelected;
import L2jFT.util.Location;

import java.util.concurrent.ScheduledFuture;

public class L2PlayableAI extends L2CharacterAI
{
	private boolean thinking = false; // to prevent recursive thinking

	private nextAction _nextAction;
	private Object _nextAction_arg0;
	private Object _nextAction_arg1;
	private boolean _nextAction_arg2;
	private boolean _nextAction_arg3;

	private boolean _forceUse;
	private boolean _dontMove;

	private L2Player _owner;

	private ScheduledFuture _fearTask;

	//private L2Playable _thisActor;

	public L2PlayableAI(L2Character actor, L2Player owner)
	{
		super(actor);
		_owner = owner;
		//_thisActor = (L2Playable) actor;
	}

	public enum nextAction
	{
		ATTACK,
		CAST,
		MOVE,
		REST,
		PICKUP,
		INTERACT
	}

	@Override
	public void setNextAction(nextAction action, Object arg0, Object arg1, boolean arg2, boolean arg3)
	{
		_nextAction = action;
		_nextAction_arg0 = arg0;
		_nextAction_arg1 = arg1;
		_nextAction_arg2 = arg2;
		_nextAction_arg3 = arg3;
	}

	public boolean setNextIntention()
	{
		nextAction nextAction = _nextAction;
		Object nextAction_arg0 = _nextAction_arg0;
		Object nextAction_arg1 = _nextAction_arg1;
		boolean nextAction_arg2 = _nextAction_arg2;
		boolean nextAction_arg3 = _nextAction_arg3;

		if(nextAction == null)
			return false;

		L2Skill skill;
		L2Character target;
		L2Object object;

		switch(nextAction)
		{
			case ATTACK:
				if(nextAction_arg0 == null)
					return false;
				target = (L2Character) nextAction_arg0;
				_forceUse = nextAction_arg2;
				clearNextAction();
				setIntention(AI_INTENTION_ATTACK, target);
				break;
			case CAST:
				if(nextAction_arg0 == null || nextAction_arg1 == null)
					return false;
				skill = (L2Skill) nextAction_arg0;
				target = (L2Character) nextAction_arg1;
				_forceUse = nextAction_arg2;
				_dontMove = nextAction_arg3;
				clearNextAction();
				if(skill.checkCondition(_actor, target, _forceUse, _dontMove, true))
					setIntention(AI_INTENTION_CAST, skill, target);
				break;
			case MOVE:
				if(nextAction_arg0 == null || nextAction_arg1 == null)
					return false;
				Location loc = (Location) nextAction_arg0;
				Integer offset = (Integer) nextAction_arg1;
				boolean pathfinding = nextAction_arg2;
				clearNextAction();
				_actor.moveToLocation(loc.x, loc.y, loc.z, offset, pathfinding);
				break;
			case REST:
				_actor.sitDown();
				break;
			case INTERACT:
				if(nextAction_arg0 == null)
					return false;
				object = (L2Object) nextAction_arg0;
				clearNextAction();
				onIntentionInteract(object);
				break;
			case PICKUP:
				if(nextAction_arg0 == null)
					return false;
				object = (L2Object) nextAction_arg0;
				clearNextAction();
				onIntentionPickUp(object);
				break;
			default:
				return false;
		}
		return true;
	}

	@Override
	public void clearNextAction()
	{
		_nextAction = null;
		_nextAction_arg0 = null;
		_nextAction_arg1 = null;
		_nextAction_arg2 = false;
		_nextAction_arg3 = false;
	}

	@Override
	protected void onEvtFinishCasting()
	{
		if(!setNextIntention())
			setIntention(AI_INTENTION_ACTIVE);
	}

	@Override
	protected void onEvtReadyToAct()
	{
		if(!setNextIntention())
			onEvtThink();
	}

	@Override
	protected void onEvtArrived()
	{
		if(!setNextIntention())
			if(_intention == AI_INTENTION_INTERACT || _intention == AI_INTENTION_PICK_UP)
				onEvtThink();
			else
				setIntention(AI_INTENTION_ACTIVE);
	}

	@Override
	protected void onEvtThink()
	{
		if(thinking || _actor.isActionsDisabled())
			return;

		if(Config.DEBUG)
			_log.warning("L2PlayableAI: onEvtThink -> Check intention");

		thinking = true;

		try
		{
			switch(_intention)
			{
				case AI_INTENTION_ATTACK:
					thinkAttack();
					break;
				case AI_INTENTION_CAST:
					thinkCast();
					break;
				case AI_INTENTION_PICK_UP:
					thinkPickUp();
					break;
				case AI_INTENTION_INTERACT:
					thinkInteract();
					break;
				case AI_INTENTION_FOLLOW:
					thinkFollow();
			}
		}
		catch(Exception e)
		{
			_log.warning("Exception onEvtThink(): " + e);
			e.printStackTrace();
		}
		finally
		{
			thinking = false;
		}
	}

	protected void thinkFollow()
	{
		L2Character target = (L2Character) _intention_arg0;
		Integer offset = (Integer) _intention_arg1;
		_actor.followToCharacter(target, offset, false);
	}

	@Override
	protected void onIntentionInteract(L2Object object)
	{
		if(_actor.isActionsDisabled())
		{
			setNextAction(nextAction.INTERACT, object, null, false, false);
			clientActionFailed();
			return;
		}

		clearNextAction();
		changeIntention(AI_INTENTION_INTERACT, object, null);
		onEvtThink();
	}

	protected void thinkInteract()
	{
		L2Object target = (L2Object) _intention_arg0;

		if(target == null)
		{
			setIntention(AI_INTENTION_ACTIVE);
			return;
		}

		if(_actor.isInRangeZ(target, 80))
		{
			if(_actor.isPlayer)
				((L2Player) _actor).doInteract(target);
			setIntention(AI_INTENTION_IDLE);
		}
		else
		{
			_actor.moveToLocation(target.getX(), target.getY(), target.getZ(), 40, true);
			setNextAction(nextAction.INTERACT, target, null, false, false);
		}
	}

	@Override
	protected void onIntentionPickUp(L2Object object)
	{
		if(_actor.isActionsDisabled())
		{
			setNextAction(nextAction.PICKUP, object, null, false, false);
			clientActionFailed();
			return;
		}

		clearNextAction();
		changeIntention(AI_INTENTION_PICK_UP, object, null);
		onEvtThink();
	}

	protected void thinkPickUp()
	{
		L2Object target = (L2Object) _intention_arg0;

		if(target == null)
		{
			setIntention(AI_INTENTION_ACTIVE);
			return;
		}

		if(_actor.isInRangeZ(target, 80))
		{
			if(_actor.isPlayer)
				((L2Player) _actor).doPickupItem(target);
			else if(_actor.isPet)
				((L2PetInstance) _actor).doPickupItem(target);
		}
		else
		{
			_actor.moveToLocation(target.getX(), target.getY(), target.getZ(), 40, true);
			setNextAction(nextAction.PICKUP, target, null, false, false);
		}
	}

	protected void thinkAttack()
	{
		if(_attack_target == null || _attack_target.isDead() || !(_forceUse ? _attack_target.isAttackable() : _attack_target.isAutoAttackable(_owner)))
		{
			setIntention(AI_INTENTION_ACTIVE);
			_actor.sendActionFailed();
			return;
		}

		int range = _actor.getPhysicalAttackRange() + _actor.getMinDistance(_attack_target);
		if(range < 40)
			range = 40;

		boolean canSee = GeoData.getInstance().canSeeTarget(_actor, _attack_target);

		if(range > 200 && !canSee)
		{
			_actor.sendPacket(Msg.CANNOT_SEE_TARGET);
			setIntention(AI_INTENTION_ACTIVE);
			_actor.sendActionFailed();
			return;
		}

		if(_actor.isFakeDeath())
			_actor.stopEffects(L2Effect.EffectType.FAKE_DEATH);

		if(_actor.isInRange(_attack_target, range + 40))
		{
			if(!canSee || Math.abs(_actor.getZ() - _attack_target.getZ()) > range + 100)
			{
				_actor.sendPacket(Msg.CANNOT_SEE_TARGET);
				setIntention(AI_INTENTION_ACTIVE);
				_actor.sendActionFailed();
				return;
			}

			clientStopMoving(null);
			_actor.doAttack(_attack_target);
		}
		else
			_actor.followToCharacter(_attack_target, range, true);
	}

	protected void thinkCast()
	{
		if(_attack_target == null)
		{
			setIntention(AI_INTENTION_ACTIVE);
			_actor.sendActionFailed();
			return;
		}

		int range = _actor.getMagicalAttackRange(_skill) + _actor.getMinDistance(_attack_target);
		if(range < 40)
			range = 40;

		boolean canSee = GeoData.getInstance().canSeeTarget(_actor, _attack_target);
		boolean noRangeSkill = _skill.getCastRange() == Integer.MAX_VALUE;

		if(range > 200 && !canSee && !noRangeSkill)
		{
			_actor.sendPacket(Msg.CANNOT_SEE_TARGET);
			setIntention(AI_INTENTION_ACTIVE);
			_actor.sendActionFailed();
			return;
		}

		if(_actor.isFakeDeath())
			_actor.stopEffects(L2Effect.EffectType.FAKE_DEATH);

		if(_actor.isInRange(_attack_target, range + 40) || noRangeSkill)
		{
			if((!canSee || Math.abs(_actor.getZ() - _attack_target.getZ()) > range + 100) && !noRangeSkill)
			{
				_actor.sendPacket(Msg.CANNOT_SEE_TARGET);
				setIntention(AI_INTENTION_ACTIVE);
				_actor.sendActionFailed();
				return;
			}

			// Если скилл имеет следующее действие, назначим это действие после окончания действия скилла
			if(_skill.getNextAction() == NextAction.ATTACK && !_actor.equals(_attack_target))
				setNextAction(nextAction.ATTACK, _attack_target, null, _forceUse, false);
			else
				clearNextAction();

			clientStopMoving(null);

			if(_skill.checkCondition(_actor, _attack_target, _forceUse, _dontMove, true))
				_actor.doCast(_skill, _attack_target, _forceUse);
			else
			{
				setIntention(AI_INTENTION_ACTIVE);
				_actor.sendActionFailed();
				return;
			}
		}
		else
		{
			if(_skill.getId().equals(L2Skill.BLINDING_BLOW))
				_skill.getEffects(_actor, _actor, false);

			_actor.followToCharacter(_attack_target, range, true);
		}
	}

	@Override
	protected void onEvtDead()
	{
		clearNextAction();
		super.onEvtDead();
	}

	@Override
	protected void onEvtFakeDeath()
	{
		clearNextAction();
		super.onEvtFakeDeath();
	}

	@Override
	protected void onEvtAggression(L2Character target, int aggro)
	{
		if(aggro > 0)
			if(_actor.getTarget() != target)
			{
				_actor.setTarget(target);
				_actor.sendPacket(new MyTargetSelected(target.getObjectId(), 0));
				if(_actor.getCastingSkill() != null && _actor.getCastingSkill().isOffensive())
					_actor.setCastingTarget(target);
			}
			else if(!_actor.isAfraid() && _fearTask == null)
			{
				_actor.startFear();
				Attack(target, false);
				_fearTask = ThreadPoolManager.getInstance().scheduleGeneral(new StopFear(_actor), 1000);
			}
	}

	@Override
	public void Attack(L2Object target, boolean forceUse)
	{
		// Если не можем атаковать, то атаковать позже
		if(_actor.isActionsDisabled() || _actor.isAttackingDisabled())
		{
			setNextAction(nextAction.ATTACK, target, null, forceUse, false);
			_actor.sendActionFailed();
			return;
		}

		//_actor.stopMove(null);
		_forceUse = forceUse;
		clearNextAction();
		setIntention(AI_INTENTION_ATTACK, target);
	}

	@Override
	public void Cast(L2Skill skill, L2Character target, boolean forceUse, boolean dontMove)
	{
		// Если скилл альтернативного типа (например, бутылка на хп),
		// то он может использоваться во время каста других скиллов, или во время атаки, или на бегу.
		// Поэтому пропускаем дополнительные проверки.
		if(skill.altUse())
		{
			if(!_actor.isPotionsDisabled())
				_actor.altUseSkill(skill, target);
			else
				clientActionFailed();
			return;
		}

		// Если не можем кастовать, то использовать скилл позже
		if(_actor.isActionsDisabled())
		{
			if(!_actor.isSkillDisabled(skill.getId()))
				setNextAction(nextAction.CAST, skill, target, forceUse, dontMove);
			clientActionFailed();
			return;
		}

		//_actor.stopMove(null);
		_forceUse = forceUse;
		clearNextAction();
		setIntention(CtrlIntention.AI_INTENTION_CAST, skill, target);
	}

	public class StopFear implements Runnable
	{
		private L2Character _activeChar;

		public StopFear(L2Character activeChar)
		{
			_activeChar = activeChar;
		}

		public void run()
		{
			_activeChar.stopFear();
			_fearTask = null;
		}
	}
}