package L2jFT.game.ai;

import static L2jFT.game.ai.CtrlIntention.AI_INTENTION_IDLE;
import static L2jFT.game.ai.CtrlIntention.AI_INTENTION_REST;
import L2jFT.game.cache.Msg;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill;

public class L2PlayerAI extends L2PlayableAI
{
	private L2Player _thisActor;

	public L2PlayerAI(L2Character actor)
	{
		super(actor, (L2Player) actor);
		_thisActor = (L2Player) _actor;
	}

	@Override
	protected void onIntentionRest()
	{
		changeIntention(AI_INTENTION_REST, null, null);
		_attack_target = null;
		clientStopMoving(null);
	}

	@Override
	protected void onIntentionActive()
	{
		changeIntention(AI_INTENTION_IDLE, null, null);
	}

	@Override
	public void onIntentionInteract(L2Object object)
	{
		if(_thisActor.getSittingTask())
		{
			setNextAction(nextAction.INTERACT, object, null, false, false);
			return;
		}
		else if(_thisActor.isSitting())
		{
			_thisActor.sendPacket(Msg.YOU_CANNOT_MOVE_WHILE_SITTING);
			clientActionFailed();
			return;
		}
		super.onIntentionInteract(object);
	}

	@Override
	public void onIntentionPickUp(L2Object object)
	{
		if(_thisActor.getSittingTask())
		{
			setNextAction(nextAction.PICKUP, object, null, false, false);
			return;
		}
		else if(_thisActor.isSitting())
		{
			_thisActor.sendPacket(Msg.YOU_CANNOT_MOVE_WHILE_SITTING);
			clientActionFailed();
			return;
		}
		super.onIntentionPickUp(object);
	}

	@Override
	public void Attack(L2Object target, boolean forceUse)
	{
		if(_thisActor.getSittingTask())
		{
			setNextAction(nextAction.ATTACK, target, null, forceUse, false);
			return;
		}
		else if(_thisActor.isSitting())
		{
			_thisActor.sendPacket(Msg.YOU_CANNOT_MOVE_WHILE_SITTING);
			clientActionFailed();
			return;
		}
		super.Attack(target, forceUse);
	}

	@Override
	public void Cast(L2Skill skill, L2Character target, boolean forceUse, boolean dontMove)
	{
		if(!skill.altUse())
			// Если в этот момент встаем, то использовать скилл когда встанем 
			if(_thisActor.getSittingTask())
			{
				setNextAction(nextAction.CAST, skill, target, forceUse, dontMove);
				clientActionFailed();
				return;
			}
			// если сидим - скиллы нельзя использовать
			else if(_thisActor.isSitting())
			{
				_thisActor.sendPacket(Msg.YOU_CANNOT_MOVE_WHILE_SITTING);
				clientActionFailed();
				return;
			}
		super.Cast(skill, target, forceUse, dontMove);
	}

	@Override
	protected void onEvtAttacked(L2Character attacker, Boolean canBeAttackedByMob)
	{
		// notify the tamed beast of attacks 
		if(_thisActor.getTrainedBeast() != null)
			_thisActor.getTrainedBeast().onOwnerGotAttacked(attacker);
		super.onEvtAttacked(attacker, canBeAttackedByMob);
	}
}
