package L2jFT.game.ai;

import L2jFT.game.ThreadPoolManager;
import L2jFT.game.instancemanager.SiegeManager;
import L2jFT.util.Rnd;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill.SkillTargetType;
import L2jFT.game.model.entity.siege.Siege;
import L2jFT.game.model.instances.L2ArtefactInstance;
import L2jFT.game.model.instances.L2DoorInstance;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.model.instances.L2SiegeGuardInstance;

public class L2StaticObjectAI extends L2CharacterAI
{
	L2Character _actor;
	L2Player _attacker;

	public L2StaticObjectAI(L2Character actor)
	{
		super(actor);
		_actor = actor;
	}

	@Override
	protected void onEvtAttacked(L2Character attacker, Boolean canBeAttackedByMob)
	{
		if(!(_actor instanceof L2DoorInstance) || attacker == null)
			return;

		L2Player player = attacker.getPlayer();

		if(player != null)
		{
			L2Clan clan = player.getClan();

			Siege siege = SiegeManager.getInstance().getSiege(_actor);

			if(siege == null)
				return;

			//TODO присвоить осаду обьекту при спавне, чтобы избавиться от перебора
			if(clan != null && siege == clan.getSiege() && clan.getIsDefender())
				return;

			for(L2NpcInstance npc : _actor.getKnownNpc(900))
			{
				if(!(npc instanceof L2SiegeGuardInstance))
					continue;

				if(Math.abs(attacker.getZ() - npc.getZ()) < 200)
					if(Rnd.chance(20))
						npc.getAI().notifyEvent(CtrlEvent.EVT_AGGRESSION, attacker, 10000);
					else
						npc.getAI().notifyEvent(CtrlEvent.EVT_AGGRESSION, attacker, 2000);
			}
		}
	}

	@Override
	protected void onEvtAggression(L2Character attacker, int aggro)
	{
		if(!(_actor instanceof L2ArtefactInstance))
			return;

		if(attacker != null && attacker.getPlayer() != null)
		{
			L2Clan clan = attacker.getPlayer().getClan();
			//TODO присвоить осаду обьекту при спавне, чтобы избавиться от перебора
			if(clan != null && SiegeManager.getInstance().getSiege(_actor) == clan.getSiege() && clan.getIsDefender())
				return;
			ThreadPoolManager.getInstance().scheduleAi(new notifyGuard(attacker.getPlayer()), 1000);
		}
	}

	public class notifyGuard implements Runnable
	{
		notifyGuard(L2Player attacker)
		{
			_attacker = attacker;
		}

		public void run()
		{
			if(_attacker == null)
				return;

			for(L2NpcInstance npc : _actor.getKnownNpc(900))
				if(npc instanceof L2SiegeGuardInstance && Rnd.chance(20))
					npc.getAI().notifyEvent(CtrlEvent.EVT_AGGRESSION, _attacker, 5000);

			if(_attacker.getCastingSkill() != null && _attacker.getCastingSkill().getTargetType() == SkillTargetType.TARGET_HOLY)
				ThreadPoolManager.getInstance().scheduleAi(new notifyGuard(_attacker), 10000);
		}
	}
}
