package L2jFT.game.ai;

import static L2jFT.game.ai.CtrlIntention.AI_INTENTION_ACTIVE;
import L2jFT.util.Rnd;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.L2Skill.SkillType;

import java.util.ArrayList;

/**
 * This class manages AI of L2NpcInstance.<BR><BR>
 */
public class Mystic extends DefaultAI
{
	public Mystic(L2Character actor)
	{
		super(actor);
	}

	@Override
	protected boolean thinkActive()
	{
		if(super.thinkActive())
			return true;

		// BUFF
		if(_buff.length > 0 && Rnd.chance(5))
		{
			_r_skill = _buff[Rnd.get(_buff.length)];
			if(_thisActor.getEffect(_r_skill) == null)
			{
				// Добавить новое задание
				Task task = new Task();
				task.type = TaskType.BUFF;
				task.target = _thisActor;
				task.skill = _r_skill;
				_task_list.add(task);
				_def_think = true;
				return true;
			}
		}

		return false;
	}

	@Override
	protected boolean createNewTask()
	{
		// Удаляем все задания
		clearTasks();

		L2Character _temp_attack_target = _attack_target;

		// Новая цель исходя из агрессивности
		L2Character hated = _thisActor.isConfused() ? _temp_attack_target : _thisActor.getMostHated();

		if(hated != null && hated != _thisActor)
		{
			_attack_target = hated;
			_temp_attack_target = hated;
		}
		else
		{
			_thisActor.setAttackTimeout(Integer.MAX_VALUE);
			_attack_target = null;
			_temp_attack_target = null;
			clientStopMoving(null);
			setIntention(AI_INTENTION_ACTIVE);
			return false;
		}

		// Базовые параметры
		int phys_per = 0;

		int stun_per = 10;
		int debuff_per = 25;
		int dot_per = 25;
		int pdam_per = 10;
		int mdam_per = 75;

		int rnd_per = Rnd.get(100);

		_d_skill = new ArrayList<L2Skill>();
		_r_skill = null;

		double distance = _thisActor.getDistance(_temp_attack_target);

		_att_hp = _temp_attack_target.getCurrentHp();
		_att_mp = _temp_attack_target.getCurrentMp();
		_att_max_hp = _temp_attack_target.getMaxHp();
		_att_max_mp = _temp_attack_target.getMaxMp();
		_def_hp = _thisActor.getCurrentHp();
		_def_mp = _thisActor.getCurrentMp();
		_def_max_hp = _thisActor.getMaxHp();
		_def_max_mp = _thisActor.getMaxMp();

		if(!Rnd.chance(phys_per))
		{
			// DOT
			if(rnd_per < dot_per && _att_hp / _att_max_hp > 0.25 && _dot_skills.length > 0)
			{
				_t_skill.clear();
				for(L2Skill sk : _dot_skills)
				{
					if(sk == null || _temp_attack_target.getEffect(sk) != null)
						continue;
					if(sk.getCastRange() > 200 && distance <= 200)
						continue;
					if(sk.getCastRange() <= 200 && distance > 200)
						continue;
					_t_skill.add(sk);
				}
				if(_t_skill.size() > 0)
					_d_skill.add(_t_skill.get(Rnd.get(_t_skill.size())));
			}

			// DEBUFF
			if(rnd_per < debuff_per && _att_hp / _att_max_hp > 0.25 && _debuff_skills.length > 0)
			{
				_t_skill.clear();
				for(L2Skill sk : _debuff_skills)
				{
					if(sk == null || _temp_attack_target.getEffect(sk) != null)
						continue;
					if(sk.getCastRange() > 200 && distance <= 200)
						continue;
					if(sk.getCastRange() <= 200 && distance > 200)
						continue;
					_t_skill.add(sk);
				}
				if(_t_skill.size() > 0)
					_d_skill.add(_t_skill.get(Rnd.get(_t_skill.size())));
			}

			// PDAM
			if(rnd_per < pdam_per && _pdam.length > 0)
			{
				_t_skill.clear();
				for(L2Skill sk : _pdam)
				{
					if(sk == null || sk.getCastRange() > 200 && distance <= 200)
						continue;
					if(sk.getCastRange() <= 200 && distance > 200)
						continue;
					_t_skill.add(sk);
				}
				if(_t_skill.size() > 0)
					_d_skill.add(_t_skill.get(Rnd.get(_t_skill.size())));
			}

			// MDAM
			if(rnd_per < mdam_per && _mdam.length > 0)
			{
				_t_skill.clear();
				for(L2Skill sk : _mdam)
				{
					if(sk == null || sk.getCastRange() > 200 && distance <= 200)
						continue;
					if(sk.getCastRange() <= 200 && distance > 200)
						continue;
					_t_skill.add(sk);
				}
				if(_t_skill.size() > 0)
					_d_skill.add(_t_skill.get(Rnd.get(_t_skill.size())));
			}

			// STUN
			if(rnd_per < stun_per && _stun.length > 0)
			{
				_t_skill.clear();
				for(L2Skill sk : _stun)
				{
					if(sk == null || _temp_attack_target.getEffect(sk) != null)
						continue;
					if(sk.getCastRange() > 200 && distance <= 200)
						continue;
					if(sk.getCastRange() <= 200 && distance > 200)
						continue;
					_t_skill.add(sk);
				}
				if(_t_skill.size() > 0)
					_d_skill.add(_t_skill.get(Rnd.get(_t_skill.size())));
			}

			// BUFF && HEAL
			if(_buff_skills.length > 0)
			{
				_t_skill.clear();
				for(L2Skill sk : _buff_skills)
					_t_skill.add(sk);
				if(_t_skill.size() > 0)
					_d_skill.add(_t_skill.get(Rnd.get(_t_skill.size())));
			}

			// Выбрать 1 скилл из полученных
			if(_d_skill.size() > 0)
				_r_skill = _d_skill.get(Rnd.get(_d_skill.size()));

			// Смена цели для BUFF скиллов.
			if(_r_skill != null && (_r_skill.getSkillType() == SkillType.BUFF || _r_skill.getSkillType() == SkillType.HEAL_PERCENT))
				if(_thisActor.getEffect(_r_skill) == null)
					_temp_attack_target = _thisActor;
				else
					_r_skill = null;

			// Смена цели для HEAL скиллов
			if(_r_skill != null && _r_skill.getSkillType() == SkillType.HEAL)
				if(_thisActor.getMaxHp() - _thisActor.getCurrentHp() > _r_skill.getPower())
					_temp_attack_target = _thisActor;
				else
					_r_skill = null;
		}

		// Использовать скилл если можно, иначе атаковать
		if(_r_skill != null && !(_thisActor.isMuted() && _r_skill.isMagic() || _thisActor.isPMuted() && !_r_skill.isMagic()) && _def_mp >= _r_skill.getMpConsume() && !_thisActor.isSkillDisabled(_r_skill.getId()))
		{
			// Проверка таргета
			if(_r_skill.getTargetType() == L2Skill.SkillTargetType.TARGET_SELF)
				_temp_attack_target = _thisActor;
			if(_r_skill.getTargetType() == L2Skill.SkillTargetType.TARGET_AURA)
				_temp_attack_target = _thisActor;

			// Добавить новое задание
			Task task = new Task();
			task.type = _r_skill.isOffensive() ? TaskType.CAST : TaskType.BUFF;
			task.target = _temp_attack_target;
			task.skill = _r_skill;
			_task_list.add(task);
			_def_think = true;
			return true;
		}

		// Добавить новое задание
		Task task = new Task();
		task.type = TaskType.ATTACK;
		task.target = _temp_attack_target;
		_task_list.add(task);
		_def_think = true;
		return true;
	}
}