package L2jFT.game.ai;

import javolution.util.FastList;
import L2jFT.Config;
import L2jFT.game.ThreadPoolManager;
import static L2jFT.game.ai.CtrlIntention.AI_INTENTION_IDLE;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.instances.L2RaceManagerInstance;
import L2jFT.game.network.serverpackets.MonRaceInfo;

import java.util.concurrent.Future;

public class RaceManager extends DefaultAI implements Runnable
{
	private Future aiTask;
	private Boolean thinking = false; // to prevent recursive thinking
	FastList<L2Player> _knownPlayers = new FastList<L2Player>();

	public RaceManager(L2Character actor)
	{
		super(actor);
	}

	@Override
	public void run()
	{
		onEvtThink();
	}

	@Override
	protected void onEvtThink()
	{
		L2RaceManagerInstance actor = (L2RaceManagerInstance) _actor;
		MonRaceInfo packet = actor.getPacket();

		if(packet == null)
			return;

		synchronized (thinking)
		{
			if(thinking)
				return;
			thinking = true;
		}

		try
		{
			FastList<L2Player> newPlayers = new FastList<L2Player>();

			FastList<L2Player> players = _actor.getAroundPlayers(1200);
			for(FastList.Node<L2Player> n = players.head(), end = players.tail(); (n = n.getNext()) != end;)
			{
				L2Player player = n.getValue();
				newPlayers.add(player);
				if(!_knownPlayers.contains(player))
					player.sendPacket(packet);
				_knownPlayers.remove(player);
			}

			for(L2Player player : _knownPlayers)
				actor.removeKnownPlayer(player);

			_knownPlayers = newPlayers;

		}
		finally
		{
			// Stop thinking action
			thinking = false;
		}
	}

	@Override
	public void changeIntention(CtrlIntention intention, Object arg0, Object arg1)
	{
		if(Config.DEBUG)
			_log.info("L2RaceManagerAI: " + _actor + ".changeIntention(" + intention + ", " + arg0 + ", " + arg1 + ")");

		super.changeIntention(intention, arg0, arg1);

		if(intention == AI_INTENTION_IDLE)
		{
			if(aiTask != null)
			{
				aiTask.cancel(true);
				aiTask = null;
			}

			_actor.detachAI();
			return;
		}

		synchronized (this)
		{
			if(aiTask == null)
				aiTask = ThreadPoolManager.getInstance().scheduleAiAtFixedRate(this, 5000, 5000);
		}
	}
}
