package L2jFT.game.ai;

import L2jFT.game.geodata.GeoData;
import L2jFT.game.instancemanager.SiegeManager;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2Playable;
import L2jFT.game.model.instances.L2SiegeGuardInstance;
import L2jFT.util.Rnd;

public abstract class SiegeGuard extends DefaultAI
{
	protected L2SiegeGuardInstance _thisActor;

	public SiegeGuard(L2Character actor)
	{
		super(actor);
		_thisActor = (L2SiegeGuardInstance) actor;
		MAX_PATHFIND_FAILS = Integer.MAX_VALUE;
		MAX_ATTACK_TIMEOUT = Integer.MAX_VALUE;
		AI_TASK_DELAY = 2000;
	}

	@Override
	protected boolean randomWalk()
	{
		if(_thisActor.noTarget())
		{
			int homeX = _thisActor.getX() - _thisActor.getSpawnedX();
			int homeY = _thisActor.getY() - _thisActor.getSpawnedY();

			// Проверка на расстояние до точки спауна
			if(homeX * homeX + homeY * homeY > 100 * 100)
			{
				returnHome();
				return true;
			}
		}
		return false;
	}

	@Override
	protected boolean randomAnimation()
	{
		return false;
	}

	@Override
	protected boolean isSilentMoveNotVisible(L2Playable target)
	{
		// Осадные гварды могут видеть игроков в режиме Silent Move с вероятностью 1%
		return target.isSilentMoving() && Rnd.chance(99);
	}

	@Override
	protected boolean autoAttackCondition(L2Playable target)
	{
		if(target == null || !target.isPlayer || target.isInvul() || !target.isVisible())
			return false;
		if(target.isAlikeDead() || isSilentMoveNotVisible(target))
			return false;
		if(Math.abs(target.getZ() - _thisActor.getZ()) > Math.max(200, _thisActor.getPhysicalAttackRange()))
			return false;
		if(!GeoData.getInstance().canSeeTarget(_actor, target))
			return false;
		return _thisActor.isAutoAttackable(target);
	}

	@Override
	protected void onEvtAttacked(L2Character attacker, Boolean canBeAttackedByMob)
	{
		if(attacker != null && attacker.getPlayer() != null)
		{
			L2Clan clan = attacker.getPlayer().getClan();
			//TODO присвоить осаду гварду при спавне, чтобы избавиться от перебора
			if(clan != null && SiegeManager.getInstance().getSiege(_thisActor) == clan.getSiege() && clan.getIsDefender())
				return;
		}
		super.onEvtAttacked(attacker, canBeAttackedByMob);
	}

	@Override
	protected void onEvtAggression(L2Character target, int aggro)
	{
		if(target != null && target.getPlayer() != null)
		{
			L2Clan clan = target.getPlayer().getClan();
			//TODO присвоить осаду гварду при спавне, чтобы избавиться от перебора
			if(clan != null && SiegeManager.getInstance().getSiege(_thisActor) == clan.getSiege() && clan.getIsDefender())
				return;
		}
		super.onEvtAggression(target, aggro);
	}

	@Override
	protected boolean createNewTask()
	{
		int homeX = _thisActor.getX() - _thisActor.getSpawnedX();
		int homeY = _thisActor.getY() - _thisActor.getSpawnedY();

		// Проверка на расстояние до точки спауна
		if(homeX * homeX + homeY * homeY > 2000 * 2000)
		{
			returnHome();
			return false;
		}

		// Удаляем все задания
		clearTasks();

		return true;
	}
}