package L2jFT.game.geodata;

import L2jFT.Config;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;
import L2jFT.util.Location;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.LineNumberReader;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.channels.FileChannel;
import java.util.StringTokenizer;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @Author: Diamond
 * @Date: 20/5/2007
 * @Time: 9:57:48
 */
public class GeoEngine extends GeoData
{
	private static Logger _log = Logger.getLogger(GeoData.class.getName());
	private static GeoEngine _instance;
	private final static byte E = 1;
	private final static byte W = 2;
	private final static byte S = 4;
	private final static byte N = 8;

	private static byte[][][] Geodata = new byte[11][16][];
	private static int[][][] Geodata_index = new int[11][16][];

	private static BufferedOutputStream _geo_bugs_out;

	public static GeoEngine getInstance()
	{
		if(_instance == null)
			_instance = new GeoEngine();
		return _instance;
	}

	public GeoEngine()
	{
		NinitGeodata();
	}

	@Override
	public short getType(int x, int y)
	{
		return NgetType(x - L2World.MAP_MIN_X >> 4, y - L2World.MAP_MIN_Y >> 4);
	}

	@Override
	public int getHeight(int x, int y, int z)
	{
		return NgetHeight(x - L2World.MAP_MIN_X >> 4, y - L2World.MAP_MIN_Y >> 4, z);
	}

	@Override
	public int getSpawnHeight(int x, int y, int zmin, int zmax)
	{
		return NgetSpawnHeight(x - L2World.MAP_MIN_X >> 4, y - L2World.MAP_MIN_Y >> 4, zmin, zmax);
	}

	@Override
	public void getInfo(L2Player gm, int x, int y, int z)
	{
		NgetInfo(gm, x - L2World.MAP_MIN_X >> 4, y - L2World.MAP_MIN_Y >> 4, z);
	}

	@Override
	public boolean canSeeCoord(int x, int y, int z, int tx, int ty, int tz)
	{
		return canSeeTarget(x, y, z, tx, ty, tz);
	}

	@Override
	public boolean canSeeTarget(L2Object cha, L2Object target)
	{
		return !(cha == null || target == null) && canSeeTarget(cha.getX(), cha.getY(), cha.getZ(), target.getX(), target.getY(), target.getZ());
	}

	@Override
	public boolean canMoveToCoord(int x, int y, int z, int tx, int ty, int tz)
	{
		return canMoveToTarget(x, y, z, tx, ty, tz);
	}

	@Override
	public boolean canMoveToCoordWithCollision(int x, int y, int z, int tx, int ty, int tz)
	{
		return canMoveToTargetWithCollision(x, y, z, tx, ty, tz);
	}

	@Override
	public boolean canMoveToTarget(L2Object cha, L2Object target)
	{
		return cha != null && canMoveToTarget(cha.getX(), cha.getY(), cha.getZ(), target.getX(), target.getY(), target.getZ());
	}

	@Override
	public short getNSWE(int x, int y, int z)
	{
		return NgetNSWE(x - L2World.MAP_MIN_X >> 4, y - L2World.MAP_MIN_Y >> 4, z);
	}

	@Override
	public Location moveCheck(L2Object cha, L2Object target)
	{
		return moveCheck(cha.getX(), cha.getY(), cha.getZ(), target.getX(), target.getY(), target.getZ());
	}

	@Override
	public Location moveCheck(int x, int y, int z, int tx, int ty, int tz)
	{
		return MoveCheck(new Location(x, y, z), new Location(tx, ty, tz), x - L2World.MAP_MIN_X >> 4, y - L2World.MAP_MIN_Y >> 4, z, tx - L2World.MAP_MIN_X >> 4, ty - L2World.MAP_MIN_Y >> 4, tz);
	}

	@Override
	public Location moveCheckForAI(Location loc1, Location loc2)
	{
		return MoveCheckForAI(loc1, loc2, loc1.x - L2World.MAP_MIN_X >> 4, loc1.y - L2World.MAP_MIN_Y >> 4, loc1.z, loc2.x - L2World.MAP_MIN_X >> 4, loc2.y - L2World.MAP_MIN_Y >> 4, loc2.z);
	}

	@Override
	public void addGeoDataBug(L2Player gm, String comment)
	{
		int gx = gm.getX() - L2World.MAP_MIN_X >> 4;
		int gy = gm.getY() - L2World.MAP_MIN_Y >> 4;
		int bx = getBlock(gx);
		int by = getBlock(gy);
		int cx = getCell(gx);
		int cy = getCell(gy);
		int rx = (gx >> 11) + 16;
		int ry = (gy >> 11) + 10;
		String out = rx + ";" + ry + ";" + bx + ";" + by + ";" + cx + ";" + cy + ";" + gm.getZ() + ";" + comment + "\n";
		try
		{
			_geo_bugs_out.write(out.getBytes());
			_geo_bugs_out.flush();
			gm.sendMessage("GeoData bug saved!", "Баг геодаты сохранен!");
		}
		catch(Exception e)
		{
			e.printStackTrace();
			gm.sendMessage("GeoData bug save Failed!", "Ошибка сохранения бага геодаты!");
		}
	}

	@Override
	public int getWaterLevel(int x, int y, int z)
	{
		return NgetWaterLevel(x - L2World.MAP_MIN_X >> 4, y - L2World.MAP_MIN_Y >> 4, z);
	}

	private boolean canSeeTarget(int x, int y, int z, int tx, int ty, int tz)
	{
		if(tz >= z)
			return canSee(x - L2World.MAP_MIN_X >> 4, y - L2World.MAP_MIN_Y >> 4, z, tx - L2World.MAP_MIN_X >> 4, ty - L2World.MAP_MIN_Y >> 4, tz);
		return canSee(tx - L2World.MAP_MIN_X >> 4, ty - L2World.MAP_MIN_Y >> 4, tz, x - L2World.MAP_MIN_X >> 4, y - L2World.MAP_MIN_Y >> 4, z);
	}

	private boolean canMoveToTarget(int x, int y, int z, int tx, int ty, int tz)
	{
		return canMove(x - L2World.MAP_MIN_X >> 4, y - L2World.MAP_MIN_Y >> 4, z, tx - L2World.MAP_MIN_X >> 4, ty - L2World.MAP_MIN_Y >> 4, tz);
	}

	private boolean canMoveToTargetWithCollision(int x, int y, int z, int tx, int ty, int tz)
	{
		return canMoveWithCollision(x - L2World.MAP_MIN_X >> 4, y - L2World.MAP_MIN_Y >> 4, z, tx - L2World.MAP_MIN_X >> 4, ty - L2World.MAP_MIN_Y >> 4, tz);
	}

	private static boolean canSee(int _x, int _y, int _z, int _tx, int _ty, int _tz)
	{
		int dx = _tx - _x;
		int dy = _ty - _y;
		int dz = _tz - _z;
		int inc_x = sign(dx);
		int inc_y = sign(dy);
		dx = Math.abs(dx);
		dy = Math.abs(dy);
		// Avoid too long check
		if(dx + dy > 300)
			return false;
		else if(dx + dy == 0)
			return true;
		float inc_z_for_x = dx == 0 ? 0 : dz / dx;
		float inc_z_for_y = dy == 0 ? 0 : dz / dy;
		int x = _x;
		int y = _y;
		int z = _z + 64;
		int next_x = x;
		int next_y = y;
		int next_z = z;
		if(dx >= dy) // dy/dx <= 1
		{
			int delta_A = 2 * dy;
			int d = delta_A - dx;
			int delta_B = delta_A - 2 * dx;
			for(int i = 0; i < dx; i++)
			{
				x = next_x;
				y = next_y;
				z = next_z;
				if(d > 0)
				{
					d += delta_B;
					next_x += inc_x;
					next_z += inc_z_for_x;
					next_y += inc_y;
					next_z += inc_z_for_y;
					if(!NLOS(x, y, z))
						return false;
				}
				else
				{
					d += delta_A;
					next_x += inc_x;
					next_z += inc_z_for_x;
					if(!NLOS(x, y, z))
						return false;
				}
			}
		}
		else
		{
			int delta_A = 2 * dx;
			int d = delta_A - dy;
			int delta_B = delta_A - 2 * dy;
			for(int i = 0; i < dy; i++)
			{
				x = next_x;
				y = next_y;
				z = next_z;
				if(d > 0)
				{
					d += delta_B;
					next_x += inc_x;
					next_z += inc_z_for_x;
					next_y += inc_y;
					next_z += inc_z_for_y;
					if(!NLOS(x, y, z))
						return false;
				}
				else
				{
					d += delta_A;
					next_y += inc_y;
					next_z += inc_z_for_y;
					if(!NLOS(x, y, z))
						return false;
				}
			}
		}
		return true;
	}

	public static boolean canMove(int x, int y, int z, int tx, int ty, @SuppressWarnings("unused")
	int tz)
	{
		int dx = tx - x;
		int dy = ty - y;
		int inc_x = sign(dx);
		int inc_y = sign(dy);
		dx = Math.abs(dx);
		dy = Math.abs(dy);
		// Avoid too long check
		//if(dx + dy > 300)
		//	return false;
		//else
		if(dx + dy == 0)
			return true;
		int next_x = x;
		int next_y = y;
		int next_z = z;
		if(dx >= dy) // dy/dx <= 1
		{
			int delta_A = 2 * dy;
			int d = delta_A - dx;
			int delta_B = delta_A - 2 * dx;
			for(int i = 0; i < dx; i++)
			{
				x = next_x;
				y = next_y;
				z = next_z;
				if(d > 0)
				{
					d += delta_B;
					next_x += inc_x;
					next_y += inc_y;
					next_z = NcanMoveNext(x, y, z, next_x, next_y);
					if(next_z == 0)
						return false;
				}
				else
				{
					d += delta_A;
					next_x += inc_x;
					next_z = NcanMoveNext(x, y, z, next_x, next_y);
					if(next_z == 0)
						return false;
				}
			}
		}
		else
		{
			int delta_A = 2 * dx;
			int d = delta_A - dy;
			int delta_B = delta_A - 2 * dy;
			for(int i = 0; i < dy; i++)
			{
				x = next_x;
				y = next_y;
				z = next_z;
				if(d > 0)
				{
					d += delta_B;
					next_x += inc_x;
					next_y += inc_y;
					next_z = NcanMoveNext(x, y, z, next_x, next_y);
					if(next_z == 0)
						return false;
				}
				else
				{
					d += delta_A;
					next_y += inc_y;
					next_z = NcanMoveNext(x, y, z, next_x, next_y);
					if(next_z == 0)
						return false;
				}
			}
		}
		return true;
	}

	public static boolean canMoveWithCollision(int x, int y, int z, int tx, int ty, @SuppressWarnings("unused")
	int tz)
	{
		int dx = tx - x;
		int dy = ty - y;
		int inc_x = sign(dx);
		int inc_y = sign(dy);
		dx = Math.abs(dx);
		dy = Math.abs(dy);
		// Avoid too long check
		//if(dx + dy > 300)
		//	return false;
		//else
		if(dx + dy == 0)
			return true;
		int next_x = x;
		int next_y = y;
		int next_z = z;
		if(dx >= dy) // dy/dx <= 1
		{
			int delta_A = 2 * dy;
			int d = delta_A - dx;
			int delta_B = delta_A - 2 * dx;
			for(int i = 0; i < dx; i++)
			{
				x = next_x;
				y = next_y;
				z = next_z;
				if(d > 0)
				{
					d += delta_B;
					next_x += inc_x;
					next_y += inc_y;
					next_z = NcanMoveNext(x, y, z, next_x, next_y);
					if(next_z == 0 || NcanMoveNext(x - 1, y, z, next_x - 1, next_y) == 0 || NcanMoveNext(x + 1, y, z, next_x + 1, next_y) == 0)
						return false;
				}
				else
				{
					d += delta_A;
					next_x += inc_x;
					next_z = NcanMoveNext(x, y, z, next_x, next_y);
					if(next_z == 0 || NcanMoveNext(x - 1, y, z, next_x - 1, next_y) == 0 || NcanMoveNext(x + 1, y, z, next_x + 1, next_y) == 0)
						return false;
				}
			}
		}
		else
		{
			int delta_A = 2 * dx;
			int d = delta_A - dy;
			int delta_B = delta_A - 2 * dy;
			for(int i = 0; i < dy; i++)
			{
				x = next_x;
				y = next_y;
				z = next_z;
				if(d > 0)
				{
					d += delta_B;
					next_x += inc_x;
					next_y += inc_y;
					next_z = NcanMoveNext(x, y, z, next_x, next_y);
					if(next_z == 0 || NcanMoveNext(x - 1, y, z, next_x - 1, next_y) == 0 || NcanMoveNext(x + 1, y, z, next_x + 1, next_y) == 0)
						return false;
				}
				else
				{
					d += delta_A;
					next_y += inc_y;
					next_z = NcanMoveNext(x, y, z, next_x, next_y);
					if(next_z == 0 || NcanMoveNext(x - 1, y, z, next_x - 1, next_y) == 0 || NcanMoveNext(x + 1, y, z, next_x + 1, next_y) == 0)
						return false;
				}
			}
		}
		return true;
	}

	private static Location MoveCheck(Location begin, Location end, int x, int y, int z, int tx, int ty, @SuppressWarnings("unused")
	int tz)
	{
		int dx = tx - x;
		int dy = ty - y;
		int inc_x = sign(dx);
		int inc_y = sign(dy);
		dx = Math.abs(dx);
		dy = Math.abs(dy);
		// Avoid too long check
		//if(dx + dy > 600)
		//	return begin;
		//else 
		if(dx + dy < 2 || dx == 2 && dy == 0 || dx == 0 && dy == 2)
			return begin;
		int prev_x = x;
		int prev_y = y;
		int prev_z = z;
		int next_x = x;
		int next_y = y;
		int next_z = z;
		if(dx >= dy) // dy/dx <= 1
		{
			int delta_A = 2 * dy;
			int d = delta_A - dx;
			int delta_B = delta_A - 2 * dx;
			for(int i = 0; i < dx; i++)
			{
				prev_x = x;
				prev_y = y;
				prev_z = z;
				x = next_x;
				y = next_y;
				z = next_z;
				if(d > 0)
				{
					d += delta_B;
					next_x += inc_x;
					next_y += inc_y;
					next_z = NcanMoveNext(x, y, z, next_x, next_y);
					if(next_z == 0)
						return new Location((prev_x << 4) + L2World.MAP_MIN_X, (prev_y << 4) + L2World.MAP_MIN_Y, prev_z);
				}
				else
				{
					d += delta_A;
					next_x += inc_x;
					next_z = NcanMoveNext(x, y, z, next_x, next_y);
					if(next_z == 0)
						return new Location((prev_x << 4) + L2World.MAP_MIN_X, (prev_y << 4) + L2World.MAP_MIN_Y, prev_z);
				}
			}
		}
		else
		{
			int delta_A = 2 * dx;
			int d = delta_A - dy;
			int delta_B = delta_A - 2 * dy;
			for(int i = 0; i < dy; i++)
			{
				prev_x = x;
				prev_y = y;
				prev_z = z;
				x = next_x;
				y = next_y;
				z = next_z;
				if(d > 0)
				{
					d += delta_B;
					next_x += inc_x;
					next_y += inc_y;
					next_z = NcanMoveNext(x, y, z, next_x, next_y);
					if(next_z == 0)
						return new Location((prev_x << 4) + L2World.MAP_MIN_X, (prev_y << 4) + L2World.MAP_MIN_Y, prev_z);
				}
				else
				{
					d += delta_A;
					next_y += inc_y;
					next_z = NcanMoveNext(x, y, z, next_x, next_y);
					if(next_z == 0)
						return new Location((prev_x << 4) + L2World.MAP_MIN_X, (prev_y << 4) + L2World.MAP_MIN_Y, prev_z);
				}
			}
		}
		return end;
	}

	public static Location MoveCheckForPathFind(int x, int y, int z, int tx, int ty, int tz)
	{
		int dx = tx - x;
		int dy = ty - y;
		int inc_x = sign(dx);
		int inc_y = sign(dy);
		dx = Math.abs(dx);
		dy = Math.abs(dy);
		// Avoid too long check
		int prev_x = x;
		int prev_y = y;
		int prev_z = z;
		int next_x = x;
		int next_y = y;
		int next_z = z;
		if(dx >= dy) // dy/dx <= 1
		{
			int delta_A = 2 * dy;
			int d = delta_A - dx;
			int delta_B = delta_A - 2 * dx;
			for(int i = 0; i < dx; i++)
			{
				prev_x = x;
				prev_y = y;
				prev_z = z;
				x = next_x;
				y = next_y;
				z = next_z;
				if(d > 0)
				{
					d += delta_B;
					next_x += inc_x;
					next_y += inc_y;
					next_z = NcanMoveNextForPathFind(x, y, z, next_x, next_y);
					if(next_z == 0 || NcanMoveNextForPathFind(x - 1, y, z, next_x - 1, next_y) == 0 || NcanMoveNextForPathFind(x + 1, y, z, next_x + 1, next_y) == 0)
						return new Location(prev_x, prev_y, prev_z);
				}
				else
				{
					d += delta_A;
					next_x += inc_x;
					next_z = NcanMoveNextForPathFind(x, y, z, next_x, next_y);
					if(next_z == 0 || NcanMoveNextForPathFind(x - 1, y, z, next_x - 1, next_y) == 0 || NcanMoveNextForPathFind(x + 1, y, z, next_x + 1, next_y) == 0)
						return new Location(prev_x, prev_y, prev_z);
				}
			}
		}
		else
		{
			int delta_A = 2 * dx;
			int d = delta_A - dy;
			int delta_B = delta_A - 2 * dy;
			for(int i = 0; i < dy; i++)
			{
				prev_x = x;
				prev_y = y;
				prev_z = z;
				x = next_x;
				y = next_y;
				z = next_z;
				if(d > 0)
				{
					d += delta_B;
					next_x += inc_x;
					next_y += inc_y;
					next_z = NcanMoveNextForPathFind(x, y, z, next_x, next_y);
					if(next_z == 0 || NcanMoveNextForPathFind(x - 1, y, z, next_x - 1, next_y) == 0 || NcanMoveNextForPathFind(x + 1, y, z, next_x + 1, next_y) == 0)
						return new Location(prev_x, prev_y, prev_z);
				}
				else
				{
					d += delta_A;
					next_y += inc_y;
					next_z = NcanMoveNextForPathFind(x, y, z, next_x, next_y);
					if(next_z == 0 || NcanMoveNextForPathFind(x - 1, y, z, next_x - 1, next_y) == 0 || NcanMoveNextForPathFind(x + 1, y, z, next_x + 1, next_y) == 0)
						return new Location(prev_x, prev_y, prev_z);
				}
			}
		}
		return new Location(tx, ty, tz);
	}

	/**
	 * Используется только для антипаровоза в AI
	 */
	private static Location MoveCheckForAI(Location begin, Location end, int x, int y, int z, int tx, int ty, @SuppressWarnings("unused")
	int tz)
	{
		int dx = tx - x;
		int dy = ty - y;
		int inc_x = sign(dx);
		int inc_y = sign(dy);
		dx = Math.abs(dx);
		dy = Math.abs(dy);
		if(dx + dy < 2 || dx == 2 && dy == 0 || dx == 0 && dy == 2)
			return begin;
		int prev_x = x;
		int prev_y = y;
		int prev_z = z;
		int next_x = x;
		int next_y = y;
		int next_z = z;
		if(dx >= dy) // dy/dx <= 1
		{
			int delta_A = 2 * dy;
			int d = delta_A - dx;
			int delta_B = delta_A - 2 * dx;
			for(int i = 0; i < dx; i++)
			{
				prev_x = x;
				prev_y = y;
				prev_z = z;
				x = next_x;
				y = next_y;
				z = next_z;
				if(d > 0)
				{
					d += delta_B;
					next_x += inc_x;
					next_y += inc_y;
					next_z = NcanMoveNextForAI(x, y, z, next_x, next_y);
					if(next_z == 0)
						return new Location((prev_x << 4) + L2World.MAP_MIN_X, (prev_y << 4) + L2World.MAP_MIN_Y, prev_z);
				}
				else
				{
					d += delta_A;
					next_x += inc_x;
					next_z = NcanMoveNextForAI(x, y, z, next_x, next_y);
					if(next_z == 0)
						return new Location((prev_x << 4) + L2World.MAP_MIN_X, (prev_y << 4) + L2World.MAP_MIN_Y, prev_z);
				}
			}
		}
		else
		{
			int delta_A = 2 * dx;
			int d = delta_A - dy;
			int delta_B = delta_A - 2 * dy;
			for(int i = 0; i < dy; i++)
			{
				prev_x = x;
				prev_y = y;
				prev_z = z;
				x = next_x;
				y = next_y;
				z = next_z;
				if(d > 0)
				{
					d += delta_B;
					next_x += inc_x;
					next_y += inc_y;
					next_z = NcanMoveNextForAI(x, y, z, next_x, next_y);
					if(next_z == 0)
						return new Location((prev_x << 4) + L2World.MAP_MIN_X, (prev_y << 4) + L2World.MAP_MIN_Y, prev_z);
				}
				else
				{
					d += delta_A;
					next_y += inc_y;
					next_z = NcanMoveNextForAI(x, y, z, next_x, next_y);
					if(next_z == 0)
						return new Location((prev_x << 4) + L2World.MAP_MIN_X, (prev_y << 4) + L2World.MAP_MIN_Y, prev_z);
				}
			}
		}
		return end;
	}

	private static byte sign(int x)
	{
		if(x >= 0)
			return +1;
		return -1;
	}

	// GeoEngine
	private static void NinitGeodata()
	{
		_log.info("Geo Engine: - Loading Geodata...");

		File f = new File("./data/geodata");

		if(!f.exists() || !f.isDirectory())
		{
			_log.info("Geo Engine: Files missing, loading aborted.");
			return;
		}

		Pattern p = Pattern.compile(Config.GEOFILES_PATTERN);
		for(File q : f.listFiles())
		{
			if(q.isHidden() || q.isDirectory())
				continue;

			String fn = q.getName();
			Matcher m = p.matcher(fn);
			if(m.matches())
			{
				fn = fn.substring(0, 5); // обрезаем .l2j
				String[] xy = fn.split("_");
				LoadGeodataFile(Byte.parseByte(xy[0]), Byte.parseByte(xy[1]));
			}
		}
		try
		{
			File geo_bugs = new File("./data/geodata/geo_bugs.txt");

			_geo_bugs_out = new BufferedOutputStream(new FileOutputStream(geo_bugs, true));
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new Error("Failed to Load geo_bugs.txt File.");
		}
	}

	public static void unloadGeodata(byte rx, byte ry)
	{
		Geodata_index[rx - 16][ry - 10] = null;
		Geodata[rx - 16][ry - 10] = null;
	}

	public static void unloadGeodata()
	{
		Geodata_index = new int[11][16][];
		Geodata = new byte[11][16][];
		_instance = null;
	}

	public static boolean LoadGeodataFile(byte rx, byte ry)
	{
		String fname = "./data/geodata/" + rx + "_" + ry + ".l2j";

		int ix = rx - 16;
		int iy = ry - 10;

		if(ix < 0 || iy < 0 || ix > 10 || iy > 15)
			return false;

		_log.info("Geo Engine: - Loading: " + fname);

		File Geo = new File(fname);
		int size, index = 0, block = 0, flor = 0;
		try
		{
			// Create a read-only memory-mapped file
			FileChannel roChannel = new RandomAccessFile(Geo, "r").getChannel();
			size = (int) roChannel.size();

			ByteBuffer buffer = roChannel.map(FileChannel.MapMode.READ_ONLY, 0, size);
			roChannel.close();
			buffer.order(ByteOrder.LITTLE_ENDIAN);
			byte[] geo = new byte[buffer.remaining()];
			buffer.get(geo, 0, geo.length);

			if(size > 196608)
			{
				// Indexing geo files, so we will know where each block starts
				int[] indexs = new int[65536];
				while(block < 65536)
				{
					byte type = geo[index];
					indexs[block] = index;
					block++;
					index++;
					if(type == 0)
						index += 2; // 1x short
					else if(type == 1)
						index += 128; // 64 x short
					else
					{
						int b;
						for(b = 0; b < 64; b++)
						{
							byte layers = geo[index];
							index += (layers << 1) + 1;
							if(layers > flor)
								flor = layers;
						}
					}
				}

				Geodata_index[ix][iy] = indexs;
			}

			Geodata[ix][iy] = geo;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			_log.warning("Failed to Load GeoFile at block: " + block + "\n");
			return false;
		}
		return true;
	}

	/**
	 * @param pos
	 * @return Block Index: 0-255
	 */
	private static int getBlock(int geo_pos)
	{
		return (geo_pos >> 3) % 256;
	}

	/**
	 * @param pos
	 * @return Cell Index: 0-7
	 */
	private static int getCell(int geo_pos)
	{
		return geo_pos % 8;
	}

	// Geodata Functions

	/**
	 * @param x
	 * @param y
	 * @return Type of geo_block: 0-2
	 */
	private static short NgetType(int x, int y)
	{
		int ix = x >> 11;
		int iy = y >> 11;
		int blockX = getBlock(x);
		int blockY = getBlock(y);
		int index;
		// Geodata without index - it is just empty so index can be calculated on the fly
		if(Geodata_index[ix][iy] == null)
			index = ((blockX << 8) + blockY) * 3;
		// Get Index for current block of current geodata region
		else
			index = Geodata_index[ix][iy][(blockX << 8) + blockY];
		// Buffer that Contains current Region GeoData
		byte[] geo = Geodata[ix][iy];
		if(geo == null)
			return 0;
		if(index >= geo.length)
			return 0;
		return geo[index];
	}

	/**
	 * @param x
	 * @param y
	 * @param z
	 * @return Nearlest Z
	 */
	public static int NgetHeight(int geox, int geoy, int z)
	{
		int ix = geox >> 11;
		int iy = geoy >> 11;
		int blockX = getBlock(geox);
		int blockY = getBlock(geoy);
		int cellX, cellY, index;
		// Get Index for current block of current region geodata
		// Geodata without index - it is just empty so index can be calculated on the fly
		if(Geodata_index[ix][iy] == null)
			index = ((blockX << 8) + blockY) * 3;
		else
			index = Geodata_index[ix][iy][(blockX << 8) + blockY];
		// Buffer that Contains current Region GeoData
		byte[] geo = Geodata[ix][iy];
		if(geo == null)
			return z;
		if(index >= geo.length)
			return z;
		// Read current block type: 0 - flat, 1 - complex, 2 - multilevel
		byte type = geo[index];
		index++;
		if(type == 0)
		{
			short height = makeShort(geo[index + 1], geo[index]);
			return (short) (height & 0x0fff0);
		}
		else if(type == 1)
		{ // complex
			cellX = getCell(geox);
			cellY = getCell(geoy);
			index += (cellX << 3) + cellY << 1;
			short height = makeShort(geo[index + 1], geo[index]);
			height = (short) (height & 0x0fff0);
			height = (short) (height >> 1); // height / 2
			return height;
		}
		else
		{ // multilevel
			cellX = getCell(geox);
			cellY = getCell(geoy);
			int offset = (cellX << 3) + cellY;
			while(offset > 0)
			{
				byte lc = geo[index];
				index += (lc << 1) + 1;
				offset--;
			}
			byte layers = geo[index];
			index++;
			short height = -1;
			if(layers <= 0 || layers > 125)
				return (short) z;
			short temph = Short.MIN_VALUE;
			while(layers > 0)
			{
				height = makeShort(geo[index + 1], geo[index]);
				height = (short) (height & 0x0fff0);
				height = (short) (height >> 1); // height / 2
				if((z - temph) * (z - temph) > (z - height) * (z - height))
					temph = height;
				layers--;
				index += 2;
			}
			return temph;
		}
	}

	/**
	 * @param x
	 * @param y
	 * @param zmin
	 * @param zmax
	 * @return Z betwen zmin and zmax
	 */
	private static int NgetSpawnHeight(int geox, int geoy, int zmin, int zmax)
	{
		int ix = geox >> 11;
		int iy = geoy >> 11;
		int blockX = getBlock(geox);
		int blockY = getBlock(geoy);
		int cellX, cellY, index;
		short temph = Short.MIN_VALUE;
		// Get Index for current block of current region geodata
		// Geodata without index - it is just empty so index can be calculated on the fly
		if(Geodata_index[ix][iy] == null)
			index = ((blockX << 8) + blockY) * 3;
		else
			index = Geodata_index[ix][iy][(blockX << 8) + blockY];
		// Buffer that Contains current Region GeoData
		byte[] geo = Geodata[ix][iy];
		if(geo == null)
			return zmin;
		if(index >= geo.length)
			return zmin;
		// Read current block type: 0 - flat, 1 - complex, 2 - multilevel
		byte type = geo[index];
		index++;
		if(type == 0)
			temph = makeShort(geo[index + 1], geo[index]);
		else if(type == 1)
		{ // complex
			cellX = getCell(geox);
			cellY = getCell(geoy);
			index += (cellX << 3) + cellY << 1;
			short height = makeShort(geo[index + 1], geo[index]);
			height = (short) (height & 0x0fff0);
			height = (short) (height >> 1); // height / 2
			temph = height;
		}
		else
		{ // multilevel
			cellX = getCell(geox);
			cellY = getCell(geoy);
			short height;
			int offset = (cellX << 3) + cellY;
			while(offset > 0)
			{
				byte lc = geo[index];
				index += (lc << 1) + 1;
				offset--;
			}
			// Read current block type: 0 - flat, 1 - complex, 2 - multilevel
			byte layers = geo[index];
			index++;
			if(layers <= 0 || layers > 125)
				return zmin;
			while(layers > 0)
			{
				height = makeShort(geo[index + 1], geo[index]);
				height = (short) (height & 0x0fff0);
				height = (short) (height >> 1); // height / 2
				if((zmin - temph) * (zmin - temph) > (zmin - height) * (zmin - height))
					temph = height;
				layers--;
				index += 2;
			}
			if(temph > zmax + 150 || temph < zmin - 150)
				return temph;
		}
		return temph;
	}

	public static int NcanMoveNext(int x, int y, int z, int next_x, int next_y)
	{
		Layer[] layers1 = NGetLayers(x, y);
		Layer[] layers2 = NGetLayers(next_x, next_y);

		if(layers1.length == 0 || layers2.length == 0)
			return z;

		short z1 = Short.MIN_VALUE;
		short z2 = Short.MIN_VALUE;
		short NSWE1 = 15;

		for(Layer layer : layers1)
			if(Math.abs(z - z1) > Math.abs(z - layer.height))
			{
				z1 = layer.height;
				NSWE1 = layer.nswe;
			}

		for(Layer layer : layers2)
			if(Math.abs(z + 16 - z2) > Math.abs(z + 16 - layer.height))
				z2 = layer.height;

		if(z2 < -30000)
			return 0;

		if(z2 > z1 && z2 - z1 >= 64)
			return 0;

		if(CheckNSWE(NSWE1, x, y, next_x, next_y))
			return z2;

		return 0;
	}

	public static int NcanMoveNextForPathFind(int x, int y, int z, int next_x, int next_y)
	{
		Layer[] layers1 = NGetLayers(x, y);
		Layer[] layers2 = NGetLayers(next_x, next_y);

		if(layers1.length == 0 || layers2.length == 0)
			return 0;

		short z1 = Short.MIN_VALUE;
		short z2 = Short.MIN_VALUE;
		short NSWE1 = 15;
		short NSWE2 = 15;

		for(Layer layer : layers1)
			if(Math.abs(z - z1) > Math.abs(z - layer.height))
			{
				z1 = layer.height;
				NSWE1 = layer.nswe;
			}

		for(Layer layer : layers2)
			if(Math.abs(z + 16 - z2) > Math.abs(z + 16 - layer.height))
			{
				z2 = layer.height;
				NSWE2 = layer.nswe;
			}

		if(z2 < -30000)
			return 0;

		if(Math.abs(z2 - z1) >= 64)
			return 0;

		if(CheckNSWE(NSWE1, x, y, next_x, next_y) && CheckNSWE(NSWE2, next_x, next_y, x, y))
			return z2;

		return 0;
	}

	/**
	 * Используется только для антипаровоза в AI
	 */
	public static int NcanMoveNextForAI(int x, int y, int z, int next_x, int next_y)
	{
		Layer[] layers1 = NGetLayers(x, y);
		Layer[] layers2 = NGetLayers(next_x, next_y);

		if(layers1.length == 0 || layers2.length == 0)
			return z;

		short z1 = Short.MIN_VALUE;
		short z2 = Short.MIN_VALUE;
		short NSWE1 = 15;
		short NSWE2 = 15;

		for(Layer layer : layers1)
			if(Math.abs(z - z1) > Math.abs(z - layer.height))
			{
				z1 = layer.height;
				NSWE1 = layer.nswe;
			}

		for(Layer layer : layers2)
			if(Math.abs(z + 16 - z2) > Math.abs(z + 16 - layer.height))
			{
				z2 = layer.height;
				NSWE2 = layer.nswe;
			}

		if(z2 < -30000)
			return 0;

		if(z2 > z1 && z2 - z1 >= 32)
			return 0;

		if(z1 > z2 && z1 - z2 >= 32)
			return 0;

		if(CheckNSWE(NSWE1, x, y, next_x, next_y) && CheckNSWE(NSWE2, next_x, next_y, x, y))
			return z2;

		return 0;
	}

	private static boolean NLOS(int ray_x, int ray_y, int ray_z)
	{
		Layer[] layers = NGetLayers(ray_x, ray_y);
		if(layers.length == 0)
			return true;
		short geo_z = Short.MIN_VALUE;
		for(Layer layer : layers)
			if(Math.abs(ray_z - geo_z) > Math.abs(ray_z - layer.height))
				geo_z = layer.height;
		return ray_z >= geo_z;
	}

	private static Layer[] NGetLayers(int x, int y)
	{
		int ix = x >> 11;
		int iy = y >> 11;
		int blockX = getBlock(x);
		int blockY = getBlock(y);
		int cellX, cellY;
		int index;
		// Get Index for current block of current region geodata
		// Geodata without index - it is just empty so index can be calculated on the fly
		if(Geodata_index[ix][iy] == null)
			index = ((blockX << 8) + blockY) * 3;
		else
			index = Geodata_index[ix][iy][(blockX << 8) + blockY];
		// Buffer that Contains current Region GeoData
		byte[] geo = Geodata[ix][iy];
		if(geo == null)
			return new Layer[0];
		if(index >= geo.length)
			return new Layer[0];
		// Read current block type: 0 - flat, 1 - complex, 2 - multilevel
		byte type = geo[index];
		index++;
		if(type == 0)
		{
			short height = makeShort(geo[index + 1], geo[index]);
			height = (short) (height & 0x0fff0);
			return new Layer[] { new Layer(height, (short) 15) };
		}
		else if(type == 1)
		{ // complex
			cellX = getCell(x);
			cellY = getCell(y);
			index += (cellX << 3) + cellY << 1;
			short height = makeShort(geo[index + 1], geo[index]);
			return new Layer[] { new Layer((short) ((short) (height & 0x0fff0) >> 1), (short) (height & 0x0F)) };
		}
		else
		{ // multilevel
			cellX = getCell(x);
			cellY = getCell(y);
			int offset = (cellX << 3) + cellY;
			while(offset > 0)
			{
				byte lc = geo[index];
				index += (lc << 1) + 1;
				offset--;
			}
			byte layer_count = geo[index];
			index++;
			if(layer_count <= 0 || layer_count > 125)
				return new Layer[0];
			Layer[] layers = new Layer[layer_count];
			short height;
			while(layer_count > 0)
			{
				height = makeShort(geo[index + 1], geo[index]);
				layer_count--;
				layers[layer_count] = new Layer((short) ((short) (height & 0x0fff0) >> 1), (short) (height & 0x0F));
				index += 2;
			}
			return layers;
		}
	}

	/**
	 * @param x
	 * @param y
	 * @param z
	 * @return NSWE: 0-15
	 */
	public static short NgetNSWE(int x, int y, int z)
	{
		int ix = x >> 11;
		int iy = y >> 11;
		int blockX = getBlock(x);
		int blockY = getBlock(y);
		int cellX, cellY;
		short NSWE = 0;
		int index;
		// Geodata without index - it is just empty so index can be calculated on the fly
		if(Geodata_index[ix][iy] == null)
			index = ((blockX << 8) + blockY) * 3;
		// Get Index for current block of current region geodata
		else
			index = Geodata_index[ix][iy][(blockX << 8) + blockY];
		// Buffer that Contains current Region GeoData
		byte[] geo = Geodata[ix][iy];
		if(geo == null)
			return 15;
		if(index >= geo.length)
			return 15;
		//Read current block type: 0 - flat, 1 - complex, 2 - multilevel
		byte type = geo[index];
		index++;
		if(type == 0)
			return 15;
		else if(type == 1)
		{ // complex
			cellX = getCell(x);
			cellY = getCell(y);
			index += (cellX << 3) + cellY << 1;
			short height = makeShort(geo[index + 1], geo[index]);
			NSWE = (short) (height & 0x0F);
		}
		else
		{ // multilevel
			cellX = getCell(x);
			cellY = getCell(y);
			int offset = (cellX << 3) + cellY;
			while(offset > 0)
			{
				byte lc = geo[index];
				index += (lc << 1) + 1;
				offset--;
			}
			byte layers = geo[index];
			index++;
			short height;
			if(layers <= 0 || layers > 125)
				return 15;
			short tempz = Short.MIN_VALUE;
			while(layers > 0)
			{
				height = makeShort(geo[index + 1], geo[index]);
				height = (short) (height & 0x0fff0);
				height = (short) (height >> 1); // height / 2                        

				if((z - tempz) * (z - tempz) > (z - height) * (z - height))
				{
					tempz = height;
					NSWE = makeShort(geo[index + 1], geo[index]);
					NSWE = (short) (NSWE & 0x0F);
				}
				layers--;
				index += 2;
			}
		}
		return NSWE;
	}

	/**
	 * @param NSWE
	 * @param x
	 * @param y
	 * @param tx
	 * @param ty
	 * @return True if NSWE dont block given direction
	 */
	public static boolean CheckNSWE(short NSWE, int x, int y, int tx, int ty)
	{
		// Check NSWE
		if(NSWE == 15)
			return true;
		if(tx > x) // E
		{
			if((NSWE & E) == 0)
				return false;
		}
		else if(tx < x)
			if((NSWE & W) == 0)
				return false;
		if(ty > y) // S
		{
			if((NSWE & S) == 0)
				return false;
		}
		else if(ty < y)
			if((NSWE & N) == 0)
				return false;
		return true;
	}

	private static int NgetWaterLevel(int geox, int geoy, int z)
	{
		int ix = geox >> 11;
		int iy = geoy >> 11;
		int blockX = getBlock(geox);
		int blockY = getBlock(geoy);
		// int cellX, cellY
		int index;
		// Geodata without index - it is just empty so index can be calculated on the fly
		if(Geodata_index[ix][iy] == null)
			index = ((blockX << 8) + blockY) * 3;
		// Get Index for current block of current region geodata
		else
			index = Geodata_index[ix][iy][(blockX << 8) + blockY];
		// Buffer that Contains current Region GeoData
		byte[] geo = Geodata[ix][iy];
		if(geo == null)
			return z;
		if(index >= geo.length)
			return z;
		// Read current block type: 0 - flat, 1 - complex, 2 - multilevel
		byte type = geo[index];
		index += 2;
		if(type == 0)
			return makeShort(geo[index + 1], geo[index]);
		return Short.MIN_VALUE;
	}

	private static void NgetInfo(L2Player gm, int geox, int geoy, int z)
	{
		gm.sendMessage("geox: " + geox, "");
		gm.sendMessage("geoy: " + geoy, "");
		gm.sendMessage("z: " + z, "");

		int ix = geox >> 11;
		int iy = geoy >> 11;
		int blockX = getBlock(geox);
		int blockY = getBlock(geoy);
		int cellX, cellY, index;
		// Get Index for current block of current region geodata
		// Geodata without index - it is just empty so index can be calculated on the fly
		if(Geodata_index[ix][iy] == null)
			index = ((blockX << 8) + blockY) * 3;
		else
			index = Geodata_index[ix][iy][(blockX << 8) + blockY];
		// Buffer that Contains current Region GeoData
		byte[] geo = Geodata[ix][iy];
		if(geo == null)
			return;
		if(index >= geo.length)
		{
			gm.sendMessage("Incorrect index for geo");
			return;
		}
		// Read current block type: 0 - flat, 1 - complex, 2 - multilevel
		byte type = geo[index];
		index++;
		if(type == 0)
		{ // flat
			short height = makeShort(geo[index + 1], geo[index]);
			height = (short) (height & 0x0fff0);

			gm.sendMessage("type: " + type, "");
			gm.sendMessage("index: " + index, "");
			gm.sendMessage("height: " + height, "");
		}
		else if(type == 1)
		{ // complex
			cellX = getCell(geox);
			cellY = getCell(geoy);
			index += (cellX << 3) + cellY << 1;
			short height = makeShort(geo[index + 1], geo[index]);
			height = (short) (height & 0x0fff0);
			height = (short) (height >> 1); // height / 2

			gm.sendMessage("type: " + type, "");
			gm.sendMessage("index: " + index, "");
			gm.sendMessage("height: " + height, "");
		}
		else
		{ // multilevel
			cellX = getCell(geox);
			cellY = getCell(geoy);
			int offset = (cellX << 3) + cellY;
			while(offset > 0)
			{
				byte lc = geo[index];
				index += (lc << 1) + 1;
				offset--;
			}
			byte layers = geo[index];
			index++;

			gm.sendMessage("type: " + type, "");
			gm.sendMessage("index: " + index, "");
			gm.sendMessage("layers: " + layers, "");

			short height;
			if(layers <= 0 || layers > 125)
				gm.sendMessage("Geo Engine: - invalid layers count: " + layers + " at: " + geox + " " + geoy, "");
			while(layers > 0)
			{
				height = makeShort(geo[index + 1], geo[index]);
				height = (short) (height & 0x0fff0);
				height = (short) (height >> 1); // height / 2
				gm.sendMessage("height: " + height, "");
				layers--;
				index += 2;
			}
		}
	}

	public static class Layer
	{
		public short height;
		public short nswe;

		public Layer(short h, short n)
		{
			height = h;
			nswe = n;
		}
	}

	private static short makeShort(byte b1, byte b0)
	{
		return (short) (b1 << 8 | b0 & 0xff);
	}
}
