package L2jFT.game.geodata;

import java.util.ArrayList;

import L2jFT.Config;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2World;
import L2jFT.util.Location;

/**
 * @Author: Diamond
 * @Date: 20/5/2007
 * @Time: 9:57:48
 */
public class PathFind
{
	private static final byte EAST = 1, WEST = 2, SOUTH = 4, NORTH = 8;

	public GeoNode[][] map = new GeoNode[250][250];

	private GeoNode[] mapFast = new GeoNode[62500];
	private int mapFastIndex = 0;

	private int offsetX;
	private int offsetY;

	private Location startpoint;
	private Location endpoint;

	private L2Character _actor;

	ArrayList<GeoNode> path;

	long time;

	public PathFind(L2Character actor, int destX, int destY, int destZ)
	{
		time = System.currentTimeMillis();
		_actor = actor;

		startpoint = new Location(actor.getX() - L2World.MAP_MIN_X >> 4, actor.getY() - L2World.MAP_MIN_Y >> 4, actor.getZ());
		endpoint = new Location(destX - L2World.MAP_MIN_X >> 4, destY - L2World.MAP_MIN_Y >> 4, destZ);

		if(Config.PATHFINDING_BOOST)
		{
			startpoint = GeoEngine.MoveCheckForPathFind(startpoint.x, startpoint.y, startpoint.z, endpoint.x, endpoint.y, endpoint.z);
			endpoint = GeoEngine.MoveCheckForPathFind(endpoint.x, endpoint.y, endpoint.z, startpoint.x, startpoint.y, startpoint.z);
		}

		startpoint.z = GeoEngine.NgetHeight(startpoint.x, startpoint.y, startpoint.z);
		endpoint.z = GeoEngine.NgetHeight(endpoint.x, endpoint.y, endpoint.z);

		offsetX = startpoint.x - 125;
		offsetY = startpoint.y - 125;

		path = findPath();
	}

	private GeoNode getBestOpenNode()
	{
		GeoNode bestNode = null;

		for(int i = 0; i < mapFastIndex; i++)
		{
			GeoNode n = mapFast[i];
			if(n.closed)
				continue;
			if(bestNode == null)
				bestNode = n;
			else if(n.score < bestNode.score)
				bestNode = n;
		}

		return bestNode;
	}

	private ArrayList<GeoNode> tracePath(GeoNode f)
	{
		ArrayList<GeoNode> nodes = new ArrayList<GeoNode>();
		GeoNode parent = f.parent;
		nodes.add(0, f);

		while(true)
		{
			if(parent.parent == null)
				break;
			nodes.add(0, parent);
			parent = parent.parent;
		}
		return nodes;
	}

	public ArrayList<GeoNode> findPath()
	{
		GeoNode n = new GeoNode(startpoint, 0, null);
		map[startpoint.x - offsetX][startpoint.y - offsetY] = n;
		mapFast[mapFastIndex] = n;
		mapFastIndex++;

		GeoNode nextNode;
		nextNode = getBestOpenNode();
		GeoNode finish;

		while(nextNode != null)
		{
			if(Config.PATHFINDING_BOOST)
				if(GeoEngine.canMove(nextNode.location.x, nextNode.location.y, nextNode.location.z, endpoint.x, endpoint.y, endpoint.z))
					return tracePath(new GeoNode(endpoint, nextNode.moveCost, nextNode));
			finish = handleNode(nextNode);
			if(finish != null)
				return tracePath(finish);
			// Ограничение по времени поиска
			if(_actor.isPlayer ? System.currentTimeMillis() - time > 50 : System.currentTimeMillis() - time > 25)
				return null;
			nextNode = getBestOpenNode();
		}
		return null;
	}

	public GeoNode handleNode(GeoNode node)
	{
		node.closed = true;

		Location cl = node.location;
		GeoNode result = null;

		int clX = cl.x;
		int clY = cl.y;
		int clZ = cl.z;
		short NSWE;

		NSWE = GeoEngine.NgetNSWE(clX, clY, clZ);

		if((NSWE & EAST) == EAST)
			result = getNeighbour(clX + 1, clY, node);

		if(result != null)
			return result;

		if((NSWE & WEST) == WEST)
			result = getNeighbour(clX - 1, clY, node);

		if(result != null)
			return result;

		if((NSWE & SOUTH) == SOUTH)
			result = getNeighbour(clX, clY + 1, node);

		if(result != null)
			return result;

		if((NSWE & NORTH) == NORTH)
			result = getNeighbour(clX, clY - 1, node);

		return result;
	}

	public GeoNode getNeighbour(int x, int y, GeoNode from)
	{
		if(x - offsetX > 249 || x - offsetX < 0 || y - offsetY > 249 || y - offsetY < 0)
			return null;

		int z = GeoEngine.NgetHeight(x, y, from.location.z);

		if(map[x - offsetX][y - offsetY] != null && map[x - offsetX][y - offsetY].closed)
			return null;

		int height = Math.abs(z - from.location.z);

		if(height >= 64)
			return null;

		int weight = 0;

		if(_actor.isPlayer)
			if(GeoEngine.NgetNSWE(x, y, z) != 15 || height > 8)
				weight = 8;
			else if(GeoEngine.NgetNSWE(x + 1, y, z) != 15 || Math.abs(z - GeoEngine.NgetHeight(x + 1, y, z)) > 8)
				weight = 4;
			else if(GeoEngine.NgetNSWE(x - 1, y, z) != 15 || Math.abs(z - GeoEngine.NgetHeight(x - 1, y, z)) > 8)
				weight = 4;
			else if(GeoEngine.NgetNSWE(x, y + 1, z) != 15 || Math.abs(z - GeoEngine.NgetHeight(x, y + 1, z)) > 8)
				weight = 4;
			else if(GeoEngine.NgetNSWE(x, y - 1, z) != 15 || Math.abs(z - GeoEngine.NgetHeight(x, y - 1, z)) > 8)
				weight = 4;

		int dx = Math.abs(endpoint.x - x);
		int dy = Math.abs(endpoint.y - y);
		int dz = Math.abs(endpoint.z - z) / 16;

		GeoNode n = new GeoNode(new Location(x, y, z), 1, from);

		n.moveCost += from.moveCost + weight;
		n.score = n.moveCost + (int) Math.sqrt(dx * dx + dy * dy + dz * dz);

		if(n.location.equals(endpoint))
			return n;

		if(n.score < 200)
		{
			GeoNode on = map[x - offsetX][y - offsetY];
			if(on == null || n.moveCost < on.moveCost)
			{
				map[x - offsetX][y - offsetY] = n;
				mapFast[mapFastIndex] = n;
				mapFastIndex++;
			}
		}

		return null;
	}

	public class GeoNode
	{
		public GeoNode parent = null;
		public Location location = null;
		public int moveCost = 0;
		public int score = 0;
		public boolean closed = false;

		public GeoNode(Location loc, int mCost, GeoNode pNode)
		{
			location = loc;
			moveCost = mCost;
			parent = pNode;
		}
	}
}
