package L2jFT.game.handler;

import L2jFT.Config;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.logging.Logger;

public class AdminCommandHandler
{
	private static Logger _log = Logger.getLogger(AdminCommandHandler.class.getName());
	private static AdminCommandHandler _instance;
	private Map<String, IAdminCommandHandler> _datatable;

	public static final String GROUP_NONE = "none";
	public static final String GROUP_HELP = "help";
	public static final String GROUP_TELEPORT = "teleport";
	public static final String GROUP_CW = "CursedWeapons";
	public static final String GROUP_BANS = "bans";
	public static final String GROUP_EDITCHAR = "EditChar";
	public static final String GROUP_MANAGESERVER = "ManageServer";
    public static final String GROUP_MANAGESERVER1 = "ManageServer_1";
    public static final String GROUP_MANAGESERVER2 = "ManageServer_2";
    public static final String GROUP_MANAGESERVER3 = "ManageServer_3";
    public static final String GROUP_MANAGESERVER4 = "ManageServer_4";
    public static final String GROUP_MANAGESERVER5 = "ManageServer_5";
    public static final String GROUP_MANAGESERVER6 = "ManageServer_6";

    public static AdminCommandHandler getInstance()
	{
		if(_instance == null)
			_instance = new AdminCommandHandler();
		return _instance;
	}

	private AdminCommandHandler()
	{
		_datatable = new HashMap<String, IAdminCommandHandler>();
	}

	public void registerAdminCommandHandler(IAdminCommandHandler handler)
	{
		try
		{
			if(handler.getAdminCommandList().length != handler.getCommandGroup().length || handler.getAdminCommandList().length != handler.getDescription().length)
				//_log.warning("Incorrect help imlemented in class: "+handler.getClass().getName());
				System.out.println("Incorrect help imlemented in class: " + handler.getClass().getName());
		}
		catch(Exception e)
		{
			System.out.println("Null help in class: " + handler.getClass().getName());
		}

		String[] ids = handler.getAdminCommandList();
		for(String element : ids)
		{
			if(Config.DEBUG)
				_log.fine("Adding handler for command " + element);
			_datatable.put(element.toLowerCase(), handler);
		}
	}

	public IAdminCommandHandler getAdminCommandHandler(String adminCommand)
	{
		String command = adminCommand;
		if(adminCommand.indexOf(" ") != -1)
			command = adminCommand.substring(0, adminCommand.indexOf(" "));
		if(Config.DEBUG)
			_log.fine("getting handler for command: " + command + " -> " + (_datatable.get(command) != null));
		return _datatable.get(command);
	}

	/**
	 * Алгоритм не оптимальный, но затраты на построение списка команд мизерны
	 * по сравнению со всем другим
	 * @param command команда для поиска хелпа
	 * @return хелп к команде
	 */
	public String getDescriptionForCommand(String command)
	{
		String hlp = "Описание отсутсвует";

		IAdminCommandHandler handler = _datatable.get(command);
		for(int i = 0; i < handler.getAdminCommandList().length; i++)
			if(handler.getAdminCommandList()[i].equals(command))
			{
				String[] descList = handler.getDescription();
				return descList != null ? descList[i] : hlp;
			}
		return hlp;
	}

	/**
	 * Ищет группу к команде
	 * @param command команда для поиска группы
	 * @return группа команды
	 */
	public String getGroupForCommand(String command)
	{
		String group = GROUP_NONE;

		IAdminCommandHandler handler = _datatable.get(command);
		for(int i = 0; i < handler.getAdminCommandList().length; i++)
			if(handler.getAdminCommandList()[i].equals(command))
			{
				String[] groupList = handler.getCommandGroup();
				return groupList != null ? groupList[i] : group;
			}
		return group;
	}

	/**
	 * @return размер комманд
	 */
	public int size()
	{
		return _datatable.size();
	}

	public void clear()
	{
		_datatable.clear();
	}

	/**
	 * Получение списка зарегистрированных админ команд
	 * @return список команд
	 */
	public Set<String> getAllCommands()
	{
		return _datatable.keySet();
	}
}
