package L2jFT.game.instancemanager;

import javolution.util.FastList;
import javolution.util.FastMap;
import L2jFT.game.GameTimeController;
import L2jFT.game.model.L2Spawn;
import L2jFT.game.model.instances.L2MonsterInstance;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.model.instances.L2RaidBossInstance;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

public class DayNightSpawnManager
{

	private static Logger _log = Logger.getLogger(DayNightSpawnManager.class.getName());

	private static DayNightSpawnManager _instance;
	private static HashMap<Integer, FastMap<L2Spawn, FastList<L2NpcInstance>>> _dayMobs;
	private static HashMap<Integer, FastMap<L2Spawn, FastList<L2NpcInstance>>> _nightMobs;
	private static Map<L2Spawn, L2RaidBossInstance> _bosses;

	private static final FastList<L2NpcInstance> _emptyMobs = new FastList<L2NpcInstance>();

	private static int _currentState; // 0 = Undefined, 1 = Day, 2 = Night

	public static DayNightSpawnManager getInstance()
	{
		if(_instance == null)
			_instance = new DayNightSpawnManager();

		return _instance;
	}

	public DayNightSpawnManager()
	{
		_dayMobs = new HashMap<Integer, FastMap<L2Spawn, FastList<L2NpcInstance>>>();
		_nightMobs = new HashMap<Integer, FastMap<L2Spawn, FastList<L2NpcInstance>>>();
		_bosses = new FastMap<L2Spawn, L2RaidBossInstance>();

		_currentState = 0;
	}

	public void addDayMob(L2Spawn spawnDat)
	{
		if(_dayMobs.containsKey(spawnDat.getId()))
			return;

		FastMap<L2Spawn, FastList<L2NpcInstance>> emptySpawns = new FastMap<L2Spawn, FastList<L2NpcInstance>>();
		emptySpawns.put(spawnDat, _emptyMobs);
		_dayMobs.put(spawnDat.getId(), emptySpawns);
	}

	public void addNightMob(L2Spawn spawnDat)
	{
		if(_nightMobs.containsKey(spawnDat.getId()))
			return;

		FastMap<L2Spawn, FastList<L2NpcInstance>> emptySpawns = new FastMap<L2Spawn, FastList<L2NpcInstance>>();
		emptySpawns.put(spawnDat, _emptyMobs);
		_nightMobs.put(spawnDat.getId(), emptySpawns);
	}

	public HashMap<Integer, FastMap<L2Spawn, FastList<L2NpcInstance>>> deleteMobs(HashMap<Integer, FastMap<L2Spawn, FastList<L2NpcInstance>>> mobsSpawnsList)
	{
		if(mobsSpawnsList.size() == 0)
			return mobsSpawnsList;

		HashMap<Integer, FastMap<L2Spawn, FastList<L2NpcInstance>>> newMobsSpawnsList = new HashMap<Integer, FastMap<L2Spawn, FastList<L2NpcInstance>>>();

		for(int key : mobsSpawnsList.keySet())
		{
			FastMap<L2Spawn, FastList<L2NpcInstance>> tmpSpawnsList = new FastMap<L2Spawn, FastList<L2NpcInstance>>();
			for(L2Spawn spawnDat : mobsSpawnsList.get(key).keySet())
			{
				FastList<L2NpcInstance> mobsList = mobsSpawnsList.get(key).get(spawnDat);
				FastList<L2NpcInstance> tmpMobsList = new FastList<L2NpcInstance>();
				tmpMobsList = mobsList;

				for(L2NpcInstance mob : mobsList)
				{
					if(mob == null)
						continue;

					mob.getSpawn().stopRespawn();
					if(mob.isMonster && ((L2MonsterInstance) mob).hasMinions())
						((L2MonsterInstance) mob).removeMinions();
					spawnDat.decreaseCount(mob);
					mob.deleteMe();

					tmpMobsList.remove(mob);
				}
				tmpSpawnsList.put(spawnDat, tmpMobsList == null || tmpMobsList.isEmpty() ? _emptyMobs : tmpMobsList);
			}
			newMobsSpawnsList.put(key, tmpSpawnsList);
		}
		return newMobsSpawnsList;
	}

	public HashMap<Integer, FastMap<L2Spawn, FastList<L2NpcInstance>>> spawnMobs(HashMap<Integer, FastMap<L2Spawn, FastList<L2NpcInstance>>> mobsSpawnsList)
	{
		if(mobsSpawnsList.size() == 0)
			return mobsSpawnsList;

		HashMap<Integer, FastMap<L2Spawn, FastList<L2NpcInstance>>> newMobsSpawnsList = new HashMap<Integer, FastMap<L2Spawn, FastList<L2NpcInstance>>>();
		L2NpcInstance tMob = null;

		for(int key : mobsSpawnsList.keySet())
		{
			FastMap<L2Spawn, FastList<L2NpcInstance>> tmpSpawnsList = new FastMap<L2Spawn, FastList<L2NpcInstance>>();
			for(L2Spawn spawnDat : mobsSpawnsList.get(key).keySet())
			{
				FastList<L2NpcInstance> tmpMobsList = new FastList<L2NpcInstance>();
				while(spawnDat.getSpawnedCount() + spawnDat.getSheduledCount() < spawnDat.getAmount())
				{
					tMob = spawnDat.doSpawn(false);
					if(tMob != null)
					{
						tMob.getSpawn().startRespawn();
						tmpMobsList.add(tMob);
					}
				}
				tmpSpawnsList.put(spawnDat, tmpMobsList);
			}
			newMobsSpawnsList.put(key, tmpSpawnsList);
		}
		return newMobsSpawnsList;
	}

	public void changeMode(int mode)
	{
		if(_currentState == mode)
			return;

		switch(mode)
		{
			case 1: // day spawns
				_nightMobs = deleteMobs(_nightMobs);
				_dayMobs = spawnMobs(_dayMobs);

				specialNightBoss(mode);
				break;
			case 2: // night spawns
				_dayMobs = deleteMobs(_dayMobs);
				_nightMobs = spawnMobs(_nightMobs);

				specialNightBoss(mode);
				break;
			default:
				_log.warning("DayNightSpawnManager: Wrong mode sent");
				break;
		}
		_currentState = mode == 1 || mode == 2 ? mode : 0;
	}

	public void notifyChangeMode()
	{
		if(GameTimeController.getInstance().isNowNight())
			changeMode(2);
		else
			changeMode(1);
	}

	public void cleanUp()
	{
		_nightMobs = deleteMobs(_nightMobs);
		_dayMobs = deleteMobs(_dayMobs);

		if(_bosses != null && !_bosses.isEmpty())
			for(L2RaidBossInstance mob : _bosses.values())
			{
				if(mob == null)
					continue;

				mob.getSpawn().stopRespawn();
				if(mob.hasMinions())
					mob.removeMinions();
				mob.deleteMe();
			}

		_nightMobs.clear();
		_dayMobs.clear();
		_bosses.clear();
	}

	private void specialNightBoss(int mode)
	{
		try
		{
			FastMap<L2Spawn, L2RaidBossInstance> tmpBosses = new FastMap<L2Spawn, L2RaidBossInstance>();
			for(L2Spawn spawnDat : _bosses.keySet())
			{
				L2RaidBossInstance boss = _bosses.get(spawnDat);

				if(boss == null && mode == 2)
				{
					boss = (L2RaidBossInstance) spawnDat.doSpawn(false);
					RaidBossSpawnManager.getInstance().notifySpawnNightBoss(boss);
				}
				else if(boss != null && boss.getNpcId() == 25328 && boss.getRaidStatus().equals(RaidBossSpawnManager.StatusEnum.ALIVE))
					boss = handleHellmans(boss, mode);

				tmpBosses.put(spawnDat, boss);
			}
			_bosses = tmpBosses;
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	private L2RaidBossInstance handleHellmans(L2RaidBossInstance boss, int mode)
	{
		L2RaidBossInstance tempBoss = null;
		switch(mode)
		{
			case 1:
				boss.getSpawn().stopRespawn();
				boss.deleteMe();
				if(boss.hasMinions())
					boss.removeMinions();
				//_log.info("DayNightSpawnManager: Deleting Hellman raidboss");
				break;
			case 2:
				boss.getSpawn().startRespawn();
				tempBoss = (L2RaidBossInstance) boss.getSpawn().doSpawn(false);
				//_log.info("DayNightSpawnManager: Spawning Hellman raidboss");
				break;
		}
		return tempBoss;
	}

	public L2RaidBossInstance handleBoss(L2Spawn spawnDat)
	{
		if(_bosses.containsKey(spawnDat))
			return _bosses.get(spawnDat);

		if(GameTimeController.getInstance().isNowNight())
		{
			L2RaidBossInstance raidboss = (L2RaidBossInstance) spawnDat.doSpawn(false);
			_bosses.put(spawnDat, raidboss);

			return raidboss;
		}

		_bosses.put(spawnDat, null);
		return null;
	}
}