package L2jFT.game.instancemanager;

import java.util.Collection;
import java.util.logging.Logger;

import javolution.util.FastMap;
import L2jFT.Config;
import L2jFT.game.model.quest.Quest;
import L2jFT.game.model.quest.QuestJython;

public class QuestManager
{
	protected static final Logger _log = Logger.getLogger(QuestManager.class.getName());

	private static QuestManager _instance;

	public static final QuestManager getInstance()
	{
		if(_instance == null)
		{
			_log.info("Initializing QuestManager");
			_instance = new QuestManager();
			if(!Config.DONTLOADQUEST)
				_instance.load();
		}
		return _instance;
	}

	private FastMap<String, Quest> _questsByName = new FastMap<String, Quest>();
	private FastMap<Integer, Quest> _questsById = new FastMap<Integer, Quest>();

	public QuestManager()
	{}

	public final boolean reload(String questFolder)
	{
		return QuestJython.reloadQuest(questFolder);
	}

	/**
	 * Reloads a the quest given by questId.<BR>
	 * <B>NOTICE: Will only work if the quest name is equal the quest folder name</B>
	 * @param questId The id of the quest to be reloaded
	 * @return true if reload was succesful, false otherwise
	 */
	public final boolean reload(int questId)
	{
		Quest q = getQuest(questId);
		if(q == null)
			return false;
		return QuestJython.reloadQuest(q.getName());
	}

	private final void load()
	{
		QuestJython.init();
		System.out.println("Loaded: " + getQuests().size() + " quests");
	}

	public final Quest getQuest(String name)
	{
		return _questsByName.get(name);
	}

	public final Quest getQuest(int questId)
	{
		return _questsById.get(questId);
	}

	public final void addQuest(Quest newQuest)
	{
		_questsByName.put(newQuest.getName(), newQuest);
		_questsById.put(newQuest.getQuestIntId(), newQuest);
	}

	public final Collection<Quest> getQuests()
	{
		return _questsByName.values();
	}
}