package L2jFT.game.instancemanager;

import javolution.util.FastList;
import javolution.util.FastMap;
import L2jFT.Config;
import L2jFT.database.DatabaseUtils;
import L2jFT.database.FiltredPreparedStatement;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.database.ThreadConnection;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.entity.Castle;
import L2jFT.game.model.entity.siege.Siege;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.SkillTable;
import L2jFT.util.Location;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.sql.ResultSet;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.logging.Logger;

public class SiegeManager
{
	protected static Logger _log = Logger.getLogger(SiegeManager.class.getName());

	private static SiegeManager _Instance;
	private int _Attacker_Max_Clans = 500; // Max number of clans
	private int _Attacker_RespawnDelay = 20000; // Time in ms. Changeable in siege.config
	private int _Defender_Max_Clans = 500; // Max number of clans
	private int _Defender_RespawnDelay = 10000; // Time in ms. Changeable in siege.config

	private FastMap<Integer, FastList<SiegeSpawn>> _artefactSpawnList;
	private FastMap<Integer, FastList<SiegeSpawn>> _controlTowerSpawnList;

	private int _ControlTowerLosePenalty = 20000; // Time in ms. Changeable in siege.config
	private int _Flag_BuyCost = 50000; // Changeable in siege.config
	private int _Flag_BuyItemId = 57; // Changeable in siege.config
	private int _Flag_MaxCount = 1; // Changeable in siege.config
	private int _Siege_Clan_MinLevel = 4; // Changeable in siege.config
	private int _Siege_Length = 120; // Time in minute. Changeable in siege.config

	public SiegeManager()
	{
		_Instance = this;
		load();

		for(Castle castle : CastleManager.getInstance().getCastles().values())
			if(castle != null)
				castle.getSiege().startAutoTask();
	}

	/**
	 * Return true if character summon<BR><BR>
	 * @param activeChar The L2Character of the character can summon
	 */
	public final boolean checkIfOkToSummon(L2Character activeChar, boolean isCheckOnly)
	{
		if(activeChar == null || !activeChar.isPlayer)
			return false;

		SystemMessage sm;
		L2Player player = (L2Player) activeChar;
		Siege siege = getSiege(player);

		if(siege == null)
			sm = new SystemMessage(SystemMessage.YOU_MAY_NOT_SUMMON_FROM_YOUR_CURRENT_LOCATION);
		else if(!siege.isInProgress())
			sm = new SystemMessage(SystemMessage.YOU_ARE_NOT_IN_SIEGE);
		else if(player.getClanId() != 0 && siege.getAttackerClan(player.getClanId()) == null)
			sm = new SystemMessage(SystemMessage.OBSERVATION_IS_ONLY_POSSIBLE_DURING_A_SIEGE);
		else
			return true;

		if(!isCheckOnly)
			player.sendPacket(sm);

		return false;
	}

	/**
	 * Return true if the clan is registered or owner of a castle<BR><BR>
	 * @param clan The L2Clan of the player
	 */
	public final boolean checkIsRegistered(L2Clan clan, int castleid)
	{
		if(clan == null)
			return false;

		if(castleid > 0 && clan.getHasCastle() == castleid)
			return true;

		return checkIsRegisteredForSiege(clan, castleid);
	}

	public final boolean checkIsRegisteredForSiege(L2Clan clan, int castleid)
	{
		if(clan == null)
			return false;

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		boolean register = false;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT clan_id FROM siege_clans where clan_id=?" + (castleid == 0 ? "" : " and castle_id=?"));
			statement.setInt(1, clan.getClanId());
			if(castleid != 0)
				statement.setInt(2, castleid);
			rset = statement.executeQuery();
			if(rset.next())
				register = true;
		}
		catch(Exception e)
		{
			System.out.println("Exception: checkIsRegistered(): " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
		return register;
	}

	public final void addSiegeSkills(L2Player character)
	{
		character.addSkill(SkillTable.getInstance().getInfo(246, 1), false);
		character.addSkill(SkillTable.getInstance().getInfo(247, 1), false);
	}

	public final void removeSiegeSkills(L2Player character)
	{
		character.removeSkill(SkillTable.getInstance().getInfo(246, 1), false);
		character.removeSkill(SkillTable.getInstance().getInfo(247, 1), false);
	}

	private void load()
	{
		try
		{
			InputStream is = new FileInputStream(new File(Config.SIEGE_CONFIGURATION_FILE));
			Properties siegeSettings = new Properties();
			siegeSettings.load(is);
			is.close();

			// Siege setting
			_Attacker_Max_Clans = Integer.decode(siegeSettings.getProperty("AttackerMaxClans", "500"));
			_Attacker_RespawnDelay = Integer.decode(siegeSettings.getProperty("AttackerRespawn", "30000"));
			_ControlTowerLosePenalty = Integer.decode(siegeSettings.getProperty("CTLossPenalty", "20000"));
			_Defender_Max_Clans = Integer.decode(siegeSettings.getProperty("DefenderMaxClans", "500"));
			_Defender_RespawnDelay = Integer.decode(siegeSettings.getProperty("DefenderRespawn", "20000"));
			_Flag_BuyItemId = Integer.decode(siegeSettings.getProperty("FlagItem", "57"));
			_Flag_BuyCost = Integer.decode(siegeSettings.getProperty("FlagCost", "200000"));
			_Flag_MaxCount = Integer.decode(siegeSettings.getProperty("MaxFlags", "1"));
			_Siege_Clan_MinLevel = Integer.decode(siegeSettings.getProperty("SiegeClanMinLevel", "4"));
			_Siege_Length = Integer.decode(siegeSettings.getProperty("SiegeLength", "120"));

			// Siege spawns settings 
			_controlTowerSpawnList = new FastMap<Integer, FastList<SiegeSpawn>>();
			_artefactSpawnList = new FastMap<Integer, FastList<SiegeSpawn>>();

			for(Castle castle : CastleManager.getInstance().getCastles().values())
			{
				FastList<SiegeSpawn> _controlTowersSpawns = new FastList<SiegeSpawn>();

				for(int i = 1; i < 0xFF; i++)
				{
					String _spawnParams = siegeSettings.getProperty(castle.getName() + "ControlTower" + Integer.toString(i), "");

					if(_spawnParams.length() == 0)
						break;

					StringTokenizer st = new StringTokenizer(_spawnParams.trim(), ",");

					try
					{
						int x = Integer.parseInt(st.nextToken());
						int y = Integer.parseInt(st.nextToken());
						int z = Integer.parseInt(st.nextToken());
						int npc_id = Integer.parseInt(st.nextToken());
						int hp = Integer.parseInt(st.nextToken());

						_controlTowersSpawns.add(new SiegeSpawn(castle.getCastleId(), x, y, z, 0, npc_id, hp));
					}
					catch(Exception e)
					{
						_log.warning("Error while loading control tower(s) for " + castle.getName() + " castle.");
					}
				}

				FastList<SiegeSpawn> _artefactSpawns = new FastList<SiegeSpawn>();

				for(int i = 1; i < 0xFF; i++)
				{
					String _spawnParams = siegeSettings.getProperty(castle.getName() + "Artefact" + Integer.toString(i), "");

					if(_spawnParams.length() == 0)
						break;

					StringTokenizer st = new StringTokenizer(_spawnParams.trim(), ",");

					try
					{
						int x = Integer.parseInt(st.nextToken());
						int y = Integer.parseInt(st.nextToken());
						int z = Integer.parseInt(st.nextToken());
						int heading = Integer.parseInt(st.nextToken());
						int npc_id = Integer.parseInt(st.nextToken());

						_artefactSpawns.add(new SiegeSpawn(castle.getCastleId(), x, y, z, heading, npc_id));
					}
					catch(Exception e)
					{
						_log.warning("Error while loading artefact(s) for " + castle.getName() + " castle.");
					}
				}

				_controlTowerSpawnList.put(castle.getCastleId(), _controlTowersSpawns);
				_artefactSpawnList.put(castle.getCastleId(), _artefactSpawns);

				//castle.getSiege().spawnArtifacts(castle.getCastleId()); // Spawn artifacts
				//castle.getSiege().spawnControlTowers(castle.getCastleId()); // Spawn control towers
				castle.getSiege().getZone().setActive(false);
			}
		}
		catch(Exception e)
		{
			//_initialized = false;
			System.err.println("Error while loading siege data.");
			e.printStackTrace();
		}
	}

	public static SiegeManager getInstance()
	{
		if(_Instance == null)
			new SiegeManager();
		return _Instance;
	}

	public final FastList<SiegeSpawn> getArtefactSpawnList(int _castleId)
	{
		if(_artefactSpawnList.containsKey(_castleId))
			return _artefactSpawnList.get(_castleId);
		return null;
	}

	public final FastList<SiegeSpawn> getControlTowerSpawnList(int _castleId)
	{
		if(_controlTowerSpawnList.containsKey(_castleId))
			return _controlTowerSpawnList.get(_castleId);
		return null;
	}

	public final int getAttackerMaxClans()
	{
		return _Attacker_Max_Clans;
	}

	public final int getAttackerRespawnDelay()
	{
		return _Attacker_RespawnDelay;
	}

	public final int getControlTowerLosePenalty()
	{
		return _ControlTowerLosePenalty;
	}

	public final int getDefenderMaxClans()
	{
		return _Defender_Max_Clans;
	}

	public final int getDefenderRespawnDelay()
	{
		return _Defender_RespawnDelay;
	}

	public final int getFlagBuyCost()
	{
		return _Flag_BuyCost;
	}

	public final int getFlagBuyItemId()
	{
		return _Flag_BuyItemId;
	}

	public final int getFlagMaxCount()
	{
		return _Flag_MaxCount;
	}

	public final Siege getSiege(L2Object activeObject)
	{
		return getSiege(activeObject.getX(), activeObject.getY());
	}

	public final Siege getSiege(int x, int y)
	{
		for(Castle castle : CastleManager.getInstance().getCastles().values())
			if(castle.getSiege().checkIfInZone(x, y))
				return castle.getSiege();
		return null;
	}

	public final int getSiegeClanMinLevel()
	{
		return _Siege_Clan_MinLevel;
	}

	public final int getSiegeLength()
	{
		return _Siege_Length;
	}

	public class SiegeSpawn
	{
		Location _location;
		private int _npc_id;
		private int _heading;
		private int _castle_id;
		private int _hp;

		public SiegeSpawn(int castle_id, int x, int y, int z, int heading, int npc_id)
		{
			_castle_id = castle_id;
			_location = new Location(x, y, z, heading);
			_heading = heading;
			_npc_id = npc_id;
		}

		public SiegeSpawn(int castle_id, int x, int y, int z, int heading, int npc_id, int hp)
		{
			_castle_id = castle_id;
			_location = new Location(x, y, z, heading);
			_heading = heading;
			_npc_id = npc_id;
			_hp = hp;
		}

		public int getCastleId()
		{
			return _castle_id;
		}

		public int getNpcId()
		{
			return _npc_id;
		}

		public int getHeading()
		{
			return _heading;
		}

		public int getHp()
		{
			return _hp;
		}

		public Location getLocation()
		{
			return _location;
		}
	}

	public boolean getCanRide()
	{
		for(Castle castle : CastleManager.getInstance().getCastles().values())
			if(castle != null && (castle.getSiege().isInProgress() || castle.getSiege().getTimeRemaining() <= 13600000))
				return false;
		return true;
	}
}
