package L2jFT.game.instancemanager;

import javolution.util.FastList;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Zone;
import L2jFT.game.model.L2Zone.ZoneType;
import L2jFT.game.model.entity.Town;
import L2jFT.game.tables.MapRegionTable;

import java.util.List;

public final class TownManager
{
	private static TownManager _instance;
	private List<Town> _towns = new FastList<Town>();

	private TownManager()
	{
		FastList<L2Zone> zones = ZoneManager.getInstance().getZoneByType(ZoneType.Town);
		if(zones.size() == 0)
			System.out.println("Not found zones for Towns!!!");
		else
			for(L2Zone zone : zones)
				_towns.add(new Town(zone.getIndex()));
	}

	public static TownManager getInstance()
	{
		return _instance;
	}

	public static void init()
	{
		_instance = new TownManager();
	}

	public Town getClosestTown(int x, int y)
	{
		switch(MapRegionTable.getInstance().getMapRegion(x, y))
		{
			case 0:
				return getTown(2); // TI
			case 1:
				return getTown(3); // Elven
			case 2:
				return getTown(1); // DE
			case 3:
				return getTown(4); // Orc
			case 4:
				return getTown(6); // Dwarven
			case 5:
				return getTown(7); // Gludio
			case 6:
				return getTown(5); // Gludin
			case 7:
				return getTown(8); // Dion
			case 8:
				return getTown(9); // Giran
			case 9:
				return getTown(10); // Oren
			case 10:
				return getTown(12); // Aden
			case 11:
				return getTown(11); // HV
			case 12:
				return getTown(16); // Floran
			case 13:
				return getTown(15); // Heine
			case 14:
				return getTown(14); // Rune Castle Town
			case 15:
				return getTown(13); // Goddard
			case 16:
				return getTown(18); // Schuttgart
			case 17:
				return getTown(19); // Primeval Isl
			default:
				return getTown(16); // Floran 
		}
	}

	public Town getClosestTown(L2Object activeObject)
	{
		return getClosestTown(activeObject.getX(), activeObject.getY());
	}

	public Town getTown(int townId)
	{
		for(Town town : _towns)
			if(town.getTownId() == townId)
				return town;

		return null;
	}
}