package L2jFT.game.instancemanager;

import java.sql.ResultSet;
import java.util.logging.Logger;

import javolution.util.FastList;
import javolution.util.FastList.Node;
import javolution.util.FastMap;
import L2jFT.Config;
import L2jFT.database.DatabaseUtils;
import L2jFT.database.FiltredPreparedStatement;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.database.ThreadConnection;
import L2jFT.extensions.listeners.L2ZoneEnterLeaveListener;
import L2jFT.extensions.listeners.PropertyCollection;
import L2jFT.game.ThreadPoolManager;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Territory;
import L2jFT.game.model.L2Zone;
import L2jFT.game.model.L2Zone.ZoneType;
import L2jFT.game.network.serverpackets.Ride;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.TerritoryTable;

public class ZoneManager
{
	protected static Logger _log = Logger.getLogger(ZoneManager.class.getName());

	private static ZoneManager _instance;
	private static FastMap<ZoneType, FastList<L2Zone>> _zonesByType;

	private final ZoneListener _zoneListener = new ZoneListener();

	private ZoneManager()
	{
		loadZone();
	}

	public static ZoneManager getInstance()
	{
		if(_instance == null)
			_instance = new ZoneManager();
		return _instance;
	}

	public boolean checkIfInZone(ZoneType zoneType, int x, int y)
	{
		FastList<L2Zone> list = _zonesByType.get(zoneType);
		if(list == null)
			return false;
		for(Node<L2Zone> n = list.head(), end = list.tail(); (n = n.getNext()) != end;)
		{
			if(n == null)
				break;
			L2Zone zone = n.getValue();
			if(zone.isActive() && zone.getLoc1() != null && zone.getLoc1().isInside(x, y))
				return true;
		}
		return false;
	}

	public boolean checkIfInZone(ZoneType zoneType, int x, int y, int z)
	{
		FastList<L2Zone> list = _zonesByType.get(zoneType);
		if(list == null)
			return false;
		for(Node<L2Zone> n = list.head(), end = list.tail(); (n = n.getNext()) != end;)
		{
			if(n == null)
				break;
			L2Zone zone = n.getValue();
			if(zone.isActive() && zone.getLoc1() != null && zone.getLoc1().isInside(x, y) && z >= zone.getLoc1().getZmin() && z <= zone.getLoc1().getZmax())
				return true;
		}
		return false;
	}

	public FastList<L2Zone> getZoneByType(ZoneType zoneType)
	{
		FastList<L2Zone> list = _zonesByType.get(zoneType);
		FastList<L2Zone> result = new FastList<L2Zone>();
		if(list == null)
			return result;
		for(Node<L2Zone> n = list.head(), end = list.tail(); (n = n.getNext()) != end;)
		{
			if(n == null)
				break;
			L2Zone zone = n.getValue();
			if(zone.isActive())
				result.add(zone);
		}
		return result;
	}

	public L2Zone getZoneByIndex(ZoneType zoneType, Integer index, Boolean onlyActive)
	{
		FastList<L2Zone> list = _zonesByType.get(zoneType);
		if(list == null)
			return null;
		for(Node<L2Zone> n = list.head(), end = list.tail(); (n = n.getNext()) != end;)
		{
			if(n == null)
				break;
			L2Zone zone = n.getValue();
			if((!onlyActive || zone.isActive()) && zone.getIndex() == index)
				return zone;
		}
		return null;
	}

	private void loadZone()
	{
		_zonesByType = new FastMap<ZoneType, FastList<L2Zone>>();

		int count = 0;

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT `id`, `index`, `taxById`, `name`, `type`, `loc1_id`, `loc2_id`, `loc3_id`, `restart_point`, `PKrestart_point`, `target`, `skill_name`, `skill_prob`, `unit_tick`, `default_status`, `initial_delay`, `on_time`, `off_time`, `random_time`, `skill_action_type`, `restart_time`, `restart_allowed_time`, `affect_race`, `move_bonus`, `hp_regen_bonus`, `mp_regen_bonus`, `damage_on_hp`, `damage_on_mp`, `message_no`, `entering_message_no`, `leaving_message_no`, `exp_penalty_per`, `item_drop`, `blocked_actions`, `event_id` FROM `zone`");
			rset = statement.executeQuery();
			while(rset.next())
			{
				Integer zone_id = rset.getInt("id");
				if(zone_id == 0)
				{
					_log.warning("Null zone!");
					continue;
				}

				L2Zone z = new L2Zone(zone_id);

				z.setIndex(rset.getInt("index"));
				z.setTaxById(rset.getInt("taxById"));
				z.setName(rset.getString("name"));
				z.setType(ZoneType.valueOf(rset.getString("type")));
				z.setEnteringMessageId(rset.getInt("entering_message_no"));
				z.setLeavingMessageId(rset.getInt("leaving_message_no"));
				z.setTarget(rset.getString("target"));
				z.setSkill(rset.getString("skill_name"));
				z.setSkillProb(rset.getString("skill_prob"));
				z.setUnitTick(rset.getString("unit_tick"));
				z.setInitialDelay(rset.getString("initial_delay"));
				z.setRestartTime(rset.getLong("restart_time"));
				z.setBlockedActions(rset.getString("blocked_actions"));
				z.setDamageOnHP(rset.getString("damage_on_hp"));
				z.setDamageOnМP(rset.getString("damage_on_mp"));
				z.setMessageNumber(rset.getString("message_no"));
				z.setMoveBonus(rset.getString("move_bonus"));
				z.setRegenBonusHP(rset.getString("hp_regen_bonus"));
				z.setRegenBonusMP(rset.getString("mp_regen_bonus"));
				z.setAffectRace(rset.getString("affect_race"));

				/**
				 StatsSet zoneDat = new StatsSet();
				 zoneDat.set("on_time", rset.getString("on_time"));
				 zoneDat.set("off_time", rset.getString("off_time"));
				 zoneDat.set("random_time", rset.getString("random_time"));
				 zoneDat.set("skill_action_type", rset.getString("skill_action_type"));
				 zoneDat.set("restart_allowed_time", rset.getString("restart_allowed_time"));
				 zoneDat.set("exp_penalty_per", rset.getString("exp_penalty_per"));
				 zoneDat.set("item_drop", rset.getString("item_drop"));
				 zoneDat.set("event_id", rset.getString("event_id"));
				 */

				if(z.getType() == ZoneType.no_landing)
					z.getListenerEngine().addMethodInvokedListener(_zoneListener);

				if(_zonesByType.get(z.getType()) == null)
					_zonesByType.put(z.getType(), new FastList<L2Zone>());

				_zonesByType.get(z.getType()).add(z);

				L2Territory terr;

				terr = TerritoryTable.getInstance().getLocations().get("sql_terr_" + rset.getString("loc1_id"));
				if(terr != null)
				{
					z.setLoc1(terr);
					terr.setZone(z);
				}
				else
					_log.warning("Not defined territory " + rset.getString("loc1_id") + " of " + " " + rset.getString("type") + " " + rset.getString("name"));

				terr = TerritoryTable.getInstance().getLocations().get("sql_terr_" + rset.getString("loc2_id"));
				if(terr != null)
				{
					z.setLoc2(terr);
					terr.setZone(z);
				}

				terr = TerritoryTable.getInstance().getLocations().get("sql_terr_" + rset.getString("loc3_id"));
				if(terr != null)
				{
					z.setLoc3(terr);
					terr.setZone(z);
				}

				String name;
				name = rset.getString("restart_point");
				if(name != null)
				{
					terr = TerritoryTable.getInstance().getLocations().get("sql_terr_" + name);
					if(terr != null)
					{
						z.setRestartPoints(terr);
						terr.setZone(z);
					}
					else
						_log.warning("Unknown restart point " + name);
				}

				terr = TerritoryTable.getInstance().getLocations().get("sql_terr_" + rset.getString("PKrestart_point"));
				if(terr != null)
				{
					z.setPKRestartPoints(terr);
					terr.setZone(z);
				}

				String s = rset.getString("default_status");
				if(s == null)
					//_log.warning("Zone with id " + zone_id + " has null default_status. Setting as active.");
					z.setActive(true);
				else if(s.equals("on") || z.getType() == ZoneType.water)
					z.setActive(true);

				count++;
			}
		}
		catch(Exception e1)
		{
			_log.warning("zones couldnt be initialized: " + e1);
			e1.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}

		_log.config("ZoneManager: Loaded " + count + " zones");

		TerritoryTable.getInstance().registerZones();
	}

	public void reload()
	{
		loadZone();
	}

	public boolean checkIfInZoneFishing(int x, int y, int z)
	{
		return !Config.ENABLE_FISHING_CHECK_WATER || checkIfInZone(ZoneType.water, x, y, z);
	}

	private class ZoneListener extends L2ZoneEnterLeaveListener
	{
		@Override
		public void objectEntered(L2Zone zone, L2Object object)
		{
			L2Player player = object.getPlayer();
			if(player != null && player.getMountType() == 2)
			{
				player.addProperty(PropertyCollection.ZoneEnteredNoLandingFlying, System.currentTimeMillis());
				player.stopMove(null);
				player.sendPacket(new SystemMessage(SystemMessage.THIS_AREA_CANNOT_BE_ENTERED_WHILE_MOUNTED_ATOP_OF_A_WYVERN_YOU_WILL_BE_DISMOUNTED_FROM_YOUR_WYVERN_IF_YOU_DO_NOT_LEAVE));
				// Даём 5 секунд, что бы покинуть зону, запрещенную для полетов
				ThreadPoolManager.getInstance().scheduleGeneral(new Dismount(player), 5000);
			}
		}

		@Override
		public void objectLeaved(L2Zone zone, L2Object object)
		{
			L2Player player = object.getPlayer();
			if(player != null)
				player.addProperty(PropertyCollection.ZoneEnteredNoLandingFlying, 0);
		}
	}

	private class Dismount implements Runnable
	{
		private L2Player _player;

		public Dismount(L2Player player)
		{
			_player = player;
		}

		public void run()
		{
			Long enterTime = (Long) _player.getProperty(PropertyCollection.ZoneEnteredNoLandingFlying);

			if(enterTime == null || enterTime == 0)
				return;

			// Если он "влетел-вылетел-влетел"
			if(enterTime + 5000L > System.currentTimeMillis())
				return;

			if(_player.getMountType() == 2 && !_player.checkLandingState())
			{
				_player.broadcastPacket(new Ride(_player, Ride.ACTION_DISMOUNT, 0));
				_player.setMountType(0);
				_player.addProperty(PropertyCollection.ZoneEnteredNoLandingFlying, 0);
			}
		}
	}

}