package L2jFT.game.model;

import java.util.logging.Logger;

import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.instances.L2ItemInstance.ItemLocation;
import L2jFT.util.Log;

public final class ClanWarehouse extends Warehouse
{
	private static final Logger _log = Logger.getLogger(ClanWarehouse.class.getName());

	private L2Clan _clan;

	public ClanWarehouse(L2Clan clan)
	{
		_clan = clan;
	}

	@Override
	public int getOwnerId()
	{
		return _clan.getClanId();
	}

	@Override
	public ItemLocation getLocationType()
	{
		return ItemLocation.CLANWH;
	}

	@Override
	public String getLocationId()
	{
		return "0";
	}

	@Override
	public byte getLocationId(@SuppressWarnings("unused")
	boolean dummy)
	{
		return 0;
	}

	@Override
	public void setLocationId(@SuppressWarnings("unused")
	L2Player dummy)
	{}

	@Override
	public synchronized L2ItemInstance addItem(L2ItemInstance newItem)
	{
		if(newItem.getCount() < 0)
		{
			_log.warning("addItem to ClanWarehouse: count < 0 owner:" + _clan.getName());
			return null;
		}
		Log.add("ClanWarehouse|" + _clan.getName() + "|add item|" + newItem.getItemId() + "|" + newItem.getCount() + "|" + newItem.getObjectId(), "items");
		return super.addItem(newItem);
	}

	@Override
	public synchronized L2ItemInstance getItemByObj(int objectId, int count)
	{
		if(count < 0)
		{
			_log.warning("getItem ClanWarehouse: count < 0 owner:" + _clan.getName());
			return null;
		}
		L2ItemInstance newItem = super.getItemByObj(objectId, count);
		if(newItem != null)
			Log.add("ClanWarehouse|" + _clan.getName() + "|get item|" + newItem.getItemId() + "|" + newItem.getCount() + "|" + newItem.getObjectId(), "items");
		return newItem;
	}

	@Override
	public synchronized L2ItemInstance destroyItemByObj(int objectId, int count)
	{
		if(count < 0)
		{
			_log.warning("destroyItem ClanWarehouse: count < 0 owner:" + _clan.getName());
			return null;
		}
		L2ItemInstance destroyItem = super.destroyItem(objectId, count);
		if(destroyItem != null)
			Log.add("ClanWarehouse|" + _clan.getName() + "|destroy item|" + destroyItem.getItemId() + "|" + destroyItem.getCount() + "|" + destroyItem.getObjectId(), "items");
		return destroyItem;
	}

	@Override
	public synchronized L2ItemInstance destroyItem(int itemId, int count)
	{
		if(count < 0)
		{
			_log.warning("destroyItem ClanWarehouse: count < 0 owner:" + _clan.getName());
			return null;
		}
		L2ItemInstance destroyItem = super.destroyItem(itemId, count);
		if(destroyItem != null)
			Log.add("ClanWarehouse|" + _clan.getName() + "|destroy item|" + destroyItem.getItemId() + "|" + destroyItem.getCount() + "|" + destroyItem.getObjectId(), "items");
		return destroyItem;

	}

	@Override
	public final WarehouseType getWarehouseType()
	{
		return WarehouseType.CLAN;
	}
}