package L2jFT.game.model;

import java.util.concurrent.Future;

import jsr166x.ConcurrentSkipListSet;

import L2jFT.game.ThreadPoolManager;
import L2jFT.game.skills.effects.EffectForce;
import L2jFT.game.tables.SkillTable;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class ForceBuff
{
	static final Log _log = LogFactory.getLog(ForceBuff.class.getName());

	private L2Character _caster;
	private L2Character _target;
	private L2Skill _skill;
	private L2Skill _force;
	private Future _task;

	public L2Character getCaster()
	{
		return _caster;
	}

	public L2Character getTarget()
	{
		return _target;
	}

	public L2Skill getSkill()
	{
		return _skill;
	}

	public L2Skill getForce()
	{
		return _force;
	}

	private void setTask(Future task)
	{
		_task = task;
	}

	public ForceBuff(L2Character caster, L2Character target, L2Skill skill)
	{
		_caster = caster;
		_target = target;
		_skill = skill;
		_force = SkillTable.getInstance().getInfo(skill.getForceId(), 1);

		Runnable r = new Runnable(){
			public void run()
			{
				setTask(null);

				int forceId = getForce().getId();
				boolean create = true;
				ConcurrentSkipListSet<L2Effect> effects = getTarget().getAllEffects();
				if(effects != null)
					for(L2Effect e : effects)
						if(e.getSkill().getId() == forceId)
						{
							EffectForce ef = (EffectForce) e;
							if(ef.forces < 3)
								ef.increaseForce();
							create = false;
							break;
						}

				if(create)
					getForce().getEffects(_caster, getTarget(), false);
			}
		};

		setTask(ThreadPoolManager.getInstance().scheduleGeneral(r, 3300));
	}

	public void delete()
	{
		if(_task != null)
		{
			_task.cancel(false);
			_task = null;
		}

		int toDeleteId = getForce().getId();

		ConcurrentSkipListSet<L2Effect> effects = _target.getAllEffects();
		if(effects != null)
			for(L2Effect e : effects)
				if(e.getSkill().getId() == toDeleteId)
				{
					((EffectForce) e).decreaseForce();
					break;
				}

		_caster.setForceBuff(null);
	}
}