package L2jFT.game.model;

import javolution.util.FastList;
import javolution.util.FastTable;
import jsr166x.ConcurrentSkipListSet;
import L2jFT.Config;
import L2jFT.extensions.listeners.MethodCollection;
import L2jFT.extensions.listeners.PropertyCollection;
import L2jFT.extensions.multilang.CustomMessage;
import L2jFT.game.GameTimeController;
import L2jFT.game.ThreadPoolManager;
import L2jFT.game.ai.CtrlEvent;
import L2jFT.game.ai.CtrlIntention;
import static L2jFT.game.ai.CtrlIntention.AI_INTENTION_ACTIVE;
import L2jFT.game.ai.L2CharacterAI;
import L2jFT.game.ai.L2PlayableAI.nextAction;
import L2jFT.game.cache.Msg;
import L2jFT.game.geodata.GeoData;
import L2jFT.game.geodata.GeoEngine;
import L2jFT.game.geodata.GeoMove;
import L2jFT.game.handler.IScriptHandler;
import L2jFT.game.handler.ScriptHandler;
import L2jFT.game.instancemanager.DimensionalRiftManager;
import L2jFT.game.model.L2Effect.EffectType;
import L2jFT.game.model.L2Skill.SkillTargetType;
import L2jFT.game.model.L2Skill.SkillType;
import L2jFT.game.model.L2Zone.ZoneType;
import L2jFT.game.model.entity.Duel;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.instances.L2MinionInstance;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.model.instances.L2PetInstance;
import L2jFT.game.model.instances.L2RaidBossInstance;
import L2jFT.game.model.instances.L2SummonInstance;
import L2jFT.game.model.quest.QuestState;
import L2jFT.game.network.serverpackets.ActionFailed;
import L2jFT.game.network.serverpackets.Attack;
import L2jFT.game.network.serverpackets.AutoAttackStart;
import L2jFT.game.network.serverpackets.AutoAttackStop;
import L2jFT.game.network.serverpackets.ChangeMoveType;
import L2jFT.game.network.serverpackets.ChangeWaitType;
import L2jFT.game.network.serverpackets.CharMoveToLocation;
import L2jFT.game.network.serverpackets.L2GameServerPacket;
import L2jFT.game.network.serverpackets.MagicSkillCanceld;
import L2jFT.game.network.serverpackets.MagicSkillLaunched;
import L2jFT.game.network.serverpackets.MagicSkillUse;
import L2jFT.game.network.serverpackets.MoveToLocationInVehicle;
import L2jFT.game.network.serverpackets.MyTargetSelected;
import L2jFT.game.network.serverpackets.PartyMemberPosition;
import L2jFT.game.network.serverpackets.Revive;
import L2jFT.game.network.serverpackets.SetupGauge;
import L2jFT.game.network.serverpackets.ShortBuffStatusUpdate;
import L2jFT.game.network.serverpackets.StatusUpdate;
import L2jFT.game.network.serverpackets.StopMove;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.network.serverpackets.TargetUnselected;
import L2jFT.game.network.serverpackets.TeleportToLocation;
import L2jFT.game.network.serverpackets.UserInfo;
import L2jFT.game.network.serverpackets.ValidateLocation;
import L2jFT.game.skills.Calculator;
import L2jFT.game.skills.Env;
import L2jFT.game.skills.Formulas;
import L2jFT.game.skills.Stats;
import L2jFT.game.skills.effects.ResistType;
import L2jFT.game.skills.funcs.Func;
import L2jFT.game.templates.L2CharTemplate;
import L2jFT.game.templates.L2NpcTemplate;
import L2jFT.game.templates.L2Weapon;
import L2jFT.game.templates.L2Weapon.WeaponType;
import L2jFT.util.CharPosition;
import L2jFT.util.Location;
import L2jFT.util.Rnd;
import L2jFT.util.Util;

import java.awt.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArraySet;
import java.util.concurrent.Future;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Mother class of all character objects of the world (PC, NPC...)<BR>
 * <BR>
 * L2Character :<BR>
 * <BR>
 * <li>L2CastleGuardInstance</li>
 * <li>L2DoorInstance</li>
 * <li>L2NpcInstance</li>
 * <li>L2PlayableInstance </li>
 * <BR>
 * <BR>
 * 
 * <B><U> Concept of L2CharTemplate</U> :</B><BR>
 * <BR>
 * Each L2Character owns generic and static properties (ex : all Keltir have the
 * same number of HP...). All of those properties are stored in a different
 * template for each type of L2Character. Each template is loaded once in the
 * server cache memory (reduce memory use). When a new instance of L2Character
 * is spawned, server just create a link between the instance and the template.
 * This link is stored in <B>_template</B><BR>
 * <BR>
 * 
 * 
 * @version $Revision: 1.53.2.45.2.34 $ $Date: 2005/04/11 10:06:08 $
 */
public abstract class L2Character extends L2Object
{
	class AltMagicUseTask implements Runnable
	{
		public final L2Skill _skill;
		public L2Character _target;

		public AltMagicUseTask(L2Character target, L2Skill skill)
		{
			_skill = skill;
			_target = target;
		}

		public void run()
		{
			altOnMagicUseTimer(_target, _skill);
		}
	}

	public class CancelAttackStance implements Runnable
	{
		public void run()
		{
			broadcastPacket(new AutoAttackStop(getObjectId()));
			_stanceTask.cancel(false);
			_stanceTask = null;
		}
	}

	/** Task lauching the function enableSkill() */
	class EnableSkill implements Runnable
	{
		short _skillId;

		public EnableSkill(short skillId)
		{
			_skillId = skillId;
		}

		public void run()
		{
			try
			{
				enableSkill(_skillId);
			}
			catch(Throwable e)
			{
				_log.log(Level.SEVERE, "", e);
			}
		}
	}

	/**
	 * Task lauching the function onHitTimer().<BR>
	 * <BR>
	 * 
	 * <B><U> Actions</U> :</B><BR>
	 * <BR>
	 * <li>If the attacker/target is dead or use fake death, notify the AI with
	 * EVT_CANCEL and send a Server->Client packet ActionFailed (if attacker is
	 * a L2Player)</li>
	 * <li>If attack isn't aborted, send a message system (critical hit,
	 * missed...) to attacker/target if they are L2Player </li>
	 * <li>If attack isn't aborted and hit isn't missed, reduce HP of the
	 * target and calculate reflection damage to reduce HP of attacker if
	 * necessary </li>
	 * <li>if attack isn't aborted and hit isn't missed, manage attack or cast
	 * break of the target (calculating rate, sending message...) </li>
	 * <BR>
	 * <BR>
	 * 
	 */
	class HitTask implements Runnable
	{
		boolean _crit;
		int _damage;
		L2Character _hitTarget;
		boolean _miss;
		boolean _shld;
		boolean _soulshot;
		boolean _unchargeSS;

		public HitTask(L2Character target, int damage, boolean crit, boolean miss, boolean soulshot, boolean shld, boolean unchargeSS)
		{
			_hitTarget = target;
			_damage = damage;
			_crit = crit;
			_shld = shld;
			_miss = miss;
			_soulshot = soulshot;
			_unchargeSS = unchargeSS;
		}

		public void run()
		{
			try
			{
				onHitTimer(_hitTarget, _damage, _crit, _miss, _soulshot, _shld, _unchargeSS);
			}
			catch(Throwable e)
			{
				_log.log(Level.SEVERE, "", e);
			}
		}
	}

	/** Task lauching the function onMagicUseTimer() */
	class MagicUseTask implements Runnable
	{
		public final boolean _forceUse;

		public MagicUseTask(L2Character target, L2Skill skill, boolean forceUse)
		{
			_castingSkill = skill;
			_castingTarget = target;
			_forceUse = forceUse;
		}

		public void run()
		{
			onMagicUseTimer(_castingTarget, _castingSkill, _forceUse);
		}
	}

	public class IntentionTask implements Runnable
	{
		private final CtrlIntention _intention;
		private final Object _arg0, _arg1;

		public IntentionTask(CtrlIntention evt, Object arg0, Object arg1)
		{
			_intention = evt;
			_arg0 = arg0;
			_arg1 = arg1;
		}

		public void run()
		{
			try
			{
				getAI().setIntention(_intention, _arg0, _arg1);
			}
			catch(Throwable t)
			{
				_log.warning("Exception " + t);
				t.printStackTrace();
			}
		}
	}

	/** Task of AI notification */
	public class NotifyAITask implements Runnable
	{
		private final CtrlEvent _evt;

		public NotifyAITask(CtrlEvent evt)
		{
			_evt = evt;
		}

		public void run()
		{
			try
			{
				getAI().notifyEvent(_evt, null, null);
			}
			catch(Throwable t)
			{
				_log.warning("Exception " + t);
				t.printStackTrace();
			}
		}
	}

	/** Task of HP/MP/CP regeneration */
	class RegenTask implements Runnable
	{
		public void run()
		{
			try
			{
				if(isDead() || _currentHp >= getMaxHp() && _currentMp >= getMaxMp())
					stopHpMpRegeneration();

				double addHp = 0;
				double addMp = 0;

				// Caculate the HP regen rate for this L2Character
				if(_currentHp < getMaxHp())
					addHp += Formulas.calcHpRegen(L2Character.this);

				// Caculate the HP regen rate for this L2Character
				if(_currentMp < getMaxMp())
					addMp += Formulas.calcMpRegen(L2Character.this);

				// Added regen bonus when character is sitting
				if(isPlayer && Config.REGEN_SIT_WAIT)
				{
					L2Player pl = (L2Player) L2Character.this;
					if(pl.isSitting())
					{
						pl.updateWaitSitTime();
						if(pl.getWaitSitTime() > 5)
						{
							addHp += pl.getWaitSitTime();
							addMp += pl.getWaitSitTime();
						}
					}
				}
				else if(L2Character.this instanceof L2RaidBossInstance)
				{
					addHp *= Config.RATE_RAID_REGEN;
					addMp *= Config.RATE_RAID_REGEN;
				}

				// Modify the current HP and MP of the L2Character and broadcast
				// Server->Client packet StatusUpdate
				setCurrentHpMp(_currentHp + addHp, _currentMp + addMp);

				if(_currentCp < getMaxCp())
					// Caculate the CP regen rate for this L2Character
					// Modify the current CP of the L2Character and broadcast
					// Server->Client packet StatusUpdate
					setCurrentCp(getCurrentCp() + Formulas.calcCpRegen(L2Character.this));
			}
			catch(Throwable e)
			{
				_log.log(Level.SEVERE, "", e);
			}
		}
	}

	protected static final Logger _log = Logger.getLogger(L2Character.class.getName());

	private static final ActionFailed _actionfailed = new ActionFailed();

	public static final int ABNORMAL_EFFECT_AFFRAID = 0x000010;
	public static final int ABNORMAL_EFFECT_BIG_HEAD = 0x002000;
	public static final int ABNORMAL_EFFECT_BLEEDING = 0x000001;
	public static final int ABNORMAL_EFFECT_CONFUSED = 0x000020;
	public static final int ABNORMAL_EFFECT_FLAME = 0x004000;
	public static final int ABNORMAL_EFFECT_HOLD_1 = 0x000400;
	public static final int ABNORMAL_EFFECT_HOLD_2 = 0x000800;
	public static final int ABNORMAL_EFFECT_MUTED = 0x000100;
	public static final int ABNORMAL_EFFECT_POISON = 0x000002;
	public static final int ABNORMAL_EFFECT_ROOT = 0x000200;
	public static final int ABNORMAL_EFFECT_SILENT_MOVE = 0x100000;
	public static final int ABNORMAL_EFFECT_SLEEP = 0x000080;
	public static final int ABNORMAL_EFFECT_STUN = 0x000040;

	public static final int ABNORMAL_EFFECT_FLOATING_ROOT = 0x020000;
	public static final int ABNORMAL_EFFECT_DANCE_STUNNED = 0x040000;
	public static final int ABNORMAL_EFFECT_FIREROOT_STUN = 0x080000;
	public static final int ABNORMAL_EFFECT_IMPRISIONING_1 = 0x200000;
	public static final int ABNORMAL_EFFECT_IMPRISIONING_2 = 0x400000;
	public static final int ABNORMAL_EFFECT_MAGIC_CIRCLE = 0x800000;

	public static final int ABNORMAL_EFFECT_UNKNOWN_13 = 0x001000;
	public static final int ABNORMAL_EFFECT_UNKNOWN_16 = 0x008000;
	public static final int ABNORMAL_EFFECT_UNKNOWN_17 = 0x010000;
	public static final int ABNORMAL_EFFECT_UNKNOWN_3 = 0x000004;
	public static final int ABNORMAL_EFFECT_UNKNOWN_4 = 0x000008;

	private int _customEffect;

	public static final double HEADINGS_IN_PI = 10430.378350470452724949566316381;
	public static final int INTERACTION_DISTANCE = 150;

	/**
	 * List of all QuestState instance that needs to be notified of this
	 * character's death
	 */
	private ArrayList<QuestState> _NotifyQuestOfDeathList = new ArrayList<QuestState>();

	/**
	 * Array containing all clients that need to be notified about hp/mp updates
	 * of the L2Character
	 */
	private CopyOnWriteArraySet<L2Character> _statusListener;

	@SuppressWarnings("unchecked")
	public Future _skillTask;
	@SuppressWarnings("unchecked")
	public Future _altSkillTask;
	@SuppressWarnings("unchecked")
	public Future _regTask;
	@SuppressWarnings("unchecked")
	public Future _stanceTask;

	private long _stanceInited;

	private int _flagsRegenActive;
	private double _lastHpUpdate = -99999999;

	private static final int HP_REGEN_FLAG = 1;
	private static final int MP_REGEN_FLAG = 2;
	private static final int CP_REGEN_FLAG = 4;

	protected double _currentCp = 1;
	protected double _currentHp = 1;
	protected double _currentMp = 1;

	protected boolean _isAttackAborted;
	protected long _attackEndTime;
	protected long _attackReuseEndTime;

	/** HashMap(Integer, L2Skill) containing all skills of the L2Character */
	protected HashMap<Short, L2Skill> _skills = new HashMap<Short, L2Skill>();
	protected HashMap<Short, L2Skill> _skillsOnAttack = new HashMap<Short, L2Skill>();

	public L2Skill _castingSkill;
	public L2Character _castingTarget;

	public int _castEndTime;
	public int _castInterruptTime;
	public int _altCastEndTime;
	public int _altCastInterruptTime;

	protected long _animationEndTime;

	/** Table containing all skillId that are disabled */
	protected List<Short> _disabledSkills;

	protected ForceBuff _forceBuff;

	/**
	 * HashMap containing all active skills effects in progress of a
	 * L2Character. The Short key of this HashMap is the L2Skill Identifier that
	 * has created the effect.
	 */
	private ConcurrentHashMap<Short, ConcurrentSkipListSet<L2Effect>> _effects = null;

	/**
	 * The table containing the List of all stacked effect in progress for each
	 * Stack group Identifier
	 */
	private ConcurrentHashMap<String, Short> _stackedEffects = null;

	private byte _buffs = 0;
	private byte _debuffs = 0;

	private FastList<Stats> _blockedStats;

	/** Map 32 bits (0x00000000) containing all abnormal effect in progress */
	private int _abnormalEffects;

	private boolean _flying;
	private boolean _riding;

	private boolean _fake_death;
	private boolean _fishing;

	protected boolean _isInvul;

	protected boolean _IsPendingRevive;

	protected boolean _IsTeleporting;

	protected boolean _overloaded;

	protected boolean _killedAlready;

	private long _dropDisabled;

	private byte _IsBlessedByNoblesse; // Восстанавливает все бафы после смерти
	private byte _IsSalvation; // Восстанавливает все бафы после смерти и
	// полностью CP, MP, HP
	private byte _buffImmunity; // Иммунитет к бафам/дебафам
	private final HashMap<Short, Byte> _skillMastery = new HashMap<Short, Byte>();

	private byte _afraid;
	private byte _meditated;
	private byte _muted;
	private byte _pmuted;
	private byte _paralyzed;
	private byte _rooted;
	private byte _sleeping;
	private byte _stunned;
	private byte _imobilised;
	private byte _confused;
	private byte _blocked;
	private byte _circle;
	private byte _bigHead;

	public boolean _running;

	public boolean isMoving;
	public boolean isFollow;

	public boolean _pathfinding;
	public boolean _forestalling;

	public L2Character followTarget;

	public Vector<Location> _targetRecorder = new Vector<Location>();

	public int _xDestination;
	public int _yDestination;
	public int _zDestination;

	public int _xFrom;
	public int _yFrom;
	public int _zFrom;

	public int _xTo;
	public int _yTo;
	public int _zTo;

	public int _offset;

	public float _xSpeedTicks;
	public float _ySpeedTicks;

	public int _followTimestamp;
	public int _startMoveTick;
	public double previous_speed = -1;

	/** Orientation of the L2Character */
	private int _heading;

	protected L2Object _target;

	/** Table of Calculators containing all used calculator */
	private Calculator[] _calculators;

	/**
	 * Table of calculators containing all standard NPC calculator (ex :
	 * ACCURACY_COMBAT, EVASION_RATE
	 */
	private static final Calculator[] NPC_STD_CALCULATOR = Formulas.getStdNPCCalculators();

	/**
	 * The link on the L2CharTemplate object containing generic and static
	 * properties of this L2Character type (ex : Max HP, Speed...)
	 */
	protected L2CharTemplate _template;
	protected L2CharTemplate _basetemplate;
	protected L2CharacterAI _ai;

	protected String _name;
	protected String _title;

	/**
	 * Constructor of L2Character.<BR>
	 * <BR>
	 * 
	 * <B><U> Concept</U> :</B><BR>
	 * <BR>
	 * Each L2Character owns generic and static properties (ex : all Keltir have
	 * the same number of HP...). All of those properties are stored in a
	 * different template for each type of L2Character. Each template is loaded
	 * once in the server cache memory (reduce memory use). When a new instance
	 * of L2Character is spawned, server just create a link between the instance
	 * and the template This link is stored in <B>_template</B><BR>
	 * <BR>
	 * 
	 * <B><U> Actions</U> :</B><BR>
	 * <BR>
	 * <li>Set the _template of the L2Character </li>
	 * <li>Set _overloaded to false (the charcater can take more items)</li>
	 * <BR>
	 * <BR>
	 * 
	 * <li>If L2Character is a L2NpcInstance, copy skills from template to
	 * object</li>
	 * <li>If L2Character is a L2NpcInstance, link _calculators to
	 * NPC_STD_CALCULATOR</li>
	 * <BR>
	 * <BR>
	 * 
	 * <li>If L2Character is NOT a L2NpcInstance, create an empty _skills slot</li>
	 * <li>If L2Character is a L2Player or L2Summon, copy basic Calculator set
	 * to object</li>
	 * <BR>
	 * <BR>
	 * 
	 * @param objectId
	 *            Identifier of the object to initialized
	 * @param template
	 *            The L2CharTemplate to apply to the object
	 */
	public L2Character(int objectId, L2CharTemplate template)
	{
		super(objectId);

		isCharacter = true;

		// Set its template to the new L2Character
		_template = template;

		if(template != null && this instanceof L2NpcInstance)
		{
			// Copy the Standard Calcultors of the L2NpcInstance in _calculators
			_calculators = NPC_STD_CALCULATOR;

			// Copy the skills of the L2NpcInstance from its template to the
			// L2Character Instance
			// The skills list can be affected by spell effects so it's
			// necessary to make a copy
			// to avoid that a spell affecting a L2NpcInstance, affects others
			// L2NpcInstance of the same type too.
			_skills = ((L2NpcTemplate) template).getSkills();

			if(_skills != null)
				for(L2Skill skill : _skills.values())
					addStatFuncs(skill.getStatFuncs(this));
			// Formulas.addFuncsToNewCharacter(this);
		}
		else
		{
			// If L2Character is a L2Player or a L2Summon, create the basic
			// calculator set
			_calculators = new Calculator[Stats.NUM_STATS];
			if(this instanceof L2Playable)
				Formulas.addFuncsToNewCharacter(this);
		}
	}

	/**
	 * Abort the attack of the L2Character and send Server->Client ActionFailed
	 * packet.<BR>
	 * <BR>
	 */
	public final void abortAttack()
	{
		if(isAttackingNow())
		{
			_attackEndTime = 0;
			_isAttackAborted = true;
			sendActionFailed();
		}
	}

	/**
	 * Abort the cast of the L2Character and send Server->Client
	 * MagicSkillCanceld/ActionFailed packet.<BR>
	 * <BR>
	 */
	public final void abortCast()
	{
		if(isCastingNow())
		{
			_castEndTime = 0;
			_castInterruptTime = 0;
			if(_castingSkill != null)
				_skillMastery.remove(_castingSkill.getId());
			_castingSkill = null;
			if(_skillTask != null)
				_skillTask.cancel(true); // cancels the skill hit scheduled task

			if(_forceBuff != null)
				_forceBuff.delete();

			broadcastPacket(new MagicSkillCanceld(_objectId)); // broadcast packet to stop animations client-side
			sendActionFailed(); // send an "action failed" packet to the caster
			getAI().setIntention(AI_INTENTION_ACTIVE, null, null);
		}
	}

	public void absorbAndReflect(L2Character target, int damage)
	{
		double reflect = target.calcStat(Stats.REFLECT_DAMAGE_PERCENT, 0, null, null);
		if(reflect > 0)
		{
			double rdmg = damage * reflect / 100.;
			if(this instanceof L2Playable)
				reduceCurrentHp(rdmg, this);
			else
				reduceCurrentHp(rdmg, target);
		}

		damage = (int) (damage - target.getCurrentCp());

		if(damage <= 0)
			return;

		double absorb = calcStat(Stats.ABSORB_DAMAGE_PERCENT, 0, target, null);

		if(absorb > 0)
			setCurrentHp(_currentHp + damage * absorb / 100);

		absorb = calcStat(Stats.ABSORB_DAMAGEMP_PERCENT, 0, target, null);

		if(absorb > 0)
			setCurrentMp(_currentMp + damage * absorb / 100);
	}

	public void addBlockStats(FastList<Stats> stats)
	{
		if(_blockedStats == null)
			_blockedStats = new FastList<Stats>(stats);
		else
			_blockedStats.addAll(stats);
	}

	public synchronized void addEffect(L2Effect newEffect)
	{
		if(newEffect == null)
			return;

		// Проверка на имунность к бафам/дебафам
		if(isBuffImmunity() && newEffect.getEffectType() != L2Effect.EffectType.BUFFIMMUNITY)
		{
			SkillType st = newEffect.getSkill().getSkillType();
			if(st == L2Skill.SkillType.BUFF || st == L2Skill.SkillType.DEBUFF)
				return;
		}

		if(_effects == null)
			_effects = new ConcurrentHashMap<Short, ConcurrentSkipListSet<L2Effect>>();

		// определяем нужно ли освобождать место - если скилл такого типа уже есть то место будет освобождено за счет его удаления
		boolean remove = true;
		if(_effects.containsKey(newEffect.getSkill().getId()))
			remove = false;
		for(ConcurrentSkipListSet<L2Effect> efs : _effects.values())
			for(L2Effect ef : efs)
				if(ef.getEffectType() == newEffect.getEffectType())
				{
					remove = false;
					break;
				}

		// если нужно - очищаем слот
		if(remove && _effects.size() > 0)
		{
			boolean flag = false; // для выхода из внешнего цикла.

			// Ограничение на количество дебафов
			// Если больше максимального - удаляем первый в списке
			if(_debuffs >= 10 && (newEffect.getSkill().isPvpSkill() || newEffect.getSkill().isToggle()))
			{
				for(ConcurrentSkipListSet<L2Effect> efs : _effects.values())
				{
					for(L2Effect ef : efs)
						if(ef.getSkill().isPvpSkill() || ef.getSkill().isToggle())
						{
							ef.exit();
							flag = true;
							break;
						}
					if(flag)
						break;
				}
			}

			flag = false;

			// Ограничение на количество бафов
			// Если больше максимального - удаляем первый в списке
			if(!newEffect.getSkill().isPvpSkill() && !newEffect.getSkill().isToggle() && _buffs >= getBuffLimit())
			{
				if(newEffect.getSkill().getId() >= 2278 && newEffect.getSkill().getId() <= 2285)
					return;

				for(ConcurrentSkipListSet<L2Effect> efs : _effects.values())
				{
					for(L2Effect ef : efs)
						if(!ef.getSkill().isPvpSkill() && !ef.getSkill().isToggle())
						{
							ef.exit();
							flag = true;
							break;
						}
					if(flag)
						break;
				}
			}
		}

		short effectSkillId = newEffect.getSkill().getId();
		String stackType = newEffect.getStackType();

		ConcurrentSkipListSet<L2Effect> oldEffects = null;

		if(stackType.equals("none"))
		{
			if(_effects != null)
				oldEffects = _effects.get(effectSkillId);

			// Удаляем эффекты с таким EffectType
			if(oldEffects != null)
				for(L2Effect ef : oldEffects)
					if(ef != null && ef.getEffectType() == newEffect.getEffectType())
						// Если оставшаяся длительность старого эффекта больше чем длительность нового, то оставляем старый.
						if(newEffect.getPeriod() * newEffect.getCount() > ef.getPeriod() * ef.getCount() - ef.getTime())
							ef.exit();
						else
							return;

			if(_effects == null)
				_effects = new ConcurrentHashMap<Short, ConcurrentSkipListSet<L2Effect>>();

			// Добавляем новый эффект
			oldEffects = _effects.get(effectSkillId);
			if(oldEffects == null)
				oldEffects = new ConcurrentSkipListSet<L2Effect>();
			oldEffects.add(newEffect);
			_effects.put(effectSkillId, oldEffects);

			// Применяем эффект к параметрам персонажа
			addStatFuncs(newEffect.getStatFuncs());

			// Запускаем эффект
			newEffect.setInUse(true);

			// Обновляем иконки
			updateEffectIcons();

			// Увеличиваем счетчик бафов/дебафов
			if(newEffect.getSkill().isPvpSkill() || newEffect.getSkill().isToggle())
				_debuffs++;
			else
				_buffs++;
			return;
		}

		short stackQueue = 0;
		if(_stackedEffects != null && _stackedEffects.get(stackType) != null)
			stackQueue = _stackedEffects.get(stackType);

		if(stackQueue > 0)
		{
			if(_effects != null)
				oldEffects = _effects.get(stackQueue);

			if(oldEffects != null && oldEffects.size() > 0)
			{
				// Проверяем, нужно ли накладывать эффект, при совпадении StackType.
				// Новый эффект накладывается только в том случае, если у него больше StackOrder и больше длительность.
				// Эффекты со StackOrder == -1 заменить нельзя (например, Root).
				// Если условия подходят - удаляем старый, иначе выходим.
				for(L2Effect ef : oldEffects)
					if(ef != null && ef.getStackType().equals(stackType))
						if(newEffect.getStackOrder() >= ef.getStackOrder() && newEffect.getPeriod() * newEffect.getCount() > ef.getPeriod() * ef.getCount() - ef.getTime() && ef.getStackOrder() != -1)
							ef.exit();
						else
							return;
			}
			else
				_stackedEffects.remove(stackType);
		}

		if(_stackedEffects == null)
			_stackedEffects = new ConcurrentHashMap<String, Short>();
		_stackedEffects.put(stackType, effectSkillId);

		if(_effects == null)
			_effects = new ConcurrentHashMap<Short, ConcurrentSkipListSet<L2Effect>>();

		// Добавляем новый эффект
		oldEffects = _effects.get(effectSkillId);
		if(oldEffects == null)
			oldEffects = new ConcurrentSkipListSet<L2Effect>();
		oldEffects.add(newEffect);
		_effects.put(effectSkillId, oldEffects);

		// Применяем эффект к параметрам персонажа
		addStatFuncs(newEffect.getStatFuncs());

		// Запускаем эффект
		newEffect.setInUse(true);

		// Обновляем иконки
		updateEffectIcons();

		// Увеличиваем счетчик бафов/дебафов
		if(newEffect.getSkill().isPvpSkill() || newEffect.getSkill().isToggle())
			_debuffs++;
		else
			_buffs++;
	}

	/**
	 * Add QuestState instance that is to be notified of character's death.<BR>
	 * <BR>
	 * 
	 * @param qs
	 *            The QuestState that subscribe to this event
	 * 
	 */
	public void addNotifyQuestOfDeath(QuestState qs)
	{
		if(qs == null || _NotifyQuestOfDeathList.contains(qs))
			return;
		_NotifyQuestOfDeathList.add(qs);
	}

	/**
	 * Add a skill to the L2Character _skills and its Func objects to the
	 * calculator set of the L2Character.<BR>
	 * <BR>
	 * 
	 * <B><U> Concept</U> :</B><BR>
	 * <BR>
	 * All skills own by a L2Character are identified in <B>_skills</B><BR>
	 * <BR>
	 * 
	 * <B><U> Actions</U> :</B><BR>
	 * <BR>
	 * <li>Replace oldSkill by newSkill or Add the newSkill </li>
	 * <li>If an old skill has been replaced, remove all its Func objects of
	 * L2Character calculator set</li>
	 * <li>Add Func objects of newSkill to the calculator set of the
	 * L2Character </li>
	 * <BR>
	 * <BR>
	 * 
	 * <B><U> Overriden in </U> :</B><BR>
	 * <BR>
	 * <li> L2Player : Save update in the character_skills table of the database</li>
	 * <BR>
	 * <BR>
	 * 
	 * @param newSkill
	 *            The L2Skill to add to the L2Character
	 * 
	 * @return The L2Skill replaced or null if just added a new L2Skill
	 * 
	 */
	public L2Skill addSkill(L2Skill newSkill)
	{
		L2Skill oldSkill = null;

		if(newSkill != null)
		{
			// Replace oldSkill by newSkill or Add the newSkill
			oldSkill = _skills.put(newSkill.getId(), newSkill);

			if(newSkill.isOnAttack())
				_skillsOnAttack.put(newSkill.getId(), newSkill);

			// If an old skill has been replaced, remove all its Func objects
			if(oldSkill != null)
				removeStatsOwner(oldSkill);

			// Add Func objects of newSkill to the calculator set of the
			// L2Character
			addStatFuncs(newSkill.getStatFuncs(this));
		}

		return oldSkill;
	}

	/**
	 * Add a Func to the Calculator set of the L2Character.<BR>
	 * <BR>
	 * 
	 * <B><U> Concept</U> :</B><BR>
	 * <BR>
	 * A L2Character owns a table of Calculators called <B>_calculators</B>.
	 * Each Calculator (a calculator per state) own a table of Func object. A
	 * Func object is a mathematic function that permit to calculate the
	 * modifier of a state (ex : REGENERATE_HP_RATE...). To reduce cache memory
	 * use, L2NpcInstances who don't have skills share the same Calculator set
	 * called <B>NPC_STD_CALCULATOR</B>.<BR>
	 * <BR>
	 * 
	 * That's why, if a L2NpcInstance is under a skill/spell effect that modify
	 * one of its state, a copy of the NPC_STD_CALCULATOR must be create in its
	 * _calculators before addind new Func object.<BR>
	 * <BR>
	 * 
	 * <B><U> Actions</U> :</B><BR>
	 * <BR>
	 * <li>If _calculators is linked to NPC_STD_CALCULATOR, create a copy of
	 * NPC_STD_CALCULATOR in _calculators</li>
	 * <li>Add the Func object to _calculators</li>
	 * <BR>
	 * <BR>
	 * 
	 * @param f
	 *            The Func object to add to the Calculator corresponding to the
	 *            state affected
	 */
	public final synchronized void addStatFunc(Func f)
	{
		if(f == null)
			return;

		if(_calculators == NPC_STD_CALCULATOR)
		{
			_calculators = new Calculator[Stats.NUM_STATS];
			for(int i = 0; i < Stats.NUM_STATS; i++)
				if(NPC_STD_CALCULATOR[i] != null)
					_calculators[i] = new Calculator(NPC_STD_CALCULATOR[i]);
		}

		// Select the Calculator of the affected state in the Calculator set
		int stat = f._stat.ordinal();

		if(_calculators[stat] == null)
			_calculators[stat] = new Calculator();

		_calculators[stat].addFunc(f);
	}

	/**
	 * Add a list of Funcs to the Calculator set of the L2Character.<BR>
	 * <BR>
	 * 
	 * <B><U> Concept</U> :</B><BR>
	 * <BR>
	 * A L2Character owns a table of Calculators called <B>_calculators</B>.
	 * Each Calculator (a calculator per state) own a table of Func object. A
	 * Func object is a mathematic function that permit to calculate the
	 * modifier of a state (ex : REGENERATE_HP_RATE...). <BR>
	 * <BR>
	 * 
	 * <FONT COLOR=#FF0000><B> <U>Caution</U> : This method is ONLY for
	 * L2Player</B></FONT><BR>
	 * <BR>
	 * 
	 * <B><U> Example of use </U> :</B><BR>
	 * <BR>
	 * <li> Equip an item from inventory</li>
	 * <li> Learn a new passive skill</li>
	 * <li> Use an active skill</li>
	 * <BR>
	 * <BR>
	 * 
	 * @param funcs
	 *            The list of Func objects to add to the Calculator
	 *            corresponding to the state affected
	 */
	public final synchronized void addStatFuncs(Func[] funcs)
	{
		for(Func f : funcs)
			addStatFunc(f);
	}

	/**
	 * Add the object to the list of L2Character that must be informed of HP/MP
	 * updates of this L2Character.<BR>
	 * <BR>
	 * 
	 * <B><U> Concept</U> :</B><BR>
	 * <BR>
	 * Each L2Character owns a list called <B>_statusListener</B> that contains
	 * all L2Player to inform of HP/MP updates. Players who must be informed are
	 * players that target this L2Character. When a RegenTask is in progress
	 * sever just need to go through this list to send Server->Client packet
	 * StatusUpdate.<BR>
	 * <BR>
	 * 
	 * <B><U> Example of use </U> :</B><BR>
	 * <BR>
	 * <li> Target a PC or NPC</li>
	 * <BR>
	 * <BR>
	 * 
	 * @param object
	 *            L2Character to add to the listener
	 * 
	 */
	public void addStatusListener(L2Character object)
	{
		if(object == this)
			return;
		CopyOnWriteArraySet<L2Character> listeners;
		synchronized (this)
		{
			if(_statusListener == null)
				_statusListener = new CopyOnWriteArraySet<L2Character>();

			listeners = _statusListener;
		}
		listeners.add(object);
	}

	public void altOnMagicUseTimer(L2Character aimingTarget, L2Skill skill)
	{
		_altCastEndTime = 0;
		_altCastInterruptTime = 0;
		_altSkillTask = null;
		if(isAlikeDead())
			return;
		short magicId = skill.getDisplayId();
		int level = Math.max(1, getSkillDisplayLevel(skill.getId()));
		FastList<L2Character> targets = skill.getTargets(this, aimingTarget, true);
		broadcastPacket(new MagicSkillLaunched(_objectId, magicId, level, targets));
		double mpConsume2 = skill.getMpConsume2();
		if(mpConsume2 > 0)
		{
			if(_currentMp < mpConsume2)
			{
				sendPacket(Msg.NOT_ENOUGH_MP);
				return;
			}
			if(skill.isMagic())
				reduceCurrentMp(calcStat(Stats.MP_MAGIC_SKILL_CONSUME, mpConsume2, null, skill));
			else
				reduceCurrentMp(calcStat(Stats.MP_PHYSICAL_SKILL_CONSUME, mpConsume2, null, skill));
		}
		callSkill(skill, targets);
	}

	public void altUseSkill(L2Skill skill, L2Character target)
	{
		if(skill == null)
			return;
		short magicId = skill.getId();
		if(isSkillDisabled(magicId))
		{
			if(Config.ALT_SHOW_REUSE_MSG && isPlayer)
				sendPacket(new SystemMessage(SystemMessage.S1_IS_NOT_AVAILABLE_AT_THIS_TIME_BEING_PREPARED_FOR_REUSE).addSkillName(magicId, skill.getLevel()));
			return;
		}
		if(target == null)
		{
			target = skill.getAimingTarget(this);
			if(target == null)
				return;
		}

		int itemConsume[] = skill.getItemConsume();

		if(itemConsume[0] > 0)
			for(int i = 0; i < itemConsume.length; i++)
				if(!consumeItem(skill.getItemConsumeId()[i], itemConsume[i]))
				{
					sendPacket(Msg.INCORRECT_ITEM_COUNT);
					sendChanges();
					return;
				}

		int level = Math.max(1, getSkillDisplayLevel(magicId));
		_altCastEndTime = 10 + GameTimeController.getGameTicks() + skill.getSkillTime() / GameTimeController.MILLIS_IN_TICK;
		_altCastInterruptTime = GameTimeController.getGameTicks() + skill.getSkillInterruptTime() / GameTimeController.MILLIS_IN_TICK;
		Formulas.calcSkillMastery(skill, this);
		long reuseDelay = Formulas.calcSkillReuseDelay(this, skill);
		broadcastPacket(new MagicSkillUse(this, target, skill.getDisplayId(), level, skill.getSkillTime(), reuseDelay));
		if(!skill.isHandler())
			sendPacket(new SystemMessage(SystemMessage.USE_S1).addSkillName(magicId, (short) level));
		else if(!(skill.getId() >= 2278 && skill.getId() <= 2285)) // Не показывать сообщение для хербов
			sendPacket(new SystemMessage(SystemMessage.USE_S1).addItemName(skill.getItemConsumeId()[0]));
		// Skill reuse check
		if(reuseDelay > 30000)
			addSkillTimeStamp(skill.getId(), reuseDelay);
		if(reuseDelay > 10)
		{
			disableItem(skill, reuseDelay, reuseDelay);
			disableSkill(skill.getId(), reuseDelay);
		}
		_altSkillTask = ThreadPoolManager.getInstance().scheduleAi(new AltMagicUseTask(target, skill), skill.getSkillTime());
	}

	/**
	 * Break an attack and send Server->Client ActionFailed packet and a System
	 * Message to the L2Character.<BR>
	 * <BR>
	 */
	public void breakAttack()
	{
		if(isAttackingNow())
		{
			abortAttack();

			if(isPlayer)
				sendPacket(Msg.ATTACK_FAILED);
		}
	}

	/**
	 * Break a cast and send Server->Client ActionFailed packet and a System
	 * Message to the L2Character.<BR>
	 * <BR>
	 */
	public void breakCast(boolean force)
	{
		if(isCastingNow() && (force || canAbortCast()))
		{
			abortCast();

			if(isPlayer)
				sendPacket(new SystemMessage(SystemMessage.CASTING_HAS_BEEN_INTERRUPTED));
		}
	}

	/**
	 * Send a packet to the L2Character AND to all L2Player in the _KnownPlayers
	 * of the L2Character.<BR>
	 * <BR>
	 * 
	 * <B><U> Concept</U> :</B><BR>
	 * <BR>
	 * L2Player in the detection area of the L2Character are identified in
	 * <B>_knownPlayers</B>. In order to inform other players of state
	 * modification on the L2Character, server just need to go through
	 * _knownPlayers to send Server->Client Packet<BR>
	 * <BR>
	 * 
	 */
	public void broadcastPacket(L2GameServerPacket mov)
	{
		sendPacket(mov);
		broadcastPacketToOthers(mov);
	}

	/**
	 * Send a packet to all L2Player in the _KnownPlayers of the L2Character.<BR>
	 * <BR>
	 * 
	 * <B><U> Concept</U> :</B><BR>
	 * <BR>
	 * L2Player in the detection area of the L2Character are identified in
	 * <B>_knownPlayers</B>. In order to inform other players of state
	 * modification on the L2Character, server just need to go through
	 * _knownPlayers to send Server->Client Packet<BR>
	 * <BR>
	 * 
	 * <FONT COLOR=#FF0000><B> <U>Caution</U> : This method DOESN'T SEND
	 * Server->Client packet to this L2Character (to do this use function
	 * broadcastPacket)</B></FONT><BR>
	 * <BR>
	 * 
	 */
	public final void broadcastPacketToOthers(L2GameServerPacket mov)
	{
		if(!isVisible())
			return;

		FastList<L2Player> others = L2World.getAroundPlayers(this);
		for(FastList.Node<L2Player> n = others.head(), end = others.tail(); (n = n.getNext()) != end;)
		{
			L2Player target = n.getValue();
			if(_objectId != target.getObjectId())
				target.sendPacket(mov);
		}
	}

	/**
	 * Send the Server->Client packet StatusUpdate with current HP and MP to all
	 * other L2Player to inform.<BR>
	 * <BR>
	 * 
	 * <B><U> Actions</U> :</B><BR>
	 * <BR>
	 * <li>Create the Server->Client packet StatusUpdate with current HP and MP
	 * </li>
	 * <li>Send the Server->Client packet StatusUpdate with current HP and MP
	 * to all L2Character called _statusListener that must be informed of HP/MP
	 * updates of this L2Character </li>
	 * <BR>
	 * <BR>
	 * 
	 * <FONT COLOR=#FF0000><B> <U>Caution</U> : This method DOESN'T SEND CP
	 * information</B></FONT><BR>
	 * <BR>
	 * 
	 * <B><U> Overriden in </U> :</B><BR>
	 * <BR>
	 * <li> L2Player : Send current HP,MP and CP to the L2Player and only
	 * current HP, MP and Level to all other L2Player of the Party</li>
	 * <BR>
	 * <BR>
	 * 
	 */
	public void broadcastStatusUpdate()
	{
		CopyOnWriteArraySet<L2Character> list = _statusListener;

		if(list == null || list.isEmpty())
			return;

		if(!needStatusUpdate())
		{
			if(Config.DEBUG)
				_log.info("" + getName() + ": saving statusUpdate.");
			return;
		}

		StatusUpdate su = new StatusUpdate(_objectId);
		su.addAttribute(StatusUpdate.CUR_HP, (int) _currentHp);
		su.addAttribute(StatusUpdate.CUR_MP, (int) _currentMp);
		su.addAttribute(StatusUpdate.CUR_CP, (int) _currentCp);

		for(L2Character temp : list)
			if(!Config.FORCE_STATUSUPDATE)
			{
				if(temp.getTarget() == this)
					temp.sendPacket(su);
			}
			else
				temp.sendPacket(su);
	}

	public int calcHeading(int x_dest, int y_dest)
	{
		return (int) (Math.atan2(_y - y_dest, _x - x_dest) * HEADINGS_IN_PI) + 32768;
	}

	/**
	 * Calculate the new value of the state with modifiers that will be applied
	 * on the targeted L2Character.<BR>
	 * <BR>
	 * 
	 * <B><U> Concept</U> :</B><BR>
	 * <BR>
	 * A L2Character owns a table of Calculators called <B>_calculators</B>.
	 * Each Calculator (a calculator per state) own a table of Func object. A
	 * Func object is a mathematic function that permit to calculate the
	 * modifier of a state (ex : REGENERATE_HP_RATE...) : <BR>
	 * <BR>
	 * 
	 * FuncAtkAccuracy -> Math.sqrt(_player.getDEX())*6+_player.getLevel()<BR>
	 * <BR>
	 * 
	 * When the calc method of a calculator is launched, each mathematic
	 * function is called according to its priority <B>_order</B>. Indeed, Func
	 * with lowest priority order is executed firsta and Funcs with the same
	 * order are executed in unspecified order. The result of the calculation is
	 * stored in the value property of an Env class instance.<BR>
	 * <BR>
	 * 
	 * @param stat
	 *            The stat to calculate the new value with modifiers
	 * @param init
	 *            The initial value of the stat before applying modifiers
	 * @param object
	 *            The L2Charcater whose properties will be used in the
	 *            calculation (ex : CON, INT...)
	 * @param skill
	 *            The L2Skill whose properties will be used in the calculation
	 *            (ex : Level...)
	 * 
	 */
	public final double calcStat(Stats stat, double init, L2Character object, L2Skill skill)
	{
		int id = stat.ordinal();
		Calculator c = _calculators[id];
		// If no Func object found, no modifier is applied
		if(c == null || c.size() == 0)
			return init;
		// Create and init an Env object to pass parameters to the Calculator
		Env env = new Env();
		env.character = this;
		env.target = object;
		env.skill = skill;
		env.value = init;
		// Launch the calculation
		c.calc(env);
		if(env.value < 1 && (stat == Stats.STAT_CON || stat == Stats.STAT_DEX || stat == Stats.STAT_INT || stat == Stats.STAT_MEN || stat == Stats.STAT_STR || stat == Stats.STAT_WIT))
			env.value = 1;
		else if(env.value < 0)
			env.value = 0;

		return env.value;
	}

	/**
	 * Return the Attack Speed of the L2Character (delay (in milliseconds)
	 * before next attack).<BR>
	 * <BR>
	 */
	public int calculateAttackSpeed()
	{
		return Formulas.calcPAtkSpd(getPAtkSpd());
	}

	public void callSkill(L2Skill skill, FastList<L2Character> targets)
	{
		try
		{
			boolean needsMirrageCheck = isPlayer && getTarget() != null && skill.isOffensive();
			boolean needsOverhitFlags = skill.isOverhit();

			for(L2Character target : targets)
			{
				if(target.isMonster && needsOverhitFlags)
					target.setOverhitEnabled(true);

				if(needsMirrageCheck && Rnd.chance((int) target.calcStat(Stats.CANCEL_TARGET, 0, null, null)))
				{
					setTarget(null);
					needsMirrageCheck = false;
				}
			}

			skill.useSkill(this, targets);
		}
		catch(Exception e)
		{
			_log.log(Level.WARNING, "", e);
		}
	}

	/**
	 * Return True if the cast of the L2Character can be aborted.<BR>
	 * <BR>
	 */
	public final boolean canAbortCast()
	{
		return _castInterruptTime > GameTimeController.getGameTicks();
	}

	public boolean checkBlockedStat(Stats stat)
	{
		return _blockedStats != null && _blockedStats.contains(stat);
	}

	public boolean checkReflectSkill(L2Skill skill)
	{
		return Rnd.chance((int) calcStat(skill.isMagic() ? Stats.REFLECT_MAGIC_SKILL : Stats.REFLECT_PHYSIC_SKILL, 0, null, skill));
	}

	public void doCounterAttack(L2Skill skill, L2Character target)
	{
		if(skill.isMagic() || !skill.isOffensive() || skill.getCastRange() > 200)
			return;

		if(Rnd.chance((int) calcStat(Stats.COUNTER_ATTACK, 0, null, skill)))
		{
			double damage = Formulas.calcPhysDam(this, target, null, false, false, false, false);
			if(damage < 1)
				damage = 1;
			sendPacket(new SystemMessage(SystemMessage.YOU_HIT_FOR_S1_DAMAGE).addNumber((int) damage));
			target.reduceCurrentHp(damage, this);
		}
	}

	public void detachAI()
	{
		_ai = null;
	}

	public final void disableDrop(int time)
	{
		_dropDisabled = System.currentTimeMillis() + time;
	}

	/**
	 * Disable this skill id for the duration of the delay in milliseconds.
	 * 
	 * @param skillId
	 * @param delay (seconds * 1000)
	 */
	public void disableSkill(short skillId, long delay)
	{
		if(_disabledSkills == null)
			_disabledSkills = Collections.synchronizedList(new FastList<Short>());

		_disabledSkills.add(skillId);

		if(delay > 10)
			ThreadPoolManager.getInstance().scheduleAi(new EnableSkill(skillId), delay);
	}

	public void doAttack(L2Character target)
	{
		if(isAttackingNow())
			return;

		if(target == null)
			return;

		// Get the active weapon item corresponding to the active weapon
		// instance (always equipped in the right hand)
		L2Weapon weaponItem = getActiveWeaponItem();

		if(isAlikeDead() || target.isAlikeDead() || !isInRange(target, 2000))
			return;

		// Get the Attack Speed of the L2Character (delay (in milliseconds)
		// before next attack)
		int sAtk = Math.max(calculateAttackSpeed(), 200);

		// Get the Attack Reuse Delay of the L2Weapon
		int reuse = 0;
		if(weaponItem != null)
			if(weaponItem.getItemType() == WeaponType.BOW)
				reuse = (int) (sAtk / Config.BOW_REUSE_MODIFIER * getReuseModifier(target));
			else
				reuse = weaponItem.getAttackReuseDelay();

		// reuse = Math.max(100, reuse);

		_attackEndTime = sAtk + System.currentTimeMillis();
		_attackReuseEndTime = sAtk + reuse + System.currentTimeMillis();

		// Create a Server->Client packet Attack
		Attack attack = new Attack(this, getChargedSoulShot());

		_isAttackAborted = false;

		setHeading(target, true);

		// Select the type of attack to start
		if(weaponItem == null)
			doAttackHitSimple(attack, target, weaponItem, 1., true, sAtk);
		else
			switch(weaponItem.getItemType())
			{
				case BOW:
					doAttackHitByBow(attack, target, sAtk, reuse, weaponItem);
					break;
				case POLE:
					if(calcStat(Stats.POLE_ATTACK_ANGLE, 0, null, null) == -1)
						doAttackHitSimple(attack, target, weaponItem, 1., true, sAtk);
					else
						doAttackHitByPole(attack, weaponItem, sAtk);
					break;
				case DUAL:
				case DUALFIST:
					doAttackHitByDual(attack, target, weaponItem, sAtk);
					break;
				default:
					doAttackHitSimple(attack, target, weaponItem, 1., true, sAtk);
			}

		// If the Server->Client packet Attack contains at least 1 hit, send the
		// Server->Client packet Attack
		// to the L2Character AND to all L2Player in the _KnownPlayers of the
		// L2Character
		if(attack.hasHits())
			broadcastPacket(attack);

		ThreadPoolManager.getInstance().scheduleAi(new NotifyAITask(CtrlEvent.EVT_READY_TO_ACT), sAtk + reuse + 100);
	}

	/**
	 * Launch a Bow attack.<BR>
	 * <BR>
	 * 
	 * <B><U> Actions</U> :</B><BR>
	 * <BR>
	 * <li>Calculate if hit is missed or not </li>
	 * <li>Consumme arrows </li>
	 * <li>If hit isn't missed, calculate if shield defense is efficient </li>
	 * <li>If hit isn't missed, calculate if hit is critical </li>
	 * <li>If hit isn't missed, calculate physical damages </li>
	 * <li>If the L2Character is a L2Player, Send a Server->Client packet
	 * SetupGauge </li>
	 * <li>Create a new hit task with Medium priority</li>
	 * <li>Calculate and set the disable delay of the bow in function of the
	 * Attack Speed</li>
	 * <li>Add this hit to the Server-Client packet Attack </li>
	 * <BR>
	 * <BR>
	 * 
	 * @param attack
	 *            Server->Client packet Attack in which the hit will be added
	 * @param target
	 *            The L2Character targeted
	 * @param sAtk
	 *            The Attack Speed of the attacker
	 * @param weapon
	 *            L2Weapon item used by this
	 * 
	 * @return True if the hit isn't missed
	 * 
	 */
	protected boolean doAttackHitByBow(Attack attack, L2Character target, int sAtk, int reuse, L2Weapon weapon)
	{
		int damage1 = 0;
		boolean shld1 = false;
		boolean crit1 = false;

		// Calculate if hit is missed or not
		boolean miss1 = Formulas.calcHitMiss(this, target);

		reduceArrowCount();

		isMoving = false;

		if(!miss1)
		{
			shld1 = Formulas.calcShldUse(this, target);
			crit1 = Formulas.calcCrit(getCriticalHit(target, null));
			damage1 = (int) Formulas.calcPhysDam(this, target, null, shld1, crit1, false, attack._soulshot);
			if(crit1 && weapon != null)
				weapon.getEffect(true, this, target, true);
		}

		if(isPlayer)
		{
			sendPacket(Msg.GETTING_READY_TO_SHOOT_ARROWS);
			sendPacket(new SetupGauge(SetupGauge.RED, sAtk + reuse));
		}

		// Create a new hit task with Medium priority
		ThreadPoolManager.getInstance().scheduleAi(new HitTask(target, damage1, crit1, miss1, attack._soulshot, shld1, true), sAtk);

		attack.addHit(target, damage1, miss1, crit1, shld1);

		return !miss1;
	}

	/**
	 * Launch a Dual attack.<BR>
	 * <BR>
	 * 
	 * <B><U> Actions</U> :</B><BR>
	 * <BR>
	 * <li>Calculate if hits are missed or not </li>
	 * <li>If hits aren't missed, calculate if shield defense is efficient
	 * </li>
	 * <li>If hits aren't missed, calculate if hit is critical </li>
	 * <li>If hits aren't missed, calculate physical damages </li>
	 * <li>Create 2 new hit tasks with Medium priority</li>
	 * <li>Add those hits to the Server-Client packet Attack </li>
	 * <BR>
	 * <BR>
	 * 
	 * @param attack
	 *            Server->Client packet Attack in which the hit will be added
	 * @param target
	 *            The L2Character targeted
	 * @param weapon
	 *            L2Weapon item used by this
	 * 
	 * @return True if hit 1 or hit 2 isn't missed
	 * 
	 */
	protected boolean doAttackHitByDual(Attack attack, L2Character target, L2Weapon weapon, int sAtk)
	{
		int damage1 = 0;
		int damage2 = 0;
		boolean shld1 = false;
		boolean shld2 = false;
		boolean crit1 = false;
		boolean crit2 = false;

		boolean miss1 = Formulas.calcHitMiss(this, target);
		boolean miss2 = Formulas.calcHitMiss(this, target);

		if(!miss1)
		{
			shld1 = Formulas.calcShldUse(this, target);
			crit1 = Formulas.calcCrit(getCriticalHit(target, null));
			damage1 = (int) Formulas.calcPhysDam(this, target, null, shld1, crit1, true, attack._soulshot);
			if(crit1 && weapon != null)
				weapon.getEffect(true, this, target, true);
		}

		if(!miss2)
		{
			shld2 = Formulas.calcShldUse(this, target);
			crit2 = Formulas.calcCrit(getCriticalHit(target, null));
			damage2 = (int) Formulas.calcPhysDam(this, target, null, shld2, crit2, true, attack._soulshot);
			if(crit1 && weapon != null)
				weapon.getEffect(true, this, target, true);
		}

		if(damage1 + damage2 > 0)
			absorbAndReflect(target, damage1 + damage2);

		// Create a new hit task with Medium priority for hit 1 and for hit 2
		// with a higher delay
		ThreadPoolManager.getInstance().scheduleAi(new HitTask(target, damage1, crit1, miss1, attack._soulshot, shld1, true), sAtk / 2);
		ThreadPoolManager.getInstance().scheduleAi(new HitTask(target, damage2, crit2, miss2, attack._soulshot, shld2, false), sAtk);

		attack.addHit(target, damage1, miss1, crit1, shld1);
		attack.addHit(target, damage2, miss2, crit2, shld2);
		return !miss1 || !miss2;
	}

	/**
	 * Launch a Pole attack.<BR>
	 * <BR>
	 * 
	 * <B><U> Actions</U> :</B><BR>
	 * <BR>
	 * <li>Get all visible objects in a spheric area near the L2Character to
	 * obtain possible targets </li>
	 * <li>If possible target is the L2Character targeted, launch a simple
	 * attack against it </li>
	 * <li>If possible target isn't the L2Character targeted but is attakable,
	 * launch a simple attack against it </li>
	 * <BR>
	 * <BR>
	 * 
	 * @param attack
	 *            Server->Client packet Attack in which the hit will be added
	 * @param weapon
	 *            L2Weapon item used by this
	 * 
	 * @return True if one hit isn't missed
	 * 
	 */
	protected boolean doAttackHitByPole(Attack attack, L2Weapon weapon, int sAtk)
	{
		boolean hitted = false;
		int angle = (int) calcStat(Stats.POLE_ATTACK_ANGLE, 60, null, null);
		int range = (int) calcStat(Stats.POWER_ATTACK_RANGE, getTemplate().baseAtkRange, null, null);
		double mult = 1.;

		int attackcount = 0;
		// Используем Math.round т.к. обычный кастинг обрезает к меньшему
		// double d = 2.95. int i = (int)d, выйдет что i = 2
		// если 1% угла или 1 дистанции не играет огромной роли, то для
		// количества целей это критично
		int attackcountmax = (int) Math.round(calcStat(Stats.POLE_TARGERT_COUNT, 3, null, null));

		if(this instanceof L2RaidBossInstance)
			attackcountmax += 12;
		else if(this instanceof L2NpcInstance && getLevel() > 0)
			attackcountmax += getLevel() / 7.5;

		L2Character targ = getAI().getAttackTarget();
		if(!targ.isDead() && isInFront(targ, angle))
			if(doAttackHitSimple(attack, targ, weapon, mult, true, sAtk))
			{
				mult *= 0.85;
				attackcount++;
				hitted = true;
			}

		for(L2Character target : getKnownCharacters(range, 400))
			if(attackcount <= attackcountmax)
			{
				if(!target.isDead() && target.isAutoAttackable(this))
				{
					if(target == getAI().getAttackTarget() || !isInFront(target, angle))
						continue;

					hitted |= doAttackHitSimple(attack, target, weapon, mult, attackcount == 0, sAtk);
					target.getAI().notifyEvent(CtrlEvent.EVT_ATTACKED, this, false);

					mult *= 0.85;
				}
				attackcount++;
			}
			else
				break;

		// Return true if one hit isn't missed
		return hitted;
	}

	/**
	 * Launch a simple attack.<BR>
	 * <BR>
	 * 
	 * <B><U> Actions</U> :</B><BR>
	 * <BR>
	 * <li>Calculate if hit is missed or not </li>
	 * <li>If hit isn't missed, calculate if shield defense is efficient </li>
	 * <li>If hit isn't missed, calculate if hit is critical </li>
	 * <li>If hit isn't missed, calculate physical damages </li>
	 * <li>Create a new hit task with Medium priority</li>
	 * <li>Add this hit to the Server-Client packet Attack </li>
	 * <BR>
	 * <BR>
	 * 
	 * @param attack
	 *            Server->Client packet Attack in which the hit will be added
	 * @param target
	 *            The L2Character targeted
	 * @param weapon
	 *            L2Weapon item used by <b>this</b>
	 * @param multiplier
	 *            damage multiplier for polearmers
	 * @return True if the hit isn't missed
	 * 
	 */
	protected boolean doAttackHitSimple(Attack attack, L2Character target, L2Weapon weapon, double multiplier, boolean unchargeSS, int sAtk)
	{
		int damage1 = 0;
		boolean shld1 = false;
		boolean crit1 = false;

		// Calculate if hit is missed or not
		boolean miss1 = Formulas.calcHitMiss(this, target);

		// Check if hit isn't missed
		if(!miss1)
		{
			// Calculate if shield defense is efficient
			shld1 = Formulas.calcShldUse(this, target);

			// Calculate if hit is critical
			crit1 = Formulas.calcCrit(getCriticalHit(target, null));

			// Calculate physical damages
			damage1 = (int) Formulas.calcPhysDam(this, target, null, shld1, crit1, false, attack._soulshot);
			damage1 *= multiplier;

			absorbAndReflect(target, damage1);
			if(crit1 && weapon != null)
				weapon.getEffect(true, this, target, true);
		}
		else if(target.isPlayer)
			if(this instanceof L2NpcInstance)
				target.sendPacket(new SystemMessage(SystemMessage.AVOIDED_S1S_ATTACK).addNpcName(((L2NpcInstance) this).getTemplate().npcId));
			else
				target.sendPacket(new SystemMessage(SystemMessage.AVOIDED_S1S_ATTACK).addString(_name));

		// Create a new hit task with Medium priority
		ThreadPoolManager.getInstance().scheduleAi(new HitTask(target, damage1, crit1, miss1, attack._soulshot, shld1, unchargeSS), sAtk);

		// Add this hit to the Server-Client packet Attack
		attack.addHit(target, damage1, miss1, crit1, shld1);

		// Return true if hit isn't missed
		return !miss1;
	}

	public long getAnimationEndTime()
	{
		return _animationEndTime;
	}

	public void doCast(L2Skill skill, L2Character target, boolean forceUse)
	{
		// Прерывать дуэли если цель не дуэлянт
		if(getDuel() != null && target.getDuel() != getDuel())
			getDuel().setDuelState(getOwner(), Duel.DuelState.Interrupted);

		if(skill == null)
		{
			getAI().notifyEvent(CtrlEvent.EVT_CANCEL, null, null);
			return;
		}

		int itemConsume[] = skill.getItemConsume();

		if(itemConsume[0] > 0)
			for(int i = 0; i < itemConsume.length; i++)
				if(!consumeItem(skill.getItemConsumeId()[i], itemConsume[i]))
				{
					sendPacket(Msg.INCORRECT_ITEM_COUNT);
					sendChanges();
					return;
				}

		short magicId = skill.getId();

		if(magicId == L2Skill.BLINDING_BLOW)
			stopEffect(magicId);

		if(target == null)
		{
			target = skill.getAimingTarget(this);
			if(target == null)
				return;
		}

		int level = getSkillDisplayLevel(magicId);
		if(level < 1)
			level = 1;

		int skillTime = skill.isSkillTimePermanent() ? skill.getSkillTime() : Formulas.calcMAtkSpd(this, target, skill, skill.getSkillTime());
		int skillInterruptTime = skill.isMagic() ? Formulas.calcMAtkSpd(this, target, skill, skill.getSkillInterruptTime()) : 0;

		_animationEndTime = System.currentTimeMillis() + skillTime;

		if(skill.isMagic() && !skill.isSkillTimePermanent() && getChargedSpiritShot() > 0)
		{
			skillTime = (int) (0.70 * skillTime);
			skillInterruptTime = (int) (0.70 * skillInterruptTime);
		}

		_castEndTime = 10 + GameTimeController.getGameTicks() + skillTime / GameTimeController.MILLIS_IN_TICK;
		_castInterruptTime = GameTimeController.getGameTicks() + skillInterruptTime / GameTimeController.MILLIS_IN_TICK;

		Formulas.calcSkillMastery(skill, this); // Calculate skill mastery for
		// current cast
		long reuseDelay = Formulas.calcSkillReuseDelay(this, skill);

		if(reuseDelay < 500)
			reuseDelay = 500;

		broadcastPacket(new MagicSkillUse(this, target, skill.getDisplayId(), level, skillTime, reuseDelay));

		if(isPlayer)
			if(!skill.isHandler())
				sendPacket(new SystemMessage(SystemMessage.USE_S1).addSkillName(magicId, (short) level));
			else
				sendPacket(new SystemMessage(SystemMessage.USE_S1).addItemName(skill.getItemConsumeId()[0]));

		if(skill.getTargetType() == SkillTargetType.TARGET_HOLY)
			target.getAI().notifyEvent(CtrlEvent.EVT_AGGRESSION, this, 1);

		// Skill reuse check
		if(reuseDelay > 30000)
			addSkillTimeStamp(skill.getId(), reuseDelay);

		if(reuseDelay > 10)
		{
			disableItem(skill, reuseDelay, reuseDelay);
			disableSkill(skill.getId(), reuseDelay);
		}

		double mpConsume1 = skill.getMpConsume1();
		SkillType skillType = skill.getSkillType();

		if(mpConsume1 > 0)
		{
			if(skillType == SkillType.DANCE || skillType == SkillType.SONG)
			{
				int add = -30;
				for(L2Effect e : getAllEffects())
					if((e.getSkill().getSkillType() == SkillType.DANCE || e.getSkill().getSkillType() == SkillType.SONG) && e.getTime() < e.getPeriod() - 30000)
						add += 30;
				mpConsume1 += add;
				mpConsume1 = calcStat(Stats.MP_DANCE_SKILL_CONSUME, mpConsume1, null, skill);
			}

			if(skill.isMagic())
				reduceCurrentMp(calcStat(Stats.MP_MAGIC_SKILL_CONSUME, mpConsume1, null, skill));
			else
				reduceCurrentMp(calcStat(Stats.MP_PHYSICAL_SKILL_CONSUME, mpConsume1, null, skill));
		}

		boolean forceBuff = skill.getSkillType() == SkillType.FORCE_BUFF;

		if(forceBuff)
			startForceBuff(target, skill);

		// launch the magic in skillTime milliseconds
		if(skillTime > 50)
		{
			// Send a Server->Client packet SetupGauge with the color of the
			// gauge and the casting time
			if(isPlayer && !forceBuff)
				sendPacket(new SetupGauge(SetupGauge.BLUE, skillTime));

			if(magicId == L2Skill.EVADE_SHOT)
				skill.getEffects(this, this, false);

			// Create a task MagicUseTask with Medium priority to launch the
			// MagicSkill at the end of the casting time
			_skillTask = ThreadPoolManager.getInstance().scheduleAi(new MagicUseTask(target, skill, forceUse), skillTime);
		}
		else
			onMagicUseTimer(target, skill, forceUse);
	}

	public void startForceBuff(L2Character target, L2Skill skill)
	{
		if(_forceBuff == null)
			_forceBuff = new ForceBuff(this, target, skill);
	}

	public ForceBuff getForceBuff()
	{
		return _forceBuff;
	}

	public void setForceBuff(ForceBuff value)
	{
		_forceBuff = value;
	}

	public void doDie(L2Character killer)
	{
		// Set target to null and cancel Attack or Cast
		setTarget(null);

		// Stop movement
		stopMove(null);

		// Stop HP/MP/CP Regeneration task
		stopHpMpRegeneration();

		_currentHp = 0;

		if(isPlayer && killer instanceof L2Playable)
			_currentCp = 0;

		// Stop all active skills effects in progress on the L2Character
		if(isBlessedByNoblesse() || isSalvation())
		{
			if(isSalvation())
				((L2Player) this).reviveRequest((L2Player) this, 50, false);
			for(L2Effect e : getAllEffects())
				// Noblesse Blessing Buff/debuff effects are retained after
				// death. However, Noblesse Blessing and Lucky Charm are lost as
				// normal.
				if(e.getEffectType() == EffectType.BLESS_NOBLESSE || e.getSkill().getId() == 1325 || e.getSkill().getId() == 2168)
					e.exit();
		}
		else
			stopAllEffects();

		// Send the Server->Client packet StatusUpdate with current HP and MP to
		// all other L2Player to inform
		broadcastStatusUpdate();

		// Notify L2Character AI
		getAI().notifyEvent(CtrlEvent.EVT_DEAD, null, null);

		IScriptHandler handler = ScriptHandler.getInstance().getScriptHandler();
		if(handler != null)
			handler.onDie(this, killer);

		// Notify Quest of character's death
		L2NpcInstance npc = null;
		if(killer instanceof L2NpcInstance)
			npc = (L2NpcInstance) killer;

		for(QuestState qs : getNotifyQuestOfDeath())
			qs.getQuest().notifyDeath(npc, this, qs);

		_NotifyQuestOfDeathList.clear();
	}

	/** Sets HP, MP and CP and revives the L2Character. */
	public void doRevive()
	{
		if(!isTeleporting())
		{
			setIsPendingRevive(false);

			if(isSalvation())
			{
				for(L2Effect e : getAllEffects())
					// Stop Salvation effect
					if(e.getEffectType() == EffectType.SALVATION)
					{
						e.exit();
						break;
					}
				setCurrentCp(getMaxCp());
				setCurrentHp(getMaxHp());
				setCurrentMp(getMaxMp());
			}
			else
			{

				if(!isPlayer)
					setCurrentCp(getMaxCp() * Config.RESPAWN_RESTORE_CP);

				setCurrentHp(getMaxHp() * Config.RESPAWN_RESTORE_HP);

				if(Config.RESPAWN_RESTORE_MP >= 0)
					setCurrentMp(getMaxMp() * Config.RESPAWN_RESTORE_MP);
			}

			// Start broadcast status
			broadcastPacket(new Revive(this));
		}
		else
			setIsPendingRevive(true);

		if(isPet)
		{
			L2PetInstance pet = (L2PetInstance) this;
			pet.stopDecay();
			pet.setCurrentFed(pet.getMaxFed() / 10);
			pet.startFeed(false);
			setRunning();
		}
	}

	/**
	 * Enable a skill (remove it from _disabledSkills of the L2Character).<BR>
	 * <BR>
	 * 
	 * <B><U> Concept</U> :</B><BR>
	 * <BR>
	 * All skills disabled are identified by their skillId in <B>_disabledSkills</B>
	 * of the L2Character <BR>
	 * <BR>
	 * 
	 * @param skillId
	 *            The identifier of the L2Skill to enable
	 * 
	 */
	public void enableSkill(short skillId)
	{
		if(_disabledSkills == null)
			return;

		_disabledSkills.remove(new Short(skillId));
		removeSkillTimeStamp(skillId);
	}

	/**
	 * Return a map of 32 bits (0x00000000) containing all abnormal effect in
	 * progress for this L2Character.<BR>
	 * <BR>
	 * 
	 * <B><U> Concept</U> :</B><BR>
	 * <BR>
	 * In Server->Client packet, each effect is represented by 1 bit of the map
	 * (ex : BLEEDING = 0x0001 (bit 1), SLEEP = 0x0080 (bit 8)...). The map is
	 * calculated by applying a BINARY OR operation on each effect.<BR>
	 * <BR>
	 * 
	 * <B><U> Example of use </U> :</B><BR>
	 * <BR>
	 * <li> Server Packet : CharInfo, NpcInfo, NpcInfoPoly, UserInfo...</li>
	 * <BR>
	 * <BR>
	 */
	public int getAbnormalEffect()
	{
		int ae = _abnormalEffects;
		if(isStunned())
			ae |= ABNORMAL_EFFECT_STUN;
		if(isRooted())
			ae |= ABNORMAL_EFFECT_ROOT;
		if(isSleeping())
			ae |= ABNORMAL_EFFECT_SLEEP;
		if(isConfused())
			ae |= ABNORMAL_EFFECT_CONFUSED;
		if(isMuted() || isPMuted())
			ae |= ABNORMAL_EFFECT_MUTED;
		if(isAfraid())
			ae |= ABNORMAL_EFFECT_AFFRAID;
		if(getEffect(EffectType.SILENT_MOVE) != null)
			ae |= ABNORMAL_EFFECT_SILENT_MOVE;
		if(getEffect(EffectType.BIG_HEAD) != null)
			ae |= ABNORMAL_EFFECT_BIG_HEAD;
		if(isCastingCircle())
			ae |= ABNORMAL_EFFECT_MAGIC_CIRCLE;
		if(getCustomEffect() > 0)
			ae |= getCustomEffect();
		return ae;
	}

	/**
	 * Return the Accuracy (base+modifier) of the L2Character in function of the
	 * Weapon Expertise Penalty.<BR>
	 * <BR>
	 */
	public int getAccuracy()
	{
		return (int) (calcStat(Stats.ACCURACY_COMBAT, 0, null, null) / getWeaponExpertisePenalty());
	}

	/**
	 * Return the L2CharacterAI of the L2Character and if its null create a new
	 * one.<BR>
	 * <BR>
	 */
	@Override
	public L2CharacterAI getAI()
	{
		if(_ai == null)
			_ai = new L2CharacterAI(this);
		return _ai;
	}

	public final ConcurrentSkipListSet<L2Effect> getAllEffects()
	{
		if(_effects == null || _effects.isEmpty())
			return new ConcurrentSkipListSet<L2Effect>();

		ConcurrentSkipListSet<L2Effect> temp = new ConcurrentSkipListSet<L2Effect>();
		for(ConcurrentSkipListSet<L2Effect> effects : _effects.values())
			if(effects != null)
				for(L2Effect effect : effects)
					if(effect != null)
						temp.add(effect);

		return temp;
	}

	/**
	 * Возвращает первые эффекты для всех скиллов. Нужно для отображения не
	 * более чем 1 иконки для каждого скилла.
	 */
	public final ConcurrentSkipListSet<L2Effect> getAllFirstEffects()
	{
		if(_effects == null || _effects.isEmpty())
			return new ConcurrentSkipListSet<L2Effect>();

		ConcurrentSkipListSet<L2Effect> temp = new ConcurrentSkipListSet<L2Effect>();
		for(ConcurrentSkipListSet<L2Effect> effects : _effects.values())
			if(effects != null)
				for(L2Effect effect : effects)
					if(effect != null)
					{
						temp.add(effect);
						break;
					}

		return temp;
	}

	/**
	 * Возвращает коллекцию скиллов для быстрого перебора
	 */
	public final Collection<L2Skill> getAllSkills()
	{
		return _skills.values();
	}

	/**
	 * Возвращает массив скиллов для безопасного перебора
	 */
	public final L2Skill[] getAllSkillsArray()
	{
		if(_skills == null)
			return new L2Skill[0];

		synchronized (_skills)
		{
			return _skills.values().toArray(new L2Skill[_skills.values().size()]);
		}
	}

	/**
	 * Return the Armour Expertise Penalty of the L2Character.<BR>
	 * <BR>
	 */
	public float getArmourExpertisePenalty()
	{
		return 1.f;
	}

	public FastList<L2Player> getAroundPlayers(int radius)
	{
		if(!isVisible())
			return new FastList<L2Player>();

		return L2World.getAroundPlayers(this, radius, Config.PLAYER_VISIBILITY_Z);
	}

	/**
	 * Return the Attack Speed multiplier (base+modifier) of the L2Character to
	 * get proper animations.<BR>
	 * <BR>
	 */
	public final float getAttackSpeedMultiplier()
	{
		return (float) (1.1 * getPAtkSpd() / getTemplate().basePAtkSpd);
	}

	public L2CharTemplate getBaseTemplate()
	{
		return _template;
	}

	public int getBuffLimit()
	{
		return (int) calcStat(Stats.BUFF_LIMIT, 20, null, null);
	}

	public L2Skill getCastingSkill()
	{
		return _castingSkill;
	}

	/**
	 * If target is L2Character return it, else return null.<BR>
	 * <BR>
	 */
	public final L2Character getCharTarget()
	{
		if(_target == null || !_target.isCharacter)
			return null;
		return (L2Character) _target;
	}

	/**
	 * Return the COM of the L2Character (base+modifier).<BR>
	 * <BR>
	 */
	public byte getCON()
	{
		return (byte) calcStat(Stats.STAT_CON, _template.baseCON, null, null);
	}

	/**
	 * Return the Critical Hit rate (base+modifier) of the L2Character.<BR>
	 * <BR>
	 */
	public int getCriticalHit(L2Character target, L2Skill skill)
	{
		int val = (int) calcStat(Stats.CRITICAL_RATE_STATIC, _template.baseCritRate, target, skill);
		if(Config.LIM_CRIT != 0 && val > Config.LIM_CRIT)
			val = Config.LIM_CRIT;
		return val;
	}

	/**
	 * Return the current CP of the L2Character.<BR>
	 * <BR>
	 */
	public final double getCurrentCp()
	{
		return _currentCp;
	}

	/**
	 * Return the current HP of the L2Character.<BR>
	 * <BR>
	 */
	public final double getCurrentHp()
	{
		return _currentHp;
	}

	/**
	 * Return the current MP of the L2Character.<BR>
	 * <BR>
	 */
	public final double getCurrentMp()
	{
		return _currentMp;
	}

	public Location getDestination()
	{
		return new Location(_xTo, _yTo, _zTo);
	}

	/**
	 * Return the DEX of the L2Character (base+modifier).<BR>
	 * <BR>
	 */
	public byte getDEX()
	{
		return (byte) calcStat(Stats.STAT_DEX, _template.baseDEX, null, null);
	}

	public synchronized final L2Effect getEffect(EffectType et)
	{
		if(_effects == null)
			return null;

		for(ConcurrentSkipListSet<L2Effect> es : _effects.values())
			for(L2Effect e : es)
				if(e.getEffectType() == et)
					return e;

		return null;
	}

	public synchronized final ConcurrentSkipListSet<L2Effect> getEffect(L2Skill skill)
	{
		return _effects == null || skill == null ? null : _effects.get(skill.getId());
	}

	public synchronized final L2Effect getEffect(ResistType rt)
	{
		if(_effects == null)
			return null;

		for(ConcurrentSkipListSet<L2Effect> es : _effects.values())
			for(L2Effect e : es)
				if(e.getResistType() == rt)
					return e;

		return null;
	}

	public final ConcurrentSkipListSet<L2Effect> getEffect(Short index)
	{
		return _effects == null ? null : _effects.get(index);
	}

	public final L2Effect getEffectByIndexAndType(Short index, EffectType type)
	{
		if(_effects == null || _effects.get(index) == null)
			return null;
		for(L2Effect ef : _effects.get(index))
			if(ef != null && ef.getEffectType().equals(type))
				return ef;
		return null;
	}

	/**
	 * Return the Attack Evasion rate (base+modifier) of the L2Character.<BR>
	 * <BR>
	 */
	public int getEvasionRate(L2Character target)
	{
		return (int) (calcStat(Stats.EVASION_RATE, 0, target, null) / getArmourExpertisePenalty());
	}

	/**
	 * Return the orientation of the L2Character.<BR>
	 * <BR>
	 */
	@Override
	public final int getHeading()
	{
		return _heading;
	}

	/**
	 * Return heading to L2Character<BR>
	 * If <b>boolean toChar</b> is true heading calcs this->target, else
	 * target->this.<BR>
	 * <BR>
	 */
	public int getHeadingTo(L2Object target, boolean toChar)
	{
		if(target == null || target == this)
			return -1;

		int dx = target.getX() - _x;
		int dy = target.getY() - _y;
		int heading = (int) (Math.atan2(-dy, -dx) * 32768. / Math.PI + 32768);

		heading = toChar ? target.getHeading() - heading : _heading - heading;

		if(heading < 0)
			heading += 65536;
		return heading;
	}

	/**
	 * Return the INT of the L2Character (base+modifier).<BR>
	 * <BR>
	 */
	public byte getINT()
	{
		return (byte) calcStat(Stats.STAT_INT, _template.baseINT, null, null);
	}

	public FastList<L2Character> getKnownCharacters(int radius)
	{
		if(!isVisible())
			return new FastList<L2Character>();

		return L2World.getAroundCharacters(this, radius, Config.PLAYER_VISIBILITY_Z);
	}

	public FastList<L2Character> getKnownCharacters(int radius, int height)
	{
		if(!isVisible())
			return new FastList<L2Character>();

		return L2World.getAroundCharacters(this, radius, height);
	}

	public FastList<L2NpcInstance> getKnownNpc(int range)
	{
		if(!isVisible())
			return new FastList<L2NpcInstance>();

		return L2World.getAroundNpc(this, range, Config.PLAYER_VISIBILITY_Z);
	}

	/**
	 * Return Skill if the skill is known by the L2Character.<BR>
	 * <BR>
	 * 
	 * @param skillId
	 *            The identifier of the L2Skill to check the knowledge
	 * 
	 */
	public final L2Skill getKnownSkill(int skillId)
	{
		if(_skills == null)
			return null;

		return _skills.get(new Short((short) skillId));
	}

	/**
	 * Return the Magical Attack range (base+modifier) of the L2Character.<BR>
	 * <BR>
	 */
	public final int getMagicalAttackRange(L2Skill skill)
	{
		if(skill != null)
			return (int) calcStat(Stats.MAGIC_ATTACK_RANGE, skill.getCastRange(), null, skill);
		return getTemplate().baseAtkRange;
	}

	/**
	 * Return the MAtk (base+modifier) of the L2Character for a skill used in
	 * function of abnormal effects in progress.<BR>
	 * <BR>
	 * 
	 * <B><U> Example of use </U> :</B><BR>
	 * <BR>
	 * <li> Calculate Magic damage </li>
	 * <BR>
	 * <BR>
	 * 
	 * @param target
	 *            The L2Character targeted by the skill
	 * @param skill
	 *            The L2Skill used against the target
	 * 
	 */
	public int getMAtk(L2Character target, L2Skill skill)
	{
		return (int) calcStat(Stats.MAGIC_ATTACK, _template.baseMAtk, target, skill);
	}

	public double getMAtkDebuff(L2Character target, L2Skill skill)
	{
		double matk = calcStat(Stats.MAGIC_ATTACK, _template.baseMAtk, target, skill);
		switch(getChargedSpiritShot())
		{
			case 1:
				return matk * 1.15;
			case 2:
				return matk * 1.3;
			default:
				return matk;
		}
	}

	/**
	 * Return the MAtk Speed (base+modifier) of the L2Character in function of
	 * the Armour Expertise Penalty.<BR>
	 * <BR>
	 */
	public int getMAtkSpd()
	{
		int val = (int) (calcStat(Stats.MAGIC_ATTACK_SPEED, _template.baseMAtkSpd, null, null) / getArmourExpertisePenalty());
		if(Config.LIM_MATK_SPD != 0 && val > Config.LIM_MATK_SPD)
			val = Config.LIM_MATK_SPD;
		return val;
	}

	public double getMAtkSps(L2Character target, L2Skill skill)
	{
		double matk = calcStat(Stats.MAGIC_ATTACK, _template.baseMAtk, target, skill);
		switch(getChargedSpiritShot())
		{
			case 1:
				return matk * 1.41;
			case 2:
				return matk * 2;
			default:
				return matk;
		}
	}

	/**
	 * Return the Max CP (base+modifier) of the L2Character.<BR>
	 * <BR>
	 */
	public final int getMaxCp()
	{
		return (int) calcStat(Stats.MAX_CP, _template.baseCpMax, null, null);
	}

	/**
	 * Return the Max HP (base+modifier) of the L2Character.<BR>
	 * <BR>
	 */
	public int getMaxHp()
	{
		if(this instanceof L2NpcInstance)
			return (int) calcStat(Stats.MAX_HP, _template.baseHpMax * ((L2NpcTemplate) _template).rateHp, null, null);

		return (int) calcStat(Stats.MAX_HP, _template.baseHpMax, null, null);
	}

	/**
	 * Return the Max MP (base+modifier) of the L2Character.<BR>
	 * <BR>
	 */
	public int getMaxMp()
	{
		return (int) calcStat(Stats.MAX_MP, _template.baseMpMax, null, null);
	}

	/**
	 * Return the MDef (base+modifier) of the L2Character against a skill in
	 * function of abnormal effects in progress.<BR>
	 * <BR>
	 * 
	 * <B><U> Example of use </U> :</B><BR>
	 * <BR>
	 * <li> Calculate Magic damage </li>
	 * <BR>
	 * <BR>
	 * 
	 * @param target
	 *            The L2Character targeted by the skill
	 * @param skill
	 *            The L2Skill used against the target
	 * 
	 */
	public int getMDef(L2Character target, L2Skill skill)
	{
		return Math.max((int) calcStat(Stats.MAGIC_DEFENCE, _template.baseMDef, null, skill), 1);
	}

	/**
	 * Return the MEN of the L2Character (base+modifier).<BR>
	 * <BR>
	 */
	public byte getMEN()
	{
		return (byte) calcStat(Stats.STAT_MEN, _template.baseMEN, null, null);
	}

	public int getMinDistance(L2Object obj)
	{
		int distance = getTemplate().collisionRadius;

		if(obj.isCharacter)
			distance += ((L2Character) obj).getTemplate().collisionRadius;

		return distance;
	}

	public float getMovementSpeedMultiplier()
	{
		return getRunSpeed() * 1f / _template.baseRunSpd;
	}

	/**
	 * Return the RunSpeed (base+modifier) or WalkSpeed (base+modifier) of the
	 * L2Character in function of the movement type.<BR>
	 * <BR>
	 */
	@Override
	public float getMoveSpeed()
	{
		if(isRunning())
			return getRunSpeed();

		return getWalkSpeed();
	}

	/**
	 * Return the MReuse rate (base+modifier) of the L2Character.<BR>
	 * <BR>
	 */
	public final double getMReuseRate(L2Skill skill)
	{
		return calcStat(Stats.MAGIC_REUSE_RATE, _template.baseMReuseRate, null, skill);
	}

	public String getName()
	{
		return _name;
	}

	/**
	 * Return a list of L2Character that attacked.<BR>
	 * <BR>
	 */
	public final ArrayList<QuestState> getNotifyQuestOfDeath()
	{
		if(_NotifyQuestOfDeathList == null)
			_NotifyQuestOfDeathList = new ArrayList<QuestState>();
		return _NotifyQuestOfDeathList;
	}

	/**
	 * Return the PAtk (base+modifier) of the L2Character.<BR>
	 * <BR>
	 */
	public int getPAtk(L2Character target)
	{
		return (int) calcStat(Stats.POWER_ATTACK, _template.basePAtk, target, null);
	}

	/**
	 * Return the PAtk Speed (base+modifier) of the L2Character in function of
	 * the Armour Expertise Penalty.<BR>
	 * <BR>
	 */
	public int getPAtkSpd()
	{
		int val = (int) (calcStat(Stats.POWER_ATTACK_SPEED, _template.basePAtkSpd, null, null) / getArmourExpertisePenalty());
		if(Config.LIM_PATK_SPD != 0 && val > Config.LIM_PATK_SPD)
			val = Config.LIM_PATK_SPD;
		return val;
	}

	/**
	 * Return the PDef (base+modifier) of the L2Character.<BR>
	 * <BR>
	 */
	public int getPDef(L2Character target)
	{
		return Math.max((int) calcStat(Stats.POWER_DEFENCE, _template.basePDef, target, null), 1);
	}

	/**
	 * Return the Physical Attack range (base+modifier) of the L2Character.<BR>
	 * <BR>
	 */
	public final int getPhysicalAttackRange()
	{
		return (int) calcStat(Stats.POWER_ATTACK_RANGE, getTemplate().baseAtkRange, null, null);
	}

	/**
	 * Return a Random Damage in function of the weapon.<BR>
	 * <BR>
	 */
	public final int getRandomDamage()
	{
		L2Weapon weaponItem = getActiveWeaponItem();

		if(weaponItem == null)
			return 5 + (int) Math.sqrt(getLevel());

		return weaponItem.getRandomDamage();
	}

	/**
	 * Return the Skill/Spell reuse modifier.<BR>
	 * <BR>
	 */
	public double getReuseModifier(L2Character target)
	{
		return calcStat(Stats.ATK_REUSE, 1, target, null);
	}

	/**
	 * @return the RunSpeed of the L2Character.
	 */
	public int getRunSpeed()
	{
		if(isSwimming())
			return getSwimSpeed();
		return getSpeed(_template.baseRunSpd);
	}

	/**
	 * @return the ShieldDef rate (base+modifier) of the L2Character.
	 */
	public final int getShldDef()
	{
		if(isPlayer)
			return (int) calcStat(Stats.SHIELD_DEFENCE, 0, null, null);
		return (int) calcStat(Stats.SHIELD_DEFENCE, _template.baseShldDef, null, null);
	}

	public final short getSkillDisplayLevel(Short skillId)
	{
		if(_skills == null)
			return -1;

		L2Skill skill = _skills.get(skillId);

		if(skill == null)
			return -1;

		return skill.getDisplayLevel();
	}

	/**
	 * Return the level of a skill owned by the L2Character.<BR>
	 * <BR>
	 * 
	 * @param skillId
	 *            The identifier of the L2Skill whose level must be returned
	 * 
	 * @return The level of the L2Skill identified by skillId
	 * 
	 */
	public final short getSkillLevel(Short skillId)
	{
		if(_skills == null)
			return -1;

		L2Skill skill = _skills.get(skillId);

		if(skill == null)
			return -1;

		return skill.getLevel();
	}

	public byte getSkillMastery(short skillId)
	{
		Byte val = _skillMastery.get(skillId);
		return val == null ? 0 : val;
	}

	public void removeSkillMastery(short skillId)
	{
		_skillMastery.remove(skillId);
	}

	/**
	 * Return all skills by type own by the L2Character in a table of L2Skill.<BR>
	 * <BR>
	 * 
	 * <B><U> Concept</U> :</B><BR>
	 * <BR>
	 * All skills own by a L2Character are identified in <B>_skills</B> the
	 * L2Character <BR>
	 * <BR>
	 * 
	 */
	public final FastTable<L2Skill> getSkillsByType(SkillType type)
	{
		if(_skills == null)
			return new FastTable<L2Skill>();

		FastTable<L2Skill> result = new FastTable<L2Skill>();
		for(L2Skill sk : _skills.values())
			if(sk.getSkillType() == type)
				result.add(sk);

		return result;
	}

	/**
	 * Return the Speed (base+modifier) of the L2Character in function of the
	 * Armour Expertise and Territories Penalty.<BR>
	 * <BR>
	 */
	public final int getSpeed(int baseSpeed)
	{
		if(isFlying())
			return Config.WYVERN_SPEED;

		if(isRiding())
			return Config.STRIDER_SPEED;

		// err we should be adding TO the persons speed
		// not making it a constant
		double val = calcStat(Stats.RUN_SPEED, baseSpeed, null, null);

		val /= getArmourExpertisePenalty();

		return (int) (val + 0.5);
	}

	/**
	 * Return the list of L2Character that must be informed of HP/MP updates of
	 * this L2Character.<BR>
	 * <BR>
	 * 
	 * <B><U> Concept</U> :</B><BR>
	 * <BR>
	 * Each L2Character owns a list called <B>_statusListener</B> that contains
	 * all L2Player to inform of HP/MP updates. Players who must be informed are
	 * players that target this L2Character. When a RegenTask is in progress
	 * sever just need to go through this list to send Server->Client packet
	 * StatusUpdate.<BR>
	 * <BR>
	 * 
	 * @return The list of L2Character to inform or null if empty
	 * 
	 */
	private CopyOnWriteArraySet<L2Character> getStatusListener()
	{
		return _statusListener;
	}

	/**
	 * Return the STR of the L2Character (base+modifier).<BR>
	 * <BR>
	 */
	public byte getSTR()
	{
		return (byte) calcStat(Stats.STAT_STR, _template.baseSTR, null, null);
	}

	public int getSwimSpeed()
	{
		return (int) calcStat(Stats.RUN_SPEED, Config.SWIMING_SPEED, null, null);
	}

	/**
	 * Return the L2Object targeted or null.<BR>
	 * <BR>
	 */
	public final L2Object getTarget()
	{
		return _target;
	}

	/**
	 * Return the identifier of the L2Object targeted or -1.<BR>
	 * <BR>
	 */
	public final int getTargetId()
	{
		if(_target != null)
			return _target.getObjectId();
		return -1;
	}

	/**
	 * Return the template of the L2Character.<BR>
	 * <BR>
	 * 
	 * <B><U> Concept</U> :</B><BR>
	 * <BR>
	 * Each L2Character owns generic and static properties (ex : all Keltir have
	 * the same number of HP...). All of those properties are stored in a
	 * different template for each type of L2Character. Each template is loaded
	 * once in the server cache memory (reduce memory use). When a new instance
	 * of L2Character is spawned, server just create a link between the instance
	 * and the template This link is stored in <B>_template</B><BR>
	 * <BR>
	 * 
	 */
	public L2CharTemplate getTemplate()
	{
		return _template;
	}

	/**
	 * Return the Title of the L2Character.<BR>
	 * <BR>
	 */
	public String getTitle()
	{
		if(_title != null && _title.length() > 16)
			return _title.substring(0, 16);
		return _title;
	}

	/**
	 * Return the WalkSpeed of the L2Character.<BR>
	 * <BR>
	 */
	public final int getWalkSpeed()
	{
		if(isSwimming())
			return getSwimSpeed();
		return getSpeed(_template.baseWalkSpd);
		// return (getSpeed(_template.baseWalkSpd) * 70) / 100;
	}

	/**
	 * Return the Weapon Expertise Penalty of the L2Character.<BR>
	 * <BR>
	 */
	public float getWeaponExpertisePenalty()
	{
		return 1.f;
	}

	/**
	 * Return the WIT of the L2Character (base+modifier).<BR>
	 * <BR>
	 */
	public byte getWIT()
	{
		return (byte) calcStat(Stats.STAT_WIT, _template.baseWIT, null, null);
	}

	/**
	 * Return True if the L2Character has a L2CharacterAI.<BR>
	 * <BR>
	 */
	@Override
	public boolean hasAI()
	{
		return _ai != null;
	}

	public double headingToRadians(int heading)
	{
		return (heading - 32768) / HEADINGS_IN_PI;
	}

	/**
	 * Return True if the L2Character is dead or use fake death.<BR>
	 * <BR>
	 */
	public final boolean isAlikeDead()
	{
		return _fake_death || _currentHp < 0.5;
	}

	/**
	 * Return True if the L2Character has aborted its attack.<BR>
	 * <BR>
	 */
	public boolean isAttackAborted()
	{
		return _isAttackAborted;
	}

	/**
	 * Return True if the L2Character is attacking.<BR>
	 * <BR>
	 */
	public final boolean isAttackingNow()
	{
		return _attackEndTime > System.currentTimeMillis();
	}

	/**
	 * Return True if the L2Character is behind the target and can't be seen.<BR>
	 * <BR>
	 */
	public boolean isBehindTarget()
	{
		if(_target != null && _target.isCharacter)
		{
			int head = getHeadingTo(_target, true);
			return head != -1 && (head <= 10430 || head >= 55105);
		}

		return false;
	}

	/**
	 * Return True if the L2Character is behind the target and can't be seen.<BR>
	 * <BR>
	 */
	public boolean isBehindTarget(L2Object target)
	{
		if(target != null && target.isCharacter)
		{
			int head = getHeadingTo(target, true);
			return head != -1 && (head <= 10430 || head >= 55105);
		}
		return false;
	}

	public final boolean isBlessedByNoblesse()
	{
		return _IsBlessedByNoblesse > 0;
	}

	public final boolean isSalvation()
	{
		return _IsSalvation > 0;
	}

	public final boolean isBuffImmunity()
	{
		return _buffImmunity > 0;
	}

	/**
	 * Return True if the L2Character is dead.<BR>
	 * <BR>
	 */
	public final boolean isDead()
	{
		return _currentHp < 0.5;
	}

	public final boolean isDropDisabled()
	{
		return _dropDisabled > System.currentTimeMillis();
	}

	/**
	 * Return True if the L2Character is flying.<BR>
	 * <BR>
	 */
	public final boolean isFlying()
	{
		return _flying;
	}

	/**
	 * Return True if the L2Character is in combat.<BR>
	 * <BR>
	 */
	public final boolean isInCombat()
	{
		return _stanceTask != null;
	}

	/**
	 * Return True if the target is front L2Character and can be seen.<BR>
	 * <BR>
	 * degrees = 0..180, front->sides->back
	 */
	public boolean isInFront(L2Object target, int degrees)
	{
		int head = getHeadingTo(target, false);
		return head <= 32768 * degrees / 180 || head >= 65536 - 32768 * degrees / 180;
	}

	/**
	 * Return True if the L2Player is invulnerable.<BR>
	 * <BR>
	 */
	public boolean isInvul()
	{
		return _isInvul;
	}

	public boolean isKnownPlayers()
	{
		return isVisible() && L2World.isAroundPlayers(this);
	}

	/**
	 * Check if player is mage
	 */
	public boolean isMageClass()
	{
		return getTemplate().baseMAtk > 3;

	}

	public final boolean isPendingRevive()
	{
		return isDead() && _IsPendingRevive;
	}

	/**
	 * Return True if the L2Character is riding.<BR>
	 * <BR>
	 */
	public final boolean isRiding()
	{
		return _riding;
	}

	/**
	 * Return True if the L2Character is running.<BR>
	 * <BR>
	 */
	public final boolean isRunning()
	{
		return _running;
	}

	public boolean isSkillDisabled(short skillId)
	{
		return _disabledSkills == null ? false : _disabledSkills.contains(skillId);
	}

	public boolean isSwimming()
	{
		return isPlayer && ((L2Player) this).getWaterTask() != null;
	}

	public final boolean isTeleporting()
	{
		return _IsTeleporting;
	}

	public boolean knowsObject(L2Object obj)
	{
		return isInRange(obj, Config.PLAYER_VISIBILITY);
	}

	private void pathClean()
	{
		try
		{
			long ti = System.currentTimeMillis();

			int current = 0;
			int sub;
			synchronized (_targetRecorder)
			{
				while(current < _targetRecorder.size() - 2)
				{
					sub = current + 2;
					while(sub < _targetRecorder.size())
					{
						if(clean(_targetRecorder.elementAt(current), _targetRecorder.elementAt(sub)))
						{
							_targetRecorder.remove(current + 1);
							sub--;
						}
						sub++;
					}
					current++;
				}
			}

			if(isPlayer && getAccessLevel() >= 100 && Config.GEODATA_DEBUG)
			{
				sendMessage("Размер пути после сглаживания: " + _targetRecorder.size(), "");
				sendMessage("Потраченное время: " + (System.currentTimeMillis() - ti) + "ms", "");
			}
		}
		catch(Exception e)
		{
			if(isPlayer && getAccessLevel() >= 100 && Config.GEODATA_DEBUG)
			{
				sendMessage("CheckMap excep : " + e.getMessage(), "");
				e.printStackTrace();
			}
		}
	}

	private boolean clean(Location loc1, Location loc2)
	{
		//if(isPlayer)
		return GeoEngine.getInstance().canMoveToCoordWithCollision(loc1.x, loc1.y, loc1.z, loc2.x, loc2.y, loc2.z);
		//else
		//	return GeoEngine.getInstance().canMoveToCoord(loc1.x, loc1.y, loc1.z, loc2.x, loc2.y, loc2.z);
	}

	/**
	 * Возвращает позицию цели, в которой она будет через секунду.
	 */
	public static Location getIntersectionPoint(L2Character target)
	{
		double angle = Util.convertHeadingToDegree(target.getHeading()); // угол в градусах
		double radian = Math.toRadians(angle - 90); // угол в радианах
		double range = target.getMoveSpeed(); // расстояние, пройденное за 1 секунду, равно скорости
		return new Location((int) (target.getX() - range * Math.sin(radian)), (int) (target.getY() + range * Math.cos(radian)), target.getZ());
	}

	public Location applyOffset(Location point, int offset)
	{
		if(_forestalling && isFollow && followTarget != null && (followTarget.isMoving || followTarget.isFollow))
			return getIntersectionPoint(followTarget);

		if(offset <= 0)
			return point;

		int dx = point.x - _x;
		int dy = point.y - _y;
		int dz = point.z - _z;

		double distance = Math.sqrt(dx * dx + dy * dy + dz * dz);

		if(distance <= offset)
		{
			point.x = _x;
			point.y = _y;
			point.z = _z;
			return point;
		}

		if(distance >= 1)
		{
			double cut = offset / distance;
			point.x -= dx * cut;
			point.y -= dy * cut;
			point.z -= dz * cut;

			if(!isFlying() && !isSwimming() && !isInBoat())
				point.z = GeoData.getInstance().getHeight(point.x, point.y, point.z);
		}

		return point;
	}

	public Location getNextPointToMove()
	{
		synchronized (_targetRecorder)
		{
			if(isSwimming() || isFlying() || isInBoat() || inWater(_xDestination, _yDestination, _zDestination))
				return new Location(_xDestination, _yDestination, _zDestination);

			if(!isInRangeZ(_xDestination, _yDestination, _zDestination, 2000))
				return isFollow ? null : GeoData.getInstance().moveCheck(_x, _y, _z, _xDestination, _yDestination, _zDestination);

			if(_targetRecorder.size() > 0)
				return _targetRecorder.remove(0);

			if(GeoData.getInstance().canMoveToCoord(_x, _y, _z, _xDestination, _yDestination, _zDestination))
				return new Location(_xDestination, _yDestination, _zDestination);
			else if(_offset > 200)
				return null;
			else if(_pathfinding)
			{
				_targetRecorder = GeoMove.checkMovement(this, new CharPosition(_xDestination, _yDestination, _zDestination, 0));
				if(_targetRecorder.size() == 0)
					return isFollow ? null : GeoData.getInstance().moveCheck(_x, _y, _z, _xDestination, _yDestination, _zDestination);

				if(Config.PATH_CLEAN && _targetRecorder.size() > 2)
					pathClean();

				if(_targetRecorder.size() > 1)
				{
					if(isPlayer && getAccessLevel() >= 100 && Config.GEODATA_DEBUG)
						spawnWayPoints(_targetRecorder);

					// Удаляем первую точку (начальную позицию чара)
					_targetRecorder.remove(0);

					if(_targetRecorder.isEmpty())
						return null;
					return _targetRecorder.remove(0);
				}

				return GeoData.getInstance().moveCheck(_x, _y, _z, _xDestination, _yDestination, _zDestination);
			}
			else
				return isFollow ? null : GeoData.getInstance().moveCheck(_x, _y, _z, _xDestination, _yDestination, _zDestination);
		}
	}

	public void followToCharacter(L2Character target, int offset, boolean forestalling)
	{
		if(isFollow && target == followTarget && offset == _offset)
			return;

		if(!hasAI() || isMovementDisabled() || target == null)
		{
			sendActionFailed();
			return;
		}

		getAI().clearNextAction();

		isMoving = false;
		isFollow = false;

		if(isFlying() || isInBoat())
		{
			sendActionFailed();
			return;
		}

		if(offset < 10)
			offset = 10;

		followTarget = target;

		_xDestination = followTarget.getX();
		_yDestination = followTarget.getY();
		_zDestination = followTarget.getZ();

		isFollow = true;

		_pathfinding = true;
		_forestalling = forestalling;

		_targetRecorder.clear();
		Location nextPoint = getNextPointToMove();

		if(nextPoint == null)
		{
			breakMove();
			return;
		}

		if(_targetRecorder.size() == 0)
			nextPoint = applyOffset(nextPoint, offset);

		_xFrom = _x;
		_yFrom = _y;
		_zFrom = _z;

		_xTo = nextPoint.x;
		_yTo = nextPoint.y;
		_zTo = nextPoint.z;

		setHeading(calcHeading(_xTo, _yTo));

		broadcastPacket(new CharMoveToLocation(this));

		_offset = offset;

		_startMoveTick = _followTimestamp = GameTimeController.getGameTicks() - 1;

		previous_speed = getMoveSpeed();

		callMinions();
	}

	public boolean moveToLocation(int x_dest, int y_dest, int z_dest, int offset, boolean pathfinding)
	{
		if(isMoving && _xDestination == x_dest && _yDestination == y_dest && _zDestination == z_dest && _offset == offset && _pathfinding == pathfinding)
			return true;

		if(!hasAI())
			return false;

		getAI().clearNextAction();

		if(isMovementDisabled())
		{
			getAI().setNextAction(nextAction.MOVE, new Location(x_dest, y_dest, z_dest), offset, pathfinding, false);
			sendActionFailed();
			return true;
		}

		if(isPlayer)
			getAI().setIntention(AI_INTENTION_ACTIVE);

		isMoving = false;
		isFollow = false;

		if(isFlying() || isInBoat())
			pathfinding = false;

		if(offset < 0)
			offset = 0;

		_xDestination = x_dest;
		_yDestination = y_dest;
		_zDestination = z_dest;

		_pathfinding = pathfinding;

		_targetRecorder.clear();
		Location nextPoint = getNextPointToMove();

		if(nextPoint == null)
		{
			breakMove();
			return false;
		}

		if(_targetRecorder.size() == 0)
			nextPoint = applyOffset(nextPoint, offset);

		_xFrom = _x;
		_yFrom = _y;
		_zFrom = _z;

		_xTo = nextPoint.x;
		_yTo = nextPoint.y;
		_zTo = nextPoint.z;

		setHeading(calcHeading(_xTo, _yTo));

		broadcastPacket(new CharMoveToLocation(this));

		_offset = offset;

		_startMoveTick = _followTimestamp = GameTimeController.getGameTicks() - 1;

		isMoving = true;

		previous_speed = getMoveSpeed();

		callMinions();

		return true;
	}

	public void updatePosition(int currentGameTick)
	{
		double speed = getMoveSpeed();

		if(previous_speed != speed)
		{
			previous_speed = speed;

			_startMoveTick = currentGameTick;

			_xFrom = _x;
			_yFrom = _y;
			_zFrom = _z;
		}

		double dx = _xTo - _xFrom;
		double dy = _yTo - _yFrom;
		double dz = _zTo - _zFrom;

		double distance = Math.sqrt(dx * dx + dy * dy + dz * dz);

		double ticksToMove = speed > 0 ? GameTimeController.TICKS_PER_SECOND * distance / speed : 0.0;
		double ticksFromStart = currentGameTick - _startMoveTick;
		double elapsed = ticksToMove > 0 ? ticksFromStart / ticksToMove : 1.0;

		if(ticksToMove <= ticksFromStart || (1.0 - elapsed) * distance < 3)
		{
			setXYZ(_xTo, _yTo, _zTo);

			if(_targetRecorder.size() == 0)
			{
				// Мы достигли последней точки в списке, а значит и цели
				isMoving = false;

				if(isFollow)
				{
					isFollow = false;
					ThreadPoolManager.getInstance().scheduleGeneral(new NotifyAITask(CtrlEvent.EVT_READY_TO_ACT), 1);
				}
				else
					ThreadPoolManager.getInstance().scheduleGeneral(new NotifyAITask(CtrlEvent.EVT_ARRIVED), 1);

				return;
			}

			_startMoveTick = currentGameTick;

			Location nextPoint = getNextPointToMove();

			if(nextPoint == null)
			{
				breakMove();
				return;
			}

			if(_targetRecorder.size() == 0)
				nextPoint = applyOffset(nextPoint, _offset);

			_xFrom = _x;
			_yFrom = _y;
			_zFrom = _z;

			_xTo = nextPoint.x;
			_yTo = nextPoint.y;
			_zTo = nextPoint.z;

			setHeading(calcHeading(_xTo, _yTo));

			broadcastPacket(new CharMoveToLocation(this));
			return;
		}

		int new_x = (int) (_xFrom + elapsed * (_xTo - _xFrom) + 0.5);
		int new_y = (int) (_yFrom + elapsed * (_yTo - _yFrom) + 0.5);
		int new_z = (int) (_zFrom + elapsed * (_zTo - _zFrom) + 0.5);

		setXYZ(new_x, new_y, new_z);

		if(isFollow)
		{
			if(followTarget == null || isFlying() || isInBoat())
			{
				breakMove();
				return;
			}

			if(isInRange(followTarget, _offset + 20))
			{
				_targetRecorder.clear();
				isMoving = false;
				isFollow = false;
				ThreadPoolManager.getInstance().scheduleGeneral(new NotifyAITask(CtrlEvent.EVT_READY_TO_ACT), 1);
				return;
			}

			if((currentGameTick - _followTimestamp) * GameTimeController.MILLIS_IN_TICK > 500 && !followTarget.isInRange(_xDestination, _yDestination, 100))
			{
				_followTimestamp = currentGameTick;
				_startMoveTick = currentGameTick;

				_xDestination = followTarget.getX();
				_yDestination = followTarget.getY();
				_zDestination = followTarget.getZ();

				_targetRecorder.clear();
				Location nextPoint = getNextPointToMove();

				if(nextPoint == null)
				{
					breakMove();
					return;
				}

				if(_targetRecorder.size() == 0)
					nextPoint = applyOffset(nextPoint, _offset);

				_xFrom = _x;
				_yFrom = _y;
				_zFrom = _z;

				_xTo = nextPoint.x;
				_yTo = nextPoint.y;
				_zTo = nextPoint.z;

				setHeading(calcHeading(_xTo, _yTo));

				broadcastPacket(new CharMoveToLocation(this));
			}
		}
	}

	private boolean inWater(int x, int y, int z)
	{
		FastList<L2Territory> current_territories = L2World.getActiveTerritory(x, y);
		for(FastList.Node<L2Territory> n = current_territories.head(), end = current_territories.tail(); (n = n.getNext()) != end;)
		{
			L2Territory terr = n.getValue();
			if(terr.getZone() != null && terr.getZone().getType() == ZoneType.water && terr.isInside(x, y, z))
				return true;
		}
		return false;
	}

	private void breakMove()
	{
		isMoving = false;
		isFollow = false;

		if(this instanceof L2Summon)
			setFollowStatus(false);
		else
		{
			sendActionFailed();
			sendPacket(Msg.CANNOT_SEE_TARGET);
			getAI().setIntention(AI_INTENTION_ACTIVE);
		}

		broadcastPacket(new ValidateLocation(this));
		broadcastPacket(new StopMove(this));
	}

	public void stopMove(CharPosition pos)
	{
		isMoving = false;
		isFollow = false;

		if(pos != null)
		{
			setXYZ(pos.x, pos.y, pos.z);
			setHeading(pos.h);
		}

		broadcastPacket(new StopMove(this));
	}

	public void moveToInABoat(CharPosition destination, CharPosition origin)
	{
		if(isActionsDisabled() || isSitting())
		{
			sendActionFailed();
			return;
		}

		if(!isMovementDisabled())
			broadcastPacket(new MoveToLocationInVehicle(this, destination, origin));
		else
			sendActionFailed();
	}

	/**
	 * Returns true if status update should be done, false if not
	 * 
	 * @return boolean
	 */
	protected boolean needStatusUpdate()
	{
		if(Config.FORCE_STATUSUPDATE)
			return true;

		if(!isNpc)
			return true;

		double _intervalHpUpdate = getMaxHp() / 352;

		if(_lastHpUpdate == -99999999)
		{
			_lastHpUpdate = -9999999;
			return true;
		}

		if(getCurrentHp() <= 0 || getMaxHp() < 352)
			return true;

		if(_lastHpUpdate + _intervalHpUpdate < getCurrentHp() && getCurrentHp() > _lastHpUpdate)
		{
			_lastHpUpdate = getCurrentHp();
			return true;
		}

		if(_lastHpUpdate - _intervalHpUpdate > getCurrentHp() && getCurrentHp() < _lastHpUpdate)
		{
			_lastHpUpdate = getCurrentHp();
			return true;
		}
		return false;
	}

	/**
	 * Remove the L2Character from the world when the decay task is launched.<BR>
	 * <BR>
	 */
	public void onDecay()
	{
		decayMe();
	}

	/**
	 * Manage Forced attack (ctrl + select target).<BR>
	 * <BR>
	 * 
	 * <B><U> Actions</U> :</B><BR>
	 * <BR>
	 * <li>If L2Character or target is in a town area, send a system message
	 * TARGET_IN_PEACEZONE a Server->Client packet ActionFailed </li>
	 * <li>If target is confused, send a Server->Client packet ActionFailed
	 * </li>
	 * <li>If L2Character is a L2ArtefactInstance, send a Server->Client packet
	 * ActionFailed </li>
	 * <li>Send a Server->Client packet MyTargetSelected to start attack and
	 * Notify AI with AI_INTENTION_ATTACK </li>
	 * <BR>
	 * <BR>
	 * 
	 * @param player
	 *            The L2Player to attack
	 * 
	 */
	@Override
	public void onForcedAttack(L2Player player)
	{
		if(player.isConfused() || player.isBlocked())
		{
			player.sendActionFailed();
			return;
		}

		player.sendPacket(new MyTargetSelected(getObjectId(), player.getLevel() - getLevel()));
		player.getAI().Attack(this, true);
	}

	/**
	 * Manage hit process (called by Hit Task).<BR>
	 * <BR>
	 * 
	 * <B><U> Actions</U> :</B><BR>
	 * <BR>
	 * <li>If the attacker/target is dead or use fake death, notify the AI with
	 * EVT_CANCEL and send a Server->Client packet ActionFailed (if attacker is
	 * a L2Player)</li>
	 * <li>If attack isn't aborted, send a message system (critical hit,
	 * missed...) to attacker/target if they are L2Player </li>
	 * <li>If attack isn't aborted and hit isn't missed, reduce HP of the
	 * target and calculate reflection damage to reduce HP of attacker if
	 * necessary </li>
	 * <li>if attack isn't aborted and hit isn't missed, manage attack or cast
	 * break of the target (calculating rate, sending message...) </li>
	 * <BR>
	 * <BR>
	 * 
	 * @param target
	 *            The L2Character targeted
	 * @param damage
	 *            Nb of HP to reduce
	 * @param crit
	 *            True if hit is critical
	 * @param miss
	 *            True if hit is missed
	 * @param soulshot
	 *            True if SoulShot are charged
	 * @param shld
	 *            True if shield is efficient
	 * 
	 */
	protected void onHitTimer(L2Character target, int damage, boolean crit, boolean miss, boolean soulshot, boolean shld, boolean unchargeSS)
	{
		if(isAlikeDead())
		{
			getAI().notifyEvent(CtrlEvent.EVT_CANCEL, null, null);
			return;
		}

		if(target.isDead() || !isInRange(target, 1500))
		{
			getAI().notifyEvent(CtrlEvent.EVT_CANCEL, null, null);
			sendActionFailed();
			return;
		}

		// If attack isn't aborted, send a message system (critical hit,
		// missed...) to attacker/target if they are L2Player
		if(!isAttackAborted())
		{
			target.getAI().notifyEvent(CtrlEvent.EVT_ATTACKED, this, false);

			if(target.isStunned() && Formulas.calcStunBreak(crit))
				target.stopEffects(EffectType.STUN);

			if(isPlayer)
			{
				if(crit)
					sendPacket(Msg.CRITICAL_HIT);

				if(miss)
					sendPacket(Msg.MISSED_TARGET);
				else
					sendPacket(new SystemMessage(SystemMessage.YOU_HIT_FOR_S1_DAMAGE).addNumber(damage));
			}
			else if(isSummon)
				((L2SummonInstance) this).displayHitMessage(damage, crit, miss);

			else if(isPet)
				((L2PetInstance) this).displayHitMessage(damage, crit, miss);

			if(target.isPlayer)
			{
				L2Player enemy = (L2Player) target;

				if(shld && damage > 1)
					enemy.sendPacket(Msg.SHIELD_DEFENSE_HAS_SUCCEEDED);
				else if(shld && damage == 1)
					enemy.sendPacket(Msg.YOUR_EXCELLENT_SHIELD_DEFENSE_WAS_A_SUCCESS);
			}

			// Reduce HP of the target and calculate reflection damage to reduce
			// HP of attacker if necessary
			if(!miss && damage > 0)
			{
				if(target.getForceBuff() != null)
					target.abortCast();

				target.reduceCurrentHp(damage, this);

				// Скиллы, кастуемые при физ атаке
				if(!target.isDead())
					for(L2Skill skill : _skillsOnAttack.values())
						if(Rnd.chance(skill.getChance()))
						{
							FastList<L2Character> targets = new FastList<L2Character>();
							targets.add(target);
							broadcastPacket(new MagicSkillLaunched(_objectId, skill.getId(), skill.getLevel(), targets));
							skill.useSkill(this, targets);
						}

				// Проверка на мираж
				if(getTarget() != null && isPlayer)
					if(Rnd.chance((int) target.calcStat(Stats.CANCEL_TARGET, 0, null, null)))
						setTarget(null);

				if(soulshot && unchargeSS)
					unChargeShots(false);

				// Manage attack or cast break of the target (calculating rate,
				// sending message...)
				if(Formulas.calcCastBreak(target, crit))
					target.breakCast(false);
			}
		}

		startAttackStanceTask();

		if(checkPvP(target, null))
			startPvPFlag(target);
	}

	public void onMagicUseTimer(L2Character aimingTarget, L2Skill skill, boolean forceUse)
	{
		_castEndTime = 0;
		_castInterruptTime = 0;
		_castingSkill = null;
		_skillTask = null;

		if(_forceBuff != null)
		{
			_forceBuff.delete();
			return;
		}

		if(skill.getCastRange() < Integer.MAX_VALUE && !GeoData.getInstance().canSeeTarget(this, aimingTarget))
		{
			sendPacket(Msg.CANNOT_SEE_TARGET);
			broadcastPacket(new MagicSkillCanceld(_objectId));
			return;
		}

		Short magicId = skill.getDisplayId();

		if(skill.getId().equals(L2Skill.EVADE_SHOT))
			stopEffect(L2Skill.EVADE_SHOT);

		if(!skill.checkCondition(this, aimingTarget, forceUse, false, false))
			return;

		int level = getSkillDisplayLevel(skill.getId());
		if(level < 1)
			level = 1;

		FastList<L2Character> targets = skill.getTargets(this, aimingTarget, forceUse);

		broadcastPacket(new MagicSkillLaunched(_objectId, magicId, level, targets));

		double mpConsume2 = skill.getMpConsume2();
		int hpConsume = skill.getHpConsume();

		if(hpConsume > 0)
			synchronized (this)
			{
				reduceCurrentHp(hpConsume >= _currentHp ? _currentHp - 1 : hpConsume, this);
			}

		SkillType skillType = skill.getSkillType();
		// _log.warning("OnMUT use skill : "+skillType+ " "+ skill.getName()+ "
		// current MP "+ _currentMp +" trb MP "+mpConsume2);
		if(mpConsume2 > 0)
			if(skillType == SkillType.DANCE || skillType == SkillType.SONG)
			{
				int add = -30;
				for(L2Effect e : getAllEffects())
					if((e.getSkill().getSkillType() == SkillType.DANCE || e.getSkill().getSkillType() == SkillType.SONG) && e.getTime() < e.getPeriod() - 20000)
						add += 30;
				mpConsume2 += add;
				mpConsume2 = calcStat(Stats.MP_DANCE_SKILL_CONSUME, mpConsume2, null, skill);
			}

		if(_currentMp < mpConsume2)
		{
			sendPacket(Msg.NOT_ENOUGH_MP);
			return;
		}

		if(mpConsume2 > 0)
			if(skill.isMagic())
				reduceCurrentMp(calcStat(Stats.MP_MAGIC_SKILL_CONSUME, mpConsume2, null, skill));
			else
				reduceCurrentMp(calcStat(Stats.MP_PHYSICAL_SKILL_CONSUME, mpConsume2, null, skill));

		callSkill(skill, targets);

		getAI().notifyEvent(CtrlEvent.EVT_FINISH_CASTING, null, null);
	}

	/**
	 * Reduce the current HP of the L2Character.<BR>
	 * <BR>
	 * 
	 * <B><U> Overriden in </U> :</B><BR>
	 * <BR>
	 * <li> L2NpcInstance : No change</li>
	 * <li> L2Player : Send a System Message S1_GAVE_YOU_S2_DMG to the L2Player
	 * and PvP Flag the attacker (if necessary)</li>
	 * <BR>
	 * <BR>
	 * 
	 * @param i
	 *            The HP decrease value
	 * @param attacker
	 *            The L2Character who attacks
	 * 
	 */
	public void reduceCurrentHp(double i, L2Character attacker)
	{
		reduceCurrentHp(i, attacker, true, true, false);
	}

	/**
	 * Reduce the current HP of the L2Character and launch the doDie Task if
	 * necessary.<BR>
	 * <BR>
	 * 
	 * <B><U> Overriden in </U> :</B><BR>
	 * <BR>
	 * <li> L2NpcInstance : Update the attacker AggroInfo of the L2NpcInstance
	 * _aggroList</li>
	 * <BR>
	 * <BR>
	 * 
	 * @param i
	 *            The HP decrease value
	 * @param attacker
	 *            The L2Character who attacks
	 * @param awake
	 *            The awake state (If True : stop sleeping)
	 * @param standUp
	 *            If True : stand up
	 * 
	 */
	public void reduceCurrentHp(double i, L2Character attacker, boolean awake, boolean standUp)
	{
		reduceCurrentHp(i, attacker, awake, standUp, false);
	}

	/**
	 * Reduce the current HP of the L2Character and launch the doDie Task if
	 * necessary.<BR>
	 * <BR>
	 * 
	 * <B><U> Overriden in </U> :</B><BR>
	 * <BR>
	 * <li> L2NpcInstance : Update the attacker AggroInfo of the L2NpcInstance
	 * _aggroList</li>
	 * <BR>
	 * <BR>
	 * 
	 * @param i
	 *            The HP decrease value
	 * @param attacker
	 *            The L2Character who attacks
	 * @param awake
	 *            The awake state (If True : stop sleeping)
	 * @param standUp
	 *            If True : stand up
	 * @param directHp
	 *            If True : hit on HP instead of CP first
	 * 
	 */
	public void reduceCurrentHp(double i, L2Character attacker, boolean awake, boolean standUp, boolean directHp)
	{
		fireMethodInvoked(MethodCollection.ReduceCurrentHp, new Object[] {
				i, attacker, awake, standUp, directHp, });

		if(isDead() || isInvul())
			return;

		if(awake)
			if(isSleeping())
				stopEffects(L2Effect.EffectType.SLEEP);

		if(isMeditated())
			stopEffects(L2Effect.EffectType.MEDITATION);

		if(standUp && isPlayer)
		{
			if(isSitting())
				standUp();

			if(isFakeDeath() && getEffect(L2Effect.EffectType.FAKE_DEATH).getTime() > 2000)
				stopEffects(L2Effect.EffectType.FAKE_DEATH);
		}

		if(attacker != this)
			startAttackStanceTask();

		if(attacker != this && attacker instanceof L2Playable)
		{
			// Check if the L2Player isn't dead
			// Flag the attacker if it's a L2Player outside a PvP area
			if(!isDead() && attacker.checkPvP(this, null))
				attacker.startPvPFlag(this);

			if(isOverhit())
				setOverhitValues(attacker, i);
			else
				setOverhitEnabled(false);

			double ii;
			if(!directHp)
			{
				i = _currentCp - i;
				ii = i;

				if(ii < 0)
					ii *= -1;

				if(i < 0)
					i = 0;

				setCurrentCp(i);
			}
			else
				ii = i;

			if(_currentCp == 0 || directHp)
			{
				ii = _currentHp - ii;

				if(ii < 0)
					ii = 0;

				setCurrentHp(ii);
			}
		}
		else
		{
			i = _currentHp - i;

			if(i < 0)
				i = 0;

			setCurrentHp(i);
		}

		if(isDead())
		{
			// killing is only possible one time
			synchronized (this)
			{
				if(_killedAlready)
					return;

				_killedAlready = true;
			}

			stopMove(null);
			doDie(attacker);
			_currentHp = 0;
		}
		else
			setOverhitEnabled(false);
	}

	/**
	 * Reduce the current MP of the L2Character.<BR>
	 * <BR>
	 * 
	 * @param i
	 *            The MP decrease value
	 * 
	 */
	public void reduceCurrentMp(double i)
	{
		i = _currentMp - i;

		if(i < 0)
			i = 0;

		setCurrentMp(i);
	}

	public double relativeSpeed(L2Object target)
	{
		return getMoveSpeed() - target.getMoveSpeed() * Math.cos(headingToRadians(getHeading()) - headingToRadians(target.getHeading()));
	}

	public void removeAllSkills()
	{
		for(L2Skill s : getAllSkillsArray())
			removeSkill(s);
	}

	public void removeBlockStats(FastList<Stats> stats)
	{
		if(_blockedStats != null)
		{
			_blockedStats.removeAll(stats);
			if(_blockedStats.isEmpty())
				_blockedStats = null;
		}
	}

	/**
	 * Вызывающий метод синхронизирован, дополнительная синхронизация не нужна.
	 * @see L2jFT.game.model.L2Effect#stopEffectTask()
	 * @param effect еффект для удаления
	 */
	public void removeEffect(L2Effect effect)
	{
		if(effect == null || _effects == null)
			return;
		synchronized (_effects)
		{
			short effectSkillId = effect.getSkill().getId();
			String stackType = effect.getStackType();

			if(!effect.getStackType().equals("none"))
				if(_stackedEffects != null && _stackedEffects.get(stackType) != null)
				{
					if(_stackedEffects.get(stackType).equals(effectSkillId))
						_stackedEffects.remove(stackType);
					if(_stackedEffects.isEmpty())
						_stackedEffects = null;
				}

			removeStatsOwner(effect);
			effect.setInUse(false);

			if(_effects != null)
			{
				// Удаляем только нужный эффект
				ConcurrentSkipListSet<L2Effect> oldEffects = _effects.get(effectSkillId);
				if(oldEffects != null)
					oldEffects.remove(effect);

				if(oldEffects == null || oldEffects.isEmpty())
					_effects.remove(effectSkillId);

				if((effect.getSkill().isPvpSkill() || effect.getSkill().isToggle()) && _debuffs > 0)
					_debuffs--;
				else if(_buffs > 0)
					_buffs--;

				if(_effects.isEmpty())
					_effects = null;
			}

			if(effect.getStackType().equalsIgnoreCase("HpRecoverCast"))
				sendPacket(new ShortBuffStatusUpdate());
			else
				updateEffectIcons();
		}
	}

	/**
	 * Удаляет все эффекты для данного скилла
	 */
	public synchronized void removeEffectsBySkill(L2Skill skill)
	{
		if(skill == null || _effects == null)
			return;
		Short effectSkillId = skill.getId();

		ConcurrentSkipListSet<L2Effect> effects = _effects.get(effectSkillId);
		if(effects == null)
			return;

		for(L2Effect effect : effects)
		{
			if(effect == null)
				continue;

			String stackType = effect.getStackType();

			if(!stackType.equals("none"))
				if(_stackedEffects != null && _stackedEffects.get(stackType) != null)
				{
					if(_stackedEffects.get(stackType).equals(effectSkillId))
						_stackedEffects.remove(stackType);
					if(_stackedEffects.isEmpty())
						_stackedEffects = null;
				}

			removeStatsOwner(effect);
			effect.setInUse(false);

			if((skill.isPvpSkill() || skill.isToggle()) && _debuffs > 0)
				_debuffs--;
			else if(_buffs > 0)
				_buffs--;

			if(stackType.equalsIgnoreCase("HpRecoverCast"))
				sendPacket(new ShortBuffStatusUpdate());
		}

		_effects.remove(effectSkillId);

		if(_effects.isEmpty())
			_effects = null;

		updateEffectIcons();
	}

	/**
	 * Удаляет все эффекты для данного скилла, кроме указанного эффекта.
	 */
	public synchronized void removeEffectsBySkillExclude(L2Skill skill, L2Effect exclude)
	{
		if(skill == null || _effects == null)
			return;
		Short effectSkillId = skill.getId();

		ConcurrentSkipListSet<L2Effect> effects = _effects.get(effectSkillId);
		if(effects == null)
			return;

		for(L2Effect effect : effects)
		{
			if(effect == null || effect.equals(exclude))
				continue;

			String stackType = effect.getStackType();

			if(!stackType.equals("none"))
				if(_stackedEffects != null && _stackedEffects.get(stackType) != null)
				{
					if(_stackedEffects.get(stackType).equals(effectSkillId))
						_stackedEffects.remove(stackType);
					if(_stackedEffects.isEmpty())
						_stackedEffects = null;
				}

			removeStatsOwner(effect);
			effect.setInUse(false);

			if((skill.isPvpSkill() || skill.isToggle()) && _debuffs > 0)
				_debuffs--;
			else if(_buffs > 0)
				_buffs--;

			if(stackType.equalsIgnoreCase("HpRecoverCast"))
				sendPacket(new ShortBuffStatusUpdate());
		}

		if(effects.isEmpty())
			_effects.remove(effectSkillId);

		if(_effects.isEmpty())
			_effects = null;

		updateEffectIcons();
	}

	/**
	 * Remove a skill from the L2Character and its Func objects from calculator
	 * set of the L2Character.<BR>
	 * <BR>
	 * 
	 * <B><U> Concept</U> :</B><BR>
	 * <BR>
	 * All skills own by a L2Character are identified in <B>_skills</B><BR>
	 * <BR>
	 * 
	 * <B><U> Actions</U> :</B><BR>
	 * <BR>
	 * <li>Remove the skill from the L2Character _skills </li>
	 * <li>Remove all its Func objects from the L2Character calculator set</li>
	 * <BR>
	 * <BR>
	 * 
	 * <B><U> Overriden in </U> :</B><BR>
	 * <BR>
	 * <li> L2Player : Save update in the character_skills table of the database</li>
	 * <BR>
	 * <BR>
	 * 
	 * @param skill
	 *            The L2Skill to remove from the L2Character
	 * 
	 * @return The L2Skill removed
	 * 
	 */
	public L2Skill removeSkill(L2Skill skill)
	{
		if(skill == null)
			return null;

		return removeSkillById(skill.getId());
	}

	public L2Skill removeSkillById(Short id)
	{
		// Remove the skill from the L2Character _skills
		L2Skill oldSkill = _skills.remove(id);
		_skillsOnAttack.remove(id);

		// Remove all its Func objects from the L2Character calculator set
		if(oldSkill != null)
			removeStatsOwner(oldSkill);

		return oldSkill;
	}

	/**
	 * Remove a Func from the Calculator set of the L2Character.<BR>
	 * <BR>
	 * 
	 * <B><U> Concept</U> :</B><BR>
	 * <BR>
	 * A L2Character owns a table of Calculators called <B>_calculators</B>.
	 * Each Calculator (a calculator per state) own a table of Func object. A
	 * Func object is a mathematic function that permit to calculate the
	 * modifier of a state (ex : REGENERATE_HP_RATE...). To reduce cache memory
	 * use, L2NpcInstances who don't have skills share the same Calculator set
	 * called <B>NPC_STD_CALCULATOR</B>.<BR>
	 * <BR>
	 * 
	 * That's why, if a L2NpcInstance is under a skill/spell effect that modify
	 * one of its state, a copy of the NPC_STD_CALCULATOR must be create in its
	 * _calculators before addind new Func object.<BR>
	 * <BR>
	 * 
	 * <B><U> Actions</U> :</B><BR>
	 * <BR>
	 * <li>Remove the Func object from _calculators</li>
	 * <BR>
	 * <BR>
	 * <li>If L2Character is a L2NpcInstance and _calculators is equal to
	 * NPC_STD_CALCULATOR, free cache memory and just create a link on
	 * NPC_STD_CALCULATOR in _calculators</li>
	 * <BR>
	 * <BR>
	 * 
	 * @param f
	 *            The Func object to remove from the Calculator corresponding to
	 *            the state affected
	 */
	public final synchronized void removeStatFunc(Func f)
	{
		if(f == null)
			return;

		// Select the Calculator of the affected state in the Calculator set
		int stat = f._stat.ordinal();

		if(_calculators[stat] == null)
			return;

		// Remove the Func object from the Calculator
		_calculators[stat].removeFunc(f);

		if(_calculators[stat].size() == 0)
			_calculators[stat] = null;

		// If possible, free the memory and just create a link on
		// NPC_STD_CALCULATOR
		if(this instanceof L2NpcInstance)
		{
			int i = 0;
			for(; i < Stats.NUM_STATS; i++)
				if(!Calculator.equalsCals(_calculators[i], NPC_STD_CALCULATOR[i]))
					break;

			if(i >= Stats.NUM_STATS)
				_calculators = NPC_STD_CALCULATOR;
		}
	}

	/**
	 * Remove a list of Funcs from the Calculator set of the L2Player.<BR>
	 * <BR>
	 * 
	 * <B><U> Concept</U> :</B><BR>
	 * <BR>
	 * A L2Character owns a table of Calculators called <B>_calculators</B>.
	 * Each Calculator (a calculator per state) own a table of Func object. A
	 * Func object is a mathematic function that permit to calculate the
	 * modifier of a state (ex : REGENERATE_HP_RATE...). <BR>
	 * <BR>
	 * 
	 * <FONT COLOR=#FF0000><B> <U>Caution</U> : This method is ONLY for
	 * L2Player</B></FONT><BR>
	 * <BR>
	 * 
	 * <B><U> Example of use </U> :</B><BR>
	 * <BR>
	 * <li> Unequip an item from inventory</li>
	 * <li> Stop an active skill</li>
	 * <BR>
	 * <BR>
	 * 
	 * @param funcs
	 *            The list of Func objects to add to the Calculator
	 *            corresponding to the state affected
	 */
	public final synchronized void removeStatFuncs(Func[] funcs)
	{
		for(Func f : funcs)
			removeStatFunc(f);
	}

	/**
	 * Remove all Func objects with the selected owner from the Calculator set
	 * of the L2Character.<BR>
	 * <BR>
	 * 
	 * <B><U> Concept</U> :</B><BR>
	 * <BR>
	 * A L2Character owns a table of Calculators called <B>_calculators</B>.
	 * Each Calculator (a calculator per state) own a table of Func object. A
	 * Func object is a mathematic function that permit to calculate the
	 * modifier of a state (ex : REGENERATE_HP_RATE...). To reduce cache memory
	 * use, L2NpcInstances who don't have skills share the same Calculator set
	 * called <B>NPC_STD_CALCULATOR</B>.<BR>
	 * <BR>
	 * 
	 * That's why, if a L2NpcInstance is under a skill/spell effect that modify
	 * one of its state, a copy of the NPC_STD_CALCULATOR must be create in its
	 * _calculators before addind new Func object.<BR>
	 * <BR>
	 * 
	 * <B><U> Actions</U> :</B><BR>
	 * <BR>
	 * <li>Remove all Func objects of the selected owner from _calculators</li>
	 * <BR>
	 * <BR>
	 * <li>If L2Character is a L2NpcInstance and _calculators is equal to
	 * NPC_STD_CALCULATOR, free cache memory and just create a link on
	 * NPC_STD_CALCULATOR in _calculators</li>
	 * <BR>
	 * <BR>
	 * 
	 * <B><U> Example of use </U> :</B><BR>
	 * <BR>
	 * <li> Unequip an item from inventory</li>
	 * <li> Stop an active skill</li>
	 * <BR>
	 * <BR>
	 * 
	 * @param owner
	 *            The Object(Skill, Item...) that has created the effect
	 */
	public final void removeStatsOwner(Object owner)
	{
		synchronized (_calculators)
		{
			// Go through the Calculator set
			for(int i = 0; i < _calculators.length; i++)
				if(_calculators[i] != null)
				{
					// Delete all Func objects of the selected owner
					_calculators[i].removeOwner(owner);

					if(_calculators[i].size() == 0)
						_calculators[i] = null;
				}

			// If possible, free the memory and just create a link on
			// NPC_STD_CALCULATOR
			if(this instanceof L2NpcInstance)
			{
				int i = 0;
				for(; i < Stats.NUM_STATS; i++)
					if(!Calculator.equalsCals(_calculators[i], NPC_STD_CALCULATOR[i]))
						break;

				if(i >= Stats.NUM_STATS)
					_calculators = NPC_STD_CALCULATOR;
			}
		}
	}

	/**
	 * Remove the object from the list of L2Character that must be informed of
	 * HP/MP updates of this L2Character.<BR>
	 * <BR>
	 * 
	 * <B><U> Concept</U> :</B><BR>
	 * <BR>
	 * Each L2Character owns a list called <B>_statusListener</B> that contains
	 * all L2Player to inform of HP/MP updates. Players who must be informed are
	 * players that target this L2Character. When a RegenTask is in progress
	 * sever just need to go through this list to send Server->Client packet
	 * StatusUpdate.<BR>
	 * <BR>
	 * 
	 * <B><U> Example of use </U> :</B><BR>
	 * <BR>
	 * <li> Untarget a PC or NPC</li>
	 * <BR>
	 * <BR>
	 * 
	 * @param object
	 *            L2Character to add to the listener
	 * 
	 */
	public void removeStatusListener(L2Character object)
	{
		synchronized (this)
		{
			if(getStatusListener() == null)
				return;
			getStatusListener().remove(object);
			if(getStatusListener() != null && getStatusListener().isEmpty())
				setStatusListener(null);
		}
	}

	public void sendActionFailed()
	{
		sendPacket(_actionfailed);
	}

	public L2CharacterAI setAI(L2CharacterAI new_ai)
	{
		if(new_ai == null)
			return _ai = null;
		if(_ai != null)
			_ai.stopAITask();
		_ai = new_ai;
		return _ai;
	}

	/**
	 * Set the current CP of the L2Character.<BR>
	 * <BR>
	 * 
	 * <B><U> Actions</U> :</B><BR>
	 * <BR>
	 * <li>Get the Max CP of the L2Character </li>
	 * <li>Set the RegenActive flag to True/False </li>
	 * <li>Launch/Stop the HP/MP/CP Regeneration task with Medium priority</li>
	 * <li>Send the Server->Client packet StatusUpdate with current HP and MP
	 * to all other L2Player to inform</li>
	 * <BR>
	 * <BR>
	 * 
	 */
	public final void setCurrentCp(double newCp)
	{
		// Get the Max CP of the L2Character
		int maxCp = getMaxCp();

		if(newCp < 0)
			newCp = 0;

		synchronized (this)
		{
			if(newCp >= maxCp)
			{
				// Set the RegenActive flag to false
				_currentCp = maxCp;
				_flagsRegenActive &= ~CP_REGEN_FLAG;

				// Stop the HP/MP/CP Regeneration task
				if(_flagsRegenActive == 0)
					stopHpMpRegeneration();
			}
			else
			{
				// Set the RegenActive flag to true
				_currentCp = newCp;
				_flagsRegenActive |= CP_REGEN_FLAG;

				// Start the HP/MP/CP Regeneration task with Medium priority
				startHpMpRegeneration();
			}
		}

		// Send the Server->Client packet StatusUpdate with current HP and MP to
		// all other L2Player to inform
		broadcastStatusUpdate();
	}

	/**
	 * Set the current HP of the L2Character.<BR>
	 * <BR>
	 * 
	 * <B><U> Actions</U> :</B><BR>
	 * <BR>
	 * <li>Get the Max HP of the L2Character </li>
	 * <li>Set the RegenActive flag to True/False </li>
	 * <li>Launch/Stop the HP/MP/CP Regeneration task with Medium priority</li>
	 * <li>Send the Server->Client packet StatusUpdate with current HP and MP
	 * to all other L2Player to inform</li>
	 * <BR>
	 * <BR>
	 * 
	 */
	public final void setCurrentHp(double newHp)
	{
		// Get the Max HP of the L2Character
		int maxHp = getMaxHp();
		double hpStart = _currentHp;

		synchronized (this)
		{
			if(newHp >= maxHp)
			{
				// Set the RegenActive flag to false
				_currentHp = maxHp;
				_flagsRegenActive &= ~HP_REGEN_FLAG;

				// Stop the HP/MP/CP Regeneration task
				if(_flagsRegenActive == 0)
					stopHpMpRegeneration();
			}
			else
			{
				// Set the RegenActive flag to true
				_currentHp = newHp;
				_flagsRegenActive |= HP_REGEN_FLAG;

				// Start the HP/MP/CP Regeneration task with Medium priority
				startHpMpRegeneration();
			}
		}

		checkHpMessages(hpStart, newHp);

		firePropertyChanged(PropertyCollection.HitPoints, hpStart, _currentHp);

		// Send the Server->Client packet StatusUpdate with current HP and MP to
		// all other L2Player to inform
		broadcastStatusUpdate();
	}

	/**
	 * Set the current HP and MP of the L2Character, Launch/Stop a HP/MP/CP
	 * Regeneration Task and send StatusUpdate packet to all other L2Player to
	 * inform (exclusive broadcast).<BR>
	 * <BR>
	 * 
	 * <B><U> Actions</U> :</B><BR>
	 * <BR>
	 * <li>Get the Max HP and MP of the L2Character </li>
	 * <li>Set the RegenActive flag to True/False </li>
	 * <li>Launch/Stop the HP/MP/CP Regeneration task with Medium priority</li>
	 * <li>Send the Server->Client packet StatusUpdate with current HP and MP
	 * to all other L2Player to inform</li>
	 * <BR>
	 * <BR>
	 * 
	 * <FONT COLOR=#FF0000><B> <U>Caution</U> : This method DOESN'T SEND
	 * Server->Client StatusUpdate packet to the L2Character</B></FONT><BR>
	 * <BR>
	 * 
	 * @param newHp
	 *            The new HP value of the L2Player
	 * @param newMp
	 *            The new MP value of the L2Player
	 * 
	 */
	public final void setCurrentHpMp(double newHp, double newMp)
	{
		// Get the Max HP and MP of the L2Character
		int maxHp = getMaxHp();
		int maxMp = getMaxMp();
		double curHp = _currentHp;

		synchronized (this)
		{
			if(newHp >= maxHp)
			{
				// Set the RegenActive flag to false
				_killedAlready = false;
				_currentHp = maxHp;
				_flagsRegenActive &= ~HP_REGEN_FLAG;

			}
			else
			{
				_currentHp = newHp;

				if(!isDead())
				{
					// Set the RegenActive flag to true
					_killedAlready = false;
					_flagsRegenActive |= HP_REGEN_FLAG;

					// Start the HP/MP/CP Regeneration task with Medium priority
					startHpMpRegeneration();
				}
			}

			if(newMp >= maxMp)
			{
				// Set the RegenActive flag to false
				_currentMp = maxMp;
				_flagsRegenActive &= ~MP_REGEN_FLAG;
			}
			else
			{
				_currentMp = newMp;

				if(!isDead())
				{
					// Set the RegenActive flag to true
					_flagsRegenActive |= MP_REGEN_FLAG;

					// Start the HP/MP/CP Regeneration task with Medium priority
					startHpMpRegeneration();
				}
			}

			// Stop the HP/MP/CP Regeneration task
			if(_flagsRegenActive == 0)
				stopHpMpRegeneration();
		}

		checkHpMessages(curHp, newHp);

		// Send the Server->Client packet StatusUpdate with current HP and MP to
		// all other L2Player to inform
		broadcastStatusUpdate();
	}

	public final void setCurrentMp(double newMp)
	{
		// Get the Max MP of the L2Character
		int maxMp = getMaxMp();

		synchronized (this)
		{
			if(newMp >= maxMp)
			{
				// Set the RegenActive flag to false
				_currentMp = maxMp;
				_flagsRegenActive &= ~MP_REGEN_FLAG;

				// Stop the HP/MP/CP Regeneration task
				if(_flagsRegenActive == 0)
					stopHpMpRegeneration();
			}
			else
			{
				// Set the RegenActive flag to true
				_currentMp = newMp;
				_flagsRegenActive |= MP_REGEN_FLAG;

				// Start the HP/MP/CP Regeneration task with Medium priority
				startHpMpRegeneration();
			}
		}

		// Send the Server->Client packet StatusUpdate with current HP and MP to
		// all other L2Player to inform
		broadcastStatusUpdate();
	}

	/**
	 * Set the current MP of the L2Character.<BR>
	 * <BR>
	 * 
	 * <B><U> Actions</U> :</B><BR>
	 * <BR>
	 * <li>Get the Max HP of the L2Character </li>
	 * <li>Set the RegenActive flag to True/False </li>
	 * <li>Launch/Stop the HP/MP/CP Regeneration task with Medium priority</li>
	 * <li>Send the Server->Client packet StatusUpdate with current HP and MP
	 * to all other L2Player to inform</li>
	 * <BR>
	 * <BR>
	 * 
	 */
	public final void setCurrentMp(Double newMp)
	{
		setCurrentMp((double) newMp);
	}

	/**
	 * Set the L2Character flying mode to True.<BR>
	 * <BR>
	 */
	public final void setFlying(boolean mode)
	{
		_flying = mode;
	}

	/**
	 * Set the orientation of the L2Character.<BR>
	 * <BR>
	 */
	public final void setHeading(int heading)
	{
		_heading = heading;
	}

	/**
	 * Set the orientation of the L2Character to the target.<BR>
	 * <BR>
	 */
	public final void setHeading(L2Character target, boolean toChar)
	{
		if(target == null || target == this)
			return;

		_heading = (int) (Math.atan2(_y - target.getY(), _x - target.getX()) * 32768. / Math.PI) + (toChar ? 32768 : 0);
		if(_heading < 0)
			_heading += 65536;
	}

	public final void setIsBlessedByNoblesse(boolean value)
	{
		if(value)
			_IsBlessedByNoblesse++;
		else
			_IsBlessedByNoblesse--;
	}

	public final void setIsSalvation(boolean value)
	{
		if(value)
			_IsSalvation++;
		else
			_IsSalvation--;
	}

	public final void setBuffImmunity(boolean value)
	{
		if(value)
			_buffImmunity++;
		else
			_buffImmunity--;
	}

	/**
	 * Set the invulnerability Flag of the L2Character.<BR>
	 */
	public void setIsInvul(boolean b)
	{
		_isInvul = b;
	}

	public final void setIsPendingRevive(boolean value)
	{
		_IsPendingRevive = value;
	}

	public final void setIsTeleporting(boolean value)
	{
		_IsTeleporting = value;
	}

	public final void setName(String name)
	{
		_name = name;
	}

	public L2Character getCastingTarget()
	{
		return _castingTarget;
	}

	public void setCastingTarget(L2Character target)
	{
		_castingTarget = target;
	}

	/**
	 * Set the L2Character riding mode to True.<BR>
	 * <BR>
	 */
	public final void setRiding(boolean mode)
	{
		_riding = mode;
	}

	/**
	 * Set the L2Character movement type to run and send Server->Client packet
	 * ChangeMoveType to all others L2Player.<BR>
	 * <BR>
	 */
	public final void setRunning()
	{
		if(!_running)
		{
			_running = true;
			broadcastPacket(new ChangeMoveType(this));
		}
	}

	public void setSkillMastery(short skill, byte mastery)
	{
		_skillMastery.put(skill, mastery);
	}

	private void setStatusListener(CopyOnWriteArraySet<L2Character> value)
	{
		_statusListener = value;
	}

	public void setTarget(L2Object object)
	{
		if(object != null && !object.isVisible())
			object = null;
		// If object==null, Cancel Attak or Cast
		if(object == null)
		{
			if(_target != null)
				broadcastPacket(new TargetUnselected(this));
			if(isAttackingNow() && getAI().getAttackTarget() == _target)
			{
				abortAttack();
				getAI().setIntention(AI_INTENTION_ACTIVE, null, null);
				sendActionFailed();
				sendMessage("Attack is aborted", "Атака отменена");
			}
			if(isCastingNow() && canAbortCast() && getAI().getAttackTarget() == _target)
			{
				abortCast();
				getAI().setIntention(AI_INTENTION_ACTIVE, null, null);
				sendActionFailed();
				sendMessage("Casting is aborted", "Заклинание отменено");
			}
		}
		_target = object;
	}

	/**
	 * Set the template of the L2Character.<BR>
	 * <BR>
	 * 
	 * <B><U> Concept</U> :</B><BR>
	 * <BR>
	 * Each L2Character owns generic and static properties (ex : all Keltir have
	 * the same number of HP...). All of those properties are stored in a
	 * different template for each type of L2Character. Each template is loaded
	 * once in the server cache memory (reduce memory use). When a new instance
	 * of L2Character is spawned, server just create a link between the instance
	 * and the template This link is stored in <B>_template</B><BR>
	 * <BR>
	 * 
	 * <B><U> Assert </U> :</B><BR>
	 * <BR>
	 * <li> isCharacter</li>
	 * <BR>
	 * <BR
	 * 
	 */
	protected final void setTemplate(L2CharTemplate template)
	{
		_template = template;
	}

	/**
	 * Set the Title of the L2Character.<BR>
	 * <BR>
	 * 
	 * @param title
	 *            The text to set as title
	 * 
	 */
	public final void setTitle(String title)
	{
		_title = title;
	}

	/**
	 * Set the L2Character movement type to walk and send Server->Client packet
	 * ChangeMoveType to all others L2Player.<BR>
	 * <BR>
	 */
	public final void setWalking()
	{
		if(_running)
		{
			_running = false;
			broadcastPacket(new ChangeMoveType(this));
		}
	}

	/**
	 * Active abnormal effects flags in the binary mask and send Server->Client
	 * UserInfo/CharInfo packet.<BR>
	 * <BR>
	 */
	public final void startAbnormalEffect(int mask)
	{
		_abnormalEffects |= mask;
		updateAbnormalEffect();
	}

	@Override
	public void startAttackStanceTask()
	{
		if(System.currentTimeMillis() < _stanceInited + 10000)
			return;

		_stanceInited = System.currentTimeMillis();

		// Бесконечной рекурсии не будет, потому что выше проверка на
		// _stanceInited
		if(this instanceof L2Summon && this.getOwner() != null)
			this.getOwner().startAttackStanceTask();
		else if(isPlayer && getPet() != null)
			getPet().startAttackStanceTask();

		if(_stanceTask != null)
			_stanceTask.cancel(false);
		else
			broadcastPacket(new AutoAttackStart(getObjectId()));

		_stanceTask = ThreadPoolManager.getInstance().scheduleAi(new CancelAttackStance(), 15000);
	}

	/**
	 * Start the HP/MP/CP Regeneration task.<BR>
	 * <BR>
	 * 
	 * <B><U> Actions</U> :</B><BR>
	 * <BR>
	 * <li>Calculate the regen task period </li>
	 * <li>Launch the HP/MP/CP Regeneration task with Medium priority </li>
	 * <BR>
	 * <BR>
	 * 
	 */
	protected synchronized void startHpMpRegeneration()
	{
		if(_regTask == null && !isDead())
		{
			if(Config.DEBUG)
				_log.fine("HP/MP/CP regen started");

			// Get the Regeneration periode
			int period = Formulas.getRegeneratePeriod(this);

			// Create the HP/MP/CP Regeneration task
			_regTask = ThreadPoolManager.getInstance().scheduleEffectAtFixedRate(new RegenTask(), period, period);
			if(isPlayer)
				sendPacket(new UserInfo((L2Player) this));
		}
	}

	/**
	 * Modify the abnormal effect map according to the mask.<BR>
	 * <BR>
	 */
	public final void stopAbnormalEffect(int mask)
	{
		_abnormalEffects &= ~mask;
		updateAbnormalEffect();
	}

	public void stopAllCancelableEffects()
	{
		if(_effects == null)
			return;

		for(ConcurrentSkipListSet<L2Effect> effects : _effects.values())
			for(L2Effect e : effects)
				if(e != null && e.getSkill().isCancelable())
					e.exit();
	}

	public void stopAllEffects()
	{
		if(_effects == null)
			return;

		for(ConcurrentSkipListSet<L2Effect> effects : _effects.values())
			for(L2Effect e : effects)
				if(e != null)
					e.exit();
	}

	public final void stopEffect(Short effectId)
	{
		ConcurrentSkipListSet<L2Effect> effects = getEffect(effectId);
		if(effects != null)
			for(L2Effect e : effects)
				if(e != null)
					e.exit();
	}

	public final void stopEffects(L2Effect.EffectType type)
	{
		for(L2Effect e : getAllEffects())
			if(e.getEffectType() == type)
				e.exit();
	}

	/**
	 * Stop the HP/MP/CP Regeneration task.<BR>
	 * <BR>
	 * 
	 * <B><U> Actions</U> :</B><BR>
	 * <BR>
	 * <li>Set the RegenActive flag to False </li>
	 * <li>Stop the HP/MP/CP Regeneration task </li>
	 * <BR>
	 * <BR>
	 * 
	 */
	public synchronized void stopHpMpRegeneration()
	{
		if(_regTask != null)
		{
			// Stop the HP/MP/CP Regeneration task
			_regTask.cancel(false);
			_regTask = null;

			// Set the RegenActive flag to false
			_flagsRegenActive = 0;

			if(Config.DEBUG)
				_log.fine("HP/MP/CP regen stop");
		}
	}

	public void startConfused()
	{
		_confused++;
		startAttackStanceTask();
		updateAbnormalEffect();
	}

	public void block()
	{
		_blocked++;
	}

	public void unblock()
	{
		if(_blocked > 0)
		{
			_blocked--;
			getAI().notifyEvent(CtrlEvent.EVT_READY_TO_ACT, null, null);
		}
	}

	public void stopConfused()
	{
		if(_confused <= 0)
		{
			_log.severe("Double effect stop?");
			//Thread.dumpStack();
			return;
		}
		_confused--;
		updateAbnormalEffect();
		getAI().notifyEvent(CtrlEvent.EVT_READY_TO_ACT, null, null);
	}

	public void startFakeDeath()
	{
		_fake_death = true;
		breakAttack();
		breakCast(true);
		getAI().notifyEvent(CtrlEvent.EVT_FAKE_DEATH, null, null);
		broadcastPacket(new ChangeWaitType(this, ChangeWaitType.WT_START_FAKEDEATH));
		updateAbnormalEffect();
	}

	public void stopFakeDeath()
	{
		_fake_death = false;
		broadcastPacket(new ChangeWaitType(this, ChangeWaitType.WT_STOP_FAKEDEATH));
		updateAbnormalEffect();
		getAI().notifyEvent(CtrlEvent.EVT_READY_TO_ACT, null, null);
	}

	public void startFear()
	{
		_afraid++;
		breakAttack();
		breakCast(true);
		sendActionFailed();
		stopMove(null);
		startAttackStanceTask();
		updateAbnormalEffect();
	}

	public void stopFear()
	{
		if(_afraid <= 0)
		{
			_log.severe("Double effect stop?");
			//Thread.dumpStack();
			return;
		}
		_afraid--;
		updateAbnormalEffect();
		getAI().notifyEvent(CtrlEvent.EVT_READY_TO_ACT, null, null);
	}

	public void startMuted()
	{
		_muted++;
		if(getCastingSkill() != null && getCastingSkill().isMagic())
			breakCast(false);
		startAttackStanceTask();
		updateAbnormalEffect();
	}

	public void stopMuted()
	{
		if(_muted <= 0)
		{
			_log.severe("Double effect stop?");
			//Thread.dumpStack();
			return;
		}
		_muted--;
		updateAbnormalEffect();
	}

	public void startPMuted()
	{
		_pmuted++;
		if(getCastingSkill() != null && !getCastingSkill().isMagic())
			breakCast(false);
		startAttackStanceTask();
		updateAbnormalEffect();
	}

	public void stopPMuted()
	{
		if(_pmuted <= 0)
		{
			_log.severe("Double effect stop?");
			//Thread.dumpStack();
			return;
		}
		_pmuted--;
		updateAbnormalEffect();
	}

	public void startRooted()
	{
		_rooted++;
		getAI().clientStopMoving(null);
		startAttackStanceTask();
		updateAbnormalEffect();
	}

	public void stopRooting()
	{
		if(_rooted <= 0)
		{
			_log.severe("Double effect stop?");
			//Thread.dumpStack();
			return;
		}
		_rooted--;
		updateAbnormalEffect();
		getAI().notifyEvent(CtrlEvent.EVT_READY_TO_ACT, null, null);
	}

	public void startSleeping()
	{
		_sleeping++;
		breakAttack();
		breakCast(true);
		sendActionFailed();
		stopMove(null);
		startAttackStanceTask();
		updateAbnormalEffect();
	}

	public void stopSleeping()
	{
		if(_sleeping <= 0)
		{
			_log.severe("Double effect stop?");
			//Thread.dumpStack();
			return;
		}
		_sleeping--;
		updateAbnormalEffect();
		getAI().notifyEvent(CtrlEvent.EVT_READY_TO_ACT, null, null);
	}

	public void startStunning()
	{
		_stunned++;
		breakAttack();
		breakCast(true);
		sendActionFailed();
		stopMove(null);
		startAttackStanceTask();
		updateAbnormalEffect();
	}

	public void stopStunning()
	{
		if(_stunned <= 0)
		{
			_log.severe("Double effect stop?");
			//Thread.dumpStack();
			return;
		}
		_stunned--;
		updateAbnormalEffect();
		getAI().notifyEvent(CtrlEvent.EVT_READY_TO_ACT, null, null);
	}

	public void startCircle()
	{
		_circle++;
		setImobilised(true);
		updateAbnormalEffect();
	}

	public void stopCircle()
	{
		if(_circle <= 0)
		{
			_log.severe("Double effect stop?");
			//Thread.dumpStack();
			return;
		}
		_circle--;
		setImobilised(false);
		updateAbnormalEffect();
		getAI().notifyEvent(CtrlEvent.EVT_READY_TO_ACT, null, null);
	}

	public void setMeditated(boolean meditated)
	{
		if(meditated)
			_meditated++;
		else if(_meditated > 0)
			_meditated--;
		else
			_log.severe("Double effect stop?");
		//Thread.dumpStack();
	}

	public void setParalyzed(boolean paralyzed)
	{
		if(paralyzed)
			_paralyzed++;
		else if(_paralyzed > 0)
			_paralyzed--;
		else
			_log.severe("Double effect stop?");
		//Thread.dumpStack();
	}

	/**
	 * Set the Imobilised status of the L2Character.<BR>
	 * <BR>
	 */
	public final void setImobilised(boolean imobilised)
	{
		if(imobilised)
			_imobilised++;
		else if(_imobilised > 0)
			_imobilised--;
		else
			_log.severe("Double effect stop?");
		//Thread.dumpStack();
	}

	/**
	 * Set the Big Head status of the L2Character.<BR>
	 * <BR>
	 */
	public final void setBigHead(boolean bigHead)
	{
		if(bigHead)
			_bigHead++;
		else if(_bigHead > 0)
			_bigHead--;
		else
			_log.severe("Double effect stop?");
		//Thread.dumpStack();

		updateAbnormalEffect();
	}

	/**
	 * Set the overloaded status of the L2Character is overloaded (if True, the
	 * L2Player can't take more item).<BR>
	 * <BR>
	 */
	public void setOverloaded(boolean overloaded)
	{
		_overloaded = overloaded;
	}

	public boolean isConfused()
	{
		return _confused > 0;
	}

	public boolean isFakeDeath()
	{
		return _fake_death;
	}

	public boolean isAfraid()
	{
		return _afraid > 0;
	}

	public boolean isBlocked()
	{
		return _blocked > 0;
	}

	public boolean isMuted()
	{
		return _muted > 0;
	}

	public boolean isPMuted()
	{
		return _pmuted > 0;
	}

	public boolean isRooted()
	{
		return _rooted > 0;
	}

	public boolean isSleeping()
	{
		return _sleeping > 0;
	}

	public boolean isStunned()
	{
		return _stunned > 0;
	}

	public boolean isCastingCircle()
	{
		return _circle > 0;
	}

	public final boolean isMeditated()
	{
		return _meditated > 0;
	}

	public final boolean isParalyzed()
	{
		return _paralyzed > 0;
	}

	public final boolean isImobilised()
	{
		return _imobilised > 0;
	}

	/**
	 * Return True if the L2Character is casting.<BR>
	 * <BR>
	 */
	public final boolean isCastingNow()
	{
		return _castEndTime > GameTimeController.getGameTicks();
	}

	/**
	 * Return True if the L2Character can't move (stun, root, sleep, overload,
	 * paralyzed).<BR>
	 * <BR>
	 */
	public boolean isMovementDisabled()
	{
		return isSitting() || isStunned() || isRooted() || isSleeping() || isParalyzed() || isImobilised() || isAlikeDead() || isAttackingNow() || isCastingNow() || _overloaded || _fishing;
	}

	/**
	 * Return True if the L2Character can't use its skills (ex : stun,
	 * sleep...).<BR>
	 * <BR>
	 */
	public boolean isActionsDisabled()
	{
		return isStunned() || isSleeping() || isParalyzed() || isAttackingNow() || isCastingNow() || isInBoat() || isAlikeDead();
	}

	public boolean isPotionsDisabled()
	{
		return isStunned() || isSleeping() || isParalyzed() || isAlikeDead() || isInBoat();
	}

	/**
	 * Return True if the L2Character can't attack (stun, sleep, reuse).<BR>
	 * <BR>
	 */
	public final boolean isAttackingDisabled()
	{
		return _attackReuseEndTime > System.currentTimeMillis();
	}

	/**
	 * Return True if the L2Character can't be controlled by the player
	 * (confused, affraid).<BR>
	 * <BR>
	 */
	public boolean isOutOfControl()
	{
		return isConfused() || isAfraid() || isBlocked();
	}

	public void teleToLocation(Location loc)
	{
		int x = loc.x;
		int y = loc.y;
		int z = loc.z;

		if(DimensionalRiftManager.getInstance().checkIfInRiftZone(_x, _y, _z, true))
		{
			int[] newCoords = DimensionalRiftManager.getInstance().getRoom((byte) 0, (byte) 0).getTeleportCoords();
			x = newCoords[0];
			y = newCoords[1];
			z = newCoords[2];

			if(isPlayer)
			{
				L2Player player = (L2Player) this;
				if(player.isInParty() && player.getParty().isInDimensionalRift())
					player.getParty().getDimensionalRift().usedTeleport(player);
			}
		}

		teleToLocation(x, y, z);
	}

	public void teleToLocation(int x, int y, int z)
	{
		setTarget(null);
		stopMove(null);

		if(Config.DEBUG)
			_log.fine("Teleporting to: " + x + ", " + y + ", " + z);

		if(isPlayer)
		{
			L2Player player = (L2Player) this;
			if(player.isInBoat())
			{
				player.setInBoat(false);
				player.setBoat(null);
				player.setBoatId(0);
			}

			decayMe();

			setXYZInvisible(x, y, z);
			setIsTeleporting(true);

			// Нужно при телепорте с более высокой точки на более низкую, иначе наносится вред от "падения"
			player.setLastClientPosition(null);
			player.setLastServerPosition(null);

			player.sendPacket(new TeleportToLocation(player, x, y, z));
			if(player.getParty() != null)
			{
				Point position = player.getLastPartyPositionSent();
				if(player.getDistance(position.x, position.y) < 500)
				{
					player.getParty().broadcastToPartyMembers(player, new PartyMemberPosition(player));
					position.move(player.getX(), player.getY());
				}
			}
		}
		else
		{
			setXYZ(x, y, z);
			callMinions(); // call minions for change position
			broadcastPacket(new TeleportToLocation(this, x, y, z));
		}
	}

	public void onTeleported()
	{
		if(isPlayer)
		{
			spawnMe();

			L2Player player = (L2Player) this;

			setLastClientPosition(new CharPosition(_x, _y, _z, _heading));
			setLastServerPosition(new CharPosition(_x, _y, _z, _heading));

			setIsTeleporting(false);

			if(_IsPendingRevive)
				doRevive();

			// Modify the position of the pet if necessary
			if(getPet() != null)
			{
				getPet().setXYZ(_x + Rnd.get(-100, 100), _y + Rnd.get(-100, 100), _z);
				getPet().setFollowStatus(true);
				getPet().setInRange(true);
			}

			if(player.getTrainedBeast() != null)
				player.getTrainedBeast().setXYZ(_x + Rnd.get(-100, 100), _y + Rnd.get(-100, 100), _z);

			sendActionFailed();
		}
	}

	public void sendMessage(CustomMessage message)
	{
		sendMessage(message.toString());
	}

	private long _nonAggroTime;

	public long getNonAggroTime()
	{
		return _nonAggroTime;
	}

	public void setNonAggroTime(long time)
	{
		_nonAggroTime = time;
	}

	public int getCustomEffect()
	{
		return _customEffect;
	}

	public void setCustomEffect(int effect)
	{
		_customEffect = effect;
	}

	@Override
	public String toString()
	{
		return "mob " + getObjectId();
	}

	// ---------------------------- Not Implemented -------------------------------

	@SuppressWarnings("unused")
	public void addExpAndSp(long addToExp, long addToSp)
	{}

	@SuppressWarnings("unused")
	public void addSkillTimeStamp(short s, long r)
	{}

	@SuppressWarnings("unused")
	public void broadcastUserInfo()
	{}

	@SuppressWarnings("unused")
	public void checkHpMessages(double currentHp, double newHp)
	{}

	@SuppressWarnings("unused")
	public boolean checkPvP(L2Character target, L2Skill skill)
	{
		return false;
	}

	@SuppressWarnings("unused")
	public void clearAggroList()
	{}

	@SuppressWarnings("unused")
	public boolean consumeItem(int itemConsumeId, int itemCount)
	{
		return true;
	}

	@SuppressWarnings("unused")
	public void doPickupItem(L2Object object)
	{}

	@SuppressWarnings("unused")
	public boolean isFearImmune()
	{
		return false;
	}

	@SuppressWarnings("unused")
	public boolean isLethalImmune()
	{
		return false;
	}

	@SuppressWarnings("unused")
	public boolean getChargedSoulShot()
	{
		return false;
	}

	@SuppressWarnings("unused")
	public int getChargedSpiritShot()
	{
		return 0;
	}

	@SuppressWarnings("unused")
	public Duel getDuel()
	{
		return null;
	}

	@SuppressWarnings("unused")
	public int getIncreasedForce()
	{
		return 0;
	}

	@SuppressWarnings("unused")
	public int getKarma()
	{
		return 0;
	}

	@SuppressWarnings("unused")
	public double getLevelMod()
	{
		return 1;
	}

	@SuppressWarnings("unused")
	public int getNpcId()
	{
		return 0;
	}

	@SuppressWarnings("unused")
	public L2Summon getPet()
	{
		return null;
	}

	@SuppressWarnings("unused")
	public int getPvpFlag()
	{
		return 0;
	}

	@SuppressWarnings("unused")
	public int getTeam()
	{
		return 0;
	}

	@SuppressWarnings("unused")
	public boolean isOverhit()
	{
		return false;
	}

	@SuppressWarnings("unused")
	public boolean isSitting()
	{
		return false;
	}

	@SuppressWarnings("unused")
	public boolean isUndead()
	{
		return false;
	}

	@SuppressWarnings("unused")
	public boolean isUsingDualWeapon()
	{
		return false;
	}

	@SuppressWarnings("unused")
	public void setOverhitEnabled(boolean b)
	{}

	@SuppressWarnings("unused")
	public boolean paralyzeImmune()
	{
		return false;
	}

	@SuppressWarnings("unused")
	public void reduceArrowCount()
	{}

	@SuppressWarnings("unused")
	public void removeSkillTimeStamp(short s)
	{}

	@SuppressWarnings("unused")
	public void sendChanges()
	{}

	@SuppressWarnings("unused")
	public void sendMessage(String message)
	{}

	@SuppressWarnings("unused")
	public void sendMessage(String message, String message_ru)
	{}

	@SuppressWarnings("unused")
	public void sendPacket(L2GameServerPacket mov)
	{}

	@SuppressWarnings("unused")
	public void setIncreasedForce(int i)
	{}

	@SuppressWarnings("unused")
	public void setOverhitValues(L2Character attacker, double i)
	{}

	public void sitDown()
	{}

	@SuppressWarnings("unused")
	public void standUp()
	{}

	@SuppressWarnings("unused")
	public void startPvPFlag(L2Character target)
	{}

	@SuppressWarnings("unused")
	public boolean unChargeShots(boolean spirit)
	{
		return false;
	}

	@SuppressWarnings("unused")
	public void updateEffectIcons()
	{}

	@SuppressWarnings("unused")
	public void updateStats()
	{}

	@SuppressWarnings("unused")
	public void callMinions()
	{}

	@SuppressWarnings("unused")
	public boolean hasMinions()
	{
		return false;
	}

	@SuppressWarnings("unused")
	public boolean isCursedWeaponEquipped()
	{
		return false;
	}

	@SuppressWarnings("unused")
	public boolean isHero()
	{
		return false;
	}

	@SuppressWarnings("unused")
	public L2Player getOwner()
	{
		return null;
	}

	@SuppressWarnings("unused")
	public boolean isInBoat()
	{
		return false;
	}

	@SuppressWarnings("unused")
	public int getAccessLevel()
	{
		return 0;
	}

	@SuppressWarnings("unused")
	public void spawnWayPoints(Vector<Location> recorder)
	{}

	@SuppressWarnings("unused")
	public void setFollowStatus(boolean b)
	{}

	@SuppressWarnings("unused")
	public void setLastClientPosition(CharPosition charPosition)
	{}

	@SuppressWarnings("unused")
	public void setLastServerPosition(CharPosition charPosition)
	{}

	@SuppressWarnings("unused")
	public FastList<L2MinionInstance> getSpawnedMinions()
	{
		return null;
	}

	@SuppressWarnings("unused")
	public boolean hasRandomAnimation()
	{
		return true;
	}

	@SuppressWarnings("unused")
	public boolean hasRandomWalk()
	{
		return true;
	}

	@SuppressWarnings("unused")
	public void disableItem(L2Skill handler, long timeTotal, long timeLeft)
	{}

	// --------------------------- End Of Not Implemented ------------------------------

	// --------------------------------- Abstract --------------------------------------

	public abstract byte getLevel();

	public abstract void updateAbnormalEffect();

	public abstract L2ItemInstance getActiveWeaponInstance();

	public abstract L2Weapon getActiveWeaponItem();

	public abstract L2ItemInstance getSecondaryWeaponInstance();

	public abstract L2Weapon getSecondaryWeaponItem();

	// ----------------------------- End Of Abstract -----------------------------------
}
