package L2jFT.game.model;

import javolution.util.FastList;
import L2jFT.game.model.base.ItemToDrop;

public class L2Drop
{
	public static final int MAX_CHANCE = 1000000;
	private FastList<L2DropGroup> _drop = new FastList<L2DropGroup>();
	private FastList<L2DropGroup> _spoil = new FastList<L2DropGroup>();
	private FastList<L2DropData> _quest = new FastList<L2DropData>();

	public void addData(L2DropData d)
	{
		if(d.isQuestDrop())
			addQuest(d);
		else if(d.isSweep())
			addSpoil(d);
		else
			addDrop(d);
	}

	public void addDrop(L2DropData d)
	{
		short gid = d.getGroupId();
		if(_drop.size() != 0)
			for(L2DropGroup g : _drop)
				if(g.getId() == gid)
				{
					g.addDropItem(d);
					return;
				}
		L2DropGroup temp = new L2DropGroup(d.getGroupId(), d.getGroupChance());
		temp.addDropItem(d);
		_drop.add(temp);
	}

	public void addSpoil(L2DropData s)
	{
		L2DropGroup temp = new L2DropGroup((short) _spoil.size(), s.getChance());
		s.setGroupChance(s.getChance());
		s.setChance(L2Drop.MAX_CHANCE);
		temp.addDropItem(s);
		_spoil.add(temp);
	}

	public void addQuest(L2DropData q)
	{
		_quest.add(q);
	}

	public FastList<ItemToDrop> rollDrop(int diff, boolean isRaidBoss, L2Player player)
	{
		FastList<ItemToDrop> temp = new FastList<ItemToDrop>();
		for(L2DropGroup g : _drop)
		{
			ItemToDrop itd = g.roll(diff, false, isRaidBoss, player);
			if(itd != null)
				temp.add(itd);
		}
		return temp;
	}

	public FastList<ItemToDrop> rollSpoil(int diff, boolean isRaidBoss, L2Player player)
	{
		FastList<ItemToDrop> temp = new FastList<ItemToDrop>();
		for(L2DropGroup g : _spoil)
		{
			ItemToDrop itd = g.roll(diff, true, isRaidBoss, player);
			if(itd != null)
				temp.add(itd);
		}
		return temp;
	}

	public FastList<L2DropData> getAll()
	{
		FastList<L2DropData> temp = new FastList<L2DropData>();
		for(L2DropGroup g : _drop)
			temp.addAll(g.getDropItems(false));
		for(L2DropGroup s : _spoil)
			temp.addAll(s.getDropItems(false));
		return temp;
	}

	public FastList<L2DropGroup> getAllGroups()
	{
		FastList<L2DropGroup> temp = new FastList<L2DropGroup>();
		for(L2DropGroup g : _drop)
			temp.add(g);
		for(L2DropGroup s : _spoil)
			temp.add(s);
		return temp;
	}

	public FastList<L2DropGroup> getSpoil()
	{
		return _spoil;
	}

	public FastList<L2DropGroup> getNormal()
	{
		return _drop;
	}
}