package L2jFT.game.model;

import javolution.util.FastList;
import L2jFT.Config;
import L2jFT.util.Rnd;
import L2jFT.game.model.base.Experience;
import L2jFT.game.model.base.ItemToDrop;

public class L2DropGroup
{
	private short _id;
	private int _chance;
	private boolean _isAdena = false;
	private FastList<L2DropData> _items = new FastList<L2DropData>();

	public L2DropGroup(short id, int chance)
	{
		_id = id;
		_chance = chance;
	}

	public short getId()
	{
		return _id;
	}

	public void addDropItem(L2DropData item)
	{
		if(item.getItemId() == 57 || item.getItemId() == 6360 || item.getItemId() == 6361 || item.getItemId() == 6362)
		{
			_isAdena = true;
			_chance = 700000;
		}
		_items.add(item);
	}

	public FastList<L2DropData> getDropItems(boolean copy)
	{
		if(!copy)
			return _items;
		FastList<L2DropData> temp = new FastList<L2DropData>();
		temp.addAll(_items);
		for(L2DropData i : _items)
			temp.add(i);
		return temp;
	}

	public ItemToDrop roll(int diff, boolean isSpoil, boolean isRaidBoss, L2Player player)
	{
		float rate = isSpoil ? Config.RATE_DROP_SPOIL * player.getRateSpoil() : Config.RATE_DROP_ITEMS * player.getRateItems();
		float adenarate = Config.RATE_DROP_ADENA * player.getRateAdena();
		if(isRaidBoss && !Config.APPLY_RATES_ON_RBS)
		{
			rate = 1;
			adenarate = 1;
		}

		// calc group chance
		float calcChance = _chance * (_isAdena ? 1 : rate);

		if(Config.DEEPBLUE_DROP_RULES && (isSpoil ? !Config.ALT_SPOIL_FORMULA : true))
			if(diff > 1)
			{
				calcChance *= Experience.penaltyModifier(diff - 1, 9);
				adenarate *= Experience.penaltyModifier(diff - 1, 9);
			}

		int chance = Rnd.get(L2Drop.MAX_CHANCE);

		if(chance > calcChance)
			return null;

		// iterate each item in group and check chance
		chance = Rnd.get(L2Drop.MAX_CHANCE);
		for(L2DropData i : _items)
		{
			if(chance > i.getChance())
			{
				chance -= i.getChance();
				continue;
			}

			ItemToDrop t = new ItemToDrop(i.getItemId());
			// well, we have our drop-item now, let's get count
			int count = 0;

			if(i.getMinDrop() >= i.getMaxDrop())
				count = i.getMinDrop();
			else
				count = Rnd.get(i.getMinDrop(), i.getMaxDrop());
			if(_isAdena)
				count *= adenarate;
			t.count = count;
			t.isAdena = _isAdena;
			return t;
		}
		// can it be?
		return null;
	}

	public int getChance()
	{
		return _chance;
	}
}