package L2jFT.game.model;

import L2jFT.game.ThreadPoolManager;
import L2jFT.util.Rnd;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.instances.L2PenaltyMonsterInstance;
import L2jFT.game.network.serverpackets.ExFishingHpRegen;
import L2jFT.game.network.serverpackets.ExFishingStartCombat;
import L2jFT.game.network.serverpackets.InventoryUpdate;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.ItemTable;
import L2jFT.game.tables.NpcTable;
import L2jFT.game.templates.L2NpcTemplate;

import java.util.concurrent.Future;
import java.util.logging.Logger;

public class L2Fishing implements Runnable
{
	protected static Logger _log = Logger.getLogger(L2Fishing.class.getName());
	private L2Player _Fisher;
	private int _time;
	private int _stop = 0;
	private int _gooduse = 0;
	private int _anim = 0;
	private int _mode = 0;
	private int _deceptiveMode = 0;
	private Future _fishAItask;
	private boolean thinking;
	// Fish datas
	private int _FishID;
	private int _FishMaxHP;
	private int _FishCurHP;
	private double _regenHP;
	private boolean _isUpperGrade;

	public void run()
	{
		if(_FishCurHP >= _FishMaxHP * 2)
		{
			// The fish got away
			_Fisher.sendPacket(new SystemMessage(SystemMessage.THE_FISH_GOT_AWAY));
			PenaltyMonster(); //Random chance to spawn monster
			DoDie(false);
		}
		else if(_time <= 0)
		{
			// Time is up, so that fish got away
			_Fisher.sendPacket(new SystemMessage(SystemMessage.TIME_IS_UP_SO_THAT_FISH_GOT_AWAY));
			PenaltyMonster(); //Random chance to spawn monster
			DoDie(false);
		}
		else
			AiTask();
	}

	// =========================================================
	public L2Fishing(L2Player Fisher, FishData fish, boolean isNoob, boolean isUpperGrade)
	{
		_Fisher = Fisher;

		_FishMaxHP = fish.getHP();
		_FishCurHP = _FishMaxHP;
		_regenHP = fish.getHpRegen();
		_FishID = fish.getId();
		_time = fish.getCombatTime() / 1000;
		_isUpperGrade = isUpperGrade;
		int _lureType;
		if(isUpperGrade)
		{
			_deceptiveMode = Rnd.chance(10) ? 1 : 0;
			_lureType = 2;
		}
		else
		{
			_deceptiveMode = 0;
			_lureType = isNoob ? 0 : 1;
		}
		_mode = Rnd.chance(20) ? 1 : 0;

		ExFishingStartCombat efsc = new ExFishingStartCombat(_Fisher, _time, _FishMaxHP, _mode, _lureType, _deceptiveMode);
		_Fisher.broadcastPacket(efsc);

		// Succeeded in getting a bite
		_Fisher.sendPacket(new SystemMessage(SystemMessage.SUCCEEDED_IN_GETTING_A_BITE));

		if(_fishAItask == null)
			_fishAItask = ThreadPoolManager.getInstance().scheduleEffectAtFixedRate(this, 1000, 1000);

	}

	public void ChangeHp(int hp, int pen)
	{
		_FishCurHP -= hp;
		if(_FishCurHP < 0)
			_FishCurHP = 0;

		ExFishingHpRegen efhr = new ExFishingHpRegen(_Fisher, _time, _FishCurHP, _mode, _gooduse, _anim, pen, _deceptiveMode);
		_Fisher.broadcastPacket(efhr);
		_gooduse = 0;
		_anim = 0;
		if(_FishCurHP > _FishMaxHP * 2)
		{
			_FishCurHP = _FishMaxHP * 2;
			PenaltyMonster();
			DoDie(false);
		}
		else if(_FishCurHP == 0)
			DoDie(true);
	}

	public void DoDie(boolean win)
	{
		_fishAItask = null;

		if(_Fisher == null)
			return;

		try
		{
			if(win)
				if(Rnd.chance(5))
					PenaltyMonster();
				else
				{
					_Fisher.sendPacket(new SystemMessage(SystemMessage.SUCCEEDED_IN_FISHING));
					L2ItemInstance item = ItemTable.getInstance().createItem((short) _FishID, _Fisher.getObjectId(), 0, "FishItem_add");
					item.setCount(1);
					InventoryUpdate iu = new InventoryUpdate();
					iu.addItem(_Fisher.getInventory().addItem(item));
					_Fisher.sendPacket(iu);
				}
			_Fisher.endFishing(win);
		}
		catch(NullPointerException e)
		{}

		_Fisher = null;
	}

	protected void AiTask()
	{
		if(thinking)
			return;
		thinking = true;
		_time--;

		try
		{
			if(_mode == 1)
			{
				if(_deceptiveMode == 0)
					_FishCurHP += (int) _regenHP;
			}
			else if(_deceptiveMode == 1)
				_FishCurHP += (int) _regenHP;
			if(_stop == 0)
			{
				_stop = 1;
				if(Rnd.chance(30))
					_mode = _mode == 0 ? 1 : 0;

				if(_isUpperGrade)
					if(Rnd.chance(10))
						_deceptiveMode = _deceptiveMode == 0 ? 1 : 0;
			}
			else
				_stop--;
		}
		finally
		{
			thinking = false;
			ExFishingHpRegen efhr = new ExFishingHpRegen(_Fisher, _time, _FishCurHP, _mode, 0, _anim, 0, _deceptiveMode);
			if(_anim != 0)
				_Fisher.broadcastPacket(efhr);
			else
				_Fisher.sendPacket(efhr);
		}
	}

	public void UseRealing(int dmg, int pen)
	{
		_anim = 2;
		if(Rnd.chance(10))
		{
			_Fisher.sendPacket(new SystemMessage(SystemMessage.FISH_HAS_RESISTED));
			_gooduse = 0;
			ChangeHp(0, pen);
			return;
		}
		if(_Fisher == null)
			return;
		if(_mode == 1)
		{
			if(_deceptiveMode == 0)
			{
				// Reeling is successful, Damage: $s1
				SystemMessage sm = new SystemMessage(SystemMessage.REELING_IS_SUCCESSFUL_DAMAGE_S1);
				sm.addNumber(dmg);
				_Fisher.sendPacket(sm);
				if(pen == 50)
				{
					sm = new SystemMessage(SystemMessage.YOUR_REELING_WAS_SUCCESSFUL_MASTERY_PENALTYS1_);
					sm.addNumber(pen);
					_Fisher.sendPacket(sm);
				}
				_gooduse = 1;
				ChangeHp(dmg, pen);
			}
			else
			{
				// Reeling failed, Damage: $s1
				SystemMessage sm = new SystemMessage(SystemMessage.REELING_FAILED_DAMAGE_S1);
				sm.addNumber(dmg);
				_Fisher.sendPacket(sm);
				_gooduse = 2;
				ChangeHp(-dmg, pen);
			}
		}
		else if(_deceptiveMode == 0)
		{
			// Reeling failed, Damage: $s1
			SystemMessage sm = new SystemMessage(SystemMessage.REELING_FAILED_DAMAGE_S1);
			sm.addNumber(dmg);
			_Fisher.sendPacket(sm);
			_gooduse = 2;
			ChangeHp(-dmg, pen);
		}
		else
		{
			// Reeling is successful, Damage: $s1
			SystemMessage sm = new SystemMessage(SystemMessage.REELING_IS_SUCCESSFUL_DAMAGE_S1);
			sm.addNumber(dmg);
			_Fisher.sendPacket(sm);
			if(pen == 50)
			{
				sm = new SystemMessage(SystemMessage.REELING_IS_SUCCESSFUL_DAMAGE_S1);
				sm.addNumber(pen);
				_Fisher.sendPacket(sm);
			}
			_gooduse = 1;
			ChangeHp(dmg, pen);
		}
	}

	public void UsePomping(int dmg, int pen)
	{
		_anim = 1;
		if(Rnd.chance(10))
		{
			_Fisher.sendPacket(new SystemMessage(SystemMessage.FISH_HAS_RESISTED));
			_gooduse = 0;
			ChangeHp(0, pen);
			return;
		}
		if(_Fisher == null)
			return;
		if(_mode == 0)
		{
			if(_deceptiveMode == 0)
			{
				// Pumping is successful. Damage: $s1
				SystemMessage sm = new SystemMessage(SystemMessage.PUMPING_IS_SUCCESSFUL_DAMAGE_S1);
				sm.addNumber(dmg);
				_Fisher.sendPacket(sm);
				if(pen == 50)
				{
					sm = new SystemMessage(SystemMessage.YOUR_PUMPING_WAS_SUCCESSFUL_MASTERY_PENALTYS1_);
					sm.addNumber(pen);
					_Fisher.sendPacket(sm);
				}
				_gooduse = 1;
				ChangeHp(dmg, pen);
			}
			else
			{
				// Pumping failed, Regained: $s1
				SystemMessage sm = new SystemMessage(SystemMessage.PUMPING_FAILED_DAMAGE_S1);
				sm.addNumber(dmg);
				_Fisher.sendPacket(sm);
				_gooduse = 2;
				ChangeHp(-dmg, pen);
			}
		}
		else if(_deceptiveMode == 0)
		{
			// Pumping failed, Regained: $s1
			SystemMessage sm = new SystemMessage(SystemMessage.PUMPING_FAILED_DAMAGE_S1);
			sm.addNumber(dmg);
			_Fisher.sendPacket(sm);
			_gooduse = 2;
			ChangeHp(-dmg, pen);
		}
		else
		{
			// Pumping is successful. Damage: $s1
			SystemMessage sm = new SystemMessage(SystemMessage.PUMPING_IS_SUCCESSFUL_DAMAGE_S1);
			sm.addNumber(dmg);
			_Fisher.sendPacket(sm);
			if(pen == 50)
			{
				sm = new SystemMessage(SystemMessage.YOUR_PUMPING_WAS_SUCCESSFUL_MASTERY_PENALTYS1_);
				sm.addNumber(pen);
				_Fisher.sendPacket(sm);
			}
			_gooduse = 1;
			ChangeHp(dmg, pen);
		}
	}

	private void PenaltyMonster()
	{
		int lvl = (int) Math.round(_Fisher.getLevel() * 0.1);
		int npcid;

		_Fisher.sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_CAUGHT_A_MONSTER));
		switch(lvl)
		{
			case 0:
			case 1:
				npcid = 18319;
				break;
			case 2:
				npcid = 18320;
				break;
			case 3:
				npcid = 18321;
				break;
			case 4:
				npcid = 18322;
				break;
			case 5:
				npcid = 18323;
				break;
			case 6:
				npcid = 18324;
				break;
			case 7:
				npcid = 18325;
				break;
			case 8:
				npcid = 18326;
				break;
			default:
				npcid = 18319;
				break;
		}

		L2NpcTemplate temp = NpcTable.getTemplate(npcid);
		if(temp != null)
		{
			L2Spawn spawn;
			try
			{
				spawn = new L2Spawn(temp);
				spawn.setLocx(_Fisher.getFishx());
				spawn.setLocy(_Fisher.getFishy());
				spawn.setLocz(_Fisher.getFishz());
				spawn.setAmount(1);
				spawn.setHeading(_Fisher.getHeading() - 32768);
				spawn.stopRespawn();
				L2PenaltyMonsterInstance monster = (L2PenaltyMonsterInstance) spawn.doSpawn(false);
				monster.SetPlayerToKill(_Fisher);
			}
			catch(Exception e)
			{
				_log.warning("Could not spawn Penalty Monster " + npcid + ", exception: " + e);
			}
		}
	}
}