package L2jFT.game.model;

import javolution.util.FastList;
import L2jFT.Config;
import L2jFT.game.ThreadPoolManager;
import L2jFT.game.ai.CtrlEvent;
import L2jFT.game.ai.CtrlIntention;
import L2jFT.game.cache.Msg;
import L2jFT.game.geodata.GeoData;
import L2jFT.game.instancemanager.SiegeManager;
import L2jFT.util.Rnd;
import L2jFT.game.model.L2Skill.SkillType;
import L2jFT.game.model.L2Zone.ZoneType;
import static L2jFT.game.model.L2Zone.ZoneType.Siege;
import static L2jFT.game.model.L2Zone.ZoneType.peace_zone;
import L2jFT.game.model.entity.Duel;
import L2jFT.game.model.entity.siege.Siege;
import L2jFT.game.model.instances.L2ArtefactInstance;
import L2jFT.game.model.instances.L2DoorInstance;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.model.instances.L2SiegeGuardInstance;
import L2jFT.game.network.serverpackets.Attack;
import L2jFT.game.skills.Stats;
import L2jFT.game.tables.SkillTable;
import L2jFT.game.templates.L2CharTemplate;
import L2jFT.game.templates.L2Weapon;
import L2jFT.game.templates.L2Weapon.WeaponType;

/**
 * This class represents all Playable characters in the world.
 *
 * L2PlayableInstance:
 *  * L2Player
 *  * L2Summon
 *
 */
@SuppressWarnings( { "nls", "unqualified-field-access", "boxing" })
public abstract class L2Playable extends L2Character
{
	/**
	 * this если Player или owner если Summon 
	 */
	public L2Player _owner;

	public L2Playable(int objectId, L2CharTemplate template, L2Player owner)
	{
		super(objectId, template);
		if(owner == null)
			_owner = (L2Player) this;
		else
			_owner = owner;
	}

	@Override
	public boolean isAttackable()
	{
		return true;
	}

	public abstract Inventory getInventory();

	/**
	 * Проверяет, выставлять ли PvP флаг для игрока.<BR><BR>
	 */
	@Override
	public boolean checkPvP(final L2Character target, L2Skill skill)
	{
		if(target == null || _owner == null || target == this || target == _owner || target == _owner.getPet() || _owner.getKarma() > 0)
			return false;

		if(skill != null && skill.altUse())
			return false;

		// Проверка на дуэли... Мэмбэры одной дуэли не флагаются
		if(getDuel() != null && getDuel() == target.getDuel())
			return false;

		if(isInZone(peace_zone) || target.isInZone(peace_zone) || isInZoneBattle() || target.isInZoneBattle())
			return false;
		if(isInZone(Siege) && target.isInZone(Siege))
			return false;
		if(skill == null || skill.isOffensive())
		{
			if(target.getKarma() > 0)
				return false;
			else if(target instanceof L2Playable)
				return true;
		}
		else if(target.getPvpFlag() > 0 || target.getKarma() > 0 || target.isMonster)
			return true;

		return false;
	}

	/**
	 * Проверяет, можно ли атаковать цель (для физ атак)
	 */
	public boolean checkAttack(L2Character target)
	{
		if(target == null || target.isDead())
		{
			_owner.sendPacket(Msg.INCORRECT_TARGET);
			return false;
		}

		if(!isInRange(target, 2000))
		{
			_owner.sendPacket(Msg.YOUR_TARGET_IS_OUT_OF_RANGE);
			return false;
		}

		if(target instanceof L2DoorInstance && !((L2DoorInstance) target).isAttackable(this))
		{
			_owner.sendPacket(Msg.INCORRECT_TARGET);
			return false;
		}

		if(_owner.inObserverMode())
		{
			_owner.sendMessage("Cant attack in observer mode", "Нельзя атаковать в режиме просмотра");
			return false;
		}

		if(target.isRaid)
			if(_owner.getLevel() > target.getLevel() + Config.RAID_MAX_LEVEL_DIFF)
			{
				L2Skill revengeSkill = SkillTable.getInstance().getInfo(4515, 1);
				revengeSkill.getEffects(target, _owner, false);
				if(!isPlayer)
					revengeSkill.getEffects(target, this, false);
			}

		if(!GeoData.getInstance().canSeeTarget(this, target))
		{
			_owner.sendPacket(Msg.CANNOT_SEE_TARGET);
			return false;
		}

		if(target instanceof L2Playable)
		{
			// Нельзя атаковать того, кто находится на арене, если ты сам не на арене
			if(isInZoneBattle() != target.isInZoneBattle())
			{
				_owner.sendPacket(Msg.INCORRECT_TARGET);
				return false;
			}

			// Если цель либо атакующий находится в мирной зоне - атаковать нельзя
			if((isInZonePeace() || target.isInZonePeace()) && !_owner.getPlayerAccess().PeaceAttack)
				if(!Config.ALT_GAME_KARMAPLAYERCANBEKILLEDINPEACEZONE || getKarma() <= 0 && target.getKarma() <= 0)
				{
					_owner.sendPacket(Msg.YOU_CANNOT_ATTACK_THE_TARGET_IN_THE_PEACE_ZONE);
					return false;
				}

			if(target.isPlayer && (getKarma() > 0 || getPvpFlag() > 0) && getLevel() > target.getLevel() + 10 && target.getEffect((short) 5182) != null) //5182 = Blessing of protection
			{
				sendPacket(Msg.TARGET_IS_INCORRECT);
				return false;
			}
		}

		return true;
	}

	@Override
	public void doAttack(L2Character target)
	{
		if(isAttackingNow())
		{
			_owner.sendActionFailed();
			return;
		}

		if(!checkAttack(target))
		{
			getAI().setIntention(CtrlIntention.AI_INTENTION_ACTIVE, null, null);
			_owner.sendActionFailed();
			return;
		}

		// Прерывать дуэли если цель не дуэлянт
		if(getDuel() != null && target.getDuel() != getDuel())
			getDuel().setDuelState(_owner, Duel.DuelState.Interrupted);

		// Get the active weapon item corresponding to the active weapon instance (always equipped in the right hand)
		L2Weapon weaponItem = getActiveWeaponItem();

		if(weaponItem != null && weaponItem.getItemType() == WeaponType.BOW)
		{
			double bowMpConsume = weaponItem.getMpConsume();

			// cheap shot SA
			double chance = calcStat(Stats.MP_USE_BOW_CHANCE, 0., null, null);
			if(chance > 0 && chance > Rnd.get())
				bowMpConsume = calcStat(Stats.MP_USE_BOW, bowMpConsume, null, null);

			if(_currentMp < bowMpConsume)
			{
				getAI().setIntention(CtrlIntention.AI_INTENTION_ACTIVE, null, null);
				_owner.sendPacket(Msg.NOT_ENOUGH_MP);
				_owner.sendActionFailed();
				return;
			}

			reduceCurrentMp(bowMpConsume);

			if(!_owner.checkAndEquipArrows())
			{
				getAI().setIntention(CtrlIntention.AI_INTENTION_ACTIVE, null, null);
				_owner.sendPacket(Msg.NOT_ENOUGH_ARROWS);
				_owner.sendActionFailed();
				return;
			}
		}

		// Verify if SoulShot are charged
		boolean wasSSCharged = getChargedSoulShot();

		// Get the Attack Speed of the L2Character (delay (in milliseconds) before next attack)
		int sAtk = Math.max(calculateAttackSpeed(), 200);

		// Get the Attack Reuse Delay of the L2Weapon
		int reuse = 0;
		if(weaponItem != null)
			if(weaponItem.getItemType() == WeaponType.BOW)
				reuse = (int) (sAtk / Config.BOW_REUSE_MODIFIER * getReuseModifier(target));
			else
				reuse = weaponItem.getAttackReuseDelay();

		//reuse = Math.max(100, reuse);

		_attackEndTime = sAtk + System.currentTimeMillis();
		_attackReuseEndTime = sAtk + reuse + System.currentTimeMillis();

		// Create a Server->Client packet Attack
		Attack attack = new Attack(this, wasSSCharged);

		boolean hitted;

		_isAttackAborted = false;

		setHeading(target, true);

		// Select the type of attack to start
		if(weaponItem == null)
			hitted = doAttackHitSimple(attack, target, weaponItem, 1., isSummon || isPet, sAtk);
		else
			switch(weaponItem.getItemType())
			{
				case BOW:
					hitted = doAttackHitByBow(attack, target, sAtk, reuse, weaponItem);
					break;
				case POLE:
					if(calcStat(Stats.POLE_ATTACK_ANGLE, 0, null, null) == -1)
						hitted = doAttackHitSimple(attack, target, weaponItem, 1., true, sAtk);
					else
						hitted = doAttackHitByPole(attack, weaponItem, sAtk);
					break;
				case DUAL:
				case DUALFIST:
					hitted = doAttackHitByDual(attack, target, weaponItem, sAtk);
					break;
				default:
					hitted = doAttackHitSimple(attack, target, weaponItem, 1., true, sAtk);
			}

		if(hitted)
			// if hitted by a cursed weapon, Cp is reduced to 0
			// if a cursed weapon is hitted by a Hero, Cp is reduced to 0
			if(isPlayer && target.isPlayer && !target.isInvul())
				if(isCursedWeaponEquipped() || isHero() && target.isCursedWeaponEquipped())
					target.setCurrentCp(0);

		if(attack.hasHits())
			broadcastPacket(attack);

		ThreadPoolManager.getInstance().scheduleAi(new NotifyAITask(CtrlEvent.EVT_READY_TO_ACT), sAtk + reuse + 100);
	}

	@Override
	protected void onHitTimer(L2Character target, int damage, boolean crit, boolean miss, boolean soulshot, boolean shld, boolean unchargeSS)
	{
		if(target instanceof L2Playable)
			if(isInZoneBattle() != target.isInZoneBattle())
			{
				_owner.sendPacket(Msg.INCORRECT_TARGET);
				_owner.sendActionFailed();
				return;
			}

		super.onHitTimer(target, damage, crit, miss, soulshot, shld, unchargeSS);
	}

	@Override
	public boolean isAutoAttackable(final L2Character attacker)
	{
		if(_owner == null || attacker == this || attacker == _owner || isDead())
			return false;

		if(_owner.isInOlympiadMode() && !_owner.isOlympiadCompStart())
			return false;

		// Автоатака на дуэлях, только враг и только если он еше не проиграл.
		if(getDuel() != null && attacker.getDuel() == getDuel())
		{
			//Тут не может быть ClassCastException у attacker, т.к. L2Character.getDuel() всегда возвращает null
			L2Player enemy = attacker.isPlayer ? (L2Player) attacker : ((L2Summon) attacker).getOwner();

			if(enemy.getDuel().getTeamForPlayer(enemy) != getDuel().getTeamForPlayer(_owner) && getDuel().getDuelState(_owner) == Duel.DuelState.Fighting && getDuel().getDuelState(enemy) == Duel.DuelState.Fighting)
				return true;
		}

		// TODO: check for friendly mobs
		if(attacker.isMonster)
			return true;
		final L2Player pcAttacker = attacker.getPlayer();
		final L2Clan clan1 = _owner.getClan();
		if(pcAttacker != null)
		{
			if(_owner.getParty() != null && _owner.getParty().getPartyMembers().contains(pcAttacker))
				return false;
			if(isInZoneBattle() && attacker.isInZoneBattle())
				return true;
			final L2Clan clan2 = pcAttacker.getClan();
			if(clan1 != null && clan2 != null)
			{
				if(clan1.getClanId() == clan2.getClanId())
					return false;
				if(pcAttacker.atMutualWarWith(_owner))
					return true;
			}
			if(isInZone(ZoneType.Siege) && attacker.isInZone(ZoneType.Siege))
			{
				if(clan1 == null || clan2 == null)
					return true;
				if(!clan1.getIsDefender() && !clan1.getIsAttacker())
					return true;
				if(!clan2.getIsDefender() && !clan2.getIsAttacker())
					return true;
				if(clan1.getIsDefender() && clan2.getIsAttacker() || clan2.getIsDefender() && clan1.getIsAttacker())
					return true;
			}
			if(_owner.getKarma() > 0 || _owner.getPvpFlag() != 0)
				return true;
		}
		else if(attacker instanceof L2SiegeGuardInstance)
			if(clan1 != null)
			{
				final Siege siege = SiegeManager.getInstance().getSiege(this);
				return siege != null && siege.checkIsAttacker(clan1);
			}
		return false;
	}

	@Override
	public int getKarma()
	{
		if(_owner == null)
			return 0;
		return _owner.getKarma();
	}

	@Override
	public void callSkill(L2Skill skill, FastList<L2Character> targets)
	{
		FastList<L2Character> toRemove = new FastList<L2Character>();

		if(!skill.altUse() && !skill.getSkillType().equals(SkillType.BEAST_FEED))
			for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
			{
				if(n == null)
					break;
				L2Character target = n.getValue();

				if(target.isInvul() && !(target instanceof L2ArtefactInstance))
					toRemove.add(target);

				if(!skill.isOffensive())
				{
					if(target instanceof L2Playable)
					{
						int aggro = skill.getEffectPoint() != 0 ? skill.getEffectPoint() : skill.getPower() > 0 ? (int) skill.getPower() : skill.getLevel() * 5;
						FastList<L2NpcInstance> enemyList = ((L2Playable) target).getEnemyList();
						for(FastList.Node<L2NpcInstance> m = enemyList.head(), e = enemyList.tail(); (m = m.getNext()) != e;)
						{
							L2NpcInstance monster = m.getValue();
							if(monster != null && !monster.isDead())
							{
								if(monster.hasAI())
									monster.getAI().notifyEvent(CtrlEvent.EVT_SEE_SPELL, skill, this);
								monster.addDamageHate(this, 0, aggro);
								if(monster.isRaid && _owner.getLevel() > monster.getLevel() + Config.RAID_MAX_LEVEL_DIFF)
								{
									final L2Skill revengeSkill = SkillTable.getInstance().getInfo(4515, 1);
									revengeSkill.getEffects(target, _owner, false);
									if(this instanceof L2Summon)
										revengeSkill.getEffects(target, this, false);
								}
							}
						}
					}
				}
				else if(target instanceof L2NpcInstance)
				{
					if(target.hasAI())
						target.getAI().notifyEvent(CtrlEvent.EVT_SEE_SPELL, skill, this);
					// mobs will hate on debuff
					if(!skill.isAI())
						((L2NpcInstance) target).addDamageHate(this, 1, skill.getEffectPoint() != 0 ? skill.getEffectPoint() : 1);
					if(target.isRaid)
						if(_owner.getLevel() > target.getLevel() + Config.RAID_MAX_LEVEL_DIFF)
						{
							L2Skill revengeSkill = SkillTable.getInstance().getInfo(4515, 1);
							revengeSkill.getEffects(target, _owner, false);
							if(this instanceof L2Summon)
								revengeSkill.getEffects(target, this, false);
						}
				}
				// Check for PvP Flagging / Drawing Aggro
				if(checkPvP(target, skill))
					startPvPFlag(target);
			}

		for(L2Character cha : toRemove)
			targets.remove(cha);

		super.callSkill(skill, targets);
	}

	public FastList<L2NpcInstance> getEnemyList()
	{
		if(!isVisible())
			return new FastList<L2NpcInstance>();

		return L2World.getEnemyNpc(this, 2000, 400);
	}

	@Override
	public L2Player getOwner()
	{
		return _owner;
	}

	public boolean isSilentMoving()
	{
		return false;
	}
}