package L2jFT.game.model;

import L2jFT.game.network.serverpackets.RadarControl;

import java.util.Vector;

/**
 * @author dalrond
 */
public final class L2Radar
{
	private L2Player player;
	private Vector<RadarMarker> markers;

	public L2Radar(L2Player player_)
	{
		player = player_;
		markers = new Vector<RadarMarker>();
	}

	// Add a marker to player's radar
	public void addMarker(int x, int y, int z)
	{
		RadarMarker newMarker = new RadarMarker(x, y, z);
		markers.add(newMarker);
		player.sendPacket(new RadarControl(0, 1, x, y, z));
	}

	// Remove a marker from player's radar
	public void removeMarker(int x, int y, int z)
	{
		RadarMarker newMarker = new RadarMarker(x, y, z);
		markers.remove(newMarker);
		player.sendPacket(new RadarControl(1, 1, x, y, z));
	}

	public void removeAllMarkers()
	{
		// TODO: Need method to remove all markers from radar at once
		for(RadarMarker tempMarker : markers)
			player.sendPacket(new RadarControl(1, tempMarker.type, tempMarker.x, tempMarker.y, tempMarker.z));
		markers.removeAllElements();
	}

	public void loadMarkers()
	{
	// TODO: Need method to re-send radar markers after load/teleport/death
	// etc.
	}

	public class RadarMarker
	{
		// Simple class to model radar points.
		public int type, x, y, z;

		public RadarMarker(int type_, int x_, int y_, int z_)
		{
			type = type_;
			x = x_;
			y = y_;
			z = z_;
		}

		public RadarMarker(int x_, int y_, int z_)
		{
			type = 1;
			x = x_;
			y = y_;
			z = z_;
		}

		@Override
		public boolean equals(Object obj)
		{
			try
			{
				RadarMarker temp = (RadarMarker) obj;
				if(temp.x == x && temp.y == y && temp.z == z && temp.type == type)
					return true;
				return false;
			}
			catch(Exception e)
			{
				return false;
			}
		}
	}
}