package L2jFT.game.model;

import javolution.util.FastList;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.network.serverpackets.InventoryUpdate;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.ItemTable;
import L2jFT.util.Log;

import java.util.HashSet;
import java.util.logging.Logger;

public class L2TradeList
{
	private static Logger _log = Logger.getLogger(L2TradeList.class.getName());

	private FastList<L2ItemInstance> _items;
	private int _listId;
	private boolean _confirmed;
	private String _Buystorename, _Sellstorename;

	private String _npcId;

	public L2TradeList(int listId)
	{
		_items = new FastList<L2ItemInstance>();
		_listId = listId;
		_confirmed = false;
	}

	public void setNpcId(String id)
	{
		_npcId = id;
	}

	public String getNpcId()
	{
		return _npcId;
	}

	public void addItem(L2ItemInstance item)
	{
		synchronized (_items)
		{
			_items.add(item);
		}
	}

	public void replaceItem(int itemID, int price)
	{
		synchronized (_items)
		{
			for(L2ItemInstance item : _items)
				if(item.getItemId() == itemID)
				{
					item.setPriceToSell(price);
					_items.add(item);
				}
		}
	}

	public void removeItem(int itemID)
	{
		synchronized (_items)
		{
			for(L2ItemInstance item : _items)
				if(item.getItemId() == itemID)
					_items.remove(item);
		}
	}

	public void removeAll()
	{
		_items = new FastList<L2ItemInstance>();
	}

	/**
	 * @return Returns the listId.
	 */
	public int getListId()
	{
		return _listId;
	}

	public void setSellStoreName(String name)
	{
		_Sellstorename = name;
	}

	public String getSellStoreName()
	{
		return _Sellstorename;
	}

	public void setBuyStoreName(String name)
	{
		_Buystorename = name;
	}

	public String getBuyStoreName()
	{
		return _Buystorename;
	}

	/**
	 * @return Returns the items.
	 */
	public FastList<L2ItemInstance> getItems()
	{
		return _items;
	}

	public HashSet<L2ItemInstance> getItems(int start, int end)
	{
		L2ItemInstance[] items;
		synchronized (_items)
		{
			items = _items.toArray(new L2ItemInstance[_items.size()]);
		}
		HashSet<L2ItemInstance> ret = new HashSet<L2ItemInstance>();
		for(int i = start; i == end; i++)
			ret.add(items[i]);
		return ret;
	}

	public int getPriceForItemId(int itemId)
	{
		synchronized (_items)
		{
			for(L2ItemInstance item : _items)
				if(item.getItemId() == itemId)
					return item.getPriceToSell();
		}
		return -1;
	}

	public L2ItemInstance getItemByItemId(int itemId)
	{
		synchronized (_items)
		{
			for(L2ItemInstance item : _items)
				if(item.getItemId() == itemId)
					return item;
		}
		return null;
	}

	public L2ItemInstance getItem(int ObjectId)
	{
		synchronized (_items)
		{
			for(L2ItemInstance item : _items)
				if(item.getObjectId() == ObjectId)
					return item;
		}
		return null;
	}

	public void setConfirmedTrade(boolean x)
	{
		_confirmed = x;
	}

	public boolean hasConfirmed()
	{
		return _confirmed;
	}

	public boolean removeItem(int objId, int count)
	{
		synchronized (_items)
		{
			for(L2ItemInstance item : _items)
				if(item.getObjectId() == objId)
					if(count == item.getCount())
					{
						_items.remove(item);
						return true;
					}
		}
		return false;
	}

	public boolean contains(int objId)
	{
		synchronized (_items)
		{
			for(L2ItemInstance item : _items)
				if(item.getObjectId() == objId)
					return true;
		}
		return false;
	}

	public boolean validateTrade(L2Player player)
	{
		Inventory playersInv = player.getInventory();
		L2ItemInstance playerItem;
		synchronized (_items)
		{
			for(L2ItemInstance item : _items)
			{
				playerItem = playersInv.getItemByObjectId(item.getObjectId());
				if(playerItem == null || playerItem.getCount() < item.getCount() || playerItem.isEquipped())
					return false;
			}
		}
		return true;
	}

	//Call validate before this
	// synchronized не трогать - CME фикс!
	public synchronized void tradeItems(L2Player player, L2Player reciever)
	{
		Inventory playersInv = player.getInventory();
		Inventory recieverInv = reciever.getInventory();
		L2ItemInstance recieverItem, TransferItem;
		InventoryUpdate update1 = new InventoryUpdate();
		InventoryUpdate update2 = new InventoryUpdate();

		for(L2ItemInstance temp : _items)
		{
			//If player trades the enchant scroll he was using remove its effect
			if(player.getEnchantScroll() != null && temp.getObjectId() == player.getEnchantScroll().getObjectId())
				player.setEnchantScroll(null);

			L2ItemInstance oldItem = playersInv.getItemByObjectId(temp.getObjectId());
			TransferItem = playersInv.dropItem(temp.getObjectId(), temp.getCount());
			Log.LogItem(player, reciever, Log.TradeGive, TransferItem);

			if(TransferItem == null)
			{
				_log.warning("L2TradeList: PlayersInv.destroyItem returned NULL! player: " + player.getName() + " reciver:" + reciever.getName());
				continue;
			}

			temp.setLastChange((byte) oldItem.getLastChange());

			// TODO: этот бред снести 
			if(temp.getCount() == oldItem.getCount())
				update1.addItem(temp);
			else
				update1.addItem(oldItem);

			recieverItem = recieverInv.addItem(TransferItem);
			update2.addItem(recieverItem);

			Log.LogItem(player, reciever, Log.TradeGet, recieverItem);
		}

		player.sendPacket(update1);
		reciever.sendPacket(update2);

		player.sendChanges();
		reciever.sendChanges();
	}

	public void updateBuyList(L2Player player, HashSet<TradeItem> list)
	{
		Inventory playersInv = player.getInventory();

		for(L2ItemInstance temp : _items)
			if(playersInv.findItemByItemId(temp.getItemId()) == null)
				list.remove(temp);
			else if(temp.getCount() == 0)
				list.remove(temp);
	}

	public void updateSellList(L2Player player, HashSet<TradeItem> list)
	{
		Inventory playersInv = player.getInventory();
		L2ItemInstance temp2;
		for(L2ItemInstance temp : _items)
		{
			temp2 = playersInv.getItemByObjectId(temp.getObjectId());
			if(temp2 == null)
				list.remove(temp);
			else if(temp2.getCount() < temp.getCount())
				temp.setCount(temp2.getCount());
		}
	}

	@SuppressWarnings("unchecked")
	public synchronized void buySellItems(L2Player buyer, HashSet<TradeItem> listToBuy, L2Player seller, HashSet<TradeItem> listToSell)
	{
		Inventory sellerInv = seller.getInventory();
		Inventory buyerInv = buyer.getInventory();

		TradeItem sellerTradeItem = null;
		L2ItemInstance sellerInventoryItem = null;
		L2ItemInstance TransferItem = null;
		L2ItemInstance temp = null;
		HashSet<TradeItem> unsold = (HashSet<TradeItem>) listToSell.clone();

		InventoryUpdate buyerupdate = new InventoryUpdate();
		InventoryUpdate sellerupdate = new InventoryUpdate();

		long cost = 0, amount = 0;

		for(TradeItem buyerTradeItem : listToBuy)
		{
			for(TradeItem itm : unsold)
				if(itm.getItemId() == buyerTradeItem.getItemId())
				{
					sellerTradeItem = itm;
					if(itm.getObjectId() != 0 && !ItemTable.getInstance().getTemplate(itm.getItemId()).isStackable())
						sellerInventoryItem = sellerInv.getItemByObjectId(buyerTradeItem.getObjectId());
					else
						sellerInventoryItem = sellerInv.findItemByItemId(buyerTradeItem.getItemId());
					break;
				}
			if(sellerTradeItem == null)
				continue;

			unsold.remove(sellerTradeItem);

			if(sellerInventoryItem == null)
				continue;

			int buyerItemCount = buyerTradeItem.getCount();
			int sellerItemCount = sellerTradeItem.getCount();

			if(sellerItemCount > sellerInventoryItem.getCount())
				sellerItemCount = sellerInventoryItem.getCount();

			if(seller.getPrivateStoreType() == L2Player.STORE_PRIVATE_SELL)
			{
				if(buyerItemCount > sellerItemCount)
					buyerTradeItem.setCount(sellerItemCount);
				if(buyerItemCount > sellerInventoryItem.getCount())
					buyerTradeItem.setCount(sellerInventoryItem.getCount());
				buyerItemCount = buyerTradeItem.getCount();
				amount = buyerItemCount;
				cost = amount * sellerTradeItem.getOwnersPrice();
			}

			if(buyer.getPrivateStoreType() == L2Player.STORE_PRIVATE_BUY)
			{
				if(sellerItemCount > buyerItemCount)
					sellerTradeItem.setCount(buyerItemCount);
				if(sellerItemCount > sellerInventoryItem.getCount())
					sellerTradeItem.setCount(sellerInventoryItem.getCount());
				sellerItemCount = sellerTradeItem.getCount();
				amount = sellerItemCount;
				cost = amount * buyerTradeItem.getOwnersPrice();
			}

			long sum = (long) buyerItemCount * buyerTradeItem.getOwnersPrice();
			if(sum > Integer.MAX_VALUE)
			{
				_log.warning("Integer Overflow on Cost. Possible Exploit attempt between " + buyer.getName() + " and " + seller.getName() + ".");
				_log.warning(buyer.getName() + " try to use exploit, ban this player!");
				seller.sendMessage("Buyer try to use Overflow Exploit. Don't trust him!", "Покупатель пытается использовать читы. Не доверяйте ему!");
				return;
			}
			sum = (long) sellerItemCount * sellerTradeItem.getOwnersPrice();
			if(sum > Integer.MAX_VALUE)
			{
				_log.warning("Integer Overflow on Cost. Possible Exploit attempt between " + buyer.getName() + " and " + seller.getName() + ".");
				_log.warning(seller.getName() + " try to use exploit, ban this player!");
				buyer.sendMessage("Seller try to use Overflow Exploit. Don't trust him!", "Продавец пытается использовать читы. Не доверяйте ему!");
				return;
			}

			if(buyer.getAdena() < cost)
			{
				_log.warning("buy item without full adena sum " + buyer.getName() + " and " + seller.getName() + ".");
				return;
			}

			TransferItem = sellerInv.dropItem(sellerInventoryItem.getObjectId(), amount);
			Log.LogItem(seller, buyer, Log.TradeGive, TransferItem, amount);
			buyer.reduceAdena(cost, false);
			seller.addAdena(cost);
			temp = buyerInv.addItem(TransferItem);
			Log.LogItem(buyer, seller, Log.TradeGive, TransferItem, amount);

			if(!temp.isStackable())
			{
				if(temp.getEnchantLevel() > 0)
				{
					seller.sendPacket(new SystemMessage(SystemMessage._S2S3_HAS_BEEN_SOLD_TO_S1_AT_THE_PRICE_OF_S4_ADENA).addString(buyer.getName()).addNumber(temp.getEnchantLevel()).addItemName(sellerInventoryItem.getItemId()).addNumber((int) cost));
					buyer.sendPacket(new SystemMessage(SystemMessage._S2S3_HAS_BEEN_PURCHASED_FROM_S1_AT_THE_PRICE_OF_S4_ADENA).addString(seller.getName()).addNumber(temp.getEnchantLevel()).addItemName(sellerInventoryItem.getItemId()).addNumber((int) cost));
				}
				else
				{
					seller.sendPacket(new SystemMessage(SystemMessage.S2_IS_SOLD_TO_S1_AT_THE_PRICE_OF_S3_ADENA).addString(buyer.getName()).addItemName(sellerInventoryItem.getItemId()).addNumber((int) cost));
					buyer.sendPacket(new SystemMessage(SystemMessage.S2_HAS_BEEN_PURCHASED_FROM_S1_AT_THE_PRICE_OF_S3_ADENA).addString(seller.getName()).addItemName(sellerInventoryItem.getItemId()).addNumber((int) cost));
				}
			}
			else
			{
				seller.sendPacket(new SystemMessage(SystemMessage.S2_S3_HAVE_BEEN_SOLD_TO_S1_FOR_S4_ADENA).addString(buyer.getName()).addNumber((int) amount).addItemName(sellerInventoryItem.getItemId()).addNumber((int) cost));
				buyer.sendPacket(new SystemMessage(SystemMessage.S3_S2_HAS_BEEN_PURCHASED_FROM_S1_FOR_S4_ADENA).addString(seller.getName()).addItemName(sellerInventoryItem.getItemId()).addNumber((int) amount).addNumber((int) cost));
			}

			sellerupdate.addItem(sellerInventoryItem);
			buyerupdate.addItem(temp);

			sellerInventoryItem = null;
		}

		if(TransferItem != null)
		{
			sellerupdate.addItem(seller.getInventory().getItemByItemId(57));
			buyerupdate.addItem(buyer.getInventory().getItemByItemId(57));

			seller.sendPacket(sellerupdate);
			seller.sendChanges();
			buyer.sendPacket(buyerupdate);
			buyer.sendChanges();
		}

		HashSet<TradeItem> tmp;
		// update seller's sell list
		if(seller.getPrivateStoreType() == L2Player.STORE_PRIVATE_SELL)
		{
			tmp = (HashSet<TradeItem>) seller.getSellList().clone();
			for(TradeItem sl : listToSell)
				for(TradeItem bl : listToBuy)
					if(sl.getItemId() == bl.getItemId() && ItemTable.getInstance().getTemplate(bl.getItemId()).isStackable())
					{
						sl.setCount(sl.getCount() - bl.getCount());
						if(sl.getCount() <= 0)
						{
							tmp.remove(sl);
							continue;
						}
						L2ItemInstance inst = seller.getInventory().getItemByObjectId(sl.getObjectId());
						if(inst == null || inst.getCount() <= 0)
						{
							tmp.remove(sl);
							continue;
						}
						if(inst.getCount() < sl.getCount())
							sl.setCount(inst.getCount());
						tmp.add(sl);
					}

			for(TradeItem sl : listToSell)
				if(!ItemTable.getInstance().getTemplate(sl.getItemId()).isStackable())
				{
					L2ItemInstance inst = seller.getInventory().getItemByObjectId(sl.getObjectId());
					if(inst == null)
						tmp.remove(sl);
				}
			seller.setSellList(tmp);
		}

		// update buyer's buy list
		if(buyer.getPrivateStoreType() == L2Player.STORE_PRIVATE_BUY)
		{
			tmp = (HashSet<TradeItem>) buyer.getBuyList().clone();
			for(TradeItem bl : listToBuy)
				for(TradeItem sl : listToSell)
					if(sl.getItemId() == bl.getItemId() && ItemTable.getInstance().getTemplate(bl.getItemId()).isStackable())
					{
						bl.setCount(bl.getCount() - sl.getCount());
						if(bl.getCount() <= 0)
						{
							tmp.remove(bl);
							continue;
						}
						tmp.add(bl);
					}

			for(TradeItem sl : listToSell)
				if(!ItemTable.getInstance().getTemplate(sl.getItemId()).isStackable())
				{
					TradeItem toremove = null;
					for(TradeItem bl : listToBuy)
						if(bl.getItemId() == sl.getItemId())
							toremove = bl;
					tmp.remove(toremove);
				}

			buyer.setBuyList(tmp);
		}
	}
}
