package L2jFT.game.model;

import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.instances.L2ItemInstance.ItemLocation;
import L2jFT.util.Log;

import java.util.logging.Logger;

public class PcWarehouse extends Warehouse
{
	private static final Logger _log = Logger.getLogger(PcWarehouse.class.getName());

	private L2Player _owner;

	public PcWarehouse(L2Player owner)
	{
		_owner = owner;
	}

	@Override
	public int getOwnerId()
	{
		return _owner.getObjectId();
	}

	@Override
	public ItemLocation getLocationType()
	{
		return ItemLocation.WAREHOUSE;
	}

	@Override
	public String getLocationId()
	{
		return "0";
	}

	@Override
	public byte getLocationId(@SuppressWarnings("unused")
	boolean dummy)
	{
		return 0;
	}

	@Override
	public void setLocationId(@SuppressWarnings("unused")
	L2Player dummy)
	{
		return;
	}

	@Override
	public final WarehouseType getWarehouseType()
	{
		return WarehouseType.PRIVATE;
	}

	@Override
	public synchronized L2ItemInstance addItem(L2ItemInstance newItem)
	{
		if(newItem.getCount() < 0)
		{
			_log.warning("addItem to warhouse: count < 0 owner:" + _owner.getName());
			return null;
		}
		Log.add("PcWarehouse|" + _owner.getName() + "|add item|" + newItem.getItemId() + "|" + newItem.getCount() + "|" + newItem.getObjectId(), "items");
		return super.addItem(newItem);
	}

	@Override
	public synchronized L2ItemInstance getItemByObj(int objectId, int count)
	{
		if(count < 0)
		{
			_log.warning("getItem  warhouse: count < 0 owner:" + _owner.getName());
			return null;
		}
		L2ItemInstance newItem = super.getItemByObj(objectId, count);
		if(newItem != null)
			Log.add("PcWarehouse|" + _owner.getName() + "|get item|" + newItem.getItemId() + "|" + newItem.getCount() + "|" + newItem.getObjectId(), "items");
		return newItem;
	}

	@Override
	public synchronized L2ItemInstance destroyItemByObj(int objectId, int count)
	{
		if(count < 0)
		{
			_log.warning("destroyItem  warhouse: count < 0 owner:" + _owner.getName());
			return null;
		}
		L2ItemInstance destroyItem = super.destroyItem(objectId, count);
		if(destroyItem != null)
			Log.add("PcWarehouse|" + _owner.getName() + "|destroy item|" + destroyItem.getItemId() + "|" + destroyItem.getCount() + "|" + destroyItem.getObjectId(), "items");
		return destroyItem;
	}

	@Override
	public synchronized L2ItemInstance destroyItem(int itemId, int count)
	{
		if(count < 0)
		{
			_log.warning("destroyItem  warhouse: count < 0 owner:" + _owner.getName());
			return null;
		}
		L2ItemInstance destroyItem = super.destroyItem(itemId, count);
		if(destroyItem != null)
			Log.add("PcWarehouse|" + _owner.getName() + "|destroy item|" + destroyItem.getItemId() + "|" + destroyItem.getCount() + "|" + destroyItem.getObjectId(), "items");
		return destroyItem;

	}
}