package L2jFT.game.model;

import L2jFT.game.network.serverpackets.StatusUpdate;
import L2jFT.game.network.serverpackets.UserInfo;

@SuppressWarnings( { "nls", "unqualified-field-access", "boxing" })
public class StatsChangeRecorder
{
	private L2Player _activeChar;
	private int _accuracy;
	private int _attackSpeed;
	private int _castSpeed;
	private int _criticalHit;
	private int _evasion;
	private int _magicAttack;
	private int _magicDefence;
	private int _maxCp;
	private int _maxHp;
	private int _maxLoad;
	private int _curLoad;
	private int _maxMp;
	private int _physicAttack;
	private int _physicDefence;

	private int _level;
	private long _exp;
	private int _sp;
	private int _karma;
	private int _pk;
	private int _pvp;

	private int _runSpeed;
	private int _abnormalEffects;

	public StatsChangeRecorder(L2Player activeChar)
	{
		_activeChar = activeChar;
		refreshSaves();
	}

	public void refreshSaves()
	{
		if(_activeChar == null)
			return;

		_accuracy = _activeChar.getAccuracy();
		_attackSpeed = _activeChar.getPAtkSpd();
		_castSpeed = _activeChar.getMAtkSpd();
		_criticalHit = _activeChar.getCriticalHit(null, null);
		_evasion = _activeChar.getEvasionRate(null);
		_magicAttack = _activeChar.getMAtk(null, null);
		_magicDefence = _activeChar.getMDef(null, null);
		_maxCp = _activeChar.getMaxCp();
		_maxHp = _activeChar.getMaxHp();
		_maxLoad = _activeChar.getMaxLoad();
		_curLoad = _activeChar.getCurrentLoad();
		_maxMp = _activeChar.getMaxMp();
		_physicAttack = _activeChar.getPAtk(null);
		_physicDefence = _activeChar.getPDef(null);

		_level = _activeChar.getLevel();
		_exp = _activeChar.getExp();
		_sp = _activeChar.getSp();
		_karma = _activeChar.getKarma();

		_runSpeed = _activeChar.getRunSpeed();
		_pk = _activeChar.getPkKills();
		_pvp = _activeChar.getPvpKills();
		_abnormalEffects = _activeChar.getAbnormalEffect(); //TODO: почему-то мне кажется что его можно отослать отдельным пакетом
	}

	public void sendChanges()
	{
		if(_activeChar == null)
			return;

		// Броадкаст UserInfo и charInfo();
		if(needsUserInfoBroadcast())
		{
			_activeChar.broadcastUserInfo();
			return;
		}

		sendGlobalInfo();
		sendPartyInfo();
		sendSelfInfo();

		refreshSaves();
	}

	/**
	 * Отправляет броадкастом инфу всем игрокам
	 */
	private void sendGlobalInfo()
	{
		StatusUpdate globalUpdate = new StatusUpdate(_activeChar.getObjectId());
		/**if(_attackSpeed != _activeChar.getPAtkSpd())
			globalUpdate.addAttribute(StatusUpdate.ATK_SPD, _activeChar.getPAtkSpd());

		if(_castSpeed != _activeChar.getMAtkSpd())
			globalUpdate.addAttribute(StatusUpdate.CAST_SPD, _activeChar.getMAtkSpd());
		 */

		if(_karma != _activeChar.getKarma())
			globalUpdate.addAttribute(StatusUpdate.KARMA, _activeChar.getKarma());

		if(globalUpdate.hasAttributes())
			_activeChar.broadcastPacket(globalUpdate);
	}

	/**
	 * Отправляет инфу парти игрока.
	 * Если парти нет, то отправляет лично игроку
	 */
	private void sendPartyInfo()
	{
		// Эти статы нужно рассылать только для партии игрока
		StatusUpdate partyUpdate = new StatusUpdate(_activeChar.getObjectId());
		if(_maxCp != _activeChar.getMaxCp())
			partyUpdate.addAttribute(StatusUpdate.MAX_CP, _activeChar.getMaxCp());

		if(_maxHp != _activeChar.getMaxHp())
			partyUpdate.addAttribute(StatusUpdate.MAX_HP, _activeChar.getMaxHp());

		if(_maxMp != _activeChar.getMaxMp())
			partyUpdate.addAttribute(StatusUpdate.MAX_MP, _activeChar.getMaxMp());

		L2Party party = _activeChar.getParty();
		if(partyUpdate.hasAttributes())
			if(party != null)
				party.broadcastToPartyMembers(partyUpdate);
			else
				_activeChar.sendPacket(partyUpdate);
	}

	/**
	 * Отправляет инфу только игроку
	 */
	private void sendSelfInfo()
	{
		// Количество exp, sp, pk и левел - характеристики о которых другие игроки не обязаны знать
		if(_pk != _activeChar.getPkKills() || _pvp != _activeChar.getPvpKills() || _exp != _activeChar.getExp() || _sp != _activeChar.getSp())
		{
			_activeChar.sendPacket(new UserInfo(_activeChar));
			return;
		}

		/*StatusUpdate personalUpdate = new StatusUpdate(_activeChar.getObjectId());
		if(_accuracy != _activeChar.getAccuracy())
			personalUpdate.addAttribute(StatusUpdate.ACCURACY, _activeChar.getAccuracy());

		if(_criticalHit != _activeChar.getCriticalHit(null, null))
			personalUpdate.addAttribute(StatusUpdate.CRITICAL, _activeChar.getCriticalHit(null, null));

		if(_evasion != _activeChar.getEvasionRate(null))
			personalUpdate.addAttribute(StatusUpdate.EVASION, _activeChar.getEvasionRate(null));

		if(_magicAttack != _activeChar.getMAtk(null, null))
			personalUpdate.addAttribute(StatusUpdate.M_ATK, _activeChar.getMAtk(null, null));

		if(_magicDefence != _activeChar.getMDef(null, null))
			personalUpdate.addAttribute(StatusUpdate.M_DEF, _activeChar.getMDef(null, null));

		if(_maxLoad != _activeChar.getMaxLoad())
			personalUpdate.addAttribute(StatusUpdate.MAX_LOAD, _activeChar.getMaxLoad());

		if(_curLoad != _activeChar.getCurrentLoad())
			personalUpdate.addAttribute(StatusUpdate.CUR_LOAD, _activeChar.getCurrentLoad());

		if(_physicAttack != _activeChar.getPAtk(null))
			personalUpdate.addAttribute(StatusUpdate.P_ATK, _activeChar.getPAtk(null));

		if(_physicDefence != _activeChar.getPDef(null))
			personalUpdate.addAttribute(StatusUpdate.P_DEF, _activeChar.getPDef(null));

		if(_level != _activeChar.getLevel())
			personalUpdate.addAttribute(StatusUpdate.LEVEL, _activeChar.getLevel());

		if(_exp != _activeChar.getExp())
				 personalUpdate.addAttribute(StatusUpdate.EXP, _activeChar.getExp());

		  if(_sp != _activeChar.getSp())
			 personalUpdate.addAttribute(StatusUpdate.SP, _activeChar.getSp());

		if(personalUpdate.hasAttributes())
			_activeChar.sendPacket(personalUpdate);
		*/

		

		// Тупой корейский мосх решил что отсылать UserInfo выгоднее... В таком случае делаем по тупому :(
		if(_accuracy != _activeChar.getAccuracy())
		{
			_activeChar.sendPacket(new UserInfo(_activeChar));
			return;
		}

		if(_criticalHit != _activeChar.getCriticalHit(null, null))
		{
			_activeChar.sendPacket(new UserInfo(_activeChar));
			return;
		}

		if(_evasion != _activeChar.getEvasionRate(null))
		{
			_activeChar.sendPacket(new UserInfo(_activeChar));
			return;
		}

		if(_magicAttack != _activeChar.getMAtk(null, null))
		{
			_activeChar.sendPacket(new UserInfo(_activeChar));
			return;
		}

		if(_magicDefence != _activeChar.getMDef(null, null))
		{
			_activeChar.sendPacket(new UserInfo(_activeChar));
			return;
		}

		if(_maxLoad != _activeChar.getMaxLoad())
		{
			_activeChar.sendPacket(new UserInfo(_activeChar));
			return;
		}

		if(_curLoad != _activeChar.getCurrentLoad())
		{
			_activeChar.sendPacket(new UserInfo(_activeChar));
			return;
		}

		if(_physicAttack != _activeChar.getPAtk(null))
		{
			_activeChar.sendPacket(new UserInfo(_activeChar));
			return;
		}

		if(_physicDefence != _activeChar.getPDef(null))
		{
			_activeChar.sendPacket(new UserInfo(_activeChar));
			return;
		}

		if(_level != _activeChar.getLevel())
		{
			_activeChar.sendPacket(new UserInfo(_activeChar));
		}
	}

	/**
	 * Проверяет нужно ли делать UserInfo broadcast. Дорогостоящая опирация. 
	 * @return тру ели нужно.
	 */
	private boolean needsUserInfoBroadcast()
	{
		return _runSpeed != _activeChar.getRunSpeed() ||
			   _abnormalEffects != _activeChar.getAbnormalEffect() ||
			   _attackSpeed != _activeChar.getPAtkSpd() ||
			   _castSpeed != _activeChar.getMAtkSpd();
	}
}