package L2jFT.game.model.entity;

import javolution.util.FastList;
import javolution.util.FastMap;
import L2jFT.database.DatabaseUtils;
import L2jFT.database.FiltredPreparedStatement;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.database.ThreadConnection;
import L2jFT.game.instancemanager.CastleManager;
import L2jFT.game.instancemanager.CastleManorManager;
import L2jFT.game.instancemanager.SiegeGuardManager;
import L2jFT.game.instancemanager.SiegeManager;
import L2jFT.game.instancemanager.ZoneManager;
import L2jFT.game.instancemanager.CastleManorManager.CropProcure;
import L2jFT.game.instancemanager.CastleManorManager.SeedProduction;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2ClanMember;
import L2jFT.game.model.L2Manor;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;
import L2jFT.game.model.L2Zone;
import L2jFT.game.model.L2Zone.ZoneType;
import L2jFT.game.model.Warehouse;
import L2jFT.game.model.entity.siege.Siege;
import L2jFT.game.model.instances.L2DoorInstance;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.network.serverpackets.PledgeShowInfoUpdate;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.ClanTable;
import L2jFT.game.tables.DoorTable;
import L2jFT.game.tables.ItemTable;
import L2jFT.game.tables.MapRegionTable;

import java.sql.ResultSet;
import java.util.List;
import java.util.logging.Logger;

public class Castle
{
	protected static Logger _log = Logger.getLogger(Castle.class.getName());

	private FastList<CropProcure> _procure = new FastList<CropProcure>();
	private FastList<SeedProduction> _production = new FastList<SeedProduction>();
	private FastList<CropProcure> _procureNext = new FastList<CropProcure>();
	private FastList<SeedProduction> _productionNext = new FastList<SeedProduction>();
	private boolean _isNextPeriodApproved = false;

	private static final String CASTLE_MANOR_DELETE_PRODUCTION = "DELETE FROM castle_manor_production WHERE castle_id=?;";
	private static final String CASTLE_MANOR_DELETE_PRODUCTION_PERIOD = "DELETE FROM castle_manor_production WHERE castle_id=? AND period=?;";
	private static final String CASTLE_MANOR_DELETE_PROCURE = "DELETE FROM castle_manor_procure WHERE castle_id=?;";
	private static final String CASTLE_MANOR_DELETE_PROCURE_PERIOD = "DELETE FROM castle_manor_procure WHERE castle_id=? AND period=?;";
	private static final String CASTLE_UPDATE_CROP = "UPDATE castle_manor_procure SET can_buy=? WHERE crop_id=? AND castle_id=? AND period=?";
	private static final String CASTLE_UPDATE_SEED = "UPDATE castle_manor_production SET can_produce=? WHERE seed_id=? AND castle_id=? AND period=?";

	private int _CastleId = 0;
	private List<L2DoorInstance> _Doors = new FastList<L2DoorInstance>();
	private String _Name = "";
	private int _OwnerId = 0;
	private Siege _Siege = null;
	private int _SiegeDayOfWeek = 7; // Default to saturday
	private int _SiegeHourOfDay = 20; // Default to 8 pm server time
	private int _TaxPercent = 0;
	private double _TaxRate = 0;
	private int _treasury = 0;
	private int _TownId = 0;
	private L2Zone _Zone;

	private FastMap<Integer, Integer> _engrave = new FastMap<Integer, Integer>();

	public void Engrave(L2Clan clan, int objId)
	{
		_engrave.put(objId, clan.getClanId());
		if(_engrave.size() == getArtefactCount())
		{
			boolean rst = true;
			for(int id : _engrave.values())
				if(id != clan.getClanId())
					rst = false;
			if(rst)
			{
				_engrave.clear();
				setOwner(clan);
			}
		}
	}

	public int getArtefactCount()
	{
		return SiegeManager.getInstance().getArtefactSpawnList(_CastleId).size();
	}

	private int _collectedShops;
	private int _collectedSeed;

	public int getCollectedShops()
	{
		return _collectedShops;
	}

	public int getCollectedSeed()
	{
		return _collectedSeed;
	}

	public void setCollectedShops(int value)
	{
		_collectedShops = value;
	}

	public void setCollectedSeed(int value)
	{
		_collectedSeed = value;
	}

	public Castle(int castleId)
	{
		_CastleId = castleId;

		loadData();
		loadDoor();
	}

	// This method add to the treasury
	/** Add amount to castle instance's treasury (warehouse). */
	public void addToTreasury(int amount, boolean shop, boolean seed)
	{
		if(getOwnerId() <= 0)
			return;

		if(amount > 1 && !_Name.equalsIgnoreCase("aden")) // If current castle instance is not Aden
		{
			Castle aden = CastleManager.getInstance().getCastleByName("aden");
			if(aden != null)
			{
				int adenTax = (int) (amount * aden.getTaxRate()); // Find out what Aden gets from the current castle instance's income
				if(aden.getOwnerId() > 0)
					aden.addToTreasury(adenTax, shop, seed); // Only bother to really add the tax to the treasury if not npc owned

				amount -= adenTax; // Subtract Aden's income from current castle instance's income
			}
		}

		addToTreasuryNoTax(amount, shop, seed);
	}

	/** Add amount to castle instance's treasury (warehouse), no tax paying. */
	public void addToTreasuryNoTax(int amount, boolean shop, boolean seed)
	{
		if(getOwnerId() <= 0)
			return;

		if(_treasury + amount >= Integer.MAX_VALUE)
			return;

		_treasury += amount; // Add to the current treasury total.  Use "-" to substract from treasury

		if(shop)
			_collectedShops += amount;

		if(seed)
			_collectedSeed += amount;

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;

		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("Update castle set treasury = ? where id = ?");
			statement.setInt(1, getTreasury());
			statement.setInt(2, getCastleId());
			statement.execute();
			statement.close();

			DatabaseUtils.closeDatabaseCS(con, statement);
		}
		catch(Exception e)
		{
			System.out.println("Exception: addToTreasuryNoTax(): " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	/** Move non clan members off castle area and to nearest town. */
	public void banishForeigner(L2Player activeChar)
	{
		// Get all players
		for(L2Player player : L2World.getAroundPlayers(activeChar))
		{
			// Skip if player is in clan
			if(player.getClanId() == getOwnerId())
				continue;

			if(checkIfInZone(player))
				player.teleToLocation(MapRegionTable.getInstance().getTeleToLocation(player, MapRegionTable.TeleportWhereType.ClosestTown));
		}
	}

	/** Return true if object is inside the zone */
	public boolean checkIfInZone(L2Object obj)
	{
		return checkIfInZone(obj.getX(), obj.getY());
	}

	/** Return true if object is inside the zone */
	public boolean checkIfInZone(int x, int y)
	{
		return getZone().checkIfInZone(x, y);
	}

	public void closeDoor(L2Player activeChar, int doorId)
	{
		openCloseDoor(activeChar, doorId, false);
	}

	public void openDoor(L2Player activeChar, int doorId)
	{
		openCloseDoor(activeChar, doorId, true);
	}

	public void openCloseDoor(L2Player activeChar, int doorId, boolean open)
	{
		if(activeChar.getClanId() != getOwnerId())
			return;

		L2DoorInstance door = getDoor(doorId);
		if(door != null)
			if(open)
				door.openMe();
			else
				door.closeMe();
	}

	// This method is used to begin removing all castle upgrades
	public void removeUpgrade()
	{
		removeDoorUpgrade();
	}

	// This method sets the castle owner; null here means give it back to NPC
	public void setOwner(L2Clan clan)
	{
		if(getOwnerId() > 0 && (clan == null || clan.getClanId() != getOwnerId()))
		{
			setTaxPercent(null, 0);
			L2Clan oldOwner = ClanTable.getInstance().getClan(getOwnerId()); // Try to find clan instance 
			if(oldOwner != null)
			{
				int amount = getTreasury();
				if(amount > 0)
				{
					Warehouse warehouse = oldOwner.getWarehouse();
					if(warehouse != null)
					{
						L2ItemInstance item = ItemTable.getInstance().createItem((short) 57, warehouse.getOwnerId(), getCastleId(), "setOwner");
						item.setCount(amount);
						warehouse.addItem(item);
						addToTreasuryNoTax(-amount, false, false);
					}
				}
				if(!getSiege().isInProgress())
				{
					SiegeGuardManager sgm = new SiegeGuardManager(this);
					sgm.removeMercs();
					removeUpgrade();
				}
				oldOwner.setHasCastle(0); // Unset has castle flag for old owner
			}
		}

		if(clan != null)
		{
			clan.setHasCastle(getCastleId()); // Set has castle flag for new owner
			Siege.announceToPlayer(new SystemMessage(SystemMessage.THE_TEMPORARY_ALLIANCE_OF_THE_CASTLE_ATTACKER_TEAM_HAS_BEEN_DISSOLVED), false);
		}

		updateOwnerInDB(clan); // Update in database

		if(clan != null && getSiege().isInProgress()) // If siege in progress
			getSiege().midVictory(); // Mid victory phase of siege
	}

	// This method updates the castle tax rate
	public void setTaxPercent(L2Player activeChar, int taxPercent)
	{
		_TaxPercent = taxPercent;
		_TaxRate = _TaxPercent / 100.0;

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("UPDATE castle SET taxPercent = ? WHERE id = ? LIMIT 1");
			statement.setInt(1, taxPercent);
			statement.setInt(2, getCastleId());
			statement.execute();
		}
		catch(Exception e)
		{}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}

		if(activeChar != null)
			activeChar.sendMessage(getName() + " castle tax changed to " + taxPercent + "%.", "Налог замка " + getName() + " теперь равен " + taxPercent + "%.");
	}

	/** Respawn all doors on castle grounds */
	public void spawnDoor()
	{
		spawnDoor(false);
	}

	/** Respawn all doors on castle grounds */
	public void spawnDoor(boolean isDoorWeak)
	{
		for(int i = 0; i < getDoors().size(); i++)
		{
			L2DoorInstance door = getDoors().get(i);
			if(door.getCurrentHp() <= 0)
			{
				door.decayMe(); // Kill current if not killed already
				if(isDoorWeak)
					door.setCurrentHp(door.getMaxHp() / 2);
				else
					door.setCurrentHp(door.getMaxHp());
				door.spawnMe(door.getX(), door.getY(), door.getZ());
				getDoors().set(i, door);
			}
			else if(door.isOpen())
				door.closeMe();
		}
		loadDoorUpgrade(); // Check for any upgrade the doors may have
	}

	FastMap<Integer, Integer> doorUpgrades = new FastMap<Integer, Integer>();

	// This method upgrade door
	public void upgradeDoor(int doorId, int hp, boolean db)
	{
		L2DoorInstance door = getDoor(doorId);
		if(door == null)
			return;

		if(door.getDoorId() == doorId)
		{
			door.setUpgradeHp(hp);
			door.setCurrentHp(door.getMaxHp());
			door.broadcastStatusUpdate();

			if(db)
				saveDoorUpgrade(doorId, hp);
			doorUpgrades.put(doorId, hp);
		}
	}

	public Integer getDoorUpgrade(int doorId)
	{
		if(doorUpgrades.get(doorId) != null)
			return doorUpgrades.get(doorId);
		return 0;
	}

	// =========================================================
	// Method - Private
	// This method loads castle
	private void loadData()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;

		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			statement = con.prepareStatement("SELECT * FROM castle WHERE id=? LIMIT 1");
			statement.setInt(1, getCastleId());
			rset = statement.executeQuery();
			while(rset.next())
			{
				_Name = rset.getString("name");

				_SiegeDayOfWeek = rset.getInt("siegeDayOfWeek");
				if(_SiegeDayOfWeek < 1 || _SiegeDayOfWeek > 7)
					_SiegeDayOfWeek = 7;

				_SiegeHourOfDay = rset.getInt("siegeHourOfDay");
				if(_SiegeHourOfDay < 0 || _SiegeHourOfDay > 23)
					_SiegeHourOfDay = 20;

				_TaxPercent = rset.getInt("taxPercent");
				_treasury = rset.getInt("treasury");
				_TownId = rset.getInt("townId");
				getSiege().setSiegeDateTime(rset.getLong("siegeDate") * 1000);
			}
			DatabaseUtils.closeDatabaseSR(statement, rset);

			_TaxRate = _TaxPercent / 100.0;

			statement = con.prepareStatement("Select clan_id from clan_data where hasCastle = ? LIMIT 1");
			statement.setInt(1, getCastleId());
			rset = statement.executeQuery();

			if(rset.next())
				_OwnerId = rset.getInt("clan_id");

			_Zone = ZoneManager.getInstance().getZoneByIndex(ZoneType.Castle, getCastleId(), true);
		}
		catch(Exception e)
		{
			System.out.println("Exception: loadCastleData(): " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	// This method loads castle door data from database
	private void loadDoor()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("Select id from castle_door where castleId = ?");
			statement.setInt(1, getCastleId());
			rset = statement.executeQuery();

			while(rset.next())
			{
				L2DoorInstance door = DoorTable.getInstance().getDoor(rset.getInt("id"));
				door.spawnMe(door.getX(), door.getY(), door.getZ());
				_Doors.add(door);
			}
		}
		catch(Exception e)
		{
			System.out.println("Exception: loadCastleDoor(): " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	// This method loads castle door upgrade data from database
	private void loadDoorUpgrade()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("Select * from castle_doorupgrade where doorId in (Select Id from castle_door where castleId = ?)");
			statement.setInt(1, getCastleId());
			rset = statement.executeQuery();

			while(rset.next())
				upgradeDoor(rset.getInt("id"), rset.getInt("hp"), false);
		}
		catch(Exception e)
		{
			System.out.println("Exception: loadCastleDoorUpgrade(): " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	private void removeDoorUpgrade()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("delete from castle_doorupgrade where doorId in (select id from castle_door where castleId=?)");
			statement.setInt(1, getCastleId());
			statement.execute();
		}
		catch(Exception e)
		{
			System.out.println("Exception: removeDoorUpgrade(): " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	private void saveDoorUpgrade(int doorId, int hp)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("REPLACE INTO castle_doorupgrade (doorId, hp) values (?,?)");
			statement.setInt(1, doorId);
			statement.setInt(2, hp);
			statement.execute();
		}
		catch(Exception e)
		{
			System.out.println("Exception: saveDoorUpgrade(int doorId, int hp): " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	private void updateOwnerInDB(L2Clan clan)
	{
		_OwnerId = clan == null ? 0 : clan.getClanId(); // Update owner id property

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("UPDATE clan_data SET hasCastle=0 WHERE hasCastle=? LIMIT 1");
			statement.setInt(1, getCastleId());
			statement.execute();
			DatabaseUtils.closeStatement(statement);
			statement = null;

			if(clan != null)
			{
				statement = con.prepareStatement("UPDATE clan_data SET hasCastle=? WHERE clan_id=? LIMIT 1");
				statement.setInt(1, getCastleId());
				statement.setInt(2, getOwnerId());
				statement.execute();

				for(L2ClanMember member : clan.getMembers())
					if(member.isOnline())
						member.getPlayer().sendPacket(new PledgeShowInfoUpdate(clan));
			}
		}
		catch(Exception e)
		{
			System.out.println("Exception: updateOwnerInDB(L2Clan clan): " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	// =========================================================
	// Proeprty
	public final int getCastleId()
	{
		return _CastleId;
	}

	public final L2DoorInstance getDoor(int doorId)
	{
		if(doorId <= 0)
			return null;

		for(int i = 0; i < getDoors().size(); i++)
		{
			L2DoorInstance door = getDoors().get(i);
			if(door.getDoorId() == doorId)
				return door;
		}
		return null;
	}

	public final List<L2DoorInstance> getDoors()
	{
		return _Doors;
	}

	public final String getName()
	{
		return _Name;
	}

	public final int getOwnerId()
	{
		return _OwnerId;
	}

	public final Siege getSiege()
	{
		if(_Siege == null)
			_Siege = new Siege(this);
		return _Siege;
	}

	public final int getSiegeDayOfWeek()
	{
		return _SiegeDayOfWeek;
	}

	public final int getSiegeHourOfDay()
	{
		return _SiegeHourOfDay;
	}

	public final int getTaxPercent()
	{
		// Если печатью SEAL_STRIFE владеют DUSK то налог можно выставлять не более 5%
		if(_TaxPercent > 5 && SevenSigns.getInstance().getSealOwner(SevenSigns.SEAL_STRIFE) == SevenSigns.CABAL_DUSK)
			_TaxPercent = 5;
		return _TaxPercent;
	}

	public final double getTaxRate()
	{
		// Если печатью SEAL_STRIFE владеют DUSK то налог можно выставлять не более 5%
		if(_TaxRate > 0.05 && SevenSigns.getInstance().getSealOwner(SevenSigns.SEAL_STRIFE) == SevenSigns.CABAL_DUSK)
			_TaxRate = 0.05;
		return _TaxRate;
	}

	public final int getTreasury()
	{
		return _treasury;
	}

	/*
	 * Возвращает ID города, за которым закреплен замок
	 */
	public final int getTown()
	{
		return _TownId;
	}

	public final L2Zone getZone()
	{
		return _Zone;
	}

	public FastList<SeedProduction> getSeedProduction(int period)
	{
		return period == CastleManorManager.PERIOD_CURRENT ? _production : _productionNext;
	}

	public FastList<CropProcure> getCropProcure(int period)
	{
		return period == CastleManorManager.PERIOD_CURRENT ? _procure : _procureNext;
	}

	public void setSeedProduction(FastList<SeedProduction> seed, int period)
	{
		if(period == CastleManorManager.PERIOD_CURRENT)
			_production = seed;
		else
			_productionNext = seed;
	}

	public void setCropProcure(FastList<CropProcure> crop, int period)
	{
		if(period == CastleManorManager.PERIOD_CURRENT)
			_procure = crop;
		else
			_procureNext = crop;
	}

	public synchronized SeedProduction getSeed(int seedId, int period)
	{
		for(SeedProduction seed : getSeedProduction(period))
			if(seed.getId() == seedId)
				return seed;
		return null;
	}

	public synchronized CropProcure getCrop(int cropId, int period)
	{
		for(CropProcure crop : getCropProcure(period))
			if(crop.getId() == cropId)
				return crop;
		return null;
	}

	public int getManorCost(int period)
	{
		FastList<CropProcure> procure;
		FastList<SeedProduction> production;

		if(period == CastleManorManager.PERIOD_CURRENT)
		{
			procure = _procure;
			production = _production;
		}
		else
		{
			procure = _procureNext;
			production = _productionNext;
		}

		int total = 0;
		if(production != null)
			for(SeedProduction seed : production)
				total += L2Manor.getInstance().getSeedBuyPrice(seed.getId()) * seed.getStartProduce();
		if(procure != null)
			for(CropProcure crop : procure)
				total += crop.getPrice() * crop.getStartAmount();
		return total;
	}

	//save manor production data
	public void saveSeedData()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;

		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			statement = con.prepareStatement(CASTLE_MANOR_DELETE_PRODUCTION);
			statement.setInt(1, getCastleId());

			statement.execute();
			statement.close();

			if(_production != null)
			{
				int count = 0;
				String query = "INSERT INTO castle_manor_production VALUES ";
				String values[] = new String[_production.size()];
				for(SeedProduction s : _production)
				{
					values[count] = "(" + getCastleId() + "," + s.getId() + "," + s.getCanProduce() + "," + s.getStartProduce() + "," + s.getPrice() + "," + CastleManorManager.PERIOD_CURRENT + ")";
					count++;
				}
				if(values.length > 0)
				{
					query += values[0];
					for(int i = 1; i < values.length; i++)
						query += "," + values[i];
					statement = con.prepareStatement(query);
					statement.execute();
					statement.close();
				}
			}

			if(_productionNext != null)
			{
				int count = 0;
				String query = "INSERT INTO castle_manor_production VALUES ";
				String values[] = new String[_productionNext.size()];
				for(SeedProduction s : _productionNext)
				{
					values[count] = "(" + getCastleId() + "," + s.getId() + "," + s.getCanProduce() + "," + s.getStartProduce() + "," + s.getPrice() + "," + CastleManorManager.PERIOD_NEXT + ")";
					count++;
				}
				if(values.length > 0)
				{
					query += values[0];
					for(int i = 1; i < values.length; i++)
						query += "," + values[i];
					statement = con.prepareStatement(query);
					statement.execute();
					statement.close();
				}
			}

			DatabaseUtils.closeDatabaseCS(con, statement);
		}
		catch(Exception e)
		{
			_log.info("Error adding seed production data for castle " + getName() + ": " + e.getMessage());
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	//save manor production data for specified period
	public void saveSeedData(int period)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			statement = con.prepareStatement(CASTLE_MANOR_DELETE_PRODUCTION_PERIOD);
			statement.setInt(1, getCastleId());
			statement.setInt(2, period);
			statement.execute();
			statement.close();

			FastList<SeedProduction> prod = null;
			prod = getSeedProduction(period);

			if(prod != null)
			{
				int count = 0;
				String query = "INSERT INTO castle_manor_production VALUES ";
				String values[] = new String[prod.size()];
				for(SeedProduction s : prod)
				{
					values[count] = "(" + getCastleId() + "," + s.getId() + "," + s.getCanProduce() + "," + s.getStartProduce() + "," + s.getPrice() + "," + period + ")";
					count++;
				}
				if(values.length > 0)
				{
					query += values[0];
					for(int i = 1; i < values.length; i++)
						query += "," + values[i];
					statement = con.prepareStatement(query);
					statement.execute();
					statement.close();
				}
			}

			DatabaseUtils.closeDatabaseCS(con, statement);
		}
		catch(Exception e)
		{
			_log.info("Error adding seed production data for castle " + getName() + ": " + e.getMessage());
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	//save crop procure data
	public void saveCropData()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			statement = con.prepareStatement(CASTLE_MANOR_DELETE_PROCURE);
			statement.setInt(1, getCastleId());
			statement.execute();
			statement.close();
			if(_procure != null)
			{
				int count = 0;
				String query = "INSERT INTO castle_manor_procure VALUES ";
				String values[] = new String[_procure.size()];
				for(CropProcure cp : _procure)
				{
					values[count] = "(" + getCastleId() + "," + cp.getId() + "," + cp.getAmount() + "," + cp.getStartAmount() + "," + cp.getPrice() + "," + cp.getReward() + "," + CastleManorManager.PERIOD_CURRENT + ")";
					count++;
				}
				if(values.length > 0)
				{
					query += values[0];
					for(int i = 1; i < values.length; i++)
						query += "," + values[i];
					statement = con.prepareStatement(query);
					statement.execute();
					statement.close();
				}
			}
			if(_procureNext != null)
			{
				int count = 0;
				String query = "INSERT INTO castle_manor_procure VALUES ";
				String values[] = new String[_procureNext.size()];
				for(CropProcure cp : _procureNext)
				{
					values[count] = "(" + getCastleId() + "," + cp.getId() + "," + cp.getAmount() + "," + cp.getStartAmount() + "," + cp.getPrice() + "," + cp.getReward() + "," + CastleManorManager.PERIOD_NEXT + ")";
					count++;
				}
				if(values.length > 0)
				{
					query += values[0];
					for(int i = 1; i < values.length; i++)
						query += "," + values[i];
					statement = con.prepareStatement(query);
					statement.execute();
					statement.close();
				}
			}

			DatabaseUtils.closeDatabaseCS(con, statement);
		}
		catch(Exception e)
		{
			_log.info("Error adding crop data for castle " + getName() + ": " + e.getMessage());
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	//	save crop procure data for specified period
	public void saveCropData(int period)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			statement = con.prepareStatement(CASTLE_MANOR_DELETE_PROCURE_PERIOD);
			statement.setInt(1, getCastleId());
			statement.setInt(2, period);
			statement.execute();
			statement.close();

			FastList<CropProcure> proc = null;
			proc = getCropProcure(period);

			if(proc != null)
			{
				int count = 0;
				String query = "INSERT INTO castle_manor_procure VALUES ";
				String values[] = new String[proc.size()];

				for(CropProcure cp : proc)
				{
					values[count] = "(" + getCastleId() + "," + cp.getId() + "," + cp.getAmount() + "," + cp.getStartAmount() + "," + cp.getPrice() + "," + cp.getReward() + "," + period + ")";
					count++;
				}
				if(values.length > 0)
				{
					query += values[0];
					for(int i = 1; i < values.length; i++)
						query += "," + values[i];
					statement = con.prepareStatement(query);
					statement.execute();
					statement.close();
				}
			}

			DatabaseUtils.closeDatabaseCS(con, statement);
		}
		catch(Exception e)
		{
			_log.info("Error adding crop data for castle " + getName() + ": " + e.getMessage());
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	public void updateCrop(int cropId, int amount, int period)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			statement = con.prepareStatement(CASTLE_UPDATE_CROP);
			statement.setInt(1, amount);
			statement.setInt(2, cropId);
			statement.setInt(3, getCastleId());
			statement.setInt(4, period);
			statement.execute();
			statement.close();
		}
		catch(Exception e)
		{
			_log.info("Error adding crop data for castle " + getName() + ": " + e.getMessage());
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	public void updateSeed(int seedId, int amount, int period)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			statement = con.prepareStatement(CASTLE_UPDATE_SEED);
			statement.setInt(1, amount);
			statement.setInt(2, seedId);
			statement.setInt(3, getCastleId());
			statement.setInt(4, period);
			statement.execute();
			statement.close();
		}
		catch(Exception e)
		{
			_log.info("Error adding seed production data for castle " + getName() + ": " + e.getMessage());
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	public boolean isNextPeriodApproved()
	{
		return _isNextPeriodApproved;
	}

	public void setNextPeriodApproved(boolean val)
	{
		_isNextPeriodApproved = val;
	}
}
