package L2jFT.game.model.entity;

import javolution.util.FastList;
import L2jFT.Config;
import L2jFT.game.instancemanager.DimensionalRiftManager;
import L2jFT.util.Rnd;
import L2jFT.game.model.L2Party;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.instances.L2NpcInstance;

import java.util.Timer;
import java.util.TimerTask;

public class DimensionalRift
{
	private byte _type;
	L2Party _party;
	FastList<Byte> _completedRooms = new FastList<Byte>();
	private long seconds_5 = 5000L;
	private final int MILLISECONDS_IN_MINUTE = 60000;
	byte jumps_current = 0;
	private Timer teleporter_timer;
	private TimerTask teleporter_timer_task;
	byte _choosenRoom = -1;
	private boolean _hasJumped = false;
	FastList<L2Player> deadPlayers = new FastList<L2Player>();
	FastList<L2Player> reviedInWaitingRoom = new FastList<L2Player>();
	private boolean isBossRoom = false;

	public DimensionalRift(L2Party party, byte type, byte room)
	{
		_type = type;
		_party = party;
		_choosenRoom = room;
		int[] coords = getRoomCoord(room);
		party.setDimensionalRift(this);
		for(L2Player p : party.getPartyMembers())
			p.teleToLocation(coords[0], coords[1], coords[2]);

		createTeleporterTimer(true);
	}

	void createTeleporterTimer(final boolean reasonTP)
	{
		if(teleporter_timer_task != null)
		{
			teleporter_timer_task.cancel();
			teleporter_timer_task = null;
		}

		if(teleporter_timer != null)
		{
			teleporter_timer.cancel();
			teleporter_timer = null;
		}

		teleporter_timer = new Timer();
		teleporter_timer_task = new TimerTask(){
			@Override
			public void run()
			{
				if(reasonTP && jumps_current < getMaxJumps() && _party.getMemberCount() > deadPlayers.size())
				{
					jumps_current++;

					_completedRooms.add(_choosenRoom);
					_choosenRoom = -1;

					for(L2Player p : _party.getPartyMembers())
						if(!reviedInWaitingRoom.contains(p))
							teleportToNextRoom(p);
					createTeleporterTimer(true);
				}
				else
				{
					for(L2Player p : _party.getPartyMembers())
						if(!reviedInWaitingRoom.contains(p))
							teleportToWaitingRoom(p);
					killRift();
					cancel();
				}
			}
		};
		if(reasonTP)
			teleporter_timer.schedule(teleporter_timer_task, calcTimeToNextJump()); //Teleporter task, 8-10 minutes
		else
			teleporter_timer.schedule(teleporter_timer_task, seconds_5); //incorrect party member invited.

	}

	public void partyMemberInvited()
	{
		createTeleporterTimer(false);
	}

	public void partyMemberExited(L2Player player)
	{
		if(deadPlayers.contains(player))
			deadPlayers.remove(player);

		if(reviedInWaitingRoom.contains(player))
			reviedInWaitingRoom.remove(player);

		if(_party.getMemberCount() < Config.RIFT_MIN_PARTY_SIZE || _party.getMemberCount() == 1)
		{
			for(L2Player p : _party.getPartyMembers())
				teleportToWaitingRoom(p);
			killRift();
		}
	}

	public void manualTeleport(L2Player player, L2NpcInstance npc)
	{
		if(!player.isInParty() || !player.getParty().isInDimensionalRift())
			return;

		if(player.getObjectId() != player.getParty().getPartyLeaderOID())
		{
			DimensionalRiftManager.getInstance().showHtmlFile(player, "data/html/rift/NotPartyLeader.htm", npc);
			return;
		}

		if(_hasJumped)
		{
			DimensionalRiftManager.getInstance().showHtmlFile(player, "data/html/rift/AllreadyTeleported.html", npc);
			return;
		}

		_hasJumped = true;

		_completedRooms.add(_choosenRoom);
		_choosenRoom = -1;

		for(L2Player p : _party.getPartyMembers())
			teleportToNextRoom(p);

		createTeleporterTimer(true);
	}

	public void manualExitRift(L2Player player, L2NpcInstance npc)
	{
		if(!player.isInParty() || !player.getParty().isInDimensionalRift())
			return;

		if(player.getObjectId() != player.getParty().getPartyLeaderOID())
		{
			DimensionalRiftManager.getInstance().showHtmlFile(player, "data/html/rift/NotPartyLeader.htm", npc);
			return;
		}

		for(L2Player p : player.getParty().getPartyMembers())
			teleportToWaitingRoom(p);
		killRift();
	}

	void teleportToNextRoom(L2Player player)
	{
		if(_choosenRoom == -1)
		{
			_choosenRoom = (byte) Rnd.get(1, 9);

			while(_completedRooms.contains(_choosenRoom))
				//Do not tp in the same room
				_choosenRoom = (byte) Rnd.get(1, 9);
		}

		checkBossRoom(_choosenRoom);
		int[] coords = getRoomCoord(_choosenRoom);
		player.teleToLocation(coords[0], coords[1], coords[2]);
	}

	void teleportToWaitingRoom(L2Player player)
	{
		DimensionalRiftManager.getInstance().teleportToWaitingRoom(player);
	}

	public void killRift()
	{
		_completedRooms = null;

		if(_party != null)
			_party.setDimensionalRift(null);

		_party = null;
		reviedInWaitingRoom = null;
		deadPlayers = null;
		DimensionalRiftManager.getInstance().killRift(this);
	}

	public Timer getTimer()
	{
		return teleporter_timer;
	}

	public TimerTask getTimerTask()
	{
		return teleporter_timer_task;
	}

	public void setTimer(Timer t)
	{
		teleporter_timer = t;
	}

	public void setTimerTask(TimerTask tt)
	{
		teleporter_timer_task = tt;
	}

	private long calcTimeToNextJump()
	{
		if(isBossRoom)
			return (long) ((Config.RIFT_AUTO_JUMPS_TIME * MILLISECONDS_IN_MINUTE + Rnd.get(Config.RIFT_AUTO_JUMPS_TIME_RAND)) * Config.RIFT_BOSS_ROOM_TIME_MUTIPLY);
		return Config.RIFT_AUTO_JUMPS_TIME * MILLISECONDS_IN_MINUTE + Rnd.get(Config.RIFT_AUTO_JUMPS_TIME_RAND);
	}

	public void memberDead(L2Player player)
	{
		if(!deadPlayers.contains(player))
			deadPlayers.add(player);
	}

	public void memberRessurected(L2Player player)
	{
		if(deadPlayers.contains(player))
			deadPlayers.remove(player);
	}

	public void usedTeleport(L2Player player)
	{
		if(!reviedInWaitingRoom.contains(player))
			reviedInWaitingRoom.add(player);

		if(!deadPlayers.contains(player))
			deadPlayers.add(player);

		if(_party.getMemberCount() - reviedInWaitingRoom.size() < Config.RIFT_MIN_PARTY_SIZE)
		{
			for(L2Player p : _party.getPartyMembers())
				if(!reviedInWaitingRoom.contains(p))
					teleportToWaitingRoom(p);
			killRift();
		}
	}

	public FastList<L2Player> getDeadMemberList()
	{
		return deadPlayers;
	}

	public FastList<L2Player> getRevivedAtWaitingRoom()
	{
		return reviedInWaitingRoom;
	}

	public void checkBossRoom(byte room)
	{
		isBossRoom = DimensionalRiftManager.getInstance().getRoom(_type, room).isBossRoom();
	}

	public int[] getRoomCoord(byte room)
	{
		return DimensionalRiftManager.getInstance().getRoom(_type, room).getTeleportCoords();
	}

	public byte getMaxJumps()
	{
		if(Config.RIFT_MAX_JUMPS <= 8 && Config.RIFT_MAX_JUMPS >= 1)
			return (byte) Config.RIFT_MAX_JUMPS;
		return 4;
	}
}
