package L2jFT.game.model.entity;

import javolution.util.FastList;
import javolution.util.FastMap;
import jsr166x.ConcurrentSkipListSet;
import L2jFT.Config;
import L2jFT.database.DatabaseUtils;
import L2jFT.database.FiltredPreparedStatement;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.database.ThreadConnection;
import L2jFT.game.Announcements;
import L2jFT.game.ThreadPoolManager;
import L2jFT.game.model.Inventory;
import L2jFT.game.model.L2Party;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.L2Summon;
import L2jFT.game.model.L2World;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.network.serverpackets.ExOlympiadUserInfoSpectator;
import L2jFT.game.network.serverpackets.InventoryUpdate;
import L2jFT.game.network.serverpackets.MagicSkillUse;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.SkillTable;
import L2jFT.game.templates.StatsSet;
import L2jFT.util.Rnd;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Calendar;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.ScheduledFuture;
import java.util.logging.Logger;

public class Olympiad
{
	public static enum BattleStatus
	{
		Begining,
		PortPlayers,
		Started,
		CastBuffs,
		CountDown,
		StartComp,
		ValidateWinner,
		Ending
	}

	private Map<Integer, OlympiadGameTask> _gamesQueue = new FastMap<Integer, OlympiadGameTask>();
	private Map<Integer, ScheduledFuture> _gamesQueueScheduled = new FastMap<Integer, ScheduledFuture>();

	public class OlympiadGameTask implements Runnable
	{
		private L2OlympiadGame _game;
		private BattleStatus _status;
		private int _count;

		private boolean _terminated = false;

		public boolean isTerminated()
		{
			return _terminated;
		}

		public BattleStatus getStatus()
		{
			return _status;
		}

		public int getCount()
		{
			return _count;
		}

		public OlympiadGameTask(L2OlympiadGame game, BattleStatus status)
		{
			_game = game;
			_status = status;
			_count = 0;
		}

		public OlympiadGameTask(L2OlympiadGame game, BattleStatus status, int count)
		{
			_game = game;
			_status = status;
			_count = count;
		}

		public void run()
		{
			OlympiadGameTask task = null;
			ScheduledFuture sf = null;
			//_started = true;
			if(!Olympiad.getInstance().inCompPeriod())
				return;
			if(_game != null)
				if(_game._aborted || (_game._playerOne != null && _game._playerTwo != null))
				{
					switch(_status)
					{
						case Begining:
						{
							_game.sendMessageToPlayers(false, 30);
							task = new OlympiadGameTask(_game, BattleStatus.PortPlayers);
							sf = ThreadPoolManager.getInstance().scheduleGeneral(task, 30000);
							break;
						}
						case PortPlayers:
						{
							_game.portPlayersToArena();
							_game.makePlayersInvisible();
							task = new OlympiadGameTask(_game, BattleStatus.Started);
							sf = ThreadPoolManager.getInstance().scheduleGeneral(task, 20000);
							break;
						}
						case Started:
						{
							_game.removals();

							synchronized (this)
							{
								//first battle of the perios set the battles started
								if(!_battleStarted)
									_battleStarted = true;
							}
							//Wait 1mins
							task = new OlympiadGameTask(_game, BattleStatus.CastBuffs, 30);
							sf = ThreadPoolManager.getInstance().scheduleGeneral(task, 30000);
							break;
						}
						case CastBuffs:
						{
							_game.sendMessageToPlayers(true, _count);
							if(_count == 20)
								_game.additions();
							_count -= 10;
							if(_count > 10)
							{
								task = new OlympiadGameTask(_game, BattleStatus.CastBuffs, _count);
								sf = ThreadPoolManager.getInstance().scheduleGeneral(task, 10000);
							}
							else
							{
								task = new OlympiadGameTask(_game, BattleStatus.CountDown, 5);
								sf = ThreadPoolManager.getInstance().scheduleGeneral(task, 5000);
							}
							break;
						}
						case CountDown:
						{
							_game.sendMessageToPlayers(true, _count);
							_count--;
							if(_count == 0)
								task = new OlympiadGameTask(_game, BattleStatus.StartComp, 36);
							else
								task = new OlympiadGameTask(_game, BattleStatus.CountDown, _count);
							sf = ThreadPoolManager.getInstance().scheduleGeneral(task, 1000);
							break;
						}
						case StartComp:
						{
							if(_count == 36)
							{
								_game.makePlayersVisible();
								_game.makeCompetitionStart();
							}
							//Wait 3 mins (Battle)
							if(_game._aborted || _game._playerOne == null || _game._playerTwo == null)
							{
								task = new OlympiadGameTask(_game, BattleStatus.ValidateWinner);
								sf = ThreadPoolManager.getInstance().scheduleGeneral(task, 1000);
							}
							else
							{
								_count--;
								if(_count == 0)
									task = new OlympiadGameTask(_game, BattleStatus.ValidateWinner);
								else
									task = new OlympiadGameTask(_game, BattleStatus.StartComp, _count);
								sf = ThreadPoolManager.getInstance().scheduleGeneral(task, 10000);
							}
							break;
						}
						case ValidateWinner:
						{
							try
							{
								_game.validateWinner();
							}
							catch(Exception e)
							{
								e.printStackTrace();
							}
							task = new OlympiadGameTask(_game, BattleStatus.Ending);
							sf = ThreadPoolManager.getInstance().scheduleGeneral(task, 20000);
							break;
						}
						case Ending:
						{
							_game.portPlayersBack();
							_game.clearSpectators();
							_terminated = true;
							task = this;
							break;
						}
					}
					_gamesQueue.put(_game.getId(), task);
					_gamesQueueScheduled.put(_game.getId(), sf);
				}
				else
				{
					_log.warning("OlympiadGameTask: players is null");
					_game._aborted = true;
					task = new OlympiadGameTask(_game, BattleStatus.ValidateWinner);
					sf = ThreadPoolManager.getInstance().scheduleGeneral(task, 1000);
					_gamesQueue.put(_game.getId(), task);
					_gamesQueueScheduled.put(_game.getId(), sf);
				}
		}
	}

	protected static final Logger _log = Logger.getLogger(Olympiad.class.getName());

	private static Olympiad _instance;

	protected static Map<Integer, StatsSet> _nobles;
	protected static List<StatsSet> _heroesToBe;
	protected static List<L2Player> _nonClassBasedRegisters;
	protected static Map<Short, List<L2Player>> _classBasedRegisters;

	private static final String OLYMPIAD_DATA_FILE = "config/functions/olympiad.ini";
	public static final String OLYMPIAD_HTML_FILE = "data/html/olympiad/";
	private static final String OLYMPIAD_LOAD_NOBLES = "SELECT * from olympiad_nobles";
	private static final String OLYMPIAD_SAVE_NOBLES = "INSERT INTO olympiad_nobles " + "values (?,?,?,?,?,?,?)";
	private static final String OLYMPIAD_UPDATE_NOBLES = "UPDATE olympiad_nobles set " + "olympiad_points = ?, competitions_done = ?, competitions_win = ?, competitions_loose = ? where char_id = ?";
	private static final String OLYMPIAD_GET_HEROS = "SELECT char_id, char_name from " + "olympiad_nobles where class_id = ? and competitions_done >= 9 order by " + "olympiad_points desc, competitions_done desc";
	private static final String GET_EACH_CLASS_LEADER = "SELECT char_name FROM " + "olympiad_nobles WHERE class_id = ? ORDER BY olympiad_points DESC, " + "competitions_done DESC";
	private static final String OLYMPIAD_DELETE_ALL = "DELETE from olympiad_nobles";

	private static final int COMP_START = Config.ALT_OLY_START_TIME; // 6PM - 10AM
	private static final int COMP_MIN = Config.ALT_OLY_MIN; // 00 mins
	private static final long COMP_PERIOD = Config.ALT_OLY_CPERIOD; // 6hours
	protected static final long BATTLE_PERIOD = Config.ALT_OLY_BATTLE; // 6mins
	protected static final long BATTLE_WAIT = Config.ALT_OLY_BWAIT; // 10mins
	protected static final long INITIAL_WAIT = Config.ALT_OLY_IWAIT; // 5mins
	protected static final long WEEKLY_PERIOD = Config.ALT_OLY_WPERIOD; // 1 week
	protected static final long VALIDATION_PERIOD = Config.ALT_OLY_VPERIOD; // 24 hours

	/* forTESTING
	 private static final int COMP_START = 8; // 1PM - 2PM
	 private static final int COMP_MIN = 15; // 20mins
	 private static final long COMP_PERIOD = 7200000; // 2hours
	 private static final long BATTLE_PERIOD = 180000; // 3mins
	 private static final long BATTLE_WAIT = 600000; // 10mins
	 private static final long INITIAL_WAIT = 300000;	// 5mins
	 private static final long WEEKLY_PERIOD = 7200000; // 2 hours
	 private static final long VALIDATION_PERIOD = 3600000; // 1 hour */

	private static final int DEFAULT_POINTS = 18;
	protected static final int WEEKLY_POINTS = 3;

	public static final String CHAR_ID = "char_id";
	public static final String CLASS_ID = "class_id";
	public static final String CHAR_NAME = "char_name";
	public static final String POINTS = "olympiad_points";
	public static final String COMP_DONE = "competitions_done";
	public static final String COMP_WIN = "competitions_win";
	public static final String COMP_LOOSE = "competitions_loose";

	protected long _olympiadEnd;
	protected long _validationEnd;
	protected int _period;
	protected long _nextWeeklyChange;
	protected int _currentCycle;
	private long _compEnd;
	private Calendar _compStart;
	protected static boolean _inCompPeriod;
	protected static boolean _isOlympiadEnd;
	protected static boolean _compStarted;
	protected static boolean _battleStarted;
	protected static boolean _cycleTerminated;
	protected ScheduledFuture _scheduledCompStart;
	protected ScheduledFuture _scheduledCompEnd;
	protected ScheduledFuture _scheduledOlympiadEnd;
	protected ScheduledFuture _scheduledManagerTask;
	protected ScheduledFuture _scheduledWeeklyTask;
	protected ScheduledFuture _scheduledValdationTask;

	public static class Stadia
	{
		private boolean _freeToUse = true;

		public boolean isFreeToUse()
		{
			return _freeToUse;
		}

		public void setStadiaBusy()
		{
			_freeToUse = false;
		}

		public void setStadiaFree()
		{
			_freeToUse = true;
		}

		private int[] _coords = new int[3];

		public int[] getCoordinates()
		{
			return _coords;
		}

		public Stadia(int[] coords)
		{
			_coords = coords;
		}

		public Stadia(int x, int y, int z)
		{
			_coords[0] = x;
			_coords[1] = y;
			_coords[2] = z;
		}
	}

	protected static final Stadia[] STADIUMS = {
			new Stadia(-20814, -21189, -3030), new Stadia(-120324, -225077, -3331),
			new Stadia(-102495, -209023, -3331), new Stadia(-120156, -207378, -3331),
			new Stadia(-87628, -225021, -3331), new Stadia(-81705, -213209, -3331),
			new Stadia(-87593, -207339, -3331), new Stadia(-93709, -218304, -3331),
			new Stadia(-77157, -218608, -3331), new Stadia(-69682, -209027, -3331),
			new Stadia(-76887, -201256, -3331), new Stadia(-109985, -218701, -3331),
			new Stadia(-126367, -218228, -3331), new Stadia(-109629, -201292, -3331),
			new Stadia(-87523, -240169, -3331), new Stadia(-81748, -245950, -3331),
			new Stadia(-77123, -251473, -3331), new Stadia(-69778, -241801, -3331),
			new Stadia(-76754, -234014, -3331), new Stadia(-93742, -251032, -3331),
			new Stadia(-87466, -257752, -3331), new Stadia(-114413, -213241, -3331) };

	private static enum COMP_TYPE
	{
		CLASSED,
		NON_CLASSED
	}

	protected static OlympiadManager _manager;

	public static Olympiad getInstance()
	{
		if(_instance == null)
			_instance = new Olympiad();

		return _instance;
	}

	public Olympiad()
	{
		try
		{
			load();
		}
		catch(IOException e)
		{
			e.printStackTrace();
		}

		if(_period == 0)
			init();
	}

	private void load() throws IOException
	{
		_nobles = new FastMap<Integer, StatsSet>();

		Properties OlympiadProperties = new Properties();
		InputStream is = new FileInputStream(new File("./" + OLYMPIAD_DATA_FILE));
		OlympiadProperties.load(is);
		is.close();

		_currentCycle = Integer.parseInt(OlympiadProperties.getProperty("CurrentCycle", "1"));
		_period = Integer.parseInt(OlympiadProperties.getProperty("Period", "0"));
		_olympiadEnd = Long.parseLong(OlympiadProperties.getProperty("OlympiadEnd", "0"));
		_validationEnd = Long.parseLong(OlympiadProperties.getProperty("ValdationEnd", "0"));
		_nextWeeklyChange = Long.parseLong(OlympiadProperties.getProperty("NextWeeklyChange", "0"));

		switch(_period)
		{
			case 0:
				if(_olympiadEnd == 0 || _olympiadEnd < Calendar.getInstance().getTimeInMillis())
					setNewOlympiadEnd();
				else
					_isOlympiadEnd = false;
				break;
			case 1:
				if(_validationEnd > Calendar.getInstance().getTimeInMillis())
				{
					_isOlympiadEnd = true;

					_scheduledValdationTask = ThreadPoolManager.getInstance().scheduleGeneral(new Runnable(){
						public void run()
						{
							_period = 0;
							_currentCycle++;
							deleteNobles();
							setNewOlympiadEnd();
							init();
						}
					}, getMillisToValidationEnd());
				}
				else
				{
					_currentCycle++;
					_period = 0;
					deleteNobles();
					setNewOlympiadEnd();
				}
				break;
			default:
				_log.warning("Olympiad System: Omg something went wrong in loading!! Period = " + _period);
				return;
		}

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement(OLYMPIAD_LOAD_NOBLES);
			rset = statement.executeQuery();

			while(rset.next())
			{
				StatsSet statDat = new StatsSet();
				int charId = rset.getInt(CHAR_ID);
				statDat.set(CLASS_ID, rset.getInt(CLASS_ID));
				statDat.set(CHAR_NAME, rset.getString(CHAR_NAME));
				statDat.set(POINTS, rset.getInt(POINTS));
				statDat.set(COMP_DONE, rset.getInt(COMP_DONE));
				statDat.set(COMP_WIN, rset.getInt(COMP_WIN));
				statDat.set(COMP_LOOSE, rset.getInt(COMP_LOOSE));
				statDat.set("to_save", false);

				_nobles.put(charId, statDat);
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}

		synchronized (this)
		{
			_log.info("Olympiad System: Loading Olympiad System....");
			if(_period == 0)
				_log.info("Olympiad System: Currently in Olympiad Period");
			else
				_log.info("Olympiad System: Currently in Validation Period");

			_log.info("Olympiad System: Period Ends....");

			long milliToEnd;
			if(_period == 0)
				milliToEnd = getMillisToOlympiadEnd();
			else
				milliToEnd = getMillisToValidationEnd();

			double numSecs = milliToEnd / 1000 % 60;
			double countDown = (milliToEnd / 1000 - numSecs) / 60;
			int numMins = (int) Math.floor(countDown % 60);
			countDown = (countDown - numMins) / 60;
			int numHours = (int) Math.floor(countDown % 24);
			int numDays = (int) Math.floor((countDown - numHours) / 24);

			_log.info("Olympiad System: In " + numDays + " days, " + numHours + " hours and " + numMins + " mins.");

			if(_period == 0)
			{
				_log.info("Olympiad System: Next Weekly Change is in....");

				milliToEnd = getMillisToWeekChange();

				double numSecs2 = milliToEnd / 1000 % 60;
				double countDown2 = (milliToEnd / 1000 - numSecs2) / 60;
				int numMins2 = (int) Math.floor(countDown2 % 60);
				countDown2 = (countDown2 - numMins2) / 60;
				int numHours2 = (int) Math.floor(countDown2 % 24);
				int numDays2 = (int) Math.floor((countDown2 - numHours2) / 24);

				_log.info("Olympiad System: In " + numDays2 + " days, " + numHours2 + " hours and " + numMins2 + " mins.");
			}
		}

		_log.info("Olympiad System: Loaded " + _nobles.size() + " Nobles");

	}

	protected void init()
	{
		_nonClassBasedRegisters = new FastList<L2Player>();
		_classBasedRegisters = new FastMap<Short, List<L2Player>>();

		if(_period == 1)
			return;
		
		_compStart = Calendar.getInstance();
		_compStart.set(Calendar.HOUR_OF_DAY, COMP_START);
		_compStart.set(Calendar.MINUTE, COMP_MIN);
		_compEnd = _compStart.getTimeInMillis() + COMP_PERIOD;

		_scheduledOlympiadEnd = ThreadPoolManager.getInstance().scheduleGeneral(new Runnable(){
			public void run()
			{
				SystemMessage sm = new SystemMessage(SystemMessage.OLYMPIAD_PERIOD_S1_HAS_ENDED);
				sm.addNumber(_currentCycle);

				Announcements.getInstance().announceToAll(sm);
				Announcements.getInstance().announceToAll("Olympiad Validation Period has began");

				_isOlympiadEnd = true;
				if(_scheduledManagerTask != null)
					_scheduledManagerTask.cancel(true);
				if(_scheduledWeeklyTask != null)
					_scheduledWeeklyTask.cancel(true);

				Calendar validationEnd = Calendar.getInstance();
				_validationEnd = validationEnd.getTimeInMillis() + VALIDATION_PERIOD;

				saveNobleData();
				_period = 1;
				Hero.getInstance().clearHeroes();

				try
				{
					save();
				}
				catch(Exception e)
				{
					_log.warning("Olympiad System: Failed to save Olympiad configuration: " + e);
				}

				_scheduledValdationTask = ThreadPoolManager.getInstance().scheduleGeneral(new Runnable(){
					public void run()
					{
						sortHerosToBe();
						giveHeroBonus();
						Hero.getInstance().computeNewHeroes(_heroesToBe);
						Announcements.getInstance().announceToAll("Olympiad Validation Period has ended");
						_period = 0;
						_currentCycle++;
						deleteNobles();
						setNewOlympiadEnd();
						init();
					}
				}, getMillisToValidationEnd());
			}
		}, getMillisToOlympiadEnd());

		updateCompStatus();
		scheduleWeeklyChange();
	}

	public boolean registerNoble(L2Player noble, boolean classBased)
	{
		SystemMessage sm;
		if(!_inCompPeriod)
		{
			sm = new SystemMessage(SystemMessage.THE_OLYMPIAD_GAME_IS_NOT_CURRENTLY_IN_PROGRESS);
			noble.sendPacket(sm);
			return false;
		}

		if(noble.isCursedWeaponEquipped())
		{
			noble.sendMessage("You can't participate to Olympiad while holding a cursed weapon.", "Вы не можете участвовать в Олимпиаде, пока владеете проклятым оружием");
			return false;
		}

		if(!noble.isNoble())
		{
			sm = new SystemMessage(SystemMessage.ONLY_NOBLESS_CAN_PARTICIPATE_IN_THE_OLYMPIAD);
			noble.sendPacket(sm);
			return false;
		}

		if(noble.getBaseClass() != noble.getClassId().getId())
		{
			sm = new SystemMessage(SystemMessage.YOU_CANT_JOIN_THE_OLYMPIAD_WITH_A_SUB_JOB_CHARACTER);
			noble.sendPacket(sm);
			return false;
		}

		if(!_nobles.containsKey(noble.getObjectId()))
		{
			StatsSet statDat = new StatsSet();
			statDat.set(CLASS_ID, noble.getClassId().getId());
			statDat.set(CHAR_NAME, noble.getName());
			statDat.set(POINTS, DEFAULT_POINTS);
			statDat.set(COMP_DONE, 0);
			statDat.set(COMP_WIN, 0);
			statDat.set(COMP_LOOSE, 0);
			statDat.set("to_save", true);
			_nobles.put(noble.getObjectId(), statDat);
		}

		if(_classBasedRegisters.containsKey(noble.getClassId().getId()))
		{
			List<L2Player> classed = _classBasedRegisters.get(noble.getClassId().getId());
			for(L2Player partecipant : classed)
				if(partecipant.getObjectId() == noble.getObjectId())
				{
					noble.sendPacket(new SystemMessage(SystemMessage.YOU_ARE_ALREADY_ON_THE_WAITING_LIST_TO_PARTICIPATE_IN_THE_GAME_FOR_YOUR_CLASS));
					return false;
				}
		}

		for(L2Player partecipant : _nonClassBasedRegisters)
			if(partecipant.getObjectId() == noble.getObjectId())
			{
				noble.sendPacket(new SystemMessage(SystemMessage.YOU_ARE_ALREADY_ON_THE_WAITING_LIST_FOR_ALL_CLASSES_WAITING_TO_PARTICIPATE_IN_THE_GAME));
				return false;
			}
		for(L2OlympiadGame g : _manager.getOlympiadGames().values())
		{
			if(g == null || g.getPlayers() == null)
				continue;
			for(L2Player player : g.getPlayers())
				if(player.getObjectId() == noble.getObjectId())
				{
					sm = new SystemMessage(SystemMessage.YOU_ARE_ALREADY_ON_THE_WAITING_LIST_FOR_ALL_CLASSES_WAITING_TO_PARTICIPATE_IN_THE_GAME);
					noble.sendPacket(sm);
					return false;
				}

		}
		if(getNoblePoints(noble.getObjectId()) < 3)
		{
			noble.sendMessage("Cant register when you have less than 3", "Не возможно зарегистрировать потому, у вас меньше 3 очков");
			return false;
		}

		if(classBased)
		{
			if(_classBasedRegisters.containsKey(noble.getClassId().getId()))
			{
				List<L2Player> classed = _classBasedRegisters.get(noble.getClassId().getId());
				classed.add(noble);

				_classBasedRegisters.remove(noble.getClassId().getId());
				_classBasedRegisters.put(noble.getClassId().getId(), classed);

				sm = new SystemMessage(SystemMessage.YOU_HAVE_BEEN_REGISTERED_IN_A_WAITING_LIST_OF_CLASSIFIED_GAMES);
				noble.sendPacket(sm);
			}
			else
			{
				List<L2Player> classed = new FastList<L2Player>();
				classed.add(noble);

				_classBasedRegisters.put(noble.getClassId().getId(), classed);

				sm = new SystemMessage(SystemMessage.YOU_HAVE_BEEN_REGISTERED_IN_A_WAITING_LIST_OF_CLASSIFIED_GAMES);
				noble.sendPacket(sm);
			}
		}
		else
		{
			_nonClassBasedRegisters.add(noble);
			sm = new SystemMessage(SystemMessage.YOU_HAVE_BEEN_REGISTERED_IN_A_WAITING_LIST_OF_NO_CLASS_GAMES);
			noble.sendPacket(sm);
		}

		return true;
	}

	public boolean isRegistered(L2Player noble)
	{
		if(_nonClassBasedRegisters == null)
			return false;
		if(_classBasedRegisters == null)
			return false;

		if(!_nonClassBasedRegisters.contains(noble))
		{
			if(!_classBasedRegisters.containsKey(noble.getClassId().getId()))
				return false;
			List<L2Player> classed = _classBasedRegisters.get(noble.getClassId().getId());
			if(!classed.contains(noble))
				return false;
		}
		return true;
	}

	public boolean unRegisterNoble(L2Player noble)
	{
		SystemMessage sm;
		if(!_inCompPeriod)
		{
			sm = new SystemMessage(SystemMessage.THE_OLYMPIAD_GAME_IS_NOT_CURRENTLY_IN_PROGRESS);
			noble.sendPacket(sm);
			return false;
		}

		if(!noble.isNoble())
		{
			sm = new SystemMessage(SystemMessage.ONLY_NOBLESS_CAN_PARTICIPATE_IN_THE_OLYMPIAD);
			noble.sendPacket(sm);
			return false;
		}

		if(!isRegistered(noble))
		{
			sm = new SystemMessage(SystemMessage.YOU_HAVE_NOT_BEEN_REGISTERED_IN_A_WAITING_LIST_OF_A_GAME);
			noble.sendPacket(sm);
			return false;
		}

		if(_nonClassBasedRegisters.contains(noble))
			_nonClassBasedRegisters.remove(noble);
		else
		{
			List<L2Player> classed = _classBasedRegisters.get(noble.getClassId().getId());
			classed.remove(noble);
			_classBasedRegisters.remove(noble.getClassId().getId());
			_classBasedRegisters.put(noble.getClassId().getId(), classed);
		}
		sm = new SystemMessage(SystemMessage.YOU_HAVE_BEEN_DELETED_FROM_THE_WAITING_LIST_OF_A_GAME);
		noble.sendPacket(sm);

		return true;
	}

	private void updateCompStatus()
	{
		//_compStarted = false;

		synchronized (this)
		{
			long milliToStart = getMillisToCompBegin();

			double numSecs = milliToStart / 1000 % 60;
			double countDown = (milliToStart / 1000 - numSecs) / 60;
			int numMins = (int) Math.floor(countDown % 60);
			countDown = (countDown - numMins) / 60;
			int numHours = (int) Math.floor(countDown % 24);
			int numDays = (int) Math.floor((countDown - numHours) / 24);

			_log.info("Olympiad System: Competition Period Starts in " + numDays + " days, " + numHours + " hours and " + numMins + " mins.");

			_log.info("Olympiad System: Event starts/started : " + _compStart.getTime());
		}

		_scheduledCompStart = ThreadPoolManager.getInstance().scheduleGeneral(new Runnable(){
			public void run()
			{
				if(isOlympiadEnd())
					return;

				_inCompPeriod = true;
				OlympiadManager om = new OlympiadManager();

				Announcements.getInstance().announceToAll(new SystemMessage(SystemMessage.THE_OLYMPIAD_GAME_HAS_STARTED));
				_log.info("Olympiad System: Olympiad Game Started");

				Thread olyCycle = new Thread(om);
				olyCycle.start();

				//_scheduledManagerTask = ThreadPoolManager.getInstance().scheduleGeneralAtFixedRate(om, INITIAL_WAIT, BATTLE_WAIT);

				_scheduledCompEnd = ThreadPoolManager.getInstance().scheduleGeneral(new compEndTask(), getMillisToCompEnd());
			}
		}, getMillisToCompBegin());
	}

	private class compEndTask implements Runnable
	{
		public void run()
		{
			if(isOlympiadEnd())
				return;
			//_scheduledManagerTask.cancel(true);
			_inCompPeriod = false;

			try
			{
				if(_battleStarted)
				{
					//wait 1 minutes for end of pendings games
					_scheduledCompEnd = ThreadPoolManager.getInstance().scheduleGeneral(new compEndTask(), 60000);
					return;
				}
				Announcements.getInstance().announceToAll(new SystemMessage(SystemMessage.THE_OLYMPIAD_GAME_HAS_ENDED));
				_log.info("Olympiad System: Olympiad Game Ended");
				save();
			}
			catch(Exception e)
			{
				_log.warning("Olympiad System: Failed to save Olympiad configuration: " + e);
			}
			init();
		}
	}

	public boolean startOlympiad()
	{
		if(_inCompPeriod)
			return false;
		Announcements.getInstance().announceToAll(new SystemMessage(SystemMessage.THE_OLYMPIAD_GAME_HAS_STARTED));
		_log.info("Olympiad System: Olympiad Game Started");
		_inCompPeriod = true;
		OlympiadManager om = new OlympiadManager();
		Thread olyCycle = new Thread(om);
		olyCycle.start();
		_scheduledCompEnd = ThreadPoolManager.getInstance().scheduleGeneral(new Runnable(){
			public void run()
			{
				_scheduledManagerTask.cancel(true);
				_inCompPeriod = false;
				Announcements.getInstance().announceToAll(new SystemMessage(SystemMessage.THE_OLYMPIAD_GAME_HAS_ENDED));
				_log.info("Olympiad System: Olympiad Game Ended");
			}
		}, COMP_PERIOD);
		return true;
	}

	private long getMillisToOlympiadEnd()
	{
		//if(_olympiadEnd > Calendar.getInstance().getTimeInMillis())
		return _olympiadEnd - Calendar.getInstance().getTimeInMillis();
		//return 10L;
	}

	public void manualSelectHeroes()
	{
		SystemMessage sm = new SystemMessage(SystemMessage.OLYMPIAD_PERIOD_S1_HAS_ENDED);
		sm.addNumber(_currentCycle);

		Announcements.getInstance().announceToAll(sm);
		Announcements.getInstance().announceToAll("Olympiad Validation Period has began");

		_isOlympiadEnd = true;
		if(_scheduledManagerTask != null)
			_scheduledManagerTask.cancel(true);
		if(_scheduledWeeklyTask != null)
			_scheduledWeeklyTask.cancel(true);
		if(_scheduledOlympiadEnd != null)
			_scheduledOlympiadEnd.cancel(true);

		Calendar validationEnd = Calendar.getInstance();
		_validationEnd = validationEnd.getTimeInMillis() + VALIDATION_PERIOD;

		saveNobleData();

		_period = 1;

		Hero.getInstance().clearHeroes();
		
		try
		{
			save();
		}
		catch(Exception e)
		{
			_log.warning("Olympiad System: Failed to save Olympiad configuration: " + e);
		}

		_scheduledValdationTask = ThreadPoolManager.getInstance().scheduleGeneral(new Runnable(){
			public void run()
			{
				sortHerosToBe();
				giveHeroBonus();
				Hero.getInstance().computeNewHeroes(_heroesToBe);
				Announcements.getInstance().announceToAll("Olympiad Validation Period has ended");
				_period = 0;
				_currentCycle++;
				deleteNobles();
				setNewOlympiadEnd();
				init();
			}
		}, getMillisToValidationEnd());
	}

	protected long getMillisToValidationEnd()
	{
		if(_validationEnd > Calendar.getInstance().getTimeInMillis())
			return _validationEnd - Calendar.getInstance().getTimeInMillis();
		return 10L;
	}

	public boolean isOlympiadEnd()
	{
		return _isOlympiadEnd;
	}

	protected void setNewOlympiadEnd()
	{
		SystemMessage sm = new SystemMessage(SystemMessage.OLYMPIAD_PERIOD_S1_HAS_STARTED);
		sm.addNumber(_currentCycle);

		Announcements.getInstance().announceToAll(sm);

		Calendar currentTime = Calendar.getInstance();
		currentTime.set(Calendar.DAY_OF_MONTH, 1);
		currentTime.add(Calendar.MONTH, 1);
		currentTime.set(Calendar.HOUR_OF_DAY, 00);
		currentTime.set(Calendar.MINUTE, 00);
		_olympiadEnd = currentTime.getTimeInMillis();

		Calendar nextChange = Calendar.getInstance();
		_nextWeeklyChange = nextChange.getTimeInMillis() + WEEKLY_PERIOD;

		_isOlympiadEnd = false;
	}

	public boolean inCompPeriod()
	{
		return _inCompPeriod;
	}

	private long getMillisToCompBegin()
	{
		if(_compStart.getTimeInMillis() < Calendar.getInstance().getTimeInMillis() && _compEnd > Calendar.getInstance().getTimeInMillis())
			return 10L;

		if(_compStart.getTimeInMillis() > Calendar.getInstance().getTimeInMillis())
			return _compStart.getTimeInMillis() - Calendar.getInstance().getTimeInMillis();

		return setNewCompBegin();
	}

	private long setNewCompBegin()
	{
		_compStart = Calendar.getInstance();
		_compStart.set(Calendar.HOUR_OF_DAY, COMP_START);
		_compStart.set(Calendar.MINUTE, COMP_MIN);
		_compStart.add(Calendar.HOUR_OF_DAY, 24);
		_compEnd = _compStart.getTimeInMillis() + COMP_PERIOD;

		_log.info("Olympiad System: New Schedule @ " + _compStart.getTime());

		return _compStart.getTimeInMillis() - Calendar.getInstance().getTimeInMillis();
	}

	protected long getMillisToCompEnd()
	{
		//if(_compEnd > Calendar.getInstance().getTimeInMillis())
		return _compEnd - Calendar.getInstance().getTimeInMillis();
		//return 10L;
	}

	private long getMillisToWeekChange()
	{
		if(_nextWeeklyChange > Calendar.getInstance().getTimeInMillis())
			return _nextWeeklyChange - Calendar.getInstance().getTimeInMillis();
		return 10L;
	}

	private void scheduleWeeklyChange()
	{
		_scheduledWeeklyTask = ThreadPoolManager.getInstance().scheduleGeneralAtFixedRate(new Runnable(){
			public void run()
			{
				addWeeklyPoints();
				_log.info("Olympiad System: Added weekly points to nobles");

				Calendar nextChange = Calendar.getInstance();
				_nextWeeklyChange = nextChange.getTimeInMillis() + WEEKLY_PERIOD;
			}
		}, getMillisToWeekChange(), WEEKLY_PERIOD);
	}

	protected synchronized void addWeeklyPoints()
	{
		if(_period == 1)
			return;

		for(Integer nobleId : _nobles.keySet())
		{
			StatsSet nobleInfo = _nobles.get(nobleId);
			int currentPoints = nobleInfo.getInteger(POINTS);
			currentPoints += WEEKLY_POINTS;
			nobleInfo.set(POINTS, currentPoints);

			_nobles.remove(nobleId);
			_nobles.put(nobleId, nobleInfo);
		}
	}

	public String[] getAllTitles()
	{
		String[] msg = new String[STADIUMS.length];
		for(int i = 0; i < STADIUMS.length; i++)
			if(_manager != null && _manager.getOlympiadInstance(i) != null)
				msg[i] = i + 1 + "_In Progress_" + _manager.getOlympiadInstance(i).getTitle();
			else
				msg[i] = i + 1 + "_Initial State";

		return msg;
	}

	public int getCurrentCycle()
	{
		return _currentCycle;
	}

	public void addSpectator(int id, L2Player spectator)
	{
		if(spectator.getOlympiadGameId() != -1)
		{
			spectator.sendMessage("You are already registered for a competition");
			return;
		}
		if(_manager == null || _manager.getOlympiadInstance(id) == null || !_battleStarted)
		{
			spectator.sendPacket(new SystemMessage(SystemMessage.THE_OLYMPIAD_GAME_IS_NOT_CURRENTLY_IN_PROGRESS));
			return;
		}

		L2Player[] players = _manager.getOlympiadInstance(id).getPlayers();

		if(players == null)
			return;

		spectator.enterOlympiadObserverMode(STADIUMS[id].getCoordinates()[0], STADIUMS[id].getCoordinates()[1], STADIUMS[id].getCoordinates()[2], id);

		_manager.getOlympiadInstance(id).addSpectator(spectator);

		spectator.sendPacket(new ExOlympiadUserInfoSpectator(players[0], 2));
		spectator.sendPacket(new ExOlympiadUserInfoSpectator(players[1], 1));
	}

	public void removeSpectator(int id, L2Player spectator)
	{
		if(_manager == null || _manager.getOlympiadInstance(id) == null)
			return;

		_manager.getOlympiadInstance(id).removeSpectator(spectator);
	}

	public List<L2Player> getSpectators(int id)
	{
		if(_manager == null || _manager.getOlympiadInstance(id) == null)
			return null;
		return _manager.getOlympiadInstance(id).getSpectators();
	}

	public Map<Integer, L2OlympiadGame> getOlympiadGames()
	{
		return _manager.getOlympiadGames();
	}

	public L2OlympiadGame getOlympiadGame(int gameId)
	{
		return _manager.getOlympiadGames().get(gameId);
	}

	public OlympiadGameTask getOlympiadGameQueue(int gameId)
	{
		return _gamesQueue.get(gameId);
	}

	public ScheduledFuture getOlympiadGameQueueScheduled(int gameId)
	{
		return _gamesQueueScheduled.get(gameId);
	}

	public void startValidateWinner(int id)
	{
		OlympiadGameTask task;
		ScheduledFuture sf;
		task = new OlympiadGameTask(_manager.getOlympiadGames().get(id), BattleStatus.ValidateWinner);
		sf = ThreadPoolManager.getInstance().scheduleGeneral(task, 10);
		_gamesQueue.put(id, task);
		_gamesQueueScheduled.put(id, sf);
	}

	public int[] getWaitingList()
	{
		int[] array = new int[2];

		if(!inCompPeriod())
			return null;

		int classCount = 0;

		if(_classBasedRegisters.size() != 0)
			for(List<L2Player> classed : _classBasedRegisters.values())
				classCount += classed.size();

		array[0] = classCount;
		array[1] = _nonClassBasedRegisters.size();

		return array;
	}

	protected synchronized void saveNobleData()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		if(_nobles == null)
			return;

		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			for(Integer nobleId : _nobles.keySet())
			{
				StatsSet nobleInfo = _nobles.get(nobleId);

				int classId = nobleInfo.getInteger(CLASS_ID);
				String charName = nobleInfo.getString(CHAR_NAME);
				int points = nobleInfo.getInteger(POINTS);
				int compDone = nobleInfo.getInteger(COMP_DONE);
				int compWin = nobleInfo.getInteger(COMP_WIN);
				int compLoose = nobleInfo.getInteger(COMP_LOOSE);
				boolean toSave = nobleInfo.getBool("to_save");

				if(toSave)
				{
					statement = con.prepareStatement(OLYMPIAD_SAVE_NOBLES);
					statement.setInt(1, nobleId);
					statement.setInt(2, classId);
					statement.setString(3, charName);
					statement.setInt(4, points);
					statement.setInt(5, compDone);
					statement.setInt(6, compWin);
					statement.setInt(7, compLoose);
					statement.execute();
					DatabaseUtils.closeStatement(statement);

					nobleInfo.set("to_save", false);

					_nobles.remove(nobleId);
					_nobles.put(nobleId, nobleInfo);
				}
				else
				{
					statement = con.prepareStatement(OLYMPIAD_UPDATE_NOBLES);
					statement.setInt(1, points);
					statement.setInt(2, compDone);
					statement.setInt(3, compWin);
					statement.setInt(4, compLoose);
					statement.setInt(5, nobleId);
					statement.execute();
					DatabaseUtils.closeStatement(statement);
				}
			}
		}
		catch(SQLException e)
		{
			_log.warning("Olympiad System: Couldnt save nobles info in db");
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	protected void sortHerosToBe()
	{
		if(_period != 1)
			return;

		_heroesToBe = new FastList<StatsSet>();

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;

		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			StatsSet hero;

			for(int i = 88; i < 119; i++)
			{
				statement = con.prepareStatement(OLYMPIAD_GET_HEROS);
				statement.setInt(1, i);
				rset = statement.executeQuery();

				if(rset.next())
				{
					hero = new StatsSet();
					hero.set(CLASS_ID, i);
					hero.set(CHAR_ID, rset.getInt(CHAR_ID));
					hero.set(CHAR_NAME, rset.getString(CHAR_NAME));

					_heroesToBe.add(hero);
				}
				DatabaseUtils.closeDatabaseSR(statement, rset);
			}
		}
		catch(SQLException e)
		{
			_log.warning("Olympiad System: Couldnt heros from db");
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}

	}

	public List<String> getClassLeaderBoard(int classId)
	{
		//if(_period != 1) return;

		List<String> names = new FastList<String>();

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;

		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement(GET_EACH_CLASS_LEADER);
			statement.setInt(1, classId);
			rset = statement.executeQuery();

			while(rset.next())
				names.add(rset.getString(CHAR_NAME));

			return names;
		}
		catch(SQLException e)
		{
			_log.warning("Olympiad System: Couldnt get heros from db: ");
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}

		return names;

	}

	protected void giveHeroBonus()
	{
		if(_heroesToBe.size() == 0)
			return;

		for(StatsSet hero : _heroesToBe)
		{
			int charId = hero.getInteger(CHAR_ID);

			StatsSet noble = _nobles.get(charId);
			int currentPoints = noble.getInteger(POINTS);
			currentPoints += 300;
			noble.set(POINTS, currentPoints);

			_nobles.remove(charId);
			_nobles.put(charId, noble);
		}
	}

	public int getNoblessePasses(int objId)
	{
		if(_period != 1 || _nobles.size() == 0)
			return 0;

		StatsSet noble = _nobles.get(objId);
		if(noble == null)
			return 0;
		int points = noble.getInteger(POINTS);
		if(points <= 50)
			return 0;

		noble.set(POINTS, 0);
		_nobles.remove(objId);
		_nobles.put(objId, noble);

		points *= 1000;

		return points;
	}

	public boolean isRegisteredInComp(L2Player player)
	{
		boolean result = false;

		if(_nonClassBasedRegisters != null && _nonClassBasedRegisters.contains(player))
			result = true;

		else if(_classBasedRegisters != null && _classBasedRegisters.containsKey(player.getClassId().getId()))
		{
			List<L2Player> classed = _classBasedRegisters.get(player.getClassId().getId());
			if(classed.contains(player))
				result = true;
		}

		return result;
	}

	public int getNoblePoints(int objId)
	{
		if(_nobles.size() == 0)
			return 0;

		StatsSet noble = _nobles.get(objId);
		if(noble == null)
			return 0;

		return noble.getInteger(POINTS);
	}

	public int getCompetitionDone(int objId)
	{
		if(_nobles.size() == 0)
			return 0;

		StatsSet noble = _nobles.get(objId);
		if(noble == null)
			return 0;

		return noble.getInteger(COMP_DONE);
	}

	public int getCompetitionWin(int objId)
	{
		if(_nobles.size() == 0)
			return 0;

		StatsSet noble = _nobles.get(objId);
		if(noble == null)
			return 0;

		return noble.getInteger(COMP_WIN);
	}

	public int getCompetitionLoose(int objId)
	{
		if(_nobles.size() == 0)
			return 0;

		StatsSet noble = _nobles.get(objId);
		if(noble == null)
			return 0;

		return noble.getInteger(COMP_LOOSE);
	}

	protected void deleteNobles()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement(OLYMPIAD_DELETE_ALL);
			statement.execute();
		}
		catch(SQLException e)
		{
			_log.warning("Olympiad System: Couldnt delete nobles from db");
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}

		_nobles.clear();
	}

	public void save() throws IOException
	{
		saveNobleData();

		Properties OlympiadProperties = new Properties();
		FileOutputStream fos = new FileOutputStream(new File(Config.DATAPACK_ROOT, OLYMPIAD_DATA_FILE));

		OlympiadProperties.setProperty("CurrentCycle", String.valueOf(_currentCycle));
		OlympiadProperties.setProperty("Period", String.valueOf(_period));
		OlympiadProperties.setProperty("OlympiadEnd", String.valueOf(_olympiadEnd));
		OlympiadProperties.setProperty("ValdationEnd", String.valueOf(_validationEnd));
		OlympiadProperties.setProperty("NextWeeklyChange", String.valueOf(_nextWeeklyChange));

		OlympiadProperties.store(fos, "Olympiad Properties");
		fos.close();
	}

	public Stadia[] getStadiums()
	{
		return STADIUMS;
	}

	private class OlympiadManager implements Runnable
	{
		private Map<Integer, L2OlympiadGame> _olympiadInstances;

		// TODO зачем они нужны, если нигде не используются?
		//private Map<Integer, List<L2Player>> _classBasedParticipants;
		//private Map<Integer, List<L2Player>> _nonClassBasedParticipants;

		public OlympiadManager()
		{
			_olympiadInstances = new FastMap<Integer, L2OlympiadGame>();
			_manager = this;
		}

		public synchronized void run()
		{
			_cycleTerminated = false;
			if(isOlympiadEnd())
			{
				_cycleTerminated = true;
				return;
			}

			while(inCompPeriod())
			{
				if(_nobles.size() == 0)
				{
					try
					{
						wait(60000);
					}
					catch(InterruptedException ex)
					{}
					continue;
				}

				int classBasedPgCount = 0;
				for(List<L2Player> classList : _classBasedRegisters.values())
					classBasedPgCount += classList.size();
				//_compStarted = true;
				while((_gamesQueue.size() > 0 || classBasedPgCount > 1 || _nonClassBasedRegisters.size() > 1) && inCompPeriod())
				{
					//first cycle do nothing
					for(OlympiadGameTask ogt : _gamesQueue.values())
					{
						if(ogt == null || ogt._game == null)
							continue;
						int i = ogt._game.getId();
						if(ogt.isTerminated())
						{
							//removes terminated games from the queue
							_olympiadInstances.remove(i);
							_gamesQueue.remove(i);
							_gamesQueueScheduled.remove(i);
							STADIUMS[i].setStadiaFree();
						}
						else if(ogt.getStatus() == BattleStatus.Begining)
						{
							//start new games
							ScheduledFuture sf = ThreadPoolManager.getInstance().scheduleGeneral(ogt, 10);
							_gamesQueueScheduled.put(i, sf);
						}
					}
					//set up the games queue
					for(int i = 0; i < STADIUMS.length; i++)
					{
						if(!existNextOpponents(_nonClassBasedRegisters) && !existNextOpponents(getRandomClassList(_classBasedRegisters)))
							break;
						if(STADIUMS[i].isFreeToUse())
							if(existNextOpponents(_nonClassBasedRegisters) && i % 2 == 0)
								try
								{
									_olympiadInstances.put(i, new L2OlympiadGame(i, COMP_TYPE.NON_CLASSED, nextOpponents(_nonClassBasedRegisters), STADIUMS[i].getCoordinates()));
									_gamesQueue.put(i, new OlympiadGameTask(_olympiadInstances.get(i), BattleStatus.Begining));
									STADIUMS[i].setStadiaBusy();
								}
								catch(Exception ex)
								{
									ex.printStackTrace();
									if(_olympiadInstances.get(i) != null)
									{
										for(L2Player player : _olympiadInstances.get(i).getPlayers())
										{
											player.sendMessage("Your olympiad registration was canceled due to an error");
											player.setIsInOlympiadMode(false);
											player.setOlympiadSide(-1);
											player.setOlympiadGameId(-1);
										}
										_olympiadInstances.remove(i);
									}
									if(_gamesQueue.get(i) != null)
										_gamesQueue.remove(i);
									if(_gamesQueueScheduled.get(i) != null)
										_gamesQueueScheduled.remove(i);
									STADIUMS[i].setStadiaFree();
									//try to reuse this stadia next time
									i--;
								}
							else if(existNextOpponents(getRandomClassList(_classBasedRegisters)))
								try
								{
									_olympiadInstances.put(i, new L2OlympiadGame(i, COMP_TYPE.CLASSED, nextOpponents(getRandomClassList(_classBasedRegisters)), STADIUMS[i].getCoordinates()));
									_gamesQueue.put(i, new OlympiadGameTask(_olympiadInstances.get(i), BattleStatus.Begining));
									STADIUMS[i].setStadiaBusy();
								}
								catch(Exception ex)
								{
									ex.printStackTrace();
									if(_olympiadInstances.get(i) != null)
									{
										for(L2Player player : _olympiadInstances.get(i).getPlayers())
										{
											player.sendMessage("Your olympiad registration was canceled due to an error");
											player.setIsInOlympiadMode(false);
											player.setOlympiadSide(-1);
											player.setOlympiadGameId(-1);
										}
										_olympiadInstances.remove(i);
									}
									if(_gamesQueue.get(i) != null)
										_gamesQueue.remove(i);
									if(_gamesQueueScheduled.get(i) != null)
										_gamesQueueScheduled.remove(i);
									STADIUMS[i].setStadiaFree();
									//try to reuse this stadia next time
									i--;
								}
					}
					//wait 30 sec for !stress the server
					try
					{
						wait(30000);
					}
					catch(InterruptedException e)
					{}
				}
				//    			wait 30 sec for !stress the server
				try
				{
					wait(30000);
				}
				catch(InterruptedException e)
				{}
			}

			//when comp time finish wait for all games terminated before execute the cleanup code
			boolean allGamesTerminated = false;
			//wait for all games terminated
			while(!allGamesTerminated)
			{
				try
				{
					wait(30000);
				}
				catch(InterruptedException e)
				{}

				if(_gamesQueue.size() == 0)
					allGamesTerminated = true;
				else
					for(OlympiadGameTask game : _gamesQueue.values())
						allGamesTerminated = allGamesTerminated || game.isTerminated();
			}
			_cycleTerminated = true;
			//when all games terminated clear all
			_gamesQueue.clear();
			_gamesQueueScheduled.clear();

			//TODO Какого хрена они тут очищаются, если нигде не используются?
			//_classBasedParticipants.clear();
			//_nonClassBasedParticipants.clear();

			_olympiadInstances.clear();
			_classBasedRegisters.clear();
			_nonClassBasedRegisters.clear();

			_battleStarted = false;
			//_compStarted = false;

		}

		protected L2OlympiadGame getOlympiadInstance(int index)
		{
			if(_olympiadInstances != null && _olympiadInstances.size() > 0 /*|| _compStarted*/)
				return _olympiadInstances.get(index);
			return null;
		}

		protected Map<Integer, L2OlympiadGame> getOlympiadGames()
		{
			return _olympiadInstances == null ? null : _olympiadInstances;
		}

		private List<L2Player> getRandomClassList(Map<Short, List<L2Player>> list)
		{
			if(list.size() == 0)
				return null;

			Map<Integer, List<L2Player>> tmp = new FastMap<Integer, List<L2Player>>();
			int tmpIndex = 0;
			for(List<L2Player> l : list.values())
			{
				tmp.put(tmpIndex, l);
				tmpIndex++;
			}

			List<L2Player> rndList;
			int classIndex;
			if(tmp.size() == 1)
				classIndex = 0;
			else
				classIndex = Rnd.nextInt(tmp.size());
			rndList = tmp.get(classIndex);
			return rndList;
		}

		private List<L2Player> nextOpponents(List<L2Player> list)
		{
			List<L2Player> opponents = new FastList<L2Player>();
			if(list.size() == 0)
				return opponents;
			int loopCount = list.size() / 2;

			int first;
			int second;

			if(loopCount < 1)
				return opponents;

			first = Rnd.nextInt(list.size());
			opponents.add(list.get(first));
			removeOpponent(list.get(first));

			second = Rnd.nextInt(list.size());
			opponents.add(list.get(second));
			removeOpponent(list.get(second));

			return opponents;

		}

		private void removeOpponent(L2Player noble)
		{
			if(_nonClassBasedRegisters.contains(noble))
				_nonClassBasedRegisters.remove(noble);
			else
			{
				List<L2Player> classed = _classBasedRegisters.get(noble.getClassId().getId());
				classed.remove(noble);
				_classBasedRegisters.remove(noble.getClassId().getId());
				_classBasedRegisters.put(noble.getClassId().getId(), classed);
			}
		}

		private boolean existNextOpponents(List<L2Player> list)
		{
			if(list == null)
				return false;
			if(list.size() == 0)
				return false;
			int loopCount = list.size() / 2;

			return loopCount >= 1;
		}
	}

	public class L2OlympiadGame
	{
		private int _id;
		protected COMP_TYPE _type;
		private boolean _aborted;
		private String _playerOneName;
		private String _playerTwoName;
		private int _playerOneID = 0;
		private int _playerTwoID = 0;
		private L2Player _playerOne;
		private L2Player _playerTwo;
		private List<L2Player> _players = new FastList<L2Player>();
		private int[] _playerOneLocation;
		private int[] _playerTwoLocation;
		private int[] _stadiumPort;
		private List<L2Player> _spectators;
		private SystemMessage _sm;

		protected L2OlympiadGame(int id, COMP_TYPE type, List<L2Player> list, int[] stadiumPort)
		{
			_id = id;
			_aborted = false;
			_type = type;
			_stadiumPort = stadiumPort;
			_spectators = new FastList<L2Player>();

			if(list != null)
			{
				_playerOne = L2World.getPlayer(list.get(0).getName());
				_playerTwo = L2World.getPlayer(list.get(1).getName());
				_players.add(_playerOne);
				_players.add(_playerTwo);

				if(_playerOne == null || _playerTwo == null)
					return;

				try
				{
					_playerOneName = _playerOne.getName();
					_playerTwoName = _playerTwo.getName();
					_playerOneID = _playerOne.getObjectId();
					_playerTwoID = _playerTwo.getObjectId();
				}
				catch(Exception e)
				{
					e.printStackTrace();
					_aborted = true;
					clearPlayers();
				}
				_log.info("Olympiad System: Game - " + id + ": " + _playerOne.getName() + " Vs " + _playerTwo.getName());
			}
			else
			{
				_log.warning("Player list is null.");
				_aborted = true;
				clearPlayers();
			}
		}

		protected void removals()
		{
			if(_aborted)
				return;

			if(_playerOne == null || _playerTwo == null)
				return;

			for(L2Player player : _players)
				try
				{
					//Remove clan skill
					if(player.getClan() != null)
						for(L2Skill skill : player.getClan().getAllSkills())
							player.removeSkill(skill, false);

					// Remove Hero Skills
					if(player.isHero())
					{
						player.removeSkillById((short) 395);
						player.removeSkillById((short) 396);
						player.removeSkillById((short) 1374);
						player.removeSkillById((short) 1375);
						player.removeSkillById((short) 1376);
					}

					// Abort casting if player casting
					if(player.isCastingNow())
						player.abortCast();

					// Remove Buffs
					player.stopAllEffects();

					// Remove Summon's Buffs
					if(player.getPet() != null)
					{
						L2Summon summon = player.getPet();
						summon.stopAllEffects();
						if(summon.isPet)
							summon.unSummon();
					}

					//Remove player from his party
					if(player.getParty() != null)
					{
						L2Party party = player.getParty();
						party.removePartyMember(player);
					}

					//Remove Hero weapons
					L2ItemInstance wpn = player.getInventory().getPaperdollItem(Inventory.PAPERDOLL_RHAND);
					if(wpn == null)
						wpn = player.getInventory().getPaperdollItem(Inventory.PAPERDOLL_LRHAND);
					if(wpn != null && (wpn.getItemId() >= 6611 && wpn.getItemId() <= 6621 || wpn.getItemId() == 6842))
					{
						L2ItemInstance[] unequipped = player.getInventory().unEquipItemInBodySlotAndRecord(wpn.getItem().getBodyPart(), wpn);
						InventoryUpdate iu = new InventoryUpdate();
						for(L2ItemInstance element : unequipped)
							iu.addModifiedItem(element);
						player.sendPacket(iu);
						player.abortAttack();
						player.refreshExpertisePenalty();
						player.broadcastUserInfo();

						// this can be 0 if the user pressed the right mousebutton twice very fast
						if(unequipped.length > 0)
						{
							if(unequipped[0].isWear())
								return;
							if(unequipped[0].getEnchantLevel() > 0)
								player.sendPacket(new SystemMessage(SystemMessage.EQUIPMENT_OF__S1_S2_HAS_BEEN_REMOVED).addNumber(unequipped[0].getEnchantLevel()).addItemName(unequipped[0].getItemId()));
							else
								player.sendPacket(new SystemMessage(SystemMessage.S1__HAS_BEEN_DISARMED).addItemName(unequipped[0].getItemId()));
						}
					}

					//remove bsps/sps/ss automation
					ConcurrentSkipListSet<Integer> activeSoulShots = player.getAutoSoulShot();
					for(int itemId : activeSoulShots)
						player.removeAutoSoulShot(itemId);
				}
				catch(Exception e)
				{}

			broadcastMessage(new SystemMessage(SystemMessage.THE_GAME_WILL_START_IN_S1_SECOND_S).addNumber(120), false);
		}

		protected void clearPlayers()
		{
			_playerOne = null;
			_playerTwo = null;
			_players = null;
			_playerOneName = "";
			_playerTwoName = "";
			_playerOneID = 0;
			_playerTwoID = 0;
		}

		protected boolean portPlayersToArena()
		{
			_playerOneLocation = new int[3];
			_playerTwoLocation = new int[3];

			boolean _playerOneCrash = true;
			boolean _playerTwoCrash = true;

			try
			{
				if(_playerOne != null)
					_playerOneCrash = false;
			}
			catch(Exception e)
			{}

			try
			{
				if(_playerTwo != null)
					_playerTwoCrash = false;
			}
			catch(Exception e)
			{}

			if(_playerOneCrash)
			{
				StatsSet playerOneStat;
				playerOneStat = _nobles.get(_playerOneID);
				int playerOnePoints = playerOneStat.getInteger(POINTS);
				int i = playerOnePoints - playerOnePoints / 5;
				playerOneStat.set(POINTS, i);
				_log.info("Olympia Result: " + _playerOneName + " vs " + _playerTwoName + " ... " + _playerOneName + " lost " + i + " points for crash before teleport to arena");
			}

			if(_playerTwoCrash)
			{
				StatsSet playerTwoStat;
				playerTwoStat = _nobles.get(_playerTwoID);
				int playerTwoPoints = playerTwoStat.getInteger(POINTS);
				int i = playerTwoPoints - playerTwoPoints / 5;
				playerTwoStat.set(POINTS, i);
				_log.info("Olympia Result: " + _playerOneName + " vs " + _playerTwoName + " ... " + _playerTwoName + " lost " + i + " points for crash before teleport to arena");
			}

			if(_playerOneCrash || _playerTwoCrash)
			{
				_playerOne = null;
				_playerTwo = null;
				_log.warning("PortPlayers: player is null");
				_aborted = true;
				return false;
			}

			try
			{
				_playerOneLocation[0] = _playerOne.getX();
				_playerOneLocation[1] = _playerOne.getY();
				_playerOneLocation[2] = _playerOne.getZ();

				_playerTwoLocation[0] = _playerTwo.getX();
				_playerTwoLocation[1] = _playerTwo.getY();
				_playerTwoLocation[2] = _playerTwo.getZ();

				//_playerOne.setVisible(false);
				//_playerTwo.setVisible(false);

				if(_playerOne.isSitting())
					_playerOne.standUp();

				if(_playerTwo.isSitting())
					_playerTwo.standUp();

				_playerOne.setTarget(null);
				_playerTwo.setTarget(null);

				_playerOne.teleToLocation(_stadiumPort[0] - 450, _stadiumPort[1], _stadiumPort[2]);
				_playerTwo.teleToLocation(_stadiumPort[0] + 450, _stadiumPort[1], _stadiumPort[2]);

				_playerOne.setIsInOlympiadMode(true);
				_playerTwo.setIsInOlympiadMode(true);
				_playerOne.setOlympiadGameId(_id);
				_playerTwo.setOlympiadGameId(_id);
			}
			catch(Exception e)
			{
				_log.info("Olympia Result: " + _playerOneName + " vs " + _playerTwoName + " ...  aborted due to crash before teleport to arena");
				e.printStackTrace();
				return false;
			}
			return true;
		}

		protected void sendMessageToPlayers(boolean toBattleBegin, int nsecond)
		{
			if(!toBattleBegin)
				_sm = new SystemMessage(SystemMessage.YOU_WILL_ENTER_THE_OLYMPIAD_STADIUM_IN_S1_SECOND_S);
			else
				_sm = new SystemMessage(SystemMessage.THE_GAME_WILL_START_IN_S1_SECOND_S);

			_sm.addNumber(nsecond);
			try
			{
				for(L2Player player : _players)
					player.sendPacket(_sm);
			}
			catch(Exception e)
			{}
		}

		protected void portPlayersBack()
		{
			if(_playerOne == null || _playerTwo == null)
				return;

			try
			{
				_playerOne.teleToLocation(_playerOneLocation[0], _playerOneLocation[1], _playerOneLocation[2]);
				_playerTwo.teleToLocation(_playerTwoLocation[0], _playerTwoLocation[1], _playerTwoLocation[2]);
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}

			for(L2Player player : _players)
				try
				{
					player.setIsInOlympiadMode(false);
					player.setOlympiadSide(-1);
					player.setOlympiadGameId(-1);
					player.stopAllEffects();
					player.setCurrentCp(player.getMaxCp());
					player.setCurrentHp(player.getMaxHp());
					player.setCurrentMp(player.getMaxMp());
					//Add clan skill
					if(player.getClan() != null)
						for(L2Skill skill : player.getClan().getAllSkills())
							if(skill.getMinPledgeClass() <= player.getPledgeClass())
								player.addSkill(skill, false);

					// Add Hero Skills
					if(player.isHero())
					{
						player.addSkill(SkillTable.getInstance().getInfo(395, 1));
						player.addSkill(SkillTable.getInstance().getInfo(396, 1));
						player.addSkill(SkillTable.getInstance().getInfo(1374, 1));
						player.addSkill(SkillTable.getInstance().getInfo(1375, 1));
						player.addSkill(SkillTable.getInstance().getInfo(1376, 1));
					}
					makePlayersVisible();
				}
				catch(Exception e)
				{
					e.printStackTrace();
				}
		}

		protected boolean haveWinner()
		{
			boolean retval = false;
			if(_aborted || _playerOne == null && _playerTwo == null)
			{
				_log.info("Olympia Result: " + _playerOneName + " vs " + _playerTwoName + " ... aborted/tie due to crashes!");
				return true;
			}

			double playerOneHp = 0;

			try
			{
				if(_playerOne != null && _playerOne.getOlympiadGameId() != -1)
					playerOneHp = _playerOne.getCurrentHp();
			}
			catch(Exception e)
			{
				playerOneHp = 0;
			}

			double playerTwoHp = 0;
			try
			{
				if(_playerTwo != null && _playerTwo.getOlympiadGameId() != -1)
					playerTwoHp = _playerTwo.getCurrentHp();
			}
			catch(Exception e)
			{
				playerTwoHp = 0;
			}

			if(playerTwoHp == 0 || playerOneHp == 0)
				retval = true;

			return retval;
		}

		public void validateWinner() throws Exception
		{
			if(_aborted || _playerOne == null && _playerTwo == null)
			{
				_log.info("Olympia Result: " + _playerOneName + " vs " + _playerTwoName + " ... aborted/tie due to crashes!");
				return;
			}
			StatsSet playerOneStat;
			StatsSet playerTwoStat;

			playerOneStat = _nobles.get(_playerOneID);
			playerTwoStat = _nobles.get(_playerTwoID);

			int playerOneWin = playerOneStat.getInteger(COMP_WIN);
			int playerTwoWin = playerTwoStat.getInteger(COMP_WIN);
			int playerOneLoose = playerOneStat.getInteger(COMP_LOOSE);
			int playerTwoLoose = playerTwoStat.getInteger(COMP_LOOSE);
			int playerOnePlayed = playerOneStat.getInteger(COMP_DONE);
			int playerTwoPlayed = playerTwoStat.getInteger(COMP_DONE);

			int playerOnePoints = playerOneStat.getInteger(POINTS);
			int playerTwoPoints = playerTwoStat.getInteger(POINTS);

			//_playerOne = L2World.getPlayer(_playerOneName);
			//_playerTwo = L2World.getPlayer(_playerTwoName); // why this...

			double playerOneHp = 0;
			double hpDiffOne = 9999999;
			try
			{
				if(_playerOne != null && _playerOne.getOlympiadGameId() != -1)
				{
					playerOneHp = _playerOne.getCurrentHp() + _playerOne.getCurrentCp();
					hpDiffOne = _playerOne.getMaxHp() + _playerOne.getMaxCp() - playerOneHp;
				}
			}
			catch(Exception e)
			{
				playerOneHp = 0;
				hpDiffOne = 9999999;
			}

			double playerTwoHp = 0;
			double hpDiffTwo = 9999999;
			try
			{
				if(_playerTwo != null && _playerTwo.getOlympiadGameId() != -1)
				{
					playerTwoHp = _playerTwo.getCurrentHp() + _playerTwo.getCurrentCp();
					hpDiffTwo = _playerTwo.getMaxHp() + _playerTwo.getMaxCp() - playerTwoHp;
				}
			}
			catch(Exception e)
			{
				playerTwoHp = 0;
				hpDiffTwo = 9999999;
			}

			_sm = new SystemMessage(SystemMessage.S1_HAS_WON_THE_GAME);
			SystemMessage _sm2 = new SystemMessage(SystemMessage.S1_HAS_GAINED_S2_OLYMPIAD_POINTS);
			SystemMessage _sm3 = new SystemMessage(SystemMessage.S1_HAS_LOST_S2_OLYMPIAD_POINTS);

			String result = "";

			if(_playerOne == null && _playerTwo == null)
			{
				_log.info("Olympia Result: " + _playerOneName + " vs " + _playerTwoName + " ... aborted/tie due to crashes!");
				return;
			}

			if(_playerTwo == null || playerTwoHp == 0 || hpDiffOne < hpDiffTwo)
			{
				int pointDiff;
				pointDiff = playerTwoPoints / 3;
				playerOneStat.set(POINTS, playerOnePoints + pointDiff);
				playerTwoStat.set(POINTS, playerTwoPoints - pointDiff);
				playerOneStat.set(COMP_WIN, playerOneWin + 1);
				playerTwoStat.set(COMP_LOOSE, playerTwoLoose + 1);

				_sm.addString(_playerOneName);
				broadcastMessage(_sm, true);
				_sm2.addString(_playerOneName);
				_sm2.addNumber(pointDiff);
				broadcastMessage(_sm2, true);
				_sm3.addString(_playerTwoName);
				_sm3.addNumber(pointDiff);
				broadcastMessage(_sm3, true);

				try
				{
					result = " (" + playerOneHp + "hp vs " + playerTwoHp + "hp - " + hpDiffOne + " vs " + hpDiffTwo + ") " + _playerOneName + " win " + pointDiff + " points";
					L2ItemInstance item = _playerOne.getInventory().addItem(6651, 30, 0, "Olympiad");
					InventoryUpdate iu = new InventoryUpdate();
					iu.addModifiedItem(item);
					_playerOne.sendPacket(iu);

					SystemMessage sm = new SystemMessage(SystemMessage.EARNED_S2_S1_s);
					sm.addItemName(item.getItemId());
					sm.addNumber(30);
					_playerOne.sendPacket(sm);
				}
				catch(Exception e)
				{
					e.printStackTrace();
				}
			}
			else if(_playerTwo == null || playerOneHp == 0 || hpDiffOne > hpDiffTwo)
			{
				int pointDiff;
				pointDiff = playerOnePoints / 3;
				playerTwoStat.set(POINTS, playerTwoPoints + pointDiff);
				playerOneStat.set(POINTS, playerOnePoints - pointDiff);
				playerOneStat.set(COMP_LOOSE, playerOneLoose + 1);
				playerTwoStat.set(COMP_WIN, playerTwoWin + 1);

				_sm.addString(_playerTwo.getName());
				broadcastMessage(_sm, true);
				_sm2.addString(_playerTwo.getName());
				_sm2.addNumber(pointDiff);
				broadcastMessage(_sm2, true);
				_sm3.addString(_playerOne.getName());
				_sm3.addNumber(pointDiff);
				broadcastMessage(_sm3, true);

				try
				{
					result = " (" + playerOneHp + "hp vs " + playerTwoHp + "hp - " + hpDiffOne + " vs " + hpDiffTwo + ") " + _playerTwoName + " win " + pointDiff + " points";
					L2ItemInstance item = _playerTwo.getInventory().addItem(6651, 30, 0, "Olympiad");
					InventoryUpdate iu = new InventoryUpdate();
					iu.addModifiedItem(item);
					_playerTwo.sendPacket(iu);

					SystemMessage sm = new SystemMessage(SystemMessage.EARNED_S2_S1_s);
					sm.addItemName(item.getItemId());
					sm.addNumber(30);
					_playerTwo.sendPacket(sm);
				}
				catch(Exception e)
				{
					e.printStackTrace();
				}
			}
			else
			{
				result = " tie";
				_sm = new SystemMessage(SystemMessage.THE_GAME_ENDED_IN_A_TIE);
				broadcastMessage(_sm, true);
			}
			_log.info("Olympia Result: " + _playerOneName + " vs " + _playerTwoName + " ... " + result);

			playerOneStat.set(COMP_DONE, playerOnePlayed + 1);
			playerTwoStat.set(COMP_DONE, playerTwoPlayed + 1);

			_nobles.remove(_playerOneID);
			_nobles.remove(_playerTwoID);

			_nobles.put(_playerOneID, playerOneStat);
			_nobles.put(_playerTwoID, playerTwoStat);

			_sm = new SystemMessage(SystemMessage.YOU_WILL_GO_BACK_TO_THE_VILLAGE_IN_S1_SECOND_S);
			_sm.addNumber(20);
			broadcastMessage(_sm, true);
		}

		protected void additions()
		{
			for(L2Player player : _players)
				try
				{
					//Set HP/CP/MP to Max
					player.setCurrentCp(player.getMaxCp());
					player.setCurrentHp(player.getMaxHp());
					player.setCurrentMp(player.getMaxMp());

					//Buff ww to both
					L2Skill skill;
					SystemMessage sm;

					skill = SkillTable.getInstance().getInfo(1204, 2);
					skill.getEffects(player, player, false);
					player.broadcastPacket(new MagicSkillUse(player, player, skill.getId(), 2, skill.getSkillTime(), 0));
					sm = new SystemMessage(SystemMessage.YOU_CAN_FEEL_S1S_EFFECT);
					sm.addSkillName((short) 1204, (short) 2);
					player.sendPacket(sm);

					if(!player.isMageClass())
					{
						//Buff haste to non-mages
						skill = SkillTable.getInstance().getInfo(1086, 1);
						skill.getEffects(player, player, false);
						player.broadcastPacket(new MagicSkillUse(player, player, skill.getId(), 1, skill.getSkillTime(), 0));
						sm = new SystemMessage(SystemMessage.YOU_CAN_FEEL_S1S_EFFECT);
						sm.addSkillName((short) 1086, (short) 1);
						player.sendPacket(sm);
					}
					else
					{
						//Buff acumen to mages
						skill = SkillTable.getInstance().getInfo(1085, 1);
						skill.getEffects(player, player, false);
						player.broadcastPacket(new MagicSkillUse(player, player, skill.getId(), 1, skill.getSkillTime(), 0));
						sm = new SystemMessage(SystemMessage.YOU_CAN_FEEL_S1S_EFFECT);
						sm.addSkillName((short) 1085, (short) 1);
						player.sendPacket(sm);
					}
				}
				catch(Exception e)
				{}
		}

		protected boolean makePlayersVisible()
		{
			try
			{
				for(L2Player player : _players)
				{
					player.setInvisible(false);
					player.broadcastUserInfo();
					if(player.getPet() != null)
						player.getPet().updateAbnormalEffect();
				}
			}
			catch(NullPointerException e)
			{
				e.printStackTrace();
				_aborted = true;
				return false;
			}
			return true;
		}

		protected boolean makePlayersInvisible()
		{
			_sm = new SystemMessage(SystemMessage.STARTS_THE_GAME);
			try
			{
				for(L2Player player : _players)
				{
					player.setInvisible(false);
					player.broadcastUserInfo();
					player.sendPacket(_sm);
					if(player.getPet() != null)
						player.getPet().updateAbnormalEffect();
				}
			}
			catch(NullPointerException e)
			{
				e.printStackTrace();
				_aborted = true;
				return false;
			}
			return true;
		}

		protected boolean makeCompetitionStart()
		{
			if(_playerOne == null || _playerTwo == null)
			{
				_log.warning("CompStart: player is null");
				_aborted = true;
				return false;
			}

			_sm = new SystemMessage(SystemMessage.STARTS_THE_GAME);

			try
			{
				_playerOne.sendPacket(_sm);
				_playerTwo.sendPacket(_sm);
			}
			catch(Exception e)
			{
				e.printStackTrace();
				_aborted = true;
				return false;
			}
			return true;
		}

		protected int getId()
		{
			return _id;
		}

		protected String getTitle()
		{
			String msg = "";
			msg += _playerOneName + " : " + _playerTwoName;
			return msg;
		}

		protected L2Player[] getPlayers()
		{
			L2Player[] players = new L2Player[2];

			if(_playerOne == null || _playerTwo == null)
				return null;

			players[0] = _playerOne;
			players[1] = _playerTwo;

			return players;
		}

		protected List<L2Player> getSpectators()
		{
			return _spectators;
		}

		protected void addSpectator(L2Player spec)
		{
			_spectators.add(spec);
		}

		protected void removeSpectator(L2Player spec)
		{
			if(_spectators != null && _spectators.contains(spec))
				_spectators.remove(spec);
		}

		protected void clearSpectators()
		{
			if(_spectators != null)
			{
				for(L2Player pc : _spectators)
					try
					{
						if(!pc.inObserverMode())
							continue;
						pc.leaveOlympiadObserverMode();
					}
					catch(NullPointerException e)
					{}
				_spectators.clear();
			}
		}

		private void broadcastMessage(SystemMessage sm, boolean toAll)
		{
			try
			{
				_playerOne.sendPacket(sm);
			}
			catch(Exception e)
			{}
			try
			{
				_playerTwo.sendPacket(sm);
			}
			catch(Exception e)
			{}

			if(toAll && _spectators != null)
				for(L2Player spec : _spectators)
					try
					{
						spec.sendPacket(sm);
					}
					catch(NullPointerException e)
					{}
		}
	}
}
