package L2jFT.game.model.entity.sevensigns;

import javolution.util.FastList;
import L2jFT.game.ThreadPoolManager;
import L2jFT.util.Rnd;
import L2jFT.game.model.L2Spawn;
import L2jFT.game.tables.NpcTable;

import java.util.logging.Logger;

public class AutoSpawn
{
	private static Logger _log = Logger.getLogger(AutoSpawn.class.getName());

	private int _npcId;
	L2Spawn _spawn;
	private long _unspawnDelay;
	FastList<SpawnPoint> _points = new FastList<SpawnPoint>();
	boolean _isActive;
	private byte _side;

	public AutoSpawn(int npcId, byte side)
	{
		_npcId = npcId;
		_side = side;
		try
		{
			_spawn = new L2Spawn(NpcTable.getTemplate(npcId));
			_spawn.stopRespawn();
		}
		catch(ClassNotFoundException e)
		{
			_log.warning("SevenSigns: AutoSpawn: Can't create npc with id " + npcId + ", " + e);
		}
	}

	public void setSpawn(boolean start)
	{
		if(_side != SevenSigns.getInstance().CURRENT_WINNER)
			return;

		_isActive = start;
		if(start)
		{
			if(_points.size() > 1)
				startRandomSpawn();
			else
			{
				_spawn.setLocx(_points.get(0).getX());
				_spawn.setLocy(_points.get(0).getY());
				_spawn.setLocz(_points.get(0).getZ());
				_spawn.setHeading(_points.get(0).getHeading());
				_spawn.setAmount(1);
				_spawn.stopRespawn();
				_spawn.doSpawn(true);
			}
		}
		else if(_spawn.getLastSpawn() != null)
			_spawn.getLastSpawn().deleteMe();
	}

	void startRandomSpawn()
	{
		ThreadPoolManager.getInstance().scheduleGeneral(new Runnable(){
			public void run()
			{
				if(!_isActive)
				{
					for(SpawnPoint sp : _points)
						sp.setActive(false);
					return;
				}

				if(_spawn.getLastSpawn() != null)
					_spawn.getLastSpawn().deleteMe();

				SpawnPoint sp;
				while(!(sp = _points.get(Rnd.get(0, _points.size()))).isActive()) //http://trac.fortress.com.ua/trac.cgi/ticket/88#preview
				{} //тут ничего не делаем, просто нужна не активная точка

				sp.setActive(true);
				_spawn.setLocx(sp.getX());
				_spawn.setLocy(sp.getY());
				_spawn.setLocz(sp.getZ());
				_spawn.setHeading(sp.getHeading());
				_spawn.doSpawn(true);

				for(SpawnPoint sp1 : _points)
					if(sp1 != sp)
						sp1.setActive(false);

				startRandomSpawn();
			}
		}, _unspawnDelay);
	}

	public boolean isActive()
	{
		return _isActive;
	}

	public int getNpcId()
	{
		return _npcId;
	}

	public void setUnspawnDelay(long d)
	{
		_unspawnDelay = d;
	}

	public void addSpawnPoint(int x, int y, int z, int heading)
	{
		_points.add(new SpawnPoint(x, y, z, heading));
	}
}
