package L2jFT.game.model.entity.sevensigns;

import javolution.util.FastList;
import javolution.util.FastMap;
import L2jFT.database.DatabaseUtils;
import L2jFT.database.FiltredPreparedStatement;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.database.ThreadConnection;
import L2jFT.game.ThreadPoolManager;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;
import static L2jFT.game.model.entity.sevensigns.Seal.TYPE_AVARICE;
import static L2jFT.game.model.entity.sevensigns.Seal.TYPE_GNOSIS;
import static L2jFT.game.model.entity.sevensigns.Seal.TYPE_NONE;
import static L2jFT.game.model.entity.sevensigns.Seal.TYPE_STRIFE;
import L2jFT.game.network.serverpackets.SignsSky;
import L2jFT.game.network.serverpackets.SystemMessage;

import java.sql.ResultSet;
import java.util.Calendar;
import java.util.logging.Logger;

public final class SevenSigns
{
	private static Logger _log = Logger.getLogger(SevenSigns.class.getName());

	public static final byte PERIOD_COMPETITION_PREPARE = 0;
	public static final byte PERIOD_COMPETITION = 1;
	public static final byte PERIOD_COMPETITION_RESULT = 2;
	public static final byte PERIOD_VALIDATION = 3;

	public static final int SEAL_STONE_BLUE_ID = 6360;
	public static final int SEAL_STONE_GREEN_ID = 6361;
	public static final int SEAL_STONE_RED_ID = 6362;

	public static long MILLIS_IN_WEAK = 3600 * 24 * 7 * 1000;

	public static byte SIDE_NONE = 0;
	public static byte SIDE_DUSK = 1;
	public static byte SIDE_DAWN = 2;

	public byte CURRENT_WINNER;
	public byte CURRENT_PERIOD;
	public int CURRENT_CYCLE;
	public long LAST_CYCLE_CHANGE_TIME;

	public FastMap<Byte, Seal> _seals = new FastMap<Byte, Seal>();
	public FastMap<Integer, Score> _scores = new FastMap<Integer, Score>();
	public FastList<AutoSpawn> _spawns = new FastList<AutoSpawn>();

	private static SevenSigns _instance;

	private int[] _dawnPriestsId = {
			31078, 31079, 31080, 31081, 31082, 31083, 31084, 31168, 31692, 31694,
			31997 };
	private int[] _duskPriestsId = {
			31085, 31086, 31087, 31088, 31089, 31090, 31091, 31169, 31693, 31695,
			31998 };

	public static SevenSigns getInstance()
	{
		if(_instance == null)
			_instance = new SevenSigns();
		return _instance;
	}

	private SevenSigns()
	{
		//Инициализируем печати
		_seals.put(TYPE_AVARICE, new Seal(TYPE_AVARICE));
		_seals.put(TYPE_GNOSIS, new Seal(TYPE_GNOSIS));
		_seals.put(TYPE_STRIFE, new Seal(TYPE_STRIFE));

		restoreData();
		_instance = this;
		startPeriodTask();
		if(CURRENT_PERIOD > 1)
			setSpawn(true);
		//else TODO: старт фестиваля
		//SevenSignsFestival.getInstance().startFestivalManager();

		_log.info("SS_NEW: Cycle: " + CURRENT_CYCLE + ", PERIOD: " + CURRENT_PERIOD);
	}

	private void restoreData()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rs = null;

		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			//Востановление инфы печатей
			try
			{
				statement = con.prepareStatement("SELECT * FROM ss_data ORDER BY cycle DESC");
				rs = statement.executeQuery();

				byte loaded = 0;

				while(rs.next())
				{
					int cycle = rs.getInt("cycle");
					byte period = rs.getByte("period");
					byte winner = rs.getByte("winner");
					byte avarice = rs.getByte("owner_avarice");
					byte gnosis = rs.getByte("owner_gnosis");
					byte strife = rs.getByte("owner_strife");
					long last_cycle_change_time = rs.getLong("last_cycle_change_time");

					if(loaded == 0)
					{
						CURRENT_CYCLE = cycle;
						CURRENT_PERIOD = period;
						CURRENT_WINNER = winner;
						_seals.get(TYPE_AVARICE).setOwner(avarice);
						_seals.get(TYPE_GNOSIS).setOwner(gnosis);
						_seals.get(TYPE_STRIFE).setOwner(strife);
						LAST_CYCLE_CHANGE_TIME = last_cycle_change_time * 1000;
					}
					else if(loaded == 1)
					{
						_seals.get(TYPE_AVARICE).setPreviousOwner(avarice);
						_seals.get(TYPE_GNOSIS).setPreviousOwner(gnosis);
						_seals.get(TYPE_STRIFE).setPreviousOwner(strife);
					}

					loaded++;

					//Загружаем только 2 последних периода
					if(loaded == 2)
						break;
				}

				//первый запуск, если сегодня не понедельник, то ставим прошлый понедельник первым днем
				//если понедельник, то ставим переуд как периуд подготовки, и длится он будет до 18-00, после чего сменится
				//на периуд сревнования
				if(loaded == 0)
				{
					Calendar c = Calendar.getInstance();
					c.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);
					c.set(Calendar.HOUR_OF_DAY, 17);
					c.set(Calendar.MINUTE, 45);
					c.set(Calendar.SECOND, 0);
					c.set(Calendar.MILLISECOND, 0);

					Calendar c2 = Calendar.getInstance();
					if(c2.get(Calendar.DAY_OF_WEEK) != Calendar.MONDAY)
						LAST_CYCLE_CHANGE_TIME = c.getTimeInMillis();
					else
						LAST_CYCLE_CHANGE_TIME = c.getTimeInMillis() + MILLIS_IN_WEAK;

					CURRENT_PERIOD = PERIOD_COMPETITION_PREPARE;
				}
			}
			catch(Exception e1)
			{
				_log.warning("SevenSigns: Can't restore data from ss_data " + e1);
				throw e1;
			}
			finally
			{
				DatabaseUtils.closeDatabaseSR(statement, rs);
			}
			_log.info("SevenSigns: Cycle info restored.");

			//востановление инфы игроков
			try
			{
				statement = con.prepareStatement("SELECT * FROM ss_data_players");
				rs = statement.executeQuery();

				while(rs.next())
				{
					int player_id = rs.getInt("player_id");
					byte side = rs.getByte("side");
					byte seal = rs.getByte("seal");
					int stones_dawn_red = rs.getInt("stones_red_dawn");
					int stones_dawn_green = rs.getInt("stones_green_dawn");
					int stones_dawn_blue = rs.getInt("stones_blue_dawn");
					int stones_dusk_red = rs.getInt("stones_red_dusk");
					int stones_dusk_green = rs.getInt("stones_green_dusk");
					int stones_dusk_blue = rs.getInt("stones_blue_dusk");

					Score score = new Score(player_id, side, seal);

					score.setStoneCountRed(stones_dawn_red, SIDE_DAWN);
					score.setStoneCountRed(stones_dusk_red, SIDE_DUSK);
					score.setStoneCountGreen(stones_dawn_green, SIDE_DAWN);
					score.setStoneCountGreen(stones_dusk_green, SIDE_DUSK);
					score.setStoneCountBlue(stones_dawn_blue, SIDE_DAWN);
					score.setStoneCountBlue(stones_dusk_blue, SIDE_DUSK);

					_scores.put(player_id, score);
				}
			}
			catch(Exception e2)
			{
				_log.warning("SevenSigns: Can't restore data from ss_data_players " + e2);
				throw e2;
			}
			finally
			{
				DatabaseUtils.closeStatement(statement);
			}

			//востановление спавнов
			try
			{
				statement = con.prepareStatement("SELECT * FROM ss_data_spawn");
				rs = statement.executeQuery();

				while(rs.next())
				{
					int npcId = rs.getInt("npc_id");
					int x = rs.getInt("x");
					int y = rs.getInt("y");
					int z = rs.getInt("z");
					int heading = rs.getInt("heading");
					long despawn_delay = rs.getInt("despawn_delay") * 1000; // mysql info is in seconds
					boolean isRandom = rs.getBoolean("isRandomSpawn");
					byte side = rs.getByte("side");

					if(isRandom)
					{
						boolean added = false;
						for(AutoSpawn as : _spawns)
							if(as.getNpcId() == npcId)
							{
								as.addSpawnPoint(x, y, z, heading);
								added = true;
								break;
							}

						if(!added)
						{
							AutoSpawn as = new AutoSpawn(npcId, side);
							as.addSpawnPoint(x, y, z, heading);
							as.setUnspawnDelay(despawn_delay);
							_spawns.add(as);
						}
					}
					else
					{
						AutoSpawn as = new AutoSpawn(npcId, side);
						as.addSpawnPoint(x, y, z, heading);
						as.setUnspawnDelay(despawn_delay);
						_spawns.add(as);
					}
				}
			}
			catch(Exception e3)
			{
				_log.warning("SevenSigns: Can't restore spawn data from ss_data_spawn " + e3);
				throw e3;
			}
			finally
			{
				DatabaseUtils.closeDatabaseSR(statement, rs);
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeConnection(con);
		}

	}

	private void storeData()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		//сохранение инфы о печятях в бд
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			try
			{
				statement = con.prepareStatement("REPLACE INTO ss_data VALUES (?,?,?,?,?,?,?)");
				statement.setInt(1, CURRENT_CYCLE);
				statement.setInt(2, CURRENT_PERIOD);
				statement.setInt(3, CURRENT_WINNER);
				statement.setByte(4, CURRENT_WINNER != SIDE_NONE ? _seals.get(TYPE_AVARICE).getOwner() : SIDE_NONE);
				statement.setByte(5, CURRENT_WINNER != SIDE_NONE ? _seals.get(TYPE_GNOSIS).getOwner() : SIDE_NONE);
				statement.setByte(6, CURRENT_WINNER != SIDE_NONE ? _seals.get(TYPE_STRIFE).getOwner() : SIDE_NONE);
				statement.setLong(7, LAST_CYCLE_CHANGE_TIME / 1000);
				statement.execute();
				DatabaseUtils.closeStatement(statement);

				//Чистка базы от ненужных записей
				statement = con.prepareStatement("DELETE FROM ss_data WHERE cycle < ?");
				statement.setInt(1, CURRENT_CYCLE - 1);
				statement.execute();
			}
			catch(Exception e1)
			{
				_log.warning("SevenSigns: Can't save data into ss_data " + e1);
				throw e1;
			}
			finally
			{
				DatabaseUtils.closeStatement(statement);
				statement = null;
			}

			//соханение инфы о игроках
			try
			{
				statement = con.prepareStatement("REPLACE INTO ss_data_players VALUES (?,?,?,?,?,?,?,?,?)");

				//сохраняем камушки игроков
				for(Score ps : _scores.values())
				{
					statement.setInt(1, ps.getPlayer());
					statement.setByte(2, ps.getSide());
					statement.setByte(3, ps.getSeal());
					statement.setInt(4, ps.getStoneCountRed(SIDE_DAWN));
					statement.setInt(5, ps.getStoneCountRed(SIDE_DUSK));
					statement.setInt(6, ps.getStoneCountGreen(SIDE_DAWN));
					statement.setInt(7, ps.getStoneCountGreen(SIDE_DUSK));
					statement.setInt(8, ps.getStoneCountBlue(SIDE_DAWN));
					statement.setInt(9, ps.getStoneCountBlue(SIDE_DUSK));
					statement.execute();
				}
				DatabaseUtils.closeStatement(statement);

				//чистим от возможного мусора
				statement = con.prepareStatement("DELETE FROM ss_data_players WHERE seal = 0 AND side = 0 AND stones_red_dawn = 0 AND stones_red_dusk = 0 AND stones_green_dawn = 0 AND stones_green_dusk = 0 AND stones_blue_dawn = 0 AND stones_blue_dusk = 0");
				statement.execute();
			}
			catch(Exception e2)
			{
				_log.warning("SevenSigns: Can't save data into ss_data_players " + e2);
				throw e2;
			}
			finally
			{
				DatabaseUtils.closeStatement(statement);
				statement = null;
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}

	}

	private void startPeriodTask()
	{
		long timeToChange = getMillisToPeriodChange();
		if(timeToChange < 1000)
			timeToChange = 1000;

		ThreadPoolManager.getInstance().scheduleGeneral(new Runnable(){
			public void run()
			{
				changePeriod();
			}
		}, timeToChange);
	}

	/**
	 * Периуд который закончился.
	 */
	public void changePeriod()
	{
		switch(CURRENT_PERIOD)
		{
			case 0:

				sendMessageToAllPlayers(SystemMessage.SEVEN_SIGNS_THE_QUEST_EVENT_PERIOD_HAS_BEGUN_VISIT_A_PRIEST_OF_DAWN_OR_DUSK_TO_PARTICIPATE_IN_THE_EVENT);

				CURRENT_PERIOD++;
				break;
			case 1:

				sendMessageToAllPlayers(SystemMessage.SEVEN_SIGNS_THE_QUEST_EVENT_PERIOD_HAS_ENDED_THE_NEXT_QUEST_EVENT_WILL_START_IN_ONE_WEEK);

				calcWinner();

				//возможность ничьи.
				if(CURRENT_WINNER == SIDE_DUSK)
					sendMessageToAllPlayers(SystemMessage.SEVEN_SIGNS_THE_REVOLUTIONARIES_OF_DUSK_HAVE_WON);
				else if(CURRENT_WINNER == SIDE_DAWN)
					sendMessageToAllPlayers(SystemMessage.SEVEN_SIGNS_THE_LORDS_OF_DAWN_HAVE_WON);

				// небеса.
				//sendSky(null, true); TODO: исправить пакет под новые печати
				//SevenSignsFestival.getInstance().getFestivalManagerSchedule().cancel(false); TODO: старт фестиваля

				CURRENT_PERIOD++;
				break;
			case 2:

				sendMessageToAllPlayers(SystemMessage.SEVEN_SIGNS_THE_SEAL_VALIDATION_PERIOD_HAS_BEGUN);

				setSpawn(true);
				CURRENT_PERIOD++;
				break;
			case 3:

				sendMessageToAllPlayers(SystemMessage.SEVEN_SIGNS_THE_SEAL_VALIDATION_PERIOD_HAS_ENDED);

				for(Score ps : _scores.values())
				{
					ps.setSeal(TYPE_NONE);
					ps.setSide(SIDE_NONE);
				}
				setSpawn(false);

				for(Seal s : _seals.values())
				{
					s.setPreviousOwner(s.getPreviousOwner());
					s.setOwner(SIDE_NONE);
				}

				if(CURRENT_WINNER != SIDE_NONE)
					for(Score s : _scores.values())
					{
						s.setSide(SIDE_NONE);
						s.setSeal(TYPE_NONE);
						s.setStonesToZero(CURRENT_WINNER);
					}
				nextCycle();
				CURRENT_PERIOD = PERIOD_COMPETITION_PREPARE;
				CURRENT_WINNER = SIDE_NONE;

				//sendSky(null, true); TODO: исправить пакет под новые печати
				//SevenSignsFestival.getInstance().startFestivalManager(); TODO: остановка фестиваля
				break;
		}
		storeData();
		startPeriodTask();

		_log.info("SS_NEW: Cycle: " + CURRENT_CYCLE + ", PERIOD: " + CURRENT_PERIOD);
	}

	private long getMillisToPeriodChange()
	{
		long curTime = System.currentTimeMillis();

		while(LAST_CYCLE_CHANGE_TIME + MILLIS_IN_WEAK * 2 < curTime)
		{
			for(Seal s : _seals.values())
			{
				s.setPreviousOwner(s.getOwner());
				s.setOwner(SIDE_NONE);
			}

			CURRENT_WINNER = SIDE_NONE;
			nextCycle();
			storeData();
		}

		curTime = System.currentTimeMillis(); // если цикл будет долго работать

		//Календарь который мы устанавливаем на нужное время
		Calendar c = Calendar.getInstance();
		c.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);
		c.set(Calendar.SECOND, 0);
		c.set(Calendar.MILLISECOND, 0);

		//Календарь для сверки
		Calendar c2 = Calendar.getInstance();
		switch(CURRENT_PERIOD)
		{
			case 0: //Периуд подготовки, 17:45 - 18:00
				c.set(Calendar.HOUR_OF_DAY, 18);
				c.set(Calendar.MINUTE, 0);

				if(LAST_CYCLE_CHANGE_TIME + 60 * 60 * 24 * 1000 < curTime)
					return 1000; //если в прошлый понедельник выкличили, а в следующий включили

				if(c2.get(Calendar.DAY_OF_WEEK) != Calendar.MONDAY)
					return 1000; // надо сменить, т.к. только в понедельник может длится периуд
				else if(c2.get(Calendar.HOUR_OF_DAY) < 18)
					return c.getTimeInMillis();
				else
					return 1000; // надо сменить, т.к. периуд длится 15 минут
			case 1: //Периуд соревнования, 18:00 - 18:00
				c.set(Calendar.HOUR_OF_DAY, 18);
				c.set(Calendar.MINUTE, 0);

				if(LAST_CYCLE_CHANGE_TIME + MILLIS_IN_WEAK + 15 * 60 * 1000 < curTime) // сервер просидел в оффлайне
					return 1000;
				else if(c.getTimeInMillis() <= LAST_CYCLE_CHANGE_TIME + 15 * 60 * 1000) // если время получилось минимально возможное
					return c.getTimeInMillis() + MILLIS_IN_WEAK;
				else
					return c.getTimeInMillis();
			case 2: //Периуд подсчета, 18:00 - 18:15
				c.set(Calendar.HOUR_OF_DAY, 18);
				c.set(Calendar.MINUTE, 15);

				if(LAST_CYCLE_CHANGE_TIME + MILLIS_IN_WEAK + 30 * 60 * 1000 < curTime) //если пропустили
					return 1000;

				if(c2.get(Calendar.DAY_OF_WEEK) != Calendar.MONDAY) // только в понедельник возможна смена, иначе пропустили
					return 1000;

				if(c2.get(Calendar.HOUR_OF_DAY) < 18)
					return c.getTimeInMillis();

				if(c2.get(Calendar.HOUR_OF_DAY) < 18 || c2.get(Calendar.HOUR_OF_DAY) == 18 && c2.get(Calendar.MINUTE) < 15)
					return c.getTimeInMillis();

				return 1000;
			case 3: //Периуд владения, 18:15 - 17:45
				c.set(Calendar.HOUR, 17);
				c.set(Calendar.MINUTE, 45);

				if(LAST_CYCLE_CHANGE_TIME + MILLIS_IN_WEAK * 2 < curTime) //был в оффлайне
					return 1000;

				if(c2.get(Calendar.DAY_OF_WEEK) != Calendar.MONDAY)
					return c.getTimeInMillis() + MILLIS_IN_WEAK;

				if(c.get(Calendar.HOUR_OF_DAY) < 17 || c2.get(Calendar.HOUR_OF_DAY) == 17 && c2.get(Calendar.MINUTE) < 45)
					return c.getTimeInMillis();

				return 1000;
			default:
				_log.warning("PeriodError in SevenSigns");
				return Long.MAX_VALUE;
		}
	}

	private void nextCycle()
	{
		CURRENT_CYCLE++;
		LAST_CYCLE_CHANGE_TIME += MILLIS_IN_WEAK * 2;
	}

	private void setSpawn(boolean start)
	{
		if(CURRENT_WINNER == SIDE_NONE)
			return;

		for(AutoSpawn as : _spawns)
			as.setSpawn(start);
	}

	/**
	 * Send info on the current Seven Signs period to the specified player.
	 *
	 * @param player кому посылать
	 * @param broadcast если тру, то будет рассылка всем игрокам, player может быть null.
	 */
	public void sendCurrentPeriodMsg(L2Player player, boolean broadcast)
	{
		SystemMessage sm = null;

		switch(CURRENT_PERIOD)
		{
			case PERIOD_COMPETITION_PREPARE:
				sm = new SystemMessage(SystemMessage.SEVEN_SIGNS_PREPARATIONS_HAVE_BEGUN_FOR_THE_NEXT_QUEST_EVENT);
				break;
			case PERIOD_COMPETITION:
				sm = new SystemMessage(SystemMessage.SEVEN_SIGNS_THE_QUEST_EVENT_PERIOD_HAS_BEGUN_SPEAK_WITH_A_PRIEST_OF_DAWN_OR_DUSK_PRIESTESS_IF_YOU_WISH_TO_PARTICIPATE_IN_THE_EVENT);
				break;
			case PERIOD_COMPETITION_RESULT:
				sm = new SystemMessage(SystemMessage.SEVEN_SIGNS_QUEST_EVENT_HAS_ENDED_RESULTS_ARE_BEING_TALLIED);
				break;
			case PERIOD_VALIDATION:
				sm = new SystemMessage(SystemMessage.SEVEN_SIGNS_THIS_IS_THE_SEAL_VALIDATION_PERIOD_A_NEW_QUEST_EVENT_PERIOD_BEGINS_NEXT_MONDAY);
				break;
		}

		if(broadcast)
			for(L2Player p : L2World.getAllPlayers())
				p.sendPacket(sm);
		else
			player.sendPacket(sm);
	}

	public void sendMessageToAllPlayers(int messageId)
	{
		SystemMessage sm = new SystemMessage(messageId);
		for(L2Player p : L2World.getAllPlayers())
			p.sendPacket(sm);
	}

	private void calcWinner()
	{
		//TODO: поддержка фестиваля тьмы, адену перевести в %-е соотношение к 500.

		long duskScore = 0;
		long dawnScore = 0;

		int dusk_avarice_players = 0;
		int dawn_avarice_players = 0;

		int dusk_gnosis_players = 0;
		int dawn_gnosis_players = 0;

		int dusk_strife_players = 0;
		int dawn_strife_players = 0;

		for(Score s : _scores.values())
		{
			if(s.getSide() == SIDE_NONE || s.getSeal() == TYPE_NONE)
				continue;

			// Avarice
			if(s.getSeal() == TYPE_AVARICE)
			{
				if(s.getSide() == SIDE_DUSK)
				{
					duskScore += s.getToalAdenaCount(SIDE_DUSK);
					dusk_avarice_players++;
				}
				else
				{
					dawnScore += s.getToalAdenaCount(SIDE_DAWN);
					dawn_avarice_players++;
				}
			}
			else if(s.getSeal() == TYPE_GNOSIS)
			{
				if(s.getSide() == SIDE_DUSK)
				{
					duskScore += s.getToalAdenaCount(SIDE_DUSK);
					dusk_gnosis_players++;
				}
				else
				{
					dawnScore += s.getToalAdenaCount(SIDE_DAWN);
					dawn_gnosis_players++;
				}
			}
			else if(s.getSeal() == TYPE_STRIFE)
				if(s.getSide() == SIDE_DUSK)
				{
					duskScore += s.getToalAdenaCount(SIDE_DUSK);
					dusk_strife_players++;
				}
				else
				{
					dawnScore += s.getToalAdenaCount(SIDE_DAWN);
					dawn_strife_players++;
				}
		}

		int dusk_total_players = dusk_avarice_players + dusk_strife_players + dusk_gnosis_players;
		int dawn_total_players = dawn_avarice_players + dawn_strife_players + dawn_gnosis_players;

		// Определение победителя
		if(duskScore > dawnScore)
			CURRENT_WINNER = SIDE_DUSK;
		else if(dawnScore > duskScore)
			CURRENT_WINNER = SIDE_DAWN;
		else
			CURRENT_WINNER = SIDE_NONE;

		// обсчет процентов для победителей
		float percent_avarice;
		float percent_gnosis;
		float percent_strife;

		if(CURRENT_WINNER == SIDE_DUSK)
		{
			percent_avarice = Math.round(dusk_total_players / 100 * dusk_avarice_players);
			percent_gnosis = Math.round(dusk_total_players / 100 * dusk_gnosis_players);
			percent_strife = Math.round(dusk_total_players / 100 * dusk_strife_players);
		}
		else
		{
			percent_avarice = Math.round(dawn_total_players / 100 * dawn_avarice_players);
			percent_gnosis = Math.round(dawn_total_players / 100 * dawn_gnosis_players);
			percent_strife = Math.round(dawn_total_players / 100 * dawn_strife_players);
		}

		// владение печятями
		int neededPercent;
		float countPercent;
		for(Seal s : _seals.values())
		{
			if(CURRENT_WINNER == SIDE_NONE)
			{
				s.setOwner(SIDE_NONE);
				continue;
			}

			//10% если прошлый овнер равен теперешнему
			//35% если нет
			neededPercent = s.getPreviousOwner() == CURRENT_WINNER ? 10 : 35;

			if(s.getType() == TYPE_AVARICE)
				countPercent = percent_avarice;
			else if(s.getType() == TYPE_GNOSIS)
				countPercent = percent_gnosis;
			else
				countPercent = percent_strife;

			if(countPercent >= neededPercent)
				s.setOwner(CURRENT_WINNER);
			else
				s.setOwner(SIDE_NONE);
		}
	}

	public int getPriestCabal(int id)
	{
		for(int i : _dawnPriestsId)
			if(i == id)
				return SIDE_DAWN;

		for(int i : _duskPriestsId)
			if(i == id)
				return SIDE_DUSK;

		return SIDE_NONE;
	}

	public void contributeStones(short itemId, int count, L2Player player)
	{
		Score s = _scores.get(player.getObjectId());

		if(s == null || s.getSide() == SIDE_NONE || s.getSeal() == TYPE_NONE || CURRENT_WINNER != SIDE_NONE)
			return;

		s.makeContribution(itemId, count, player);
	}

	public void sendSky(L2Player player, boolean broadcast)
	{
		SignsSky ss = new SignsSky();
		if(broadcast)
			for(L2Player p : L2World.getAllPlayers())
				p.sendPacket(ss);
		else
			player.sendPacket(ss);
	}

	public void register(L2Player player, byte side, byte seal)
	{
		Score s = _scores.get(player.getObjectId());

		if(s == null)
		{
			s = new Score(player.getObjectId(), side, seal);
			_scores.put(player.getObjectId(), s);
			return;
		}

		s.setSide(side);
		s.setSeal(seal);
	}
}
