package L2jFT.game.model.entity.siege;

import L2jFT.database.DatabaseUtils;
import L2jFT.database.FiltredPreparedStatement;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.database.ThreadConnection;
import L2jFT.game.instancemanager.SiegeManager;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2SiegeClan.SiegeClanType;

import java.sql.ResultSet;

public class Database
{
	Siege _siege;

	public Database(Siege siege)
	{
		_siege = siege;
	}

	/** Clear all registered siege clans from database for castle */
	public void clearSiegeClan()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("DELETE FROM siege_clans WHERE castle_id=?");
			statement.setInt(1, _siege.getCastle().getCastleId());
			statement.execute();
			DatabaseUtils.closeStatement(statement);
			if(_siege.getCastle().getOwnerId() > 0)
			{
				statement = con.prepareStatement("DELETE FROM siege_clans WHERE clan_id=?");
				statement.setInt(1, _siege.getCastle().getOwnerId());
				statement.execute();
			}

			_siege.getAttackerClans().clear();
			_siege.getDefenderClans().clear();
			_siege.getDefenderWaitingClans().clear();
		}
		catch(Exception e)
		{
			System.out.println("Exception: clearSiegeClan(): " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	/** Clear all siege clans waiting for approval from database for castle */
	public void clearSiegeWaitingClan()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("DELETE FROM siege_clans WHERE castle_id=? and type = 2");
			statement.setInt(1, _siege.getCastle().getCastleId());
			statement.execute();

			_siege.getDefenderWaitingClans().clear();
		}
		catch(Exception e)
		{
			System.out.println("Exception: clearSiegeWaitingClan(): " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	/**
	 * Remove clan from siege<BR><BR>
	 * @param clanId The int of player's clan id
	 */
	public void removeSiegeClan(int clanId)
	{
		if(clanId <= 0)
			return;
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("DELETE FROM siege_clans WHERE castle_id=? and clan_id=?");
			statement.setInt(1, _siege.getCastle().getCastleId());
			statement.setInt(2, clanId);
			statement.execute();
			loadSiegeClan();
		}
		catch(Exception e)
		{}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	/** Load siege clans. */
	public void loadSiegeClan()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			_siege.getAttackerClans().clear();
			_siege.getDefenderClans().clear();
			_siege.getDefenderWaitingClans().clear();
			// Add castle owner as defender
			if(_siege.getCastle().getOwnerId() > 0)
				_siege.addDefender(_siege.getCastle().getOwnerId(), SiegeClanType.OWNER);
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT clan_id, type FROM siege_clans where castle_id = ?");
			statement.setInt(1, _siege.getCastle().getCastleId());
			rset = statement.executeQuery();
			int typeId;
			while(rset.next())
			{
				typeId = rset.getInt("type");
				if(typeId == 0)
					_siege.addDefender(rset.getInt("clan_id"), SiegeClanType.DEFENDER);
				else if(typeId == 1)
					_siege.addAttacker(rset.getInt("clan_id"));
				else if(typeId == 2)
					_siege.addDefenderWaiting(rset.getInt("clan_id"));
			}
		}
		catch(Exception e)
		{
			System.out.println("Exception: loadSiegeClan(): " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	/** Save siege date to database. */
	public void saveSiegeDate()
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("UPDATE castle SET siegeDate = ? WHERE id = ?");
			statement.setLong(1, _siege.getSiegeDate().getTimeInMillis() / 1000);
			statement.setInt(2, _siege.getCastle().getCastleId());
			statement.execute();
		}
		catch(Exception e)
		{
			System.out.println("Exception: saveSiegeDate(): " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	/**
	 * Save registration to database.<BR><BR>
	 * @param clan The L2Clan of player
	 * @param typeId 0 = defender, 1 = attacker, 2 = defender waiting
	 */
	public void saveSiegeClan(L2Clan clan, int typeId, @SuppressWarnings("unused")
	boolean isUpdateRegistration)
	{
		if(clan == null || clan.getHasCastle() > 0)
			return;

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			if(typeId == 0 || typeId == 2)
			{
				if(_siege.getDefenderClans().size() + _siege.getDefenderWaitingClans().size() >= SiegeManager.getInstance().getDefenderMaxClans())
					return;
			}
			else if(_siege.getAttackerClans().size() >= SiegeManager.getInstance().getAttackerMaxClans())
				return;

			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("REPLACE INTO siege_clans (clan_id,castle_id,type,castle_owner) values (?,?,?,0)");
			statement.setInt(1, clan.getClanId());
			statement.setInt(2, _siege.getCastle().getCastleId());
			statement.setInt(3, typeId);
			statement.execute();

			if(typeId == 0 || typeId == -1)
				_siege.addDefender(clan.getClanId(), SiegeClanType.DEFENDER);
			else if(typeId == 1)
				_siege.addAttacker(clan.getClanId());
			else if(typeId == 2)
				_siege.addDefenderWaiting(clan.getClanId());
		}
		catch(Exception e)
		{
			System.out.println("Exception: saveSiegeClan(L2Clan clan, int typeId, boolean isUpdateRegistration): " + e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}
}
