package L2jFT.game.model.entity.siege;

import java.util.Calendar;
import java.util.List;

import javolution.util.FastList;
import javolution.util.FastMap;
import L2jFT.Config;
import L2jFT.extensions.listeners.L2ZoneEnterLeaveListener;
import L2jFT.extensions.scripts.Functions;
import L2jFT.game.ThreadPoolManager;
import L2jFT.game.idfactory.IdFactory;
import L2jFT.game.instancemanager.MercTicketManager;
import L2jFT.game.instancemanager.SiegeGuardManager;
import L2jFT.game.instancemanager.SiegeManager;
import L2jFT.game.instancemanager.ZoneManager;
import L2jFT.game.instancemanager.SiegeManager.SiegeSpawn;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2SiegeClan;
import L2jFT.game.model.L2Spawn;
import L2jFT.game.model.L2World;
import L2jFT.game.model.L2Zone;
import L2jFT.game.model.L2SiegeClan.SiegeClanType;
import L2jFT.game.model.L2Zone.ZoneType;
import L2jFT.game.model.entity.Castle;
import L2jFT.game.model.instances.L2ArtefactInstance;
import L2jFT.game.model.instances.L2ControlTowerInstance;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.network.serverpackets.RelationChanged;
import L2jFT.game.network.serverpackets.SiegeInfo;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.ClanTable;
import L2jFT.game.tables.MapRegionTable;
import L2jFT.game.tables.NpcTable;
import L2jFT.game.templates.L2NpcTemplate;
import L2jFT.util.Location;

public class Siege
{
	// ==========================================================================================
	// Message to add/check
	//  id=17  msg=[Castle siege has begun.] c3_attr1=[SystemMsg_k.17]
	//  id=18  msg=[Castle siege is over.]   c3_attr1=[SystemMsg_k.18]
	//  id=288 msg=[The castle gate has been broken down.]
	//  id=291 msg=[Clan $s1 is victorious over $s2's castle siege!]
	//  id=292 msg=[$s1 has announced the castle siege time.]
	//  - id=293 msg=[The registration term for $s1 has ended.]
	//  - id=358 msg=[$s1 hour(s) until castle siege conclusion.]
	//  - id=359 msg=[$s1 minute(s) until castle siege conclusion.]
	//  - id=360 msg=[Castle siege $s1 second(s) left!]
	//  id=640 msg=[You have failed to refuse castle defense aid.]
	//  id=641 msg=[You have failed to approve castle defense aid.]
	//  id=644 msg=[You are not yet registered for the castle siege.]
	//  - id=645 msg=[Only clans with Level 4 and higher may register for a castle siege.]
	//  id=646 msg=[You do not have the authority to modify the castle defender list.]
	//  - id=688 msg=[The clan that owns the castle is automatically registered on the defending side.]
	//  id=689 msg=[A clan that owns a castle cannot participate in another siege.]
	//  id=690 msg=[You cannot register on the attacking side because you are part of an alliance with the clan that owns the castle.]
	//  id=718 msg=[The castle gates cannot be opened and closed during a siege.]
	//  - id=295 msg=[$s1's siege was canceled because there were no clans that participated.]
	//  id=659 msg=[This is not the time for siege registration and so registrations cannot be accepted or rejected.]
	//  - id=660 msg=[This is not the time for siege registration and so registration and cancellation cannot be done.]
	//  id=663 msg=[The siege time has been declared for $s. It is not possible to change the time after a siege time has been declared. Do you want to continue?]
	//  id=667 msg=[You are registering on the attacking side of the $s1 siege. Do you want to continue?]
	//  id=668 msg=[You are registering on the defending side of the $s1 siege. Do you want to continue?]
	//  id=669 msg=[You are canceling your application to participate in the $s1 siege battle. Do you want to continue?]
	//  id=707 msg=[You cannot teleport to a village that is in a siege.]
	//  - id=711 msg=[The siege of $s1 has started.]
	//  - id=712 msg=[The siege of $s1 has finished.]
	//  id=844 msg=[The siege to conquer $s1 has begun.]
	//  - id=845 msg=[The deadline to register for the siege of $s1 has passed.]
	//  - id=846 msg=[The siege of $s1 has been canceled due to lack of interest.]
	//  - id=856 msg=[The siege of $s1 has ended in a draw.]
	//  id=285 msg=[Clan $s1 has succeeded in engraving the ruler!]
	//  - id=287 msg=[The opponent clan has begun to engrave the ruler.]

	public static enum TeleportWhoType
	{
		All,
		Attacker,
		Defender,
		Owner,
		Spectator
	}

	private FastMap<Integer, L2SiegeClan> _attackerClans = new FastMap<Integer, L2SiegeClan>(); // L2SiegeClan
	private FastMap<Integer, L2SiegeClan> _defenderClans = new FastMap<Integer, L2SiegeClan>(); // L2SiegeClan
	private FastMap<Integer, L2SiegeClan> _defenderWaitingClans = new FastMap<Integer, L2SiegeClan>(); // L2SiegeClan
	private int _Defender_RespawnDelay_Penalty;

	private int _ownerBeforeStart;

	// Castle setting
	private List<L2ArtefactInstance> _artifacts = new FastList<L2ArtefactInstance>();
	private List<L2ControlTowerInstance> _controlTowers = new FastList<L2ControlTowerInstance>();
	private Castle _castle;
	private boolean _isInProgress = false;
	protected boolean _isRegistrationOver = false;
	private Calendar _siegeDate;
	protected Calendar _siegeEndDate;
	private SiegeGuardManager _siegeGuardManager;
	protected Calendar _SiegeRegistrationEndDate;

	private static final short[] SIEGE_SUMMONS = {
			1459, 14768, 14769, 14770, 14771, 14772, 14773, 14774, 14775, 14776,
			14777, 14778, 14779, 14780, 14781, 14782, 14783, 14784, 14785, 14786,
			14787, 14788, 14789, 14790, 14791, 14792, 14793, 14794, 14795, 14796,
			14797, 14798, 14839 };

	private Database _database = new Database(this);

	public Siege(Castle castle)
	{
		_castle = castle;
		_siegeDate = Calendar.getInstance();
		_siegeGuardManager = new SiegeGuardManager(getCastle());
	}

	private final TrapPacketSender trapListener = new TrapPacketSender();

	public Database getDatabase()
	{
		return _database;
	}

	/**
	 * When siege ends<BR><BR>
	 */
	public void endSiege()
	{
		getZone().setActive(false);
		getZone().getListenerEngine().removeMethodInvokedListener(trapListener); // Слушаетель убирается после деактивации т.к. зона выполнит выход
		if(isInProgress())
		{
			announceToPlayer(new SystemMessage(SystemMessage.THE_SIEGE_OF_S1_HAS_FINISHED).addString(getCastle().getName()), false);

			if(getCastle().getOwnerId() <= 0)
				announceToPlayer(new SystemMessage(SystemMessage.THE_SIEGE_OF_S1_HAS_ENDED_IN_A_DRAW).addString(getCastle().getName()), false);
			else
			{
				L2Clan oldOwner = null;
				if(_ownerBeforeStart != 0)
					oldOwner = ClanTable.getInstance().getClan(_ownerBeforeStart);
				L2Clan newOwner = ClanTable.getInstance().getClan(getCastle().getOwnerId());

				if(oldOwner == null)
				{ // castle was taken over from scratch
					if(newOwner.getLevel() >= 5)
						newOwner.broadcastToOnlineMembers(new SystemMessage(SystemMessage.SINCE_YOUR_CLAN_EMERGED_VICTORIOUS_FROM_THE_SIEGE_S1_POINTS_HAVE_BEEN_ADDED_TO_YOUR_CLAN_REPUTATION_SCORE).addNumber(newOwner.incReputation(1000, true)));
				}
				else if(newOwner.equals(oldOwner))
				{ // castle was defended
					if(newOwner.getLevel() >= 5)
						newOwner.broadcastToOnlineMembers(new SystemMessage(SystemMessage.SINCE_YOUR_CLAN_EMERGED_VICTORIOUS_FROM_THE_SIEGE_S1_POINTS_HAVE_BEEN_ADDED_TO_YOUR_CLAN_REPUTATION_SCORE).addNumber(newOwner.incReputation(500, true)));
				}
				else
				{ // castle was taken over by another clan
					announceToPlayer(new SystemMessage(SystemMessage.CLAN_S1_IS_VICTORIOUS_OVER_S2S_CASTLE_SIEGE).addString(newOwner.getName()).addString(getCastle().getName()), false);
					if(newOwner.getLevel() >= 5)
						newOwner.broadcastToOnlineMembers(new SystemMessage(SystemMessage.SINCE_YOUR_CLAN_EMERGED_VICTORIOUS_FROM_THE_SIEGE_S1_POINTS_HAVE_BEEN_ADDED_TO_YOUR_CLAN_REPUTATION_SCORE).addNumber(newOwner.incReputation(1000, true)));
					if(oldOwner.getLevel() >= 5)
						oldOwner.broadcastToOnlineMembers(new SystemMessage(SystemMessage.YOUR_CLAN_HAS_FAILED_TO_DEFEND_THE_CASTLE_S1_POINTS_HAVE_BEEN_DEDUCTED_FROM_YOUR_CLAN_REPUTATION_SCORE).addNumber(-oldOwner.incReputation(-2000, true)));
				}
			}

			removeFlags(); // Removes all flags. Note: Remove flag before teleporting players
			teleportPlayer(TeleportWhoType.Attacker, MapRegionTable.TeleportWhereType.ClosestTown); // Teleport to the closest town
			teleportPlayer(TeleportWhoType.Spectator, MapRegionTable.TeleportWhereType.ClosestTown); // Teleport to the closest town
			removeSiegeSummons();
			_isInProgress = false; // Flag so that siege instance can be started
			updatePlayerSiegeStateFlags(true);
			saveCastleSiege(); // Save castle specific data
			_database.clearSiegeClan(); // Clear siege clan from db
			respawnArtifacts(); // Remove artifact from this castle
			respawnControlTowers(); // Remove all control tower from this castle
			getSiegeGuardManager().unspawnSiegeGuard(); // Remove all spawned siege guard from this castle
			if(getCastle().getOwnerId() > 0)
				getSiegeGuardManager().removeMercs();
			getCastle().spawnDoor(); // Respawn door to castle
			defendersUpdate(true);
			attackersUpdate(true);
		}
	}

	private void removeDefender(L2SiegeClan sc)
	{
		if(sc != null)
			getDefenderClans().remove(sc.getClanId());
	}

	private void removeAttacker(L2SiegeClan sc)
	{
		if(sc != null)
			getAttackerClans().remove(sc.getClanId());
	}

	private void addDefender(L2SiegeClan sc, SiegeClanType type)
	{
		if(sc == null)
			return;
		sc.setTypeId(type);
		_defenderClans.put(sc.getClanId(), sc);
	}

	private void addAttacker(L2SiegeClan sc)
	{
		if(sc == null)
			return;
		sc.setTypeId(SiegeClanType.ATTACKER);
		_attackerClans.put(sc.getClanId(), sc);
	}

	/**
	 * When control of castle changed during siege<BR><BR>
	 */
	public void midVictory()
	{
		if(isInProgress()) // Siege still in progress
		{
			if(getCastle().getOwnerId() > 0)
				getSiegeGuardManager().removeMercs(); // Remove all merc entry from db

			if(getDefenderClans().size() == 0 && // If defender doesn't exist (Pc vs Npc)
			getAttackerClans().size() == 1 // Only 1 attacker
			)
			{
				L2SiegeClan sc_newowner = getAttackerClan(getCastle().getOwnerId());
				removeAttacker(sc_newowner);
				addDefender(sc_newowner, SiegeClanType.OWNER);
				endSiege();
				return;
			}
			if(getCastle().getOwnerId() > 0)
			{
				int allyId = ClanTable.getInstance().getClan(getCastle().getOwnerId()).getAllyId();
				if(getDefenderClans().size() == 0)
					// The player's clan is in an alliance
					if(allyId != 0)
					{
						boolean allinsamealliance = true;
						for(L2SiegeClan sc : getAttackerClans().values())
							if(sc != null && sc.getClan().getAllyId() != allyId)
								allinsamealliance = false;
						if(allinsamealliance)
						{
							L2SiegeClan sc_newowner = getAttackerClan(getCastle().getOwnerId());
							removeAttacker(sc_newowner);
							addDefender(sc_newowner, SiegeClanType.OWNER);
							endSiege();
							return;
						}
					}

				for(L2SiegeClan sc : getDefenderClans().values())
					if(sc != null)
					{
						removeDefender(sc);
						addAttacker(sc);
					}

				L2SiegeClan sc_newowner = getAttackerClan(getCastle().getOwnerId());
				removeAttacker(sc_newowner);
				addDefender(sc_newowner, SiegeClanType.OWNER);

				// The player's clan is in an alliance
				if(allyId != 0)
				{
					L2Clan[] clanList = ClanTable.getInstance().getClans();
					for(L2Clan clan : clanList)
						if(clan.getAllyId() == allyId)
						{
							L2SiegeClan sc = getAttackerClan(clan.getClanId());
							if(sc != null)
							{
								removeAttacker(sc);
								addDefender(sc, SiegeClanType.DEFENDER);
							}
						}
				}
				teleportPlayer(Siege.TeleportWhoType.Attacker, MapRegionTable.TeleportWhereType.SiegeFlag); // Teleport to the siege flag town
				teleportPlayer(Siege.TeleportWhoType.Spectator, MapRegionTable.TeleportWhereType.ClosestTown); // Teleport to the closest town
				removeFlags(); // Remove all flags
				getCastle().removeUpgrade(); // Remove all castle upgrade
				getCastle().spawnDoor(true); // Respawn door to castle but make them weaker (50% hp)
				defendersUpdate(false);
				attackersUpdate(false);
				updatePlayerSiegeStateFlags(false);
			}
		}
	}

	/**
	 * When siege starts<BR><BR>
	 */
	public void startSiege()
	{
		if(!isInProgress())
		{
			if(getAttackerClans().size() <= 0)
			{
				if(getCastle().getOwnerId() <= 0)
					announceToPlayer(new SystemMessage(SystemMessage.THE_SIEGE_OF_S1_HAS_BEEN_CANCELED_DUE_TO_LACK_OF_INTEREST).addString(getCastle().getName()), false);
				else
					announceToPlayer(new SystemMessage(SystemMessage.S1S_SIEGE_WAS_CANCELED_BECAUSE_THERE_WERE_NO_CLANS_THAT_PARTICIPATED).addString(getCastle().getName()), false);
				return;
			}

			// Слушатель добавляется перед активацией т.к. зона выполнит вход
			getZone().getListenerEngine().addMethodInvokedListener(trapListener);
			getZone().setActive(true);

			_isInProgress = true; // Flag so that same siege instance cannot be started again
			_ownerBeforeStart = getCastle().getOwnerId();

			_database.loadSiegeClan(); // Load siege clan from db
			updatePlayerSiegeStateFlags(false);
			teleportPlayer(TeleportWhoType.Attacker, MapRegionTable.TeleportWhereType.ClosestTown); // Teleport to the closest town
			teleportPlayer(TeleportWhoType.Spectator, MapRegionTable.TeleportWhereType.ClosestTown); // Teleport to the closest town
			respawnArtifacts(); // Respawn artifacts
			respawnControlTowers(); // Respawn control towers
			getCastle().spawnDoor(); // Spawn door
			spawnSiegeGuard(); // Spawn siege guard
			defendersUpdate(false); // Add defenders to list
			attackersUpdate(false); // Add attackers to list
			MercTicketManager.getInstance().deleteTickets(getCastle().getCastleId()); // remove the tickets from the ground
			_Defender_RespawnDelay_Penalty = 0; // Reset respawn delay
			// Schedule a task to prepare auto siege end
			_siegeEndDate = Calendar.getInstance();
			_siegeEndDate.add(Calendar.MINUTE, SiegeManager.getInstance().getSiegeLength());
			ThreadPoolManager.getInstance().scheduleGeneral(new EndSiegeTask(this), 1000); // Prepare auto end task
			announceToPlayer(new SystemMessage(SystemMessage.THE_TEMPORARY_ALLIANCE_OF_THE_CASTLE_ATTACKER_TEAM_IS_IN_EFFECT_IT_WILL_BE_DISSOLVED_WHEN_THE_CASTLE_LORD_IS_REPLACED), false);
			announceToPlayer(new SystemMessage(SystemMessage.THE_SIEGE_OF_S1_HAS_STARTED).addString(getCastle().getName()), false);
		}
	}

	/**
	 * Announce to player.<BR><BR>
	 * @param message The String of the message to send to player
	 * @param inAreaOnly The boolean flag to show message to players in area only.
	 */
	public static void announceToPlayer(SystemMessage message, boolean inAreaOnly)
	{
		// Get all players
		for(L2Player player : L2World.getAllPlayers())
			if(player != null && (!inAreaOnly || player.isInZone(ZoneType.Siege)))
				player.sendPacket(message);
	}

	public void updatePlayerSiegeStateFlags(boolean clear)
	{
		L2Clan clan;
		for(L2SiegeClan siegeClan : getAttackerClans().values())
		{
			clan = ClanTable.getInstance().getClan(siegeClan.getClanId());
			for(L2Player member : clan.getOnlineMembers(""))
			{
				if(clear)
					member.setSiegeState(0);
				else
					member.setSiegeState(1);
				for(L2Character pl : member.getKnownCharacters(Config.PLAYER_VISIBILITY))
				{
					if(!pl.isPlayer)
						continue;
					pl.sendPacket(new RelationChanged(member, member.isAutoAttackable(pl), member.getRelation((L2Player) pl)));
				}
			}
		}
		for(L2SiegeClan siegeclan : getDefenderClans().values())
		{
			clan = ClanTable.getInstance().getClan(siegeclan.getClanId());
			for(L2Player member : clan.getOnlineMembers(""))
			{
				if(clear)
					member.setSiegeState(0);
				else
					member.setSiegeState(2);
				for(L2Character pl : member.getKnownCharacters(Config.PLAYER_VISIBILITY))
				{
					if(!pl.isPlayer)
						continue;
					pl.sendPacket(new RelationChanged(member, member.isAutoAttackable(pl), member.getRelation((L2Player) pl)));
				}
			}
		}
	}

	/**
	 * Approve clan as defender for siege<BR><BR>
	 * @param clanId The int of player's clan id
	 */
	public void approveSiegeDefenderClan(int clanId)
	{
		if(clanId <= 0)
			return;
		_database.saveSiegeClan(ClanTable.getInstance().getClan(clanId), 0, true);
		_database.loadSiegeClan();
	}

	/** Return true if object is inside the zone */
	public boolean checkIfInZone(int x, int y)
	{
		return isInProgress() && (getCastle().checkIfInZone(x, y) || getZone().checkIfInZone(x, y));
	}

	/**
	 * Return true if clan is attacker<BR><BR>
	 * @param clan The L2Clan of the player
	 */
	public boolean checkIsAttacker(L2Clan clan)
	{
		return getAttackerClan(clan) != null;
	}

	/**
	 * Return true if clan is defender<BR><BR>
	 * @param clan The L2Clan of the player
	 */
	public boolean checkIsDefender(L2Clan clan)
	{
		return getDefenderClan(clan) != null;
	}

	/**
	 * Return true if clan is defender waiting approval<BR><BR>
	 * @param clan The L2Clan of the player
	 */
	public boolean checkIsDefenderWaiting(L2Clan clan)
	{
		return getDefenderWaitingClan(clan) != null;
	}

	/**
	 * Return true if clan is registered to the siege<BR><BR>
	 * @param clanId The clan id of the player
	 */
	public boolean checkIsClanRegistered(int clanId)
	{
		return getAttackerClan(clanId) != null || getDefenderClan(clanId) != null || getDefenderWaitingClan(clanId) != null;
	}

	/** Return list of L2Player in the zone. */
	public List<L2Player> getPlayersInZone()
	{
		List<L2Player> players = new FastList<L2Player>();
		for(L2Object object : getZone().getObjects())
			if(object.isPlayer)
				players.add((L2Player) object);
		return players;
	}

	/** Return list of L2Player registered as attacker in the zone. */
	public List<L2Player> getAttackersInZone()
	{
		List<L2Player> players = new FastList<L2Player>();
		for(L2Player player : getPlayersInZone())
			if(getAttackerClan(player.getClan()) != null)
				players.add(player);
		return players;
	}

	/** Return list of L2Player registered as defender in the zone. */
	public List<L2Player> getDefendersInZone()
	{
		List<L2Player> players = new FastList<L2Player>();
		int ownerId = getCastle().getOwnerId();
		for(L2Player player : getPlayersInZone())
		{
			if(player.getClan().getClanId() == ownerId)
				continue;
			if(getDefenderClan(player.getClan()) != null)
				players.add(player);
		}
		return players;
	}

	/** Return list of L2Player owning the castle in the zone. */
	public List<L2Player> getOwnersInZone()
	{
		List<L2Player> players = new FastList<L2Player>();
		int ownerId = getCastle().getOwnerId();
		if(ownerId > 0)
			for(L2Player player : getPlayersInZone())
				if(player.getClan() != null && player.getClan().getClanId() == ownerId)
					players.add(player);
		return players;
	}

	/** Return list of L2Player not registered as attacker or defender in the zone. */
	public List<L2Player> getSpectatorsInZone()
	{
		List<L2Player> players = new FastList<L2Player>();
		for(L2Player player : getPlayersInZone())
			if(player.getClan() == null || getAttackerClan(player.getClan()) == null && getDefenderClan(player.getClan()) == null)
				players.add(player);
		return players;
	}

	/** Control Tower was skilled */
	public void killedCT()
	{
		_Defender_RespawnDelay_Penalty += SiegeManager.getInstance().getControlTowerLosePenalty(); // Add respawn penalty to defenders for each control tower lose
	}

	/** Remove the flag that was killed */
	public void killedFlag(L2NpcInstance flag)
	{
		if(flag == null)
			return;
		for(L2SiegeClan clan : getAttackerClans().values())
			if(clan != null && clan.getFlag() == flag)
			{
				clan.removeFlag();
				return;
			}
	}

	/** Display list of registered clans */
	public void listRegisterClan(L2Player player)
	{
		player.sendPacket(new SiegeInfo(getCastle()));
	}

	/**
	 * Register clan as attacker<BR><BR>
	 * @param player The L2Player of the player trying to register
	 */
	public void registerAttacker(L2Player player)
	{
		registerAttacker(player, false);
	}

	public void registerAttacker(L2Player player, boolean force)
	{
		if(player.getClan() == null)
			return;
		int allyId = 0;
		if(getCastle().getOwnerId() != 0)
			allyId = ClanTable.getInstance().getClan(getCastle().getOwnerId()).getAllyId();
		if(allyId != 0)
			if(player.getClan().getAllyId() == allyId && !force)
			{
				player.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_REGISTER_ON_THE_ATTACKING_SIDE_BECAUSE_YOU_ARE_PART_OF_AN_ALLIANCE_WITH_THE_CLAN_THAT_OWNS_THE_CASTLE));
				return;
			}
		if(force || checkIfCanRegister(player))
			_database.saveSiegeClan(player.getClan(), 1, false); // Save to database
	}

	/**
	 * Register clan as defender<BR><BR>
	 * @param player The L2Player of the player trying to register
	 */
	public void registerDefender(L2Player player)
	{
		registerDefender(player, false);
	}

	public void registerDefender(L2Player player, boolean force)
	{
		if(getCastle().getOwnerId() <= 0)
			player.sendMessage("You cannot register as a defender because " + getCastle().getName() + " is owned by NPC.", "Вы не можете зарегистрироваться как защитник потому, что " + getCastle().getName() + " захвачен NPC.");
		else if(force || checkIfCanRegister(player))
			_database.saveSiegeClan(player.getClan(), 2, false); // Save to database
	}

	/**
	 * Remove clan from siege<BR><BR>
	 * @param player The L2Player of player/clan being removed
	 */
	public void removeSiegeClan(L2Player player)
	{
		removeSiegeClan(player.getClan());
	}

	/**
	 * Remove clan from siege<BR><BR>
	 * @param clan being removed
	 */
	public void removeSiegeClan(L2Clan clan)
	{
		if(clan == null || clan.getHasCastle() == getCastle().getCastleId() || !SiegeManager.getInstance().checkIsRegistered(clan, getCastle().getCastleId()))
			return;
		_database.removeSiegeClan(clan.getClanId());
	}

	/**
	 * Set siege date time<BR><BR>
	 * @param siegeDateTime The long of date time in millisecond
	 */
	// This method changes the spawn for the attacker
	public void setSiegeDateTime(long siegeDateTime)
	{
		_siegeDate.setTimeInMillis(siegeDateTime); // Set siege date
	}

	/**
	 * Start the auto tasks<BR><BR>
	 */
	public void startAutoTask()
	{
		correctSiegeDateTime();
		System.out.println("Siege of " + getCastle().getName() + ": " + _siegeDate.getTime());
		_database.loadSiegeClan();
		// Schedule registration end
		_SiegeRegistrationEndDate = Calendar.getInstance();
		_SiegeRegistrationEndDate.setTimeInMillis(_siegeDate.getTimeInMillis());
		_SiegeRegistrationEndDate.add(Calendar.DAY_OF_MONTH, -1);
		// Schedule siege auto start
		ThreadPoolManager.getInstance().scheduleGeneral(new StartSiegeTask(this), 1000);
	}

	/**
	 * Teleport players
	 */
	public void teleportPlayer(TeleportWhoType teleportWho, MapRegionTable.TeleportWhereType teleportWhere)
	{
		List<L2Player> players;
		switch(teleportWho)
		{
			case Owner:
				players = getOwnersInZone();
				break;
			case Attacker:
				players = getAttackersInZone();
				break;
			case Defender:
				players = getDefendersInZone();
				break;
			case Spectator:
				players = getSpectatorsInZone();
				break;
			default:
				players = getPlayersInZone();
		}
		for(L2Player player : players)
			if(player != null && !player.isGM())
			{
				if(player.getCastingSkill() != null && player.getCastingSkill().getId() == 246)
					player.abortCast();
				player.teleToLocation(MapRegionTable.getInstance().getTeleToLocation(player, teleportWhere));
			}
	}

	/**
	 * Add clan as attacker<BR><BR>
	 * @param clanId The int of clan's id
	 */
	public void addAttacker(int clanId)
	{
		addAttacker(new L2SiegeClan(clanId, SiegeClanType.ATTACKER));
	}

	/**
	 * <p>Add clan as defender with the specified type</p>
	 * @param clanId The int of clan's id
	 * @param type the type of the clan
	 */
	public void addDefender(int clanId, SiegeClanType type)
	{
		addDefender(new L2SiegeClan(clanId, type), type);
	}

	/**
	 * Add clan as defender waiting approval<BR><BR>
	 * @param clanId The int of clan's id
	 */
	public void addDefenderWaiting(int clanId)
	{
		addDefender(new L2SiegeClan(clanId, SiegeClanType.DEFENDER_PENDING), SiegeClanType.DEFENDER_PENDING);
	}

	/**
	 * Return true if the player can register.<BR><BR>
	 * @param player The L2Player of the player trying to register
	 * @return true if the player can register.
	 */
	private boolean checkIfCanRegister(L2Player player)
	{
		if(getIsRegistrationOver())
			player.sendMessage("The deadline to register for the siege of " + getCastle().getName() + " has passed.", "Крайний срок регистрации на осаду " + getCastle().getName() + " прошел.");
		else if(isInProgress())
			player.sendMessage("This is not the time for siege registration and so registration and cancellation cannot be done.", "В данный момент нельзя зарегистрироваться или отменить регистрацию на осаду");
		else if(player.getClan() == null || player.getClan().getLevel() < SiegeManager.getInstance().getSiegeClanMinLevel())
			player.sendMessage("Only clans with Level " + SiegeManager.getInstance().getSiegeClanMinLevel() + " and higher may register for a castle siege.", "Только кланы уровня " + SiegeManager.getInstance().getSiegeClanMinLevel() + " и выше могут участвовать в осаде.");
		else if(player.getClan().getHasCastle() > 0)
			player.sendPacket(new SystemMessage(SystemMessage.A_CLAN_THAT_OWNS_A_CASTLE_CANNOT_PARTICIPATE_IN_ANOTHER_SIEGE));
		else if(player.getClan().getClanId() == getCastle().getOwnerId())
			player.sendPacket(new SystemMessage(SystemMessage.THE_CLAN_THAT_OWNS_THE_CASTLE_IS_AUTOMATICALLY_REGISTERED_ON_THE_DEFENDING_SIDE));
		else if(SiegeManager.getInstance().checkIsRegistered(player.getClan(), getCastle().getCastleId()))
			player.sendMessage("You are already registered in a Siege.", "Вы уже зарегистрированы на осаду");
		else if((Boolean) Functions.callScripts("events.clanhall.CHSiegeManager", "isRegistered", new Object[] { player.getClan() }, null))
			player.sendMessage("You are already registered in a Siege.", "Вы уже зарегистрированы на осаду");
		else
			return true;

		return false;
	}

	/**
	 * Return the correct siege date as Calendar.<BR><BR>
	 */
	private void correctSiegeDateTime()
	{
		boolean corrected = false;
		if(_siegeDate.getTimeInMillis() < Calendar.getInstance().getTimeInMillis())
		{
			// Since siege has past reschedule it to the next one (14 days)
			// This is usually caused by server being down
			corrected = true;
			setNextSiegeDate();
		}
		if(_siegeDate.get(Calendar.DAY_OF_WEEK) != getCastle().getSiegeDayOfWeek())
		{
			corrected = true;
			_siegeDate.set(Calendar.DAY_OF_WEEK, getCastle().getSiegeDayOfWeek());
		}
		if(_siegeDate.get(Calendar.HOUR_OF_DAY) != getCastle().getSiegeHourOfDay())
		{
			corrected = true;
			_siegeDate.set(Calendar.HOUR_OF_DAY, getCastle().getSiegeHourOfDay());
		}
		_siegeDate.set(Calendar.MINUTE, 0);
		if(corrected)
			_database.saveSiegeDate();
	}

	/** Remove artifacts spawned. */
	private void respawnArtifacts()
	{
		// Remove all instance of artifact for this castle
		for(L2ArtefactInstance art : _artifacts)
			if(art != null)
			{
				art.decayMe();
				art.spawnMe();
			}
	}

	/** Remove all control tower spawned. */
	private void respawnControlTowers()
	{
		// Remove all instance of control tower for this castle
		for(L2ControlTowerInstance ct : _controlTowers)
			if(ct != null)
			{
				ct.decayMe();
				ct.getSpawn().respawnNpc(ct);
			}
	}

	/** Remove all flags. */
	private void removeFlags()
	{
		for(L2SiegeClan sc : getAttackerClans().values())
			if(sc != null)
				sc.removeFlag();
		for(L2SiegeClan sc : getDefenderClans().values())
			if(sc != null)
				sc.removeFlag();
	}

	/** Save castle siege related to database. */
	private void saveCastleSiege()
	{
		setNextSiegeDate(); // Set the next set date for 2 weeks from now
		_database.saveSiegeDate(); // Save the new date
		startAutoTask(); // Prepare auto start siege and end registration
	}

	/** Set the date for the next siege. */
	private void setNextSiegeDate()
	{
		if(_siegeDate.getTimeInMillis() < Calendar.getInstance().getTimeInMillis())
		{
			// Set next siege date if siege has passed
			_siegeDate.add(Calendar.DAY_OF_MONTH, 14); // Schedule to happen in 14 days
			_isRegistrationOver = false; // Allow registration for next siege

			if(_siegeDate.getTimeInMillis() < Calendar.getInstance().getTimeInMillis())
				setNextSiegeDate(); // Re-run again if still in the pass
		}
	}

	/**
	 * Спавнит артефакты для замка
	 * @param Id айдишник замка
	 */
	public void spawnArtifacts(int Id)
	{
		for(SiegeSpawn _sp : SiegeManager.getInstance().getArtefactSpawnList(Id))
		{
			L2ArtefactInstance art;
			art = new L2ArtefactInstance(IdFactory.getInstance().getNextId(), NpcTable.getTemplate(_sp.getNpcId()));
			art.setCurrentHpMp(art.getMaxHp(), art.getMaxMp());
			art.setHeading(_sp.getLocation().h);
			art.spawnMe(_sp.getLocation().x, _sp.getLocation().y, _sp.getLocation().z + 50);
			_artifacts.add(art);
		}
	}

	/**
	 * Spawn control tower.
	 * @param Id Castle ID
	 */
	public void spawnControlTowers(int Id)
	{
		for(SiegeSpawn _sp : SiegeManager.getInstance().getControlTowerSpawnList(Id))
			try
			{
				L2NpcTemplate template = NpcTable.getTemplate(_sp.getNpcId());
				template.baseHpMax = _sp.getHp();
				template.basePDef = 100;
				L2Spawn spawn = new L2Spawn(template);
				Location loc = _sp.getLocation();
				spawn.setLocx(loc.x);
				spawn.setLocy(loc.y);
				spawn.setLocz(loc.z);
				L2ControlTowerInstance tower = (L2ControlTowerInstance) spawn.doSpawn(true);
				spawn.stopRespawn();

				if(tower != null)
					_controlTowers.add(tower);

			}
			catch(ClassNotFoundException e)
			{
				e.printStackTrace(System.err);
			}
	}

	private void spawnSiegeGuard()
	{
		getSiegeGuardManager().spawnSiegeGuard();
	}

	public final Castle getCastle()
	{
		return _castle;
	}

	public final L2SiegeClan getAttackerClan(L2Clan clan)
	{
		if(clan == null)
			return null;
		return getAttackerClan(clan.getClanId());
	}

	public L2SiegeClan getAttackerClan(int clanId)
	{
		return _attackerClans.get(clanId);
	}

	public FastMap<Integer, L2SiegeClan> getAttackerClans()
	{
		return _attackerClans;
	}

	public int getAttackerRespawnDelay()
	{
		return SiegeManager.getInstance().getAttackerRespawnDelay();
	}

	public final L2SiegeClan getDefenderClan(L2Clan clan)
	{
		if(clan == null)
			return null;
		return getDefenderClan(clan.getClanId());
	}

	public final L2SiegeClan getDefenderClan(int clanId)
	{
		return _defenderClans.get(clanId);
	}

	public FastMap<Integer, L2SiegeClan> getDefenderClans()
	{
		return _defenderClans;
	}

	public final L2SiegeClan getDefenderWaitingClan(L2Clan clan)
	{
		if(clan == null)
			return null;
		return getDefenderWaitingClan(clan.getClanId());
	}

	public L2SiegeClan getDefenderWaitingClan(int clanId)
	{
		return _defenderWaitingClans.get(clanId);
	}

	public FastMap<Integer, L2SiegeClan> getDefenderWaitingClans()
	{
		return _defenderWaitingClans;
	}

	public final int getDefenderRespawnDelay()
	{
		return SiegeManager.getInstance().getDefenderRespawnDelay() + _Defender_RespawnDelay_Penalty;
	}

	public final boolean isInProgress()
	{
		return _isInProgress;
	}

	public final boolean getIsRegistrationOver()
	{
		return _isRegistrationOver;
	}

	public void setRegistrationOver(boolean value)
	{
		_isRegistrationOver = value;
	}

	public final Calendar getSiegeDate()
	{
		return _siegeDate;
	}

	public final Calendar getSiegeEndDate()
	{
		return _siegeEndDate;
	}

	public L2NpcInstance getFlag(L2Clan clan)
	{
		if(clan != null)
		{
			L2SiegeClan sc = getAttackerClan(clan);
			if(sc != null)
				return sc.getFlag();
		}
		return null;
	}

	public final SiegeGuardManager getSiegeGuardManager()
	{
		if(_siegeGuardManager == null)
			_siegeGuardManager = new SiegeGuardManager(getCastle());
		return _siegeGuardManager;
	}

	public final L2Zone getZone()
	{
		return ZoneManager.getInstance().getZoneByIndex(ZoneType.Siege, getCastle().getCastleId(), false);
	}

	private void defendersUpdate(boolean end)
	{
		for(L2SiegeClan clan : getDefenderClans().values())
			if(end)
			{
				clan.getClan().setSiege(null);
				clan.getClan().setIsDefender(false);
				clan.getClan().setIsAttacker(false);
			}
			else
			{
				clan.getClan().setSiege(this);
				clan.getClan().setIsDefender(true);
				clan.getClan().setIsAttacker(false);
			}
	}

	private void attackersUpdate(boolean end)
	{
		for(L2SiegeClan clan : getAttackerClans().values())
			if(end)
			{
				clan.getClan().setSiege(null);
				clan.getClan().setIsDefender(false);
				clan.getClan().setIsAttacker(false);
			}
			else
			{
				clan.getClan().setSiege(this);
				clan.getClan().setIsDefender(false);
				clan.getClan().setIsAttacker(true);
			}
	}

	public long getTimeRemaining()
	{
		return getSiegeDate().getTimeInMillis() - Calendar.getInstance().getTimeInMillis();
	}

	private void removeSiegeSummons()
	{
		for(L2Player player : getPlayersInZone())
			for(short id : SIEGE_SUMMONS)
				if(player.getPet() != null && id == player.getPet().getNpcId())
					player.getPet().unSummon();
	}

	/**
	 * Обновляет статус ловушек у текущей осады.
	 * Если игрок входит(enter == true), то будет отослано состояние трэпов.
	 * Если выходит, то трэпы будут простро выключены
	 * Если осада не запущена, то трепы выключатся.
	 * @param player игрок
	 * @param enter вход или выход игрока
	 * <p>
	 * TODO: обработка
	 */
	public void sendTrapStatus(L2Player player, boolean enter)
	{
		if(enter)
		{

			// TODO: player.sendPacket(new EventTrigger(...));
		}
		else
		{

			// TODO: player.sendPacket(new EventTrigger(...));
		}
	}

	/**
	 * Осадной зоне добавляется слушатель для входа/выхода объекта
	 */
	private class TrapPacketSender extends L2ZoneEnterLeaveListener
	{
		@Override
		public void objectEntered(L2Zone zone, L2Object object)
		{
			if(object.isPlayer)
			{
				L2Player player = (L2Player) object;
				sendTrapStatus(player, true);
				//System.out.println(player.getName() + " -> enter; " + zone);
			}
		}

		@Override
		public void objectLeaved(L2Zone zone, L2Object object)
		{
			if(object.isPlayer)
			{
				L2Player player = (L2Player) object;
				sendTrapStatus(player, false);
				//System.out.println(player.getName() + " -> exit; " + zone);
			}
		}
	}
}
