package L2jFT.game.model.entity.siege;

import L2jFT.game.ThreadPoolManager;
import L2jFT.game.network.serverpackets.SystemMessage;

import java.util.Calendar;

public class StartSiegeTask implements Runnable
{
	private Siege _siege;

	public StartSiegeTask(Siege siege)
	{
		_siege = siege;
	}

	public void run()
	{
		if(_siege.isInProgress())
			return;

		try
		{
			long timeRemaining = _siege.getSiegeDate().getTimeInMillis() - Calendar.getInstance().getTimeInMillis();
			if(timeRemaining > 86400000)
				ThreadPoolManager.getInstance().scheduleGeneral(new StartSiegeTask(_siege), timeRemaining - 86400000); // Prepare task for 24 before siege start to end registration
			else if(timeRemaining <= 86400000 && timeRemaining > 13600000)
			{
				Siege.announceToPlayer(new SystemMessage(SystemMessage.THE_REGISTRATION_TERM_FOR_S1_HAS_ENDED).addString(_siege.getCastle().getName()), false);
				_siege.setRegistrationOver(true);
				_siege.getDatabase().clearSiegeWaitingClan();
				ThreadPoolManager.getInstance().scheduleGeneral(new StartSiegeTask(_siege), timeRemaining - 13600000); // Prepare task for 1 hr left before siege start.
			}
			else if(timeRemaining <= 13600000 && timeRemaining > 600000)
			{
				Siege.announceToPlayer(new SystemMessage(SystemMessage.S1_MINUTE_S_UNTIL_CASTLE_SIEGE_CONCLUSION).addNumber(Math.round(timeRemaining / 60000)), false);
				Siege.announceToPlayer(new SystemMessage(SystemMessage.S1_S2).addString(Math.round(timeRemaining / 60000) + " minute(s) until " + _siege.getCastle().getName() + " siege begin."), false);
				ThreadPoolManager.getInstance().scheduleGeneral(new StartSiegeTask(_siege), timeRemaining - 600000); // Prepare task for 10 minute left.
			}
			else if(timeRemaining <= 600000 && timeRemaining > 300000)
			{
				Siege.announceToPlayer(new SystemMessage(SystemMessage.S1_S2).addString(Math.round(timeRemaining / 60000) + " minute(s) until " + _siege.getCastle().getName() + " siege begin."), false);
				ThreadPoolManager.getInstance().scheduleGeneral(new StartSiegeTask(_siege), timeRemaining - 300000); // Prepare task for 5 minute left.
			}
			else if(timeRemaining <= 300000 && timeRemaining > 10000)
			{
				Siege.announceToPlayer(new SystemMessage(SystemMessage.S1_S2).addString(Math.round(timeRemaining / 60000) + " minute(s) until " + _siege.getCastle().getName() + " siege begin."), false);
				ThreadPoolManager.getInstance().scheduleGeneral(new StartSiegeTask(_siege), timeRemaining - 10000); // Prepare task for 10 seconds count down
			}
			else if(timeRemaining <= 10000 && timeRemaining > 0)
			{
				Siege.announceToPlayer(new SystemMessage(SystemMessage.S1_S2).addString(_siege.getCastle().getName() + " siege " + Math.round(timeRemaining / 1000) + " second(s) to start!"), false);
				ThreadPoolManager.getInstance().scheduleGeneral(new StartSiegeTask(_siege), timeRemaining); // Prepare task for second count down
			}
			else
				_siege.startSiege();
		}
		catch(Throwable t)
		{}
	}
}