package L2jFT.game.model.instances;

import javolution.util.FastList;
import javolution.util.FastMap;
import L2jFT.Config;
import L2jFT.game.GameTimeController;
import L2jFT.game.ThreadPoolManager;
import L2jFT.game.network.clientpackets.Say2;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;
import L2jFT.game.network.serverpackets.CreatureSay;
import L2jFT.game.network.serverpackets.OnVehicleCheckLocation;
import L2jFT.game.network.serverpackets.PlaySound;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.network.serverpackets.VehicleDeparture;
import L2jFT.game.network.serverpackets.VehicleInfo;
import L2jFT.game.templates.L2CharTemplate;
import L2jFT.game.templates.L2Weapon;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.LineNumberReader;
import java.util.Collection;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.concurrent.Future;

public class L2BoatInstance extends L2Character
{
	protected static final Log _logBoat = LogFactory.getLog(L2BoatInstance.class.getName());

	public class L2BoatTrajet
	{
		public class L2BoatPoint
		{
			public int speed1;
			public int speed2;
			public int x;
			public int y;
			public int z;
			public int time;
		}

		public L2BoatTrajet(int idWaypoint1, int idWTicket1, int ntx1, int nty1, int ntz1, String npc1, String sysmess10_1, String sysmess5_1, String sysmess1_1, String sysmess0_1, String sysmessb_1, String boatname)
		{
			_IdWaypoint1 = idWaypoint1;
			_IdWTicket1 = idWTicket1;
			_ntx1 = ntx1;
			_nty1 = nty1;
			_ntz1 = ntz1;
			_npc1 = npc1;
			_sysmess10_1 = sysmess10_1;
			_sysmess5_1 = sysmess5_1;
			_sysmess1_1 = sysmess1_1;
			_sysmessb_1 = sysmessb_1;
			_sysmess0_1 = sysmess0_1;
			_boatname = boatname;
			loadBoatPath();
		}

		public void parseLine(String line)
		{
			//L2BoatPath bp = new L2BoatPath();
			_path = new FastMap<Integer, L2BoatPoint>();
			StringTokenizer st = new StringTokenizer(line, ";");
			Integer.parseInt(st.nextToken());
			_max = Integer.parseInt(st.nextToken());
			for(int i = 0; i < _max; i++)
			{
				L2BoatPoint bp = new L2BoatPoint();
				bp.speed1 = Integer.parseInt(st.nextToken());
				bp.speed2 = Integer.parseInt(st.nextToken());
				bp.x = Integer.parseInt(st.nextToken());
				bp.y = Integer.parseInt(st.nextToken());
				bp.z = Integer.parseInt(st.nextToken());
				bp.time = Integer.parseInt(st.nextToken());
				_path.put(i, bp);
			}
			return;
		}

		private void loadBoatPath()
		{
			LineNumberReader lnr = null;
			try
			{
				File doorData = new File(Config.DATAPACK_ROOT, "data/boatpath.csv");
				lnr = new LineNumberReader(new BufferedReader(new FileReader(doorData)));
				String line = null;
				while((line = lnr.readLine()) != null)
				{
					if(line.trim().length() == 0 || !line.startsWith(_IdWaypoint1 + ";"))
						continue;
					parseLine(line);
					return;
				}
				_logBoat.warn("No path for boat " + _boatname + " !!!");
			}
			catch(FileNotFoundException e)
			{
				_logBoat.warn("boatpath.csv is missing in data folder");
			}
			catch(Exception e)
			{
				_logBoat.warn("error while creating boat table " + e);
			}
			finally
			{
				try
				{
					if(lnr != null)
						lnr.close();
				}
				catch(Exception e1)
				{ /* ignore problems */}
			}
		}

		private Map<Integer, L2BoatPoint> _path;
		public int _IdWaypoint1;
		public int _IdWTicket1;
		public int _ntx1;
		public int _nty1;
		public int _ntz1;
		public int _max;
		public String _boatname;
		public String _npc1;
		public String _sysmess10_1;
		public String _sysmess5_1;
		public String _sysmess1_1;
		public String _sysmessb_1;
		public String _sysmess0_1;

		public int state(int state, L2BoatInstance _boat)
		{
			if(state < _max)
			{
				L2BoatPoint bp = _path.get(state);
				double dx = bp.x - _boat.getX();
				double dy = bp.y - _boat.getY();
				double distance = Math.sqrt(dx * dx + dy * dy);
				double cos = dx / distance;
				double sin = dy / distance;
				int heading = (int) (Math.atan2(-sin, -cos) * 10430.378350470452724949566316381);
				heading += 32768;
				_boat.setHeading(heading);
				//_speed = bp.speed1;
				_boat._vd = new VehicleDeparture(_boat, bp.speed1, bp.speed2, bp.x, bp.y, bp.z);
				//_boat.getTemplate().baseRunSpd = bp.speed1;				
				_boat.moveToLocation(bp.x, bp.y, bp.z, bp.speed1, false);
				//FastList<L2Player> knownPlayers = L2World.getAroundPlayers(_boat);
				//if(knownPlayers.isEmpty())
				//	return bp.time;
				for(L2Player player : L2World.getAllPlayers())
					//_logBoat.warn("Boat " + _name + " send VehicleDeparture to " + player);
					//_logBoat.warn("Boat " + _name + " (" + _x + ", " + _y + ", " + _z + ") to (" + bp.x + ", " + bp.y + ", " + bp.z);
					player.sendPacket(_boat._vd);
				if(bp.time == 0)
					bp.time = 1;
				return bp.time;
			}
			return 0;
		}
	}

	private String _name;
	protected L2BoatTrajet _t1;
	protected L2BoatTrajet _t2;
	protected int _cycle = 0;
	protected VehicleDeparture _vd = null;
	private Map<Integer, L2Player> _inboat;
	private Future _move_task;

	//private int _speed;

	public L2BoatInstance(int objectId, L2CharTemplate template, String name)
	{
		super(objectId, template);
		_name = name;
	}

	@Override
	public void updatePosition(int gameTicks)
	{
		if(!isMoving)
			return;
		if(gameTicks == 0)
			gameTicks = GameTimeController.getGameTicks();
		double distance = getDistance(_xDestination, _yDestination, _zDestination);
		if(distance < 5)
		{
			updatePeopleInTheBoat(_xDestination, _yDestination, _zDestination);
			setXYZ(_xDestination, _yDestination, _zDestination);
			// Cancel the move action
			isMoving = false;
			return;
		}
		updatePeopleInTheBoat(_x + (int) _xSpeedTicks, _y + (int) _ySpeedTicks, _z);
		setXYZ(_x + (int) _xSpeedTicks, _y + (int) _ySpeedTicks, _z);
		return;
	}

	@Override
	public boolean moveToLocation(int x, int y, int z, int speed, boolean pathfind)
	{
		// Calculate distance (dx,dy) between current position and destination
		long dx = x - _x;
		long dy = y - _y;
		long s = dx * dx + dy * dy;
		Double distance = Math.sqrt(s);
		if(distance.isNaN())
		{
			_logBoat.warn("distance is NaN!!! x = " + x + " y = " + y + " _x = " + _x + " _y = " + _y);
			distance = (double) 0;
		}

		if(Config.DEBUG)
			_logBoat.debug("distance to target:" + distance);

		// Define movement angles needed
		// ^
		// |     X (x,y)
		// |   /
		// |  /distance
		// | /
		// |/ angle
		// X ---------->
		// (curx,cury)

		double cos;
		double sin;
		sin = dy / distance;
		cos = dx / distance;

		// Calculate the xspeed and yspeed in unit/ticks in function of the movement speed
		_xSpeedTicks = (float) (cos * speed / GameTimeController.TICKS_PER_SECOND);
		_ySpeedTicks = (float) (sin * speed / GameTimeController.TICKS_PER_SECOND);

		// Calculate and set the heading of the L2Character
		int heading = (int) (Math.atan2(-sin, -cos) * 10430.378350470452724949566316381);
		heading += 32768;
		setHeading(heading);

		if(distance < 5)
		{
			ThreadPoolManager.getInstance().scheduleAi(new BoatArrived(this, x, y, z), 1);
			return true;
		}

		_xDestination = x;
		_yDestination = y;
		_zDestination = z;
		isMoving = true;

		// Caclulate the Nb of ticks between the current position and the destination
		double _ticksToMove = GameTimeController.TICKS_PER_SECOND * distance / speed;

		// If necessary set Nb ticks needed to a min value to ensure small distancies movements
		if(_ticksToMove < 1)
			_ticksToMove = 1;

		int tm = (int) (_ticksToMove * GameTimeController.MILLIS_IN_TICK + 0.5);

		// Create a task to notify the AI that L2Character arrives at a check point of the movement
		if(_move_task != null)
			_move_task.cancel(false);

		_move_task = ThreadPoolManager.getInstance().scheduleAi(new BoatArrived(this, x, y, z), tm);

		return true;
	}

	class BoatCaptain implements Runnable
	{
		private int _state;
		private L2BoatInstance _boat;

		public BoatCaptain(int i, L2BoatInstance instance)
		{
			_state = i;
			_boat = instance;
		}

		public void run()
		{
			BoatCaptain bc;
			switch(_state)
			{
				case 1:
					_boat.say(5);
					bc = new BoatCaptain(2, _boat);
					ThreadPoolManager.getInstance().scheduleGeneral(bc, 240000);
					break;
				case 2:
					_boat.say(1);
					bc = new BoatCaptain(3, _boat);
					ThreadPoolManager.getInstance().scheduleGeneral(bc, 40000);
					break;
				case 3:
					_boat.say(0);
					bc = new BoatCaptain(4, _boat);
					ThreadPoolManager.getInstance().scheduleGeneral(bc, 20000);
					break;
				case 4:
					_boat.say(-1);
					_boat.begin();
					break;
			}
		}
	}

	class Boatrun implements Runnable
	{
		private int _state;
		private L2BoatInstance _boat;

		public Boatrun(int i, L2BoatInstance instance)
		{
			_state = i;
			_boat = instance;
		}

		public void run()
		{
			_boat._vd = null;
			_boat.needOnVehicleCheckLocation = false;
			if(_boat._cycle == 1)
			{
				int time = _boat._t1.state(_state, _boat);
				//_logBoat.warn("Boat " + _name + " cycle: " + _boat._cycle + " time: " + time);
				if(time > 0)
				{
					_state++;
					Boatrun bc = new Boatrun(_state, _boat);
					ThreadPoolManager.getInstance().scheduleGeneral(bc, time);
				}
				else if(time == 0)
				{
					_boat._cycle = 2;
					_boat.say(10);
					BoatCaptain bc = new BoatCaptain(1, _boat);
					ThreadPoolManager.getInstance().scheduleGeneral(bc, 300000);
				}
				else
				{
					_boat.needOnVehicleCheckLocation = true;
					_state++;
					_boat._runstate = _state;
				}
			}
			else if(_boat._cycle == 2)
			{
				int time = _boat._t2.state(_state, _boat);
				//_logBoat.warn("Boat " + _name + " cycle: " + _boat._cycle + " time: " + time);
				if(time > 0)
				{
					_state++;
					Boatrun bc = new Boatrun(_state, _boat);
					ThreadPoolManager.getInstance().scheduleGeneral(bc, time);
				}
				else if(time == 0)
				{
					_boat._cycle = 1;
					_boat.say(10);
					BoatCaptain bc = new BoatCaptain(1, _boat);
					ThreadPoolManager.getInstance().scheduleGeneral(bc, 300000);
				}
				else
				{
					_boat.needOnVehicleCheckLocation = true;
					_state++;
					_boat._runstate = _state;
				}
			}
		}
	}

	public int _runstate = 0;

	class BoatArrived implements Runnable
	{
		private L2BoatInstance _boat;
		private int x1;
		private int y1;
		private int z1;

		public BoatArrived(L2BoatInstance instance, int x, int y, int z)
		{
			_boat = instance;
			x1 = x;
			y1 = y;
			z1 = z;
		}

		public void run()
		{
			//_logBoat.warn("Boat " + _name + " EvtArrived: runstate = " + _runstate);

			updatePeopleInTheBoat(x1, y1, z1);
			setXYZ(x1, y1, z1);

			if(_runstate == 0)
			{
				//DO nothing :P
			}
			else
			{
				//_runstate++;
				Boatrun bc = new Boatrun(_runstate, _boat);
				ThreadPoolManager.getInstance().scheduleGeneral(bc, 10);
				_runstate = 0;
			}
		}
	}

	public void sendVehicleDeparture(L2Player activeChar)
	{
		if(_vd != null)
			activeChar.sendPacket(_vd);
	}

	public VehicleDeparture GetVehicleDeparture()
	{
		return _vd;
	}

	public void beginCycle()
	{
		say(10);
		BoatCaptain bc = new BoatCaptain(1, this);
		ThreadPoolManager.getInstance().scheduleGeneral(bc, 300000);
	}

	private int lastx = -1;
	private int lasty = -1;
	protected boolean needOnVehicleCheckLocation = false;

	public void updatePeopleInTheBoat(int x, int y, int z)
	{
		if(_inboat != null)
		{
			boolean check = false;
			if(lastx == -1 || lasty == -1)
			{
				check = true;
				lastx = x;
				lasty = y;
			}
			else if((x - lastx) * (x - lastx) + (y - lasty) * (y - lasty) > 2250000) // 1500 * 1500 = 2250000
			{
				check = true;
				lastx = x;
				lasty = y;
			}
			for(int i = 0; i < _inboat.size(); i++)
			{
				L2Player player = _inboat.get(i);
				if(player == null)
					continue;
				if(player.isInBoat() && player.getBoat() == this)
					player.setXYZ(x, y, z);
				if(check && needOnVehicleCheckLocation)
				{
					OnVehicleCheckLocation vcl = new OnVehicleCheckLocation(this, x, y, z);
					player.sendPacket(vcl);
				}
			}
		}
	}

	public void begin()
	{
		if(_cycle == 1)
		{
			Collection<L2Player> knownPlayers = L2World.getAroundPlayers(this);
			if(knownPlayers != null && !knownPlayers.isEmpty())
			{
				_inboat = new FastMap<Integer, L2Player>();
				int i = 0;
				for(L2Player player : knownPlayers)
					if(player != null && player.isInBoat())
					{
						L2ItemInstance it;
						it = player.getInventory().getItemByItemId(_t1._IdWTicket1);
						if(it != null && it.getCount() >= 1)
						{
							player.getInventory().destroyItem(it.getObjectId(), 1, true, false);
							_inboat.put(i, player);
							i++;
						}
						else if(it == null && _t1._IdWTicket1 == 0)
						{
							_inboat.put(i, player);
							i++;
						}
						else
						{
							player.sendPacket(new SystemMessage(SystemMessage.YOU_MAY_NOT_GET_ON_BOARD_WITHOUT_A_PASS));
							player.teleToLocation(_t1._ntx1, _t1._nty1, _t1._ntz1);
						}
					}
			}
			Boatrun bc = new Boatrun(0, this);
			ThreadPoolManager.getInstance().scheduleGeneral(bc, 0);
		}
		else if(_cycle == 2)
		{
			Collection<L2Player> knownPlayers = L2World.getAroundPlayers(this);
			if(knownPlayers != null && !knownPlayers.isEmpty())
			{
				_inboat = new FastMap<Integer, L2Player>();
				int i = 0;
				for(L2Player player : knownPlayers)
					if(player != null && player.isInBoat())
					{
						L2ItemInstance it;
						it = player.getInventory().getItemByItemId(_t2._IdWTicket1);
						if(it != null && it.getCount() >= 1)
						{
							player.getInventory().destroyItem(it.getObjectId(), 1, true, false);
							_inboat.put(i, player);
							i++;
						}
						else if(it == null && _t2._IdWTicket1 == 0)
						{
							_inboat.put(i, player);
							i++;
						}
						else
						{
							player.sendPacket(new SystemMessage(SystemMessage.YOU_MAY_NOT_GET_ON_BOARD_WITHOUT_A_PASS));
							player.teleToLocation(_t2._ntx1, _t2._nty1, _t2._ntz1);
						}
					}
			}
			Boatrun bc = new Boatrun(0, this);
			ThreadPoolManager.getInstance().scheduleGeneral(bc, 0);
		}
	}

	public void say(int i)
	{
		Collection<L2Player> knownPlayers = L2World.getAroundPlayers(this);
		CreatureSay sm;
		PlaySound ps;
		switch(i)
		{
			case 10:
				if(_cycle == 1)
					sm = new CreatureSay(0, Say2.SHOUT, _t1._npc1, _t1._sysmess10_1);
				else
					sm = new CreatureSay(0, Say2.SHOUT, _t2._npc1, _t2._sysmess10_1);
				ps = new PlaySound(0, "itemsound.ship_arrival_departure", 1, getObjectId(), getX(), getY(), getZ());
				if(knownPlayers == null || knownPlayers.isEmpty())
					return;
				for(L2Player player : knownPlayers)
				{
					player.sendPacket(sm);
					player.sendPacket(ps);
				}
				break;
			case 5:
				if(_cycle == 1)
					sm = new CreatureSay(0, Say2.SHOUT, _t1._npc1, _t1._sysmess5_1);
				else
					sm = new CreatureSay(0, Say2.SHOUT, _t2._npc1, _t2._sysmess5_1);
				ps = new PlaySound(0, "itemsound.ship_5min", 1, getObjectId(), getX(), getY(), getZ());
				if(knownPlayers == null || knownPlayers.isEmpty())
					return;
				for(L2Player player : knownPlayers)
				{
					player.sendPacket(sm);
					player.sendPacket(ps);
				}
				break;
			case 1:
				if(_cycle == 1)
					sm = new CreatureSay(0, Say2.SHOUT, _t1._npc1, _t1._sysmess1_1);
				else
					sm = new CreatureSay(0, Say2.SHOUT, _t2._npc1, _t2._sysmess1_1);
				ps = new PlaySound(0, "itemsound.ship_1min", 1, getObjectId(), getX(), getY(), getZ());
				if(knownPlayers == null || knownPlayers.isEmpty())
					return;
				for(L2Player player : knownPlayers)
				{
					player.sendPacket(sm);
					player.sendPacket(ps);
				}
				break;
			case 0:
				if(_cycle == 1)
					sm = new CreatureSay(0, Say2.SHOUT, _t1._npc1, _t1._sysmess0_1);
				else
					sm = new CreatureSay(0, Say2.SHOUT, _t2._npc1, _t2._sysmess0_1);
				if(knownPlayers == null || knownPlayers.isEmpty())
					return;
				for(L2Player player : knownPlayers)
					player.sendPacket(sm);
				break;
			case -1:
				if(_cycle == 1)
					sm = new CreatureSay(0, Say2.SHOUT, _t1._npc1, _t1._sysmessb_1);
				else
					sm = new CreatureSay(0, Say2.SHOUT, _t2._npc1, _t2._sysmessb_1);
				ps = new PlaySound(0, "itemsound.ship_arrival_departure", 1, getObjectId(), getX(), getY(), getZ());
				for(L2Player player : knownPlayers)
				{
					player.sendPacket(sm);
					player.sendPacket(ps);
				}
				break;
		}
	}

	public void spawn()
	{
		FastList<L2Player> knownPlayers = L2World.getAroundPlayers(this);
		_cycle = 1;
		beginCycle();
		if(knownPlayers.isEmpty())
			return;
		VehicleInfo vi = new VehicleInfo(this);
		for(L2Player player : knownPlayers)
			player.sendPacket(vi);
	}

	public void SetTrajet1(int idWaypoint1, int idWTicket1, int ntx1, int nty1, int ntz1, String idnpc1, String sysmess10_1, String sysmess5_1, String sysmess1_1, String sysmess0_1, String sysmessb_1)
	{
		_t1 = new L2BoatTrajet(idWaypoint1, idWTicket1, ntx1, nty1, ntz1, idnpc1, sysmess10_1, sysmess5_1, sysmess1_1, sysmess0_1, sysmessb_1, _name);
	}

	public void SetTrajet2(int idWaypoint1, int idWTicket1, int ntx1, int nty1, int ntz1, String idnpc1, String sysmess10_1, String sysmess5_1, String sysmess1_1, String sysmess0_1, String sysmessb_1)
	{
		_t2 = new L2BoatTrajet(idWaypoint1, idWTicket1, ntx1, nty1, ntz1, idnpc1, sysmess10_1, sysmess5_1, sysmess1_1, sysmess0_1, sysmessb_1, _name);
	}

	@Override
	public void updateAbnormalEffect()
	{}

	@Override
	public L2ItemInstance getActiveWeaponInstance()
	{
		return null;
	}

	@Override
	public L2Weapon getActiveWeaponItem()
	{
		return null;
	}

	@Override
	public L2ItemInstance getSecondaryWeaponInstance()
	{
		return null;
	}

	@Override
	public L2Weapon getSecondaryWeaponItem()
	{
		return null;
	}

	@Override
	public byte getLevel()
	{
		return 0;
	}

	@Override
	public boolean isAutoAttackable(L2Character attacker)
	{
		return false;
	}

	@Override
	public boolean isAttackable()
	{
		return false;
	}
}
