package L2jFT.game.model.instances;

import L2jFT.game.ai.CtrlIntention;
import L2jFT.game.model.L2Character;
import L2jFT.game.templates.L2NpcTemplate;

import java.util.logging.Logger;

/**
 * This class manages all Boss. 
 *
 * @version $Revision: 1.0.0.0 $ $Date: 2006/06/16 $
 */
public final class L2BossInstance extends L2RaidBossInstance
{
	protected static Logger _log = Logger.getLogger(L2BossInstance.class.getName());
	private boolean _teleportedToNest;

	private static final int BOSS_MAINTENANCE_INTERVAL = 10000;

	/**
	 * Constructor for L2BossInstance. This represent all grandbosses:
	 * <ul>
	 * <li>29001	Queen Ant</li>
	 * <li>29014	Orfen</li>
	 * <li>29019	Antharas</li>
	 * <li>29020	Baium</li>
	 * <li>29022	Zaken</li>
	 * <li>29028	Valakas</li>
	 * <li>29006	Core</li>
	 * </ul>
	 * <br>
	 * <b>For now it's nothing more than a L2Monster but there'll be a scripting<br>
	 * engine for AI soon and we could add special behaviour for those boss</b><br>
	 * <br>
	 * @param objectId ID of the instance
	 * @param template L2NpcTemplate of the instance
	 */
	public L2BossInstance(int objectId, L2NpcTemplate template)
	{
		super(objectId, template);
	}

	@Override
	protected int getMaintenanceInterval()
	{
		return BOSS_MAINTENANCE_INTERVAL;
	}

	@Override
	public final boolean isMovementDisabled()
	{
		// Core should stay anyway
		return getNpcId() == 29006 || super.isMovementDisabled();
	}

	@Override
	public void doDie(L2Character killer)
	{
		super.doDie(killer);
	}

	/**
	 * Used by Orfen to set 'teleported' flag, when hp goes to <50%
	 * @param flag
	 */
	public void setTeleported(boolean flag)
	{
		_teleportedToNest = flag;
	}

	public boolean getTeleported()
	{
		return _teleportedToNest;
	}

	/**
	 * Reduce the current HP of the L2NpcInstance, update its _aggroList and launch the doDie Task if necessary.<BR><BR> 
	 *
	 */
	@Override
	public void reduceCurrentHp(double damage, L2Character attacker, boolean awake, boolean standUp, boolean directHp)
	{
		if(getTemplate().npcId == 29014 && getCurrentHp() - damage < getMaxHp() / 2 && !getTeleported()) // Orfen
		{
			clearAggroList();
			getAI().setIntention(CtrlIntention.AI_INTENTION_ACTIVE, null, null);
			teleToLocation(43577, 15985, -4396);
			setTeleported(true);
		}

		super.reduceCurrentHp(damage, attacker, awake, standUp, directHp);
	}

	@Override
	public boolean hasRandomAnimation()
	{
		return false;
	}
}
