package L2jFT.game.model.instances;

import L2jFT.game.instancemanager.ClanHallManager;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.entity.ClanHall;
import L2jFT.game.network.serverpackets.ActionFailed;
import L2jFT.game.network.serverpackets.NpcHtmlMessage;
import L2jFT.game.templates.L2NpcTemplate;

import java.util.StringTokenizer;

public class L2ClanHallDoormenInstance extends L2NpcInstance
{
	protected static int Cond_All_False = 0;
	protected static int Cond_Busy_Because_Of_Siege = 1;
	protected static int Cond_Owner = 2;
	private int _clanHallId = -1;

	/**
	 * @param objectId
	 * @param template
	 */
	public L2ClanHallDoormenInstance(int objectId, L2NpcTemplate template)
	{
		super(objectId, template);
	}

	@Override
	public void onBypassFeedback(L2Player player, String command)
	{
		player.sendPacket(new ActionFailed());
		int condition = validateCondition(player);
		if(condition <= Cond_All_False)
			return;
		else if(condition == Cond_Owner)
		{
			StringTokenizer st = new StringTokenizer(command, " ");
			String actualCommand = st.nextToken(); // Get actual command
			String val = "";
			if(st.countTokens() >= 1)
				val = st.nextToken();

			if(actualCommand.equalsIgnoreCase("door"))
				if((player.getClanPrivileges() & L2Clan.CP_CH_OPEN_DOOR) == L2Clan.CP_CH_OPEN_DOOR)
				{
					if(val.equalsIgnoreCase("open"))
					{
						getClanHall().openCloseDoors(true);
						NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
						html.setFile("data/html/doormen/clanhall/AfterDoorOpen.htm");
						sendHtmlMessage(player, html);
					}
					else if(val.equalsIgnoreCase("close"))
					{
						getClanHall().openCloseDoors(false);
						NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
						html.setFile("data/html/doormen/clanhall/AfterDoorClose.htm");
						sendHtmlMessage(player, html);
					}
					else
					{
						NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
						html.setFile("data/html/clanHallManager/door.htm");
						sendHtmlMessage(player, html);
					}
				}
				else
					player.sendMessage("You are not authorized to do this!", "Вам не разрешено делать этого");
		}
		super.onBypassFeedback(player, command);
	}

	private void sendHtmlMessage(L2Player player, NpcHtmlMessage html)
	{
		html.replace("%npcname%", getName());
		html.replace("%clanname%", player.getClan().getName());
		player.sendPacket(html);
	}

	@Override
	public void showChatWindow(L2Player player, int val)
	{
		String filename = "data/html/doormen/clanhall/" + getTemplate().npcId + "-no.htm";
		int condition = validateCondition(player);
		if(condition > Cond_All_False)
			if(condition == Cond_Owner) // Clan owns CH
				filename = "data/html/doormen/clanhall/" + getTemplate().npcId + ".htm"; // Owner message window
		NpcHtmlMessage html = new NpcHtmlMessage(player, this, filename, val);
		if(condition > Cond_All_False)
			if(condition == Cond_Owner) // Clan owns CH
				html.replace("%clanname%", player.getClan() != null ? player.getClan().getName() : "GM Access");
		player.sendPacket(html);
	}

	protected int validateCondition(L2Player player)
	{
		if(player.isGM())
			return Cond_Owner;
		if(player.getClan() != null)
			if(getClanHall().getOwnerId() == player.getClanId())
				return Cond_Owner;

		return Cond_All_False;
	}

	/** Return the L2ClanHall this L2NpcInstance belongs to. */
	public final ClanHall getClanHall()
	{
		if(_clanHallId < 0)
		{
			_clanHallId = ClanHallManager.getInstance().findNearestClanHall(getX(), getY(), 1500).getId();
			if(_clanHallId < 0)
				return null;
		}
		return ClanHallManager.getInstance().getClanHall(_clanHallId);
	}
}
