package L2jFT.game.model.instances;

import javolution.util.FastList;
import L2jFT.Config;
import L2jFT.game.TradeController;
import L2jFT.game.instancemanager.ClanHallManager;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.L2Skill.SkillType;
import L2jFT.game.model.L2TeleportLocation;
import L2jFT.game.model.L2TradeList;
import L2jFT.game.model.L2World;
import L2jFT.game.model.Warehouse.WarehouseType;
import L2jFT.game.model.entity.ClanHall;
import L2jFT.game.model.instances.L2ItemInstance.ItemClass;
import L2jFT.game.network.serverpackets.ActionFailed;
import L2jFT.game.network.serverpackets.AgitDecoInfo;
import L2jFT.game.network.serverpackets.BuyList;
import L2jFT.game.network.serverpackets.NpcHtmlMessage;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.network.serverpackets.WareHouseDepositList;
import L2jFT.game.network.serverpackets.WareHouseWithdrawalList;
import L2jFT.game.tables.ClanHallTeleportsTable;
import L2jFT.game.tables.SkillTable;
import L2jFT.game.templates.L2NpcTemplate;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.StringTokenizer;
import java.util.logging.Logger;

public class L2ClanHallManagerInstance extends L2NpcInstance
{
	private static Logger _log = Logger.getLogger(L2ClanHallManagerInstance.class.getName());

	protected static int Cond_All_False = 0;
	protected static int Cond_Busy_Because_Of_Siege = 1;
	protected static int Cond_Owner = 2;
	private int _clanHallId = -1;

	/**
	 * @param objectId
	 * @param template
	 */
	public L2ClanHallManagerInstance(int objectId, L2NpcTemplate template)
	{
		super(objectId, template);
	}

	@Override
	public void onBypassFeedback(L2Player player, String command)
	{
		SimpleDateFormat format = new SimpleDateFormat("dd/MM/yyyy HH:mm");
		player.sendPacket(new ActionFailed());
		int condition = validateCondition(player);
		if(condition <= Cond_All_False)
			return;
		else if(condition == Cond_Owner)
		{
			StringTokenizer st = new StringTokenizer(command, " ");
			String actualCommand = st.nextToken(); // Get actual command
			String val = "";
			if(st.countTokens() >= 1)
				val = st.nextToken();

			if(actualCommand.equalsIgnoreCase("banish"))
			{
				NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
				html.setFile("data/html/clanHallManager/Banish.htm");
				sendHtmlMessage(player, html);
			}
			else if(actualCommand.equalsIgnoreCase("banish_foreigner"))
			{
				getClanHall().banishForeigner(player);
				return;
			}
			else if(actualCommand.equalsIgnoreCase("Buy"))
			{
				if(val.equals(""))
					return;
				showBuyWindow(player, Integer.valueOf(val));
			}
			else if(actualCommand.equalsIgnoreCase("manage_vault"))
			{
				if(val.equalsIgnoreCase("deposit"))
					showDepositWindowClan(player);
				else if(val.equalsIgnoreCase("withdraw"))
				{
					int value = Integer.valueOf(st.nextToken());
					if(value == 9)
					{
						NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
						html.setFile("data/html/clanHallManager/clan.htm");
						html.replace("%npcname%", getName());
						player.sendPacket(html);
					}
					else
						showWithdrawWindowClan(player, value);
				}
				else
				{
					NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
					html.setFile("data/html/clanHallManager/vault.htm");
					sendHtmlMessage(player, html);
				}
				return;
			}
			else if(actualCommand.equalsIgnoreCase("door"))
			{
				if((player.getClanPrivileges() & L2Clan.CP_CH_OPEN_DOOR) == L2Clan.CP_CH_OPEN_DOOR)
				{
					if(val.equalsIgnoreCase("open"))
						getClanHall().openCloseDoors(true);
					else if(val.equalsIgnoreCase("close"))
						getClanHall().openCloseDoors(false);
					else
					{
						NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
						html.setFile("data/html/clanHallManager/door.htm");
						sendHtmlMessage(player, html);
					}
				}
				else
					player.sendMessage("You are not authorized to do this!", "Вам не разрешено делать это");
			}
			else if(actualCommand.equalsIgnoreCase("functions"))
			{
				if(val.equalsIgnoreCase("tele"))
				{
					if(getClanHall().getFunction(ClanHall.FUNC_TELEPORT) == null)
						return;
					NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
					html.setFile("data/html/clanHallManager/tele" + getClanHall().getLocation() + getClanHall().getFunction(ClanHall.FUNC_TELEPORT).getLvl() + ".htm");
					sendHtmlMessage(player, html);
				}
				else if(val.equalsIgnoreCase("item_creation"))
				{
					if(getClanHall().getFunction(ClanHall.FUNC_ITEM_CREATE) == null)
						return;
					NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
					html.setFile("data/html/clanHallManager/itemcreation/item" + getTemplate().npcId + getClanHall().getLocation() + getClanHall().getFunction(ClanHall.FUNC_ITEM_CREATE).getLvl() + ".htm");
					sendHtmlMessage(player, html);
				}
				else if(val.equalsIgnoreCase("support"))
				{
					if(getClanHall().getFunction(ClanHall.FUNC_SUPPORT) == null)
						return;
					NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
					html.setFile("data/html/clanHallManager/support" + getClanHall().getLocation() + getClanHall().getFunction(ClanHall.FUNC_SUPPORT).getLvl() + ".htm");
					html.replace("%mp%", String.valueOf(Math.round(getCurrentMp())));
					sendHtmlMessage(player, html);
				}
				else if(val.equalsIgnoreCase("back"))
					showChatWindow(player, 0);
				else
				{
					NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
					html.setFile("data/html/clanHallManager/functions.htm");
					if(getClanHall().getFunction(ClanHall.FUNC_RESTORE_EXP) != null)
						html.replace("%xp_regen%", String.valueOf(getClanHall().getFunction(ClanHall.FUNC_RESTORE_EXP).getLvl()) + "%");
					else
						html.replace("%hp_regen%", "0");
					if(getClanHall().getFunction(ClanHall.FUNC_RESTORE_HP) != null)
						html.replace("%hp_regen%", String.valueOf(getClanHall().getFunction(ClanHall.FUNC_RESTORE_HP).getLvl()) + "%");
					else
						html.replace("%xp_regen%", "0");
					if(getClanHall().getFunction(ClanHall.FUNC_RESTORE_MP) != null)
						html.replace("%mp_regen%", String.valueOf(getClanHall().getFunction(ClanHall.FUNC_RESTORE_MP).getLvl()) + "%");
					else
						html.replace("%mp_regen", "0");
					sendHtmlMessage(player, html);
				}
			}
			else if(actualCommand.equalsIgnoreCase("manage"))
			{
				if((player.getClanPrivileges() & L2Clan.CP_CH_SET_FUNCTIONS) == L2Clan.CP_CH_SET_FUNCTIONS)
				{
					if(val.equalsIgnoreCase("recovery"))
					{
						if(st.countTokens() >= 1)
						{
							val = st.nextToken();
							if(val.equalsIgnoreCase("hp"))
							{
								if(st.countTokens() >= 1)
								{
									int fee;
									if(Config.DEBUG)
										_log.warning("Mp editing invoked");
									val = st.nextToken();
									int percent = Integer.valueOf(val);
									switch(percent)
									{
										case 20:
											fee = Config.CH_HPREG1_FEE;
											break;
										case 40:
											fee = Config.CH_HPREG2_FEE;
											break;
										case 80:
											fee = Config.CH_HPREG3_FEE;
											break;
										case 100:
											fee = Config.CH_HPREG4_FEE;
											break;
										case 120:
											fee = Config.CH_HPREG5_FEE;
											break;
										case 140:
											fee = Config.CH_HPREG6_FEE;
											break;
										case 160:
											fee = Config.CH_HPREG7_FEE;
											break;
										case 180:
											fee = Config.CH_HPREG8_FEE;
											break;
										case 200:
											fee = Config.CH_HPREG9_FEE;
											break;
										case 240:
											fee = Config.CH_HPREG10_FEE;
											break;
										case 260:
											fee = Config.CH_HPREG11_FEE;
											break;
										default:
											fee = Config.CH_HPREG12_FEE;
											break;
									}
									if(!getClanHall().updateFunctions(ClanHall.FUNC_RESTORE_HP, percent, fee, Config.CH_HPREG_FEE_RATIO, Calendar.getInstance().getTimeInMillis() + Config.CH_HPREG_FEE_RATIO, (getClanHall().getFunction(ClanHall.FUNC_RESTORE_HP) == null), this))
										player.sendPacket(new SystemMessage(SystemMessage.THERE_IS_NOT_ENOUGH_ADENA_IN_THE_CLAN_HALL_WAREHOUSE));
								}
							}
							else if(val.equalsIgnoreCase("mp"))
							{
								if(st.countTokens() >= 1)
								{
									int fee;
									if(Config.DEBUG)
										_log.warning("Mp editing invoked");
									val = st.nextToken();
									int percent = Integer.valueOf(val);
									switch(percent)
									{
										case 5:
											fee = Config.CH_MPREG1_FEE;
											break;
										case 10:
											fee = Config.CH_MPREG2_FEE;
											break;
										case 15:
											fee = Config.CH_MPREG3_FEE;
											break;
										case 25:
											fee = Config.CH_MPREG4_FEE;
											break;
										case 30:
											fee = Config.CH_MPREG5_FEE;
											break;
										default:
											fee = Config.CH_MPREG6_FEE;
											break;
									}
									if(!getClanHall().updateFunctions(ClanHall.FUNC_RESTORE_MP, percent, fee, Config.CH_MPREG_FEE_RATIO, Calendar.getInstance().getTimeInMillis() + Config.CH_MPREG_FEE_RATIO, (getClanHall().getFunction(ClanHall.FUNC_RESTORE_MP) == null), this))
										player.sendPacket(new SystemMessage(SystemMessage.THERE_IS_NOT_ENOUGH_ADENA_IN_THE_CLAN_HALL_WAREHOUSE));
								}
							}
							else if(val.equalsIgnoreCase("exp"))
								if(st.countTokens() >= 1)
								{
									int fee;
									if(Config.DEBUG)
										_log.warning("Exp editing invoked");
									val = st.nextToken();
									int percent = Integer.valueOf(val);
									switch(percent)
									{
										case 5:
											fee = Config.CH_EXPREG1_FEE;
											break;
										case 10:
											fee = Config.CH_EXPREG2_FEE;
											break;
										case 15:
											fee = Config.CH_EXPREG3_FEE;
											break;
										case 25:
											fee = Config.CH_EXPREG4_FEE;
											break;
										case 30:
											fee = Config.CH_EXPREG5_FEE;
											break;
										case 35:
											fee = Config.CH_EXPREG6_FEE;
											break;
										case 40:
											fee = Config.CH_EXPREG7_FEE;
											break;
										default:
											fee = Config.CH_EXPREG8_FEE;
											break;
									}
									if(!getClanHall().updateFunctions(ClanHall.FUNC_RESTORE_EXP, percent, fee, Config.CH_EXPREG_FEE_RATIO, Calendar.getInstance().getTimeInMillis() + Config.CH_EXPREG_FEE_RATIO, (getClanHall().getFunction(ClanHall.FUNC_RESTORE_EXP) == null), this))
										player.sendPacket(new SystemMessage(SystemMessage.THERE_IS_NOT_ENOUGH_ADENA_IN_THE_CLAN_HALL_WAREHOUSE));
								}
						}
						else
						{
							NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
							html.setFile("data/html/clanHallManager/edit_recovery" + getClanHall().getLocation() + ".htm");
							if(getClanHall().getFunction(ClanHall.FUNC_RESTORE_HP) != null)
							{
								html.replace("%hp%", String.valueOf(getClanHall().getFunction(ClanHall.FUNC_RESTORE_HP).getLvl()) + "%");
								html.replace("%hpPrice%", String.valueOf(getClanHall().getFunction(ClanHall.FUNC_RESTORE_HP).getLease()));
								html.replace("%hpDate%", format.format(getClanHall().getFunction(ClanHall.FUNC_RESTORE_HP).getEndTime()));
							}
							else
							{
								html.replace("%hp%", "0");
								html.replace("%hpPrice%", "0");
								html.replace("%hpDate%", "0");
							}
							if(getClanHall().getFunction(ClanHall.FUNC_RESTORE_EXP) != null)
							{
								html.replace("%exp%", String.valueOf(getClanHall().getFunction(ClanHall.FUNC_RESTORE_EXP).getLvl()) + "%");
								html.replace("%expPrice%", String.valueOf(getClanHall().getFunction(ClanHall.FUNC_RESTORE_EXP).getLease()));
								html.replace("%expDate%", format.format(getClanHall().getFunction(ClanHall.FUNC_RESTORE_EXP).getEndTime()));
							}
							else
							{
								html.replace("%exp%", "0");
								html.replace("%expPrice%", "0");
								html.replace("%expDate%", "0");
							}
							if(getClanHall().getFunction(ClanHall.FUNC_RESTORE_MP) != null)
							{
								html.replace("%mp%", String.valueOf(getClanHall().getFunction(ClanHall.FUNC_RESTORE_MP).getLvl()) + "%");
								html.replace("%mpPrice%", String.valueOf(getClanHall().getFunction(ClanHall.FUNC_RESTORE_MP).getLease()));
								html.replace("%mpDate%", format.format(getClanHall().getFunction(ClanHall.FUNC_RESTORE_MP).getEndTime()));
							}
							else
							{
								html.replace("%mp%", "0");
								html.replace("%mpPrice%", "0");
								html.replace("%mpDate%", "0");
							}
							sendHtmlMessage(player, html);
						}
					}
					else if(val.equalsIgnoreCase("other"))
					{
						if(st.countTokens() >= 1)
						{
							val = st.nextToken();
							if(val.equalsIgnoreCase("item"))
							{
								if(st.countTokens() >= 1)
								{
									if(Config.DEBUG)
										_log.warning("Item editing invoked");
									val = st.nextToken();
									int lvl = Integer.valueOf(val);
									if(!getClanHall().updateFunctions(ClanHall.FUNC_ITEM_CREATE, lvl, 76000, 86400000, Calendar.getInstance().getTimeInMillis() + 86400000, (getClanHall().getFunction(ClanHall.FUNC_ITEM_CREATE) == null), this))
										player.sendPacket(new SystemMessage(SystemMessage.THERE_IS_NOT_ENOUGH_ADENA_IN_THE_CLAN_HALL_WAREHOUSE));
								}
							}
							else if(val.equalsIgnoreCase("tele"))
							{
								if(st.countTokens() >= 1)
								{
									int fee;
									if(Config.DEBUG)
										_log.warning("Tele editing invoked");
									val = st.nextToken();
									int lvl = Integer.valueOf(val);
									switch(lvl)
									{
										case 1:
											fee = Config.CH_TELE1_FEE;
											break;
										default:
											fee = Config.CH_TELE2_FEE;
											break;
									}
									if(!getClanHall().updateFunctions(ClanHall.FUNC_TELEPORT, lvl, fee, Config.CH_TELE_FEE_RATIO, Calendar.getInstance().getTimeInMillis() + Config.CH_TELE_FEE_RATIO, (getClanHall().getFunction(ClanHall.FUNC_TELEPORT) == null), this))
										player.sendPacket(new SystemMessage(SystemMessage.THERE_IS_NOT_ENOUGH_ADENA_IN_THE_CLAN_HALL_WAREHOUSE));
								}
							}
							else if(val.equalsIgnoreCase("support"))
								if(st.countTokens() >= 1)
								{
									int fee;
									if(Config.DEBUG)
										_log.warning("Support editing invoked");
									val = st.nextToken();
									int lvl = Integer.valueOf(val);
									switch(lvl)
									{
										case 1:
											fee = Config.CH_SUPPORT1_FEE;
											break;
										case 2:
											fee = Config.CH_SUPPORT2_FEE;
											break;
										case 3:
											fee = Config.CH_SUPPORT3_FEE;
											break;
										case 4:
											fee = Config.CH_SUPPORT4_FEE;
											break;
										case 5:
											fee = Config.CH_SUPPORT5_FEE;
											break;
										case 7:
											fee = Config.CH_SUPPORT7_FEE;
											break;
										default:
											fee = Config.CH_SUPPORT8_FEE;
											break;
									}
									if(!getClanHall().updateFunctions(ClanHall.FUNC_SUPPORT, lvl, fee, Config.CH_SUPPORT_FEE_RATIO, Calendar.getInstance().getTimeInMillis() + Config.CH_SUPPORT_FEE_RATIO, (getClanHall().getFunction(ClanHall.FUNC_SUPPORT) == null), this))
										player.sendPacket(new SystemMessage(SystemMessage.THERE_IS_NOT_ENOUGH_ADENA_IN_THE_CLAN_HALL_WAREHOUSE));
								}
						}
						else
						{
							NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
							html.setFile("data/html/clanHallManager/edit_other" + getClanHall().getLocation() + ".htm");
							if(getClanHall().getFunction(ClanHall.FUNC_TELEPORT) != null)
							{
								html.replace("%tele%", String.valueOf(getClanHall().getFunction(ClanHall.FUNC_TELEPORT).getLvl()));
								html.replace("%telePrice%", String.valueOf(getClanHall().getFunction(ClanHall.FUNC_TELEPORT).getLease()));
								html.replace("%teleDate%", format.format(getClanHall().getFunction(ClanHall.FUNC_TELEPORT).getEndTime()));
							}
							else
							{
								html.replace("%tele%", "0");
								html.replace("%telePrice%", "0");
								html.replace("%teleDate%", "0");
							}
							if(getClanHall().getFunction(ClanHall.FUNC_SUPPORT) != null)
							{
								html.replace("%support%", String.valueOf(getClanHall().getFunction(ClanHall.FUNC_SUPPORT).getLvl()));
								html.replace("%supportPrice%", String.valueOf(getClanHall().getFunction(ClanHall.FUNC_SUPPORT).getLease()));
								html.replace("%supportDate%", format.format(getClanHall().getFunction(ClanHall.FUNC_SUPPORT).getEndTime()));
							}
							else
							{
								html.replace("%support%", "0");
								html.replace("%supportPrice%", "0");
								html.replace("%supportDate%", "0");
							}
							if(getClanHall().getFunction(ClanHall.FUNC_ITEM_CREATE) != null)
							{
								html.replace("%item%", String.valueOf(getClanHall().getFunction(ClanHall.FUNC_ITEM_CREATE).getLvl()));
								html.replace("%itemPrice%", String.valueOf(getClanHall().getFunction(ClanHall.FUNC_ITEM_CREATE).getLease()));
								html.replace("%itemDate%", format.format(getClanHall().getFunction(ClanHall.FUNC_ITEM_CREATE).getEndTime()));
							}
							else
							{
								html.replace("%item%", "0");
								html.replace("%itemPrice%", "0");
								html.replace("%itemDate%", "0");
							}
							sendHtmlMessage(player, html);
						}
					}
					else if(val.equalsIgnoreCase("deco"))
					{
						if(st.countTokens() >= 1)
						{
							val = st.nextToken();
							if(val.equalsIgnoreCase("platform"))
							{
								int fee;
								if(Config.DEBUG)
									_log.warning("Front Platform editing invoked");
								val = st.nextToken();
								int lvl = Integer.valueOf(val);
								switch(lvl)
								{
									case 1:
										fee = Config.CH_PLATFORM1_FEE;
										break;
									default:
										fee = Config.CH_PLATFORM2_FEE;
										break;
								}
								if(!getClanHall().updateFunctions(ClanHall.FUNC_PLATFORM, lvl, fee, Config.CH_PLATFORM_FEE_RATIO, Calendar.getInstance().getTimeInMillis() + Config.CH_PLATFORM_FEE_RATIO, (getClanHall().getFunction(ClanHall.FUNC_PLATFORM) == null), this))
									player.sendPacket(new SystemMessage(SystemMessage.THERE_IS_NOT_ENOUGH_ADENA_IN_THE_CLAN_HALL_WAREHOUSE));
							}
							else if(val.equalsIgnoreCase("curtain"))
								if(st.countTokens() >= 1)
								{
									int fee;
									if(Config.DEBUG)
										_log.warning("Curtains editing invoked");
									val = st.nextToken();
									int lvl = Integer.valueOf(val);
									switch(lvl)
									{
										case 1:
											fee = Config.CH_CURTAIN1_FEE;
											break;
										default:
											fee = Config.CH_CURTAIN2_FEE;
											break;
									}
									if(!getClanHall().updateFunctions(ClanHall.FUNC_CURTAIN, lvl, fee, Config.CH_CURTAIN_FEE_RATIO, Calendar.getInstance().getTimeInMillis() + Config.CH_CURTAIN_FEE_RATIO, (getClanHall().getFunction(ClanHall.FUNC_CURTAIN) == null), this))
										player.sendPacket(new SystemMessage(SystemMessage.THERE_IS_NOT_ENOUGH_ADENA_IN_THE_CLAN_HALL_WAREHOUSE));
								}
						}
						else
						{
							NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
							html.setFile("data/html/clanHallManager/edit_deco" + getClanHall().getLocation() + ".htm");
							if(getClanHall().getFunction(ClanHall.FUNC_CURTAIN) != null)
							{
								html.replace("%curtain%", String.valueOf(getClanHall().getFunction(ClanHall.FUNC_CURTAIN).getLvl()));
								html.replace("%curtainPrice%", String.valueOf(getClanHall().getFunction(ClanHall.FUNC_CURTAIN).getLease()));
								html.replace("%curtainDate%", format.format(getClanHall().getFunction(ClanHall.FUNC_CURTAIN).getEndTime()));
							}
							else
							{
								html.replace("%curtain%", "0");
								html.replace("%curtainPrice%", "0");
								html.replace("%curtainDate%", "0");
							}
							if(getClanHall().getFunction(ClanHall.FUNC_PLATFORM) != null)
							{
								html.replace("%platform%", String.valueOf(getClanHall().getFunction(ClanHall.FUNC_PLATFORM).getLvl()));
								html.replace("%platformPrice%", String.valueOf(getClanHall().getFunction(ClanHall.FUNC_PLATFORM).getLease()));
								html.replace("%platformDate%", format.format(getClanHall().getFunction(ClanHall.FUNC_PLATFORM).getEndTime()));
							}
							else
							{
								html.replace("%platform%", "0");
								html.replace("%platformPrice%", "0");
								html.replace("%platformDate%", "0");
							}
							sendHtmlMessage(player, html);
						}
					}
					else if(val.equalsIgnoreCase("back"))
						showChatWindow(player, 0);
					else
					{
						NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
						html.setFile("data/html/clanHallManager/manage.htm");
						sendHtmlMessage(player, html);
					}
				}
				else
					player.sendMessage("You are not authorized to do this!", "У Вас не достаточно прав для этого");
				return;
			}
			else if(actualCommand.equalsIgnoreCase("support"))
			{
				setTarget(player);
				L2Skill skill;
				if(val.equals(""))
					return;

				try
				{
					int skill_id = Integer.parseInt(val);
					try
					{
						int skill_lvl = 0;
						if(st.countTokens() >= 1)
							skill_lvl = Integer.parseInt(st.nextToken());
						skill = SkillTable.getInstance().getInfo(skill_id, skill_lvl);
						if(skill.getMpConsume() > getCurrentMp())
						{
							NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
							html.setFile("data/html/clanHallManager/NeedCoolTime.htm");
							html.replace("%mp%", String.valueOf(Math.round(getCurrentMp())));
							sendHtmlMessage(player, html);
							return;
						}
						if(skill.getSkillType() == SkillType.SUMMON)
							player.altUseSkill(skill, player);
						else
							altUseSkill(skill, player);
						NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
						html.setFile("data/html/clanHallManager/AfterSupport.htm");
						html.replace("%mp%", String.valueOf(Math.round(getCurrentMp())));
						sendHtmlMessage(player, html);
						return;
					}
					catch(Exception e)
					{
						player.sendMessage("Invalid skill level!", "");
					}
				}
				catch(Exception e)
				{
					player.sendMessage("Invalid skill!", "");
				}
				return;
			}
			else if(actualCommand.equalsIgnoreCase("goto"))
			{
				int whereTo = Integer.parseInt(val);
				doTeleport(player, whereTo);
				return;
			}
		}
		super.onBypassFeedback(player, command);
	}

	private void sendHtmlMessage(L2Player player, NpcHtmlMessage html)
	{
		html.replace("%npcname%", getName());
		player.sendPacket(html);
	}

	@Override
	public void showChatWindow(L2Player player, int val)
	{
		String filename = "data/html/clanHallManager/chamberlain-no.htm";
		int condition = validateCondition(player);
		if(condition > Cond_All_False)
			if(condition == Cond_Owner) // Clan owns CH
				filename = "data/html/clanHallManager/chamberlain.htm"; // Owner message window
		player.sendPacket(new NpcHtmlMessage(player, this, filename, val));
	}

	protected int validateCondition(L2Player player)
	{
		if(player.isGM())
			return Cond_Owner;
		if(player.getClan() != null)
			if(getClanHall().getOwnerId() == player.getClanId())
				return Cond_Owner;

		return Cond_All_False;
	}

	/** Return the L2ClanHall this L2NpcInstance belongs to. */
	public final ClanHall getClanHall()
	{
		if(_clanHallId < 0)
		{
			ClanHall clanHall = ClanHallManager.getInstance().findNearestClanHall(getX(), getY(), 1500);
			if(clanHall == null)
				return null;
			_clanHallId = clanHall.getId();
			if(_clanHallId < 0)
				return null;
		}
		return ClanHallManager.getInstance().getClanHall(_clanHallId);
	}

	private void showDepositWindowClan(L2Player player)
	{
		if(!player.getPlayerAccess().UseWarehouse)
			return;

		if(player.getClan() == null)
		{
			player.sendActionFailed();
			return;
		}

		if(player.getClan().getLevel() == 0)
		{
			player.sendPacket(new SystemMessage(SystemMessage.ONLY_CLANS_OF_CLAN_LEVEL_1_OR_HIGHER_CAN_USE_A_CLAN_WAREHOUSE));
			player.sendActionFailed();
			return;
		}

		player.setUsingWarehouseType(WarehouseType.CLAN);
		player.tempInvetoryDisable();

		if(Config.DEBUG)
			_log.fine("Showing items to deposit - clan");

		player.sendPacket(new WareHouseDepositList(player, WarehouseType.CLAN));
	}

	private void showWithdrawWindowClan(L2Player player, int val)
	{
		if(!player.getPlayerAccess().UseWarehouse)
			return;

		if(player.getClan() == null)
		{
			player.sendActionFailed();
			return;
		}

		L2Clan _clan = player.getClan();

		if(_clan.getLevel() == 0)
		{
			player.sendPacket(new SystemMessage(SystemMessage.ONLY_CLANS_OF_CLAN_LEVEL_1_OR_HIGHER_CAN_USE_A_CLAN_WAREHOUSE));
			player.sendActionFailed();
			return;
		}

		if((player.getClanPrivileges() & L2Clan.CP_CL_VIEW_WAREHOUSE) != L2Clan.CP_CL_VIEW_WAREHOUSE)
		{
			if(Config.ALT_ALLOW_OTHERS_WITHDRAW_FROM_CLAN_WAREHOUSE)
				player.sendPacket(new SystemMessage(SystemMessage.ITEMS_LEFT_AT_THE_CLAN_HALL_WAREHOUSE_CAN_ONLY_BE_RETRIEVED_BY_THE_CLAN_LEADER_DO_YOU_WANT_TO_CONTINUE));
			else
			{
				player.sendActionFailed();
				return;
			}
		}
		else if(!Config.ALT_ALLOW_OTHERS_WITHDRAW_FROM_CLAN_WAREHOUSE)
			player.sendPacket(new SystemMessage(SystemMessage.ITEMS_LEFT_AT_THE_CLAN_HALL_WAREHOUSE_CAN_ONLY_BE_RETRIEVED_BY_THE_CLAN_LEADER_DO_YOU_WANT_TO_CONTINUE));

		if(Config.DEBUG)
			_log.fine("Showing items to deposit - clan");

		player.setUsingWarehouseType(WarehouseType.CLAN);
		player.tempInvetoryDisable();
		player.sendPacket(new WareHouseWithdrawalList(player, WarehouseType.CLAN, ItemClass.values()[val]));
	}

	private void doTeleport(L2Player player, int val)
	{
		if(Config.DEBUG)
			player.sendMessage("doTeleport(L2Player player, int val) is called", "");
		L2TeleportLocation list = ClanHallTeleportsTable.getInstance().getTemplate(val);
		if(list == null)
			_log.warning("No teleport destination with id:" + val);
		if(list != null && player.getAdena() >= list.getPrice())
		{
			player.reduceAdena(list.getPrice());
			if(Config.DEBUG)
				_log.warning("Teleporting player " + player.getName() + " from CH to new location: " + list.getLocX() + "," + list.getLocY() + "," + list.getLocZ());
			player.teleToLocation(list.getLocX(), list.getLocY(), list.getLocZ());
		}
		player.sendPacket(new ActionFailed());
	}

	private void showBuyWindow(L2Player player, int val)
	{
		if(!player.getPlayerAccess().UseShop)
			return;

		double taxRate = 0;

		if(getCastle() != null)
			taxRate = getCastle().getTaxRate();

		player.tempInvetoryDisable();
		if(Config.DEBUG)
			_log.fine("Showing clanhall buylist");
		L2TradeList list = TradeController.getInstance().getBuyList(val);
		if(list != null && list.getNpcId().equals(String.valueOf(getNpcId())))
		{
			BuyList bl = new BuyList(list, player.getAdena(), taxRate);
			player.sendPacket(bl);
		}
		else
		{
			_log.warning("possible client hacker: " + player.getName() + " attempting to buy from GM shop! < Ban him!");
			_log.warning("buylist id:" + val);
		}
	}

	public void sendDecoInfo(L2Player player)
	{
		ClanHall clanHall = getClanHall();
		if(clanHall != null)
			player.sendPacket(new AgitDecoInfo(getClanHall()));
	}

	public void broadCastClanHallInfo()
	{
		FastList<L2Player> players = L2World.getAroundPlayers(this);
		for(FastList.Node<L2Player> n = players.head(), end = players.tail(); (n = n.getNext()) != end;)
			sendDecoInfo(n.getValue());
	}
}