package L2jFT.game.model.instances;

import L2jFT.Config;
import L2jFT.extensions.scripts.Events;
import L2jFT.game.ai.CtrlIntention;
import L2jFT.game.cache.Msg;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.base.ClassId;
import L2jFT.game.model.quest.Quest;
import L2jFT.game.network.serverpackets.MyTargetSelected;
import L2jFT.game.network.serverpackets.NpcHtmlMessage;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.network.serverpackets.ValidateLocation;
import L2jFT.game.templates.L2NpcTemplate;

import java.util.StringTokenizer;
import java.util.logging.Logger;

public final class L2ClassMasterInstance extends L2NpcInstance
{
	private static Logger _log = Logger.getLogger(L2ClassMasterInstance.class.getName());

	/**
	 * @param template
	 */
	public L2ClassMasterInstance(int objectId, L2NpcTemplate template)
	{
		super(objectId, template);
	}

	@Override
	public void onAction(L2Player player)
	{
		if(Events.onAction(player, this))
			return;

		if(getObjectId() != player.getTargetId())
		{
			player.getAI().setIntention(CtrlIntention.AI_INTENTION_IDLE, null, null);
			if(Config.DEBUG)
				_log.fine("ClassMaster selected:" + getObjectId());
			player.setTarget(this);
			player.sendPacket(new MyTargetSelected(getObjectId(), player.getLevel() - getLevel()));
			// correct location
			player.sendPacket(new ValidateLocation(this));
		}
		else
		{
			if(!isInRange(player, INTERACTION_DISTANCE))
			{
				player.getAI().setIntention(CtrlIntention.AI_INTENTION_INTERACT, this, null);
				player.sendActionFailed();
				return;
			}

			if(Config.DEBUG)
				_log.fine("ClassMaster activated");

			ClassId classId = player.getClassId();

			int jobLevel;
			int level = player.getLevel();

			switch(classId)
			{
				case fighter:
				case mage:
				case elvenFighter:
				case elvenMage:
				case darkFighter:
				case darkMage:
				case orcFighter:
				case orcMage:
				case dwarvenFighter:
					jobLevel = 1;
					break;
				case elvenScout:
				case elvenKnight:
				case elvenWizard:
				case oracle:
				case knight:
				case warrior:
				case rogue:
				case wizard:
				case cleric:
				case palusKnight:
				case assassin:
				case darkWizard:
				case shillienOracle:
				case orcRaider:
				case orcMonk:
				case orcShaman:
				case scavenger:
				case artisan:
					jobLevel = 2;
					break;
				case gladiator:
				case warlord:
				case paladin:
				case darkAvenger:
				case treasureHunter:
				case hawkeye:
				case sorceror:
				case necromancer:
				case warlock:
				case bishop:
				case prophet:
				case templeKnight:
				case swordSinger:
				case plainsWalker:
				case silverRanger:
				case spellsinger:
				case elementalSummoner:
				case elder:
				case shillienKnight:
				case bladedancer:
				case abyssWalker:
				case phantomRanger:
				case spellhowler:
				case phantomSummoner:
				case shillienElder:
				case destroyer:
				case tyrant:
				case overlord:
				case warcryer:
				case bountyHunter:
				case warsmith:
					jobLevel = 3;
					break;
				default:
					jobLevel = 4;
			}

			if(Config.ALLOW_CLASS_MASTERS_LIST.isEmpty() || !Config.ALLOW_CLASS_MASTERS_LIST.contains(jobLevel))
				jobLevel = 4;
			if((level >= 20 && jobLevel == 1 || level >= 40 && jobLevel == 2 || level >= 76 && jobLevel == 3) && Config.ALLOW_CLASS_MASTERS_LIST.contains(jobLevel))
			{
				StringBuilder html = new StringBuilder("<html><body>").append(classId.name()).append(" Class Master:<br>");
				if(Config.CLASS_MASTERS_PRICE_LIST[jobLevel] > 0)
					html.append("Price: ").append(Config.CLASS_MASTERS_PRICE_LIST[jobLevel]).append("<br>");
				for(ClassId cid : ClassId.values())
					if(cid.childOf(classId) && cid.getLevel() == classId.getLevel() + 1)
						html.append("<a action=\"bypass -h npc_").append(getObjectId()).append("_change_class ").append(cid.getId()).append(" ").append(Config.CLASS_MASTERS_PRICE_LIST[jobLevel]).append("\">").append(cid.name()).append("</a><br1>");
				html.append("<br></body></html>");
				player.sendPacket(new NpcHtmlMessage(_objectId).setHtml(html.toString()));
			}
			else
			{
				NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
				StringBuffer sb = new StringBuffer();
				sb.append("<html><body>");
				switch(jobLevel)
				{
					case 1:
						sb.append("Come back here when you reached level 20 to change your class.<br>");
						break;
					case 2:
						sb.append("Come back here when you reached level 40 to change your class.<br>");
						break;
					case 3:
						sb.append("Come back here when you reached level 76 to change your class.<br>");
						break;
					case 4:
						sb.append("There is no class changes for you any more.<br>");
						break;
				}
				for(Quest q : Quest.findAllEvents())
					sb.append("Event: <a action=\"bypass -h Quest " + q.getName() + "\">" + q.getDescr() + "</a><br>");
				sb.append("</body></html>");
				html.setHtml(sb.toString());
				player.sendPacket(html);
			}

			player.sendActionFailed();
		}
	}

	@Override
	public String getHtmlPath(@SuppressWarnings("unused")
	int npcId, int val)
	{
		return "data/html/default/" + val + ".htm";
	}

	@Override
	public void onBypassFeedback(L2Player player, String command)
	{
		StringTokenizer st = new StringTokenizer(command);
		if(st.nextToken().equals("change_class"))
		{
			short val = Short.parseShort(st.nextToken());
			int price = Integer.parseInt(st.nextToken());
			if(player.getAdena() >= price)
			{
				player.reduceAdena(price);
				changeClass(player, val);
			}
			else
				player.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ADENA);
		}
		else
			super.onBypassFeedback(player, command);
	}

	private void changeClass(L2Player player, short val)
	{
		if(Config.DEBUG)
		{
			_log.fine("Changing class to ClassId:" + val);
			_log.fine("name:" + player.getName());
			_log.fine("level:" + player.getLevel());
			_log.fine("classId:" + player.getClassId());
		}

		if(val >= 88)
			player.sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_COMPLETED_THE_QUEST_FOR_3RD_OCCUPATION_CHANGE_AND_MOVED_TO_ANOTHER_CLASS_CONGRATULATIONS)); // для 3 профы
		else
			player.sendPacket(new SystemMessage(SystemMessage.CONGRATULATIONS_YOU_HAVE_TRANSFERRED_TO_A_NEW_CLASS)); // для 1 и 2 профы

		player.setClassId(val, false);
		player.broadcastUserInfo();
	}
}
