package L2jFT.game.model.instances;

import L2jFT.game.instancemanager.SiegeManager;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Spawn;
import L2jFT.game.model.entity.siege.Siege;
import static L2jFT.game.model.L2Zone.ZoneType.Siege;
import L2jFT.game.network.serverpackets.MyTargetSelected;
import L2jFT.game.network.serverpackets.StatusUpdate;
import L2jFT.game.network.serverpackets.ValidateLocation;
import L2jFT.game.tables.NpcTable;
import L2jFT.game.templates.L2NpcTemplate;

public class L2ControlTowerInstance extends L2NpcInstance
{
	private L2FakeTowerInstance fakeTower;

	public L2ControlTowerInstance(int objectId, L2NpcTemplate template)
	{
		super(objectId, template);
	}

	@Override
	public boolean isAutoAttackable(L2Character attacker)
	{
		if(attacker == null)
			return false;
		L2Player player = attacker.getPlayer();
		if(player == null)
			return false;
		L2Clan clan = player.getClan();
		//TODO присвоить осаду обьекту при спавне, чтобы избавиться от перебора
		return !(clan != null && SiegeManager.getInstance().getSiege(this) == clan.getSiege() && clan.getIsDefender());
	}

	public boolean isAttackable(L2Character attacker)
	{
		return isAutoAttackable(attacker);
	}

	@Override
	public void onForcedAttack(L2Player player)
	{
		player.setTarget(this);
		player.sendPacket(new MyTargetSelected(getObjectId(), player.getLevel() - getLevel()));
		StatusUpdate su = new StatusUpdate(getObjectId());
		su.addAttribute(StatusUpdate.CUR_HP, (int) getCurrentHp());
		su.addAttribute(StatusUpdate.MAX_HP, getMaxHp());
		player.sendPacket(su);
		player.sendPacket(new ValidateLocation(this));
		if(Math.abs(player.getZ() - getZ()) < 200)
			player.getAI().Attack(this, true);
		else
			player.sendActionFailed();
	}

	@Override
	public void onAction(L2Player player)
	{
		if(this != player.getTarget())
		{
			player.setTarget(this);
			player.sendPacket(new MyTargetSelected(getObjectId(), player.getLevel() - getLevel()));
			StatusUpdate su = new StatusUpdate(getObjectId());
			su.addAttribute(StatusUpdate.CUR_HP, (int) getCurrentHp());
			su.addAttribute(StatusUpdate.MAX_HP, getMaxHp());
			player.sendPacket(su);
			player.sendPacket(new ValidateLocation(this));
		}
		else
		{
			player.sendPacket(new MyTargetSelected(getObjectId(), player.getLevel() - getLevel()));
			if(Math.abs(player.getZ() - getZ()) < 200)
				player.getAI().Attack(this, false);
			else
				player.sendActionFailed();
		}
	}

	/**
	 * Вызывает обработку смерти у вышек, тем самым делая нужные действия.
	 * @param killer убийца
	 */
	@Override
	public void doDie(L2Character killer)
	{
		onDeath();
		super.doDie(killer);
	}

	/**
	 * Спавнит фэйковую вышку на месте умершей
	 */
	@Override
	public void onDecay()
	{
		super.onDecay();
		spawnFakeTower();
	}

	/**
	 * Убирает фэйковую вышку на месте новорожденной
	 */
	@Override
	public void onSpawn()
	{
		unSpawnFakeTower();
		super.onSpawn();
	}

	/**
	 * Обработка умерания вышки
	 */
	public void onDeath()
	{
		if(isInZone(Siege))
			SiegeManager.getInstance().getSiege(this).killedCT();
	}

	/**
	 * Спавнит фэйковую вышку на месте этой умершей.
	 * Создается один L2Spawn и один Инстанс, они привязываются к даному инстансу.
	 */
	public void spawnFakeTower()
	{
		if(fakeTower == null)
			try
			{
				L2Spawn spawn = new L2Spawn(NpcTable.getTemplate(getFakeTowerNpcId()));
				spawn.setLocx(getX());
				spawn.setLocy(getY());
				spawn.setLocz(getZ());
				fakeTower = (L2FakeTowerInstance) spawn.doSpawn(true);
				spawn.stopRespawn();
			}
			catch(ClassNotFoundException e)
			{
				e.printStackTrace(System.err);
			}
		else
			fakeTower.getSpawn().respawnNpc(fakeTower);
	}

	/**
	 * Убирает с мира фэйковую вышку которая относится к даному инстансу.
	 * Ссылка на обьект не обнуляется, т.к. он еше будет использован в перспективе
	 */
	public void unSpawnFakeTower()
	{
		if(fakeTower == null)
			return;

		fakeTower.decayMe();
	}

	@Override
	public boolean hasRandomAnimation()
	{
		return false;
	}

	/**
	 * Осадные вышки должны быть уязвимы во время осады, во время осады включается осадная зона
	 * Вывод - если не в осадной зоне, то не уязвимая
	 * @return уязвимая ли вышка
	 */
	@Override
	public boolean isInvul()
	{
		Siege siege = SiegeManager.getInstance().getSiege(this);
		return siege == null || !siege.isInProgress();
	}

	@Override
	public boolean isFearImmune()
	{
		return true;
	}

	@Override
	public boolean paralyzeImmune()
	{
		return true;
	}

	/**
	 * Возвращает ID Фэйковой вышки которая спавнится после смерти настоящей.
	 * Для Life Control Tower это 13003
	 * Для Flame Control Tower это 13005
	 * @return Fake Tower NPC ID
	 */
	private int getFakeTowerNpcId()
	{
		return getNpcId() + 1;
	}
}
