package L2jFT.game.model.instances;

import javolution.util.FastList;
import L2jFT.Config;
import L2jFT.extensions.scripts.Events;
import L2jFT.game.ThreadPoolManager;
import L2jFT.game.ai.CtrlIntention;
import L2jFT.game.ai.L2CharacterAI;
import L2jFT.game.ai.L2StaticObjectAI;
import L2jFT.game.instancemanager.SiegeManager;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2SiegeClan;
import L2jFT.game.model.L2World;
import L2jFT.game.model.entity.ClanHall;
import L2jFT.game.model.entity.siege.Siege;
import L2jFT.game.network.L2GameClient;
import L2jFT.game.network.serverpackets.DoorStatusUpdate;
import L2jFT.game.network.serverpackets.MyTargetSelected;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.network.serverpackets.ValidateLocation;
import L2jFT.game.templates.L2CharTemplate;
import L2jFT.game.templates.L2Weapon;

import java.util.concurrent.ScheduledFuture;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * This class ...
 *
 * @version $Revision: 1.3.2.2.2.5 $ $Date: 2005/03/27 15:29:32 $
 */
public class L2DoorInstance extends L2Character
{
	protected static Logger _log = Logger.getLogger(L2DoorInstance.class.getName());

	protected final int _doorId;
	protected final String _name;
	private boolean _open;
	private boolean _unlockable;
	private boolean _isHPVisible;
	private ClanHall _clanhall;

	protected int _autoActionDelay = -1;
	private ScheduledFuture _autoActionTask;
	private boolean _destroyable;

	@Override
	public L2CharacterAI getAI()
	{
		if(_ai == null)
			_ai = new L2StaticObjectAI(this);
		return _ai;
	}

	class CloseTask implements Runnable
	{
		public void run()
		{
			try
			{
				onClose();
			}
			catch(Throwable e)
			{
				_log.log(Level.SEVERE, "", e);
			}
		}
	}

	/**
	 * Manages the auto open and closing of a door. 
	 */
	class AutoOpenClose implements Runnable
	{
		public void run()
		{
			try
			{
				String doorAction;

				if(!isOpen())
				{
					doorAction = "opened";
					openMe();
				}
				else
				{
					doorAction = "closed";
					closeMe();
				}

				if(Config.DEBUG)
					_log.info("Auto " + doorAction + " door ID " + _doorId + " (" + _name + ") for " + _autoActionDelay / 1000 / 60 + " minute(s).");
			}
			catch(Exception e)
			{
				_log.warning("Could not auto open/close door ID " + _doorId + " (" + _name + ")");
			}
		}
	}

	public L2DoorInstance(int objectId, L2CharTemplate template, int doorId, String name, boolean unlockable, boolean showHp)
	{
		super(objectId, template);
		_doorId = doorId;
		_name = name;
		_unlockable = unlockable;
		_isHPVisible = showHp;
	}

	public final boolean isUnlockable()
	{
		return _unlockable;
	}

	@Override
	public final byte getLevel()
	{
		return 1;
	}

	/**
	 * @return Returns the doorId.
	 */
	public int getDoorId()
	{
		return _doorId;
	}

	/**
	 * @return Returns the open.
	 */
	public boolean isOpen()
	{
		return _open;
	}

	/**
	 * @param open The open to set.
	 */
	public void setOpen(boolean open)
	{
		_open = open;
	}

	/**
	 * Sets the delay in milliseconds for automatic opening/closing 
	 * of this door instance.
	 * <BR>
	 * <B>Note:</B> A value of -1 cancels the auto open/close task.
	 *
	 * @param int actionDelay
	 */
	public void setAutoActionDelay(int actionDelay)
	{
		if(_autoActionDelay == actionDelay)
			return;

		if(actionDelay > -1)

			ThreadPoolManager.getInstance().scheduleGeneralAtFixedRate(new AutoOpenClose(), actionDelay, actionDelay);
		else if(_autoActionTask != null)
			_autoActionTask.cancel(false);

		_autoActionDelay = actionDelay;
	}

	public int getDamage()
	{
		int dmg = 6 - (int) Math.ceil(getCurrentHp() / getMaxHp() * 6);
		if(dmg > 6)
			return 6;
		if(dmg < 0)
			return 0;
		return dmg;
	}

	//TODO разобраться
	public boolean isEnemyOf(@SuppressWarnings("unused")
	L2Character cha)
	{
		return true;
	}

	@Override
	public boolean isAutoAttackable(L2Character attacker)
	{
		if(isUnlockable() || isDestroyable())
			return true;
		if(attacker == null)
			return false;
		L2Player player = attacker.getPlayer();
		if(player == null)
			return false;
		L2Clan clan = player.getClan();
		//TODO присвоить осаду обьекту при спавне, чтобы избавиться от перебора
		return !(clan != null && SiegeManager.getInstance().getSiege(this) == clan.getSiege() && clan.getIsDefender());
	}

	public boolean isAttackable(L2Character attacker)
	{
		return isAutoAttackable(attacker);
	}

	public boolean isDestroyable()
	{
		return _destroyable;
	}

	public void setDestroyable(boolean val)
	{
		_destroyable = val;
	}

	@Override
	public void updateAbnormalEffect()
	{}

	/**
	 * Return null.<BR><BR>
	 */
	@Override
	public L2ItemInstance getActiveWeaponInstance()
	{
		return null;
	}

	@Override
	public L2Weapon getActiveWeaponItem()
	{
		return null;
	}

	@Override
	public L2ItemInstance getSecondaryWeaponInstance()
	{
		return null;
	}

	@Override
	public L2Weapon getSecondaryWeaponItem()
	{
		return null;
	}

	@Override
	public void onAction(L2Player player)
	{
		if(player == null)
			return;

		if(Events.onAction(player, this))
			return;

		if(this != player.getTarget())
		{
			player.setTarget(this);
			player.sendPacket(new MyTargetSelected(getObjectId(), player.getLevel()));

			if(isAutoAttackable(player))
				player.sendPacket(new DoorStatusUpdate(this));

			// correct location
			player.sendPacket(new ValidateLocation(this));
		}
		else
		{
			player.sendPacket(new MyTargetSelected(getObjectId(), player.getLevel()));
			if(isAutoAttackable(player))
			{
				if(Math.abs(player.getZ() - getZ()) < 400) // this max heigth difference might need some tweaking
					player.getAI().Attack(this, false);
				else
					player.sendActionFailed();
			}
			else if(!isInRange(player, INTERACTION_DISTANCE))
				player.getAI().setIntention(CtrlIntention.AI_INTENTION_INTERACT, this);
			else
			{
				if(Config.ALLOW_CH_DOOR_OPEN_ON_CLICK && player.getClan() != null && getClanHall() != null && player.getClanId() == getClanHall().getOwnerId())
					if(!isOpen())
						openMe();
					else
						closeMe();
				player.sendActionFailed();
			}
		}
	}

	public void onActionShift(L2GameClient client)
	{
		L2Player player = client.getActiveChar();

		if(Events.onActionShift(player, this))
			return;

		player.sendActionFailed();
	}

	@Override
	public void broadcastStatusUpdate()
	{
		DoorStatusUpdate su = new DoorStatusUpdate(this);
		FastList<L2Player> players = L2World.getAroundPlayers(this);
		for(FastList.Node<L2Player> n = players.head(), end = players.tail(); (n = n.getNext()) != end;)
			n.getValue().sendPacket(su);
	}

	public void onOpen()
	{
		scheduleCloseMe(60000);
	}

	public void onClose()
	{
		closeMe();
	}

	/**
	 * Вызывает задание на закрытие двери через заданное время.
	 * @param delay Время в миллисекундах
	 */
	public final void scheduleCloseMe(long delay)
	{
		ThreadPoolManager.getInstance().scheduleGeneral(new CloseTask(), delay);
	}

	public final void closeMe()
	{
		_open = false;
		broadcastStatusUpdate();
	}

	public final void openMe()
	{
		_open = true;
		broadcastStatusUpdate();
	}

	@Override
	public String toString()
	{
		return "door " + _doorId;
	}

	public String getDoorName()
	{
		return _name;
	}

	public void setClanHall(ClanHall clanhall)
	{
		_clanhall = clanhall;
	}

	public ClanHall getClanHall()
	{
		return _clanhall;
	}

	@Override
	public void doDie(L2Character killer)
	{
		Siege s = SiegeManager.getInstance().getSiege(this);
		if(s != null)
		{
			for(L2SiegeClan sc : s.getDefenderClans().values())
			{
				L2Clan clan = sc.getClan();
				if(clan != null)
					for(L2Player player : clan.getOnlineMembers(""))
						if(player != null)
							player.sendPacket(new SystemMessage(SystemMessage.THE_CASTLE_GATE_HAS_BEEN_BROKEN_DOWN));
			}

			for(L2SiegeClan sc : s.getAttackerClans().values())
			{
				L2Clan clan = sc.getClan();
				if(clan != null)
					for(L2Player player : clan.getOnlineMembers(""))
						if(player != null)
							player.sendPacket(new SystemMessage(SystemMessage.THE_CASTLE_GATE_HAS_BEEN_BROKEN_DOWN));
			}
		}

		super.doDie(killer);
	}

	public boolean isHPVisible()
	{
		return _isHPVisible;
	}

	public void setHPVisible(boolean val)
	{
		_isHPVisible = val;
	}

	private int upgradeHp;

	@Override
	public int getMaxHp()
	{
		return super.getMaxHp() + upgradeHp;
	}

	public void setUpgradeHp(int hp)
	{
		upgradeHp = hp;
	}

	public int getUpgradeHp()
	{
		return upgradeHp;
	}

	// Данные для геодаты

	// TODO Требуется сделать загрузку этих параметров, и поддержку в датапаке:

	// Нижний левый угол
	public int range_xmin;
	public int range_ymin;
	public int range_zmin;

	// Верхний правый угол
	public int range_xmax;
	public int range_ymax;
	public int range_zmax;

	// TODO сделать поддержку collision_radius в датапаке (сейчас это константа)

	@Override
	public boolean isLethalImmune()
	{
		return true;
	}

	/**
	 * Двери на осадах уязвимы во время осады.
	 * Остальные двери не уязвимы вообще.
	 * @return инвульная ли дверь.
	 */
	@Override
	public boolean isInvul()
	{
		Siege siege = SiegeManager.getInstance().getSiege(this);
		return siege == null || !siege.isInProgress();
	}

	@Override
	public boolean isFearImmune()
	{
		return true;
	}

	@Override
	public boolean paralyzeImmune()
	{
		return true;
	}
}
