package L2jFT.game.model.instances;

import L2jFT.game.model.L2Character;
import L2jFT.game.templates.L2NpcTemplate;

/**
 * This class manages all Minions. 
 * In a group mob, there are one master called RaidBoss and several slaves called Minions.
 *
 * @version $Revision: 1.20.4.6 $ $Date: 2005/04/06 16:13:39 $
 */
public final class L2MinionInstance extends L2MonsterInstance
{
	//private static Logger _log = Logger.getLogger(L2RaidMinionInstance.class.getName());

	/** The master L2Character whose depends this L2MinionInstance on */
	private L2MonsterInstance _master;

	/**
	 * Constructor of L2MinionInstance (use L2Character and L2NpcInstance constructor).<BR><BR>
	 *
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>Call the L2Character constructor to set the _template of the L2MinionInstance (copy skills from template to object and link _calculators to NPC_STD_CALCULATOR) </li>
	 * <li>Set the name of the L2MinionInstance</li>
	 * <li>Create a RandomAnimation Task that will be launched after the calculated delay if the server allow it </li><BR><BR>
	 *
	 * @param objectId Identifier of the object to initialized
	 * @param L2NpcTemplate Template to apply to the NPC
	 */
	public L2MinionInstance(int objectId, L2NpcTemplate template)
	{
		super(objectId, template);
	}

	/**
	 * Return the master of this L2MinionInstance.<BR><BR>
	 */
	public L2MonsterInstance getLeader()
	{
		return _master;
	}

	@Override
	public void onSpawn()
	{
		// Notify Leader that Minion has Spawned
		getLeader().notifyMinionSpawned(this);
	}

	/**
	 * Set the master of this L2MinionInstance.<BR><BR>
	 *
	 * @param leader The L2Character that leads this L2MinionInstance
	 *
	 */
	public void setLeader(L2MonsterInstance leader)
	{
		_master = leader;
	}

	/**
	 * Manages the doDie event for this L2MinionInstance.<BR><BR>
	 *
	 * @param killer The L2Character that killed this L2MinionInstance.<BR><BR>
	 */
	@Override
	public void doDie(L2Character killer)
	{
		_master.notifyMinionDied(this);

		super.doDie(killer);
	}

	@Override
	public boolean isFearImmune()
	{
		return true;
	}
}
