package L2jFT.game.model.instances;

import static L2jFT.game.ai.CtrlIntention.AI_INTENTION_ACTIVE;

import java.io.File;
import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

import javolution.text.TextBuilder;
import javolution.util.FastList;
import L2jFT.Config;
import L2jFT.extensions.multilang.CustomMessage;
import L2jFT.extensions.scripts.Events;
import L2jFT.extensions.scripts.Scripts;
import L2jFT.game.ThreadPoolManager;
import L2jFT.game.ai.CtrlEvent;
import L2jFT.game.ai.CtrlIntention;
import L2jFT.game.ai.L2CharacterAI;
import L2jFT.game.cache.Msg;
import L2jFT.game.idfactory.IdFactory;
import L2jFT.game.instancemanager.DimensionalRiftManager;
import L2jFT.game.instancemanager.QuestManager;
import L2jFT.game.instancemanager.TownManager;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2Playable;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.L2SkillLearn;
import L2jFT.game.model.L2Spawn;
import L2jFT.game.model.L2World;
import L2jFT.game.model.base.ClassId;
import L2jFT.game.model.base.L2EnchantSkillLearn;
import L2jFT.game.model.entity.Castle;
import L2jFT.game.model.entity.Olympiad;
import L2jFT.game.model.entity.SevenSigns;
import L2jFT.game.model.quest.Quest;
import L2jFT.game.model.quest.QuestState;
import L2jFT.game.network.serverpackets.AquireSkillList;
import L2jFT.game.network.serverpackets.ExEnchantSkillList;
import L2jFT.game.network.serverpackets.ExShowVariationCancelWindow;
import L2jFT.game.network.serverpackets.ExShowVariationMakeWindow;
import L2jFT.game.network.serverpackets.InventoryUpdate;
import L2jFT.game.network.serverpackets.MultiSellList;
import L2jFT.game.network.serverpackets.MyTargetSelected;
import L2jFT.game.network.serverpackets.NpcHtmlMessage;
import L2jFT.game.network.serverpackets.NpcInfo;
import L2jFT.game.network.serverpackets.SocialAction;
import L2jFT.game.network.serverpackets.StatusUpdate;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.network.serverpackets.ValidateLocation;
import L2jFT.game.skills.Stats;
import L2jFT.game.tables.ClanTable;
import L2jFT.game.tables.ItemTable;
import L2jFT.game.tables.SkillTable;
import L2jFT.game.tables.SkillTreeTable;
import L2jFT.game.taskmanager.DecayTaskManager;
import L2jFT.game.templates.L2Item;
import L2jFT.game.templates.L2NpcTemplate;
import L2jFT.game.templates.L2Weapon;
import L2jFT.util.Files;
import L2jFT.util.Log;
import L2jFT.util.Rnd;

public class L2NpcInstance extends L2Character
{
	private static final Logger _log = Logger.getLogger(L2NpcInstance.class.getName());
	private final ClassId[] _classesToTeach;

	/** The delay after witch the attacked is stopped */
	private long _attack_timeout;
	private int _spawnedx;
	private int _spawnedy;
	private int _spawnedz;

	private Constructor _ai_constructor;

	@Override
	public synchronized L2CharacterAI getAI()
	{
		if(_ai == null)
		{
			if(_ai_constructor != null)
				try
				{
					_ai = (L2CharacterAI) _ai_constructor.newInstance(new Object[] { this });
				}
				catch(Exception e)
				{
					e.printStackTrace();
				}
			if(_ai == null)
				_ai = new L2CharacterAI(this);
		}
		return _ai;
	}

	public void setAttackTimeout(long time)
	{
		_attack_timeout = time;
	}

	public long getAttackTimeout()
	{
		return _attack_timeout;
	}

	/**
	 * Return the X position of the spawned point.<BR><BR>
	 */
	public int getSpawnedX()
	{
		return _spawnedx;
	}

	/**
	 * Return the Y position of the spawned point.<BR><BR>
	 */
	public int getSpawnedY()
	{
		return _spawnedy;
	}

	/**
	 * Return the Z position of the spawned point.<BR><BR>
	 */
	public int getSpawnedZ()
	{
		return _spawnedz;
	}

	public void setSpawnedX(int x)
	{
		_spawnedx = x;
	}

	public void setSpawnedY(int y)
	{
		_spawnedy = y;
	}

	public void setSpawnedZ(int z)
	{
		_spawnedz = z;
	}

	/**
	 * This class contains all AggroInfo of the L2NpcInstance against the attacker L2Character.<BR><BR>
	 *
	 * <B><U> Data</U> :</B><BR><BR>
	 * <li>attacker : The attaker L2Character concerned by this AggroInfo of this L2NpcInstance </li>
	 * <li>hate : Hate level of this L2NpcInstance against the attaker L2Character (hate = damage) </li>
	 * <li>damage : Number of damages that the attaker L2Character gave to this L2NpcInstance </li><BR><BR>
	 *
	 */
	public final class AggroInfo
	{
		/** The attaker L2Character concerned by this AggroInfo of this L2NpcInstance */
		public L2Character attacker;

		/** Hate level of this L2NpcInstance against the attaker L2Character (hate = damage) */
		int hate;

		/** Number of damages that the attaker L2Character gave to this L2NpcInstance */
		public int damage;

		/**
		 * Constructor of AggroInfo.<BR><BR>
		 */
		AggroInfo(final L2Character attacker)
		{
			this.attacker = attacker;
		}

		/**
		 * Verify is object is equal to this AggroInfo.<BR><BR>
		 */
		@Override
		public boolean equals(final Object obj)
		{
			return this == obj; //attacker == attacker - какой смысел этого - хз. В общем по моему бага.
		}

		/**
		 * Return the Identifier of the attacker L2Character.<BR><BR>
		 */
		@Override
		public int hashCode()
		{
			return attacker.getObjectId();
		}
	}

	/** The table containing all autoAttackable L2Character in its Aggro Range and L2Character that attacked the L2NpcInstance */
	protected ConcurrentHashMap<L2Character, AggroInfo> _aggroList = new ConcurrentHashMap<L2Character, AggroInfo>(0);
	private int _aggroListSize = 0;
	AggroInfo mosthated = null;

	public L2NpcInstance(int objectId, L2NpcTemplate template)
	{
		super(objectId, template);

		isNpc = true;

		if(template == null)
		{
			_log.warning("No template for Npc. Please check your datapack is setup correctly.");
			throw new IllegalArgumentException();
		}

		_classesToTeach = template.getTeachInfo();
		setName(template.name);
		setTitle(template.title);

		String implementationName = template.ai_type;

		try
		{
			if(!implementationName.equalsIgnoreCase("npc"))
				_ai_constructor = Class.forName("L2jFT.game.ai." + implementationName).getConstructors()[0];
		}
		catch(Exception e)
		{
			try
			{
				_ai_constructor = Scripts.getInstance().getClasses().get("ai." + implementationName).getRawClass().getConstructors()[0];
			}
			catch(Exception e1)
			{
				_log.warning("AI type " + template.ai_type + " not found!");
				e1.printStackTrace();
			}
		}

		// Т.к. у нас не создаются инстансы при каждом спавне, то все ок.
		if(hasRandomAnimation() && !(this instanceof L2MonsterInstance)) // для монстров через AI
			startRandomAnimation();
	}

	/**
	 * Reduce the current HP of the L2NpcInstance, update its _aggroList and launch the doDie Task if necessary.<BR><BR>
	 */
	@Override
	public void reduceCurrentHp(final double damage, final L2Character attacker, final boolean awake, final boolean standUp, boolean directHp)
	{
		if(this instanceof L2SiegeGuardInstance && attacker instanceof L2SiegeGuardInstance)
			//if((this.getEffect(L2Effect.EffectType.CONFUSION)!=null) && (attacker.getEffect(L2Effect.EffectType.CONFUSION)!=null))
			return;

		// Add damage and hate to the attacker AggroInfo of the L2NpcInstance _aggroList
		if(attacker != null)
			addDamage(attacker, (int) damage);

		// If this L2NpcInstance is a L2MonsterInstance and it has spawned minions, call its minions to battle
		if(isMonster)
		{
			L2MonsterInstance master = (L2MonsterInstance) this;
			if(this instanceof L2MinionInstance)
			{
				master = ((L2MinionInstance) this).getLeader();
				//	if(!master.isInCombat())
				if(!master.isInCombat() && !master.isDead())
					master.addDamage(attacker, 1);
			}
			if(master.hasMinions())
				master.callMinionsToAssist(attacker);
		}

		// Reduce the current HP of the L2NpcInstance and launch the doDie Task if necessary
		super.reduceCurrentHp(damage, attacker, awake, standUp, directHp);
	}

	/**
	 * Kill the L2NpcInstance (the corpse disappeared after 7 seconds), distribute rewards (EXP, SP, Drops...) and notify Quest Engine.<BR><BR>
	 */
	@Override
	public void doDie(L2Character killer)
	{
		setDecayed(false);
		DecayTaskManager.getInstance().addDecayTask(this);

		super.doDie(killer);
	}

	public void addDamage(L2Character attacker, int damage)
	{
		addDamageHate(attacker, damage, damage);
	}

	public void addDamageHate(L2Character attacker, int damage, int aggro)
	{
		addDamageHate(attacker, damage, aggro, false);
	}

	public void addDamageHate(L2Character attacker, int damage, int aggro, boolean canBeAttackedByMob)
	{
		if(damage < 0)
			damage = 0;

		if(damage > 0 && aggro <= 0)
			aggro = damage;

		if(attacker == null || attacker == this)
			return;

		// Get the AggroInfo of the attacker L2Character from the _aggroList of the L2NpcInstance
		AggroInfo ai = _aggroList.get(attacker);

		if(ai != null)
		{
			// Add new damage and aggro (=damage) to the AggroInfo object
			ai.damage += damage;
			ai.hate += aggro;

			if(ai.hate < 0)
				ai.hate = 0;

			if(aggro < 0 && (mosthated == null || ai.attacker == mosthated.attacker))
				calculateMostHated();
			else if(mosthated == null || ai.attacker != mosthated.attacker && ai.hate > mosthated.hate)
				mosthated = ai;
		}
		else if(aggro > 0)
		{
			// Create a AggroInfo object and Init it
			ai = new AggroInfo(attacker);
			ai.damage = damage;
			ai.hate = aggro;

			mosthated = ai;

			// Add the attacker L2Character and the AggroInfo object in the _aggroList of the L2NpcInstance
			_aggroList.put(attacker, ai);
			_aggroListSize++;
		}

		if(aggro > 0 && getAI().getIntention() == CtrlIntention.AI_INTENTION_IDLE)
			getAI().setIntention(CtrlIntention.AI_INTENTION_ACTIVE, null, null);

		if(damage > 0)
			getAI().notifyEvent(CtrlEvent.EVT_ATTACKED, attacker, canBeAttackedByMob);

		try
		{
			if(attacker instanceof L2Playable && attacker.getPlayer() != null)
			{
				QuestState[] quests = attacker.getPlayer().getQuestsForAttacks(this);
				if(quests != null)
					for(final QuestState qs : quests)
						qs.getQuest().notifyAttack(this, qs);
			}
		}
		catch(final Exception e)
		{
			_log.log(Level.SEVERE, "", e);
			e.printStackTrace();
		}
	}

	public ConcurrentHashMap<L2Character, AggroInfo> getAggroList()
	{
		return _aggroList;
	}

	public void removeFromAggrolist(L2Character target)
	{
		if(target != null)
		{
			_aggroList.remove(target);
			_aggroListSize--;
			if(mosthated == null || mosthated.attacker == target)
				calculateMostHated();
		}
	}

	private void calculateMostHated()
	{
		if(_aggroListSize == 0)
		{
			mosthated = null;
			return;
		}

		ConcurrentHashMap<L2Character, AggroInfo> temp = new ConcurrentHashMap<L2Character, AggroInfo>();

		_aggroListSize = 0;

		for(AggroInfo ai : _aggroList.values())
		{
			if(ai.attacker == null)
				continue;
			if(ai.attacker.isAlikeDead())
				continue;
			if(!ai.attacker.isVisible())
				continue;
			if(!isInRange(ai.attacker, 2000))
				continue;
			temp.put(ai.attacker, ai);
			_aggroListSize++;
		}

		_aggroList = temp;
		mosthated = null;

		for(AggroInfo ai : _aggroList.values())
			if(mosthated == null)
				mosthated = ai;
			else if(mosthated.hate < ai.hate)
				mosthated = ai;
	}

	public L2Character getMostHated()
	{
		if(_aggroListSize == 0)
		{
			mosthated = null;
			return null;
		}

		if(mosthated == null || mosthated.attacker == null || mosthated.attacker.isAlikeDead() || !mosthated.attacker.isVisible() || !isInRange(mosthated.attacker, 2000))
			calculateMostHated();

		return mosthated != null ? mosthated.attacker : null;
	}

	public int getHating(L2Character target)
	{
		if(target == null)
			return 0;

		AggroInfo ai = _aggroList.get(target);

		if(ai == null)
			return 0;

		if(ai.attacker == null || !ai.attacker.isVisible() || ai.attacker.isAlikeDead() || !isInRange(ai.attacker, 2000))
		{
			_aggroList.remove(target);
			_aggroListSize--;
			if(mosthated == null || mosthated.attacker == target)
				calculateMostHated();
			return 0;
		}

		return ai.hate;
	}

	@Override
	public void clearAggroList()
	{
		_aggroList.clear();
		_aggroListSize = 0;
		mosthated = null;
	}

	public boolean noTarget()
	{
		return _aggroListSize == 0;
	}

	public boolean containsTarget(L2Character player)
	{
		return _aggroList.containsKey(player);
	}

	public int getAggroListSize()
	{
		return _aggroListSize;
	}

	public void dropItem(L2Player lastAttacker, int itemId, int itemCount)
	{
		if(itemCount == 0 || lastAttacker == null)
			return;

		if(Config.DROP_COUNTER)
			lastAttacker.incrementDropCounter(itemId, itemCount);

		L2ItemInstance item;

		for(int i = 1; i <= itemCount; i++)
		{
			item = ItemTable.getInstance().createItem(itemId, lastAttacker.getObjectId(), getNpcId(), "L2NpcInstance.dropItem");

			// Set the Item quantity dropped if L2ItemInstance is stackable
			if(item.isStackable())
			{
				i = itemCount; // Set so loop won't happent again
				item.setCount(itemCount); // Set item count
			}
			if(Config.DEBUG)
				_log.fine("Item id to drop: " + itemId + " amount: " + item.getCount());

			if(this instanceof L2RaidBossInstance)
			{
				final SystemMessage sm = new SystemMessage(SystemMessage.S1_DIED_AND_DROPPED_S3_S2);
				sm.addString(getName());
				sm.addItemName(itemId);
				sm.addNumber(item.getCount());
				broadcastPacket(sm);
			}

			// Check if the autoLoot mode is active
			if(Config.AUTO_LOOT)
			{
				if(!Config.AUTO_LOOT_HERBS && itemId >= 8600 && itemId <= 8614)
					item.dropToTheGround(lastAttacker, this);
				else
					lastAttacker.doAutoLoot(item, this);
			}
			else
				item.dropToTheGround(lastAttacker, this);
		}
	}

	public void dropItem(L2Player lastAttacker, L2ItemInstance item)
	{
		if(item.getCount() == 0)
			return;

		if(Config.DEBUG)
			_log.fine("Item id to drop: " + item + " amount: " + item.getCount());

		if(this instanceof L2RaidBossInstance)
		{
			final SystemMessage sm = new SystemMessage(SystemMessage.S1_DIED_AND_DROPPED_S3_S2);
			sm.addString(getName());
			sm.addItemName(item.getItemId());
			sm.addNumber(item.getCount());
			broadcastPacket(sm);
		}

		// Check if the autoLoot mode is active
		if(Config.AUTO_LOOT)
			lastAttacker.doAutoLoot(item, this); // Give this or these Item(s) to the L2Player that has killed the L2NpcInstance
		else
			item.dropToTheGround(lastAttacker, this);
	}

	public L2ItemInstance getActiveWeapon()
	{
		return null;
	}

	@Override
	public boolean isAttackable()
	{
		return true;
	}

	@Override
	public boolean isAutoAttackable(L2Character attacker)
	{
		return false;
	}

	@Override
	public void onSpawn()
	{
		clearAggroList();
		setDecayed(false);
	}

	@Override
	public final L2NpcTemplate getTemplate()
	{
		return (L2NpcTemplate) _template;
	}

	@Override
	public int getNpcId()
	{
		return getTemplate().npcId;
	}

	protected boolean _unAggred = false;

	public int getAggroRange()
	{
		return _unAggred ? 0 : getTemplate().aggroRange;
	}

	public int getFactionRange()
	{
		return getTemplate().factionRange;
	}

	public final String getFactionId()
	{
		return getTemplate().factionId;
	}

	public long getExpReward()
	{
		return (long) calcStat(Stats.EXP_SP, getTemplate().revardExp * Config.RATE_XP, null, null);
	}

	public long getSpReward()
	{
		return (long) calcStat(Stats.EXP_SP, getTemplate().revardSp * Config.RATE_SP, null, null);
	}

	/**
	 * Remove PROPERLY the L2NpcInstance from the world.<BR><BR>
	 *
	 * <B><U> Actions</U> :</B><BR><BR>
	 * <li>Remove the L2NpcInstance from the world and update its spawn object </li>
	 * <li>Remove L2Object object from _allObjects of L2World </li><BR><BR>
	 *
	 * <FONT COLOR=#FF0000><B> <U>Caution</U> : This method DOESN'T SEND Server->Client packets to players</B></FONT><BR><BR>
	 *
	 */
	public void deleteMe()
	{
		decayMe();
		if(hasAI())
			getAI().stopAITask();
		detachAI();
		L2World.removeObject(this);
	}

	private L2Spawn _spawn;

	public L2Spawn getSpawn()
	{
		return _spawn;
	}

	public void setSpawn(L2Spawn spawn)
	{
		_spawn = spawn;
	}

	@Override
	public synchronized void onDecay()
	{
		if(isDecayed())
			return;

		setDecayed(true);

		// Remove the L2NpcInstance from the world when the decay task is launched
		super.onDecay();

		// Decrease its spawn counter
		if(_spawn != null)
			_spawn.decreaseCount(this);
	}

	private boolean _isDecayed = false;

	/**
	 * Set the decayed state of this L2NpcInstance<BR><BR>
	 */
	public final void setDecayed(boolean mode)
	{
		_isDecayed = mode;
	}

	/**
	 * Return the decayed status of this L2NpcInstance<BR><BR>
	 */
	public final boolean isDecayed()
	{
		return _isDecayed;
	}

	public void endDecayTask()
	{
		DecayTaskManager.getInstance().cancelDecayTask(this);
		onDecay();
	}

	/**
	 * Return True if the L2NpcInstance is agressive (ex : L2MonsterInstance in function of aggroRange).<BR><BR>
	 */
	public boolean isAggressive()
	{
		return false;
	}

	public void unAggro(boolean state, L2Character effector)
	{
		_unAggred = state;
		if(state)
		{
			int hate = getHating(effector);
			if(hate > 0)
				addDamageHate(effector, 0, -hate);
			setAttackTimeout(Integer.MAX_VALUE);
			getAI().setAttackTarget(null);
			getAI().clientStopMoving(null);
			getAI().setIntention(AI_INTENTION_ACTIVE);
		}
	}

	/**
	 * Return True if this L2NpcInstance is undead in function of the L2NpcTemplate.<BR><BR>
	 */
	@Override
	public boolean isUndead()
	{
		return getTemplate().isUndead();
	}

	/**
	 * Return the Level of this L2NpcInstance contained in the L2NpcTemplate.<BR><BR>
	 */
	@Override
	public final byte getLevel()
	{
		return getTemplate().level;
	}

	/**
	 * Return null (regular NPCs don't have weapons instancies).<BR><BR>
	 */
	@Override
	public L2ItemInstance getActiveWeaponInstance()
	{
		// regular NPCs dont have weapons instancies
		return null;
	}

	/**
	 * Return the weapon item equipped in the right hand of the L2NpcInstance or null.<BR><BR>
	 */
	@Override
	public L2Weapon getActiveWeaponItem()
	{
		// Get the weapon identifier equipped in the right hand of the L2NpcInstance
		int weaponId = getTemplate().rhand;

		if(weaponId < 1)
			return null;

		// Get the weapon item equipped in the right hand of the L2NpcInstance
		L2Item item = ItemTable.getInstance().getTemplate(getTemplate().rhand);

		if(!(item instanceof L2Weapon))
			return null;

		return (L2Weapon) item;
	}

	/**
	 * Return null (regular NPCs don't have weapons instances).<BR><BR>
	 */
	@Override
	public L2ItemInstance getSecondaryWeaponInstance()
	{
		// regular NPCs dont have weapons instances
		return null;
	}

	/**
	 * Return the weapon item equipped in the left hand of the L2NpcInstance or null.<BR><BR>
	 */
	@Override
	public L2Weapon getSecondaryWeaponItem()
	{
		// Get the weapon identifier equipped in the right hand of the L2NpcInstance
		int weaponId = getTemplate().lhand;

		if(weaponId < 1)
			return null;

		// Get the weapon item equipped in the right hand of the L2NpcInstance
		L2Item item = ItemTable.getInstance().getTemplate(getTemplate().lhand);

		if(!(item instanceof L2Weapon))
			return null;

		return (L2Weapon) item;
	}

	/**
	 * Send a packet NpcInfo with state of abnormal effect to all visible L2Player<BR><BR>
	 */
	@Override
	public void updateAbnormalEffect()
	{
		if(!isKnownPlayers())
			return;

		FastList<L2Player> players = L2World.getAroundPlayers(this);
		for(FastList.Node<L2Player> n = players.head(), end = players.tail(); (n = n.getNext()) != end;)
			n.getValue().sendPacket(new NpcInfo(this, n.getValue()));
	}

	@Override
	public void onActionShift(L2Player player)
	{
		if(this != player.getTarget())
		{
			if(Config.DEBUG)
				_log.fine("new target selected:" + getObjectId());
			player.setTarget(this);
			player.sendPacket(new MyTargetSelected(getObjectId(), player.getLevel() - getLevel()));
			if(isAutoAttackable(player))
			{
				StatusUpdate su = new StatusUpdate(getObjectId());
				su.addAttribute(StatusUpdate.CUR_HP, (int) getCurrentHp());
				su.addAttribute(StatusUpdate.MAX_HP, getMaxHp());
				player.sendPacket(su);
			}
			player.sendPacket(new ValidateLocation(this));
			player.sendActionFailed();
			Events.onActionShift(player, this);
			return;
		}

		player.sendPacket(new MyTargetSelected(getObjectId(), player.getLevel() - getLevel()));

		if(Events.onActionShift(player, this))
			return;

		player.sendActionFailed();
	}

	// У NPC всегда 2
	public void onRandomAnimation()
	{
		broadcastPacket(new SocialAction(getObjectId(), 2));
	}

	public void startRandomAnimation()
	{
		int interval = 1000 * Rnd.get(Config.MIN_NPC_ANIMATION, Config.MAX_NPC_ANIMATION);
		ThreadPoolManager.getInstance().scheduleGeneral(new RandomAnimationTask(), interval);
	}

	public class RandomAnimationTask implements Runnable
	{
		private final int interval;

		public RandomAnimationTask()
		{
			interval = 1000 * Rnd.get(Config.MIN_NPC_ANIMATION, Config.MAX_NPC_ANIMATION);
		}

		public void run()
		{
			if(!isDead() && !isMoving && !isFollow)
				onRandomAnimation();

			ThreadPoolManager.getInstance().scheduleGeneral(this, interval);
		}
	}

	@Override
	public boolean hasRandomAnimation()
	{
		return Config.MAX_NPC_ANIMATION > 0;
	}

	@Override
	public boolean isInvul()
	{
		return true;
	}

	public Castle getCastle()
	{
		return TownManager.getInstance().getClosestTown(this).getCastle();
	}

	private long _lastSocialAction;

	@Override
	public void onAction(L2Player player)
	{
		if(this != player.getTarget())
		{
			if(Config.DEBUG)
				_log.fine("new target selected:" + getObjectId());
			player.setTarget(this);
			player.sendPacket(new MyTargetSelected(getObjectId(), player.getLevel() - getLevel()));
			if(isAutoAttackable(player))
			{
				StatusUpdate su = new StatusUpdate(getObjectId());
				su.addAttribute(StatusUpdate.CUR_HP, (int) getCurrentHp());
				su.addAttribute(StatusUpdate.MAX_HP, getMaxHp());
				player.sendPacket(su);
			}
			player.sendPacket(new ValidateLocation(this));
			player.sendActionFailed();
			return;
		}

		player.sendPacket(new MyTargetSelected(getObjectId(), player.getLevel() - getLevel()));

		if(Events.onAction(player, this))
			return;

		if(isAutoAttackable(player))
		{
			player.getAI().Attack(this, false);
			return;
		}

		if(!isInRange(player, INTERACTION_DISTANCE))
		{
			if(player.getAI().getIntention() != CtrlIntention.AI_INTENTION_INTERACT)
				player.getAI().setIntention(CtrlIntention.AI_INTENTION_INTERACT, this, null);
			return;
		}

		if(!Config.ALT_GAME_KARMA_PLAYER_CAN_SHOP && player.getKarma() > 0 && !player.isGM())
		{
			player.sendActionFailed();
			return;
		}

		//С NPC нельзя разговаривать мертвым и сидя 
		if(!Config.ALLOW_TALK_WHILE_SITTING && player.isSitting() || player.isAlikeDead())
			return;

		if(System.currentTimeMillis() - _lastSocialAction > 10000)
			broadcastPacket(new SocialAction(getObjectId(), 2));

		_lastSocialAction = System.currentTimeMillis();

		player.sendActionFailed();

		if(_isBusy)
			showBusyWindow(player);
		else
		{
			Quest[] qlst = getTemplate().getEventQuests(Quest.QuestEventType.NPC_FIRST_TALK);
			if(qlst != null && qlst.length == 1)
				qlst[0].notifyFirstTalk(this, player);
			else
				showChatWindow(player, 0);
		}
	}

	public void showQuestWindow(L2Player player, String questId)
	{
		if(!player.isQuestContinuationPossible())
			return;

		String content;

		try
		{
			// Get the state of the selected quest
			QuestState qs = player.getQuestState(questId);

			if(qs != null)
			{
				// If the quest is already started, no need to show a window
				if(!qs.getQuest().notifyTalk(this, qs))
					return;
			}
			else
			{
				Quest q = QuestManager.getInstance().getQuest(questId);
				if(q != null)
				{
					// check for start point
					Quest[] qlst = getTemplate().getEventQuests(Quest.QuestEventType.QUEST_START);
					if(qlst != null && qlst.length > 0)
						for(Quest element : qlst)
							if(element == q)
							{
								qs = q.newQuestState(player);
								if(!qs.getQuest().notifyTalk(this, qs))
									return;
								break;
							}
				}
			}
			if(qs == null)
				// no quests found
				content = "<html><body>I have no tasks for you right now.</body></html>";
			else
			{
				questId = qs.getQuest().getName();
				String stateId = qs.getStateId();
				String fileName = "data/jscript/quests/" + questId + "/" + stateId + ".htm";
				content = Files.read(fileName, player);

				if(content == null)
					content = Files.read("data/scripts/" + getClass().getPackage().toString().replace(".", "/").substring(8) + "/" + stateId + ".htm", player);

				if(content == null)
					content = "Can't find file'" + fileName + "'";
			}
			NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
			html.setHtml(content);
			player.sendPacket(html);
		}
		catch(Exception e)
		{
			_log.warning("problem with npc text " + e);
			e.printStackTrace();
		}

		player.sendActionFailed();
	}

	/**
	 * Open a quest or chat window on client with the text of the L2NpcInstance in function of the command.<BR><BR>
	 *
	 * <B><U> Example of use </U> :</B><BR><BR>
	 * <li> Client packet : RequestBypassToServer</li><BR><BR>
	 *
	 * @param command The command string received from client
	 *
	 */
	public void onBypassFeedback(L2Player player, String command)
	{
		if(!isInRange(player, INTERACTION_DISTANCE))
		{
			player.getAI().setIntention(CtrlIntention.AI_INTENTION_ATTACK, this);
			player.sendActionFailed();
		}
		else
			try
			{
				if(command.equalsIgnoreCase("TerritoryStatus"))
				{
					NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
					html.setFile("data/html/merchant/territorystatus.htm");
					html.replace("%npcname%", getName());

					if(getCastle() != null && getCastle().getCastleId() > 0)
					{
						html.replace("%castlename%", getCastle().getName());
						html.replace("%taxpercent%", String.valueOf(getCastle().getTaxPercent()));

						if(getCastle().getOwnerId() > 0)
						{
							L2Clan clan = ClanTable.getInstance().getClan(getCastle().getOwnerId());
							html.replace("%clanname%", clan.getName());
							html.replace("%clanleadername%", clan.getLeaderName());
						}
						else
						{
							html.replace("%clanname%", "NPC");
							html.replace("%clanleadername%", "None");
						}
					}
					else
					{
						html.replace("%castlename%", "Open");
						html.replace("%taxpercent%", "0");

						html.replace("%clanname%", "No");
						html.replace("%clanleadername%", getName());
					}

					player.sendPacket(html);
				}
				else if(command.equalsIgnoreCase("Exchange"))
				{
					NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
					html.setFile("data/html/merchant/exchange.htm");
					html.replace("%npcname%", getName());
					player.sendPacket(html);
				}
				else if(command.startsWith("Quest"))
				{
					String quest = command.substring(5).trim();
					if(quest.length() == 0)
						showQuestWindow(player);
					else
						showQuestWindow(player, quest);
				}
				else if(command.startsWith("Chat"))
					try
					{
						int val = Integer.parseInt(command.substring(5));
						showChatWindow(player, val);
					}
					catch(NumberFormatException nfe)
					{
						String filename = command.substring(5).trim();
						if(filename.length() == 0)
							showChatWindow(player, "data/html/npcdefault.htm");
						else
							showChatWindow(player, filename);
					}
				else if(command.startsWith("Loto"))
				{
					int val = Integer.parseInt(command.substring(5));
					if(val == 0)
						// new loto ticket
						for(int i = 0; i < 5; i++)
							player.setLoto(i, 0);
					showLotoWindow(player, val);
				}
				else if(command.startsWith("CPRecovery"))
					makeCPRecovery(player);
				else if(command.startsWith("SupportMagic"))
					makeSupportMagic(player);
				else if(command.startsWith("ProtectionBlessing"))
				{
					// Не выдаём блессиг протекшена ПКшникам.
					if(player.getKarma() > 0)
						return;
					if(player.getLevel() > 39 || player.getClassId().getLevel() >= 2)
					{
						String content = "<html><body>Blessing of protection not available for characters whose level more than 39 or complited second class transfer.</body></html>";
						NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
						html.setHtml(content);
						player.sendPacket(html);
						return;
					}
					doCast(SkillTable.getInstance().getInfo(5182, 1), player, true);
				}
				else if(command.startsWith("Multisell") || command.startsWith("multisell"))
				{
					String listId = command.substring(9).trim();
					player.sendPacket(new MultiSellList(player, Integer.parseInt(listId), getCastle() != null ? getCastle().getTaxRate() : 0));
				}
				else if(command.startsWith("EnterRift"))
				{
					StringTokenizer st = new StringTokenizer(command);
					st.nextToken(); //no need for "enterRift"

					Byte b1 = Byte.parseByte(st.nextToken()); //type

					DimensionalRiftManager.getInstance().start(player, b1, this);
				}
				else if(command.startsWith("ChangeRiftRoom"))
				{
					if(player.isInParty() && player.getParty().isInDimensionalRift())
						player.getParty().getDimensionalRift().manualTeleport(player, this);
					else
					{
						DimensionalRiftManager.getInstance().showHtmlFile(player, "data/html/rift/Cheater.htm", this);
						if(!player.isGM())
							_log.warning("Player " + player.getName() + "(" + player.getObjectId() + ") was cheating in dimension rift area!");
					}
				}
				else if(command.startsWith("ExitRift"))
				{
					if(player.isInParty() && player.getParty().isInDimensionalRift())
						player.getParty().getDimensionalRift().manualExitRift(player, this);
					else
					{
						DimensionalRiftManager.getInstance().showHtmlFile(player, "data/html/rift/Cheater.htm", this);
						if(!player.isGM())
							_log.warning("Player " + player.getName() + "(" + player.getObjectId() + ") was cheating in dimension rift area!");
					}
				}
				else if(command.equalsIgnoreCase("SkillList"))
					showSkillList(player);
				else if(command.equalsIgnoreCase("ClanSkillList"))
					showClanSkillList(player);
				else if(command.equalsIgnoreCase("FishingSkillList"))
					showFishingSkillList(player);
				else if(command.equalsIgnoreCase("SkillEnchant"))
					showEnchantSkillList(player, player.getClassId());
				else if(command.startsWith("Augment"))
				{
					int cmdChoice = Integer.parseInt(command.substring(8, 9).trim());
					if(cmdChoice == 1)
					{
						player.sendPacket(new SystemMessage(SystemMessage.SELECT_THE_ITEM_TO_BE_AUGMENTED));
						player.sendPacket(new ExShowVariationMakeWindow());
					}
					else if(cmdChoice == 2)
					{
						player.sendPacket(new SystemMessage(SystemMessage.SELECT_THE_ITEM_FROM_WHICH_YOU_WISH_TO_REMOVE_AUGMENTATION));
						player.sendPacket(new ExShowVariationCancelWindow());
					}
				}
				else if(command.startsWith("Link"))
					showChatWindow(player, "data/html/" + command.substring(5));
			}
			catch(StringIndexOutOfBoundsException sioobe)
			{
				_log.info("Incorrect htm bypass! npcId=" + getTemplate().npcId + " command=[" + command + "]");
			}
			catch(NumberFormatException nfe)
			{
				_log.info("Invalid bypass to Server command parameter! npcId=" + getTemplate().npcId + " command=[" + command + "]");
			}
	}

	public void showQuestWindow(L2Player player)
	{
		// collect awaiting quests and start points
		ArrayList<Quest> options = new ArrayList<Quest>();

		QuestState[] awaits = player.getQuestsForTalk(getTemplate().npcId);
		Quest[] starts = getTemplate().getEventQuests(Quest.QuestEventType.QUEST_START);

		if(awaits != null)
			for(QuestState x : awaits)
				if(!options.contains(x.getQuest()))
					if(x.getQuest().getQuestIntId() > 0 && x.getQuest().getQuestIntId() < 1000)
						options.add(x.getQuest());

		if(starts != null)
			for(Quest x : starts)
				if(!options.contains(x))
					if(x.getQuestIntId() > 0 && x.getQuestIntId() < 1000)
						options.add(x);

		// Display a QuestChooseWindow (if several quests are available) or QuestWindow
		if(options.size() > 1)
			showQuestChooseWindow(player, options.toArray(new Quest[options.size()]));
		else if(options.size() == 1)
			showQuestWindow(player, options.get(0).getName());
		else
			showQuestWindow(player, "");
	}

	public void showQuestChooseWindow(L2Player player, Quest[] quests)
	{
		StringBuffer sb = new StringBuffer();

		sb.append("<html><body><title>Talk about:</title><br>");

		for(Quest q : quests)
			if(player.getQuestState(q.getName()) == null)
				sb.append("<a action=\"bypass -h npc_").append(getObjectId()).append("_Quest ").append(q.getName()).append("\">[").append(q.getDescr()).append("]</a><br>");
			else if(player.getQuestState(q.getName()).isCompleted())
				sb.append("<a action=\"bypass -h npc_").append(getObjectId()).append("_Quest ").append(q.getName()).append("\">[").append(q.getDescr()).append(" - complited").append("]</a><br>");
			else
				sb.append("<a action=\"bypass -h npc_").append(getObjectId()).append("_Quest ").append(q.getName()).append("\">[").append(q.getDescr()).append(" - in progress").append("]</a><br>");

		sb.append("</body></html>");

		NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
		html.setHtml(sb.toString());
		player.sendPacket(html);
	}

	public void showChatWindow(L2Player player, int val)
	{
		int npcId = getTemplate().npcId;

		/* For use with Seven Signs implementation */
		String filename = SevenSigns.SEVEN_SIGNS_HTML_PATH;
		int sealAvariceOwner = SevenSigns.getInstance().getSealOwner(SevenSigns.SEAL_AVARICE);
		int sealGnosisOwner = SevenSigns.getInstance().getSealOwner(SevenSigns.SEAL_GNOSIS);
		int playerCabal = SevenSigns.getInstance().getPlayerCabal(player);
		boolean isSealValidationPeriod = SevenSigns.getInstance().isSealValidationPeriod();
		int compWinner = SevenSigns.getInstance().getCabalHighestScore();

		switch(npcId)
		{
			case 31095: //
			case 31096: //
			case 31097: //
			case 31098: // Enter Necropolises
			case 31099: //
			case 31100: //
			case 31101: // 
			case 31102: //
				if(isSealValidationPeriod)
				{
					if(playerCabal != compWinner || sealAvariceOwner != compWinner)
						switch(compWinner)
						{
							case SevenSigns.CABAL_DAWN:
								player.sendPacket(new SystemMessage(SystemMessage.CAN_BE_USED_ONLY_BY_THE_LORDS_OF_DAWN));
								filename += "necro_no.htm";
								break;
							case SevenSigns.CABAL_DUSK:
								player.sendPacket(new SystemMessage(SystemMessage.CAN_BE_USED_ONLY_BY_THE_REVOLUTIONARIES_OF_DUSK));
								filename += "necro_no.htm";
								break;
							case SevenSigns.CABAL_NULL:
								filename = getHtmlPath(npcId, val); // do the default! 
								break;
						}
					else
						filename = getHtmlPath(npcId, val); // do the default!
				}
				else if(playerCabal == SevenSigns.CABAL_NULL)
					filename += "necro_no.htm";
				else
					filename = getHtmlPath(npcId, val); // do the default!	
				break;
			case 31114: //
			case 31115: //
			case 31116: // Enter Catacombs
			case 31117: //
			case 31118: //
			case 31119: //
				if(isSealValidationPeriod)
				{
					if(playerCabal != compWinner || sealGnosisOwner != compWinner)
						switch(compWinner)
						{
							case SevenSigns.CABAL_DAWN:
								player.sendPacket(new SystemMessage(SystemMessage.CAN_BE_USED_ONLY_BY_THE_LORDS_OF_DAWN));
								filename += "cata_no.htm";
								break;
							case SevenSigns.CABAL_DUSK:
								player.sendPacket(new SystemMessage(SystemMessage.CAN_BE_USED_ONLY_BY_THE_REVOLUTIONARIES_OF_DUSK));
								filename += "cata_no.htm";
								break;
							case SevenSigns.CABAL_NULL:
								filename = getHtmlPath(npcId, val); // do the default! 
								break;
						}
					else
						filename = getHtmlPath(npcId, val); // do the default! 
				}
				else if(playerCabal == SevenSigns.CABAL_NULL)
					filename += "cata_no.htm";
				else
					filename = getHtmlPath(npcId, val); // do the default!	
				break;
			case 31111: // Gatekeeper Spirit (Disciples)
				if(playerCabal == sealAvariceOwner && playerCabal == compWinner)
					switch(sealAvariceOwner)
					{
						case SevenSigns.CABAL_DAWN:
							filename += "spirit_dawn.htm";
							break;
						case SevenSigns.CABAL_DUSK:
							filename += "spirit_dusk.htm";
							break;
						case SevenSigns.CABAL_NULL:
							filename += "spirit_null.htm";
							break;
					}
				else
					filename += "spirit_null.htm";
				break;
			case 31112: // Gatekeeper Spirit (Disciples)
				filename += "spirit_exit.htm";
				break;
			case 31688:
				if(player.isNoble())
					filename = Olympiad.OLYMPIAD_HTML_FILE + "noble_main.htm";
				else
					filename = getHtmlPath(npcId, val);
				break;
			case 31690:
			case 31769:
			case 31770: // Monument of Heroes
			case 31771:
			case 31772:
				if(player.isHero())
					filename = Olympiad.OLYMPIAD_HTML_FILE + "hero_main.htm";
				else
					filename = getHtmlPath(npcId, val);
				break;
			default:
				if(npcId >= 31093 && npcId <= 31094 || npcId >= 31172 && npcId <= 31201 || npcId >= 31239 && npcId <= 31254)
					return;
				// Get the text of the selected HTML file in function of the npcId and of the page number
				filename = getHtmlPath(npcId, val);
				break;
		}

		player.sendPacket(new NpcHtmlMessage(player, this, filename, val));
	}

	public void showChatWindow(L2Player player, String filename)
	{
		player.sendPacket(new NpcHtmlMessage(player, this, filename, 0));
	}

	public String getHtmlPath(int npcId, int val)
	{
		String pom;
		if(val == 0)
			pom = "" + npcId;
		else
			pom = npcId + "-" + val;
		String temp = "data/html/default/" + pom + ".htm";
		File mainText = new File(temp);
		if(mainText.exists())
			return temp;

		temp = "data/html/trainer/" + pom + ".htm";
		mainText = new File(temp);
		if(mainText.exists())
			return temp;

		// If the file is not found, the standard message "I have nothing to say to you" is returned
		return "data/html/npcdefault.htm";
	}

	// 0 - buy ticket window
	// 1-20 - buttons
	// 22 - lottery ticket window
	// 21 - selected ticket with 5 numbers
	public void showLotoWindow(L2Player player, int val)
	{
		int npcId = getTemplate().npcId;
		String filename;
		SystemMessage sm;
		NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
		//if loto
		if(val == 0)
		{
			filename = getHtmlPath(npcId, 1);
			html.setFile(filename);
		}
		else if(val != 21)
		{
			filename = getHtmlPath(npcId, 5);
			html.setFile(filename);
			int count = 0;
			int found = 0;
			// counting buttons and unsetting button if found
			for(int i = 0; i < 5; i++)
				if(player.getLoto(i) == val)
				{
					//unsetting button
					player.setLoto(i, 0);
					found = 1;
				}
				else if(player.getLoto(i) > 0)
					count++;
			//if not rearched limit 5 and not unseted value
			if(count < 5 && found == 0 && val < 21)
				for(int i = 0; i < 5; i++)
					if(player.getLoto(i) == 0)
					{
						player.setLoto(i, val);
						break;
					}
			//setting pusshed buttons
			count = 0;
			for(int i = 0; i < 5; i++)
				if(player.getLoto(i) > 0)
				{
					count++;
					String button = String.valueOf(player.getLoto(i));
					if(player.getLoto(i) < 10)
						button = "0" + button;
					String search = "fore=\"L2UI.lottoNum" + button + "\" back=\"L2UI.lottoNum" + button + "a_check\"";
					String replace = "fore=\"L2UI.lottoNum" + button + "a_check\" back=\"L2UI.lottoNum" + button + "\"";
					html.replace(search, replace);
				}
			if(count == 5)
			{
				String search = "0\">Return";
				String replace = "21\">The winner selected the numbers above.";
				html.replace(search, replace);
			}
		}
		else
		{
			int price = 2000;
			int lotonumber = 1;
			int enchant = 0;
			int type2 = 0;
			for(int i = 0; i < 5; i++)
			{
				if(player.getLoto(i) == 0)
					return;
				if(player.getLoto(i) < 17)
					enchant += Math.pow(2, player.getLoto(i) - 1);
				else
					type2 += Math.pow(2, player.getLoto(i) - 17);
			}
			if(player.getAdena() < price)
			{
				player.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ADENA);
				return;
			}
			player.reduceAdena(price);
			sm = new SystemMessage(SystemMessage.ACQUIRED__S1_S2);
			sm.addNumber(lotonumber);
			sm.addItemName(4442);
			player.sendPacket(sm);
			L2ItemInstance item = new L2ItemInstance(IdFactory.getInstance().getNextId(), (short) 4442, player.getObjectId(), npcId, "showLotoWindow");
			item.setCount(1);
			item.setCustomType1(lotonumber);
			item.setEnchantLevel(enchant);
			item.setCustomType2(type2);
			player.getInventory().addItem(item);
			Log.LogItem(player, Log.BuyItem, item);
			InventoryUpdate iu = new InventoryUpdate();
			iu.addItem(item);
			L2ItemInstance adenaupdate = player.getInventory().findItemByItemId(57);
			iu.addModifiedItem(adenaupdate);
			player.sendPacket(iu);
			html.setHtml("<html><body>Lottery Ticket Seller:<br>Thank you for playing the lottery<br>The winners will be announced at ??? 19<br><center><a action=\"bypass -h npc_%objectId%_Chat 0\">Back</a></center></body></html>");
		}
		player.sendPacket(html);
		player.sendActionFailed();
	}

	public void makeCPRecovery(L2Player player)
	{
		if(getNpcId() != 31225 && getNpcId() != 31226)
			return;
		int neededmoney = 100;
		int currentmoney = player.getAdena();
		if(neededmoney > currentmoney)
		{
			player.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ADENA);
			return;
		}
		player.reduceAdena(neededmoney);
		player.setCurrentCp(getCurrentCp() + 5000);
		player.sendPacket(new SystemMessage(SystemMessage.S1_CPS_WILL_BE_RESTORED).addString(player.getName()));
	}

	public void makeSupportMagic(L2Player player)
	{
		// Prevent a cursed weapon weilder of being buffed
		if(player.isCursedWeaponEquipped())
			return;
		int lvl = player.getLevel();
		L2Skill skill;
		if(lvl < 10)
		{
			String content = "<html><body>Come back here when you have reached level 10. I will give you support magic then.</body></html>";
			NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
			html.setHtml(content);
			player.sendPacket(html);
			return;
		}
		if(lvl > 24)
		{
			String content = "<html><body>Newbie Guide:<br>Only a <font color=\"LEVEL\">novice character of level 24 or less</font> can receive my support magic.<br>Your novice character is the first one that you created and raised in this world.</body></html>";
			NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
			html.setHtml(content);
			player.sendPacket(html);
			return;
		}
		int time = 0;
		//windwalk
		if(lvl >= 10 && lvl <= 24)
		{
			skill = SkillTable.getInstance().getInfo(4322, 1);
			time += 1000;
			ThreadPoolManager.getInstance().scheduleGeneral(new doBuff(this, skill, player), time);
		}
		//shield
		if(lvl >= 11 && lvl <= 23)
		{
			skill = SkillTable.getInstance().getInfo(4323, 1);
			time += 1000;
			ThreadPoolManager.getInstance().scheduleGeneral(new doBuff(this, skill, player), time);
		}
		//lifecubic
		if(lvl >= 16 && lvl <= 19)
		{
			skill = SkillTable.getInstance().getInfo(4338, 1);
			time += 1000;
			ThreadPoolManager.getInstance().scheduleGeneral(new doBuff(this, skill, player), time);
		}
		if(!player.isMageClass())
		{
			//blessofbody
			if(lvl >= 12 && lvl <= 22)
			{
				skill = SkillTable.getInstance().getInfo(4324, 1);
				time += 1000;
				ThreadPoolManager.getInstance().scheduleGeneral(new doBuff(this, skill, player), time);
			}
			//vampirerage
			if(lvl >= 13 && lvl <= 21)
			{
				skill = SkillTable.getInstance().getInfo(4325, 1);
				time += 1000;
				ThreadPoolManager.getInstance().scheduleGeneral(new doBuff(this, skill, player), time);
			}
			//regeneration
			if(lvl >= 14 && lvl <= 20)
			{
				skill = SkillTable.getInstance().getInfo(4326, 1);
				time += 1000;
				ThreadPoolManager.getInstance().scheduleGeneral(new doBuff(this, skill, player), time);
			}
			//haste
			if(lvl >= 15 && lvl <= 19)
			{
				skill = SkillTable.getInstance().getInfo(4327, 1);
				time += 1000;
				ThreadPoolManager.getInstance().scheduleGeneral(new doBuff(this, skill, player), time);
			}
		}
		else
		{
			//blessthesoul
			if(lvl >= 12 && lvl <= 22)
			{
				skill = SkillTable.getInstance().getInfo(4328, 1);
				time += 1000;
				ThreadPoolManager.getInstance().scheduleGeneral(new doBuff(this, skill, player), time);
			}
			//acumen
			if(lvl >= 13 && lvl <= 21)
			{
				skill = SkillTable.getInstance().getInfo(4329, 1);
				time += 1000;
				ThreadPoolManager.getInstance().scheduleGeneral(new doBuff(this, skill, player), time);
			}
			//concentration
			if(lvl >= 14 && lvl <= 20)
			{
				skill = SkillTable.getInstance().getInfo(4330, 1);
				time += 1000;
				ThreadPoolManager.getInstance().scheduleGeneral(new doBuff(this, skill, player), time);
			}
			//empower
			if(lvl >= 15 && lvl <= 19)
			{
				skill = SkillTable.getInstance().getInfo(4331, 1);
				time += 1000;
				ThreadPoolManager.getInstance().scheduleGeneral(new doBuff(this, skill, player), time);
			}
		}
	}

	private class doBuff implements Runnable
	{
		L2Character _actor;
		L2Skill _skill;
		L2Player _target;

		doBuff(L2Character actor, L2Skill skill, L2Player target)
		{
			_actor = actor;
			_skill = skill;
			_target = target;
		}

		public void run()
		{
			_actor.doCast(_skill, _target, true);
		}
	}

	private boolean _isBusy;
	private String _busyMessage = "";

	/**
	 * Return the busy status of this L2NpcInstance.<BR><BR>
	 */
	public final boolean isBusy()
	{
		return _isBusy;
	}

	/**
	 * Set the busy status of this L2NpcInstance.<BR><BR>
	 */
	public void setBusy(boolean isBusy)
	{
		_isBusy = isBusy;
	}

	/**
	 * Return the busy message of this L2NpcInstance.<BR><BR>
	 */
	public final String getBusyMessage()
	{
		return _busyMessage;
	}

	/**
	 * Set the busy message of this L2NpcInstance.<BR><BR>
	 */
	public void setBusyMessage(String message)
	{
		_busyMessage = message;
	}

	public void showBusyWindow(L2Player player)
	{
		NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
		html.setFile("data/html/npcbusy.htm");
		html.replace("%npcname%", getName());
		html.replace("%playername%", player.getName());
		html.replace("%busymessage%", _busyMessage);
		player.sendPacket(html);
	}

	/**
	 * this displays SkillList to the player.
	 * @param player
	 */
	public void showSkillList(L2Player player)
	{
		if(Config.DEBUG)
			_log.fine("SkillList activated on: " + getObjectId());

		ClassId classId = player.getClassId();

		if(classId == null)
			return;

		int npcId = getTemplate().npcId;

		if(_classesToTeach == null)
		{
			NpcHtmlMessage html = new NpcHtmlMessage(getObjectId());
			TextBuilder sb = new TextBuilder();
			sb.append("<html><head><body>");
			sb.append("I cannot teach you. My class list is empty.<br> Ask admin to fix it. Need add my npcid and classes to skill_learn.sql.<br>NpcId:" + npcId + ", Your classId:" + player.getClassId().getId() + "<br>");
			sb.append("</body></html>");
			html.setHtml(sb.toString());
			player.sendPacket(html);

			return;
		}

		if(!(getTemplate().canTeach(classId) || getTemplate().canTeach(classId.getParent())))
		{
			NpcHtmlMessage html = new NpcHtmlMessage(getObjectId());
			TextBuilder sb = new TextBuilder();
			sb.append("<html><head><body>");
			sb.append(new CustomMessage("L2jFT.game.model.instances.L2NpcInstance.WrongTeacherClass", player));
			sb.append("</body></html>");
			html.setHtml(sb.toString());
			player.sendPacket(html);

			return;
		}

		AquireSkillList asl = new AquireSkillList(AquireSkillList.USUAL);
		int counts = 0;

		L2SkillLearn[] skills = SkillTreeTable.getInstance().getAvailableSkills(player, classId);
		for(L2SkillLearn s : skills)
		{
			L2Skill sk = SkillTable.getInstance().getInfo(s.getId(), s.getLevel());
			if(sk == null || !sk.getCanLearn(player.getClassId()) || !sk.canTeachBy(npcId))
				continue;
			int cost = SkillTreeTable.getInstance().getSkillCost(player, sk);
			counts++;
			asl.addSkill(s.getId(), s.getLevel(), s.getLevel(), cost, 0);
		}

		if(classId.getParent() != null)
		{
			L2SkillLearn[] skills2 = SkillTreeTable.getInstance().getAvailableSkills(player, classId.getParent());
			for(L2SkillLearn s : skills2)
			{
				L2Skill sk = SkillTable.getInstance().getInfo(s.getId(), s.getLevel());
				if(sk == null || !sk.getCanLearn(player.getClassId()) || !sk.canTeachBy(npcId))
					continue;
				int cost = SkillTreeTable.getInstance().getSkillCost(player, sk);
				counts++;
				asl.addSkill(s.getId(), s.getLevel(), s.getLevel(), cost, 0);
			}

			if(classId.getParent().getParent() != null)
			{
				L2SkillLearn[] skills3 = SkillTreeTable.getInstance().getAvailableSkills(player, classId.getParent());
				for(L2SkillLearn s : skills3)
				{
					L2Skill sk = SkillTable.getInstance().getInfo(s.getId(), s.getLevel());
					if(sk == null || !sk.getCanLearn(player.getClassId()) || !sk.canTeachBy(npcId))
						continue;
					int cost = SkillTreeTable.getInstance().getSkillCost(player, sk);
					counts++;
					asl.addSkill(s.getId(), s.getLevel(), s.getLevel(), cost, 0);
				}

				if(classId.getParent().getParent().getParent() != null)
				{
					L2SkillLearn[] skills4 = SkillTreeTable.getInstance().getAvailableSkills(player, classId.getParent());
					for(L2SkillLearn s : skills4)
					{
						L2Skill sk = SkillTable.getInstance().getInfo(s.getId(), s.getLevel());
						if(sk == null || !sk.getCanLearn(player.getClassId()) || !sk.canTeachBy(npcId))
							continue;
						int cost = SkillTreeTable.getInstance().getSkillCost(player, sk);
						counts++;
						asl.addSkill(s.getId(), s.getLevel(), s.getLevel(), cost, 0);
					}
				}
			}
		}

		if(counts == 0)
		{
			NpcHtmlMessage html = new NpcHtmlMessage(getObjectId());
			int minlevel = SkillTreeTable.getInstance().getMinLevelForNewSkill(player, classId);

			if(minlevel > 0)
			{
				SystemMessage sm = new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_ANY_FURTHER_SKILLS_TO_LEARN__COME_BACK_WHEN_YOU_HAVE_REACHED_LEVEL_S1);
				sm.addNumber(minlevel);
				player.sendPacket(sm);
			}
			else
			{
				TextBuilder sb = new TextBuilder();
				sb.append("<html><head><body>");
				sb.append("You've learned all skills for your class.");
				sb.append("</body></html>");
				html.setHtml(sb.toString());
				player.sendPacket(html);
			}
		}
		else
			player.sendPacket(asl);

		player.sendActionFailed();
	}

	/**
	 * this displays EnchantSkillList to the player.
	 * @param player
	 */
	public void showEnchantSkillList(L2Player player, ClassId classId)
	{
		if(Config.DEBUG)
			_log.fine("EnchantSkillList activated on: " + getObjectId());
		int npcId = getTemplate().npcId;

		if(_classesToTeach == null)
		{
			NpcHtmlMessage html = new NpcHtmlMessage(getObjectId());
			TextBuilder sb = new TextBuilder();
			sb.append("<html><head><body>");
			sb.append("I cannot teach you. My class list is empty.<br> Ask admin to fix it. Need add my npcid and classes to skill_learn.sql.<br>NpcId:" + npcId + ", Your classId:" + player.getClassId().getId() + "<br>");
			sb.append("</body></html>");
			html.setHtml(sb.toString());
			player.sendPacket(html);

			return;
		}

		if(!(getTemplate().canTeach(classId) || getTemplate().canTeach(classId.getParent())))
		{
			NpcHtmlMessage html = new NpcHtmlMessage(getObjectId());
			TextBuilder sb = new TextBuilder();
			sb.append("<html><head><body>");
			sb.append(new CustomMessage("L2jFT.game.model.instances.L2NpcInstance.WrongTeacherClass", player));
			sb.append("</body></html>");
			html.setHtml(sb.toString());
			player.sendPacket(html);

			return;
		}
		if(player.getClassId().getId() < 88)
		{
			NpcHtmlMessage html = new NpcHtmlMessage(getObjectId());
			TextBuilder sb = new TextBuilder();
			sb.append("<html><head><body>");
			sb.append("You must have 3rd class change quest completed.");
			sb.append("</body></html>");
			html.setHtml(sb.toString());
			player.sendPacket(html);

			return;
		}

		ArrayList<L2EnchantSkillLearn> skills = SkillTreeTable.getInstance().getAvailableEnchants(player);
		ExEnchantSkillList esl = new ExEnchantSkillList();
		int counts = 0;

		for(L2EnchantSkillLearn s : skills)
		{
			L2Skill sk = SkillTable.getInstance().getInfo(s.getId(), s.getLevel() + s.getBaseLevel() - 100);
			if(sk == null)
				continue;
			counts++;
			esl.addSkill(s.getId(), s.getLevel(), s.getSpCost(), s.getExpCost());
		}
		if(counts == 0)
		{
			player.sendPacket(new SystemMessage(SystemMessage.THERE_IS_NO_SKILL_THAT_ENABLES_ENCHANT));
			NpcHtmlMessage html = new NpcHtmlMessage(getObjectId());
			int level = player.getLevel();

			if(level < 74)
			{
				SystemMessage sm = new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_ANY_FURTHER_SKILLS_TO_LEARN__COME_BACK_WHEN_YOU_HAVE_REACHED_LEVEL_S1);
				sm.addNumber(level);
				player.sendPacket(sm);
			}
			else
			{
				TextBuilder sb = new TextBuilder();
				sb.append("<html><head><body>");
				sb.append("You've learned all skills for your class.<br>");
				sb.append("</body></html>");
				html.setHtml(sb.toString());
				player.sendPacket(html);
			}
		}
		else
			player.sendPacket(esl);

		player.sendActionFailed();
	}

	public void showFishingSkillList(L2Player player)
	{
		if(Config.DEBUG)
			_log.fine("SkillList activated on: " + getObjectId());

		AquireSkillList asl = new AquireSkillList(AquireSkillList.FISHING);
		int counts = 0;

		L2SkillLearn[] skills = SkillTreeTable.getInstance().getAvailableFishingSkills(player);
		for(L2SkillLearn s : skills)
		{
			L2Skill sk = SkillTable.getInstance().getInfo(s.getId(), s.getLevel());
			if(sk == null)
				continue;
			int cost = SkillTreeTable.getInstance().getSkillCost(player, sk);
			counts++;
			asl.addSkill(s.getId(), s.getLevel(), s.getLevel(), cost, 0);
		}

		if(counts == 0)
		{
			NpcHtmlMessage html = new NpcHtmlMessage(getObjectId());
			TextBuilder sb = new TextBuilder();
			sb.append("<html><head><body>");
			sb.append("You've learned all skills.");
			sb.append("</body></html>");
			html.setHtml(sb.toString());
			player.sendPacket(html);
		}
		else
			player.sendPacket(asl);

		player.sendActionFailed();
	}

	public void showClanSkillList(L2Player player)
	{
		if(Config.DEBUG)
			_log.fine("SkillList activated on: " + getObjectId());

		if(player.getClan() == null || !player.isClanLeader())
		{
			player.sendPacket(new SystemMessage(SystemMessage.ONLY_THE_CLAN_LEADER_IS_ENABLED));
			player.sendActionFailed();
			return;
		}

		AquireSkillList asl = new AquireSkillList(AquireSkillList.CLAN);
		int counts = 0;

		L2SkillLearn[] skills = SkillTreeTable.getInstance().getAvailableClanSkills(player.getClan());
		for(L2SkillLearn s : skills)
		{
			L2Skill sk = SkillTable.getInstance().getInfo(s.getId(), s.getLevel());
			if(sk == null)
				continue;
			int cost = SkillTreeTable.getInstance().getSkillCost(player, sk);
			counts++;
			asl.addSkill(s.getId(), s.getLevel(), s.getLevel(), cost, 0);
		}

		if(counts == 0)
		{
			NpcHtmlMessage html = new NpcHtmlMessage(getObjectId());
			html.setHtml("<html><head><body>You've learned all skills.</body></html>");
			player.sendPacket(html);
		}
		else
			player.sendPacket(asl);

		player.sendActionFailed();
	}

	/** Нужно для пакета NpcInfo **/
	private boolean _spawned;

	/**
	 * Возвращает режим NPC: свежезаспавненный или нормальное состояние 
	 * @return true, если NPC свежезаспавненный
	 */
	public boolean isSpawned()
	{
		return _spawned;
	}

	/**
	 * Необходимо для показа анимации суммона для тех моделей NPC, которые это умеют
	 * @param spawned Анимация появления
	 */
	public void setSpawned(final boolean spawned)
	{
		_spawned = spawned;
	}
}