package L2jFT.game.model.instances;

import L2jFT.Config;
import L2jFT.database.DatabaseUtils;
import L2jFT.database.FiltredPreparedStatement;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.database.ThreadConnection;
import L2jFT.game.ThreadPoolManager;
import L2jFT.game.ai.CtrlEvent;
import L2jFT.game.ai.CtrlIntention;
import L2jFT.game.cache.Msg;
import L2jFT.game.idfactory.IdFactory;
import L2jFT.game.instancemanager.CursedWeaponsManager;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2PetData;
import L2jFT.game.model.L2Playable;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.L2Summon;
import L2jFT.game.model.L2World;
import L2jFT.game.model.PetInventory;
import L2jFT.game.model.base.Experience;
import L2jFT.game.network.serverpackets.InventoryUpdate;
import L2jFT.game.network.serverpackets.PetDelete;
import L2jFT.game.network.serverpackets.PetInventoryUpdate;
import L2jFT.game.network.serverpackets.PetItemList;
import L2jFT.game.network.serverpackets.PetStatusUpdate;
import L2jFT.game.network.serverpackets.SocialAction;
import L2jFT.game.network.serverpackets.StatusUpdate;
import L2jFT.game.network.serverpackets.StopMove;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.skills.Stats;
import L2jFT.game.tables.PetDataTable;
import L2jFT.game.taskmanager.DecayTaskManager;
import L2jFT.game.templates.L2NpcTemplate;
import L2jFT.game.templates.L2Weapon;

import java.sql.ResultSet;
import java.util.concurrent.Future;
import java.util.logging.Level;
import java.util.logging.Logger;

public class L2PetInstance extends L2Summon
{
	protected static Logger _log = Logger.getLogger(L2PetInstance.class.getName());

	private byte _level;
	@SuppressWarnings("unused")
	private int _curFed;
	protected PetInventory _inventory;
	private final int _controlItemId;
	private boolean _respawned;
	private Future _feedTask;
	private boolean _mountable;

	protected boolean _feedMode;

	protected L2PetData _data;

	private int lostExp;

	private long getExpForNextLevel;
	private long getExpForThisLevel;

	class FeedTask implements Runnable
	{

		public void run()
		{
			try
			{
				// if pet is attacking
				if(isAttackingNow())
					// if its not already on battleFeed mode
					if(!_feedMode)
						startFeed(true); //switching to battle feed
					else
						startFeed(false); // normal feed

				if(getCurrentFed() > 0)
					setCurrentFed(getCurrentFed() - 1);
				else
				{
					// go back to pet control item, or simply said, unsummon it lol
					setCurrentFed(0);
					stopFeed();
					if(getOwner() != null)
						getOwner().sendMessage("Your pet is too hungry to stay summoned.", "Ваш питомец слишком голоден, что бы оставаться вызванным.");
					unSummon();
				}
				L2ItemInstance food;
				switch(getTemplate().npcId)
				{
					// Wolf & Sin Eater
					case (short) 12077:
					case (short) 12564:
						food = getInventory().findItemByItemId(2515);
						break;
					// Hatchling of Wind, Star, Twilight
					case (short) 12311:
					case (short) 12312:
					case (short) 12313:
						food = getInventory().findItemByItemId(4038);
						break;
					// Wind, Star, Twilight Strider
					case (short) 12526:
					case (short) 12527:
					case (short) 12528:
						food = getInventory().findItemByItemId(5168);
						break;
					// Baby Buffalo, Kookaburra, Cougar
					case (short) 12780:
					case (short) 12781:
					case (short) 12782:
						food = getInventory().findItemByItemId(7582);
						break;
					// unknown npc id.. should never happen
					default:
						return;
				}
				if(food != null && getCurrentFed() < 0.55 * getMaxFed())
				{
					setCurrentFed(getCurrentFed() + (int) (getMaxFed() * 0.12));
					removeItemFromInventory(food, 1, false);
					if(getCurrentFed() > getMaxFed())
						setCurrentFed(getMaxFed());
				}
				broadcastStatusUpdate();
			}
			catch(Throwable e)
			{
				_log.log(Level.SEVERE, "", e);
			}
		}
	}

	public static L2PetInstance spawnPet(L2NpcTemplate template, L2Player owner, L2ItemInstance control)
	{
		return restore(control, template, owner);
	}

	public L2PetInstance(int objectId, L2NpcTemplate template, L2Player owner, L2ItemInstance control, byte CurLevelPet)
	{
		super(objectId, template, owner);
		_controlItemId = control.getObjectId();
		_inventory = new PetInventory(this);
		if(template.npcId == 12564)
		{
			_level = (byte) owner.getInventory().getItemByObjectId(_controlItemId).getEnchantLevel();
			if(_level <= 0)
				_level = owner.getLevel();
		}
		else
			// тут добываем текущий уровень пета, вероятно есть способ лучше
			_level = CurLevelPet == 0 ? template.level : CurLevelPet;
		_data = PetDataTable.getInstance().getInfo(template.npcId, _level);
		_template = template;
		int npcId = template.npcId;
		_mountable = npcId == 12526 || //wind strider
		npcId == 12527 || //star strider
		npcId == 12528 || //twilight strider
		npcId == 12621; //wyvern
		startFeed(false);

		isPet = true;
	}

	public L2PetInstance(int objectId, L2NpcTemplate template, L2Player owner, L2ItemInstance control)
	{
		super(objectId, template, owner);
		_controlItemId = control.getObjectId();
		_inventory = new PetInventory(this);

		if(template.npcId == 12564)
		{
			_level = (byte) owner.getInventory().getItemByObjectId(_controlItemId).getEnchantLevel();
			if(_level <= 0)
				_level = owner.getLevel();
		}
		else
			_level = template.level;

		_data = PetDataTable.getInstance().getInfo(template.npcId, _level);
		_template = template;
		int npcId = template.npcId;
		_mountable = npcId == 12526 || //wind strider
		npcId == 12527 || //star strider
		npcId == 12528 || //twilight strider
		npcId == 12621; //wyvern
		startFeed(false);

		isPet = true;
	}

	@Override
	public L2NpcTemplate getTemplate()
	{
		return (L2NpcTemplate) _template;
	}

	@Override
	public final byte getLevel()
	{
		return _level;
	}

	public boolean isRespawned()
	{
		return _respawned;
	}

	@Override
	public int getSummonType()
	{
		return 2;
	}

	/**
	 * @return Returns ObjectId for pet control item
	 */
	@Override
	public int getControlItemId()
	{
		return _controlItemId;
	}

	@Override
	public int getMaxHp()
	{
		return (int) calcStat(Stats.MAX_HP, _data.getHP(), null, null);
	}

	@Override
	public int getMaxMp()
	{
		return (int) calcStat(Stats.MAX_MP, _data.getMP(), null, null);
	}

	@Override
	public int getMaxFed()
	{
		return _data.getFeedMax();
	}

	@Override
	public int getMAtk(L2Character target, L2Skill skill)
	{
		return (int) calcStat(Stats.MAGIC_ATTACK, _data.getMAtk(), target, skill);
	}

	@Override
	public int getMDef(L2Character target, L2Skill skill)
	{
		return (int) calcStat(Stats.MAGIC_DEFENCE, _data.getMDef(), target, skill);
	}

	@Override
	public int getPAtk(L2Character target)
	{
		return (int) calcStat(Stats.POWER_ATTACK, _data.getPAtk(), target, null);
	}

	@Override
	public int getPDef(L2Character target)
	{
		return (int) calcStat(Stats.POWER_DEFENCE, _data.getPDef(), target, null);
	}

	@Override
	public int getAccuracy()
	{
		return (int) calcStat(Stats.ACCURACY_COMBAT, _data.getAccuracy(), null, null);
	}

	@Override
	public int getEvasionRate(L2Character target)
	{
		return (int) calcStat(Stats.EVASION_RATE, _data.getEvasion(), target, null);
	}

	@Override
	public int getCriticalHit(L2Character target, L2Skill skill)
	{
		return (int) calcStat(Stats.CRITICAL_RATE_STATIC, _data.getCritical(), target, skill);
	}

	@Override
	public int getRunSpeed()
	{
		return getSpeed(_data.getSpeed());
	}

	@Override
	public int getPAtkSpd()
	{
		return (int) calcStat(Stats.POWER_ATTACK_SPEED, _data.getAtkSpeed(), null, null);
	}

	@Override
	public int getMAtkSpd()
	{
		return (int) calcStat(Stats.MAGIC_ATTACK_SPEED, _data.getCastSpeed(), null, null);
	}

	@Override
	public int getMaxLoad()
	{
		return (int) calcStat(Stats.MAX_LOAD, _data.getMaxLoad(), null, null);
	}

	@Override
	public int getCurrentFed()
	{
		return _curFed;
	}

	public void setCurrentFed(int num)
	{
		_curFed = num;
	}

	@Override
	public void setSp(int sp)
	{
		_sp = sp;
	}

	public void setLevel(byte level)
	{
		_level = level;
	}

	public void setRespawned(boolean respawned)
	{
		_respawned = respawned;
	}

	@Override
	public void addExpAndSp(long addToExp, long addToSp)
	{
		if(Config.DEBUG)
			_log.fine("adding " + addToExp + " exp and " + addToSp + " sp to " + getName());
		_exp += addToExp;
		_sp += addToSp;

		if(_owner == null)
			return;

		_owner.sendPacket(new SystemMessage(SystemMessage.THE_PET_ACQUIRED_EXPERIENCE_POINTS_OF_S1).addNumber((int) addToExp));

		while(_exp >= getExpForNextLevel() && _level < Experience.MAX_LEVEL)
			increaseLevel();

		broadcastPetInfo();
	}

	@Override
	public long getExpForThisLevel()
	{
		getExpForThisLevel = PetDataTable.getInstance().getInfo(getNpcId(), _level).getExp();
		return getExpForThisLevel;
	}

	@Override
	public long getExpForNextLevel()
	{
		getExpForNextLevel = PetDataTable.getInstance().getInfo(getNpcId(), (byte) (_level + 1)).getExp();
		return getExpForNextLevel;
	}
	
	@SuppressWarnings("deprecation")
	public void increaseLevel()
	{
		if(_owner == null)
			return;
		stopFeed();
		// Increase the level
		if(_level < Experience.MAX_LEVEL)
			_level++;

		if(Config.DEBUG)
			_log.finest("increasing level for pet " + getName());

		if(getTemplate().npcId == 12564)
			_owner.getInventory().getItemByObjectId(_controlItemId).setEnchantLevel(_level);
		_data = PetDataTable.getInstance().getInfo(getTemplate().npcId, _level);

		// Send a Server->Client System Message
		_owner.sendMessage("Your pet has increased it's level to " + _level, "Ваш питомец увеличил свой уровень до " + _level + ".");
		broadcastPacket(new SocialAction(getObjectId(), 15));

		_owner.sendPacket(new PetStatusUpdate(this));
		StatusUpdate su = new StatusUpdate(_objectId);
		su.addAttribute(StatusUpdate.LEVEL, _level);
		su.addAttribute(StatusUpdate.MAX_HP, getMaxHp());
		su.addAttribute(StatusUpdate.MAX_MP, getMaxMp());
		broadcastPacket(su);
		setCurrentHpMp(getMaxHp(), getMaxMp());
	}

	/**
	 * Return null.<BR><BR>
	 */
	@Override
	public L2ItemInstance getActiveWeaponInstance()
	{
		// temporary? unavailable
		return null;
	}

	@Override
	public L2Weapon getActiveWeaponItem()
	{
		// temporary? unavailable
		return null;
	}

	@Override
	public L2ItemInstance getSecondaryWeaponInstance()
	{
		// temporary? unavailable
		return null;
	}

	@Override
	public L2Weapon getSecondaryWeaponItem()
	{
		// temporary? unavailable
		return null;
	}

	@Override
	public PetInventory getInventory()
	{
		return _inventory;
	}

	public void removeItemFromInventory(L2ItemInstance item, int count, boolean toLog)
	{
		synchronized (_inventory)
		{
			_inventory.destroyItem(item.getObjectId(), count, false, toLog);
		}
		if(item.getCount() > 0)
			_owner.sendPacket(new PetInventoryUpdate().addModifiedItem(item));
		else
			_owner.sendPacket(new PetInventoryUpdate().addRemovedItem(item));
	}

	@Override
	public void doPickupItem(L2Object object)
	{
		getAI().setIntention(CtrlIntention.AI_INTENTION_IDLE, null, null);

		broadcastPacket(new StopMove(this));

		if(!(object instanceof L2ItemInstance))
		{
			_owner.sendActionFailed();
			return;
		}

		L2ItemInstance target = (L2ItemInstance) object;

		if(CursedWeaponsManager.getInstance().isCursed(target.getItemId()))
		{
			SystemMessage sm = new SystemMessage(SystemMessage.FAILED_TO_PICK_UP_S1);
			sm.addItemName(target.getItemId());
			_owner.sendPacket(sm);
			return;
		}

		synchronized (target)
		{
			if(!target.isVisible())
			{
				_owner.sendActionFailed();
				return;
			}

			if(target.getItemId() >= 8600 && target.getItemId() <= 8614)
			{
				altUseSkill(target.getItem().getSkill(), this);
				target.decayMe();
				L2World.removeObject(target);
				return;
			}

			target.pickupMe(this);
		}

		synchronized (_inventory)
		{
			_inventory.addItem(target);
		}

		//FIXME Just send the updates if possible (old way wasn't working though)
		_owner.sendPacket(new PetItemList(this));
		broadcastPetInfo();

		if(target.getItemId() == 57)
			_owner.sendPacket(new SystemMessage(SystemMessage.S1_HAS_OBTAINED_S2_ADENA).addString("You pet").addNumber(target.getCount()));
		else
			_owner.sendPacket(new SystemMessage(SystemMessage.S1_HAS_OBTAINED_S3_S2).addString("You pet").addItemName(target.getItemId()).addNumber(target.getCount()));

		if(getInventory().getTotalWeight() > getMaxLoad())
			_owner.sendPacket(new SystemMessage(SystemMessage.EXCEEDED_PET_INVENTORYS_WEIGHT_LIMIT));

		getAI().setIntention(CtrlIntention.AI_INTENTION_IDLE, null, null);
		if(getFollowStatus())
			followOwner();
	}

	@Override
	public void deleteMe()
	{
		destroyControlItem(); //this should also delete the pet from the db
		super.deleteMe();
	}

	@Override
	public void reduceCurrentHp(double damage, L2Character attacker)
	{
		if(attacker instanceof L2Playable && isInZoneBattle() != attacker.isInZoneBattle())
		{
			attacker.getPlayer().sendPacket(Msg.INCORRECT_TARGET);
			return;
		}

		super.reduceCurrentHp(damage, attacker);
		// this is usually only called in combat

		if(!isDead() && attacker != null)
		{
			SystemMessage sm = new SystemMessage(SystemMessage.THE_PET_RECEIVED_DAMAGE_OF_S2_CAUSED_BY_S1);
			if(attacker instanceof L2NpcInstance)
				sm.addNpcName(((L2NpcInstance) attacker).getTemplate().npcId);
			else
				sm.addString(attacker.getName());
			sm.addNumber((int) damage);

			if(_owner != null)
				_owner.sendPacket(sm);
			getAI().notifyEvent(CtrlEvent.EVT_ATTACKED, attacker, false);
		}
	}

	public void displayHitMessage(int damage, boolean crit, boolean miss)
	{
		//TODO Так быть не должно
		if(_owner == null)
			return;

		if(crit)
			_owner.sendPacket(Msg.PETS_CRITICAL_HIT);

		if(miss)
			_owner.sendPacket(Msg.MISSED_TARGET);
		else
			_owner.sendPacket(new SystemMessage(SystemMessage.THE_PET_GAVE_DAMAGE_OF_S1).addNumber(damage));
	}

	@Override
	public synchronized void doDie(L2Character killer)
	{
		super.doDie(killer);
		
		if(getOwner() == null)
		{
			onDecay();
			return;
		}
		
		stopFeed();
		deathPenalty();
		
		_owner.sendPacket(new SystemMessage(SystemMessage.PET_HAS_DIED_IF_HE_IS_NOT_RESURRECTED_IN_20_MINUTES_THE_PETS_CORPSE_WILL_DISAPPEAR_AND_HIS_ITEMS_WILL_DISAPPEAR_AS_WELL));
		DecayTaskManager.getInstance().addDecayTask(this, 1200000);
	}

	@Override
	public void stopDecay()
	{
		DecayTaskManager.getInstance().cancelDecayTask(this);
	}

	@Override
	public void giveAllToOwner()
	{
		try
		{
			synchronized (_inventory)
			{
				for(L2ItemInstance giveit : _inventory.getItems())
					if(_owner.getInventory().validateWeight(giveit) && _owner.getInventory().validateCapacity(giveit))
						giveItemToOwner(giveit);
					else
						dropItemHere(giveit);
			}
		}
		catch(Exception e)
		{
			_log.warning("Give all items error " + e);
		}
	}

	public void giveItemToOwner(L2ItemInstance item)
	{
		try
		{
			InventoryUpdate iu = new InventoryUpdate();

			synchronized (_inventory)
			{
				_inventory.dropItem(item, item.getCount());
				iu.addItem(_owner.getInventory().addItem(item));
			}

			PetInventoryUpdate petiu = new PetInventoryUpdate();
			petiu.addRemovedItem(item);

			_owner.sendPacket(petiu);
			_owner.sendPacket(iu);
			_owner.sendChanges();
		}
		catch(Exception e)
		{
			_log.warning("Error while giving item to owner: " + e);
		}
	}

	@Override
	public void broadcastStatusUpdate()
	{
		super.broadcastStatusUpdate();
		if(_owner != null)
			_owner.sendPacket(new PetStatusUpdate(this));
	}

	/**
	 * Remove the Pet from DB and its associated item from the player inventory
	 * @param owner The owner from whose inventory we should delete the item
	 */
	public void destroyControlItem()
	{
		if(_owner == null || getControlItemId() == 0)
			return;
		// pet control item no longer exists, delete the pet from the db
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("DELETE FROM pets WHERE item_obj_id=?");
			statement.setInt(1, getControlItemId());
			statement.execute();
		}
		catch(Exception e)
		{
			_log.warning("could not delete pet:" + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}

		try
		{
			L2ItemInstance removedItem = _owner.getInventory().destroyItem(getControlItemId(), 1, true, true);
			_owner.sendChanges();
			L2World.removeObject(removedItem);
		}
		catch(Exception e)
		{
			_log.warning("Error while destroying control item: " + e);
		}
	}

	public void dropAllItems()
	{
		try
		{
			synchronized (_inventory)
			{
				for(L2ItemInstance item : _inventory.getItems())
					dropItemHere(item);
			}
		}
		catch(Exception e)
		{
			_log.warning("Pet Drop Error: " + e);
		}
	}

	public void dropItemHere(L2ItemInstance dropit)
	{
		synchronized (_inventory)
		{
			dropit = _inventory.dropItem(dropit.getObjectId(), dropit.getCount());
		}

		if(dropit != null)
		{
			_log.finer("Item id to drop: " + dropit.getItemId() + " amount: " + dropit.getCount());
			dropit.dropMe(this, getX(), getY(), getZ() + 25);
		}

		getOwner().sendPacket(new PetItemList(this));
		broadcastPetInfo();
	}

	/**
	 * @return Returns the mountable.
	 */
	@Override
	public boolean isMountable()
	{
		return _mountable;
	}

	private static L2PetInstance restore(L2ItemInstance control, L2NpcTemplate template, L2Player owner)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT objId, name, level, curHp, curMp, exp, sp, fed FROM pets WHERE item_obj_id=?");
			statement.setInt(1, control.getObjectId());
			rset = statement.executeQuery();
			if(!rset.next())
				return new L2PetInstance(IdFactory.getInstance().getNextId(), template, owner, control);
			byte CurLevelPet = rset.getByte("level");
			L2PetInstance pet = new L2PetInstance(rset.getInt("objId"), template, owner, control, CurLevelPet);
			pet.setRespawned(true);
			pet.setName(rset.getString("name"));
			pet.setLevel(rset.getByte("level"));
			pet.setCurrentHpMp(rset.getDouble("curHp"), rset.getInt("curMp"));
			pet.setCurrentCp(pet.getMaxCp());
			pet.setExp(rset.getInt("exp"));
			pet.setSp(rset.getInt("sp"));
			pet.setCurrentFed(rset.getInt("fed"));

			pet._data = PetDataTable.getInstance().getInfo(pet.getTemplate().npcId, pet.getLevel());
			pet._inventory = new PetInventory(pet);
			return pet;
		}
		catch(Exception e)
		{
			_log.warning("could not restore Pet data: " + e);
			e.printStackTrace();
			return null;
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	public void store()
	{
		if(getControlItemId() == 0)
			return;

		String req;
		if(!isRespawned())
			req = "INSERT INTO pets (name,level,curHp,curMp,exp,sp,fed,objId,item_obj_id) " + "VALUES (?,?,?,?,?,?,?,?,?)";
		else
			req = "UPDATE pets SET name=?,level=?,curHp=?,curMp=?,exp=?,sp=?,fed=?,objId=? " + "WHERE item_obj_id = ?";
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement(req);
			statement.setString(1, _name);
			statement.setInt(2, _level);
			statement.setDouble(3, getCurrentHp());
			statement.setDouble(4, getCurrentMp());
			statement.setLong(5, _exp);
			statement.setLong(6, _sp);
			statement.setInt(7, _curFed);
			statement.setInt(8, _objectId);
			statement.setInt(9, _controlItemId);
			statement.executeUpdate();
			_respawned = true;
		}
		catch(Exception e)
		{
			_log.warning("could not store pet data: " + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	protected synchronized void stopFeed()
	{
		if(_feedTask != null)
		{
			_feedTask.cancel(false);
			_feedTask = null;
			if(Config.DEBUG)
				_log.fine("Feed task stop");
		}
	}

	public synchronized void startFeed(boolean battleFeed)
	{
		// stop feeding task if its active
		if(_feedTask != null)
			stopFeed();

		if(_feedTask == null && !isDead())
		{
			int _feedTime;
			if(battleFeed)
			{
				_feedMode = true;
				_feedTime = _data.getFeedBattle();
			}
			else
			{
				_feedMode = false;
				_feedTime = _data.getFeedNormal();
			}
			//	pet feed time must be different than 0. Changing time to bypass divide by 0
			if(_feedTime <= 0)
				_feedTime = 1;

			_feedTask = ThreadPoolManager.getInstance().scheduleGeneralAtFixedRate(new FeedTask(), 60000 / _feedTime, 60000 / _feedTime);
		}
	}

	@Override
	public synchronized void unSummon()
	{
		stopFeed();
		stopHpMpRegeneration();
		stopMove(null);
		if(_owner != null)
			_owner.sendPacket(new PetDelete(getObjectId(), 2));
		giveAllToOwner();
		store();
		decayMe();
		if(_owner != null)
			_owner.setPet(null);
		setTarget(null);
		L2World.removeObject(this);
		_owner = null;
	}

	@Override
	public void sendDamageMsg(int damage)
	{
		_owner.sendPacket(new SystemMessage(SystemMessage.THE_PET_GAVE_DAMAGE_OF_S1).addNumber(damage));
	}

	@Override
	public void sendCriticalHitMsg()
	{
		_owner.sendPacket(Msg.PETS_CRITICAL_HIT);
	}
	
	@SuppressWarnings("deprecation")
	private void deathPenalty()
	{
		int lvl = getLevel();
		double percentLost = -0.07 * lvl + 6.5;

		// Calculate the Experience loss
		lostExp = (int) Math.round((getExpForNextLevel() - getExpForThisLevel()) * percentLost / 100);

		_exp = _exp - lostExp;

		if(_exp < getExpForThisLevel())
		{
			if(_level == 1)
				return;
			_level--;
			_data = PetDataTable.getInstance().getInfo(getTemplate().npcId, _level);
			if(getTemplate().npcId == 12564)
				_owner.getPlayer().getInventory().getItemByObjectId(_controlItemId).setEnchantLevel(_level);
			broadcastStatusUpdate();
			StatusUpdate su = new StatusUpdate(_objectId);
			su.addAttribute(StatusUpdate.LEVEL, _level);
			su.addAttribute(StatusUpdate.MAX_HP, getMaxHp());
			su.addAttribute(StatusUpdate.MAX_MP, getMaxMp());
			broadcastPacket(su);
			setCurrentHpMp(getMaxHp(), getMaxMp());
		}
	}

	public void restoreExp()
	{
		restoreExp(100);
	}

	public void restoreExp(final double percent)
	{
		if(lostExp != 0)
		{
			addExpAndSp((long) (lostExp * percent / 100.), 0);
			lostExp = 0;
		}
		sendChanges();
	}

	public void doRevive(final double percent)
	{
		setCurrentHpMp(0.6 * getMaxHp(), 0.6 * getMaxMp());
		restoreExp(percent);
		doRevive();
	}

	public int getSkillLevel(int skillId)
	{
		if(_skills == null || _skills.get((short) skillId) == null)
			return -1;
		int lvl = getLevel();
		return lvl > 70 ? 7 + (lvl - 70) / 5 : lvl / 10;
	}

	@Override
	public boolean consumeItem(final int itemConsumeId, final int itemCount)
	{
		L2ItemInstance item = getInventory().findItemByItemId(itemConsumeId);
		return !(item == null || item.getCount() < itemCount) && getInventory().destroyItem(item, itemCount, true, false) != null;
	}

	@Override
	public void sendChanges()
	{
		broadcastStatusUpdate();
		_owner.sendPacket(new PetItemList(this));
	}
}