package L2jFT.game.model.instances;

import L2jFT.game.instancemanager.SiegeManager;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2SiegeClan;
import L2jFT.game.model.entity.siege.Siege;
import L2jFT.game.network.serverpackets.MyTargetSelected;
import L2jFT.game.network.serverpackets.StatusUpdate;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.network.serverpackets.ValidateLocation;
import L2jFT.game.templates.L2NpcTemplate;

public class L2SiegeFlagInstance extends L2NpcInstance
{
	private L2Player _player;
	private Siege _siege;
	private L2Clan _owner;
	private long _lastAnnouncedAttackedTime = 0;

	public L2SiegeFlagInstance(L2Player player, int objectId, L2NpcTemplate template)
	{
		super(objectId, template);

		_player = player;
		_siege = SiegeManager.getInstance().getSiege(_player.getX(), _player.getY());
		if(_player.getClan() == null || _siege == null)
			deleteMe();
		else
		{
			L2SiegeClan sc = _siege.getAttackerClan(_player.getClan());
			if(sc == null)
				deleteMe();
			else
				sc.setFlag(this);

			_owner = _player.getClan();
		}
	}

	@Override
	public String getName()
	{
		return _owner.getName();
	}

	@Override
	public String getTitle()
	{
		return "";
	}

	@Override
	public boolean isAutoAttackable(L2Character attacker)
	{
		L2Player player = attacker.getPlayer();
		if(player == null)
			return false;
		L2Clan clan = player.getClan();
		return clan == null || _owner.getLeaderId() != clan.getLeaderId();
	}

	public boolean isAttackable(L2Character attacker)
	{
		return isAutoAttackable(attacker);
	}

	@Override
	public void onForcedAttack(L2Player player)
	{
		player.setTarget(this);
		player.sendPacket(new MyTargetSelected(getObjectId(), player.getLevel() - getLevel()));
		StatusUpdate su = new StatusUpdate(getObjectId());
		su.addAttribute(StatusUpdate.CUR_HP, (int) getCurrentHp());
		su.addAttribute(StatusUpdate.MAX_HP, getMaxHp());
		player.sendPacket(su);
		player.sendPacket(new ValidateLocation(this));
		if(Math.abs(player.getZ() - getZ()) < 200)
			player.getAI().Attack(this, true);
		else
			player.sendActionFailed();
	}

	@Override
	public void onAction(L2Player player)
	{
		if(this != player.getTarget())
		{
			player.setTarget(this);
			player.sendPacket(new MyTargetSelected(getObjectId(), player.getLevel() - getLevel()));
			StatusUpdate su = new StatusUpdate(getObjectId());
			su.addAttribute(StatusUpdate.CUR_HP, (int) getCurrentHp());
			su.addAttribute(StatusUpdate.MAX_HP, getMaxHp());
			player.sendPacket(su);
			player.sendPacket(new ValidateLocation(this));
		}
		else
		{
			player.sendPacket(new MyTargetSelected(getObjectId(), player.getLevel() - getLevel()));
			if(Math.abs(player.getZ() - getZ()) < 200)
				player.getAI().Attack(this, false);
			else
				player.sendActionFailed();
		}
	}

	@Override
	public void doDie(L2Character killer)
	{
		L2SiegeClan sc = _siege.getAttackerClan(_player.getClan());
		if(sc != null)
			sc.removeFlag();

		super.doDie(killer);
	}

	@Override
	public void reduceCurrentHp(final double damage, final L2Character attacker, final boolean awake, final boolean standUp, boolean directHp)
	{
		if(System.currentTimeMillis() - _lastAnnouncedAttackedTime > 120000)
		{
			_lastAnnouncedAttackedTime = System.currentTimeMillis();
			_owner.broadcastToOnlineMembers(new SystemMessage(SystemMessage.YOUR_BASE_IS_BEING_ATTACKED));
		}
		super.reduceCurrentHp(damage, attacker, awake, standUp, directHp);
	}

	@Override
	public boolean hasRandomAnimation()
	{
		return false;
	}

	@Override
	public boolean isInvul()
	{
		return false;
	}

	@Override
	public boolean isFearImmune()
	{
		return true;
	}

	@Override
	public boolean paralyzeImmune()
	{
		return true;
	}

	@Override
	public boolean isLethalImmune()
	{
		return true;
	}
}
