package L2jFT.game.model.instances;

import L2jFT.extensions.scripts.Events;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.network.serverpackets.MyTargetSelected;
import L2jFT.game.network.serverpackets.NpcHtmlMessage;
import L2jFT.game.network.serverpackets.ShowTownMap;

import java.util.logging.Logger;

/**
 * GODSON ROX!
 */
public class L2StaticObjectInstance extends L2Object
{
	private static Logger _log = Logger.getLogger(L2StaticObjectInstance.class.getName());

	private int _staticObjectId;
	public static final int INTERACTION_DISTANCE = 150;
	private int _type = -1; // 0 - signs, 1 - throne, 2 - starter town map
	private String _filePath;
	private int _mapX;
	private int _mapY;

	/**
	 * @return Returns the StaticObjectId.
	 */
	public int getStaticObjectId()
	{
		return _staticObjectId;
	}

	/**
	 * @param doorId The doorId to set.
	 */
	public void setStaticObjectId(int StaticObjectId)
	{
		_staticObjectId = StaticObjectId;
	}

	public L2StaticObjectInstance(int objectId)
	{
		super(objectId);
	}

	public int getType()
	{
		return _type;
	}

	public void setType(int type)
	{
		_type = type;
	}

	public String getFilePath()
	{
		return _filePath;
	}

	public void setFilePath(String path)
	{
		_filePath = path;
	}

	public int getMapX()
	{
		return _mapX;
	}

	public int getMapY()
	{
		return _mapY;
	}

	public void setMapX(int x)
	{
		_mapX = x;
	}

	public void setMapY(int y)
	{
		_mapY = y;
	}

	/**
	 * this is called when a player interacts with this NPC
	 * @param player
	 */
	@Override
	public void onAction(L2Player player)
	{
		if(Events.onAction(player, this))
			return;

		if(_type < 0)
			_log.info("L2StaticObjectInstance: StaticObject with invalid type! StaticObjectId: " + getStaticObjectId());

		if(player.getTarget() != this)
		{
			player.setTarget(this);
			player.sendPacket(new MyTargetSelected(getObjectId(), 0));
			return;
		}

		MyTargetSelected my = new MyTargetSelected(getObjectId(), 0);
		player.sendPacket(my);

		// Calculate the distance between the L2Player and the L2NpcInstanceInstance
		if(!isInRange(player, INTERACTION_DISTANCE))
		{
			// Send a Server->Client packet ActionFailed (target is out of interaction range) to the L2Player player
			player.sendActionFailed();
			return;
		}

		if(_type == 0) // Arena Board
		{
			String filename = "data/html/newspaper/arena.htm";
			NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
			html.setFile(filename);
			// Open a chat window on client with the text of the Bulletin board
			player.sendPacket(html);
			// Send a Server->Client ActionFailed
			player.sendActionFailed();
		}
		else if(_type == 2) //Village map
		{
			player.sendPacket(new ShowTownMap(_filePath, getMapX(), getMapY()));
			player.sendActionFailed();
		}
	}

	@Override
	public boolean isAutoAttackable(L2Character attacker)
	{
		return false;
	}
}
