package L2jFT.game.model.instances;

import java.util.concurrent.Future;

import L2jFT.game.ThreadPoolManager;
import L2jFT.game.cache.Msg;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Playable;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Summon;
import L2jFT.game.network.serverpackets.PetLiveTime;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.taskmanager.DecayTaskManager;
import L2jFT.game.templates.L2NpcTemplate;

public class L2SummonInstance extends L2Summon
{
	public final int CYCLE = 5000; // in millis
	private float _expPenalty = 0;
	private int _itemConsumeIdInTime;
	private int _itemConsumeCountInTime;
	private int _itemConsumeDelay;

	Future _disappearTask;
	private Future _consumeTask;

	int _aliveTime;
	private int _maxTime;

	public L2SummonInstance(int objectId, L2NpcTemplate template, L2Player owner, int lifetime, int consumeid, int consumecount, int consumedelay)
	{
		super(objectId, template, owner);
		setName(template.name);
		setShowSummonAnimation(true);
		_aliveTime = _maxTime = lifetime;
		_itemConsumeIdInTime = consumeid;
		_itemConsumeCountInTime = consumecount;
		_itemConsumeDelay = consumedelay;
		_disappearTask = ThreadPoolManager.getInstance().scheduleGeneral(new Lifetime(this), CYCLE);
		if(_itemConsumeIdInTime > 0 && _itemConsumeCountInTime > 0)
			startConsume();

		isSummon = true;
	}

	@Override
	public final byte getLevel()
	{
		return getTemplate() != null ? getTemplate().level : 0;
	}

	@Override
	public int getSummonType()
	{
		return 1;
	}

	@Override
	public int getCurrentFed()
	{
		return _aliveTime;
	}

	@Override
	public int getMaxFed()
	{
		return _maxTime;
	}

	public void setExpPenalty(float expPenalty)
	{
		_expPenalty = expPenalty;
	}

	public float getExpPenalty()
	{
		return _expPenalty;
	}

	public void addExpAndSp(int addToExp, int addToSp)
	{
		_owner.addExpAndSp(addToExp, addToSp);
	}

	public void reduceCurrentHp(int damage, L2Character attacker)
	{
		if(attacker instanceof L2Playable && isInZoneBattle() != attacker.isInZoneBattle())
		{
			attacker.getPlayer().sendPacket(Msg.INCORRECT_TARGET);
			return;
		}
		super.reduceCurrentHp(damage, attacker);
		SystemMessage sm = new SystemMessage(SystemMessage.THE_SUMMONED_MONSTER_RECEIVED_DAMAGE_OF_S2_CAUSED_BY_S1);

		if(attacker instanceof L2NpcInstance)
			sm.addNpcName(((L2NpcInstance) attacker).getTemplate().npcId);
		else
			sm.addString(attacker.getName());

		_owner.sendPacket(sm.addNumber(damage));
	}

	public void displayHitMessage(int damage, boolean crit, boolean miss)
	{
		if(_owner == null)
			return;
		if(crit)
			_owner.sendPacket(Msg.SUMMONED_MONSTERS_CRITICAL_HIT);
		if(miss)
			_owner.sendPacket(Msg.MISSED_TARGET);
		else
			_owner.sendPacket(new SystemMessage(SystemMessage.THE_SUMMONED_MONSTER_GAVE_DAMAGE_OF_S1).addNumber(damage));
	}

	class Lifetime implements Runnable
	{
		private L2SummonInstance _summon;
		int _count;

		Lifetime(L2SummonInstance summon)
		{
			_summon = summon;
		}

		public void run()
		{
			_aliveTime -= CYCLE;

			if(_aliveTime <= 0)
			{
				_owner.sendPacket(new SystemMessage(SystemMessage.SERVITOR_DISAPPEASR_BECAUSE_THE_SUMMONING_TIME_IS_OVER));
				_disappearTask = null;
				unSummon();
				return;
			}

			_owner.sendPacket(new PetLiveTime(_summon));

			_disappearTask = ThreadPoolManager.getInstance().scheduleGeneral(new Lifetime(_summon), CYCLE);
		}
	}

	@Override
	public void doDie(L2Character killer)
	{
		super.doDie(killer);
		
		if(_disappearTask != null)
		{
			_disappearTask.cancel(false);
			_disappearTask = null;
		}
		
		stopConsume();
		
		DecayTaskManager.getInstance().addDecayTask(this);
	}

	static class Consume implements Runnable
	{
		private L2Player _activeChar;
		private L2SummonInstance _summon;

		Consume(L2Player activeChar, L2SummonInstance newpet)
		{
			_activeChar = activeChar;
			_summon = newpet;
		}

		public void run()
		{
			L2ItemInstance item = _activeChar.getInventory().findItemByItemId(_summon.getItemConsumeIdInTime());
			if(item != null && item.getCount() >= _summon.getItemConsumeCountInTime())
			{
				// System.out.println(_summon._aliveTime);
				L2ItemInstance dest = _activeChar.getInventory().destroyItemByItemId(_summon.getItemConsumeIdInTime(), _summon.getItemConsumeCountInTime(), true, true);
				_activeChar.sendPacket(new SystemMessage(SystemMessage.A_SUMMONED_MONSTER_USES_S1).addItemName(dest.getItemId()));
			}
			else
			{
				_activeChar.sendPacket(new SystemMessage(SystemMessage.SINCE_YOU_DO_NOT_HAVE_ENOUGH_ITEMS_TO_MAINTAIN_THE_SERVITORS_STAY_THE_SERVITOR_WILL_DISAPPEAR));
				_summon.unSummon();
			}
		}
	}

	public int getItemConsumeIdInTime()
	{
		return _itemConsumeIdInTime;
	}

	public int getItemConsumeCountInTime()
	{
		return _itemConsumeCountInTime;
	}

	public int getItemConsumeDelay()
	{
		return _itemConsumeDelay;
	}

	protected synchronized void stopConsume()
	{
		if(_consumeTask != null)
		{
			_consumeTask.cancel(true);
			_consumeTask = null;
		}
	}

	protected synchronized void stopDisappear()
	{
		if(_disappearTask != null)
		{
			_disappearTask.cancel(true);
			_disappearTask = null;
		}
	}

	public synchronized void startConsume()
	{
		if(_consumeTask == null && !isDead())
			_consumeTask = ThreadPoolManager.getInstance().scheduleGeneralAtFixedRate(new Consume(getOwner(), this), _itemConsumeDelay, _itemConsumeDelay);
	}

	@Override
	public synchronized void unSummon()
	{
		stopDisappear();
		stopConsume();
		super.unSummon();
	}

	@Override
	public void sendDamageMsg(int damage)
	{
		if(_owner != null)
			_owner.sendPacket(new SystemMessage(SystemMessage.THE_SUMMONED_MONSTER_GAVE_DAMAGE_OF_S1).addNumber(damage));
	}

	@Override
	public void sendCriticalHitMsg()
	{
		_owner.sendPacket(Msg.SUMMONED_MONSTERS_CRITICAL_HIT);
	}
}
