package L2jFT.game.model.instances;

import L2jFT.game.model.L2Player;
import L2jFT.game.model.entity.Castle;
import L2jFT.game.model.entity.SevenSigns;
import L2jFT.game.network.serverpackets.NpcHtmlMessage;
import L2jFT.game.network.serverpackets.Ride;
import L2jFT.game.templates.L2NpcTemplate;

import java.util.StringTokenizer;
import java.util.logging.Logger;

public final class L2WyvernManagerInstance extends L2NpcInstance
{
	private static Logger _log = Logger.getLogger(L2WyvernManagerInstance.class.getName());

	/**
	 * @param template
	 */
	public L2WyvernManagerInstance(int objectId, L2NpcTemplate template)
	{
		super(objectId, template);
	}

	@Override
	public void onBypassFeedback(L2Player player, String command)
	{
		StringTokenizer st = new StringTokenizer(command, " ");
		String actualCommand = st.nextToken(); // Get actual command
		boolean condition = validateCondition(player);

		if(actualCommand.equalsIgnoreCase("RideHelp"))
		{
			NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
			html.setFile("data/html/wyvern/help_ride.htm");
			html.replace("%npcname%", "Wyvern Manager " + getName());
			player.sendPacket(html);
			player.sendActionFailed();
		}
		if(condition)
		{
			if(actualCommand.equalsIgnoreCase("RideWyvern"))
				if(player.getMountType() != 1)
				{
					NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
					html.setFile("data/html/wyvern/not_ready.htm");
					html.replace("%npcname%", "Wyvern Manager " + getName());
					player.sendPacket(html);
				}
				else if(player.getInventory().getItemByItemId(1460).getCount() < 10)
				{
					NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
					html.setFile("data/html/wyvern/havenot_cry.htm");
					html.replace("%npcname%", "Wyvern Manager " + getName());
					player.sendPacket(html);
				}
				else if(SevenSigns.getInstance().getCurrentPeriod() == 3 && SevenSigns.getInstance().getCabalHighestScore() == 3)
				{
					NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
					html.setFile("data/html/wyvern/no_ride_dusk.htm");
					html.replace("%npcname%", "Wyvern Manager " + getName());
					player.sendPacket(html);
				}
				else
				{
					if(player.getInventory().destroyItemByItemId(1460, 25, true, false) == null)
						_log.info("L2WyvernManagerInstance[72]: Item not found!!!");
					Ride ride = new Ride(player, Ride.ACTION_MOUNT, 12621);
					player.broadcastPacket(ride);
					player.setMountType((short) 2);
					//ThreadPoolManager.getInstance().scheduleGeneral(new Dismount(player), 20 * 60 * 1000);
					//player.sendPacket(new SetupGauge(2, 20 * 60 * 1000));
					NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
					html.setFile("data/html/wyvern/after_ride.htm");
					html.replace("%npcname%", "Wyvern Manager " + getName());
					player.sendPacket(html);
				}
		}

		else
			super.onBypassFeedback(player, command);
	}

	@Override
	public void showChatWindow(L2Player player, int val)
	{
		player.setLastNpc(this);
		if(!validateCondition(player))
		{
			NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
			html.setFile("data/html/wyvern/lord_only.htm");
			html.replace("%npcname%", "Wyvern Manager " + getName());
			player.sendPacket(html);
			player.sendActionFailed();
			return;
		}
		NpcHtmlMessage html = new NpcHtmlMessage(_objectId);
		html.setFile("data/html/wyvern/lord_here.htm");
		html.replace("%Char_name%", String.valueOf(player.getName()));
		html.replace("%npcname%", "Wyvern Manager " + getName());
		player.sendPacket(html);
		player.sendActionFailed();
	}

	private boolean validateCondition(L2Player player)
	{
		Castle castle = getCastle();
		if(castle != null && castle.getCastleId() > 0)
			if(player.getClan() != null)
				if(castle.getOwnerId() == player.getClanId() && player.isClanLeader()) // Leader of clan
					return true; // Owner

		return false;
	}

}
