package L2jFT.game.model.inventory.listeners;

import L2jFT.game.model.Inventory;
import L2jFT.game.model.L2ArmorSet;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.tables.ArmorSetsTable;
import L2jFT.game.tables.SkillTable;

import java.util.logging.Logger;

public final class ArmorSetListener implements PaperdollListener
{
	public static short SET_COMMON_SKILL_ID = 3006;

	protected static final Logger _log = Logger.getLogger(ArmorSetListener.class.getName());

	Inventory _inv;

	public ArmorSetListener(Inventory inv)
	{
		_inv = inv;
	}

	public void notifyEquipped(int slot, L2ItemInstance item)
	{
		if(!_inv.getOwner().isPlayer)
			return;

		L2Player player = (L2Player) _inv.getOwner();

		// checks if player worns chest item
		L2ItemInstance chestItem = _inv.getPaperdollItem(Inventory.PAPERDOLL_CHEST);
		if(chestItem == null)
			return;

		// checks if there is armorset for chest item that player worns
		L2ArmorSet armorSet = ArmorSetsTable.getInstance().getSet(chestItem.getItemId());
		if(armorSet == null)
			return;

		// checks if equipped item is part of set
		if(armorSet.containItem(slot, item.getItemId()))
		{
			if(armorSet.containAll(player))
			{
				L2Skill skill = SkillTable.getInstance().getInfo(armorSet.getSkillId(), 1);
				L2Skill commonSetSkill = SkillTable.getInstance().getInfo(SET_COMMON_SKILL_ID, 1);

				if(skill != null)
				{
					player.addSkill(skill, false);
					player.addSkill(commonSetSkill, false);
				}
				else
					_log.warning("ArmorSetListener: Incorrect skill: " + armorSet.getSkillId() + ".");

				if(armorSet.containShield(player)) // has shield from set
				{
					L2Skill skills = SkillTable.getInstance().getInfo(armorSet.getShieldSkillId(), 1);
					if(skills != null)
						player.addSkill(skills, false);
					else
						_log.warning("ArmorSetListener: Incorrect skill: " + armorSet.getShieldSkillId() + ".");
				}
				if(armorSet.isEnchanted6(player)) // has all parts of set enchanted to 6 or more
				{
					int skillId = armorSet.getEnchant6skillId();
					if(skillId > 0)
					{
						L2Skill skille = SkillTable.getInstance().getInfo(skillId, 1);
						if(skille != null)
							player.addSkill(skille, false);
						else
							_log.warning("ArmorSetListener: Incorrect skill: " + armorSet.getEnchant6skillId() + ".");
					}
				}
			}
		}
		else if(armorSet.containShield(item.getItemId()))
			if(armorSet.containAll(player))
			{
				L2Skill skills = SkillTable.getInstance().getInfo(armorSet.getShieldSkillId(), 1);
				if(skills != null)
					player.addSkill(skills, false);
				else
					_log.warning("ArmorSetListener: Incorrect skill: " + armorSet.getShieldSkillId() + ".");
			}
	}

	public void notifyUnequipped(int slot, L2ItemInstance item)
	{
		boolean remove = false;
		int removeSkillId1 = 0; // set skill
		int removeSkillId2 = 0; // shield skill
		int removeSkillId3 = 0; // enchant +6 skill

		if(slot == Inventory.PAPERDOLL_CHEST)
		{
			L2ArmorSet armorSet = ArmorSetsTable.getInstance().getSet(item.getItemId());
			if(armorSet == null)
				return;

			remove = true;
			removeSkillId1 = armorSet.getSkillId();
			removeSkillId2 = armorSet.getShieldSkillId();
			removeSkillId3 = armorSet.getEnchant6skillId();

		}
		else
		{
			L2ItemInstance chestItem = _inv.getPaperdollItem(Inventory.PAPERDOLL_CHEST);
			if(chestItem == null)
				return;

			L2ArmorSet armorSet = ArmorSetsTable.getInstance().getSet(chestItem.getItemId());
			if(armorSet == null)
				return;

			if(armorSet.containItem(slot, item.getItemId())) // removed part of set
			{
				remove = true;
				removeSkillId1 = armorSet.getSkillId();
				removeSkillId2 = armorSet.getShieldSkillId();
				removeSkillId3 = armorSet.getEnchant6skillId();
			}
			else if(armorSet.containShield(item.getItemId())) // removed shield
			{
				remove = true;
				removeSkillId2 = armorSet.getShieldSkillId();
			}
		}

		if(remove)
		{
			if(removeSkillId1 != 0)
			{
				L2Skill skill = SkillTable.getInstance().getInfo(removeSkillId1, 1);
				L2Skill commonSetSkill = SkillTable.getInstance().getInfo(SET_COMMON_SKILL_ID, 1);
				if(skill != null)
				{
					((L2Player) _inv.getOwner()).removeSkill(skill, false);
					((L2Player) _inv.getOwner()).removeSkill(commonSetSkill, false);
				}
				else
					_log.warning("ArmorSetListener: Incorrect skill: " + removeSkillId1 + ".");
			}
			if(removeSkillId2 != 0)
			{
				L2Skill skill = SkillTable.getInstance().getInfo(removeSkillId2, 1);
				if(skill != null)
					_inv.getOwner().removeSkill(skill);
				else
					_log.warning("ArmorSetListener: Incorrect skill: " + removeSkillId2 + ".");
			}
			if(removeSkillId3 != 0)
			{
				L2Skill skill = SkillTable.getInstance().getInfo(removeSkillId3, 1);
				if(skill != null)
					_inv.getOwner().removeSkill(skill);
				else
					_log.warning("ArmorSetListener: Incorrect skill: " + removeSkillId3 + ".");
			}
		}
	}
}