package L2jFT.game.model.inventory.listeners;

import L2jFT.game.model.Inventory;
import L2jFT.game.model.instances.L2ItemInstance;

import java.util.ArrayList;

/**
 * Recorder of alterations in inventory
 */
public final class ChangeRecorder implements PaperdollListener
{
	private final ArrayList<L2ItemInstance> _changed;

	/**
	 * Constructor of the ChangeRecorder
	 * @param inventory inventory to watch
	 */
	public ChangeRecorder(Inventory inventory)
	{
		_changed = new ArrayList<L2ItemInstance>();
		inventory.addPaperdollListener(this);
	}

	/**
	 * Add alteration in inventory when item equipped
	 */
	@SuppressWarnings("unused")
	public void notifyEquipped(int slot, L2ItemInstance item)
	{
		if(!_changed.contains(item))
			_changed.add(item);
	}

	/**
	 * Add alteration in inventory when item unequipped
	 */
	@SuppressWarnings("unused")
	public void notifyUnequipped(int slot, L2ItemInstance item)
	{
		if(!_changed.contains(item))
			_changed.add(item);
	}

	/**
	 * Returns alterations in inventory
	 * @return L2ItemInstance[] : array of alterated items
	 */
	public L2ItemInstance[] getChangedItems()
	{
		return _changed.toArray(new L2ItemInstance[_changed.size()]);
	}
}
