package L2jFT.game.model.quest;

import javolution.util.FastList;
import L2jFT.game.ThreadPoolManager;
import L2jFT.util.Rnd;
import L2jFT.game.model.AutoChatHandler;
import L2jFT.game.model.AutoChatHandler.AutoChatInstance;
import L2jFT.game.model.AutoSpawnHandler;
import L2jFT.game.model.AutoSpawnHandler.AutoSpawnInstance;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Spawn;
import L2jFT.game.model.L2World;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.tables.NpcTable;
import L2jFT.game.templates.L2NpcTemplate;

import java.util.List;
import java.util.logging.Logger;

public class QuestPcSpawn
{
	public class DeSpawnScheduleTimerTask implements Runnable
	{
		int _ObjectId = 0;

		public DeSpawnScheduleTimerTask(int objectId)
		{
			_ObjectId = objectId;
		}

		public void run()
		{
			try
			{
				removeSpawn(_ObjectId);
			}
			catch(Throwable t)
			{}
		}
	}

	protected static Logger _log = Logger.getLogger(QuestPcSpawn.class.getName());

	private List<AutoSpawnInstance> _autoSpawns = new FastList<AutoSpawnInstance>();
	private String _Player;
	private List<L2Spawn> _Spawns = new FastList<L2Spawn>();

	public QuestPcSpawn(L2Player[] player)
	{
		_Player = player[0].getName();
	}

	private void addChatTask(int objectId, String[] messages, int chatDelay)
	{
		L2Object obj = L2World.findObject(objectId);

		if(!(obj instanceof L2NpcInstance))
			return;

		L2NpcInstance npcInst = (L2NpcInstance) obj;

		if(messages != null)
			AutoChatHandler.getInstance().registerChat(npcInst, messages, chatDelay);
	}

	private void addDeSpawnTask(int objectId, int spawnLength)
	{
		if(spawnLength > 0)
			ThreadPoolManager.getInstance().scheduleGeneral(new DeSpawnScheduleTimerTask(objectId), spawnLength);
	}

	/**
	 * Add random spawn for player instance
	 * Return object id of newly spawned npc
	 */
	public int addRandomSpawn(int npcId, int count, int initialDelay, int respawnDelay, int despawnDelay)
	{
		_autoSpawns.add(AutoSpawnHandler.getInstance().registerSpawn(npcId, initialDelay, respawnDelay, despawnDelay));
		_autoSpawns.get(_autoSpawns.size() - 1).setSpawnCount((byte) count);

		return _autoSpawns.get(_autoSpawns.size() - 1).getObjectId();
	}

	/**
	 * Add random chat for npc
	 */
	public void addRandomSpawnChat(int objectId, String[] messages, int chatDelay)
	{
		if(messages != null && chatDelay > 0)
			return;

		AutoSpawnInstance randomSpawn = getRandomSpawn(objectId);

		if(randomSpawn == null)
			return;

		for(L2NpcInstance npcInst : randomSpawn.getAttackableList())
			addChatTask(npcInst.getObjectId(), messages, chatDelay);
	}

	/**
	 * Add random spawn location for npc
	 */
	public void addRandomSpawnLoc(int objectId, int x, int y, int z)
	{
		AutoSpawnInstance randomSpawn = getRandomSpawn(objectId);

		if(randomSpawn == null)
			return;

		randomSpawn.addSpawnLocation(x, y, z, -1);
	}

	/**
	 * Add spawn for player instance
	 * Return object id of newly spawned npc
	 */
	public int addSpawn(int npcId)
	{
		return addSpawn(npcId, getPlayer().getX(), getPlayer().getY(), getPlayer().getZ(), true);
	}

	/**
	 * Add spawn for player instance
	 * Will despawn after the spawn length expires
	 * Return object id of newly spawned npc
	 */
	public int addSpawn(int npcId, int spawnLength)
	{
		return addSpawn(npcId, getPlayer().getX(), getPlayer().getY(), getPlayer().getZ(), true, spawnLength);
	}

	/**
	 * Add spawn for player instance
	 * Return object id of newly spawned npc
	 */
	public int addSpawn(int npcId, int x, int y, int z)
	{
		return addSpawn(npcId, x, y, z, false);
	}

	/**
	 * Add spawn for player instance
	 * Return object id of newly spawned npc
	 */
	public int addSpawn(int npcId, int x, int y, int z, boolean randomOffset)
	{
		try
		{
			if(randomOffset)
			{
				// Get the direction of the offset
				x += Rnd.get(50, 100) * (Rnd.nextBoolean() ? 1 : -1);
				// Get the direction of the offset
				y += Rnd.get(50, 100) * (Rnd.nextBoolean() ? 1 : -1);
			}

			L2NpcTemplate template1 = NpcTable.getTemplate(npcId);
			if(template1 != null)
			{
				L2Spawn spawn = new L2Spawn(template1);

				spawn.setId(npcId);
				if(getPlayer() != null)
					spawn.setHeading(getPlayer().getHeading());
				else
					spawn.setHeading(0);
				spawn.setLocx(x);
				spawn.setLocy(y);
				spawn.setLocz(z + 20);
				spawn.stopRespawn();

				spawn.spawnOne();

				_Spawns.add(spawn);

				return spawn.getId();
			}
		}
		catch(Exception e1)
		{
			_log.warning("Could not spawn Npc " + npcId);
		}

		return 0;
	}

	/**
	 * Add spawn for player instance
	 * Will despawn after the spawn length expires
	 * Return object id of newly spawned npc
	 */
	public int addSpawn(int npcId, int x, int y, int z, boolean randomOffset, int spawnLength)
	{
		return addSpawn(npcId, x, y, z, randomOffset, spawnLength, null, 0);
	}

	/**
	 * Add spawn for player instance
	 * Will despawn after the spawn length expires
	 * Will random message when chat delay time reached
	 * Return object id of newly spawned npc
	 */
	public int addSpawn(int npcId, int x, int y, int z, boolean randomOffset, int spawnLength, String[] messages, int chatDelay)
	{
		int objectId = addSpawn(npcId, x, y, z, randomOffset);
		addDeSpawnTask(objectId, spawnLength);
		addChatTask(objectId, messages, chatDelay);
		return objectId;
	}

	/**
	 * Add spawn for player instance
	 * Will random message when chat delay time reached
	 * Return object id of newly spawned npc
	 */
	public int addSpawn(int npcId, int x, int y, int z, boolean randomOffset, String[] messages, int chatDelay)
	{
		return addSpawn(npcId, x, y, z, randomOffset, 0, messages, chatDelay);
	}

	/**
	 * Add spawn for player instance
	 * Will despawn after the spawn length expires
	 * Return object id of newly spawned npc
	 */
	public int addSpawn(int npcId, int x, int y, int z, int spawnLength)
	{
		return addSpawn(npcId, x, y, z, false, spawnLength);
	}

	/**
	 * Add spawn for player instance
	 * Will despawn after the spawn length expires
	 * Will random message when chat delay time reached
	 * Return object id of newly spawned npc
	 */
	public int addSpawn(int npcId, int x, int y, int z, int spawnLength, String[] messages, int chatDelay)
	{
		return addSpawn(npcId, x, y, z, false, spawnLength, messages, chatDelay);
	}

	/**
	 * Add spawn for player instance
	 * Will random message when chat delay time reached
	 * Return object id of newly spawned npc
	 */
	public int addSpawn(int npcId, int x, int y, int z, String[] messages, int chatDelay)
	{
		return addSpawn(npcId, x, y, z, false, messages, chatDelay);
	}

	/**
	 * Add spawn for player instance
	 * Will despawn after the spawn length expires
	 * Will random message when chat delay time reached
	 * Return object id of newly spawned npc
	 */
	public int addSpawn(int npcId, int spawnLength, String[] messages, int chatDelay)
	{
		return addSpawn(npcId, getPlayer().getX(), getPlayer().getY(), getPlayer().getZ(), true, spawnLength, messages, chatDelay);
	}

	/**
	 * Add spawn for player instance
	 * Will random message when chat delay time reached
	 * Return object id of newly spawned npc
	 */
	public int addSpawn(int npcId, String[] messages, int chatDelay)
	{
		return addSpawn(npcId, getPlayer().getX(), getPlayer().getY(), getPlayer().getZ(), true, messages, chatDelay);
	}

	/**
	 * Return AutoChatInstance for player instance
	 */
	public AutoChatInstance getAutoChat(int objectId)
	{
		return AutoChatHandler.getInstance().getAutoChatInstance(objectId, true);
	}

	/** Return current player instance */
	public L2Player getPlayer()
	{
		return L2World.getPlayer(_Player);
	}

	/**
	 * Return AutoSpawnInstance for player instance
	 */
	public AutoSpawnInstance getRandomSpawn(int objectId)
	{
		for(AutoSpawnInstance randomSpawn : _autoSpawns)
			if(randomSpawn.getObjectId() == objectId)
				return randomSpawn;

		return null;
	}

	/**
	 * Return list of AutoSpawnInstances for player instance
	 */
	public List<AutoSpawnInstance> getRandomSpawns()
	{
		if(_autoSpawns == null)
			_autoSpawns = new FastList<AutoSpawnInstance>();

		return _autoSpawns;
	}

	/**
	 * Return spawn instance for player instance
	 */
	public L2Spawn getSpawn(int objectId)
	{
		for(AutoSpawnInstance randomSpawn : getRandomSpawns())
			for(L2NpcInstance npcInst : randomSpawn.getAttackableList())
				if(npcInst.getObjectId() == objectId)
					return npcInst.getSpawn();

		for(L2Spawn spawn : getSpawns())
			if(spawn.getId() == objectId)
				return spawn;
		return null;
	}

	/**
	 * Return list of L2Spawn for player instance
	 */
	public List<L2Spawn> getSpawns()
	{
		if(_Spawns == null)
			_Spawns = new FastList<L2Spawn>();
		return _Spawns;
	}

	/**
	 * Return true if spawn was created by addSpawn method.
	 * @param npcId
	 * @return
	 */
	public boolean isSpawnExists(int npcId)
	{
		for(int i = 0; i < _Spawns.size(); i++)
			if(_Spawns.get(i).getNpcid() == npcId)
				return true;
		return false;
	}

	/**
	 * Remove all spawn for player instance
	 */
	public void removeAllSpawn()
	{
		while(_autoSpawns.size() > 0)
		{
			AutoSpawnInstance currSpawn = _autoSpawns.get(0);
			AutoSpawnHandler.getInstance().removeSpawn(currSpawn.getObjectId());
			_autoSpawns.remove(0);
		}

		if(getSpawns() == null)
			return;

		while(getSpawns().size() > 0)
		{
			getSpawns().get(0).getLastSpawn().decayMe();
			getSpawns().remove(0);
		}
	}

	/**
	 * Remove spawn with object id for player instance
	 */
	public void removeSpawn(int objectId)
	{
		for(int i = 0; i < _autoSpawns.size(); i++)
			if(_autoSpawns.get(i).getObjectId() == objectId)
			{
				AutoSpawnHandler.getInstance().removeSpawn(objectId);
				_autoSpawns.remove(i);
				return;
			}

		for(int i = 0; i < getSpawns().size(); i++)
			if(getSpawns().get(i).getId() == objectId)
			{
				L2NpcInstance npc = getSpawns().get(i).getLastSpawn();
				if(npc != null)
					npc.decayMe();
				getSpawns().remove(i);
				return;
			}
	}

	/**
	 * Remove spawn with object id from list for player instance
	 */
	public void removeSpawns(int[] objectIds)
	{
		for(int objectId : objectIds)
			removeSpawn(objectId);
	}
}
