package L2jFT.game.model.quest;

import java.util.concurrent.ScheduledFuture;

import L2jFT.game.ThreadPoolManager;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.instances.L2NpcInstance;

public class QuestTimer
{
	public class ScheduleTimerTask implements Runnable
	{
		public void run()
		{
			if(this == null || !getIsActive())
				return;

			try
			{
				getQuest().notifyEvent(getName(), getPlayer().getQuestState(getQuest().getName()));
				cancel();
			}
			catch(Throwable t)
			{}
		}
	}

	private boolean _isActive = true;
	private String _name;
	private L2NpcInstance _npc;
	private L2Player _player;
	private Quest _quest;
	private ScheduledFuture _schedular;

	public QuestTimer(Quest quest, String name, long time, L2NpcInstance npc, L2Player player)
	{
		_name = name;
		_quest = quest;
		_player = player;
		_npc = npc;
		_schedular = ThreadPoolManager.getInstance().scheduleGeneral(new ScheduleTimerTask(), time); // Prepare auto end task
	}

	public QuestTimer(QuestState qs, String name, long time)
	{
		_name = name;
		_quest = qs.getQuest();
		_player = qs.getPlayer();
		_npc = null;
		_schedular = ThreadPoolManager.getInstance().scheduleGeneral(new ScheduleTimerTask(), time); // Prepare auto end task
	}

	public void cancel()
	{
		_isActive = false;

		if(_schedular != null)
			_schedular.cancel(true);

		getQuest().removeQuestTimer(this);
	}

	public final boolean getIsActive()
	{
		return _isActive;
	}

	public final String getName()
	{
		return _name;
	}

	public final L2NpcInstance getNpc()
	{
		return _npc;
	}

	public final L2Player getPlayer()
	{
		return _player;
	}

	public final Quest getQuest()
	{
		return _quest;
	}

	// public method to compare if this timer matches with the key attributes passed.
	// a quest and a name are required.  
	// null npc or player act as wildcards for the match
	public boolean isMatch(Quest quest, String name, L2NpcInstance npc, L2Player player)
	{
		if(quest == null || name == null)
			return false;
		if(quest != getQuest() || name.compareToIgnoreCase(getName()) != 0)
			return false;
		return (npc == null || getNpc() == null || npc == getNpc()) && (player == null || getPlayer() == null || player == getPlayer());
	}

	@Override
	public final String toString()
	{
		return _name;
	}
}
