package L2jFT.game.model.quest;

import java.util.Map;

import javolution.util.FastList;
import javolution.util.FastMap;
import L2jFT.game.model.L2DropData;

public class State
{
	private Map<Integer, FastList<L2DropData>> _drops;

	/** Name of the quest */
	private String _name;

	/** Quest object associated to the state */
	private final Quest _quest;

	/**
	 * Constructor for the state of the quest. 
	 * @param name : String pointing out the name of the quest
	 * @param quest : Quest
	 */
	public State(String name, Quest quest)
	{
		_name = name;
		_quest = quest;
		quest.addState(this);
	}

	/**
	 * Add drop for the quest at this state of the quest 
	 * @param npcId : int designating the ID of the NPC
	 * @param itemId : int designating the ID of the item dropped
	 * @param chance : int designating the chance the item dropped 
	 */
	public void addQuestDrop(int npcId, int itemId, int chance)
	{
		try
		{
			if(_drops == null)
				_drops = new FastMap<Integer, FastList<L2DropData>>();
			L2DropData d = new L2DropData();
			d.setItemId((short) itemId);
			d.setChance(chance);
			d.setQuestID(_quest.getName());
			d.addStates(new String[] { _name });
			FastList<L2DropData> lst = _drops.get(npcId);
			if(lst != null)
				lst.add(d);
			else
			{
				lst = new FastList<L2DropData>();
				lst.add(d);
				_drops.put(npcId, lst);
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	/**
	 * Return list of drops at this step/state of the quest.
	 * @return HashMap
	 */
	Map<Integer, FastList<L2DropData>> getDrops()
	{
		return _drops;
	}

	/**
	 * Return name of the quest
	 * @return String
	 */
	public String getName()
	{
		return _name;
	}

	public Quest getQuest()
	{
		return _quest;
	}

	/**
	 * Return name of the quest
	 * @return String
	 */
	@Override
	public String toString()
	{
		return _name;
	}
}
