package L2jFT.game.network.clientpackets;

import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2TradeList;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.network.serverpackets.SendTradeDone;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.network.serverpackets.TradeMove;
import L2jFT.game.network.serverpackets.TradeOtherAdd;
import L2jFT.game.network.serverpackets.TradeOwnAdd;

public class AddTradeItem extends L2GameClientPacket
{
	private static String _C__16_ADDTRADEITEM = "[C] 16 AddTradeItem";
	//private static Logger _log = Logger.getLogger(AddTradeItem.class.getName());

	@SuppressWarnings("unused")
	private int _tradeId;
	private int _objectId;
	private int _amount;

	@Override
	public void readImpl()
	{
		_tradeId = readD();
		_objectId = readD();
		_amount = readD();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;
		L2Player requestor = activeChar.getTransactionRequester();

		L2TradeList playerItemList = activeChar.getTradeList();

		if(requestor == null || requestor.getTransactionRequester() == null)
		{
			// trade partner logged off. trade is canceld
			activeChar.sendPacket(new SendTradeDone(0));
			activeChar.sendPacket(new SystemMessage(SystemMessage.TARGET_IS_NOT_FOUND_IN_THE_GAME));
			activeChar.setTransactionRequester(null);
			if(playerItemList != null && playerItemList.getItems() != null)
				playerItemList.getItems().clear();
			return;
		}

		if(requestor.getTradeList().hasConfirmed())
		{
			//Can't add item if one of players has confirmed the trade... no system message?!
			activeChar.sendActionFailed();
			return;
		}

		if(activeChar.getTradeList().hasConfirmed())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_MOVE_ADDITIONAL_ITEMS_BECAUSE_TRADE_HAS_BEEN_CONFIRMED));
			activeChar.sendActionFailed();
			return;
		}

		L2ItemInstance InvItem = activeChar.getInventory().getItemByObjectId(_objectId);

		if(InvItem == null || !InvItem.canBeTraded(activeChar))
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.THIS_ITEM_CANNOT_BE_TRADED_OR_SOLD));
			return;
		}

		if(InvItem.isWear())
			return;

		int InvItemCount = InvItem.getCount();

		L2ItemInstance TradeItem;

		int realCount = _amount;

		if(_amount > InvItemCount)
			realCount = InvItemCount;
		int leaveCount = InvItemCount - realCount;
		if(playerItemList.getItems().size() <= 0 || !playerItemList.contains(_objectId))
		{
			TradeItem = new L2ItemInstance(_objectId, InvItem.getItem(), activeChar.getObjectId(), requestor.getObjectId(), "AddTradeItem[78]");
			if(TradeItem.isWear())
				return;
			TradeItem.setCount(realCount);
			TradeItem.setEnchantLevel(InvItem.getEnchantLevel());
			playerItemList.addItem(TradeItem);
		}
		else
		{
			TradeItem = playerItemList.getItem(_objectId);
			if(TradeItem == null || TradeItem.isWear())
				return;
			int TradeItemCount = TradeItem.getCount();
			if(InvItemCount == TradeItemCount)
				return;

			if(_amount + TradeItemCount >= InvItemCount)
				realCount = InvItemCount - TradeItemCount;

			TradeItem.setCount(realCount + TradeItemCount);
			leaveCount = InvItemCount - realCount - TradeItemCount;
		}

		activeChar.sendPacket(new TradeOwnAdd(InvItem, realCount));
		activeChar.sendPacket(new TradeMove(InvItem, leaveCount));
		requestor.sendPacket(new TradeOtherAdd(InvItem, realCount));
	}

	@Override
	public String getType()
	{
		return _C__16_ADDTRADEITEM;
	}
}