package L2jFT.game.network.clientpackets;

import java.util.logging.Level;
import java.util.logging.Logger;

import L2jFT.Config;
import L2jFT.database.DatabaseUtils;
import L2jFT.database.FiltredPreparedStatement;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.database.ThreadConnection;
import L2jFT.game.instancemanager.QuestManager;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2ShortCut;
import L2jFT.game.model.L2SkillLearn;
import L2jFT.game.model.L2World;
import L2jFT.game.model.base.ClassId;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.quest.Quest;
import L2jFT.game.network.L2GameClient;
import L2jFT.game.network.serverpackets.CharCreateFail;
import L2jFT.game.network.serverpackets.CharCreateOk;
import L2jFT.game.network.serverpackets.CharSelectInfo;
import L2jFT.game.tables.CharNameTable;
import L2jFT.game.tables.ItemTable;
import L2jFT.game.tables.SkillTable;
import L2jFT.game.tables.SkillTreeTable;
import L2jFT.game.templates.L2Item;
import L2jFT.game.templates.L2PlayerTemplate;
import L2jFT.util.Util;

/**
 * This class ...
 *
 * @version $Revision: 1.9.2.3.2.8 $ $Date: 2005/03/27 15:29:30 $
 */
@SuppressWarnings("unused")
public class CharacterCreate extends L2GameClientPacket
{
	private static String _C__0B_CHARACTERCREATE = "[C] 0B CharacterCreate";

	private static Logger _log = Logger.getLogger(CharacterCreate.class.getName());

	// cSdddddddddddd
	private String _name;
	private byte _race;
	private byte _sex;
	private short _classId;
	private byte _int;
	private byte _str;
	private byte _con;
	private byte _men;
	private byte _dex;
	private byte _wit;
	private byte _hairStyle;
	private byte _hairColor;
	private byte _face;

	/**
	 * @param decrypt
	 */
	@Override
	public void readImpl()
	{
		_name = readS();
		_race = (byte) readD();
		_sex = (byte) readD();
		_classId = (short) readD();
		_int = (byte) readD();
		_str = (byte) readD();
		_con = (byte) readD();
		_men = (byte) readD();
		_dex = (byte) readD();
		_wit = (byte) readD();
		_hairStyle = (byte) readD();
		_hairColor = (byte) readD();
		_face = (byte) readD();
	}

	@Override
	public void runImpl()
	{
		for(ClassId cid : ClassId.values())
			if(cid.getId() == _classId && cid.getLevel() != 1)
				return;
		if(CharNameTable.getInstance().accountCharNumber(getClient().getLoginName()) >= 8)
		{
			if(Config.DEBUG)
				_log.fine("Max number of characters reached. Creation failed.");
			sendPacket(new CharCreateFail(CharCreateFail.REASON_TOO_MANY_CHARACTERS));
			return;
		}
		if(!Util.isMatchingRegexp(_name, Config.CNAME_TEMPLATE))
		{
			if(Config.DEBUG)
				_log.fine("charname: " + _name + " is invalid. creation failed.");
			sendPacket(new CharCreateFail(CharCreateFail.REASON_16_ENG_CHARS));
			return;
		}
		else if(CharNameTable.getInstance().doesCharNameExist(_name))
		{
			if(Config.DEBUG)
				_log.fine("charname: " + _name + " already exists. creation failed.");
			sendPacket(new CharCreateFail(CharCreateFail.REASON_NAME_ALREADY_EXISTS));
			return;
		}

		if(Config.DEBUG)
			_log.fine("charname: " + _name + " classId: " + _classId);

		L2Player newChar = L2Player.create(_classId, _sex, getClient().getLoginName(), _name, _hairStyle, _hairColor, _face);
		sendPacket(new CharCreateOk());

		initNewChar(getClient(), newChar);
	}

	private void initNewChar(L2GameClient client, L2Player newChar)
	{
		if(Config.DEBUG)
			_log.fine("Character init start");
		L2World.addObject(newChar);

		L2PlayerTemplate template = newChar.getTemplate();

		L2Player.restoreCharSubClasses(newChar);

		newChar.addAdena(Config.STARTING_ADENA);

		newChar.setXYZInvisible(template.spawnX, template.spawnY, template.spawnZ);
		newChar.setTitle("");

		// add attack, take, sit shortcut
		newChar.registerShortCut(new L2ShortCut(0, 0, 3, 2, -1));
		newChar.registerShortCut(new L2ShortCut(3, 0, 3, 5, -1));
		newChar.registerShortCut(new L2ShortCut(10, 0, 3, 0, -1));

		ItemTable itemTable = ItemTable.getInstance();
		for(L2Item i : template.getItems())
		{
			L2ItemInstance item = itemTable.createItem(i.getItemId(), newChar.getObjectId(), 0, "CharacterCreate[178]");
			newChar.getInventory().addItem(item);

			if(item.getItemId() == 5588) //tutorial book
				newChar.registerShortCut(new L2ShortCut(11, 0, 1, item.getObjectId(), -1));

			if(item.isEquipable())
				if(newChar.getActiveWeaponItem() == null || item.getItem().getType2() == L2Item.TYPE2_WEAPON)
					newChar.getInventory().equipItemAndRecord(item);
		}

		for(L2SkillLearn skill : SkillTreeTable.getInstance().getAvailableSkills(newChar, newChar.getClassId()))
		{
			int skillId = skill.id;
			newChar.addSkill(SkillTable.getInstance().getInfo(skillId, skill.skillLevel), true);
			if(skillId == 1001 || skillId == 1177)
				newChar.registerShortCut(new L2ShortCut(1, 0, 2, skillId, 1));

			if(skillId == 1216)
				newChar.registerShortCut(new L2ShortCut(10, 0, 2, skillId, 1));
		}

		startTutorialQuest(newChar);
		L2GameClient.saveCharToDisk(newChar);
		newChar.deleteMe(false); // release the world of this character and it's inventory

		CharSelectInfo cl = new CharSelectInfo(client.getLoginName(), client.getSessionId().playOkID1);
		client.getConnection().sendPacket(cl);
		client.setCharSelection(cl.getCharInfo());
		if(Config.DEBUG)
			_log.fine("Character init end");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see L2jFT.game.network.clientpackets.ClientBasePacket#getType()
	 */
	@Override
	public String getType()
	{
		return _C__0B_CHARACTERCREATE;
	}

	private void startTutorialQuest(L2Player player)
	{
		Quest q = QuestManager.getInstance().getQuest(255);

		if(q == null) //Квест не загружен, пропустить создание в базе
			return;

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("INSERT INTO character_quests (char_id,name,var,value) VALUES (?,?,?,?)");
			statement.setInt(1, player.getObjectId());
			statement.setString(2, q.getName());
			statement.setString(3, "<state>");
			statement.setString(4, "Start");
			statement.executeUpdate();
		}
		catch(Exception e)
		{
			_log.log(Level.WARNING, "could not insert char quest:", e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}
}
