package L2jFT.game.network.clientpackets;

import static L2jFT.game.model.L2Zone.ZoneType.no_restart;
import L2jFT.Config;
import L2jFT.database.DatabaseUtils;
import L2jFT.database.FiltredPreparedStatement;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.database.ThreadConnection;
import L2jFT.extensions.scripts.Scripts;
import L2jFT.game.Announcements;
import L2jFT.game.ThreadPoolManager;
import L2jFT.game.instancemanager.CastleManager;
import L2jFT.game.instancemanager.CoupleManager;
import L2jFT.game.instancemanager.CursedWeaponsManager;
import L2jFT.game.instancemanager.DimensionalRiftManager;
import L2jFT.game.instancemanager.QuestManager;
import L2jFT.game.instancemanager.SiegeManager;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;
import L2jFT.game.model.L2Zone.ZoneType;
import L2jFT.game.model.entity.Castle;
import L2jFT.game.model.entity.Couple;
import L2jFT.game.model.entity.SevenSigns;
import L2jFT.game.model.entity.siege.Siege;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.quest.Quest;
import L2jFT.game.model.quest.QuestState;
import L2jFT.game.network.serverpackets.ClientSetTime;
import L2jFT.game.network.serverpackets.Die;
import L2jFT.game.network.serverpackets.EtcStatusUpdate;
import L2jFT.game.network.serverpackets.ExStorageMaxCount;
import L2jFT.game.network.serverpackets.FriendList;
import L2jFT.game.network.serverpackets.FriendStatus;
import L2jFT.game.network.serverpackets.HennaInfo;
import L2jFT.game.network.serverpackets.ItemList;
import L2jFT.game.network.serverpackets.PledgeShowInfoUpdate;
import L2jFT.game.network.serverpackets.PledgeShowMemberListAll;
import L2jFT.game.network.serverpackets.PledgeShowMemberListUpdate;
import L2jFT.game.network.serverpackets.QuestList;
import L2jFT.game.network.serverpackets.ShortCutInit;
import L2jFT.game.network.serverpackets.SignsSky;
import L2jFT.game.network.serverpackets.SkillList;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.network.serverpackets.UserInfo;
import L2jFT.game.tables.MapRegionTable;
import L2jFT.util.Log;

import java.sql.ResultSet;
import java.util.concurrent.ScheduledFuture;
import java.util.logging.Logger;

/**
 * Enter World Packet Handler<p>
 * <p>
 * 0000: 03 <p>
 * <p>
 *
 * @version $Revision: 1.16.2.1.2.7 $ $Date: 2005/03/29 23:15:33 $
 */
public class EnterWorld extends L2GameClientPacket
{
	private static String _C__03_ENTERWORLD = "[C] 03 EnterWorld";
	private static Logger _log = Logger.getLogger(EnterWorld.class.getName());
	@SuppressWarnings("unchecked")
	protected ScheduledFuture _currentFuture;

	@Override
	public void readImpl()
	{}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();

		if(activeChar == null)
		{
			getClient().closeNow(false);
			return;
		}

		activeChar.spawnMe(); // activeChar.getX(), activeChar.getY(), activeChar.getZ()

		if(SevenSigns.getInstance().isSealValidationPeriod())
			activeChar.sendPacket(new SignsSky());

		sendPacket(new UserInfo(activeChar));
		sendPacket(new HennaInfo(activeChar));
		sendPacket(new ItemList(activeChar, false));
		sendPacket(new ShortCutInit(activeChar));
		activeChar.getMacroses().sendUpdate();
		sendPacket(new ClientSetTime());
		sendPacket(new SkillList(activeChar));
		sendPacket(new SystemMessage(SystemMessage.WELCOME_TO_THE_WORLD_OF_LINEAGE_II));

		//sendPacket(new SystemMessage(SystemMessage.S1_S2).addString("This Server is running L2 Rebellion"));
		
		// Вызов всех хэндлеров, определенных в скриптах
		for(String handler : Scripts.onPlayerEnter)
		{
			String[] list = handler.split(":");
			String _class = list[0];
			String _method = list[1];
			activeChar.callScripts(_class, _method, new Object[] { activeChar });
		}

		Announcements.getInstance().showAnnouncements(activeChar);
		SevenSigns.getInstance().sendCurrentPeriodMsg(activeChar);
		//		loadTutorial(activeChar);

		if(activeChar.isGM() && !Config.SHOW_GM_LOGIN)
			activeChar.setInvisible(true);

		// send user info again .. just like the real client
		// sendPacket(new UserInfo(activeChar));

		if(activeChar.getClanId() != 0 && activeChar.getClan() != null)
		{
			sendPacket(new PledgeShowMemberListAll(activeChar.getClan(), activeChar));
			sendPacket(new PledgeShowInfoUpdate(activeChar.getClan()));
		}

		if(activeChar.isAlikeDead())
			// no broadcast needed since the player will already spawn dead to others
			sendPacket(new Die(activeChar));

		activeChar.checkWaterState();

		//add char to online characters
		activeChar.setOnlineStatus(true);

		// engage and notify Partner
		if(Config.ALLOW_WEDDING)
		{
			engage(activeChar);
			notifyPartner(activeChar);
		}

		Log.LogChar(activeChar, Log.EnterWorld, "");

		activeChar.restoreEffects();
		activeChar.restoreDisableSkills();

		loadTutorial(activeChar);
		notifyFriends(activeChar, true);
		notifyClanMembers(activeChar);

		if(activeChar.getClan() != null)
			for(Castle castle : CastleManager.getInstance().getCastles().values())
			{
				Siege siege = castle.getSiege();
				if(!siege.isInProgress())
					continue;
				if(siege.checkIsAttacker(activeChar.getClan()))
					activeChar.setSiegeState(1);
				else if(siege.checkIsDefender(activeChar.getClan()))
					activeChar.setSiegeState(2);
			}

		_currentFuture = ThreadPoolManager.getInstance().scheduleGeneralAtFixedRate(new WarnUserTakeBreak(activeChar), 7200000, 7200000);

		activeChar.sendPacket(new ExStorageMaxCount(activeChar));
		activeChar.sendPacket(new QuestList());
		activeChar.broadcastUserInfo();

		//	 apply augmentation boni for equipped items 
		for(L2ItemInstance temp : activeChar.getInventory().getAugmentedItems())
			if(temp != null && temp.isEquipped())
				temp.getAugmentation().applyBoni(activeChar);

		if(Config.ENABLE_OLYMPIAD && activeChar.isInZone(ZoneType.OlympiadStadia))
		{
			activeChar.sendMessage("You have been teleported to the nearest town due to you being in an Olympiad Stadia", "Вы телепортированы в ближайший город, поскольку находились на олимпийском стадионе.");
			activeChar.teleToLocation(MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.ClosestTown));
		}

		if(activeChar.isInZoneIncludeZ(no_restart))
		{
			long allowed_time = activeChar.getZone(no_restart).getRestartTime();
			long last_time = activeChar.getLastAccess();
			long curr_time = System.currentTimeMillis() / 1000;

			if(curr_time - last_time > allowed_time)
			{
				activeChar.sendMessage("You have been teleported to the nearest town due to you being in an No Restart zone", "Вы телепортированы в ближайший город, поскольку находились в специальной зоне.");
				activeChar.teleToLocation(MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.ClosestTown));
			}
		}

		if(!activeChar.isGM() && activeChar.isInZone(ZoneType.Siege))
		{
			Siege siege = SiegeManager.getInstance().getSiege(activeChar.getX(), activeChar.getY());
			if(siege != null && !siege.checkIsDefender(activeChar.getClan()))
				if(siege.getFlag(activeChar.getClan()) == null)
					activeChar.teleToLocation(MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.ClosestTown));
				else
					activeChar.teleToLocation(MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.SiegeFlag));
		}

		CursedWeaponsManager.getInstance().checkPlayer(activeChar);

		// refresh player info
		refreshInfo(activeChar);

		if(DimensionalRiftManager.getInstance().checkIfInRiftZone(activeChar.getX(), activeChar.getY(), activeChar.getZ(), false))
			DimensionalRiftManager.getInstance().teleportToWaitingRoom(activeChar);
	}

	private void refreshInfo(L2Player player)
	{
		player.updateTerritories();
		player.sendPacket(new EtcStatusUpdate(player));
		player.getInventory().refreshListeners();
		player.checkHpMessages(player.getMaxHp(), player.getCurrentHp());
		player.checkDayNightMessages();
		player.doZoneCheck(0);
	}

	/**
	 * @param activeChar
	 */
	private void engage(L2Player cha)
	{
		int _chaid = cha.getObjectId();

		for(Couple cl : CoupleManager.getInstance().getCouples())
			if(cl.getPlayer1Id() == _chaid || cl.getPlayer2Id() == _chaid)
			{
				if(cl.getMaried())
					cha.setMaried(true);

				cha.setCoupleId(cl.getId());

				if(cl.getPlayer1Id() == _chaid)
					cha.setPartnerId(cl.getPlayer2Id());
				else
					cha.setPartnerId(cl.getPlayer1Id());
			}
	}

	/**
	 * @param activeChar
	 */
	private void notifyPartner(L2Player cha)
	{
		if(cha.getPartnerId() != 0)
		{
			L2Object partner = L2World.findObject(cha.getPartnerId());
			if(partner != null)
			{
				if(partner.isPlayer)
					((L2Player) partner).sendMessage("Your Partner has logged in.", "Пришла Ваша вторая половинка!");
				else
					_log.warning(cha + " partner is " + partner.getL2ClassShortName() + "?!!");
			}
			else if(Config.DEBUG)
				_log.info(cha + " partner not in world.");
		}
	}

	/**
	 * @param activeChar
	 */
	public static void notifyFriends(L2Player cha, boolean login)
	{
		if(login)
			cha.sendPacket(new FriendList(cha, false));
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT friend_id from character_friends where char_id=?");
			statement.setInt(1, cha.getObjectId());
			rset = statement.executeQuery();
			while(rset.next())
			{
				int objectId = rset.getInt("friend_id");
				L2Player friend = (L2Player) L2World.findObject(objectId);
				if(friend != null)
					if(login)
					{
						friend.sendPacket(new SystemMessage(SystemMessage.S1_FRIEND_HAS_LOGGED_IN).addString(cha.getName()));
						friend.sendPacket(new FriendStatus(cha, true));
					}
					else
					{
						friend.sendPacket(new FriendStatus(cha, false));
						cha.sendPacket(new FriendList(cha, false));
					}
			}
		}
		catch(Exception e)
		{
			_log.warning("could not restore friend data:" + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
		cha.updateEffectIcons();
	}

	/**
	 * @param activeChar
	 */
	private void notifyClanMembers(L2Player activeChar)
	{
		L2Clan clan = activeChar.getClan();
		if(clan == null)
			return;

		clan.getClanMember(activeChar.getObjectId()).setPlayerInstance(activeChar);
		//if(activeChar.isClanLeader())
		//{
		//	if(activeChar.getClan().getHasHideout() != 0 && ClanHallManager.getInstance().getClanHall(activeChar.getClan().getHasHideout()).getNotPaid())
		//		activeChar.sendPacket(new SystemMessage(SystemMessage.THE_CLAN_HALL_FEE_IS_ONE_WEEK_OVERDUE_THEREFORE_THE_CLAN_HALL_OWNERSHIP_HAS_BEEN_REVOKED));
		//}

		int sponsor = activeChar.getSponsor();
		int apprentice = activeChar.getApprentice();
		SystemMessage msg = new SystemMessage(SystemMessage.CLAN_MEMBER_S1_HAS_LOGGED_INTO_GAME).addString(activeChar.getName());
		PledgeShowMemberListUpdate memberUpdate = new PledgeShowMemberListUpdate(activeChar);
		for(L2Player clanMember : clan.getOnlineMembers(activeChar.getName()))
		{
			clanMember.sendPacket(memberUpdate);
			if(clanMember.getObjectId() == sponsor)
				clanMember.sendPacket(new SystemMessage(SystemMessage.S1_YOUR_CLAN_ACADEMYS_APPRENTICE_HAS_LOGGED_IN).addString(activeChar.getName()));
			else if(clanMember.getObjectId() == apprentice)
				clanMember.sendPacket(new SystemMessage(SystemMessage.S1_YOUR_CLAN_ACADEMYS_SPONSOR_HAS_LOGGED_IN).addString(activeChar.getName()));
			else
				clanMember.sendPacket(msg);
		}
	}

	public class WarnUserTakeBreak implements Runnable
	{
		L2Player Char = null;

		public WarnUserTakeBreak(L2Player player)
		{
			Char = player;
		}

		public void run()
		{
			if(Char.isOnline())
				Char.sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_BEEN_PLAYING_FOR_AN_EXTENDED_PERIOD_OF_TIME_PLEASE_CONSIDER_TAKING_A_BREAK));
			else
				_currentFuture.cancel(false);
		}
	}

	private void loadTutorial(L2Player player)
	{
		Quest q = QuestManager.getInstance().getQuest(255);

		if(q == null)
			return;

		String qn = q.getName();
		QuestState qs = player.getQuestState(qn);
		if(qn != null && qs != null)
			q.notifyEvent("UC", qs);
	}

	@Override
	public String getType()
	{
		return _C__03_ENTERWORLD;
	}
}
