package L2jFT.game.network.clientpackets;

import L2jFT.game.LoginServerThread;
import L2jFT.game.model.L2Party;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.entity.SevenSignsFestival;
import L2jFT.game.network.serverpackets.SystemMessage;

public class Logout extends L2GameClientPacket
{
	private static String _C__09_LOGOUT = "[C] 09 Logout";

	@Override
	protected void readImpl()
	{}

	@Override
	protected void runImpl()
	{
		// Dont allow leaving if activeChar is fighting
		L2Player activeChar = getClient().getActiveChar();

		if(activeChar == null)
			return;

		String account = activeChar.getAccountName();

		// Dont allow leaving if player is fighting
		if(activeChar.isInCombat())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_LOGOUT_WHILE_IN_COMBAT));
			activeChar.sendActionFailed();
			return;
		}

		if(activeChar.isFishing())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_DO_ANYTHING_ELSE_WHILE_FISHING));
			activeChar.sendActionFailed();
			return;
		}

		if(activeChar.isBlocked())
		{
			activeChar.sendMessage("Your character is out of control now.", "Сейчас ваш персонаж неуправляем.");
			activeChar.sendActionFailed();
			return;
		}

		// Prevent player from logging out if they are a festival participant
		// and it is in progress, otherwise notify party members that the player
		// is not longer a participant.
		if(activeChar.isFestivalParticipant())
		{
			if(SevenSignsFestival.getInstance().isFestivalInitialized())
			{
				activeChar.sendMessage("You cannot log out while you are a participant in a festival.");
				return;
			}
			L2Party playerParty = activeChar.getParty();

			if(playerParty != null)
				playerParty.broadcastMessageToPartyMembers(activeChar.getName() + " has been removed from the upcoming festival.");
		}

		if(activeChar.isInOlympiadMode())
		{
			activeChar.sendMessage("You can't disconnect while in Olympiad.", "Вы не можете выйти во время Олимпиады.");
			activeChar.sendActionFailed();
			return;
		}

		if(activeChar.inObserverMode())
		{
			activeChar.sendMessage("You can't disconnect while in Observer mode.", "Вы не можете выйти во время просмотра.");
			activeChar.sendActionFailed();
			return;
		}

		activeChar.logout(false, false);
		LoginServerThread.getInstance().sendLogout(account);
	}

	@Override
	public String getType()
	{
		return _C__09_LOGOUT;
	}
}