package L2jFT.game.network.clientpackets;

import java.util.HashMap;
import java.util.logging.Logger;

import L2jFT.game.ai.CtrlIntention;
import L2jFT.game.cache.Msg;
import L2jFT.game.model.L2Effect;
import L2jFT.game.model.L2ManufactureList;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.L2Summon;
import L2jFT.game.model.instances.L2DoorInstance;
import L2jFT.game.model.instances.L2SiegeFlagInstance;
import L2jFT.game.model.instances.L2StaticObjectInstance;
import L2jFT.game.network.serverpackets.ChairSit;
import L2jFT.game.network.serverpackets.RecipeShopManageList;
import L2jFT.game.network.serverpackets.Ride;
import L2jFT.game.network.serverpackets.SystemMessage;

@SuppressWarnings( { "nls", "unqualified-field-access", "boxing" })
public class RequestActionUse extends L2GameClientPacket
{
	private static String _C__45_REQUESTACTIONUSE = "[C] 45 RequestActionUse";
	private static Logger _log = Logger.getLogger(RequestActionUse.class.getName());

	private int _actionId;
	private boolean _ctrlPressed;
	@SuppressWarnings("unused")
	private boolean _shiftPressed;

	/**
	 * packet type id 0x45
	 * format:		cddc
	 */
	@Override
	public void readImpl()
	{
		_actionId = readD();
		_ctrlPressed = readD() == 1;
		_shiftPressed = readC() == 1;
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		boolean usePet;
		switch(_actionId)
		{
			case 22:
			case 23:
			case 16:
			case 17:
			case 19:
			case 32:
			case 36:
			case 39:
			case 41:
			case 42:
			case 43:
			case 44:
			case 45:
			case 46:
			case 47:
			case 48:
			case 52:
			case 53:
			case 54:
			case 1004:
			case 1005:
			case 1006:
			case 1007:
			case 1008:
			case 1009:
			case 1010:
			case 1011:
			case 1012:
			case 1013:
			case 1014:
			case 1015:
			case 1016:
			case 1017:
				usePet = true;
				break;
			default:
				usePet = false;
		}

		// dont do anything if player is dead or confused
		if(!usePet && (activeChar.isOutOfControl() || activeChar.isActionsDisabled()) && !(activeChar.isFakeDeath() && _actionId == 0))
		{
			activeChar.sendActionFailed();
			return;
		}

		L2Object target = activeChar.getTarget();
		L2Summon pet = activeChar.getPet();

		//	не отдавать приказы петам если хозяин слишком далеко 
		if(usePet && (pet == null || !pet.isInRange()))
		{
			activeChar.sendActionFailed();
			return;
		}

		switch(_actionId)
		{
			case 0:
				// На страйдере нельзя садиться
				if(activeChar.isMounted())
				{
					activeChar.sendActionFailed();
					break;
				}
				int distance = (int) activeChar.getDistance(activeChar.getTarget());
				if(target != null && !activeChar.isSitting() && target instanceof L2StaticObjectInstance && ((L2StaticObjectInstance) target).getType() == 1 && distance <= L2StaticObjectInstance.INTERACTION_DISTANCE)
				{
					ChairSit cs = new ChairSit(activeChar, ((L2StaticObjectInstance) target).getStaticObjectId());
					activeChar.sendPacket(cs);
					activeChar.sitDown();
					activeChar.broadcastPacket(cs);
					break;
				}
				if(activeChar.isFakeDeath())
				{
					activeChar.getEffect(L2Effect.EffectType.FAKE_DEATH).exit();
					activeChar.updateEffectIcons();
				}
				else if(activeChar.isSitting())
					activeChar.standUp();
				else
					activeChar.sitDown();
				break;
			case 1:
				if(activeChar.isRunning())
					activeChar.setWalking();
				else
					activeChar.setRunning();
				break;
			case 15:
			case 21:
				if(pet != null)
					pet.setFollowStatus(!pet.getFollowStatus());
				break;
			case 16:
			case 22: //attack?
				if(target == null || pet == null || pet == target || pet.isDead())
				{
					activeChar.sendActionFailed();
					return;
				}

				if(activeChar.isInOlympiadMode() && !activeChar.isOlympiadCompStart())
				{
					activeChar.sendActionFailed();
					return;
				}

				// Sin Eater
				if(pet.getTemplate().getNpcId() == 12564)
					return;

				if(!target.isMonster && (pet.isInZonePeace() || target.isInZonePeace()))
				{
					activeChar.sendPacket(Msg.YOU_CANNOT_ATTACK_THE_TARGET_IN_THE_PEACE_ZONE);
					return;
				}

				if(activeChar.getLevel() + 20 <= pet.getLevel())
				{
					activeChar.sendPacket(Msg.THE_PET_IS_TOO_HIGH_LEVEL_TO_CONTROL);
					return;
				}

				if(!(target instanceof L2DoorInstance) && (pet.getNpcId() == L2Summon.SIEGE_GOLEM_ID || pet.getNpcId() == L2Summon.SIEGE_CANNON_ID || pet.getNpcId() == L2Summon.SWOOP_CANNON_ID))
				{
					activeChar.sendPacket(Msg.INCORRECT_TARGET);
					return;
				}

				pet.getAI().Attack(target, _ctrlPressed);
				break;
			case 17:
			case 23:
				if(pet != null)
					pet.getAI().setIntention(CtrlIntention.AI_INTENTION_IDLE, null, null);
				break;
			case 19:
				if(pet == null)
				{
					activeChar.sendActionFailed();
					return;
				}

				if(pet.isDead())
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.A_DEAD_PET_CANNOT_BE_SENT_BACK));
					activeChar.sendActionFailed();
					return;
				}

				if(pet.isInCombat())
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.A_PET_CANNOT_BE_SENT_BACK_DURING_BATTLE));
					activeChar.sendActionFailed();
					break;
				}

				if(pet.isPet && pet.getCurrentFed() < 0.55 * pet.getMaxFed())
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_RESTORE_HUNGRY_PETS));
					activeChar.sendActionFailed();
					break;
				}

				pet.unSummon();
				break;
			case 38: //mount
				if(pet != null && pet.isMountable() && !activeChar.isMounted())
				{
					if(activeChar.isDead())
						activeChar.sendPacket(new SystemMessage(SystemMessage.A_STRIDER_CANNOT_BE_RIDDEN_WHEN_DEAD));
					else if(pet.isDead())
						activeChar.sendPacket(new SystemMessage(SystemMessage.A_DEAD_STRIDER_CANNOT_BE_RIDDEN));
					else if(pet.isInCombat())
						activeChar.sendPacket(new SystemMessage(SystemMessage.A_STRIDER_IN_BATTLE_CANNOT_BE_RIDDEN));
					else if(activeChar.isInCombat())
						activeChar.sendPacket(new SystemMessage(SystemMessage.A_STRIDER_CANNOT_BE_RIDDEN_WHILE_IN_BATTLE));
					else if(activeChar.isSitting() || activeChar.isMoving)
						activeChar.sendPacket(new SystemMessage(SystemMessage.A_STRIDER_CAN_BE_RIDDEN_ONLY_WHEN_STANDING));
					else if(activeChar.isFishing())
						activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_DO_THAT_WHILE_FISHING));
					else if(activeChar.isCursedWeaponEquipped())
						activeChar.sendPacket(new SystemMessage(SystemMessage.A_STRIDER_CANNOT_BE_RIDDEN_WHILE_IN_BATTLE));
					else if(!pet.isDead() && !activeChar.isMounted())
					{
						Ride mount = new Ride(activeChar, Ride.ACTION_MOUNT, pet.getTemplate().npcId);
						activeChar.broadcastPacket(mount);
						activeChar.setMountType(mount.getMountType());
						pet.unSummon();
					}
				}
				else if(activeChar.isMounted())
				{
					if(activeChar.getMountType() == 2 && !activeChar.checkLandingState()) // Виверна
					{
						activeChar.sendActionFailed();
						activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_ARE_NOT_ALLOWED_TO_DISMOUNT_AT_THIS_LOCATION));
						return;
					}
					activeChar.broadcastPacket(new Ride(activeChar, Ride.ACTION_DISMOUNT, 0));
					activeChar.setMountType(0);
				}
				break;
			case 32: //Wild Hog Cannon - Mode Change
				UseSkill(4230, target);
				break;
			case 36: // Soulless - Toxic Smoke
				UseSkill(4259, target);
				break;
			case 37:
			{
				if(activeChar.isMounted())
				{
					activeChar.sendActionFailed();
					break;
				}

				if(activeChar.isAlikeDead() || activeChar.getSittingTask())
				{
					activeChar.sendActionFailed();
					return;
				}

				activeChar.setInStoreMode(false);
				activeChar.standUp();

				if(activeChar.getCreateList() == null)
					activeChar.setCreateList(new L2ManufactureList());

				activeChar.sendPacket(new RecipeShopManageList(activeChar, true));
				break;
			}
			case 39: // Soulless - Parasite Burst
				UseSkill(4138, target);
				break;
			case 41: //Wild Hog Cannon - Attack
				UseSkill(4230, target);
				break;
			case 42: // Kai the Cat - Self Damage Shield
				UseSkill(4378, activeChar);
				break;
			case 43: // Unicorn Merrow - Hydro Screw
				UseSkill(4137, target);
				break;
			case 44: // Big Boom - Boom Attack
				UseSkill(4139, target);
				break;
			case 45: // Unicorn Boxer - Master Recharge
				UseSkill(4025, activeChar);
				break;
			case 46: // Mew the Cat - Mega Storm Strike
				UseSkill(4261, target);
				break;
			case 47: // Silhouette - Steal Blood
				UseSkill(4260, target);
				break;
			case 48: // Mechanic Golem - Mech. Cannon
				UseSkill(4068, target);
				break;
			case 51:
			{
				if(activeChar.isMounted())
				{
					activeChar.sendActionFailed();
					break;
				}

				if(activeChar.isAlikeDead() || activeChar.getSittingTask())
				{
					activeChar.sendActionFailed();
					return;
				}

				activeChar.setInStoreMode(false);
				activeChar.standUp();

				if(activeChar.getCreateList() == null)
					activeChar.setCreateList(new L2ManufactureList());

				activeChar.sendPacket(new RecipeShopManageList(activeChar, false));
				break;
			}
			case 52: // unsummon
				if(pet != null)
				{
					if(pet.isInCombat())
					{
						activeChar.sendPacket(new SystemMessage(SystemMessage.A_PET_CANNOT_BE_SENT_BACK_DURING_BATTLE));
						activeChar.sendActionFailed();
						break;
					}
					pet.unSummon();
				}
				break;
			case 53: // move to target
				if(target != null && pet != null && pet != target && !pet.isMovementDisabled())
				{
					pet.setFollowStatus(false);
					pet.moveToLocation(target.getX(), target.getY(), target.getZ(), 100, true);
				}
				break;
			case 54: // move to target hatch/strider
				if(target != null && pet != null && pet != target && !pet.isMovementDisabled())
				{
					pet.setFollowStatus(false);
					pet.moveToLocation(target.getX(), target.getY(), target.getZ(), 100, true);
				}
				break;
			case 96: // Quit Party Command Channel?
				_log.info("96 Accessed");
				break;
			case 97: // Request Party Command Channel Info?
				_log.info("97 Accessed");
				break;
			case 1000: // Siege Golem - Siege Hammer
				if(target instanceof L2DoorInstance)
					UseSkill(4079, target);
				break;
			case 1001: // Sin Eater - Ultimate Bombastic Buster
				break;
			case 1003: // Wind Hatchling/Strider - Wild Stun
				UseSkill(4710, target);
				break;
			case 1004: // Wind Hatchling/Strider - Wild Defense
				UseSkill(4711, activeChar);
				break;
			case 1005: // Star Hatchling/Strider - Bright Burst
				UseSkill(4712, target);
				break;
			case 1006: // Star Hatchling/Strider - Bright Heal
				UseSkill(4713, activeChar);
				break;
			case 1007: // Cat Queen - Blessing of Queen
				UseSkill(4699, activeChar);
				break;
			case 1008: // Cat Queen - Gift of Queen
				UseSkill(4700, activeChar);
				break;
			case 1009: // Cat Queen - Cure of Queen
				UseSkill(4701, target);
				break;
			case 1010: // Unicorn Seraphim - Blessing of Seraphim
				UseSkill(4702, activeChar);
				break;
			case 1011: // Unicorn Seraphim - Gift of Seraphim
				UseSkill(4703, activeChar);
				break;
			case 1012: // Unicorn Seraphim - Cure of Seraphim
				UseSkill(4704, target);
				break;
			case 1013: // Nightshade - Curse of Shade
				UseSkill(4705, target);
				break;
			case 1014: // Nightshade - Mass Curse of Shade
				UseSkill(4706, activeChar);
				break;
			case 1015: // Nightshade - Shade Sacrifice
				UseSkill(4707, target);
				break;
			case 1016: // Cursed Man - Cursed Blow
				UseSkill(4709, target);
				break;
			case 1017: // Cursed Man - Cursed Strike/Stun
				UseSkill(4708, target);
				break;
			case 1031: // Feline King - Slash
				UseSkill(5135, target);
				break;
			case 1032: // Feline King - Spinning Slash
				UseSkill(5136, target);
				break;
			case 1033: // Feline King - Grip of the Cat
				UseSkill(5137, target);
				break;
			case 1034: // Magnus the Unicorn - Whiplash
				UseSkill(5138, target);
				break;
			case 1035: // Magnus the Unicorn - Tridal Wave
				UseSkill(5139, target);
				break;
			case 1036: // Spectral Lord - Corpse Kaboom
				UseSkill(5142, target);
				break;
			case 1037: // Spectral Lord - Dicing Death
				UseSkill(5141, target);
				break;
			case 1038: // Spectral Lord - Force Curse
				UseSkill(5140, target);
				break;
			case 1039: // Swoop Cannon - Cannon Fodder (не может атаковать двери и флаги)
				if(!(target instanceof L2DoorInstance) && !(target instanceof L2SiegeFlagInstance))
					UseSkill(5110, target);
				break;
			case 1040: // Swoop Cannon - Big Bang (не может атаковать двери и флаги)
				if(!(target instanceof L2DoorInstance) && !(target instanceof L2SiegeFlagInstance))
					UseSkill(5111, target);
				break;
			default:
				_log.warning("unhandled action type " + _actionId);
		}
		activeChar.sendActionFailed();
	}

	private void UseSkill(int skillId, L2Object target)
	{
		L2Player activeChar = getClient().getActiveChar();
		L2Summon pet = activeChar.getPet();

		if(target == null)
		{
			activeChar.sendActionFailed();
			return;
		}

		if(!target.isCharacter)
		{
			activeChar.sendActionFailed();
			return;
		}

		if(pet == null)
		{
			activeChar.sendActionFailed();
			return;
		}

		HashMap<Short, L2Skill> _skills = pet.getTemplate().getSkills();
		if(_skills.size() == 0)
		{
			activeChar.sendActionFailed();
			return;
		}

		L2Skill skill = _skills.get(new Short((short) skillId));
		if(skill == null)
		{
			activeChar.sendActionFailed();
			return;
		}

		if(activeChar.getLevel() + 20 <= pet.getLevel())
		{
			activeChar.sendPacket(Msg.THE_PET_IS_TOO_HIGH_LEVEL_TO_CONTROL);
			return;
		}

		if(skill.isOffensive() && (target == activeChar || target == pet))
		{
			activeChar.sendPacket(Msg.TARGET_IS_INCORRECT);
			return;
		}

		pet.setTarget(target);
		if(skill.checkCondition(pet, skill.getAimingTarget(pet), _ctrlPressed, _shiftPressed, true))
			pet.getAI().Cast(skill, skill.getAimingTarget(pet), _ctrlPressed, _shiftPressed);
		else
			activeChar.sendActionFailed();
	}

	@Override
	public String getType()
	{
		return _C__45_REQUESTACTIONUSE;
	}
}
