package L2jFT.game.network.clientpackets;

import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2ShortCut;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.L2SkillLearn;
import L2jFT.game.model.L2World;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.model.instances.L2VillageMasterInstance;
import L2jFT.game.network.serverpackets.AquireSkillList;
import L2jFT.game.network.serverpackets.ShortCutRegister;
import L2jFT.game.network.serverpackets.SkillList;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.SkillSpellbookTable;
import L2jFT.game.tables.SkillTable;
import L2jFT.game.tables.SkillTreeTable;
import L2jFT.util.Util;

public class RequestAquireSkill extends L2GameClientPacket
{
	private static String _C__6C_REQUESTACQUIRESKILL = "[C] 6C RequestAquireSkill";
	private short _id;
	private byte _level;
	private int _skillType;

	@Override
	public void readImpl()
	{
		_id = (short) readD();
		_level = (byte) readD();
		_skillType = readD();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;
		L2NpcInstance trainer = activeChar.getLastNpc();
		if((trainer == null || activeChar.getDistance(trainer.getX(), trainer.getY()) > L2Character.INTERACTION_DISTANCE) && !activeChar.isGM())
			return;

		activeChar.setSkillLearningClassId(activeChar.getClassId());

		L2Skill skill = SkillTable.getInstance().getInfo(_id, _level);

		if(activeChar.getSkillLevel(_id) >= _level)
			return; // already knows the skill with this level

		if(_level > 1 && activeChar.getSkillLevel(_id) != _level - 1)
		{
			Util.handleIllegalPlayerAction(activeChar, "RequestAquireSkill[58]", "tried to increase skill " + _id + " level to " + _level + " while having it's level " + activeChar.getSkillLevel(_id), 2);
			return;
		}

		if(!(skill.isCommon() || SkillTreeTable.getInstance().isSkillPossible(activeChar, _id, _level)))
		{
			Util.handleIllegalPlayerAction(activeChar, "RequestAquireSkill[64]", "tried to learn skill " + _id + " while on class " + activeChar.getActiveClass(), 2);
			return;
		}

		L2SkillLearn SkillLearn = SkillTreeTable.getSkillLearn(_id, _level, activeChar.getClassId());

		if(SkillLearn.clan)
		{
			if(activeChar.getClan() == null)
				return;
			learnClanSkill(skill, activeChar.getClan());
		}
		else
		{
			int _requiredSp = SkillTreeTable.getInstance().getSkillCost(activeChar, skill);

			if(activeChar.getSp() >= _requiredSp || SkillLearn.common)
			{
				Short spb_id = SkillSpellbookTable._skillSpellbooks.get(SkillSpellbookTable.hashCode(new short[] {
						skill.getId(), skill.getLevel() }));

				if(spb_id != null)
				{
					L2ItemInstance spb = activeChar.getInventory().findItemByItemId(spb_id.shortValue());
					if(spb == null || spb.getCount() < SkillLearn.itemCount)
					{
						activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_ENOUGH_ITEMS_TO_LEARN_SKILLS));
						return;
					}
					L2ItemInstance ri = activeChar.getInventory().destroyItem(spb, SkillLearn.itemCount, true, true);

					if(ri.getItemId() != 57)
						activeChar.sendPacket(new SystemMessage(SystemMessage.S1_HAS_DISAPPEARED).addItemName(ri.getItemId()));
					else
						activeChar.sendPacket(new SystemMessage(SystemMessage.S1_ADENA_DISAPPEARED).addNumber(SkillLearn.itemCount));
					if(ri.getLastChange() == L2ItemInstance.REMOVED)
						L2World.removeObject(ri);
				}
				activeChar.addSkill(skill, true);
				if(!SkillLearn.common)
					activeChar.setSp(activeChar.getSp() - _requiredSp);
				activeChar.updateStats();
				activeChar.sendChanges();

				//update all the shortcuts to this skill
				if(_level > 1)
					for(L2ShortCut sc : activeChar.getAllShortCuts())
						if(sc.id == _id && sc.type == L2ShortCut.TYPE_SKILL)
						{
							L2ShortCut newsc = new L2ShortCut(sc.slot, sc.page, sc.type, sc.id, _level);
							activeChar.sendPacket(new ShortCutRegister(newsc));
							activeChar.registerShortCut(newsc);
						}
			}
			else
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_ENOUGH_SP_TO_LEARN_SKILLS));
				return;
			}

			activeChar.sendPacket(new SkillList(activeChar));

			if(trainer != null)
				if(_skillType == AquireSkillList.USUAL)
					trainer.showSkillList(activeChar);
				else if(_skillType == AquireSkillList.FISHING)
					trainer.showFishingSkillList(activeChar);
				else if(_skillType == AquireSkillList.CLAN)
					trainer.showClanSkillList(activeChar);
		}
	}

	private void learnClanSkill(L2Skill skill, L2Clan clan)
	{
		L2Player player = getClient().getActiveChar();
		if(player == null)
			return;
		L2NpcInstance trainer = player.getLastNpc();
		if(trainer == null)
			return;
		if(!player.isClanLeader())
		{
			player.sendPacket(new SystemMessage(SystemMessage.ONLY_THE_CLAN_LEADER_IS_ENABLED));
			return;
		}
		//L2SkillLearn SkillLearn = SkillTreeTable.getSkillLearn(_id, _level, null);
		int _requiredRep = SkillTreeTable.getInstance().getSkillRepCost(clan, skill);
		short itemId = SkillSpellbookTable._skillSpellbooks.get(SkillSpellbookTable.hashCode(new short[] {
				skill.getId(), skill.getLevel() }));
		if(skill.getMinPledgeClass() <= clan.getLevel() && clan.getReputationScore() >= _requiredRep)
		{
			if(itemId > 0)
			{
				L2ItemInstance spb = player.getInventory().findItemByItemId(itemId);
				if(spb == null)
				{
					// Haven't spellbook
					player.sendPacket(new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_ENOUGH_ITEMS_TO_LEARN_SKILLS));
					return;
				}
				L2ItemInstance ri = player.getInventory().destroyItem(spb, 1, true, true);
				player.sendPacket(new SystemMessage(SystemMessage.S1_HAS_DISAPPEARED).addItemName(ri.getItemId()));
			}
			clan.incReputation(-_requiredRep, false);
			clan.addNewSkill(skill);
			player.sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_EARNED_S1).addSkillName(_id, (short) _level));

			((L2VillageMasterInstance) trainer).showClanSkillWindow(player); //Maybe we shoud add a check here...
		}
		else
		{
			player.sendMessage("Your clan doesn't have enough reputation points to learn this skill");
			//sm = null;
			return;
		}

		//update all the shortcuts to this skill
		if(_level > 1)
			for(L2ShortCut sc : player.getAllShortCuts())
				if(sc.id == _id && sc.type == L2ShortCut.TYPE_SKILL)
				{
					L2ShortCut newsc = new L2ShortCut(sc.slot, sc.page, sc.type, sc.id, _level);
					player.sendPacket(new ShortCutRegister(newsc));
					player.registerShortCut(newsc);
				}

		clan.addAndShowSkillsToPlayer(player);
	}

	@Override
	public String getType()
	{
		return _C__6C_REQUESTACQUIRESKILL;
	}
}
