package L2jFT.game.network.clientpackets;

import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.network.serverpackets.InventoryUpdate;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.ItemTable;
import L2jFT.util.Log;

public class RequestCrystallizeItem extends L2GameClientPacket
{
	private static String _C__72_REQUESTDCRYSTALLIZEITEM = "[C] 72 RequestCrystallizeItem";
	//private static Logger _log = Logger.getLogger(RequestCrystallizeItem.class.getName());

	//0, d, c, b, a, s ... 0 is for none
	private static short[] _crystalId = { 0, 1458, 1459, 1460, 1461, 1462 };

	private int _objectId;

	@Override
	public void readImpl()
	{
		_objectId = readD();
		readD();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();

		if(activeChar == null)
			return;

		if(activeChar.getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE)
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.WHILE_OPERATING_A_PRIVATE_STORE_OR_WORKSHOP_YOU_CANNOT_DISCARD_DESTROY_OR_TRADE_AN_ITEM));
			activeChar.sendActionFailed();
			return;
		}

		L2ItemInstance item = activeChar.getInventory().getItemByObjectId(_objectId);

		if(item == null || !item.canBeCrystallized(activeChar) || item.isWear())
		{
			activeChar.sendActionFailed();
			return;
		}

		// unequip if needed
		if(item.isEquipped())
		{
			activeChar.getInventory().unEquipItem(item);
			activeChar.broadcastUserInfo();
		}

		activeChar.getInventory().destroyItem(item, 1, true, true);

		// add crystals
		int crystalAmount = item.getItem().getCrystalCount();
		short crystalId = _crystalId[item.getItem().getCrystalType()];

		boolean hasItem = activeChar.getInventory().getItemByItemId(crystalId) != null;

		L2ItemInstance createditem = ItemTable.getInstance().createItem(crystalId, activeChar.getObjectId(), item.getItem().getItemId(), "RequestCrystallizeItem[83]");
		createditem.setCount(crystalAmount);
		L2ItemInstance addedItem = activeChar.getInventory().addItem(createditem);
		activeChar.sendPacket(new SystemMessage(SystemMessage.THE_ITEM_HAS_BEEN_SUCCESSFULLY_CRYSTALLIZED));
		activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_OBTAINED_S2_S1).addItemName(crystalId).addNumber(crystalAmount));

		Log.LogItem(activeChar, Log.CrystalizeItem, item);
		Log.LogItem(activeChar, Log.Sys_GetItem, addedItem);

		InventoryUpdate iu = new InventoryUpdate();
		iu.addRemovedItem(item);
		if(hasItem)
			iu.addModifiedItem(activeChar.getInventory().getItemByItemId(crystalId));
		else
			iu.addNewItem(createditem);
		activeChar.sendPacket(iu);

		activeChar.updateStats();
		
		L2World.removeObject(item);
	}

	@Override
	public String getType()
	{
		return _C__72_REQUESTDCRYSTALLIZEITEM;
	}
}
