package L2jFT.game.network.clientpackets;

import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.network.serverpackets.InventoryUpdate;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.util.Log;
import L2jFT.util.Util;

public class RequestDestroyItem extends L2GameClientPacket
{

	private static String _C__59_REQUESTDESTROYITEM = "[C] 59 RequestDestroyItem";
	//private static Logger _log = Logger.getLogger(RequestDestroyItem.class.getName());

	private int _objectId;
	private int _count;

	/**
	 * packet type id 0x1f
	 *
	 * sample
	 *
	 * 59
	 * 0b 00 00 40		// object id
	 * 01 00 00 00		// count
	 *
	 *
	 * format:		cdd
	 * @param decrypt
	 */
	@Override
	public void readImpl()
	{
		_objectId = readD();
		_count = readD();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		int count = _count;

		L2ItemInstance itemToRemove = activeChar.getInventory().getItemByObjectId(_objectId);

		if(itemToRemove == null || itemToRemove.isWear())
			return;

		if(count < 0)
		{
			Util.handleIllegalPlayerAction(activeChar, "RequestDestroyItem[53]", "Tried to destroy negative " + _count + " count of " + itemToRemove.getItem().getName() + " id " + itemToRemove.getItemId(), 2);
			return;
		}

		if(count < 1)
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_DESTROY_IT_BECAUSE_THE_NUMBER_IS_INCORRECT));
			return;
		}

		if(itemToRemove.isHeroItem())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.HERO_WEAPONS_CANNOT_BE_DESTROYED));
			return;
		}

		if(!itemToRemove.canBeDestroyed(activeChar))
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.THIS_ITEM_CANNOT_BE_DISCARDED));
			return;
		}

		if(activeChar.getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE)
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.WHILE_OPERATING_A_PRIVATE_STORE_OR_WORKSHOP_YOU_CANNOT_DISCARD_DESTROY_OR_TRADE_AN_ITEM));
			return;
		}

		if(activeChar.isFishing())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_DO_THAT_WHILE_FISHING));
			return;
		}

		if(activeChar.getPet() != null && activeChar.getPet().getControlItemId() == itemToRemove.getObjectId())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.THE_PET_HAS_BEEN_SUMMONED_AND_CANNOT_BE_DELETED));
			return;
		}

		if(activeChar.isActionsDisabled())
		{
			activeChar.sendActionFailed();
			return;
		}

		if(_count > itemToRemove.getCount())
			count = itemToRemove.getCount();

		if(itemToRemove.isEquipped())
		{
			L2ItemInstance[] unequipped = activeChar.getInventory().unEquipItemInSlotAndRecord(itemToRemove.getEquipSlot());
			InventoryUpdate iu = new InventoryUpdate();
			for(L2ItemInstance element : unequipped)
				iu.addModifiedItem(element);
			activeChar.sendPacket(iu);
			activeChar.broadcastUserInfo();
		}

		InventoryUpdate iu = new InventoryUpdate();
		L2ItemInstance removedItem = activeChar.getInventory().destroyItem(_objectId, count, true, true);
		iu.addItem(removedItem);
		Log.LogItem(activeChar, Log.DeleteItem, removedItem);

		sendPacket(iu);
		activeChar.sendChanges();

		if(!removedItem.isStackable() || count == 1)
			activeChar.sendPacket(new SystemMessage(SystemMessage.S1_HAS_DISAPPEARED).addItemName(removedItem.getItemId()));
		else
			activeChar.sendPacket(new SystemMessage(SystemMessage.S2_S1_HAS_DISAPPEARED).addItemName(removedItem.getItemId()).addNumber(count));

		L2World.removeObject(removedItem);

		activeChar.updateStats();
	}

	@Override
	public String getType()
	{
		return _C__59_REQUESTDESTROYITEM;
	}
}
