package L2jFT.game.network.clientpackets;

import L2jFT.Config;
import L2jFT.game.model.Inventory;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.PcInventory;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.network.serverpackets.EnchantResult;
import L2jFT.game.network.serverpackets.InventoryUpdate;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.ItemTable;
import L2jFT.game.templates.L2Item;
import L2jFT.util.Log;
import L2jFT.util.Rnd;
import L2jFT.util.Util;

import java.util.logging.Logger;

public class RequestEnchantItem extends L2GameClientPacket
{
	protected static Logger _log = Logger.getLogger(Inventory.class.getName());

	private static String _C__58_REQUESTENCHANTITEM = "[C] 58 RequestEnchantItem";

	private int _objectId;

	@Override
	public void readImpl()
	{
		_objectId = readD();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		if(activeChar.isOutOfControl() || activeChar.isActionsDisabled())
		{
			activeChar.sendActionFailed();
			return;
		}

		PcInventory inventory = activeChar.getInventory();
		L2ItemInstance itemToEnchant = inventory.getItemByObjectId(_objectId);
		L2ItemInstance scroll = activeChar.getEnchantScroll();
		activeChar.setEnchantScroll(null);

		if(itemToEnchant == null || scroll == null)
		{
			activeChar.sendActionFailed();
			return;
		}

		Log.add("Enchant|" + activeChar.getName() + "|Trying to enchant|" + itemToEnchant.getItemId() + "|+" + itemToEnchant.getEnchantLevel() + "|" + itemToEnchant.getObjectId(), "items");

		if(!itemToEnchant.canBeEnchanted(activeChar))
		{
			activeChar.setEnchantScroll(null);
			activeChar.sendPacket(new EnchantResult(0));
			activeChar.sendPacket(new SystemMessage(SystemMessage.INAPPROPRIATE_ENCHANT_CONDITIONS));
			activeChar.sendActionFailed();
			return;
		}
		if(activeChar.getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE)
		{
			activeChar.sendPacket(new EnchantResult(0));
			activeChar.sendPacket(new SystemMessage(SystemMessage.INAPPROPRIATE_ENCHANT_CONDITIONS));
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_PRACTICE_ENCHANTING_WHILE_OPERATING_A_PRIVATE_STORE_OR_PRIVATE_MANUFACTURING_WORKSHOP));
			Util.handleIllegalPlayerAction(activeChar, "RequestEnchantItem[65]", " tries enchant " + itemToEnchant.getItemId() + "  with trade", 0);
			activeChar.sendActionFailed();
			return;
		}

		if(activeChar.getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE || itemToEnchant.isWear() || itemToEnchant.isStackable() || (scroll = inventory.getItemByObjectId(scroll.getObjectId())) == null)
		{
			activeChar.sendPacket(new EnchantResult(itemToEnchant.getEnchantLevel()));
			activeChar.sendActionFailed();
			return;
		}

		short crystalId = itemToEnchant.getEnchantCrystalId(scroll);

		if(crystalId == 0)
		{
			activeChar.sendPacket(new EnchantResult(itemToEnchant.getEnchantLevel()));
			activeChar.sendPacket(new SystemMessage(SystemMessage.INAPPROPRIATE_ENCHANT_CONDITIONS));
			activeChar.sendActionFailed();
			return;
		}

		if(itemToEnchant.getEnchantLevel() >= Config.ENCHANT_MAX)
		{
			activeChar.sendPacket(new EnchantResult(itemToEnchant.getEnchantLevel()));
			activeChar.sendMessage("Enchant failed as max enchant level is set to " + Config.ENCHANT_MAX, "Не удалось заточить, вы достигли максимального значения");
			activeChar.sendActionFailed();
			return;
		}

		// Запрет на заточку чужих вещей, баг может вылезти на серверных лагах
		if(itemToEnchant.getOwnerId() != activeChar.getObjectId())
		{
			activeChar.setEnchantScroll(null);
			activeChar.sendPacket(new EnchantResult(0));
			activeChar.sendPacket(new SystemMessage(SystemMessage.INAPPROPRIATE_ENCHANT_CONDITIONS));
			activeChar.sendActionFailed();
			return;
		}

		L2ItemInstance removedScroll;
		synchronized (inventory)
		{
			removedScroll = inventory.destroyItem(scroll.getObjectId(), 1, false, true);
		}

		if(removedScroll == null)
		{
			activeChar.sendPacket(new EnchantResult(itemToEnchant.getEnchantLevel()));
			Util.handleIllegalPlayerAction(activeChar, "RequestEnchantItem[115]", " tries enchant " + itemToEnchant.getItemId() + " without scrolls", 1);
			activeChar.sendActionFailed();
			return;
		}

		int safeEnchantLevel = itemToEnchant.getItem().getBodyPart() == L2Item.SLOT_FULL_ARMOR ? Config.SAFE_ENCHANT_FULL_BODY : Config.SAFE_ENCHANT_COMMON;
		int chance = itemToEnchant.getEnchantLevel() < safeEnchantLevel ? 100 : itemToEnchant.getCustomType2() == L2Item.TYPE2_WEAPON ? Config.ENCHANT_CHANCE_WEAPON : Config.ENCHANT_CHANCE_ARMOR;
		boolean chanceOk = Rnd.chance(chance);

		InventoryUpdate iu = new InventoryUpdate();

		if(chanceOk)
		{
			if(itemToEnchant.getEnchantLevel() == 0)
				activeChar.sendPacket(new SystemMessage(SystemMessage.S1_HAS_BEEN_SUCCESSFULLY_ENCHANTED).addItemName(itemToEnchant.getItemId()));
			else
			{
				SystemMessage sm = new SystemMessage(SystemMessage._S1_S2_HAS_BEEN_SUCCESSFULLY_ENCHANTED);
				sm.addNumber(itemToEnchant.getEnchantLevel());
				sm.addItemName(itemToEnchant.getItemId());
				activeChar.sendPacket(sm);
			}
			itemToEnchant.setEnchantLevel(itemToEnchant.getEnchantLevel() + 1);
			itemToEnchant.updateDatabase();
			iu.addModifiedItem(itemToEnchant);
			Log.add("Enchant|" + activeChar.getName() + "|Successfully enchanted|" + itemToEnchant.getItemId() + "|to+" + itemToEnchant.getEnchantLevel() + "|" + chance, "items");
			Log.LogItem(activeChar, Log.EnchantItem, itemToEnchant);
		}
		else
		{
			Log.add("Enchant|" + activeChar.getName() + "|Failed to enchant|" + itemToEnchant.getItemId() + "|+" + itemToEnchant.getEnchantLevel() + "|" + chance, "items");

			if(scroll.isBlessedEnchantScroll())
			{
				itemToEnchant.setEnchantLevel(0);
				iu.addModifiedItem(itemToEnchant);
				activeChar.sendPacket(new SystemMessage(SystemMessage.FAILED_IN_BLESSED_ENCHANT_THE_ENCHANT_VALUE_OF_THE_ITEM_BECAME_0));
			}
			else
			{
				if(itemToEnchant.isEquipped())
				{
					L2ItemInstance[] unequipped = activeChar.getInventory().unEquipItemInSlotAndRecord(itemToEnchant.getEquipSlot());
					InventoryUpdate iuue = new InventoryUpdate();
					for(L2ItemInstance element : unequipped)
						iuue.addModifiedItem(element);
					activeChar.sendPacket(iuue);
				}

				if(itemToEnchant.getEnchantLevel() == 0)
					activeChar.sendPacket(new SystemMessage(SystemMessage.THE_ENCHANTMENT_HAS_FAILED_YOUR_S1_HAS_BEEN_CRYSTALLIZED).addItemName(itemToEnchant.getItemId()));
				else
					activeChar.sendPacket(new SystemMessage(SystemMessage.THE_ENCHANTMENT_HAS_FAILED_YOUR__S1_S2_HAS_BEEN_CRYSTALLIZED).addNumber(itemToEnchant.getEnchantLevel()).addItemName(itemToEnchant.getItemId()));

				L2ItemInstance destroyedItem = inventory.destroyItem(itemToEnchant.getObjectId(), 1, false, true);
				if(destroyedItem == null)
				{
					_log.warning("failed to destroy " + itemToEnchant.getObjectId() + " after unsuccessful enchant attempt by char " + activeChar.getName());
					activeChar.sendActionFailed();
					return;
				}
				iu.addItem(destroyedItem);

				L2ItemInstance crystalsToAdd = ItemTable.getInstance().createItem(crystalId, activeChar.getObjectId(), itemToEnchant.getItemId(), "RequestEnchantItem[179]");

				int count = (int) (itemToEnchant.getItem().getCrystalCount() * 0.67);
				if(count < 1)
					count = 1;
				crystalsToAdd.setCount(count);

				L2ItemInstance addedCrystals = activeChar.getInventory().addItem(crystalsToAdd);

				Log.LogItem(activeChar, Log.EnchantItemFail, itemToEnchant);
				Log.LogItem(activeChar, Log.Sys_GetItem, crystalsToAdd);

				iu.addRemovedItem(itemToEnchant);
				iu.addItem(addedCrystals);
				activeChar.sendPacket(new SystemMessage(SystemMessage.EARNED_S2_S1_s).addItemName(crystalsToAdd.getItemId()).addNumber(count));
				activeChar.refreshExpertisePenalty();
				activeChar.refreshOverloaded();
			}
		}

		activeChar.setEnchantScroll(null);

		iu.addRemovedItem(removedScroll);
		activeChar.sendPacket(iu);
		activeChar.sendChanges();
		activeChar.sendPacket(new EnchantResult(itemToEnchant.getEnchantLevel()));
		activeChar.broadcastUserInfo();
	}

	@Override
	public String getType()
	{
		return _C__58_REQUESTENCHANTITEM;
	}
}