package L2jFT.game.network.clientpackets;

import L2jFT.util.Rnd;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2ShortCut;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.base.Experience;
import L2jFT.game.model.base.L2EnchantSkillLearn;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.network.serverpackets.ShortCutRegister;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.SkillTable;
import L2jFT.game.tables.SkillTreeTable;

/**
 * Format chdd
 * c: (id) 0xD0
 * h: (subid) 0x06
 * d: skill id
 * d: skill lvl
 */
public class RequestExEnchantSkill extends L2GameClientPacket
{
	private static String _C__D0_07_REQUESTEXENCHANTSKILL = "[C] D0:07 RequestExEnchantSkill";
	private int _skillID;
	private int _skillLvl;

	/**
	 * @param buf
	 * @param client
	 */
	@Override
	public void readImpl()
	{
		_skillID = readD();
		_skillLvl = readD();
	}

	@Override
	public void runImpl()
	{
		L2Player player = getClient().getActiveChar();
		if(player == null)
			return;

		L2NpcInstance trainer = player.getLastNpc();
		if(trainer == null)
			return;

		if(player.getDistance(trainer) > L2Character.INTERACTION_DISTANCE && !player.isGM())
			return;

		if(player.getSkillLevel((short) _skillID) >= _skillLvl)// already knows the skill with this level
			return;

		if(player.getClassId().getId() < 88) // requires to have 3rd class quest completed
			return;

		if(player.getLevel() < 76)
			return;

		L2EnchantSkillLearn sl = SkillTreeTable.getSkillEnchant((short) _skillID, (short) _skillLvl);
		if(sl == null)
			return; // cheater?
		L2Skill skill = SkillTable.getInstance().getInfo(_skillID, _skillLvl - 100 + sl.getBaseLevel());

		int _requiredSp = sl.getSpCost();
		long _requiredExp = sl.getExpCost();
		int _rate = sl.getRate(player);
		int _baseLvl = sl.getBaseLevel();

		if(player.getSp() >= _requiredSp)
		{
			if(player.getExp() >= _requiredExp)
			{
				if(_skillLvl == 101 || _skillLvl == 141) // only first lvl requires book 
				{
					int spbId = 6622;

					L2ItemInstance spb = player.getInventory().getItemByItemId(spbId);

					if(spb == null)// Haven't spellbook
					{
						player.sendPacket(new SystemMessage(SystemMessage.ITEMS_REQUIRED_FOR_SKILL_ENCHANT_ARE_INSUFFICIENT));
						return;
					}
					// ok
					player.getInventory().destroyItem(spb, 1, true, true);
				}
			}
			else
			{
				SystemMessage sm = new SystemMessage(SystemMessage.EXP_REQUIRED_FOR_SKILL_ENCHANT_IS_INSUFFICIENT);
				player.sendPacket(sm);
				return;
			}
		}
		else
		{
			SystemMessage sm = new SystemMessage(SystemMessage.SP_REQUIRED_FOR_SKILL_ENCHANT_IS_INSUFFICIENT);
			player.sendPacket(sm);
			return;
		}

		if(Rnd.chance(_rate))
		{
			player.addSkill(skill, true);

			player.setSp(player.getSp() - _requiredSp);
			player.setExp(player.getExp() - _requiredExp);
			if(player.getExp() < Experience.LEVEL[player.getLevel()])
				player.decreaseLevel();
			player.updateStats();

			SystemMessage ep = new SystemMessage(SystemMessage.EXPERIENCE_HAS_DECREASED_BY_S1);
			ep.addNumber((int) _requiredExp);
			sendPacket(ep);

			SystemMessage sp = new SystemMessage(SystemMessage.SP_HAS_DECREASED_BY_S1);
			sp.addNumber(_requiredSp);
			sendPacket(sp);

			SystemMessage sm = new SystemMessage(SystemMessage.SUCCEEDED_IN_ENCHANTING_SKILL_S1);
			sm.addSkillName((short) _skillID, (short) _skillLvl);
			player.sendPacket(sm);
		}
		else
		{
			if(skill.getDisplayLevel() > 100)
			{
				_skillLvl = _baseLvl;
				player.addSkill(SkillTable.getInstance().getInfo(_skillID, _skillLvl), true);
			}
			SystemMessage sm = new SystemMessage(SystemMessage.FAILED_IN_ENCHANTING_SKILL_S1);
			sm.addSkillName((short) _skillID, (short) _skillLvl);
			player.sendPacket(sm);
		}
		trainer.showEnchantSkillList(player, player.getClassId());

		// update all the shortcuts to this skill
		for(L2ShortCut sc : player.getAllShortCuts())
			if(sc.id == _skillID && sc.type == L2ShortCut.TYPE_SKILL)
			{
				L2ShortCut newsc = new L2ShortCut(sc.slot, sc.page, sc.type, sc.id, _skillLvl);
				player.sendPacket(new ShortCutRegister(newsc));
				player.registerShortCut(newsc);
			}
	}

	@Override
	public String getType()
	{
		return _C__D0_07_REQUESTEXENCHANTSKILL;
	}
}