package L2jFT.game.network.clientpackets;

import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.base.L2EnchantSkillLearn;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.network.serverpackets.ExEnchantSkillInfo;
import L2jFT.game.tables.SkillTable;
import L2jFT.game.tables.SkillTreeTable;

/**
 * Format chdd
 * c: (id) 0xD0
 * h: (subid) 0x06
 * d: skill id
 * d: skill lvl
 */
public class RequestExEnchantSkillInfo extends L2GameClientPacket
{
	private static String _C__D0_06_REQUESTEXENCHANTSKILLINFO = "[C] D0:06 RequestExEnchantSkillInfo";
	private int _skillID;
	private int _skillLvl;

	/**
	 * @param buf
	 * @param client
	 */
	@Override
	public void readImpl()
	{
		_skillID = readD();
		_skillLvl = readD();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();

		if(activeChar == null)
			return;

		if(activeChar.getLevel() < 76)
			return;

		L2NpcInstance trainer = activeChar.getLastNpc();

		if(trainer == null || activeChar.getDistance(trainer) > L2Character.INTERACTION_DISTANCE && !activeChar.isGM())
			return;

		L2EnchantSkillLearn sl = SkillTreeTable.getSkillEnchant((short) _skillID, (short) _skillLvl);
		if(sl == null)
			return; // cheater?
		L2Skill skill = SkillTable.getInstance().getInfo(_skillID, _skillLvl - 100 + sl.getBaseLevel());

		if(skill == null || skill.getId() != _skillID)
		{
			activeChar.sendMessage("This skill doesn't yet have enchant info in Datapack");
			return;
		}

		if(!(trainer.getTemplate().canTeach(activeChar.getClassId()) || trainer.getTemplate().canTeach(activeChar.getClassId().getParent())))
		{
			activeChar.sendMessage("Wrong teacher");
			return; // cheater
		}

		int requiredSp = sl.getSpCost();
		long requiredExp = sl.getExpCost();
		int rate = sl.getRate(activeChar);
		ExEnchantSkillInfo asi = new ExEnchantSkillInfo(_skillID, _skillLvl, requiredSp, requiredExp, rate);

		if(skill.getLevel() == 101 || skill.getLevel() == 141) // only first lvl requires book
		{
			int spbId = 6622;
			asi.addRequirement(4, spbId, 1, 0);
		}
		sendPacket(asi);
	}

	@Override
	public String getType()
	{
		return _C__D0_06_REQUESTEXENCHANTSKILLINFO;
	}
}