package L2jFT.game.network.clientpackets;

import L2jFT.database.DatabaseUtils;
import L2jFT.database.FiltredPreparedStatement;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.database.ThreadConnection;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;
import L2jFT.game.network.serverpackets.Friend;
import L2jFT.game.network.serverpackets.SystemMessage;

import java.sql.ResultSet;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * This class ...
 *
 * @version $Revision: 1.3.4.2 $ $Date: 2005/03/27 15:29:30 $
 */
public class RequestFriendDel extends L2GameClientPacket
{

	private static String _C__61_REQUESTFRIENDDEL = "[C] 61 RequestFriendDel";
	private static Logger _log = Logger.getLogger(RequestFriendDel.class.getName());

	private String _name;

	@Override
	public void readImpl()
	{
		_name = readS();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			L2Player friend = L2World.getPlayer(_name);
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("select friend_id from character_friends where char_id=? and friend_name=?");
			statement.setInt(1, activeChar.getObjectId());
			statement.setString(2, _name);
			rset = statement.executeQuery();
			if(!rset.next())
			{
				//Player is not in your friendlist
				activeChar.sendPacket(new SystemMessage(SystemMessage.S1_IS_NOT_ON_YOUR_FRIEND_LIST).addString(_name));
				return;
			}

			int targetId = rset.getInt("friend_id");
			DatabaseUtils.closeDatabaseSR(statement, rset);
			rset = null;

			//delete for you
			statement = con.prepareStatement("delete from character_friends where char_id=? and friend_id=?");
			statement.setInt(1, activeChar.getObjectId());
			statement.setInt(2, targetId);
			statement.execute();
			DatabaseUtils.closeStatement(statement);

			//Delete for you friend
			statement = con.prepareStatement("delete from character_friends where char_id=? and friend_id=?");
			statement.setInt(1, targetId);
			statement.setInt(2, activeChar.getObjectId());
			statement.execute();

			//Player deleted from your friendlist
			activeChar.sendPacket(new SystemMessage(SystemMessage.S1_HAS_BEEN_REMOVED_FROM_YOUR_FRIEND_LIST).addString(_name));
			activeChar.sendPacket(new Friend(_name, false, friend != null, targetId)); //Офф посылает 0xFB Friend, хотя тут нету разници что именно посылать
			if(friend != null)
			{
				friend.sendPacket(new SystemMessage(SystemMessage.S1__HAS_BEEN_DELETED_FROM_YOUR_FRIENDS_LIST).addString(activeChar.getName()));
				friend.sendPacket(new Friend(activeChar, false)); //Офф посылает 0xFB Friend, хотя тут нету разници что именно посылать
			}
		}
		catch(Exception e)
		{
			_log.log(Level.WARNING, "could not del friend objectid: ", e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}

	}

	@Override
	public String getType()
	{
		return _C__61_REQUESTFRIENDDEL;
	}
}
