package L2jFT.game.network.clientpackets;

import L2jFT.game.model.L2Player;
import L2jFT.game.model.PcInventory;
import L2jFT.game.model.PetInventory;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.instances.L2PetInstance;
import L2jFT.game.network.serverpackets.InventoryUpdate;
import L2jFT.game.network.serverpackets.PetInventoryUpdate;
import L2jFT.game.network.serverpackets.PetItemList;
import L2jFT.util.Log;

import java.util.logging.Logger;

public class RequestGetItemFromPet extends L2GameClientPacket
{
	private static String REQUESTGETITEMFROMPET__C__8C = "[C] 8C RequestGetItemFromPet";
	private static Logger _log = Logger.getLogger(RequestGetItemFromPet.class.getName());

	private int _objectId;
	private int _amount;
	@SuppressWarnings("unused")
	private int _unknown;

	@Override
	public void readImpl()
	{
		_objectId = readD();
		_amount = readD();
		_unknown = readD();// = 0 for most trades
	}

	@Override
	public void runImpl()
	{
		if(_amount < 1)
			return;
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		L2PetInstance pet = (L2PetInstance) activeChar.getPet();
		if(pet == null)
		{
			activeChar.sendActionFailed();
			return;
		}

		PetInventory petInventory = pet.getInventory();
		PcInventory playerInventory = activeChar.getInventory();

		L2ItemInstance petItem = petInventory.getItemByObjectId(_objectId);

		if(petItem == null)
		{
			_log.warning("item requested from pet, but its not there.");
			return;
		}

		if(petItem.isEquipped())
		{
			activeChar.sendActionFailed();
			return;
		}

		if(_amount >= petItem.getCount())
		{
			sendPacket(new PetInventoryUpdate().addRemovedItem(petInventory.dropItem(_objectId, petItem.getCount())));
			sendPacket(new InventoryUpdate().addItem(playerInventory.addItem(petItem)));
		}
		else
		{
			L2ItemInstance newPlayerItem = petInventory.dropItem(_objectId, _amount);
			sendPacket(new PetInventoryUpdate().addModifiedItem(petItem));
			sendPacket(new InventoryUpdate().addItem(playerInventory.addItem(newPlayerItem)));
		}

		activeChar.sendChanges();

		sendPacket(new PetItemList(pet));
		pet.broadcastPetInfo();

		Log.LogItem(activeChar, activeChar.getPet(), Log.GetItemFromPet, petItem);
	}

	@Override
	public String getType()
	{
		return REQUESTGETITEMFROMPET__C__8C;
	}
}