package L2jFT.game.network.clientpackets;

import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Effect;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.L2Skill.NextAction;
import L2jFT.game.tables.SkillTable;

import jsr166x.ConcurrentSkipListSet;

/**
 * This class ...
 *
 * @version $Revision: 1.7.2.1.2.3 $ $Date: 2005/03/27 15:29:30 $
 */
public class RequestMagicSkillUse extends L2GameClientPacket
{
	private static String _C__2F_REQUESTMAGICSKILLUSE = "[C] 2F RequestMagicSkillUse";
	//private static Logger _log = Logger.getLogger(RequestMagicSkillUse.class.getName());

	private short _magicId;
	private boolean _ctrlPressed;
	private boolean _shiftPressed;

	/**
	 * packet type id 0x2f
	 * format:		cddc
	 * @param rawPacket
	 */
	@Override
	public void readImpl()
	{
		_magicId = (short) readD();
		_ctrlPressed = readD() != 0;
		_shiftPressed = readC() != 0;
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();

		if(activeChar == null)
			return;

		if(System.currentTimeMillis() - activeChar.getLastPacket() < 100)
		{
			activeChar.sendActionFailed();
			return;
		}

		activeChar.setLastPacket();

		if(activeChar.isOutOfControl())
		{
			activeChar.sendActionFailed();
			return;
		}

		L2Skill skill = SkillTable.getInstance().getInfo(_magicId, activeChar.getSkillLevel(_magicId));
		if(skill != null)
		{
			if(skill.isPassive() || skill.isOnAttack())
				return;
			
			if(skill.isToggle())
			{
				ConcurrentSkipListSet<L2Effect> effects = activeChar.getEffect(skill);
				if(effects != null && effects.size() > 0)
				{
					for(L2Effect effect : effects)
						if(effect != null)
							effect.exit();
					activeChar.sendActionFailed();
					return;
				}
			}

			L2Character target = skill.getAimingTarget(activeChar);

			if(skill.checkCondition(activeChar, target, _ctrlPressed, _shiftPressed, true))
				activeChar.getAI().Cast(skill, target, _ctrlPressed, _shiftPressed);
			else if(target != null && skill.getNextAction() == NextAction.ATTACK && !activeChar.equals(target) && (_ctrlPressed ? target.isAttackable() : target.isAutoAttackable(activeChar)))
				activeChar.getAI().Attack(target, _ctrlPressed);
			else
				activeChar.sendActionFailed();
		}
		else
			activeChar.sendActionFailed();
	}

	@Override
	public String getType()
	{
		return _C__2F_REQUESTMAGICSKILLUSE;
	}
}
