package L2jFT.game.network.clientpackets;

import L2jFT.game.model.L2Macro;
import L2jFT.game.model.L2Macro.L2MacroCmd;

public class RequestMakeMacro extends L2GameClientPacket
{
	private L2Macro _macro;
	private static String _C__C1_REQUESTMAKEMACRO = "[C] C1 RequestMakeMacro";

	/**
	 * packet type id 0xc1
	 *
	 * sample
	 *
	 * c1
	 * d // id
	 * S // macro name
	 * S // unknown  desc
	 * S // unknown  acronym
	 * c // icon
	 * c // count
	 *
	 * c // entry
	 * c // type
	 * d // skill id
	 * c // shortcut id
	 * S // command name
	 *
	 * format:		cdSSScc (ccdcS)
	 * @param decrypt
	 */
	@Override
	public void readImpl()
	{
		int _id = readD();
		String _name = readS();
		String _desc = readS();
		String _acronym = readS();
		int _icon = readC();
		int _count = readC();
		if(_count > 12)
			_count = 12;
		L2MacroCmd[] commands = new L2MacroCmd[_count];
		for(int i = 0; i < _count; i++)
		{
			int entry = readC();
			int type = readC(); // 1 = skill, 3 = action, 4 = shortcut
			int d1 = readD(); // skill or page number for shortcuts
			int d2 = readC();
			String command = readS();
			commands[i] = new L2MacroCmd(entry, type, d1, d2, command);
		}
		_macro = new L2Macro(_id, _icon, _name, _desc, _acronym, commands);
	}

	@Override
	public void runImpl()
	{
		getClient().getActiveChar().registerMacro(_macro);
	}

	/* (non-Javadoc)
	 * @see L2jFT.game.network.clientpackets.ClientBasePacket#getType()
	 */
	@Override
	public String getType()
	{
		return _C__C1_REQUESTMAKEMACRO;
	}
}
