package L2jFT.game.network.clientpackets;

import javolution.util.FastList;
import L2jFT.Config;
import L2jFT.game.cache.Msg;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2Multisell;
import L2jFT.game.model.L2Multisell.MultiSellListContainer;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.PcInventory;
import L2jFT.game.model.base.L2Augmentation;
import L2jFT.game.model.base.MultiSellEntry;
import L2jFT.game.model.base.MultiSellIngredient;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.network.serverpackets.InventoryUpdate;
import L2jFT.game.network.serverpackets.MultiSellList;
import L2jFT.game.network.serverpackets.StatusUpdate;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.ItemTable;
import L2jFT.game.templates.L2Item;
import L2jFT.game.templates.L2Weapon;
import L2jFT.game.templates.L2Weapon.WeaponType;
import L2jFT.util.Log;
import L2jFT.util.Util;

import java.nio.BufferUnderflowException;
import java.util.logging.Logger;

public class RequestMultiSellChoose extends L2GameClientPacket
{
	private static String _C__A7_MULTISELLCHOOSE = "[C] A7 MultiSellChoose";
	private static Logger _log = Logger.getLogger(RequestMultiSellChoose.class.getName());
	private int _listId;
	private int _entryId;
	private long _amount;
	private int _enchant = 0;
	private boolean _keepenchant = false;
	private boolean _notax = false;
	private MultiSellListContainer _list = null;
	private FastList<ItemData> _items = new FastList<ItemData>();

	private class ItemData
	{
		private final int _id;
		private final int _count;
		private final L2ItemInstance _item;

		public ItemData(int id, int count, L2ItemInstance item)
		{
			_id = id;
			_count = count;
			_item = item;
		}

		public int getId()
		{
			return _id;
		}

		public int getCount()
		{
			return _count;
		}

		public L2ItemInstance getItem()
		{
			return _item;
		}

		@Override
		public boolean equals(Object obj)
		{
			if(!(obj instanceof ItemData))
				return false;

			ItemData i = (ItemData) obj;

			return _id == i._id && _count == i._count && _item == i._item;
		}
	}

	@Override
	public void readImpl()
	{
		try
		{
			_listId = readD();
			_entryId = readD();
			_amount = readD();
		}
		catch(BufferUnderflowException e)
		{
			_log.warning(getClient().getLoginName() + " maybe packet cheater!");
		}
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();

		if(!Config.ALT_GAME_KARMA_PLAYER_CAN_SHOP && activeChar.getKarma() > 0 && !activeChar.isGM())
		{
			activeChar.sendActionFailed();
			return;
		}

		if(_listId == 9999)
		{
			_keepenchant = true;
			_notax = true;

			String entry = String.valueOf(_entryId);
			while(entry.length() < 10)
				entry = "0" + entry;
			_enchant = Short.parseShort(entry.substring(0, 2));
			L2Weapon itemtobuy = (L2Weapon) ItemTable.getInstance().getTemplate(Short.parseShort(entry.substring(2, 6)));
			L2Weapon itemtosell = (L2Weapon) ItemTable.getInstance().getTemplate(Short.parseShort(entry.substring(6, 10)));

			if(!(itemtobuy != null && !itemtobuy.isSa() && itemtobuy.getItemId() != itemtosell.getItemId() && itemtobuy.getItemType() == WeaponType.DUAL == (itemtosell.getItemType() == WeaponType.DUAL) && itemtosell.getCrystalType() == itemtobuy.getCrystalType() && itemtobuy.getCrystalType() <= Config.ALT_MAMMOTH_EXCHANGE && itemtosell.getCrystalCount() == itemtobuy.getCrystalCount()))
			{
				Util.handleIllegalPlayerAction(activeChar, "RequestMultiSellChoose[79]", "Tried to change not even items in mammon even items exchange: " + itemtosell + " -> " + itemtobuy, 2);
				activeChar.sendActionFailed();
				return;
			}

			MultiSellEntry mse = new MultiSellEntry(0, itemtobuy.getItemId(), 1, 0);
			mse.addIngredient(new MultiSellIngredient(itemtosell.getItemId(), 1, _enchant));
			doExchange(activeChar, mse);
			return;
		}
		else if(_listId == 9998)
		{
			_keepenchant = false;
			_notax = true;

			String entry = String.valueOf(_entryId);
			while(entry.length() < 10)
				entry = "0" + entry;
			_enchant = Short.parseShort(entry.substring(0, 2));
			L2Weapon itemtobuy = (L2Weapon) ItemTable.getInstance().getTemplate(Short.parseShort(entry.substring(2, 6)));
			L2Weapon itemtosell = (L2Weapon) ItemTable.getInstance().getTemplate(Short.parseShort(entry.substring(6, 10)));

			if(!(itemtobuy != null && !itemtobuy.isSa() && itemtobuy.getItemType() == WeaponType.DUAL == (itemtosell.getItemType() == WeaponType.DUAL) && itemtobuy.getCrystalType() <= Config.ALT_MAMMOTH_UPGRADE))
			{
				Util.handleIllegalPlayerAction(activeChar, "RequestMultiSellChoose[105]", "Tried to change not even items in mammon even items exchange: " + itemtosell + " -> " + itemtobuy, 2);
				activeChar.sendActionFailed();
				return;
			}

			MultiSellEntry mse = new MultiSellEntry(0, itemtobuy.getItemId(), 1, 0);
			mse.addIngredient(new MultiSellIngredient(itemtosell.getItemId(), 1, _enchant));
			mse.addIngredient(new MultiSellIngredient((short) 5575, (int) ((itemtobuy.getReferencePrice() - itemtosell.getReferencePrice()) * 1.2), 0));
			doExchange(activeChar, mse);
			return;
		}
		else if(_listId == 9997)
		{
			_keepenchant = false;
			_notax = false;

			String entry = String.valueOf(_entryId);
			while(entry.length() < 10)
				entry = "0" + entry;
			_enchant = Short.parseShort(entry.substring(0, 2));
			L2Item itemtosell = ItemTable.getInstance().getTemplate(Short.parseShort(entry.substring(6, 10)));
			L2Item crystal = ItemTable.getInstance().getTemplate(itemtosell.getCrystalType() + 1457);

			MultiSellEntry mse = new MultiSellEntry(0, crystal.getItemId(), itemtosell.getCrystalCount(), 0);
			mse.addIngredient(new MultiSellIngredient(itemtosell.getItemId(), 1, _enchant));
			mse.addIngredient(new MultiSellIngredient((short) 57, (int) (itemtosell.getCrystalCount() * crystal.getReferencePrice() * 0.1), 0));
			doExchange(activeChar, mse);
			return;
		}

		if(_amount == 0)
		{
			activeChar.sendActionFailed();
			return;
		}

		if(_amount < 0 || _amount > Integer.MAX_VALUE)
		{
			Util.handleIllegalPlayerAction(activeChar, "RequestMultiSellChoose[147]", "overflow exploit, count to buy: " + _amount + ", list id: " + _listId + ", entry: " + _entryId, 2);
			activeChar.sendActionFailed();
			return;
		}

		_list = L2Multisell.getInstance().getList(_listId);

		// На всякий случай...
		if(_list == null)
		{
			activeChar.sendActionFailed();
			return;
		}

		_keepenchant = _list.getKeepEnchant();
		_notax = _list.getNoTax();

		if(_entryId > 10000 && _list.getKeepEnchant())
		{
			String entry = String.valueOf(_entryId);
			while(entry.length() < 6)
				entry = "0" + entry;
			_enchant = Short.parseShort(entry.substring(0, 2));
			_entryId = Short.parseShort(entry.substring(2, 6));
		}

		for(FastList.Node<MultiSellEntry> n = _list.getEntries().head(), end = _list.getEntries().tail(); (n = n.getNext()) != end;)
		{
			MultiSellEntry entry = n.getValue();
			if(entry.getEntryId() == _entryId)
			{
				doExchange(activeChar, entry);
				break;
			}
		}
	}

	private void doExchange(L2Player activeChar, MultiSellEntry entry)
	{
		PcInventory inv = activeChar.getInventory();

		int totalAdenaCost = 0;
		double tax = 0.;
		double taxRate = 0.;
		L2NpcInstance merchant = activeChar.getLastNpc();
		if(merchant == null)
		{
			activeChar.sendPacket(Msg.INCORRECT_TARGET);
			return;
		}

		if(merchant.getCastle() != null)
			taxRate = merchant.getCastle().getTaxRate();
		if(_notax)
			taxRate = 0.;

		FastList<MultiSellIngredient> productId = entry.getProduction();
		Integer playerId = activeChar.getObjectId();
		boolean logExchange = Config.LOG_MULTISELL_ID_LIST.contains(_listId);
		StringBuffer msgb = new StringBuffer();

		if(logExchange)
			msgb.append("<multisell id=").append(_listId).append(" player=\"").append(activeChar.getName()).append("\" oid=").append(activeChar.getObjectId()).append(">\n");

		synchronized (inv)
		{
			int slots = inv.SlotsLeft();
			if(slots == 0)
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.THE_WEIGHT_AND_VOLUME_LIMIT_OF_INVENTORY_MUST_NOT_BE_EXCEEDED));
				return;
			}

			int req = 0;
			for(MultiSellIngredient i : productId)
				if(!ItemTable.getInstance().getTemplate(i.getItemId()).isStackable())
					req += _amount;
				else
					req++;
			if(req > slots)
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.THE_WEIGHT_AND_VOLUME_LIMIT_OF_INVENTORY_MUST_NOT_BE_EXCEEDED));
				return;
			}

			if(entry.getIngredients().size() == 0)
			{
				System.out.println("WARNING Ingredients list = 0 multisell id=:" + _listId + " player:" + activeChar.getName());
				return;
			}

			L2Augmentation augmentation = null;

			// Перебор всех ингридиентов, проверка наличия и создание списка забираемого
			for(FastList.Node<MultiSellIngredient> sn = entry.getIngredients().head(), send = entry.getIngredients().tail(); (sn = sn.getNext()) != send;)
			{
				MultiSellIngredient ingridient = sn.getValue();
				int ingridientItemId = ingridient.getItemId();
				long ingridientItemCount = ingridient.getItemCount();
				double total = ingridientItemCount * _amount;

				if(total <= 0 || total > Integer.MAX_VALUE) // Проверка на переполнение
				{
					Util.handleIllegalPlayerAction(activeChar, "RequestMultiSellChoose[214]", "tried an overflow exploit: buy " + _amount + " of " + productId + ", ingridient " + ingridientItemId + " count " + ingridientItemCount, 1);
					return;
				}

				if(ingridientItemId != L2Clan.CLAN_REPUTATION_SCORE_ITEM && !ItemTable.getInstance().getTemplate(ingridientItemId).isStackable())
					for(int i = 0; i < ingridientItemCount * _amount; i++)
					{
						L2ItemInstance[] list = inv.getAllItemsById(ingridientItemId);
						// Если энчант имеет значение - то ищем вещи с точно таким энчантом
						if(_keepenchant)
						{
							L2ItemInstance itemToTake = null;
							for(L2ItemInstance itm : list)
								if((itm.getEnchantLevel() == _enchant || itm.getItem().getType2() > 2) && !_items.contains(new ItemData(itm.getItemId(), itm.getCount(), itm)) && !itm.isShadowItem())
								{
									itemToTake = itm;
									break;
								}

							if(itemToTake == null)
							{
								activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_ENOUGH_REQUIRED_ITEMS));
								return;
							}

							if(itemToTake.getAugmentation() != null)
							{
								itemToTake.setWhFlag(true);
								augmentation = itemToTake.getAugmentation();
							}
							_items.add(new ItemData(itemToTake.getItemId(), 1, itemToTake));
						}
						// Если энчант не обрабатывается берется вещь с наименьшим энчантом
						else
						{
							L2ItemInstance itemToTake = null;
							for(L2ItemInstance itm : list)
								if(!_items.contains(new ItemData(itm.getItemId(), itm.getCount(), itm)) && (itemToTake == null || itm.getEnchantLevel() < itemToTake.getEnchantLevel()) && !itm.isShadowItem())
								{
									itemToTake = itm;
									if(itemToTake.getEnchantLevel() == 0)
										break;
								}

							if(itemToTake == null)
							{
								activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_ENOUGH_REQUIRED_ITEMS));
								return;
							}
							if(itemToTake.getAugmentation() != null)
							{
								itemToTake.setWhFlag(true);
								augmentation = itemToTake.getAugmentation();
							}
							_items.add(new ItemData(itemToTake.getItemId(), 1, itemToTake));
						}
					}
				else if(ingridientItemId == L2Clan.CLAN_REPUTATION_SCORE_ITEM)
				{
					if(activeChar.getClan() == null)
					{
						activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_ARE_NOT_A_CLAN_MEMBER));
						return;
					}

					if(activeChar.getClan().getReputationScore() < total)
					{
						activeChar.sendPacket(new SystemMessage(SystemMessage.THE_CLAN_REPUTATION_SCORE_IS_TOO_LOW));
						return;
					}

					if(activeChar.getClan().getLeaderId() != activeChar.getObjectId())
					{
						activeChar.sendPacket(new SystemMessage(SystemMessage.S1_IS_NOT_A_CLAN_LEADER).addString(activeChar.getName()));
						return;
					}
					_items.add(new ItemData(ingridientItemId, (int) total, null));
				}
				else
				{
					if(ingridientItemId == 57)
						totalAdenaCost += ingridientItemCount * _amount;
					L2ItemInstance item = inv.getItemByItemId(ingridientItemId);

					if(item == null || item.getCount() < total)
					{
						activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_ENOUGH_REQUIRED_ITEMS));
						return;
					}

					tax += item.getItem().getReferencePrice() * taxRate * total;
					_items.add(new ItemData(item.getItemId(), (int) total, item));
				}

				if(activeChar.getAdena() < totalAdenaCost + tax)
				{
					activeChar.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ADENA);
					return;
				}
			}

			InventoryUpdate iu = new InventoryUpdate();

			for(ItemData id : _items)
			{
				int count = id.getCount();
				if(count > 0)
				{
					L2ItemInstance item = id.getItem();

					if(item != null)
					{
						activeChar.sendPacket(item.isStackable() ? new SystemMessage(SystemMessage.S2_S1_HAS_DISAPPEARED).addNumber(id.getCount()).addItemName(item.getItemId()) : new SystemMessage(SystemMessage.S1_HAS_DISAPPEARED).addItemName(item.getItemId()));
						if(logExchange)
							msgb.append("\t<destroy id=").append(item.getItemId()).append(" oid=").append(item.getObjectId()).append(" count=").append(id.getCount()).append(">\n");

						if(item.isEquipped())
							inv.unEquipItemInSlotAndRecord(item.getEquipSlot());
						iu.addItem(inv.destroyItem(item, count, false, true));
					}
					else if(id.getId() == L2Clan.CLAN_REPUTATION_SCORE_ITEM)
					{
						activeChar.getClan().incReputation(-count, false);
						activeChar.sendPacket(new SystemMessage(SystemMessage.S1_POINTS_HAVE_BEEN_DEDUCTED_FROM_THE_CLAN_REPUTATION_SCORE).addNumber(count));
					}
				}
			}

			if(taxRate > 0 && tax > 0)
			{
				activeChar.reduceAdena((int) tax, false);
				activeChar.sendMessage("Tax: " + (int) tax, "Налог: " + (int) tax);
				merchant.getCastle().addToTreasury((int) tax, true, false);
			}

			for(MultiSellIngredient in : productId)
				if(ItemTable.getInstance().getTemplate(in.getItemId()).isStackable())
				{
					L2ItemInstance product = ItemTable.getInstance().createItem((short) in.getItemId(), playerId, 0, "RequestMultiSellChoose[" + _listId + "|" + _entryId + "]");
					double total = in.getItemCount() * _amount;

					if(total < 0 || total > Integer.MAX_VALUE)
					{
						Util.handleIllegalPlayerAction(activeChar, "RequestMultiSellChoose[288]", "tried an overflow exploit: buy " + _amount * in.getItemCount() + " of " + product.getItemId() + ", base amount " + _amount, 1);
						return;
					}

					product.setCount((int) total);
					activeChar.sendPacket(new SystemMessage(SystemMessage.EARNED_S2_S1_s).addItemName(product.getItemId()).addNumber(product.getCount()));
					if(logExchange)
						msgb.append("\t<add id=").append(product.getItemId()).append(" count=").append(product.getCount()).append(">\n");
					iu.addItem(inv.addItem(product));
				}
				else
					for(int i = 0; i < _amount; i++)
					{
						L2ItemInstance product = inv.addItem(ItemTable.getInstance().createItem((short) in.getItemId(), playerId, 0, "RequestMultiSellChoose[" + _listId + "|" + _entryId + "]"));
						product.setCount(1);
						if(_keepenchant)
							product.setEnchantLevel(_enchant);
						if(augmentation != null && product.isEquipable())
						{
							augmentation.setItem(product);
							product.setAugmentation(augmentation);
						}
						if(logExchange)
							msgb.append("\t<add id=").append(product.getItemId()).append(" oid=").append(product.getObjectId()).append(" count=").append(product.getCount()).append(">\n");
						iu.addItem(product);
						activeChar.sendPacket(new SystemMessage(SystemMessage.EARNED_S1).addItemName(product.getItemId()));
					}
			activeChar.sendPacket(iu);
		}

		activeChar.sendPacket(new StatusUpdate(activeChar.getObjectId()).addAttribute(StatusUpdate.CUR_LOAD, activeChar.getCurrentLoad()));
		if(logExchange)
		{
			msgb.append("</multisell>\n");
			Log.add(msgb.toString(), "multisell", "");
			msgb = null;
		}

		if(_list == null || !_list.getShowAll()) // Если показывается только то, на что хватает материалов обновить окно у игрока
			activeChar.sendPacket(new MultiSellList(activeChar, _listId, activeChar.getLastNpc() != null && activeChar.getLastNpc().getCastle() != null ? activeChar.getLastNpc().getCastle().getTaxRate() : 0));
	}

	@Override
	public String getType()
	{
		return _C__A7_MULTISELLCHOOSE;
	}
}
