package L2jFT.game.network.clientpackets;

import java.util.HashMap;

import L2jFT.Config;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;
import L2jFT.game.model.PcInventory;
import L2jFT.game.model.Warehouse;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.model.instances.L2ItemInstance.ItemClass;
import L2jFT.game.network.serverpackets.InventoryUpdate;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.util.Log;

/**
 * Format: dd[dd]
 * 
 * @author SYS
 */
public class RequestPackageSend extends L2GameClientPacket
{
	private static String _C__9F_REQUESTPACKAGESEND = "[C] 9F RequestPackageSend";
	private int _objectID;
	private HashMap<Integer, Integer> _items;

	private static int _FREIGHT_FEE = 1000;

	@Override
	public void readImpl()
	{
		_objectID = readD();
		int itemsCount = readD();
		if(itemsCount <= 0)
			return;

		_items = new HashMap<Integer, Integer>(itemsCount + 1, 0.999f);
		for(int i = 0; i < itemsCount; i++)
		{
			int obj_id = readD(); // this is some id sent in PackageSendableList
			int itemQuantity = readD();
			if(itemQuantity < 0)
			{
				_items = null;
				return;
			}
			_items.put(obj_id, itemQuantity);
		}
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null || _items == null || !activeChar.getPlayerAccess().UseWarehouse)
			return;

		PcInventory inventory = activeChar.getInventory();
		int adenaDeposit = 0;
		int adenaObjId;
		L2ItemInstance adena = inventory.getItemByItemId(57);
		if(adena != null)
			adenaObjId = adena.getObjectId();
		else
			adenaObjId = -1;
		for(Integer itemObjectId : _items.keySet())
		{
			L2ItemInstance item = inventory.getItemByObjectId(itemObjectId);
			if(item == null || item.isEquipped())
				return;

			if(_items.get(itemObjectId) < 0)
				return;

			if(itemObjectId == adenaObjId)
				adenaDeposit = _items.get(itemObjectId);
		}

		L2NpcInstance freighter = activeChar.getLastNpc();
		if(freighter == null || !activeChar.isInRange(freighter.getX(), freighter.getY(), L2Character.INTERACTION_DISTANCE))
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_FAILED_AT_SENDING_THE_PACKAGE_BECAUSE_YOU_ARE_TOO_FAR_FROM_THE_WAREHOUSE));
			return;
		}

		int fee = _items.size() * _FREIGHT_FEE;

		if(fee + adenaDeposit > activeChar.getAdena())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_LACK_THE_FUNDS_NEEDED_TO_PAY_FOR_THIS_TRANSACTION));
			return;
		}

		L2Player destChar = null;
		Warehouse warehouse;
		destChar = L2Player.load(_objectID);
		if(destChar == null)
		{
			// Something went wrong!
			if(Config.DEBUG)
				_log.warning("Error retrieving a warehouse object for char " + activeChar.getName());
			return;
		}
		warehouse = destChar.getFreight();

		// Item Max Limit Check
		if(_items.size() + warehouse.listItems(ItemClass.ALL).length > destChar.getFreightLimit())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.THE_CAPACITY_OF_THE_WAREHOUSE_HAS_BEEN_EXCEEDED));
			destChar.deleteMe(false);
			return;
		}

		// Transfer the items from activeChar's Inventory Instance to destChar's Freight Instance
		InventoryUpdate iu = new InventoryUpdate();
		for(Integer itemObjectId : _items.keySet())
		{
			L2ItemInstance found = inventory.getItemByObjectId(itemObjectId);
			if(found == null || found.isHeroItem() || !found.canBeDropped(activeChar))
				continue;

			if(!found.isStackable() || found.getCount() == _items.get(itemObjectId))
				iu.addRemovedItem(found);
			else
				iu.addModifiedItem(found);

			L2ItemInstance item = inventory.dropItem(found, _items.get(itemObjectId), true);
			//add item to warehouse
			L2ItemInstance olditem = warehouse.addItem(item);
			Log.LogItem(activeChar, Log.DepositToWarehouse, item);
			L2World.removeObject(olditem);
		}

		activeChar.reduceAdena(fee);
		activeChar.sendPacket(iu);

		activeChar.updateStats();

		// Delete destination L2Player used for freight
		destChar.deleteMe(false);
		activeChar.sendPacket(new SystemMessage(SystemMessage.THE_TRANSACTION_IS_COMPLETE));
	}

	@Override
	public String getType()
	{
		return _C__9F_REQUESTPACKAGESEND;
	}
}
