package L2jFT.game.network.clientpackets;

import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2ClanMember;
import L2jFT.game.model.L2Player;
import L2jFT.game.network.serverpackets.PledgeReceiveMemberInfo;
import L2jFT.game.network.serverpackets.PledgeShowMemberListUpdate;
import L2jFT.game.network.serverpackets.SystemMessage;

import java.util.logging.Logger;

public class RequestPledgeSetAcademyMaster extends L2GameClientPacket
{
	private static String _C__D0_19_REQUESTPLEDGESETACADEMYMASTER = "[C] D0:19 RequestPledgeSetAcademyMaster";
	static Logger _log = Logger.getLogger(RequestPledgeSetAcademyMaster.class.getName());

	private int _mode; // 1=set, 0=unset
	private String _sponsorName;
	private String _apprenticeName;

	@Override
	public void readImpl()
	{
		_mode = readD();
		_sponsorName = readS();
		_apprenticeName = readS();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		L2Clan clan = activeChar.getClan();
		if(clan == null)
			return;

		if((activeChar.getClanPrivileges() & L2Clan.CP_CL_MASTER_RIGHTS) == L2Clan.CP_CL_MASTER_RIGHTS)
		{
			L2ClanMember sponsor = activeChar.getClan().getClanMember(_sponsorName);
			L2ClanMember apprentice = activeChar.getClan().getClanMember(_apprenticeName);
			if(sponsor != null && apprentice != null)
			{
				if(apprentice.getPledgeType() != L2Clan.SUBUNIT_ACADEMY || sponsor.getPledgeType() == L2Clan.SUBUNIT_ACADEMY)
					return; // hack?

				if(_mode == 1)
				{
					if(sponsor.hasApprentice())
					{
						activeChar.sendMessage("This member already has an apprentice, dismiss it first.", "Этот игрок уже имеет ученика, вначале исключите его");
						return;
					}
					if(apprentice.hasSponsor())
					{
						activeChar.sendMessage("This apprentice already has a sponsor.", "У этого ученика уже есть спонсор");
						return;
					}
					sponsor.setApprentice(apprentice.getObjectId());
					clan.broadcastToOnlineMembers(new PledgeShowMemberListUpdate(apprentice));
					clan.broadcastToOnlineMembers(new SystemMessage(SystemMessage.S2_HAS_BEEN_DESIGNATED_AS_THE_APPRENTICE_OF_CLAN_MEMBER_S1).addString(sponsor.getName()).addString(apprentice.getName()));
				}
				else
				{
					if(!sponsor.hasApprentice())
					{
						activeChar.sendMessage("This member has no apprentice.", "У этого игрока нет учеников");
						return;
					}
					sponsor.setApprentice(0);
					clan.broadcastToOnlineMembers(new PledgeShowMemberListUpdate(apprentice));
					clan.broadcastToOnlineMembers(new SystemMessage(SystemMessage.S2_CLAN_MEMBER_S1S_APPRENTICE_HAS_BEEN_REMOVED).addString(sponsor.getName()).addString(apprentice.getName()));
				}
				if(apprentice.isOnline())
					apprentice.getPlayer().broadcastUserInfo();
				activeChar.sendPacket(new PledgeReceiveMemberInfo(sponsor));
			}
		}
		else
			activeChar.sendMessage("You do not have the master rights.", "Вы не имеете достаточно прав");
	}

	@Override
	public String getType()
	{
		return _C__D0_19_REQUESTPLEDGESETACADEMYMASTER;
	}
}
