package L2jFT.game.network.clientpackets;

import L2jFT.game.cache.Msg;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.TradeItem;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.util.Util;

import java.util.HashSet;
import java.util.logging.Logger;

public class RequestPrivateStoreBuy extends L2GameClientPacket
{
	private static String _C__79_SENDPRIVATESTOREBUYLIST = "[C] 79 SendPrivateStoreBuyList";
	private static Logger _log = Logger.getLogger(RequestPrivateStoreBuy.class.getName());

	private int _sellerID;
	private int _count;
	private int[] _items; // count * 3

	@Override
	public void readImpl()
	{
		_sellerID = readD();
		_count = readD();
		_items = new int[_count * 3];
		for(int i = 0; i < _count; i++)
		{
			_items[i * 3 + 0] = readD(); //object id
			_items[i * 3 + 1] = readD(); //count
			_items[i * 3 + 2] = readD(); //price
			if(_items[i * 3 + 1] < 0)
			{
				_items = null;
				break;
			}
		}
	}

	@Override
	@SuppressWarnings("unchecked")
	public void runImpl()
	{
		L2Player buyer = getClient().getActiveChar();
		if(buyer == null)
			return;

		if(!buyer.getPlayerAccess().UseTrade)
		{
			buyer.sendPacket(new SystemMessage(SystemMessage.THIS_ACCOUNT_CANOT_USE_PRIVATE_STORES));
			buyer.sendActionFailed();
			return;
		}

		if(_items == null || _count <= 0)
			return;

		HashSet<TradeItem> buyerlist = new HashSet<TradeItem>();

		int cost = 0;

		L2Player seller = (L2Player) buyer.getVisibleObject(_sellerID);
		if(seller == null || seller.getPrivateStoreType() != L2Player.STORE_PRIVATE_SELL || seller.getDistance3D(buyer) > L2Character.INTERACTION_DISTANCE)
		{
			buyer.sendActionFailed();
			return;
		}

		HashSet<TradeItem> sellerlist = seller.getSellList();

		if(seller.getSellMode())
		{
			buyerlist = (HashSet<TradeItem>) sellerlist.clone();
			for(TradeItem ti : buyerlist)
				cost += ti.getOwnersPrice() * ti.getCount();
		}
		else
			for(int i = 0; i < _count; i++)
			{
				TradeItem temp = new TradeItem();
				temp.setObjectId(_items[i * 3 + 0]);
				temp.setCount(_items[i * 3 + 1]);
				boolean valid = false;
				for(TradeItem si : sellerlist)
					if(si.getObjectId() == _items[i * 3 + 0])
					{
						valid = true;
						if(_items[i * 3 + 2] == si.getOwnersPrice())
							temp.setOwnersPrice(_items[i * 3 + 2]);
						else
						{
							Util.handleIllegalPlayerAction(buyer, "RequestPrivateStoreBuy[75]", "tried to change the seller's price in a private store (buy)", 2);
							buyer.sendActionFailed();
							return;
						}
					}
				if(valid)
				{
					cost += temp.getOwnersPrice() * temp.getCount();
					buyerlist.add(temp);
				}
			}

		if(buyer.getAdena() < cost || _count <= 0 || cost > Integer.MAX_VALUE || cost < 0)
		{
			buyer.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ADENA);
			buyer.sendActionFailed();
			return;
		}

		for(TradeItem ti : buyerlist)
		{
			if(seller.getInventory().getItemByObjectId(ti.getObjectId()) == null)
				continue;
			ti.setItemId(seller.getInventory().getItemByObjectId(ti.getObjectId()).getItemId());
		}

		if(seller.getTradeList() == null)
		{
			buyer.sendActionFailed();
			_log.warning("TradeList == null for player " + seller.getName());
			return;
		}

		seller.getTradeList().buySellItems(buyer, buyerlist, seller, sellerlist);
		buyer.sendChanges();
		if(seller.getSellList().isEmpty())
		{
			seller.setPrivateStoreType(L2Player.STORE_PRIVATE_NONE);
			seller.getTradeList().removeAll();
			seller.setSellMode(false);
			seller.broadcastUserInfo();
		}
		seller.sendChanges();
		buyer.sendActionFailed();
	}

	@Override
	public String getType()
	{
		return _C__79_SENDPRIVATESTOREBUYLIST;
	}
}
