package L2jFT.game.network.clientpackets;

import javolution.util.FastList;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Summon;
import L2jFT.game.model.L2World;
import L2jFT.game.model.instances.L2BoatInstance;
import L2jFT.game.model.instances.L2DoorInstance;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.model.instances.L2StaticObjectInstance;
import L2jFT.game.network.serverpackets.CharInfo;
import L2jFT.game.network.serverpackets.DoorInfo;
import L2jFT.game.network.serverpackets.DoorStatusUpdate;
import L2jFT.game.network.serverpackets.NpcInfo;
import L2jFT.game.network.serverpackets.SpawnItem;
import L2jFT.game.network.serverpackets.StaticObject;
import L2jFT.game.network.serverpackets.UserInfo;
import L2jFT.game.network.serverpackets.VehicleInfo;

public class RequestRecordInfo extends L2GameClientPacket
{
	@Override
	public void readImpl()
	{}

	@Override
	public void runImpl()
	{
		L2Player player = getClient().getActiveChar();

		if(player == null)
			return;

		player.sendPacket(new UserInfo(player));

		FastList<L2Object> objs = L2World.getAroundObjects(player);
		for(FastList.Node<L2Object> n = objs.head(), end = objs.tail(); (n = n.getNext()) != end;)
		{
			if(n == null)
				break;

			L2Object obj = n.getValue();

			if(obj instanceof L2NpcInstance)
				player.sendPacket(new NpcInfo((L2NpcInstance) obj, player));
			else if(obj instanceof L2Summon)
			{
				if(player == ((L2Summon) obj).getOwner())
					((L2Summon) obj).broadcastPetInfo();
				else
					player.sendPacket(new NpcInfo((L2Summon) obj, player));
			}
			else if(obj.isPlayer)
			{
				if(player.getObjectId() != obj.getObjectId() && !((L2Player) obj).isInvisible())
					player.sendPacket(new CharInfo((L2Player) obj));
			}
			else if(obj instanceof L2DoorInstance)
			{
				player.sendPacket(new DoorStatusUpdate((L2DoorInstance) obj));
				player.sendPacket(new DoorInfo((L2DoorInstance) obj));
			}
			else if(obj instanceof L2BoatInstance)
				player.sendPacket(new VehicleInfo((L2BoatInstance) obj));
			else if(obj instanceof L2ItemInstance)
				player.sendPacket(new SpawnItem((L2ItemInstance) obj));
			else if(obj instanceof L2StaticObjectInstance)
				player.sendPacket(new StaticObject((L2StaticObjectInstance) obj));
		}
	}

	/**
	 * just for information and debug purposes
	 * @return text for trace message
	 */
	@Override
	public String getType()
	{
		return "[C] CF RequestRecordInfo";
	}
}
