package L2jFT.game.network.clientpackets;

import L2jFT.Config;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.network.serverpackets.ExVariationResult;
import L2jFT.game.network.serverpackets.InventoryUpdate;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.network.serverpackets.UserInfo;
import L2jFT.game.tables.AugmentationData;
import L2jFT.game.templates.L2Item;

public final class RequestRefine extends L2GameClientPacket
{
	private static final String _C__D0_2C_REQUESTREFINE = "[C] D0:2C RequestRefine";
	private int _targetItemObjId;
	private int _refinerItemObjId;
	private int _gemstoneItemObjId;
	@SuppressWarnings("unused")
	private int _gemstoneCount;

	@Override
	protected void readImpl()
	{
		_targetItemObjId = readD();
		_refinerItemObjId = readD();
		_gemstoneItemObjId = readD();
		_gemstoneCount = readD();
	}

	@Override
	protected void runImpl()
	{
		if(_gemstoneCount < 0)
			return;

		L2Player activeChar = getClient().getActiveChar();
		L2ItemInstance targetItem = activeChar.getInventory().getItemByObjectId(_targetItemObjId);
		L2ItemInstance refinerItem = activeChar.getInventory().getItemByObjectId(_refinerItemObjId);
		L2ItemInstance gemstoneItem = activeChar.getInventory().getItemByObjectId(_gemstoneItemObjId);

		if(targetItem == null || refinerItem == null || gemstoneItem == null || targetItem.getOwnerId() != activeChar.getObjectId() || refinerItem.getOwnerId() != activeChar.getObjectId() || gemstoneItem.getOwnerId() != activeChar.getObjectId() || activeChar.getLevel() < 46)
		{
			activeChar.sendPacket(new ExVariationResult(0, 0, 0));
			activeChar.sendPacket(new SystemMessage(SystemMessage.AUGMENTATION_FAILED_DUE_TO_INAPPROPRIATE_CONDITIONS));
			return;
		}

		if(TryAugmentItem(activeChar, targetItem, refinerItem, gemstoneItem))
		{
			int stat12 = 0x0000FFFF & targetItem.getAugmentation().getAugmentationId();
			int stat34 = targetItem.getAugmentation().getAugmentationId() >> 16;
			activeChar.sendPacket(new ExVariationResult(stat12, stat34, 1));
			activeChar.sendPacket(new SystemMessage(SystemMessage.THE_ITEM_WAS_SUCCESSFULLY_AUGMENTED));
		}
		else
		{
			activeChar.sendPacket(new ExVariationResult(0, 0, 0));
			activeChar.sendPacket(new SystemMessage(SystemMessage.AUGMENTATION_FAILED_DUE_TO_INAPPROPRIATE_CONDITIONS));
		}
	}

	boolean TryAugmentItem(L2Player player, L2ItemInstance targetItem, L2ItemInstance refinerItem, L2ItemInstance gemstoneItem)
	{
		if(targetItem.isAugmented() || targetItem.isWear())
			return false;

		int itemGrade = targetItem.getItem().getItemGrade();
		int itemType = targetItem.getItem().getType2();
		int lifeStoneId = refinerItem.getItemId();
		int gemstoneItemId = gemstoneItem.getItemId();

		// is the refiner Item a life stone?
		if(lifeStoneId < 8723 || lifeStoneId > 8762)
			return false;

		// must be a weapon, must be > d grade
		// TODO: can do better? : currently: using isdestroyable() as a check for hero / cursed weapons
		if(itemGrade < L2Item.CRYSTAL_C || itemType != L2Item.TYPE2_WEAPON && !Config.ALT_ALLOW_AUGMENT_ALL || !targetItem.isDestroyable())
			return false;

		// player must be able to use augmentation
		if(player.getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE || player.isDead() || player.isParalyzed() || player.isFishing() || player.isSitting())
			return false;

		int modifyGemstoneCount = _gemstoneCount;
		int lifeStoneLevel = getLifeStoneLevel(lifeStoneId);
		int lifeStoneGrade = getLifeStoneGrade(lifeStoneId);
		switch(itemGrade)
		{
			case L2Item.CRYSTAL_C:
				if(player.getLevel() < 46 || gemstoneItemId != 2130)
					return false;
				modifyGemstoneCount = 20;
				break;
			case L2Item.CRYSTAL_B:
				if(player.getLevel() < 52 || gemstoneItemId != 2130)
					return false;
				modifyGemstoneCount = 30;
				break;
			case L2Item.CRYSTAL_A:
				if(player.getLevel() < 61 || gemstoneItemId != 2131)
					return false;
				modifyGemstoneCount = 20;
				break;
			case L2Item.CRYSTAL_S:
				if(player.getLevel() < 76 || gemstoneItemId != 2131)
					return false;
				modifyGemstoneCount = 25;
				break;
		}

		// check if the lifestone is appropriate for this player
		switch(lifeStoneLevel)
		{
			case 1:
				if(player.getLevel() < 46)
					return false;
				break;
			case 2:
				if(player.getLevel() < 49)
					return false;
				break;
			case 3:
				if(player.getLevel() < 52)
					return false;
				break;
			case 4:
				if(player.getLevel() < 55)
					return false;
				break;
			case 5:
				if(player.getLevel() < 58)
					return false;
				break;
			case 6:
				if(player.getLevel() < 61)
					return false;
				break;
			case 7:
				if(player.getLevel() < 64)
					return false;
				break;
			case 8:
				if(player.getLevel() < 67)
					return false;
				break;
			case 9:
				if(player.getLevel() < 70)
					return false;
				break;
			case 10:
				if(player.getLevel() < 76)
					return false;
				break;
		}

		if(gemstoneItem.getCount() - modifyGemstoneCount < 0)
			return false;
		player.getInventory().destroyItem(_gemstoneItemObjId, modifyGemstoneCount, true, true);

		// consume the life stone
		player.getInventory().destroyItem(refinerItem, 1, true, true);

		// generate augmentation
		targetItem.setAugmentation(AugmentationData.getInstance().generateRandomAugmentation(targetItem, lifeStoneLevel, lifeStoneGrade));

		// send an inventory update packet
		InventoryUpdate iu = new InventoryUpdate();
		iu.addModifiedItem(targetItem);
		iu.addRemovedItem(refinerItem);
		iu.addItem(gemstoneItem);
		player.sendPacket(iu);
		player.sendPacket(new UserInfo(player));

		return true;
	}

	private int getLifeStoneGrade(int itemId)
	{
		itemId -= 8723;
		if(itemId < 10)
			return 0;
		if(itemId < 20)
			return 1;
		if(itemId < 30)
			return 2;
		return 3;
	}

	private int getLifeStoneLevel(int itemId)
	{
		itemId -= 10 * getLifeStoneGrade(itemId);
		itemId -= 8722;
		return itemId;
	}

	@Override
	public String getType()
	{
		return _C__D0_2C_REQUESTREFINE;
	}
}