package L2jFT.game.network.clientpackets;

import L2jFT.game.cache.Msg;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.network.serverpackets.ExVariationCancelResult;
import L2jFT.game.network.serverpackets.InventoryUpdate;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.templates.L2Item;

public final class RequestRefineCancel extends L2GameClientPacket
{
	private static final String _C__D0_2E_REQUESTREFINECANCEL = "[C] D0:2E RequestRefineCancel";
	private int _targetItemObjId;

	@Override
	protected void readImpl()
	{
		_targetItemObjId = readD();
	}

	@Override
	protected void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		L2ItemInstance targetItem = activeChar.getInventory().getItemByObjectId(_targetItemObjId);

		// cannot remove augmentation from a not augmented item
		if(!targetItem.isAugmented())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.AUGMENTATION_REMOVAL_CAN_ONLY_BE_DONE_ON_AN_AUGMENTED_ITEM));
			return;
		}

		// get the price
		int price = 0;
		switch(targetItem.getItem().getItemGrade())
		{
			case L2Item.CRYSTAL_C:
				if(targetItem.getItem().getCrystalCount() < 1720)
					price = 95000;
				else if(targetItem.getItem().getCrystalCount() < 2452)
					price = 150000;
				else
					price = 210000;
				break;
			case L2Item.CRYSTAL_B:
				if(targetItem.getItem().getCrystalCount() < 1746)
					price = 240000;
				else
					price = 270000;
				break;
			case L2Item.CRYSTAL_A:
				if(targetItem.getItem().getCrystalCount() < 2160)
					price = 330000;
				else if(targetItem.getItem().getCrystalCount() < 2824)
					price = 390000;
				else
					price = 420000;
				break;
			case L2Item.CRYSTAL_S:
				price = 480000;
				break;
			// any other item type is not augmentable
			default:
				return;
		}

		// try to reduce the players adena
		if(activeChar.getAdena() < price)
		{
			activeChar.sendPacket(Msg.YOU_DO_NOT_HAVE_ENOUGH_ADENA);
			return;
		}
		activeChar.reduceAdena(price);

		// cancel boni
		targetItem.getAugmentation().removeBoni(activeChar);

		// remove the augmentation
		targetItem.removeAugmentation();

		// send ExVariationCancelResult
		activeChar.sendPacket(new ExVariationCancelResult());

		// send inventory update
		InventoryUpdate iu = new InventoryUpdate();
		iu.addModifiedItem(targetItem);
		activeChar.sendPacket(iu);

		// send system message
		SystemMessage sm = new SystemMessage(SystemMessage.AUGMENTATION_HAS_BEEN_SUCCESSFULLY_REMOVED_FROM_YOUR_S1);
		sm.addItemName(targetItem.getItemId());
		activeChar.sendPacket(sm);

		activeChar.broadcastUserInfo();
	}

	@Override
	public String getType()
	{
		return _C__D0_2E_REQUESTREFINECANCEL;
	}
}