package L2jFT.game.network.clientpackets;

import L2jFT.game.model.L2Party;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.entity.SevenSignsFestival;
import L2jFT.game.network.L2GameClient.GameClientState;
import L2jFT.game.network.serverpackets.CharSelectInfo;
import L2jFT.game.network.serverpackets.RestartResponse;
import L2jFT.game.network.serverpackets.SystemMessage;

public class RequestRestart extends L2GameClientPacket
{
	private static String _C__46_REQUESTRESTART = "[C] 46 RequestRestart";

	/**
	 * packet type id 0x46
	 * format:      c
	 * @param decrypt
	 */

	@Override
	public void readImpl()
	{}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();

		if(activeChar == null)
			return;

		if(activeChar.getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE)
		{
			activeChar.sendMessage("Cannot restart while trading", "Вы не можете выйти во время торговли");
			activeChar.sendActionFailed();
			return;
		}

		if(activeChar.isInOlympiadMode())
		{
			activeChar.sendMessage("You cant logout in olympiad mode", "Вы не можете выйти во время Олимпиады");
			activeChar.sendActionFailed();
			return;
		}

		if(activeChar.isInCombat())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_RESTART_WHILE_IN_COMBAT));
			activeChar.sendActionFailed();
			return;
		}

		if(activeChar.isFishing())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_DO_ANYTHING_ELSE_WHILE_FISHING));
			activeChar.sendActionFailed();
			return;
		}

		if(activeChar.isBlocked())
		{
			activeChar.sendMessage("Your character is out of control now.", "Сейчас ваш персонаж неуправляем.");
			activeChar.sendActionFailed();
			return;
		}

		// Prevent player from restarting if they are a festival participant
		// and it is in progress, otherwise notify party members that the player
		// is not longer a participant.
		if(activeChar.isFestivalParticipant())
		{
			if(SevenSignsFestival.getInstance().isFestivalInitialized())
			{
				activeChar.sendMessage("You cannot restart while you are a participant in a festival.", "Вы не можете выйти, так как учавствуете в Фестивале");
				activeChar.sendActionFailed();
				return;
			}
			L2Party playerParty = activeChar.getParty();

			if(playerParty != null)
				playerParty.broadcastMessageToPartyMembers(activeChar.getName() + " has been removed from the upcoming festival.");
		}

		if(getClient() != null)
			getClient().setState(GameClientState.AUTHED);
		activeChar.logout(false, true);
		sendPacket(new RestartResponse());
		// send char list
		CharSelectInfo cl = new CharSelectInfo(getClient().getLoginName(), getClient().getSessionId().playOkID1);
		sendPacket(cl);
		getClient().setCharSelection(cl.getCharInfo());
	}

	@Override
	public String getType()
	{
		return _C__46_REQUESTRESTART;
	}
}
