package L2jFT.game.network.clientpackets;

import L2jFT.game.instancemanager.SiegeManager;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2Effect;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2SiegeClan;
import L2jFT.game.model.entity.ClanHall;
import L2jFT.game.model.entity.siege.Siege;
import L2jFT.game.network.serverpackets.Revive;
import L2jFT.game.tables.MapRegionTable;
import L2jFT.util.Location;
import L2jFT.util.Util;

/**
 * PointType:
 * 1 - ClanHall
 * 2 - Castle
 * 3 - SiegeFlag
 * 4 - ResurectFixed
 */
public class RequestRestartPoint extends L2GameClientPacket
{
	private static String _C__6d_REQUESTRESTARTPOINT = "[C] 6d RequestRestartPoint";

	protected int requestedPointType;
	protected boolean continuation;

	/**
	 * packet type id 0x6d
	 * format:    c
	 * @param decrypt
	 */
	@Override
	public void readImpl()
	{
		requestedPointType = readD();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();

		if(activeChar == null)
			return;

		if(activeChar.isFakeDeath())
		{
			activeChar.stopEffects(L2Effect.EffectType.FAKE_DEATH);
			activeChar.broadcastPacket(new Revive(activeChar));
			return;
		}

		if(!activeChar.isDead() && !activeChar.getPlayerAccess().IsGM)
		{
			activeChar.sendMessage("Try to play without cheating!", "Попробуйте играть без читов!");
			Util.handleIllegalPlayerAction(activeChar, "RequestRestartPoint[55]", "Tried to use revive cheat", 1);
			return;
		}

		try
		{
			Location loc = null;

			if(activeChar.isFestivalParticipant())
				requestedPointType = 4;

			L2Clan clan = activeChar.getClan();
			ClanHall clanHall = activeChar.getClanHall();
			Siege siege = SiegeManager.getInstance().getSiege(activeChar);

			switch(requestedPointType)
			{
				case 1:
					if(clan == null || clanHall == null)
					{
						if(clan == null)
							activeChar.sendMessage("Hmmm... Respawn cheat? Bad idea...Clan not found", "Хммм... Читер? Плохая идея... У вас нет клана");
						if(clanHall == null)
							activeChar.sendMessage("Hmmm... Respawn cheat? Bad idea...HasHideout=0", "Хммм... Читер? Плохая идея... У вас нет кланхолла");
						Util.handleIllegalPlayerAction(activeChar, "RequestRestartPoint[61]", "Tried clanhall respawn cheat", 1);
						loc = MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.ClosestTown);
					}
					else
					{
						loc = MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.ClanHall);
						if(clanHall.getFunction(ClanHall.FUNC_RESTORE_EXP) != null)
							activeChar.restoreExp(clanHall.getFunction(ClanHall.FUNC_RESTORE_EXP).getLvl());
					}
					break;
				case 2:
					Boolean isInDefense = false;
					if(siege != null && siege.isInProgress() && siege.checkIsDefender(activeChar.getClan()))
						isInDefense = true;
					if((clan == null || clan.getHasCastle() == 0) && !isInDefense)
					{
						activeChar.sendMessage("Hmmm... Respawn cheat? Bad idea...", "Хммм... Читер? Плохая идея...");
						Util.handleIllegalPlayerAction(activeChar, "RequestRestartPoint[79]", "Tried castle respawn cheat", 1);
						loc = MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.ClosestTown);
					}
					else
						loc = MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.Castle);
					break;
				case 3:
					L2SiegeClan siegeClan = null;
					if(siege != null && siege.isInProgress())
						siegeClan = siege.getAttackerClan(activeChar.getClan());
					if(siegeClan == null || siegeClan.getFlag() == null)
					{
						Util.handleIllegalPlayerAction(activeChar, "RequestRestartPoint[93]", "Dismissed from clan OR Dead Flag on Siege OR Using Packet Respawn", 0);
						loc = MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.ClosestTown);
					}
					else
						loc = MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.SiegeFlag);
					break;
				case 4:
					if(!activeChar.isFestivalParticipant() && !activeChar.getPlayerAccess().ResurectFixed)
					{
						activeChar.sendMessage("Hmmm... Respawn cheat? Bad idea...", "Хммм... Читер? Плохая идея...");
						Util.handleIllegalPlayerAction(activeChar, "RequestRestartPoint[103]", "Tried fixed respawn cheat", 1);
						loc = MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.ClosestTown);
					}
					else
						loc = new Location(activeChar.getX(), activeChar.getY(), activeChar.getZ());
					break;
				default:
					loc = MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.ClosestTown);
					break;
			}

			activeChar.setIsPendingRevive(true);
			activeChar.teleToLocation(loc);
		}
		catch(Throwable e)
		{

		}
	}

	@Override
	public String getType()
	{
		return _C__6d_REQUESTRESTARTPOINT;
	}
}
