package L2jFT.game.network.clientpackets;

import L2jFT.Config;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.instances.L2CastleChamberlainInstance;
import L2jFT.game.model.instances.L2ClanHallManagerInstance;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.instances.L2MercManagerInstance;
import L2jFT.game.model.instances.L2MerchantInstance;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.network.serverpackets.ItemList;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.util.Log;
import L2jFT.util.Util;

public class RequestSellItem extends L2GameClientPacket
{
	private static String _C__1E_REQUESTSELLITEM = "[C] 1E RequestSellItem";
	//private static Logger _log = Logger.getLogger(RequestSellItem.class.getName());

	@SuppressWarnings("unused")
	private int _listId;
	private int _count;
	private int[] _items; // count*3

	/**
	 * packet type id 0x1e
	 *
	 * sample
	 *
	 * 1e
	 * 00 00 00 00		// list id
	 * 02 00 00 00		// number of items
	 *
	 * 71 72 00 10		// object id
	 * ea 05 00 00		// item id
	 * 01 00 00 00		// item count
	 *
	 * 76 4b 00 10		// object id
	 * 2e 0a 00 00		// item id
	 * 01 00 00 00		// item count
	 *
	 * format:		cdd (ddd)
	 */
	@Override
	public void readImpl()
	{
		_listId = readD();
		_count = readD();
		_items = new int[_count * 3];
		for(int i = 0; i < _count; i++)
		{
			_items[i * 3 + 0] = readD();
			_items[i * 3 + 1] = readD();
			_items[i * 3 + 2] = readD();
			if(_items[i * 3 + 1] < 0)
			{
				_items = null;
				break;
			}
		}
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();

		if(activeChar == null)
			return;

		if(_items == null || _count <= 0)
			return;

		if(!Config.ALT_GAME_KARMA_PLAYER_CAN_SHOP && activeChar.getKarma() > 0 && !activeChar.isGM())
		{
			activeChar.sendActionFailed();
			return;
		}

		L2NpcInstance npc = activeChar.getLastNpc();

		boolean isValidMerchant = npc instanceof L2ClanHallManagerInstance || npc instanceof L2MerchantInstance || npc instanceof L2MercManagerInstance || npc instanceof L2CastleChamberlainInstance;

		if(!activeChar.isGM() && (npc == null || !isValidMerchant || !activeChar.isInRange(npc.getX(), npc.getY(), L2Character.INTERACTION_DISTANCE)))
		{
			activeChar.sendActionFailed();
			return;
		}

		for(int i = 0; i < _count; i++)
		{
			int objectId = _items[i * 3 + 0];
			int itemId = _items[i * 3 + 1];
			int cnt = _items[i * 3 + 2];

			if(cnt < 0)
			{
				Util.handleIllegalPlayerAction(activeChar, "Integer overflow", "RequestSellItem[100]", 0);
				continue;
			}
			else if(cnt == 0)
				continue;

			L2ItemInstance item = activeChar.getInventory().getItemByObjectId(objectId);
			if(item == null || item.isWear() || !item.canBeTraded(activeChar) || !item.getItem().isSellable() || item.isEquipped())
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.THE_ATTEMPT_TO_SELL_HAS_FAILED));
				return;
			}

			if(item.getItemId() != itemId)
			{
				Util.handleIllegalPlayerAction(activeChar, "Fake packet", "RequestSellItem[115]", 0);
				continue;
			}

			if(item.getCount() < cnt)
			{
				Util.handleIllegalPlayerAction(activeChar, "Incorrect item count", "RequestSellItem[121]", 0);
				continue;
			}

			int price = item.getReferencePrice() * cnt / 2;

			activeChar.addAdena(price);
			Log.LogItem(activeChar, Log.SellItem, item);

			// If player sells the enchant scroll he is using, deactivate it
			if(activeChar.getEnchantScroll() != null && item.getObjectId() == activeChar.getEnchantScroll().getObjectId())
				activeChar.setEnchantScroll(null);

			activeChar.getInventory().destroyItem(item, cnt, false, true);
		}

		activeChar.updateStats();
		activeChar.sendPacket(new ItemList(activeChar, true));
		activeChar.doInteract(npc);
	}

	@Override
	public String getType()
	{
		return _C__1E_REQUESTSELLITEM;
	}
}