package L2jFT.game.network.clientpackets;

import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2ShortCut;
import L2jFT.game.network.serverpackets.ShortCutRegister;

public class RequestShortCutReg extends L2GameClientPacket
{
	private static String _C__33_REQUESTSHORTCUTREG = "[C] 33 RequestShortCutReg";

	private int _type;
	private int _id;
	private int _slot;
	private int _page;

	/**
	 * packet type id 0x33
	 * format:		cdddd
	 * @param rawPacket
	 */
	@Override
	public void readImpl()
	{
		_type = readD();
		int slot = readD();
		_id = readD();
		readD(); // unknown

		_slot = slot % 12;
		_page = slot / 12;
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		if(_slot < 0 || _slot > 11 || _page < 0 || _page > 9)
		{
			activeChar.sendActionFailed();
			return;
		}

		switch(_type)
		{
			case 0x01: // item
			case 0x03: // action
			case 0x04: // macro
			case 0x05: // recipe 
			{
				L2ShortCut sc = new L2ShortCut(_slot, _page, _type, _id, -1);
				sendPacket(new ShortCutRegister(sc));
				activeChar.registerShortCut(sc);
				break;
			}
			case 0x02: // skill
			{
				int level = activeChar.getSkillDisplayLevel((short) _id);
				if(level > 0)
				{
					L2ShortCut sc = new L2ShortCut(_slot, _page, _type, _id, level);
					sendPacket(new ShortCutRegister(sc));
					activeChar.registerShortCut(sc);
				}
				break;
			}
		}

	}

	@Override
	public String getType()
	{
		return _C__33_REQUESTSHORTCUTREG;
	}
}