package L2jFT.game.network.clientpackets;

import L2jFT.Config;
import L2jFT.game.ThreadPoolManager;
import L2jFT.game.model.L2Player;
import L2jFT.game.network.serverpackets.SocialAction;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.util.Util;

public class RequestSocialAction extends L2GameClientPacket
{
	private static String _C__1B_REQUESTSOCIALACTION = "[C] 1B RequestSocialAction";
	private int _actionId;

	/**
	 * packet type id 0x1b
	 * format:		cd
	 */
	@Override
	public void readImpl()
	{
		_actionId = readD();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		if(activeChar.isOutOfControl())
		{
			activeChar.sendActionFailed();
			return;
		}

		// You cannot do anything else while fishing
		if(activeChar.isFishing())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_DO_ANYTHING_ELSE_WHILE_FISHING));
			return;
		}

		// internal Social Action check
		if(_actionId < 2 || _actionId > 13)
		{
			Util.handleIllegalPlayerAction(activeChar, "RequestSocialAction[43]", "Character " + activeChar.getName() + " at account " + activeChar.getAccountName() + "requested an internal Social Action " + _actionId, 1);
			return;
		}

		if(activeChar.getPrivateStoreType() == L2Player.STORE_PRIVATE_NONE && activeChar.getTransactionRequester() == null && !activeChar.isActionsDisabled() && !activeChar.isSitting())
		{
			activeChar.broadcastPacket(new SocialAction(activeChar.getObjectId(), _actionId));
			if(Config.ALT_SOCIAL_ACTION_REUSE)
			{
				ThreadPoolManager.getInstance().scheduleGeneral(new SocialTask(activeChar), 2600);
				activeChar.block();
			}
		}
	}

	class SocialTask implements Runnable
	{
		L2Player _player;

		SocialTask(L2Player player)
		{
			_player = player;
		}

		public void run()
		{
			_player.unblock();
		}
	}

	@Override
	public String getType()
	{
		return _C__1B_REQUESTSOCIALACTION;
	}
}