package L2jFT.game.network.clientpackets;

import L2jFT.Config;
import L2jFT.database.DatabaseUtils;
import L2jFT.database.FiltredPreparedStatement;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.database.ThreadConnection;
import L2jFT.extensions.listeners.PropertyCollection;
import L2jFT.game.handler.IVoicedCommandHandler;
import L2jFT.game.handler.VoicedCommandHandler;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;
import L2jFT.game.network.serverpackets.CreatureSay;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.GmListTable;
import L2jFT.status.GameStatusThread;
import L2jFT.status.Status;
import L2jFT.util.Calculator;
import L2jFT.util.Log;
import L2jFT.util.Strings;
import L2jFT.util.Util;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;

public class Say2 extends L2GameClientPacket
{
	private static String _C__38_SAY2 = "[C] 38 Say2";
	private static Logger _log = Logger.getLogger(Say2.class.getName());
	private static Logger _logChat = Logger.getLogger("chat");

	public final static int ALL = 0;
	public final static int SHOUT = 1; //!
	public final static int TELL = 2; //\"
	public final static int PARTY = 3; //#
	public final static int CLAN = 4; //@
	public final static int GM = 5;
	public final static int PETITION_PLAYER = 6; // used for petition
	public final static int PETITION_GM = 7; //* used for petition
	public final static int TRADE = 8; //+
	public final static int ALLIANCE = 9; //$
	public final static int ANNOUNCEMENT = 10;
	public final static int COMMANDCHANNEL_ALL = 15; //`` (pink) команды лидера СС
	public final static int COMMANDCHANNEL_COMMANDER = 16; //` (yellow) команды лидеров партий в СС
	public final static int HERO_VOICE = 17; //%

	public static String[] chatNames = {
			"ALL	", "SHOUT", "TELL ", "PARTY", "CLAN ", "GM	 ", "PETITION_PLAYER",
			"PETITION_GM", "TRADE", "ALLIANCE", "ANNOUNCEMENT", "", "", "", "",
			"COMMANDCHANNEL_ALL", "COMMANDCHANNEL_COMMANDER", "HERO_VOICE" };

	protected static List<String> _banned = new ArrayList<String>();
	private String _text;
	private int _type;
	private String _target;

	/**
	 * packet type id 0x38
	 * format:		cSd (S)
	 * @param decrypt
	 */
	@Override
	public void readImpl()
	{
		_text = readS();
		_type = readD();
		_target = _type == TELL ? readS() : null;
	}

	@Override
	public void runImpl()
	{
		if(Config.DEBUG)
			_log.info("Say type:" + _type);

		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		if(_type < 0 || _type > chatNames.length)
		{
			activeChar.sendActionFailed();
			return;
		}

		if(activeChar.isBotLevel() > 0 && Config.ALLOW_L2WALKER_CLIENT == Config.L2WalkerAllowed.Peace && (_type == ALL || _type == SHOUT))
		{
			activeChar.sendMessage("All and shout channels are not allowed for bots");
			return;
		}

		if(Config.LOG_CHAT)
		{
			LogRecord record = new LogRecord(Level.INFO, _text);
			record.setLoggerName("chat");
			if(_type == TELL)
				record.setParameters(new Object[] {
						chatNames[_type],
						"[" + activeChar.getName() + " to " + _target + "]" });
			else
				record.setParameters(new Object[] {
						chatNames[_type], "[" + activeChar.getName() + "]" });
			_logChat.log(record);
		}

		if(Config.LOG_TELNET)
		{
			String line_output;

			if(_type == TELL)
				line_output = chatNames[_type] + "[" + activeChar.getName() + " to " + _target + "] " + _text;
			else
				line_output = chatNames[_type] + "[" + activeChar.getName() + "] " + _text;
			telnet_output(line_output, _type);
		}

		if(_text.startsWith("."))
		{
			String fullcmd = _text.substring(1).trim();
			String command = fullcmd.split("\\s+")[0];
			String args = fullcmd.substring(command.length()).trim();

			if(command.length() > 0)
			{
				// then check for VoicedCommands
				IVoicedCommandHandler vch = VoicedCommandHandler.getInstance().getVoicedCommandHandler(command);
				if(vch != null)
				{
					vch.useVoicedCommand(command, activeChar, args);
					return;
				}
			}
		}
		else if(_text.startsWith("=="))
		{
			activeChar.sendMessage(Util.formatDouble(Calculator.eval(_text.substring(2)), "Wrong equation", false));
			return;
		}

		boolean globalchat = _type != ALLIANCE && _type != CLAN && _type != PARTY;
		if(activeChar.getNoChannel() != 0)
		{
			if(activeChar.getNoChannelRemained() > 0 || activeChar.getNoChannel() < 0)
			{
				if(activeChar.getNoChannel() > 0)
				{
					int time_remaiden = Math.round(activeChar.getNoChannelRemained() / 60000);
					activeChar.sendMessage("You are banned in all chats, time remained " + time_remaiden + " min.", "Ваш чат забанен. До снятия бана осталось " + time_remaiden + " минут.");
				}
				else
					activeChar.sendMessage("You are banned in all chats permanently.", "Ваш чат забанен на бессрочный период.");
				return;
			}
			activeChar.updateNoChannel(0);
		}

		if(Config.MAT_BANCHAT && !Config.MAT_REPLACE)
		{
			for(String pattern : Config.MAT_LIST)
				if(_text.matches(".*" + pattern + ".*"))
				{
					activeChar.sendMessage("You are banned in all chats. Time to unban: " + Config.UNCHATBANTIME * 60 + "sec.");
					Log.add("" + activeChar + ": " + _text, "abuse");
					activeChar.updateNoChannel(Config.UNCHATBANTIME * 60000);
					return;
				}
		}
		else if(Config.MAT_BANCHATG && globalchat && !Config.MAT_REPLACE)
			for(String pattern : Config.MAT_LIST)
				if(_text.matches(".*" + pattern + ".*"))
				{
					activeChar.sendMessage("You are banned in all chats. Time to unban: " + Config.UNCHATBANTIME * 60 + "sec.");
					Log.add("" + activeChar + ": " + _text, "abuse");
					activeChar.updateNoChannel(Config.UNCHATBANTIME * 60000);
					return;
				}
		if(Config.MAT_REPLACE)
		{
			for(String pattern : Config.MAT_LIST)
			if(_text.matches(".*" + pattern + ".*"))
			{
				_text = Config.MAT_REPLACE_STRING;
				return;
			}
		}

		if(getClient().getActiveChar().getVarB("fromTranslit"))
			_text = Strings.fromTranslit(_text);

		CreatureSay cs = new CreatureSay(activeChar.getObjectId(), _type, activeChar.getName(), _text);

		switch(_type)
		{
			case TELL:
				L2Player receiver = L2World.getPlayer(_target);
				if(receiver != null && !receiver.isInBlockList(activeChar) && !receiver.isBlockAll())
				{
					if(!receiver.getMessageRefusal())
					{
						receiver.sendPacket(cs);
						cs = new CreatureSay(activeChar.getObjectId(), _type, "->" + receiver.getName(), _text);
						activeChar.sendPacket(cs);
					}
					else
						activeChar.sendPacket(new SystemMessage(SystemMessage.THE_PERSON_IS_IN_A_MESSAGE_REFUSAL_MODE));
				}
				else if(receiver == null)
					activeChar.sendPacket(new SystemMessage(SystemMessage.S1_IS_NOT_LOGGED_IN).addString(_target));
				else
					activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_BEEN_BLOCKED_FROM_THE_CONTACT_YOU_SELECTED));
				break;
			case SHOUT:
				if(activeChar.inObserverMode())
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_CHAT_LOCALLY_WHILE_OBSERVING));
					return;
				}
				if(activeChar.getLevel() >= Config.GLOBAL_CHAT || activeChar.isGM())
				{
					for(L2Player player : L2World.getAllPlayers())
						if(!player.isInBlockList(activeChar) && !player.isBlockAll())
							player.sendPacket(cs);
				}
				else
				{
					for(L2Player player : L2World.getAroundPlayers(activeChar))
						if(!player.isInBlockList(activeChar) && !player.isBlockAll())
							player.sendPacket(cs);
					activeChar.sendPacket(cs);
				}
				break;
			case TRADE:
				if(activeChar.inObserverMode())
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_CHAT_LOCALLY_WHILE_OBSERVING));
					return;
				}
				if(activeChar.getLevel() >= Config.GLOBAL_TRADE_CHAT || activeChar.isGM())
				{
					for(L2Player player : L2World.getAllPlayers())
						if(!player.isInBlockList(activeChar) && !player.isBlockAll())
							player.sendPacket(cs);
				}
				else
				{
					for(L2Player player : L2World.getAroundPlayers(activeChar))
						if(!player.isInBlockList(activeChar) && !player.isBlockAll())
							player.sendPacket(cs);
					activeChar.sendPacket(cs);
				}
				break;
			case ALL:
				if(activeChar.inObserverMode())
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_CHAT_LOCALLY_WHILE_OBSERVING));
					return;
				}
				for(L2Player player : activeChar.getAroundPlayers(1250))
					if(!player.isInBlockList(activeChar) && !player.isBlockAll())
						player.sendPacket(cs);
				activeChar.sendPacket(cs);
				break;
			case CLAN:
				if(activeChar.getClan() != null)
					activeChar.getClan().broadcastToOnlineMembers(cs);
				break;
			case ALLIANCE:
				if(activeChar.getClan() != null && activeChar.getClan().getAlliance() != null)
					activeChar.getClan().getAlliance().broadcastToOnlineMembers(cs);
				break;
			case PARTY:
				if(activeChar.isInParty())
					activeChar.getParty().broadcastToPartyMembers(cs);
				break;
			case COMMANDCHANNEL_ALL:
				if(!activeChar.isInParty() || !activeChar.getParty().isInCommandChannel())
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_AUTHORITY_TO_USE_THE_COMMAND_CHANNEL));
					return;
				}
				if(activeChar.getParty().getCommandChannel().getChannelLeader() == activeChar)
					activeChar.getParty().getCommandChannel().broadcastToChannelMembers(cs);
				else
					activeChar.sendPacket(new SystemMessage(SystemMessage.ONLY_CHANNEL_OPENER_CAN_GIVE_ALL_COMMAND));
				break;
			case COMMANDCHANNEL_COMMANDER:
				if(!activeChar.isInParty() || !activeChar.getParty().isInCommandChannel())
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_DO_NOT_HAVE_AUTHORITY_TO_USE_THE_COMMAND_CHANNEL));
					return;
				}
				if(activeChar.getParty().isLeader(activeChar))
					activeChar.getParty().getCommandChannel().broadcastToChannelMembers(cs);
				else
					activeChar.sendPacket(new SystemMessage(SystemMessage.ONLY_A_PARTY_LEADER_CAN_ACCESS_THE_COMMAND_CHANNEL));
				break;
			case HERO_VOICE:
				if(activeChar.isHero() || activeChar.getPlayerAccess().CanAnnounce)
				{
					// Ограничение только для героев, гм-мы пускай говорят.
					if(!activeChar.getPlayerAccess().CanAnnounce)
					{
						Long lastTime = (Long) activeChar.getProperty(PropertyCollection.HeroChatLaunched);
						long curTime = System.currentTimeMillis();
						if(lastTime != null && lastTime + 10000L > curTime)
						{
							activeChar.sendMessage("Hero chat is allowed once per 10 seconds.");
							return;
						}
						activeChar.addProperty(PropertyCollection.HeroChatLaunched, curTime);
					}

					for(L2Player player : L2World.getAllPlayers())
						if(!player.isInBlockList(activeChar) && !player.isBlockAll())
							player.sendPacket(cs);
				}
				break;
			case PETITION_PLAYER:
			case PETITION_GM:
				for(L2Player gm : GmListTable.getAllGMs())
					if(!gm.getMessageRefusal())
						gm.sendPacket(cs);
				break;
			default:
				_log.warning("Character " + activeChar.getName() + " used unknown chat type: " + _type + ". Cheater?");
		}
	}

	private void telnet_output(String _text, int type)
	{
		GameStatusThread tinstance = Status.telnetlist;

		while(tinstance != null)
		{
			if(type == TELL && tinstance.log_tell == 1)
				tinstance.write(_text);
			else if(tinstance.log_chat == 1)
				tinstance.write(_text);
			tinstance = tinstance.next;
		}
	}

	@Override
	public String getType()
	{
		return _C__38_SAY2;
	}

	public class UnbanTask implements Runnable
	{
		public String _name;

		public UnbanTask(String Name)
		{
			_name = Name;
		}

		public void run()
		{
			L2Player plyr = L2World.getPlayer(_name);
			if(plyr != null)
			{
				plyr.setAccessLevel(0);
				plyr.sendMessage("Nochannel deactivated");
				Log.add("" + plyr + ": unbanchat online", "abuse");
			}
			else
			{
				setCharacterAccessLevel(_name, 0);
				Log.add("Player " + _name + ": unbanchat offline", "abuse");
			}

			_banned.remove(_name);
		}
	}

	public void setCharacterAccessLevel(String user, int banLevel)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			String stmt = "UPDATE characters SET characters.accesslevel = ? WHERE characters.char_name=?";
			statement = con.prepareStatement(stmt);
			statement.setInt(1, banLevel);
			statement.setString(2, user);
			statement.executeUpdate();

			//SystemMessage sm = new SystemMessage(SystemMessage.S1_S2);
			//sm.addString("Character Access Level for "+user+" set to "+banLevel+".");
			//activeChar.sendPacket(sm);
		}
		catch(Exception e)
		{
			//System.out.println("Error in sql");
			_log.warning("Could not set accessLevl:" + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}
}
