package L2jFT.game.network.clientpackets;

import java.util.HashMap;

import L2jFT.Config;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;
import L2jFT.game.model.PcInventory;
import L2jFT.game.model.Warehouse;
import L2jFT.game.model.Warehouse.WarehouseType;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.model.instances.L2ItemInstance.ItemClass;
import L2jFT.game.network.serverpackets.InventoryUpdate;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.util.Log;

/**
 * This class ... 31 SendWareHouseDepositList cd (dd)
 *
 * @version $Revision: 1.3.4.5 $ $Date: 2005/04/11 10:06:09 $
 */
public class SendWareHouseDepositList extends L2GameClientPacket
{
	private static String _C__31_SENDWAREHOUSEDEPOSITLIST = "[C] 31 SendWareHouseDepositList";

	private static int _WAREHOUSE_FEE = 30;
	private HashMap<Integer, Integer> _items;

	@Override
	public void readImpl()
	{
		int itemsCount = readD();
		if(itemsCount <= 0)
			return;
		_items = new HashMap<Integer, Integer>(itemsCount + 1, 0.999f);
		for(int i = 0; i < itemsCount; i++)
		{
			int obj_id = readD();
			int itemQuantity = readD();
			if(itemQuantity < 0)
			{
				_items = null;
				return;
			}
			_items.put(obj_id, itemQuantity);
		}
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null || _items == null)
			return;

		PcInventory inventory = activeChar.getInventory();
		int adenaDeposit = 0;
		int adenaObjId;
		L2ItemInstance adena = inventory.getItemByItemId(57);
		if(adena != null)
			adenaObjId = adena.getObjectId();
		else
			adenaObjId = -1;
		for(Integer itemObjectId : _items.keySet())
		{
			L2ItemInstance item = inventory.getItemByObjectId(itemObjectId);
			if(item == null || item.isEquipped())
				return;

			if(_items.get(itemObjectId) < 0)
				return;

			if(itemObjectId == adenaObjId)
				adenaDeposit = _items.get(itemObjectId);
		}

		L2NpcInstance whkeeper = activeChar.getLastNpc();
		if(whkeeper == null || !activeChar.isInRange(whkeeper.getX(), whkeeper.getY(), L2Character.INTERACTION_DISTANCE))
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.WAREHOUSE_IS_TOO_FAR));
			return;
		}

		int fee = _items.size() * _WAREHOUSE_FEE;

		if(fee + adenaDeposit > activeChar.getAdena())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_LACK_THE_FUNDS_NEEDED_TO_PAY_FOR_THIS_TRANSACTION));
			return;
		}

		Warehouse warehouse;
		boolean privatewh = false;
		if(activeChar.getUsingWarehouseType() == WarehouseType.CLAN)
		{
			warehouse = activeChar.getClan().getWarehouse();
			if(_items.size() + warehouse.listItems(ItemClass.ALL).length > Config.WAREHOUSE_SLOTS_CLAN)
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.THE_CAPACITY_OF_THE_WAREHOUSE_HAS_BEEN_EXCEEDED));
				return;
			}
		}
		else
		{
			warehouse = activeChar.getWarehouse();
			if(_items.size() + warehouse.listItems(ItemClass.ALL).length > activeChar.getWarehouseLimit())
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.YOUR_WAREHOUSE_IS_FULL));
				return;
			}
			privatewh = true;
		}

		// From now on, it's safe to conclude the requested transaction...

		// Transfer the items from activeChar's Inventory Instance to destChar's Freight Instance
		InventoryUpdate iu = new InventoryUpdate();
		for(Integer itemObjectId : _items.keySet())
		{
			L2ItemInstance found = inventory.getItemByObjectId(itemObjectId);
			if(found == null || !(privatewh || found.canBeDropped(activeChar)))
				continue;

			if(!found.isStackable() || found.getCount() == _items.get(itemObjectId))
				iu.addRemovedItem(found);
			else
				iu.addModifiedItem(found);

			L2ItemInstance item = inventory.dropItem(found, _items.get(itemObjectId), true);
			//add item to warehouse
			L2ItemInstance olditem = warehouse.addItem(item);
			Log.LogItem(activeChar, Log.DepositToWarehouse, item);
			L2World.removeObject(olditem);
		}

		activeChar.reduceAdena(fee);
		activeChar.sendPacket(iu);

		activeChar.updateStats();
	}

	@Override
	public String getType()
	{
		return _C__31_SENDWAREHOUSEDEPOSITLIST;
	}
}
