package L2jFT.game.network.clientpackets;

import L2jFT.Config;
import L2jFT.game.model.ClanWarehousePool;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;
import L2jFT.game.model.Warehouse;
import L2jFT.game.model.Warehouse.WarehouseType;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.network.serverpackets.InventoryUpdate;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.util.Log;

import java.util.logging.Logger;

public class SendWareHouseWithDrawList extends L2GameClientPacket
{
	private static String _C__32_SENDWAREHOUSEWITHDRAWLIST = "[C] 32 SendWareHouseWithDrawList";
	private static Logger _log = Logger.getLogger(SendWareHouseWithDrawList.class.getName());

	private int _count;
	private int[] _items;
	private int[] counts;

	@Override
	public void readImpl()
	{
		_count = readD();
		if(_count < 0)
		{
			_items = null;
			return;
		}

		_items = new int[_count * 2];
		counts = new int[_count];
		for(int i = 0; i < _count; i++)
		{
			_items[i * 2 + 0] = readD(); //item object id
			_items[i * 2 + 1] = readD(); //count
			if(_items[i * 2 + 1] < 0)
			{
				_items = null;
				break;
			}
		}
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		if(_items == null || _count <= 0)
			return;

		L2NpcInstance whkeeper = activeChar.getLastNpc();
		if(whkeeper == null || !activeChar.isInRange(whkeeper.getX(), whkeeper.getY(), L2Character.INTERACTION_DISTANCE))
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.WAREHOUSE_IS_TOO_FAR));
			return;
		}

		if(!Config.ALT_ALLOW_OTHERS_WITHDRAW_FROM_CLAN_WAREHOUSE && activeChar.getUsingWarehouseType() == WarehouseType.CLAN && activeChar.getClan().getLeaderId() != activeChar.getObjectId())
			return;

		if(activeChar.getUsingWarehouseType() == WarehouseType.CLAN && !((activeChar.getClanPrivileges() & L2Clan.CP_CL_VIEW_WAREHOUSE) == L2Clan.CP_CL_VIEW_WAREHOUSE))
			return;

		int weight = 0;
		int finalCount = activeChar.getInventory().getSize();
		L2ItemInstance[] olditems = new L2ItemInstance[_count];

		for(int i = 0; i < _count; i++)
		{
			int itemObjId = _items[i * 2 + 0];
			int count = _items[i * 2 + 1];
			L2ItemInstance oldinst = L2ItemInstance.restoreFromDb(itemObjId);

			if(count < 0)
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.INCORRECT_ITEM_COUNT));
				return;
			}

			if(oldinst == null)
			{
				activeChar.sendMessage("Somebody has changed Clan Warehouse. Try again.", "Кто-то изменил клановое хранилище. Попробуйте снова");
				for(int f = 0; f < i; f++)
					L2World.removeObject(olditems[i]); // FIXME don't sure...

				return;
			}
			
			if(oldinst.getCount() < count)
				count = oldinst.getCount();

			counts[i] = count;
			olditems[i] = oldinst;
			weight += oldinst.getItem().getWeight() * count;
			finalCount++;
			if(oldinst.getItem().isStackable() && activeChar.getInventory().getItemByItemId(oldinst.getItemId()) != null)
				finalCount--;
		}

		if(!activeChar.getInventory().validateCapacity(finalCount))
		{
			for(L2ItemInstance element : olditems)
				//L2World.removeObject(items[i]);
				L2World.removeObject(element); // FIXME don't sure...
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOUR_INVENTORY_IS_FULL));
			//            items = null;
			return;
		}

		if(!activeChar.getInventory().validateWeight(weight))
		{
			for(L2ItemInstance element : olditems)
				L2World.removeObject(element); // FIXME don't sure...
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_EXCEEDED_THE_WEIGHT_LIMIT));
			return;
		}

		Warehouse warehouse = null;
		if(activeChar.getUsingWarehouseType() == WarehouseType.PRIVATE)
			warehouse = activeChar.getWarehouse();
		else if(activeChar.getUsingWarehouseType() == WarehouseType.CLAN)
		{
			ClanWarehousePool.getInstance().AddWork(activeChar, olditems, counts);
			return;
		}
		else if(activeChar.getUsingWarehouseType() == WarehouseType.FREIGHT)
			warehouse = activeChar.getFreight();
		else
		{
			// Something went wrong!
			_log.warning("Error retrieving a warehouse object for char " + activeChar.getName() + " - using warehouse type: " + activeChar.getUsingWarehouseType());
			return;
		}

		InventoryUpdate iu = new InventoryUpdate();
		for(int i = 0; i < olditems.length; i++)
		{
			L2ItemInstance TransferItem = warehouse.getItemByObj(olditems[i].getObjectId(), counts[i]);
			if(TransferItem == null)
				_log.warning("Error getItem from warhouse player: " + activeChar.getName());

			iu.addItem(activeChar.getInventory().addItem(TransferItem));

			Log.LogItem(activeChar, Log.RetrieveFromWarehouse, TransferItem);
			L2World.removeObject(olditems[i]);
		}

		activeChar.sendChanges();
		activeChar.sendPacket(iu);
	}

	@Override
	public String getType()
	{
		return _C__32_SENDWAREHOUSEWITHDRAWLIST;
	}
}