package L2jFT.game.network.clientpackets;

import java.util.HashSet;

import L2jFT.game.model.L2Player;
import L2jFT.game.model.TradeItem;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.network.serverpackets.PrivateStoreMsgSell;
import L2jFT.game.network.serverpackets.SystemMessage;

public class SetPrivateStoreListSell extends L2GameClientPacket
{
	private static String _C__74_SETPRIVATESTORELISTSELL = "[C] 74 SetPrivateStoreListSell";

	private int _count;
	@SuppressWarnings("unused")
	private boolean _mode;
	private int[] _items; // count * 3

	@Override
	public void readImpl()
	{
		_mode = readD() == 1;
		_count = readD();
		_items = new int[_count * 3];
		for(int i = 0; i < _count; i++)
		{
			_items[i * 3 + 0] = readD(); //objectId
			_items[i * 3 + 1] = readD(); //count
			_items[i * 3 + 2] = readD(); //price
			if(_items[i * 3 + 1] < 0)
			{
				_items = null;
				break;
			}
		}
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		if(_items == null || _count <= 0)
			return;

		TradeItem temp;
		activeChar.setSellList(new HashSet<TradeItem>());
		activeChar.setSellMode(_mode);
		HashSet<TradeItem> listsell = activeChar.getSellList();

		int maxSlots = activeChar.getTradeLimit();

		if(_count > maxSlots)
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_EXCEEDED_THE_QUANTITY_THAT_CAN_BE_INPUTTED));
			activeChar.setPrivateStoreType(L2Player.STORE_PRIVATE_NONE);
			activeChar.broadcastUserInfo();
			activeChar.setInStoreMode(false);
			activeChar.standUp();
			return;
		}

		int count = _count;
		for(int x = 0; x < _count; x++)
		{
			temp = new TradeItem();
			int objectId = _items[x * 3 + 0];
			int cnt = _items[x * 3 + 1];
			int price = _items[x * 3 + 2];
			temp.setObjectId(objectId);

			L2ItemInstance itemToSell = activeChar.getInventory().getItemByObjectId(objectId);

			if(itemToSell == null || itemToSell.isWear() || !itemToSell.canBeTraded(activeChar))
			{
				count--;
				continue;
			}

			// If player sells the enchant scroll he is using, deactivate it
			if(activeChar.getEnchantScroll() != null && itemToSell.getObjectId() == activeChar.getEnchantScroll().getObjectId())
				activeChar.setEnchantScroll(null);

			if(cnt > itemToSell.getCount())
				cnt = itemToSell.getCount();

			temp.setCount(cnt);
			temp.setOwnersPrice(price);
			temp.setItemId(itemToSell.getItemId());
			temp.setEnchantLevel(itemToSell.getEnchantLevel());
			listsell.add(temp);
		}

		if(count != 0)
		{
			activeChar.setPrivateStoreType(L2Player.STORE_PRIVATE_SELL);
			activeChar.broadcastUserInfo();
			activeChar.broadcastPacket(new PrivateStoreMsgSell(activeChar));
			activeChar.sitDown();
		}
		else
		{
			activeChar.setPrivateStoreType(L2Player.STORE_PRIVATE_NONE);
			activeChar.broadcastUserInfo();
			activeChar.standUp();
		}
	}

	@Override
	public String getType()
	{
		return _C__74_SETPRIVATESTORELISTSELL;
	}
}
