package L2jFT.game.network.clientpackets;

import L2jFT.game.cache.Msg;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.network.serverpackets.SendTradeRequest;
import L2jFT.game.network.serverpackets.SystemMessage;

public class TradeRequest extends L2GameClientPacket
{
	private static String TRADEREQUEST__C__15 = "[C] 15 TradeRequest";
	private int _objectId;

	@Override
	public void readImpl()
	{
		_objectId = readD();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		if(!activeChar.getPlayerAccess().UseTrade)
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.THIS_ACCOUNT_CANOT_TRADE_ITEMS));
			activeChar.sendActionFailed();
			return;
		}

		if(activeChar.isDead())
		{
			activeChar.sendActionFailed();
			return;
		}

		L2Object target = activeChar.getVisibleObject(_objectId);

		if(activeChar.getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE)
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.WHILE_OPERATING_A_PRIVATE_STORE_OR_WORKSHOP_YOU_CANNOT_DISCARD_DESTROY_OR_TRADE_AN_ITEM));
			return;
		}

		if(target == null || !target.isPlayer || target.getObjectId() == activeChar.getObjectId())
		{
			activeChar.sendPacket(Msg.TARGET_IS_INCORRECT);
			return;
		}

		L2Player pcTarget = (L2Player) target;

		if(!pcTarget.getPlayerAccess().UseTrade)
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.THIS_ACCOUNT_CANOT_TRADE_ITEMS));
			activeChar.sendActionFailed();
			return;
		}

		if(pcTarget.getTeam() != 0)
		{
			activeChar.sendActionFailed();
			return;
		}

		if(pcTarget.isInOlympiadMode() || activeChar.isInOlympiadMode())
		{
			activeChar.sendPacket(Msg.TARGET_IS_INCORRECT);
			return;
		}

		if(pcTarget.getTradeRefusal() || pcTarget.isInBlockList(activeChar) || pcTarget.isBlockAll())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_BEEN_BLOCKED_FROM_THE_CONTACT_YOU_SELECTED));
			return;
		}

		if(activeChar.isTransactionInProgress())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.ALREADY_TRADING));
			return;
		}

		if(activeChar.isFishing())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_DO_THAT_WHILE_FISHING));
			return;
		}

		if(activeChar.isActionsDisabled())
		{
			activeChar.sendActionFailed();
			return;
		}

		if(activeChar.knowsObject(target) && !(pcTarget.isTransactionInProgress() || pcTarget.getTradeList() != null))
		{
			pcTarget.setTransactionRequester(activeChar, System.currentTimeMillis() + 10000);
			activeChar.setTransactionRequester(pcTarget, System.currentTimeMillis() + 10000);
			pcTarget.sendPacket(new SendTradeRequest(activeChar.getObjectId()));
			activeChar.sendPacket(new SystemMessage(SystemMessage.REQUEST_S1_FOR_TRADE).addString(pcTarget.getName()));
		}
		else
			activeChar.sendPacket(new SystemMessage(SystemMessage.S1_IS_BUSY_PLEASE_TRY_AGAIN_LATER).addString(pcTarget.getName()));
	}

	@Override
	public String getType()
	{
		return TRADEREQUEST__C__15;
	}
}
