package L2jFT.game.network.clientpackets;

import L2jFT.game.handler.IItemHandler;
import L2jFT.game.handler.ItemHandler;
import L2jFT.game.instancemanager.CursedWeaponsManager;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.network.serverpackets.InventoryUpdate;
import L2jFT.game.network.serverpackets.ShowCalculator;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.network.serverpackets.UserInfo;
import L2jFT.game.tables.ItemTable;
import L2jFT.game.templates.L2Item;

import java.nio.BufferUnderflowException;
import java.util.ArrayList;
import java.util.Arrays;

public class UseItem extends L2GameClientPacket
{
	private static String _C__14_USEITEM = "[C] 14 UseItem";

	private int _objectId;
	@SuppressWarnings("unused")
	private int _unknown;

	/**
	 * packet type id 0x14
	 * format:		cd
	 * @param decrypt
	 */
	@Override
	public void readImpl()
	{
		try
		{
			_objectId = readD();
			_unknown = readD();
		}
		catch(BufferUnderflowException e)
		{
			_log.warning(e.getMessage());
			_log.info("Attention! Possible cheater found! Login:" + getClient().getLoginName());
		}
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		if(activeChar.isOutOfControl())
		{
			activeChar.sendActionFailed();
			return;
		}

		if(activeChar.getUnstuck() != 0)
		{
			activeChar.sendActionFailed();
			return;
		}

		if(activeChar.getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE)
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_MAY_NOT_USE_ITEMS_IN_A_PRIVATE_STORE_OR_PRIVATE_WORK_SHOP));
			activeChar.sendActionFailed();
			return;
		}

		synchronized (activeChar.getInventory())
		{
			L2ItemInstance item = activeChar.getInventory().getItemByObjectId(_objectId);

			if(item == null)
				return;

			int itemId = item.getItemId();
			if(itemId == 57)
				return;

			if(activeChar.isFishing() && (itemId < 6535 || itemId > 6540))
			{
				// You cannot do anything else while fishing
				activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_DO_ANYTHING_ELSE_WHILE_FISHING));
				return;
			}

			if(activeChar.isDead())
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.S1_CANNOT_BE_USED_DUE_TO_UNSUITABLE_TERMS).addItemName(itemId));
				return;
			}

			if(item.getItem().isForPet())
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_EQUIP_A_PET_ITEM).addItemName(itemId));
				return;
			}

			if(item.isEquipable())
			{
				if(activeChar.isCastingNow())
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_USE_EQUIPMENT_WHEN_USING_OTHER_SKILLS_OR_MAGIC));
					return;
				}

				// Нельзя снимать/одевать любое снаряжение при этих условиях
				if(activeChar.isStunned() || activeChar.isSleeping() || activeChar.isParalyzed() || activeChar.isAlikeDead())
				{
					activeChar.sendActionFailed();
					return;
				}

				int bodyPart = item.getBodyPart();

				// Нельзя снимать/одевать оружие, сидя на страйдере/виверне
				if(activeChar.isMounted() && (bodyPart == L2Item.SLOT_LR_HAND || bodyPart == L2Item.SLOT_L_HAND || bodyPart == L2Item.SLOT_R_HAND))
					return;

				// Нельзя снимать/одевать проклятое оружие
				if(CursedWeaponsManager.getInstance().isCursed(itemId))
					return;

				// Don't allow weapon/shield hero equipment during Olympiads 
				if(activeChar.isInOlympiadMode() && item.isHeroItem())
					return;

				L2ItemInstance weapon;

				// Equip or unEquip
				L2ItemInstance[] items;
				boolean isEquipped = item.isEquipped();
				if(isEquipped)
				{
					weapon = activeChar.getActiveWeaponInstance();
					items = activeChar.getInventory().unEquipItemInBodySlotAndRecord(bodyPart, item);
				}
				else
				{
					items = activeChar.getInventory().equipItemAndRecord(item);
					weapon = activeChar.getActiveWeaponInstance();
				}

				if(isEquipped != item.isEquipped())
				{
					SystemMessage sm = null;
					if(isEquipped)
					{
						if(item.getEnchantLevel() > 0)
						{
							sm = new SystemMessage(SystemMessage.EQUIPMENT_OF__S1_S2_HAS_BEEN_REMOVED);
							sm.addNumber(item.getEnchantLevel());
							sm.addItemName(itemId);
						}
						else
							sm = new SystemMessage(SystemMessage.S1__HAS_BEEN_DISARMED).addItemName(itemId);
						activeChar.sendPacket(sm);

						if(weapon != null && item == weapon)
						{
							item.setChargedSpiritshot(L2ItemInstance.CHARGED_NONE);
							item.setChargedSoulshot(L2ItemInstance.CHARGED_NONE);
						}
					}
					else
					{
						if(item.getEnchantLevel() > 0)
						{
							sm = new SystemMessage(SystemMessage.EQUIPPED__S1_S2);
							sm.addNumber(item.getEnchantLevel());
							sm.addItemName(itemId);
						}
						else
							sm = new SystemMessage(SystemMessage.YOU_ARE_EQUIPPED_WITH_S1).addItemName(itemId);
						activeChar.sendPacket(sm);

						if(weapon != null && item == weapon)
							activeChar.AutoShot();
					}
					sm = null;

					if(item.getItem().getType2() == L2Item.TYPE2_ACCESSORY)
						activeChar.sendPacket(new UserInfo(activeChar));
					else
						activeChar.broadcastUserInfo();
				}

				activeChar.refreshExpertisePenalty();
				activeChar.sendPacket(new InventoryUpdate(new ArrayList<L2ItemInstance>(Arrays.asList(items))));
				return;
			}

			if(itemId == 4393)
			{
				activeChar.sendPacket(new ShowCalculator(itemId));
				return;
			}

			if(ItemTable.useHandler(activeChar, item))
				return;

			IItemHandler handler = ItemHandler.getInstance().getItemHandler(itemId);
			if(handler != null)
				handler.useItem(activeChar, item);
		}
	}

	@Override
	public String getType()
	{
		return _C__14_USEITEM;
	}
}