package L2jFT.game.network.clientpackets;

import java.awt.Point;
import java.util.logging.Logger;

import L2jFT.Config;
import L2jFT.game.geodata.GeoData;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Zone.ZoneType;
import L2jFT.game.network.serverpackets.CharMoveToLocation;
import L2jFT.game.network.serverpackets.PartyMemberPosition;
import L2jFT.game.network.serverpackets.ValidateLocation;
import L2jFT.game.tables.MapRegionTable;
import L2jFT.util.CharPosition;

public class ValidatePosition extends L2GameClientPacket
{
	private static String _C__48_VALIDATEPOSITION = "[C] 48 ValidatePosition";
	protected static Logger _log = Logger.getLogger(ValidatePosition.class.getName());

	private int _x;
	private int _y;
	private int _z;
	@SuppressWarnings("unused")
	private int _heading;
	@SuppressWarnings("unused")
	private int _data;
	private double diff;
	private CharPosition _lastClientPosition;
	private CharPosition _lastServerPosition;

	/**
	 * packet type id 0x48
	 * format:		cddddd
	 * @param decrypt
	 */
	@Override
	public void readImpl()
	{
		_x = readD();
		_y = readD();
		_z = readD();
		_heading = readD();
		_data = readD();
	}

	@Override
	public void runImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		activeChar.checkWaterState();

		if(activeChar.isTeleporting() || activeChar.inObserverMode())
			return;

		_lastClientPosition = activeChar.getLastClientPosition();
		_lastServerPosition = activeChar.getLastServerPosition();

		if(_lastClientPosition == null)
			_lastClientPosition = new CharPosition(activeChar.getX(), activeChar.getY(), activeChar.getZ(), activeChar.getHeading());
		if(_lastServerPosition == null)
			_lastServerPosition = new CharPosition(activeChar.getX(), activeChar.getY(), activeChar.getZ(), activeChar.getHeading());

		if(activeChar.getX() == 0 && activeChar.getY() == 0 && activeChar.getZ() == 0)
			if(_lastServerPosition.x != 0 && _lastServerPosition.y != 0 && _lastServerPosition.z != 0)
			{
				activeChar.setXYZ(_lastServerPosition.x, _lastServerPosition.y, _lastServerPosition.z);
				activeChar.broadcastPacket(new ValidateLocation(activeChar));
			}
			else if(_lastClientPosition.x != 0 && _lastClientPosition.y != 0 && _lastClientPosition.z != 0)
			{
				activeChar.setXYZ(_lastClientPosition.x, _lastClientPosition.y, _lastClientPosition.z);
				activeChar.broadcastPacket(new ValidateLocation(activeChar));
			}

		// use heading from client side always
		//activeChar.setHeading(_heading);

		int dz = _lastClientPosition.z - _z;

		if(Config.DAMAGE_FROM_FALLING && dz > 333 && !activeChar.isInZone(ZoneType.water))
			activeChar.falling(dz);

		diff = activeChar.getDistance(_x, _y);

		if(activeChar.isInBoat())
			activeChar.setXYZ(activeChar.getBoat().getX(), activeChar.getBoat().getY(), activeChar.getBoat().getZ());
		else if(activeChar.isFlying() && activeChar.isBlocked()) // летит на маршрутной виверне
		{
			if(diff > 150 && diff < 400)
				activeChar.broadcastPacket(new CharMoveToLocation(activeChar));
			else if(diff >= 400)
				activeChar.broadcastPacket(new ValidateLocation(activeChar));
		}
		else if(activeChar.isFlying() || activeChar.isInWater())
			activeChar.setXYZ(_x, _y, _z);
		else if(_z < -15000 || _z > 15000)
			if(Config.GEODATA)
				if(GeoData.getInstance().getNSWE(_lastServerPosition.x, _lastServerPosition.y, _lastServerPosition.z) == 15)
				{
					activeChar.stopMove(null);
					activeChar.setXYZ(_lastServerPosition.x, _lastServerPosition.y, GeoData.getInstance().getHeight(_lastServerPosition.x, _lastServerPosition.y, _lastServerPosition.z));
					activeChar.broadcastPacket(new ValidateLocation(activeChar));
				}
				else
					activeChar.teleToLocation(MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.ClosestTown));
			else
				activeChar.teleToLocation(_lastServerPosition.x, _lastServerPosition.y, _lastServerPosition.z);
		else if(diff > 600)
			activeChar.broadcastPacket(new ValidateLocation(activeChar));
		/*
		if(Config.GEODATA)
			if(GeoData.getInstance().getNSWE(_lastServerPosition.x, _lastServerPosition.y, _lastServerPosition.z) == 15)
			{
				activeChar.stopMove(null);
				activeChar.setXYZ(_lastServerPosition.x, _lastServerPosition.y, GeoData.getInstance().getHeight(_lastServerPosition.x, _lastServerPosition.y, _lastServerPosition.z));
				activeChar.broadcastPacket(new ValidateLocation(activeChar));
			}
			else
				activeChar.teleToLocation(MapRegionTable.getInstance().getTeleToLocation(activeChar, MapRegionTable.TeleportWhereType.ClosestTown));
		else
			activeChar.teleToLocation(_lastServerPosition.x, _lastServerPosition.y, _lastServerPosition.z);
		*/

		else if(diff > 150)
			if(activeChar.isMoving || activeChar.isFollow)
				activeChar.broadcastPacket(new CharMoveToLocation(activeChar));
			else
				activeChar.broadcastPacket(new ValidateLocation(activeChar));

		if(activeChar.getParty() != null)
		{
			Point position = activeChar.getLastPartyPositionSent();
			if(activeChar.getDistance(position.x, position.y) < 500)
			{
				activeChar.getParty().broadcastToPartyMembers(activeChar, new PartyMemberPosition(activeChar));
				position.move(activeChar.getX(), activeChar.getY());
			}
		}

		//Pet disappears from status when range is > 2000
		if(activeChar.getPet() != null)
			if(activeChar.getPet().isInRange() && activeChar.getDistance(activeChar.getPet()) > Config.PLAYER_VISIBILITY)
				activeChar.getPet().setInRange(false);
			else if(!activeChar.getPet().isInRange() && activeChar.getDistance(activeChar.getPet()) <= Config.PLAYER_VISIBILITY - 500)
				activeChar.getPet().setInRange(true);

		activeChar.setLastClientPosition(new CharPosition(_x, _y, _z, activeChar.getHeading()));
		activeChar.setLastServerPosition(new CharPosition(activeChar.getX(), activeChar.getY(), activeChar.getZ(), activeChar.getHeading()));
	}

	@Override
	public String getType()
	{
		return _C__48_VALIDATEPOSITION;
	}
}