package L2jFT.game.network.serverpackets;

import javolution.util.FastList;
import L2jFT.game.model.L2TradeList;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.templates.L2Item;

import java.util.ArrayList;

/**
 * sample
 *
 * 1d 
 * 1e 00 00 00 			// ??
 * 5c 4a a0 7c 			// buy list id
 * 02 00				// item count
 *
 * 04 00 				// itemType1  0-weapon/ring/earring/necklace  1-armor/shield  4-item/questitem/adena
 * 00 00 00 00 			// objectid
 * 32 04 00 00 			// itemid
 * 00 00 00 00 			// count
 * 05 00 				// itemType2  0-weapon  1-shield/armor  2-ring/earring/necklace  3-questitem  4-adena  5-item
 * 00 00 			
 * 60 09 00 00			// price
 *
 * 00 00
 * 00 00 00 00 
 * b6 00 00 00 
 * 00 00 00 00 
 * 00 00 
 * 00 00 				 
 * 80 00 				//	body slot 	 these 4 values are only used if itemtype1 = 0 or 1
 * 00 00 				//
 * 00 00 				//
 * 00 00 				//
 * 50 c6 0c 00
 *

 * format   dd h (h dddhh hhhh d)	revision 377
 * format   dd h (h dddhh dhhh d)	revision 377
 *
 * @version $Revision: 1.4.2.1.2.3 $ $Date: 2005/03/27 15:29:57 $
 */
public class BuyList extends L2GameServerPacket
{
	private static final String _S__11_BUYLIST = "[S] 11 BuyList";
	private int _listId;
	private FastList<L2ItemInstance> _list;
	private int _money;
	private double _TaxRate = 0;

	public BuyList(L2TradeList list, int currentMoney)
	{
		_listId = list.getListId();
		_list = list.getItems();
		_money = currentMoney;
	}

	public BuyList(L2TradeList list, int currentMoney, double taxRate)
	{
		_listId = list.getListId();
		_list = list.getItems();
		_money = currentMoney;
		_TaxRate = taxRate;
	}

	public BuyList(ArrayList<L2ItemInstance> lst, int listId, int currentMoney)
	{
		_listId = listId;
		_list = new FastList<L2ItemInstance>(lst);
		_money = currentMoney;
	}

	@Override
	final public void runImpl()
	{}

	@Override
	protected final void writeImpl()
	{
		writeC(0x11);
		writeD(_money); // current money
		writeD(_listId);

		writeH(_list.size());

		for(FastList.Node<L2ItemInstance> n = _list.head(), end = _list.tail(); (n = n.getNext()) != end;)
		{
			L2ItemInstance item = n.getValue();
			if(item.getCountToSell() == 0 && item.getMaxCountToSell() != 0)
				continue;

			writeH(item.getItem().getType1()); // item type1
			writeD(item.getObjectId());
			writeD(item.getItemId());
			writeD(item.getCountToSell()); // max amount of items that a player can buy at a time (with this itemid)
			writeH(item.getItem().getType2()); // item type2
			writeH(0x00); // ?

			if(item.getItem().getType1() != L2Item.TYPE1_ITEM_QUESTITEM_ADENA)
			{
				writeD(item.getBodyPart()); // rev 415  slot    0006-lr.ear  0008-neck  0030-lr.finger  0040-head  0080-??  0100-l.hand  0200-gloves  0400-chest  0800-pants  1000-feet  2000-??  4000-r.hand  8000-r.hand
				writeH(item.getEnchantLevel()); // enchant level
			}
			else
			{
				writeD(0x00); // rev 415  slot    0006-lr.ear  0008-neck  0030-lr.finger  0040-head  0080-??  0100-l.hand  0200-gloves  0400-chest  0800-pants  1000-feet  2000-??  4000-r.hand  8000-r.hand
				writeH(0x00); // enchant level
			}

			writeH(0x00); // ?
			writeH(0x00);
			writeD((int) (item.getPriceToSell() * (1 + _TaxRate)));
		}
	}

	@Override
	public String getType()
	{
		return _S__11_BUYLIST;
	}
}