package L2jFT.game.network.serverpackets;

import L2jFT.Config;
import L2jFT.game.instancemanager.CursedWeaponsManager;
import L2jFT.game.model.Inventory;
import L2jFT.game.model.L2Player;

import java.util.logging.Logger;

public class CharInfo extends L2GameServerPacket
{
	private static final Logger _log = Logger.getLogger(CharInfo.class.getName());

	private static final String _S__03_CHARINFO = "[S] 03 CharInfo";
	private L2Player _cha;
	private Inventory _inv;
	private int _mAtkSpd, _pAtkSpd;
	private int _runSpd, _walkSpd, _swimRunSpd, _swimWalkSpd, _flRunSpd,
			_flWalkSpd, _flyRunSpd, _flyWalkSpd;
	private float _moveMultiplier;

	protected boolean logHandled()
	{
		return true;
	}

	/**
	 * @param _characters
	 */
	public CharInfo(L2Player cha)
	{
		if(cha == null || cha.isInvisible())
			return;
		_cha = cha;
		_inv = cha.getInventory();
		_mAtkSpd = _cha.getMAtkSpd();
		_pAtkSpd = _cha.getPAtkSpd();
		_moveMultiplier = _cha.getMovementSpeedMultiplier();
		_runSpd = (int) (_cha.getRunSpeed() / _moveMultiplier);
		_walkSpd = (int) (_cha.getWalkSpeed() / _moveMultiplier);
		_flRunSpd = _runSpd;
		_flWalkSpd = _walkSpd;
		_swimRunSpd = _cha.getSwimSpeed();
		_swimWalkSpd = _cha.getSwimSpeed();
	}

	@Override
	final public void runImpl()
	{}

	@Override
	protected final void writeImpl()
	{
		if(getClient().getActiveChar() == _cha)
		{
			_log.severe("You cant send CharInfo about his character to active user!!!");
			return;
		}

		if(_cha.isPolymorphed())
		{
			writeC(0x16);
			writeD(_cha.getObjectId());
			writeD(_cha.getPolyid() + 1000000); // npctype id
			writeD(_cha.getKarma() > 0 ? 1 : 0);
			writeD(_cha.getX());
			writeD(_cha.getY());
			writeD(_cha.getZ());
			writeD(_cha.getHeading());
			writeD(0x11);
			writeD(_mAtkSpd);
			writeD(_pAtkSpd);
			writeD(_runSpd);
			writeD(_walkSpd);
			writeD(_swimRunSpd/*0x32*/); // swimspeed
			writeD(_swimWalkSpd/*0x32*/); // swimspeed
			writeD(_flRunSpd);
			writeD(_flWalkSpd);
			writeD(_flyRunSpd);
			writeD(_flyWalkSpd);
			writeF(_moveMultiplier);
			writeF(_cha.getAttackSpeedMultiplier());
			writeF(_cha.getPolyRadius());
			writeF(_cha.getPolyHeight());
			writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_RHAND)); // right hand weapon
			writeD(0);
			writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_LHAND)); // left hand weapon
			writeC(1); // name above char 1=true ... ??
			writeC(_cha.isRunning() ? 1 : 0);
			writeC(_cha.isInCombat() ? 1 : 0);
			writeC(_cha.isAlikeDead() ? 1 : 0);
			writeC(_cha.isInvisible() ? 1 : 0); // invisible ?? 0=false  1=true   2=summoned (only works if model has a summon animation)
			writeS(_cha.getName());
			writeS(_cha.getTitle());
			writeD(0);
			writeD(_cha.getPvpFlag());
			writeD(_cha.getKarma()); // hmm karma ??

			writeH(_cha.getAbnormalEffect()); // C2
			writeH(0x00); // C2
			writeD(0); // C2
			writeD(0); // C2
			writeD(0); // C2
			writeD(0); // C2
			writeC(0); // C2
		}
		else
		{
			writeC(0x03);
			writeD(_cha.getX());
			writeD(_cha.getY());
			writeD(_cha.getZ());
			writeD(_cha.getHeading());
			writeD(_cha.getObjectId());

			/*
			 if(_cha.isGM())
			 writeS(_cha.isInvul() ? "*GM*" + _cha.getName() : "*" + _cha.getName() + "*");
			 else
			 */
			writeS(_cha.getName());
			writeD(_cha.getBaseTemplate().race.ordinal());
			writeD(_cha.getSex());
			writeD(_cha.getBaseClass());

			writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_DHAIR));
			writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_HEAD));
			writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_RHAND));
			writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_LHAND));
			writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_GLOVES));
			writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_CHEST));
			writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_LEGS));
			writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_FEET));
			writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_BACK));
			writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_LRHAND));
			writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_HAIR));
			writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_FACE));

			//KamaeL packets
			if(getClient().getRevision() >= 817)
			{
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
			}

			writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_DHAIR));
			writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_HEAD));
			writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_RHAND));
			writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_LHAND));
			writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_GLOVES));
			writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_CHEST));
			writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_LEGS));
			writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_FEET));
			writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_BACK));
			writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_LRHAND));
			writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_HAIR));
			writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_FACE));

			//KamaeL packets
			if(getClient().getRevision() >= 817)
			{
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
				writeD(0x00);
			}

			writeD(_cha.getPvpFlag());
			writeD(_cha.getKarma());

			writeD(_mAtkSpd);
			writeD(_pAtkSpd);

			writeD(_cha.getPvpFlag());
			writeD(_cha.getKarma());

			writeD(_runSpd);
			writeD(_walkSpd);
			writeD(_swimRunSpd/* 0x32 */); // swimspeed
			writeD(_swimWalkSpd/* 0x32 */); // swimspeed
			writeD(_flRunSpd);
			writeD(_flWalkSpd);
			writeD(_flyRunSpd);
			writeD(_flyWalkSpd);
			writeF(_cha.getMovementSpeedMultiplier()); // _cha.getProperMultiplier()
			writeF(_cha.getAttackSpeedMultiplier()); // _cha.getAttackSpeedMultiplier()
			writeF(_cha.getBaseTemplate().collisionRadius);
			writeF(_cha.getBaseTemplate().collisionHeight);

			writeD(_cha.getHairStyle());
			writeD(_cha.getHairColor());
			writeD(_cha.getFace());

			writeS(_cha.getTitle());
			writeD(_cha.getClanId());
			writeD(_cha.getClanCrestId());
			writeD(_cha.getAllyId());
			writeD(_cha.getAllyCrestId());

			writeD(0);

			writeC(_cha.isSitting() ? 0 : 1); // standing = 1 sitting = 0
			writeC(_cha.isRunning() ? 1 : 0); // running = 1 walking = 0
			writeC(_cha.isInCombat() ? 1 : 0);
			writeC(_cha.isAlikeDead() ? 1 : 0);

			writeC(_cha.isInvisible() ? 1 : 0); // invisible = 1 visible =0
			writeC(_cha.getMountType()); // 1 on strider 2 on wyfern 0 no mount
			writeC(_cha.getPrivateStoreType()); // 1 - sellshop

			writeH(_cha.getCubics().size());
			for(int id : _cha.getCubics().keySet())
				writeH(id);

			writeC(0x00); // find party members

			writeD(_cha.getAbnormalEffect());

			writeC(0 /* _cha.getRecomLeft() */);
			writeH(_cha.getPlayerAccess().IsGM ? 0 : _cha.getRecomHave());
			writeD(_cha.getClassId().getId());

			writeD(0 /* _cha.getMaxCp() */);
			writeD(0 /* (int) _cha.getCurrentCp() */);
			writeC(_cha.isMounted() ? 0 : _cha.getEnchantEffect());

			writeC(_cha.getTeam()); // team circle around feet 1 = Blue, 2 = red
			writeD(_cha.getClanCrestLargeId());
			writeC(_cha.isNoble() ? 1 : 0); // 0x01: symbol on char menu ctrl+I
			writeC(_cha.isHero() || _cha.isGM() && Config.GM_HERO_AURA ? 1 : 0); // 0x01: Hero Aura

			writeC(_cha.isFishing() ? 1 : 0);
			writeD(_cha.getFishx());
			writeD(_cha.getFishy());
			writeD(_cha.getFishz());

			writeD(_cha.getNameColor());

			// New C5            
			writeD(_cha.isRunning() ? 0x01 : 0x00); //changes the Speed display on Status Window

			writeD(_cha.getPledgeClass());
			writeD(0x00); //C5 ??
			writeD(_cha.getTitleColor());
			writeD(0x00); //C5 ??
			if(_cha.isCursedWeaponEquipped())
				writeD(CursedWeaponsManager.getInstance().getLevel(_cha.getCursedWeaponEquippedId()));
			else
				writeD(0x00);
		}
	}

	@Override
	public String getType()
	{
		return _S__03_CHARINFO + ": " + _cha;
	}
}
