package L2jFT.game.network.serverpackets;

import L2jFT.game.instancemanager.SiegeManager;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2SiegeClan;
import L2jFT.game.model.entity.siege.Siege;
import L2jFT.game.model.instances.L2MonsterInstance;

/**
 * sample
 * 0b 
 * 952a1048		objectId 
 * 00000000 00000000 00000000 00000000 00000000 00000000
 * format  ddddddd   rev 417
 */
public class Die extends L2GameServerPacket
{
	private static final String _S__06_DIE = "[S] 06 Die";
	private int _chaId;
	private boolean _fake;
	private boolean _sweepable;
	private int _access;
	private L2jFT.game.model.L2Clan _clan;
	L2Character _cha;

	/**
	 * @param _characters
	 */
	public Die(L2Character cha)
	{
		_cha = cha;
		if(cha.isPlayer)
		{
			L2Player player = (L2Player) cha;
			_access = player.getPlayerAccess().ResurectFixed ? 0x01 : 0x00;
			_clan = player.getClan();
		}
		_chaId = cha.getObjectId();
		_fake = !cha.isDead();
		if(cha.isMonster)
			_sweepable = ((L2MonsterInstance) cha).isSweepActive();

	}

	@Override
	final public void runImpl()
	{
	// no long-running tasks
	}

	@Override
	protected final void writeImpl()
	{
		if(_fake)
			return;

		writeC(0x06);

		writeD(_chaId);
		// NOTE:
		// 6d 00 00 00 00 - to nearest village
		// 6d 01 00 00 00 - to hide away
		// 6d 02 00 00 00 - to castle
		// 6d 03 00 00 00 - to siege HQ
		// sweepable
		// 6d 04 00 00 00 - FIXED

		writeD(0x01); // 6d 00 00 00 00 - to nearest village
		if(_clan != null)
		{
			L2SiegeClan siegeClan = null;
			Siege siege = SiegeManager.getInstance().getSiege(_cha);
			if(siege != null && siege.isInProgress())
				siegeClan = siege.getAttackerClan(_clan);

			writeD(_clan.getHasHideout() > 0 ? 0x01 : 0x00); // 6d 01 00 00 00 - to hide away
			writeD(_clan.getHasCastle() > 0 ? 0x01 : 0x00); // 6d 02 00 00 00 - to castle
			writeD(siegeClan != null && siegeClan.getFlag() != null ? 0x01 : 0x00); // 6d 03 00 00 00 - to siege HQ
		}
		else
		{
			writeD(0x00); // 6d 01 00 00 00 - to hide away
			writeD(0x00); // 6d 02 00 00 00 - to castle
			writeD(0x00); // 6d 03 00 00 00 - to siege HQ
		}
		writeD(_sweepable ? 0x01 : 0x00); // sweepable  (blue glow)
		writeD(_access); // 6d 04 00 00 00 - to FIXED
	}

	@Override
	public String getType()
	{
		return _S__06_DIE;
	}
}
