package L2jFT.game.network.serverpackets;

import L2jFT.game.model.instances.L2ItemInstance;

/**
 * 16 
 * d6 6d c0 4b		player id who dropped it 
 * ee cc 11 43 		object id
 * 39 00 00 00 		item id
 * 8f 14 00 00 		x
 * b7 f1 00 00 		y
 * 60 f2 ff ff 		z
 * 01 00 00 00 		show item-count 1=yes
 * 7a 00 00 00      count                                         .
 *
 * format  dddddddd    rev 377
 *         ddddddddd   rev 417
 *
 * @version $Revision: 1.3.2.1.2.3 $ $Date: 2005/03/27 15:29:39 $
 */
public class DropItem extends L2GameServerPacket
{
	private static final String _S__0C_DROPITEM = "[S] 0C DropItem";
	private L2ItemInstance _item;
	private int _playerId;

	/**
	 * Constructor of the DropItem server packet
	 * @param item : L2ItemInstance designating the item
	 * @param playerId : int designating the player ID who dropped the item
	 */
	public DropItem(L2ItemInstance item, int playerId)
	{
		_item = item;
		_playerId = playerId;
	}

	@Override
	final public void runImpl()
	{
	// no long-running tasks
	}

	@Override
	protected final void writeImpl()
	{
		writeC(0x0C);
		writeD(_playerId);
		writeD(_item.getObjectId());
		writeD(_item.getItemId());

		writeD(_item.getX());
		writeD(_item.getY());
		writeD(_item.getZ());
		// only show item count if it is a stackable item
		if(_item.isStackable())
			writeD(0x01);
		else
			writeD(0x00);
		writeD(_item.getCount());

		writeD(1); // unknown
	}

	/* (non-Javadoc)
	 * @see L2jFT.game.network.serverpackets.L2GameServerPacket#getType()
	 */
	@Override
	public String getType()
	{
		return _S__0C_DROPITEM;// + "; object_id = "+_item.getObjectId();
	}

}
