package L2jFT.game.network.serverpackets;

import L2jFT.game.model.L2Player;

/**
 * @Author: Death
 * @Date: 31/1/2007
 * @Time: 16:40:20
 */
public class EtcStatusUpdate extends L2GameServerPacket
{

	/**
	 *
	 * Packet for lvl 3 client buff line
	 *
	 * Example:(C4)
	 * F3 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 - empty statusbar
	 * F3 01 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 - increased force lvl 1
	 * F3 00 00 00 00 01 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 - weight penalty lvl 1
	 * F3 00 00 00 00 00 00 00 00 01 00 00 00 00 00 00 00 00 00 00 00 - chat banned
	 * F3 00 00 00 00 00 00 00 00 00 00 00 00 01 00 00 00 00 00 00 00 - Danger Area lvl 1
	 * F3 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 01 00 00 00 - lvl 1 grade penalty
	 *
	 * packet format: cdd //and last three are ddd???
	 *
	 * Some test results:
	 * F3 07 00 00 00 04 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 - lvl 7 increased force lvl 4 weight penalty
	 *
	 * Example:(C5 709)
	 * F3 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 01 00 00 00 0F 00 00 00 - lvl 1 charm of courage lvl 15 Death Penalty
	 *
	 *
	 * NOTE:
	 * End of buff:
	 * You must send empty packet
	 * F3 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
	 * to remove the statusbar or just empty value to remove some icon.
	 */

	private static final String _S__F3_ETCSTATUSUPDATE = "[S] F3 EtcStatusUpdate";
	L2Player _player;

	public EtcStatusUpdate(L2Player player)
	{
		_player = player;
	}

	@Override
	public void runImpl()
	{}

	@Override
	protected final void writeImpl()
	{
		writeC(0xf3); //Packet type
		writeD(_player.getIncreasedForce()); // skill id 4271, 7 lvl
		writeD(_player.getWeightPenalty()); // skill id 4270, 4 lvl
		writeD(_player.getMessageRefusal() || _player.getNoChannel() != 0 ? 1 : 0); //skill id 4269, 1 lvl
		writeD(_player.isInDangerArea() ? 1 : 0); // skill id 4268, 1 lvl
		writeD(_player.getexpertisePenalty()); // skill id 4267, 1 lvl at off c4 server scripts
		writeD(_player.getCharmOfCourage() ? 1 : 0); //Charm of Courage, "Prevents experience value decreasing if killed during a siege war".
		writeD(_player.getDeathPenalty().getLevel()); //Death Penalty max lvl 15, "Combat ability is decreased due to death."
	}

	@Override
	public String getType()
	{
		return _S__F3_ETCSTATUSUPDATE;
	}
}