package L2jFT.game.network.serverpackets;

import javolution.util.FastList;
import L2jFT.database.DatabaseUtils;
import L2jFT.database.FiltredPreparedStatement;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.database.ThreadConnection;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;

import java.sql.ResultSet;
import java.util.logging.Level;
import java.util.logging.Logger;

public class FriendList extends L2GameServerPacket
{
	private static Logger _log = Logger.getLogger(FriendList.class.getName());
	private static final String _S__FA_FRIENDLIST = "[S] FA FriendList";
	private static L2Player _cha;
	private static boolean _message = false;
	private static boolean _packet = false;

	public FriendList(L2Player cha)
	{
		_cha = cha;
		_message = true;
		_packet = false;
	}

	public FriendList(L2Player cha, boolean sendMessage)
	{
		_cha = cha;
		_message = sendMessage;
		_packet = true;
	}

	@Override
	final public void runImpl()
	{
	// no long-running tasks
	}

	@Override
	protected final void writeImpl()
	{

		FastList<Integer> id = new FastList<Integer>();
		FastList<String> name = new FastList<String>();
		FastList<Boolean> online = new FastList<Boolean>();

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		L2Player activeChar = getClient().getActiveChar();
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("select friend_id,friend_name from character_friends where char_id=?");
			statement.setInt(1, activeChar.getObjectId());
			rset = statement.executeQuery();

			if(_message)
				_cha.sendPacket(new SystemMessage(SystemMessage._FRIENDS_LIST_));
			while(rset.next())
			{
				int objectId = rset.getInt("friend_id");
				String friendName = rset.getString("friend_name");
				L2Player friend = L2World.getPlayer(friendName);
				name.add(friendName);
				id.add(objectId);
				if(friend == null)
				{
					if(_message)
						_cha.sendPacket(new SystemMessage(SystemMessage.S1_CURRENTLY_OFFLINE).addString(friendName));
					online.add(false);
				}
				else
				{
					if(_message)
						_cha.sendPacket(new SystemMessage(SystemMessage.S1_CURRENTLY_ONLINE).addString(friendName));
					online.add(true);
				}
			}
			if(_message)
				_cha.sendPacket(new SystemMessage(SystemMessage.__EQUALS__));
		}
		catch(Exception e)
		{
			_log.log(Level.WARNING, "Error in friendlist ", e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}

		if(!_packet)
			return;

		writeC(0xfa);
		writeD(id.size());
		if(id.size() > 0)
			for(int i = 0; i < id.size(); i++)
			{
				writeD(0);
				writeS(name.get(i)); //name
				writeD(online.get(i) ? 1 : 0); //online or offline
				writeD(id.get(i)); //object_id
			}
	}

	@Override
	public String getType()
	{
		return _S__FA_FRIENDLIST;
	}
}