package L2jFT.game.network.serverpackets;

import L2jFT.Config;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Summon;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.model.instances.L2TamedBeastInstance;

public class NpcInfo extends L2GameServerPacket
{
	//   ddddddddddddddddddffffdddcccccSSddd dddddc

	private static final String _S__16_NPCINFO = "[S] 16 NpcInfo";
	private L2Character _cha;
	private int _npcId;
	private boolean _isAttackable, _isSummoned;
	private int _mAtkSpd, _pAtkSpd;
	private int _runSpd, _walkSpd, _swimRunSpd, _swimWalkSpd, _flRunSpd,
			_flWalkSpd, _flyRunSpd, _flyWalkSpd;
	private int _rhand, _lhand;
	private int collisionHeight, collisionRadius;
	private String _name = "";
	private String _title = "";

	/**
	 * @param _characters
	 */
	public NpcInfo(L2NpcInstance cha, L2Character attacker)
	{
		_cha = cha;
		_npcId = cha.getTemplate().displayId != 0 ? cha.getTemplate().displayId : cha.getTemplate().npcId;
		_isAttackable = cha.isAutoAttackable(attacker);
		_rhand = cha.getTemplate().rhand;
		_lhand = cha.getTemplate().lhand;
		_isSummoned = cha.isSpawned();
		collisionHeight = _cha.getTemplate().collisionHeight;
		collisionRadius = _cha.getTemplate().collisionRadius;
		if(Config.SERVER_SIDE_NPC_NAME || cha.getTemplate().displayId != 0)
			_name = cha.getName();
		if(Config.SERVER_SIDE_NPC_TITLE || cha.getTemplate().displayId != 0)
		{
			if(_cha.isMonster && Config.SERVER_SIDE_NPC_TITLE_WITH_LVL && !(_cha instanceof L2TamedBeastInstance))
				_title = "Lv" + cha.getLevel() + (cha.getAggroRange() != 0 ? "* " : " ");
			if(_cha.isMonster && Config.SERVER_SIDE_NPC_TITLE_WITH_SOCIAL && !(_cha instanceof L2TamedBeastInstance))
				_title = _title + (cha.getFactionRange() != 0 ? "+ " : " ");
			_title = _title + cha.getTitle();
		}
		_mAtkSpd = _cha.getMAtkSpd();
		_pAtkSpd = _cha.getPAtkSpd();
		_runSpd = _cha.getRunSpeed();
		_walkSpd = _cha.getWalkSpeed();
		_swimRunSpd = _flRunSpd = _flyRunSpd = _runSpd;
		_swimWalkSpd = _flWalkSpd = _flyWalkSpd = _walkSpd;
	}

	public NpcInfo(L2Summon cha, L2Character attacker)
	{
		if(cha == null || cha.getOwner() != null && cha.getOwner().isInvisible())
			return;
		_cha = cha;
		_npcId = cha.getTemplate().npcId;
		_isAttackable = cha.isAutoAttackable(attacker); //(cha.getKarma() > 0);
		_rhand = 0;
		_lhand = 0;
		_isSummoned = cha.isShowSummonAnimation();
		collisionHeight = _cha.getTemplate().collisionHeight;
		collisionRadius = _cha.getTemplate().collisionRadius;
		if(Config.SERVER_SIDE_NPC_NAME || cha.isPet)
			_name = _cha.getName();
		_title = cha.getTitle();

		_mAtkSpd = _cha.getMAtkSpd();
		_pAtkSpd = _cha.getPAtkSpd();
		_runSpd = _cha.getRunSpeed();
		_walkSpd = _cha.getWalkSpeed();
		_swimRunSpd = _flRunSpd = _flyRunSpd = _runSpd;
		_swimWalkSpd = _flWalkSpd = _flyWalkSpd = _walkSpd;
	}

	@Override
	final public void runImpl()
	{}

	@Override
	protected final void writeImpl()
	{
		writeC(0x16);
		writeD(_cha.getObjectId());
		writeD(_npcId + 1000000); // npctype id c4
		writeD(_isAttackable ? 1 : 0);
		writeD(_cha.getX());
		writeD(_cha.getY());
		writeD(_cha.getZ());
		writeD(_cha.getHeading());
		writeD(0x00);
		writeD(_mAtkSpd);
		writeD(_pAtkSpd);
		writeD(_runSpd);
		writeD(_walkSpd);
		writeD(_swimRunSpd/*0x32*/); // swimspeed
		writeD(_swimWalkSpd/*0x32*/); // swimspeed
		writeD(_flRunSpd);
		writeD(_flWalkSpd);
		writeD(_flyRunSpd);
		writeD(_flyWalkSpd);
		writeF(1.1); // Interlude
		writeF(1/*_cha.getAttackSpeedMultiplier()*/);
		writeF(collisionRadius);
		writeF(collisionHeight);
		writeD(_rhand); // right hand weapon
		writeD(0);
		writeD(_lhand); // left hand weapon
		writeC(1); // name above char 1=true ... ??
		writeC(_cha.isRunning() ? 1 : 0);
		writeC(_cha.isInCombat() ? 1 : 0);
		writeC(_cha.isAlikeDead() ? 1 : 0);
		writeC(_isSummoned ? 2 : 0); // invisible ?? 0=false  1=true   2=summoned (only works if model has a summon animation)
		writeS(_name);
		writeS(_title);
		writeD(0);
		writeD(_cha.getPvpFlag());
		writeD(_cha.getKarma()); // hmm karma ??
		writeD(_cha.getAbnormalEffect()); // C2
		writeD(0000); // C2
		writeD(0000); // C2
		writeD(0000); // C2
		writeD(0000); // C2
		writeC(0000); // C2
		writeC(_cha.getTeam()); // team aura 1-blue, 2-red
		/* Вот дальше сомнительно, из пакета с оффа:
		 00 
		 00 00 00 00 
		 00 00 1C 40 4619567317775286272? 1075576832? 16412? 28 64?
		 00 00 00 00 
		 00 00 37 40 4627167142146473984? 1077346304? 16439? 55 64?
		 00 00 00 00  
		 */
		writeF(0x00); // C4
		writeF(0x00); // C4
		writeD(0x00); // C4
	}

	@Override
	public String getType()
	{
		return _S__16_NPCINFO + (_cha != null ? " about " + _cha : "");
	}
}