package L2jFT.game.network.serverpackets;

import L2jFT.Config;
import L2jFT.game.model.instances.L2ItemInstance;

import java.util.ArrayList;
import java.util.logging.Logger;

/**
 * This class ...
 *
 * @author Yme
 * @version $Revision: 1.3.2.1.2.5 $ $Date: 2005/03/27 15:29:57 $
 */
public class PetInventoryUpdate extends L2GameServerPacket
{
	private static Logger _log = Logger.getLogger(InventoryUpdate.class.getName());
	private static final String _S__B3_INVENTORYUPDATE = "[S] B3 InventoryUpdate";
	private ArrayList<L2ItemInstance> _items;

	public PetInventoryUpdate()
	{
		_items = new ArrayList<L2ItemInstance>();
	}

	/**
	 * @param items
	 */
	public PetInventoryUpdate(ArrayList<L2ItemInstance> items)
	{
		_items = items;
	}

	public PetInventoryUpdate addNewItem(L2ItemInstance item)
	{
		item.setLastChange(L2ItemInstance.ADDED);
		_items.add(item);
		return this;
	}

	public PetInventoryUpdate addModifiedItem(L2ItemInstance item)
	{
		item.setLastChange(L2ItemInstance.MODIFIED);
		_items.add(item);
		return this;
	}

	public PetInventoryUpdate addRemovedItem(L2ItemInstance item)
	{
		item.setLastChange(L2ItemInstance.REMOVED);
		_items.add(item);
		return this;
	}

	@Override
	final public void runImpl()
	{
		if(Config.DEBUG)
			for(L2ItemInstance temp : _items)
				_log.fine("oid:" + Integer.toHexString(temp.getObjectId()) + " item:" + temp.getItem().getName() + " last change:" + temp.getLastChange());
	}

	@Override
	protected final void writeImpl()
	{
		writeC(0xB3);
		int count = _items.size();
		writeH(count);
		for(L2ItemInstance temp : _items)
		{
			writeH(temp.getLastChange());
			writeH(temp.getItem().getType1()); // item type1
			writeD(temp.getObjectId());
			writeD(temp.getItemId());
			writeD(temp.getCount());
			writeH(temp.getItem().getType2()); // item type2
			writeH(0x00); // ?

			if(temp.isEquipped())
				writeH(0x01);
			else
				writeH(0x00);
			//			writeH(temp.getItem().getBodyPart());	// rev 377   slot    0006-lr.ear  0008-neck  0030-lr.finger  0040-head  0080-??  0100-l.hand  0200-gloves  0400-chest  0800-pants  1000-feet  2000-??  4000-r.hand  8000-r.hand
			writeD(temp.getBodyPart()); // rev 415   slot    0006-lr.ear  0008-neck  0030-lr.finger  0040-head  0080-??  0100-l.hand  0200-gloves  0400-chest  0800-pants  1000-feet  2000-??  4000-r.hand  8000-r.hand
			writeH(temp.getEnchantLevel()); // enchant level
			writeH(0x00); // ?
		}
	}

	/* (non-Javadoc)
	 * @see L2jFT.game.network.serverpackets.L2GameServerPacket#getType()
	 */
	@Override
	public String getType()
	{
		return _S__B3_INVENTORYUPDATE;
	}
}
