package L2jFT.game.network.serverpackets;

import L2jFT.game.model.L2Player;
import L2jFT.game.model.TradeItem;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.tables.ItemTable;
import L2jFT.game.templates.L2Item;

import java.util.HashSet;
import java.util.List;

public class PrivateStoreListBuy extends L2GameServerPacket
{
	private static final String _S__B8_PRIVATESTORELISTBUY = "[S] B8 PrivateStoreListBuy";
	private L2Player _buyer;
	private L2Player _seller;
	private HashSet<TradeItem> _buyerslist;

	/**
	 * Список вещей в личном магазине покупки, показываемый продающему
	 * @param seller
	 * @param storePlayer
	 */
	@SuppressWarnings("unchecked")
	public PrivateStoreListBuy(L2Player seller, L2Player storePlayer)
	{
		_seller = seller;
		_buyer = storePlayer;

		List<L2ItemInstance> sellerItems = _seller.getInventory().getItemsList();
		_buyerslist = (HashSet<TradeItem>) _buyer.getBuyList().clone();

		for(TradeItem buyListItem : _buyerslist)
			buyListItem.setTempValue(0);

		for(L2ItemInstance sellerItem : sellerItems)
			for(TradeItem buyListItem : _buyerslist)
				if(sellerItem.getItemId() == buyListItem.getItemId() && sellerItem.isAvailable(_seller, false) && sellerItem.canBeTraded(_seller))
				{
					buyListItem.setTempValue(Math.min(buyListItem.getCount(), sellerItem.getCount()));
					continue;
				}
	}

	@Override
	final public void runImpl()
	{}

	@Override
	protected final void writeImpl()
	{
		writeC(0xB8);
		writeD(_buyer.getObjectId());
		writeD(_seller.getAdena());
		writeD(_buyerslist.size());

		for(TradeItem buyersitem : _buyerslist)
		{
			L2Item tmp = ItemTable.getInstance().getTemplate(buyersitem.getItemId());

			writeD(buyersitem.getObjectId());
			writeD(buyersitem.getItemId());
			writeH(buyersitem.getEnchantLevel());
			writeD(buyersitem.getTempValue());

			writeD(tmp.getReferencePrice());
			writeH(0);

			writeD(tmp.getBodyPart());
			writeH(tmp.getType2());
			writeD(buyersitem.getOwnersPrice());

			writeD(buyersitem.getCount());
		}
	}

	@Override
	public String getType()
	{
		return _S__B8_PRIVATESTORELISTBUY;
	}
}