package L2jFT.game.network.serverpackets;

import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2TradeList;
import L2jFT.game.model.TradeItem;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.tables.ItemTable;
import L2jFT.game.templates.L2Item;

import java.util.HashSet;

/**
 * 3 section to this packet
 * 1)playerinfo which is always sent
 * dd
 *
 * 2)list of items which can be added to sell
 * d(hhddddhhhd)
 *
 * 3)list of items which have already been setup 
 * for sell in previous sell private store sell manageent 
 * d(hhddddhhhdd)
 */
public class PrivateStoreManageListSell extends L2GameServerPacket
{

	private static final String _S__9A_PRIVATESTOREMANAGELISTSELL = "[S] 9A PrivateStoreManageListSell";
	private L2Player _seller;
	private final boolean _sellMode;
	private L2TradeList _list = new L2TradeList(0);
	private HashSet<TradeItem> _sellList;

	/**
	 * Окно управления личным магазином покупки
	 * @param seller
	 */
	public PrivateStoreManageListSell(L2Player seller)
	{
		_seller = seller;
		_sellMode = seller.getSellMode();

		// Проверяем список вещей в инвентаре, если вещь остутствует - убираем из списка продажи
		_sellList = _seller.getSellList();
		for(TradeItem i : _seller.getSellList())
		{
			L2ItemInstance inst = _seller.getInventory().getItemByObjectId(i.getObjectId());
			if(i.getCount() <= 0 || inst == null || inst.isWear())
				continue;
			if(inst.getCount() < i.getCount())
				i.setCount(inst.getCount());
			_sellList.add(i);
		}

		// Строим список вещей, годных для продажи имеющихся в инвентаре
		for(L2ItemInstance item : _seller.getInventory().getItemsList())
			if(!item.isEquipped() && item.getItem().getType2() != L2Item.TYPE2_QUEST && item.getItem().getType2() != L2Item.TYPE2_MONEY && item.canBeTraded(_seller))
				_list.addItem(item);

		//Убираем совпадения между списками, в сумме оба списка должны совпадать с содержимым инвентаря
		if(_sellList.size() > 0)
			for(TradeItem temp2 : _sellList)
				_list.removeItem(temp2.getObjectId(), temp2.getCount());
	}

	@Override
	final public void runImpl()
	{}

	@Override
	protected final void writeImpl()
	{
		writeC(0x9A);
		//section 1 
		writeD(_seller.getObjectId());
		writeD(_sellMode ? 1 : 0);
		writeD(_seller.getAdena());

		//Список имеющихся вещей
		int count = _list.getItems().size();
		writeD(count);
		for(L2ItemInstance temp : _list.getItems())
		{
			writeD(temp.getItem().getType2());
			writeD(temp.getObjectId());
			writeD(temp.getItemId());
			writeD(temp.getCount());
			writeH(0);
			writeH(temp.getEnchantLevel());//enchant lvl
			writeH(0);
			writeD(temp.getBodyPart());
			writeD(temp.getPriceToBuy()); //store price
		}

		//Список вещей уже поставленых на продажу
		count = _sellList.size();
		writeD(count);
		for(TradeItem temp2 : _sellList)
		{
			L2Item tempItem = ItemTable.getInstance().getTemplate(temp2.getItemId());
			writeD(tempItem.getType2());
			writeD(temp2.getObjectId());
			writeD(temp2.getItemId());
			writeD(temp2.getCount());
			writeH(0);
			writeH(temp2.getEnchantLevel());//enchant lvl
			writeH(0);
			writeD(tempItem.getBodyPart());
			writeD(temp2.getOwnersPrice());//your price
			writeD(temp2.getStorePrice()); //store price
		}
	}

	@Override
	public String getType()
	{
		return _S__9A_PRIVATESTOREMANAGELISTSELL;
	}
}
