package L2jFT.game.network.serverpackets;

import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2SiegeClan;
import L2jFT.game.model.entity.Castle;
import L2jFT.game.model.entity.ClanHall;
import L2jFT.game.tables.ClanTable;
import L2jFT.extensions.scripts.Functions;

import java.util.ArrayList;

/**
 * Populates the Siege Attacker List in the SiegeInfo Window<BR>
 * <BR>
 * packet type id 0xca<BR>
 * format: cddddddd + dSSdddSSd<BR>
 * <BR>
 * c = ca<BR>
 * d = CastleID<BR>
 * d = unknow (0x00)<BR>
 * d = unknow (0x01)<BR>
 * d = unknow (0x00)<BR>
 * d = Number of Attackers Clans?<BR>
 * d = Number of Attackers Clans<BR>
 * { //repeats<BR>
 * d = ClanID<BR>
 * S = ClanName<BR>
 * S = ClanLeaderName<BR>
 * d = ClanCrestID<BR>
 * d = signed time (seconds)<BR>
 * d = AllyID<BR>
 * S = AllyName<BR>
 * S = AllyLeaderName<BR>
 * d = AllyCrestID<BR>
 *
 * @author KenM
 */
public class SiegeAttackerList extends L2GameServerPacket
{
	private static final String _S__CA_SIEGEATTACKERLIST = "[S] CA SiegeAttackerList";
	private Castle _Castle;
	private ClanHall _ClanHall;

	public SiegeAttackerList(Castle castle)
	{
		_Castle = castle;
	}

	public SiegeAttackerList(ClanHall ch)
	{
		_ClanHall = ch;
	}

	@Override
	final public void runImpl()
	{}

	@Override
	@SuppressWarnings("unchecked")
	protected final void writeImpl()
	{
		writeC(0xCA);

		if(_Castle != null)
		{
			writeD(_Castle.getCastleId());

			writeD(0x00); //0 
			writeD(0x01); //1
			writeD(0x00); //0

			writeD(_Castle.getSiege().getAttackerClans().size());
			writeD(_Castle.getSiege().getAttackerClans().size());

			for(L2SiegeClan siegeclan : _Castle.getSiege().getAttackerClans().values())
			{
				L2Clan clan = ClanTable.getInstance().getClan(siegeclan.getClanId());
				if(clan == null)
					continue;

				writeD(clan.getClanId());
				writeS(clan.getName());
				writeS(clan.getLeaderName());
				writeD(clan.getCrestId());
				writeD(0x00); // signed time (seconds) (not storated by L2F)
				writeD(clan.getAllyId());
				if(clan.getAlliance() != null)
				{
					writeS(clan.getAlliance().getAllyName());
					writeS(clan.getAlliance().getAllyLeaderName()); // AllyLeaderName
					writeD(clan.getAlliance().getAllyCrestId());
				}
				else
				{
					writeS("");
					writeS("");
					writeD(0);
				}
			}
		}
		else
		{
			writeD(_ClanHall.getId());

			writeD(0x00); //0 
			writeD(0x01); //1
			writeD(0x00); //0

			ArrayList<L2Clan> attackers = (ArrayList<L2Clan>) Functions.callScripts("events.clanhall.CHSiegeManager", "getAttackersForSiege", new Object[] { new Integer(_ClanHall.getId()) }, null);
			if(attackers == null)
			{
				writeD(0);
				writeD(0);
				return;
			}

			writeD(attackers.size());
			writeD(attackers.size());

			for(L2Clan siegeclan : attackers)
			{
				L2Clan clan = ClanTable.getInstance().getClan(siegeclan.getClanId());
				if(clan == null)
					continue;

				writeD(clan.getClanId());
				writeS(clan.getName());
				writeS(clan.getLeaderName());
				writeD(clan.getCrestId());
				writeD(0x00); // signed time (seconds) (not storated by L2F)
				writeD(clan.getAllyId());
				if(clan.getAlliance() != null)
				{
					writeS(clan.getAlliance().getAllyName());
					writeS(clan.getAlliance().getAllyLeaderName()); // AllyLeaderName
					writeD(clan.getAlliance().getAllyCrestId());
				}
				else
				{
					writeS("");
					writeS("");
					writeD(0);
				}
			}
		}
	}

	@Override
	public String getType()
	{
		return _S__CA_SIEGEATTACKERLIST;
	}
}
