package L2jFT.game.network.serverpackets;

import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2SiegeClan;
import L2jFT.game.model.entity.Castle;
import L2jFT.game.model.entity.ClanHall;
import L2jFT.game.tables.ClanTable;

/**
 * Populates the Siege Defender List in the SiegeInfo Window<BR>
 * <BR>
 * packet type id 0xcb<BR>
 * format: cddddddd + dSSdddSSd<BR>
 * <BR>
 * c = 0xcb<BR>
 * d = CastleID<BR>
 * d = unknow (0x00)<BR>
 * d = unknow (0x01)<BR>
 * d = unknow (0x00)<BR>
 * d = Number of Defending Clans?<BR>
 * d = Number of Defending Clans<BR>
 * { //repeats<BR>
 * d = ClanID<BR>
 * S = ClanName<BR>
 * S = ClanLeaderName<BR>
 * d = ClanCrestID<BR>
 * d = signed time (seconds)<BR>
 * d = Type -> Owner = 0x01 || Waiting = 0x02 || Accepted = 0x03<BR> 
 * d = AllyID<BR>
 * S = AllyName<BR>
 * S = AllyLeaderName<BR>
 * d = AllyCrestID<BR>
 *
 * @author KenM
 */
public class SiegeDefenderList extends L2GameServerPacket
{
	private static final String _S__CB_SIEGEDEFENDERLIST = "[S] cb SiegeDefenderList";
	//private static Logger _log = Logger.getLogger(SiegeDefenderList.class.getName());
	private Castle _castle;
	private ClanHall _ClanHall;

	public SiegeDefenderList(Castle castle)
	{
		_castle = castle;
	}

	public SiegeDefenderList(ClanHall ch)
	{
		_ClanHall = ch;
	}

	@Override
	final public void runImpl()
	{
	// no long-running tasks
	}

	@Override
	protected final void writeImpl()
	{
		writeC(0xCB);
		if(_castle != null)
			writeD(_castle.getCastleId());
		else
			writeD(_ClanHall.getId());
		writeD(0x00); // 0
		writeD(0x01); // 1
		writeD(0x00); // 0
		int size = 0;
		if(_castle != null)
			size = _castle.getSiege().getDefenderClans().size() + _castle.getSiege().getDefenderWaitingClans().size();
		if(size > 0)
		{
			L2Clan clan;

			writeD(size);
			writeD(size);
			// Listing the Lord and the approved clans
			for(L2SiegeClan siegeclan : _castle.getSiege().getDefenderClans().values())
			{
				clan = ClanTable.getInstance().getClan(siegeclan.getClanId());
				if(clan == null)
					continue;

				writeD(clan.getClanId());
				writeS(clan.getName());
				writeS(clan.getLeaderName());
				writeD(clan.getCrestId());
				writeD(0x00); // signed time (seconds) (not storated by L2F)
				switch(siegeclan.getTypeId())
				{
					case OWNER:
						writeD(0x01); // owner
						break;
					case DEFENDER_PENDING:
						writeD(0x02); // waiting approved
						break;
					case DEFENDER:
						writeD(0x03); // approved
						break;
					default:
						writeD(0x00);
						break;
				}
				writeD(clan.getAllyId());
				if(clan.getAlliance() != null)
				{
					writeS(clan.getAlliance().getAllyName());
					writeS(clan.getAlliance().getAllyLeaderName()); // AllyLeaderName
					writeD(clan.getAlliance().getAllyCrestId());
				}
				else
				{
					writeS("");
					writeS("");
					writeD(0);
				}
			}
			for(L2SiegeClan siegeclan : _castle.getSiege().getDefenderWaitingClans().values())
			{
				clan = ClanTable.getInstance().getClan(siegeclan.getClanId());
				writeD(clan.getClanId());
				writeS(clan.getName());
				writeS(clan.getLeaderName());
				writeD(clan.getCrestId());
				writeD(0x00); // signed time (seconds) (not storated by L2F)
				writeD(0x02); // waiting approval
				writeD(clan.getAllyId());
				if(clan.getAlliance() != null)
				{
					writeS(clan.getAlliance().getAllyName());
					writeS(clan.getAlliance().getAllyLeaderName()); //AllyLeaderName
					writeD(clan.getAlliance().getAllyCrestId());
				}
				else
				{
					writeS("");
					writeS("");
					writeD(0);
				}
			}
		}
		else
		{
			writeD(0x00);
			writeD(0x00);
		}
	}

	@Override
	public String getType()
	{
		return _S__CB_SIEGEDEFENDERLIST;
	}
}
