package L2jFT.game.network.serverpackets;

import L2jFT.game.model.L2Alliance;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.entity.Castle;
import L2jFT.game.model.entity.ClanHall;
import L2jFT.game.tables.ClanTable;

import java.util.Calendar;
import java.util.logging.Logger;

/**
 * Shows the Siege Info<BR>
 * <BR>
 * packet type id 0xc9<BR>
 * format: cdddSSdSdd<BR>
 * <BR>
 * c = c9<BR>
 * d = CastleID<BR>
 * d = Show Owner Controls (0x00 default || >=0x02(mask?) owner)<BR>
 * d = Owner ClanID<BR>
 * S = Owner ClanName<BR>
 * S = Owner Clan LeaderName<BR>
 * d = Owner AllyID<BR>
 * S = Owner AllyName<BR>
 * d = current time (seconds)<BR>
 * d = Siege time (seconds) (0 for selectable)<BR>
 * d = (UNKNOW) Siege Time Select Related
 * @author KenM
 */
public class SiegeInfo extends L2GameServerPacket
{
	private static final String _S__C9_SIEGEINFO = "[S] C9 SiegeInfo";
	private static Logger _log = Logger.getLogger(SiegeInfo.class.getName());
	private Castle _Castle;
	private ClanHall _ClanHall;
	private int startTime;

	public SiegeInfo(Castle castle)
	{
		_Castle = castle;
	}

	public SiegeInfo(ClanHall clanhall, int time)
	{
		_ClanHall = clanhall;
		startTime = time;
	}

	@Override
	final public void runImpl()
	{
	// no long-running tasks
	}

	@Override
	protected final void writeImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar == null)
			return;

		writeC(0xC9);
		if(_Castle != null)
		{
			writeD(_Castle.getCastleId());
			writeD(_Castle.getOwnerId() == activeChar.getClanId() && activeChar.isClanLeader() ? 0x01 : 0x00);
			writeD(_Castle.getOwnerId());
		}
		else
		{
			writeD(_ClanHall.getId());
			writeD(_ClanHall.getOwnerId() == activeChar.getClanId() && activeChar.isClanLeader() ? 0x01 : 0x00);
			writeD(_ClanHall.getOwnerId());
		}
		if(_Castle != null && _Castle.getOwnerId() > 0)
		{
			L2Clan owner = ClanTable.getInstance().getClan(_Castle.getOwnerId());
			if(owner != null)
			{
				writeS(owner.getName()); // Clan Name
				writeS(owner.getLeaderName()); // Clan Leader Name
				if(owner.getAllyId() != 0)
				{
					L2Alliance alliance = ClanTable.getInstance().getAlliance(owner.getAllyId());
					writeD(alliance.getAllyId()); // Ally ID
					writeS(alliance.getAllyName()); // Ally Name
				}
				else
				{
					writeD(0);
					writeS("");
				}
			}
			else
				_log.warning("Null owner for castle: " + _Castle.getName());
		}
		else if(_ClanHall != null && _ClanHall.getOwnerId() > 0)
		{
			L2Clan owner = ClanTable.getInstance().getClan(_ClanHall.getOwnerId());
			if(owner != null)
			{
				writeS(owner.getName()); // Clan Name
				writeS(owner.getLeaderName()); // Clan Leader Name
				if(owner.getAllyId() != 0)
				{
					L2Alliance alliance = ClanTable.getInstance().getAlliance(owner.getAllyId());
					writeD(alliance.getAllyId()); // Ally ID
					writeS(alliance.getAllyName()); // Ally Name
				}
				else
				{
					writeD(0);
					writeS("");
				}
			}
			else
				_log.warning("Null owner for clanhall: " + _ClanHall.getName());
		}
		else
		{
			writeS(""); // Clan Name
			writeS(""); // Clan Leader Name
			writeD(0); // Ally ID
			writeS(""); // Ally Name
		}

		writeD((int) (Calendar.getInstance().getTimeInMillis() / 1000));
		if(_Castle != null)
			writeD((int) (_Castle.getSiege().getSiegeDate().getTimeInMillis() / 1000));
		else
			writeD(startTime);
		writeD(0x00); //number of choices?
	}

	/* (non-Javadoc)
	 * @see L2jFT.game.network.serverpackets.L2GameServerPacket#getType()
	 */
	@Override
	public String getType()
	{
		return _S__C9_SIEGEINFO;
	}

}
