package L2jFT.game.network.serverpackets;

import javolution.util.FastComparator;
import javolution.util.FastTable;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill;

/**
 * format   d (dddc)
 */
public class SkillList extends L2GameServerPacket
{
	private static final String _S__58_SKILLLIST = "[S] 58 SkillList";
	private FastTable<L2Skill> _skills;

	public SkillList(L2Player p)
	{
		_skills = new FastTable<L2Skill>();
		_skills.addAll(p.getAllSkills());
	}

	@Override
	final public void runImpl()
	{
		_skills.setValueComparator(new SkillComparator()).sort();
	}

	@Override
	protected final void writeImpl()
	{
		writeC(0x58);
		writeD(_skills.size());

		for(L2Skill temp : _skills)
		{
			writeD((temp.isPassive() || temp.isOnAttack()) ? 1 : 0);
			writeD(temp.getDisplayLevel());
			writeD(temp.getDisplayId());
			writeC(0x00); // c5     
		}
	}

	@Override
	public String getType()
	{
		return _S__58_SKILLLIST;
	}

	public class SkillComparator extends FastComparator<L2Skill>
	{
		private static final long serialVersionUID = -1235017548845689754L;

		@Override
		public boolean areEqual(L2Skill arg0, L2Skill arg1)
		{
			return arg0.getDisplayId() == arg1.getDisplayId();
		}

		@Override
		public int compare(L2Skill arg0, L2Skill arg1)
		{
			return arg0.getDisplayId() - arg1.getDisplayId();
		}

		@Override
		public int hashCodeOf(L2Skill arg0)
		{
			return arg0.hashCode();
		}
	}
}