package L2jFT.game.network.serverpackets;

import L2jFT.Config;
import L2jFT.game.instancemanager.CursedWeaponsManager;
import L2jFT.game.model.Inventory;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.PcInventory;
import L2jFT.game.tables.NpcTable;

import java.util.logging.Logger;

public class UserInfo extends L2GameServerPacket
{
	private static final Logger _log = Logger.getLogger(UserInfo.class.getName());

	private static final String _S__04_USERINFO = "[S] 04 UserInfo";
	private final L2Player _cha;
	private int _runSpd, _walkSpd, _swimRunSpd, _swimWalkSpd, _flRunSpd,
			_flWalkSpd, _flyRunSpd, _flyWalkSpd, _relation;
	private float moveMultiplier;
	private PcInventory _inv;

	/**
	 * @param _characters
	 */
	public UserInfo(L2Player cha)
	{
		_cha = cha;

		moveMultiplier = _cha.getMovementSpeedMultiplier();
		_runSpd = (int) (_cha.getRunSpeed() / moveMultiplier);
		_walkSpd = (int) (_cha.getWalkSpeed() / moveMultiplier);
		_flRunSpd = _flyRunSpd = _runSpd;
		_flWalkSpd = _flyWalkSpd = _walkSpd;
		_swimRunSpd = _cha.getSwimSpeed();
		_swimWalkSpd = _cha.getSwimSpeed();
		_inv = _cha.getInventory();
		_relation = _cha.isClanLeader() ? 0x40 : 0;
		if(_cha.getSiegeState() == 1)
			_relation |= 0x180;
		else if(_cha.getSiegeState() == 2)
			_relation |= 0x80;
	}

	@Override
	final public void runImpl()
	{}

	@Override
	protected final void writeImpl()
	{
		L2Player activeChar = getClient().getActiveChar();
		if(activeChar != null && activeChar != _cha)
		{
			_log.severe("You can't send UserInfo to other character!!! clientChar="+activeChar.getObjectId()+", packetChar="+_cha.getObjectId()+"");
			return;
		}

		writeC(0x04);

		writeD(_cha.getX());
		writeD(_cha.getY());
		writeD(_cha.getZ());
		writeD(_cha.getHeading());
		writeD(_cha.getObjectId());

		/*
		 if(_cha.isGM())
		 writeS(_cha.isInvul() ? "*GM*" + _cha.getName() : "*" + _cha.getName() + "*");
		 else
		 */
		writeS(_cha.getName());

		writeD(_cha.getRace().ordinal());
		writeD(_cha.getSex());
		writeD(_cha.getBaseClass());
		writeD(_cha.getLevel());
		writeQ(_cha.getExp());
		writeD(_cha.getSTR());
		writeD(_cha.getDEX());
		writeD(_cha.getCON());
		writeD(_cha.getINT());
		writeD(_cha.getWIT());
		writeD(_cha.getMEN());
		writeD(_cha.getMaxHp());
		writeD((int) _cha.getCurrentHp());
		writeD(_cha.getMaxMp());
		writeD((int) _cha.getCurrentMp());
		writeD(_cha.getSp());
		writeD(_cha.getCurrentLoad());
		writeD(_cha.getMaxLoad());
		writeD(0x28); // unknown
		writeD(_inv.getPaperdollObjectId(Inventory.PAPERDOLL_DHAIR));
		writeD(_inv.getPaperdollObjectId(Inventory.PAPERDOLL_REAR));
		writeD(_inv.getPaperdollObjectId(Inventory.PAPERDOLL_LEAR));
		writeD(_inv.getPaperdollObjectId(Inventory.PAPERDOLL_NECK));
		writeD(_inv.getPaperdollObjectId(Inventory.PAPERDOLL_RFINGER));
		writeD(_inv.getPaperdollObjectId(Inventory.PAPERDOLL_LFINGER));
		writeD(_inv.getPaperdollObjectId(Inventory.PAPERDOLL_HEAD));
		writeD(_inv.getPaperdollObjectId(Inventory.PAPERDOLL_RHAND));
		writeD(_inv.getPaperdollObjectId(Inventory.PAPERDOLL_LHAND));
		writeD(_inv.getPaperdollObjectId(Inventory.PAPERDOLL_GLOVES));
		writeD(_inv.getPaperdollObjectId(Inventory.PAPERDOLL_CHEST));
		writeD(_inv.getPaperdollObjectId(Inventory.PAPERDOLL_LEGS));
		writeD(_inv.getPaperdollObjectId(Inventory.PAPERDOLL_FEET));
		writeD(_inv.getPaperdollObjectId(Inventory.PAPERDOLL_BACK));
		writeD(_inv.getPaperdollObjectId(Inventory.PAPERDOLL_LRHAND));
		writeD(_inv.getPaperdollObjectId(Inventory.PAPERDOLL_HAIR));
		writeD(_inv.getPaperdollObjectId(Inventory.PAPERDOLL_FACE));

		if(getClient().getRevision() >= 817)
		{
			//KamaeL packets
			writeD(0x00);
			writeD(0x00);
			writeD(0x00);
			writeD(0x00);
			writeD(0x00);
			writeD(0x00);
			writeD(0x00);
			writeD(0x00);
		}

		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_DHAIR));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_REAR));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_LEAR));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_NECK));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_RFINGER));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_LFINGER));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_HEAD));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_RHAND));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_LHAND));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_GLOVES));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_CHEST));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_LEGS));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_FEET));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_BACK));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_LRHAND));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_HAIR));
		writeD(_inv.getPaperdollItemId(Inventory.PAPERDOLL_FACE));

		if(getClient().getRevision() >= 817)
		{
			//KamaeL packets
			writeD(0x00);
			writeD(0x00);
			writeD(0x00);
			writeD(0x00);
			writeD(0x00);
			writeD(0x00);
			writeD(0x00);
			writeD(0x00);
		}

		writeH(0x00);
		writeH(0x00);
		writeH(0x00);
		writeH(0x00);
		writeH(0x00);
		writeH(0x00);
		writeH(0x00);
		writeH(0x00);
		writeH(0x00);
		writeH(0x00);
		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_DHAIR));
		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_HEAD));
		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_RHAND));
		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_LHAND));
		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_GLOVES));
		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_CHEST));
		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_LEGS));
		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_FEET));
		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_BACK));
		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_LRHAND));
		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_HAIR));
		writeD(_inv.getPaperdollAugmentationId(Inventory.PAPERDOLL_FACE));

		if(getClient().getRevision() >= 817)
		{
			//KamaeL packets
			writeD(0x00);
			writeD(0x00);
			writeD(0x00);
			writeD(0x00);
			writeD(0x00);
			writeD(0x00);
			writeD(0x00);
			writeD(0x00);
		}

		writeD(_cha.getPAtk(null));
		writeD(_cha.getPAtkSpd());
		writeD(_cha.getPDef(null));
		writeD(_cha.getEvasionRate(null));
		writeD(_cha.getAccuracy());
		writeD(_cha.getCriticalHit(null, null));
		writeD(_cha.getMAtk(null, null));

		writeD(_cha.getMAtkSpd());
		writeD(_cha.getPAtkSpd());

		writeD(_cha.getMDef(null, null));

		writeD(_cha.getPvpFlag()); // 0=white, 1=purple, 2=purpleblink
		writeD(_cha.getKarma());

		writeD(_runSpd);
		writeD(_walkSpd);
		writeD(_swimRunSpd); // swimspeed
		writeD(_swimWalkSpd); // swimspeed
		writeD(_flRunSpd);
		writeD(_flWalkSpd);
		writeD(_flyRunSpd);
		writeD(_flyWalkSpd);
		writeF(_cha.getMovementSpeedMultiplier());
		writeF(_cha.getAttackSpeedMultiplier());
		if(_cha.getMountType() != 0 && _cha.getPet() != null && _cha.getPet().getTemplate() != null)
			try
			{
				writeF(_cha.getPet().getTemplate().collisionRadius);
				writeF(_cha.getPet().getTemplate().collisionHeight);
			}
			catch(NullPointerException e)
			{
				writeF(_cha.getBaseTemplate().collisionRadius);
				writeF(_cha.getBaseTemplate().collisionHeight);
			}
		else
		{
			writeF(_cha.getBaseTemplate().collisionRadius);
			writeF(_cha.getBaseTemplate().collisionHeight);
		}

		writeD(_cha.getHairStyle());
		writeD(_cha.getHairColor());
		writeD(_cha.getFace());
		writeD(_cha.getPlayerAccess().IsGM || _cha.getPlayerAccess().CanUseGMCommand || Config.ALLOW_SPECIAL_COMMANDS ? 1 : 0); // builder level активирует в клиенте админские команды

		String title = _cha.getTitle();
		if(_cha.isInvisible())
			title = "Invisible";
		if(_cha.isPolymorphed())
			title += " - " + NpcTable.getTemplate(_cha.getPolyid()).name;
		writeS(title);

		writeD(_cha.getClanId());
		writeD(_cha.getClanCrestId()); //clan crest
		writeD(_cha.getAllyId());
		writeD(_cha.getAllyCrestId()); // ally crest id
		// siege flags: attacker - 0x180 sword over name, defender - 0x80 shield, 0xC0 crown (|leader), 0x1C0 flag (|leader)
		writeD(_relation);
		writeC(_cha.getMountType()); // mount type
		writeC(_cha.getPrivateStoreType());
		writeC(_cha.getSkillLevel(L2Skill.SKILL_CRYSTALLIZE) > 0 ? 1 : 0);
		writeD(_cha.getPkKills());
		writeD(_cha.getPvpKills());

		writeH(_cha.getCubics().size());
		for(int id : _cha.getCubics().keySet())
			writeH(id);

		writeC(0x00); //1-find party members

		writeD(_cha.getAbnormalEffect());
		writeC(0x11);

		writeD(_cha.getClanPrivileges());
		writeH(_cha.getRecomLeft()); //c2  recommendations remaining
		writeH(_cha.getPlayerAccess().IsGM ? 0 : _cha.getRecomHave()); //c2  recommendations received
		writeD(0x00);

		writeH(_cha.getInventoryLimit());

		writeD(_cha.getClassId().getId());
		writeD(0x00); // special effects? circles around player...
		writeD(_cha.getMaxCp());
		writeD((int) _cha.getCurrentCp());

		writeC(_cha.isMounted() ? 0 : _cha.getEnchantEffect()); //should check if mounted and not use
		writeC(_cha.getTeam()); //team circle around feet 1= Blue, 2 = red
		writeD(_cha.getClanCrestLargeId());
		writeC(_cha.isNoble() || _cha.isGM() && Config.GM_HERO_AURA ? 1 : 0); //0x01: symbol on char menu ctrl+I  
		writeC(_cha.isHero() || _cha.isGM() && Config.GM_HERO_AURA ? 1 : 0); //0x01: Hero Aura and symbol

		writeC(_cha.isFishing() ? 1 : 0); //Fishing Mode
		writeD(_cha.getFishx()); //fishing x  
		writeD(_cha.getFishy()); //fishing y
		writeD(_cha.getFishz()); //fishing z

		writeD(_cha.getNameColor());

		writeC(_cha.isRunning() ? 0x01 : 0x00); //changes the Speed display on Status Window
		writeD(_cha.getPledgeClass()); //C5 ??
		writeD(0x00); //C5 ??
		writeD(_cha.getTitleColor()); //C5 ??
		writeD(0); //C5 ??
		if(_cha.isCursedWeaponEquipped())
			writeD(CursedWeaponsManager.getInstance().getLevel(_cha.getCursedWeaponEquippedId()));
		else
			writeD(0x00);

		_cha.refreshSavedStats();
	}

	@Override
	public String getType()
	{
		return _S__04_USERINFO;
	}
}
