package L2jFT.game.network.serverpackets;

import L2jFT.game.model.L2Player;
import L2jFT.game.model.Warehouse.WarehouseType;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.instances.L2ItemInstance.ItemClass;
import L2jFT.game.templates.L2Item;

import java.util.NoSuchElementException;

public class WareHouseWithdrawalList extends L2GameServerPacket
{
	private static final String _S__42_WAREHOUSEWITHDRAWALLIST = "[S] 42 WareHouseWithdrawalList";

	public static final int PRIVATE = 1;
	public static final int CLAN = 2;
	public static final int CASTLE = 3;
	public static final int FREIGHT = 4;

	private final L2Player _cha;
	private final int _money;
	private L2ItemInstance[] _items;
	private final int _type;

	public WareHouseWithdrawalList(L2Player cha, WarehouseType type, ItemClass clss)
	{
		_cha = cha;
		_money = cha.getAdena();
		_type = type.getPacketValue();
		switch(type)
		{
			case PRIVATE:
				_items = _cha.getWarehouse().listItems(clss);
				break;
			case CLAN:
			case CASTLE:
				_items = _cha.getClan().getWarehouse().listItems(clss);
				break;
			/*
			 case CASTLE:
			 items = _cha.getClan().getCastleWarehouse().listItems();
			 break;
			 */
			case FREIGHT:
				_items = _cha.getFreight().listItems(clss);
				break;
			default:
				throw new NoSuchElementException("Invalid value of 'type' argument");
		}
	}

	@Override
	final public void runImpl()
	{}

	@Override
	protected final void writeImpl()
	{
		if(_items.length == 0)
		{
			_cha.sendPacket(new SystemMessage(SystemMessage.YOU_HAVE_NOT_DEPOSITED_ANY_ITEMS_IN_YOUR_WAREHOUSE));
			return;
		}
		writeC(0x42);
		writeH(_type);
		writeD(_money);
		writeH(_items.length);
		for(L2ItemInstance temp : _items)
		{
			L2Item item = temp.getItem();
			writeH(item.getType1());
			writeD(temp.getObjectId());
			writeD(temp.getItemId());
			writeD(temp.getCount());
			writeH(item.getType2());
			writeH(temp.getCustomType1());
			writeD(temp.getBodyPart());
			writeH(temp.getEnchantLevel());
			writeH(temp.getCustomType2());
			writeH(0); // ?
			writeD(temp.getObjectId()); // return value for define item (object_id)
			writeH(0);
			writeD(temp.getAugmentationId());
			writeH(0);
		}
	}

	@Override
	public String getType()
	{
		return _S__42_WAREHOUSEWITHDRAWALLIST;
	}
}