package L2jFT.game.skills.effects;

import L2jFT.game.ThreadPoolManager;
import L2jFT.game.ai.CtrlIntention;
import L2jFT.game.cache.Msg;
import L2jFT.game.geodata.GeoData;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Effect;
import L2jFT.game.model.L2Playable;
import L2jFT.game.model.L2Summon;
import L2jFT.game.skills.Env;
import L2jFT.util.CharPosition;
import L2jFT.util.Location;
import L2jFT.util.Rnd;

final class EffectFear extends L2Effect
{
	public static final int FEAR_RANGE = 500;

	public EffectFear(Env env, EffectTemplate template)
	{
		super(env, template);
	}

	@Override
	public EffectType getEffectType()
	{
		return EffectType.FEAR;
	}

	/** Notify started */
	@Override
	public void onStart()
	{
		if(_effected.isFearImmune())
		{
			getEffector().sendPacket(Msg.TARGET_IS_INCORRECT);
			return;
		}

		if(_effected.isInZonePeace())
		{
			getEffector().sendPacket(Msg.YOU_CANNOT_ATTACK_IN_THE_PEACE_ZONE);
			return;
		}

		if(!(_effected instanceof L2Playable))
			_effected.startFear();

		onActionTime();
	}

	/** Notify exited */
	@Override
	public void onExit()
	{
		if(!(_effected instanceof L2Playable))
			_effected.stopFear();

		_effected.getAI().setIntention(CtrlIntention.AI_INTENTION_ACTIVE);
	}

	@Override
	public boolean onActionTime()
	{
		if(_effected.isFearImmune())
			return false;

		// Fear нельзя наложить на осадных саммонов
		if(_effected.getNpcId() == L2Summon.SIEGE_GOLEM_ID || _effected.getNpcId() == L2Summon.SIEGE_CANNON_ID || _effected.getNpcId() == L2Summon.SWOOP_CANNON_ID)
			return false;

		CharPosition pos = Rnd.coordsRandomize(_effected.getX(), _effected.getY(), _effected.getZ(), 0, FEAR_RANGE, FEAR_RANGE);

		/**
		int posX = _effected.getX();
		int posY = _effected.getY();
		int posZ = _effected.getZ();
		
		if(posX > _effector.getX())
			posX += FEAR_RANGE;
		else
			posX -= FEAR_RANGE;
		if(posY > _effector.getY())
			posY += FEAR_RANGE;
		else
			posY -= FEAR_RANGE;
		*/

		_effected.setRunning();

		Location loc = GeoData.getInstance().moveCheck(_effected.getX(), _effected.getY(), _effected.getZ(), pos.x, pos.y, pos.z);
		_effected.moveToLocation(loc.x, loc.y, loc.z, 0, false);

		if(_effected instanceof L2Playable)
		{
			_effected.startFear();
			getEffected().sendMessage("You can feel Fears's effect");
			ThreadPoolManager.getInstance().scheduleGeneral(new StopFear(_effected), 2000);
		}

		return true;
	}

	public class StopFear implements Runnable
	{
		private L2Character _activeChar;

		public StopFear(L2Character activeChar)
		{
			_activeChar = activeChar;
		}

		public void run()
		{
			_activeChar.stopFear();
		}
	}
}
