package L2jFT.game.skills.effects;

import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Effect;
import L2jFT.game.skills.Env;
import L2jFT.game.skills.Stats;
import L2jFT.game.skills.conditions.Condition;
import L2jFT.game.skills.funcs.FuncTemplate;
import L2jFT.game.skills.funcs.Lambda;

import java.lang.reflect.Constructor;
import java.util.logging.Logger;

@SuppressWarnings("unchecked")
public final class EffectTemplate
{
	static Logger _log = Logger.getLogger(EffectTemplate.class.getName());

	private final Class _func;
	private final Constructor _constructor;

	public final Condition _attachCond;
	public final Condition _applyCond;
	public final Lambda _lambda;
	public final int _counter;
	public final long _period; // in milliseconds
	public final int _abnormalEffect;
	public FuncTemplate[] _funcTemplates;
	public Stats[] _stats;
	public final ResistType _resistType;

	public final String _stackType;
	public final int _stackOrder;

	public final boolean _applyOnCaster;

	public EffectTemplate(Condition attachCond, Condition applyCond, String func, Lambda lambda, int counter, long period, int abnormalEffect, String stackType, int stackOrder, boolean applyOnCaster)
	{
		_attachCond = attachCond;
		_applyCond = applyCond;
		_lambda = lambda;
		_counter = counter;
		_period = period;
		_abnormalEffect = abnormalEffect;
		_stackType = stackType;
		_stackOrder = stackOrder;
		_applyOnCaster = applyOnCaster;

		//funny paradox - buff is debuff :D
		if(func.equals("Buff"))
			_resistType = ResistType.DEBUFF;
		else if(func.equals("ConfuseMob") || func.equals("Fear") || func.equals("Distrust") || func.equals("UnAggro"))
			_resistType = ResistType.MENTAL;
		else if(func.equals("Paralyze") || func.equals("Petrification"))
			_resistType = ResistType.PARALYZE;
		else if(func.equals("Root"))
			_resistType = ResistType.ROOT;
		else if(func.equals("Sleep"))
			_resistType = ResistType.SLEEP;
		else if(func.equals("Mute") || func.equals("MutePhisycal") || func.equals("MuteAll"))
			_resistType = ResistType.MUTE;
		else if(func.equals("Stun"))
			_resistType = ResistType.STUN;
		else if(func.equals("LethalStrike1") || func.equals("LethalStrike2"))
			_resistType = ResistType.DEATH;
		else if(abnormalEffect == L2Character.ABNORMAL_EFFECT_BLEEDING)
			_resistType = ResistType.BLEED;
		else if(abnormalEffect == L2Character.ABNORMAL_EFFECT_POISON)
			_resistType = ResistType.POISON;
		else
			_resistType = ResistType.NONE;

		try
		{
			_func = Class.forName("L2jFT.game.skills.effects.Effect" + func);
		}
		catch(ClassNotFoundException e)
		{
			throw new RuntimeException(e);
		}

		try
		{
			_constructor = _func.getConstructor(Env.class, EffectTemplate.class);
		}
		catch(NoSuchMethodException e)
		{
			throw new RuntimeException(e);
		}
	}

	public L2Effect getEffect(Env env)
	{
		if(_attachCond != null && !_attachCond.test(env))
			return null;

		try
		{
			return (L2Effect) _constructor.newInstance(env, this);
		}
		catch(Throwable t)
		{
			_log.warning("Error creating new instance of Class " + _func);
			t.printStackTrace();
		}
		return null;
	}

	public void attach(FuncTemplate f)
	{
		if(_funcTemplates == null)
			_funcTemplates = new FuncTemplate[] { f };
		else
		{
			int len = _funcTemplates.length;
			FuncTemplate[] tmp = new FuncTemplate[len + 1];
			System.arraycopy(_funcTemplates, 0, tmp, 0, len);
			tmp[len] = f;
			_funcTemplates = tmp;
		}
		addStat(f);
	}

	public void addStat(FuncTemplate f)
	{
		if(_stats == null)
			_stats = new Stats[] { f._stat };
		else
		{
			int len = _stats.length;
			Stats[] tmp = new Stats[len + 1];
			System.arraycopy(_stats, 0, tmp, 0, len);
			tmp[len] = f._stat;
			_stats = tmp;
		}
	}

	public boolean isOverlay()
	{
		switch(_resistType)
		{
			case MENTAL:
			case PARALYZE:
			case ROOT:
			case SLEEP:
			case STUN:
			case MUTE:
				return false;
			default:
				return true;
		}
	}

	public boolean isPvPEffect()
	{
		switch(_resistType)
		{
			case STUN:
			case SLEEP:
			case MUTE:
			case ROOT:
			case PARALYZE:
				return true;
			default:
				return false;
		}
	}

	public long getPeriod()
	{
		return _period;
	}
}
