package L2jFT.game.skills.funcs;

import L2jFT.game.skills.Env;
import L2jFT.game.skills.Stats;
import L2jFT.game.skills.conditions.Condition;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

public final class FuncTemplate
{
	public Condition _attachCond;
	public Condition _applyCond;
	public final Class _func;
	public final Constructor _constructor;
	public final Stats _stat;
	public final int _order;
	public final Lambda _lambda;

	@SuppressWarnings("unchecked")
	public FuncTemplate(Condition attachCond, Condition applyCond, String func, Stats stat, int order, Lambda lambda)
	{
		_attachCond = attachCond;
		_applyCond = applyCond;
		_stat = stat;
		_order = order;
		_lambda = lambda;
		try
		{
			_func = Class.forName("L2jFT.game.skills.funcs.Func" + func);
		}
		catch(ClassNotFoundException e)
		{
			throw new RuntimeException(e);
		}
		try
		{
			_constructor = _func.getConstructor(new Class[] { Stats.class, // stats to update
					Integer.TYPE, // order of execution
					Object.class, // owner
					Lambda.class // value for function
			});
		}
		catch(NoSuchMethodException e)
		{
			throw new RuntimeException(e);
		}
	}

	public Func getFunc(Env env, Object owner)
	{
		if(_attachCond != null && !_attachCond.test(env))
			return null;
		try
		{
			Func f = (Func) _constructor.newInstance(_stat, _order, owner, _lambda);
			if(_applyCond != null)
				f.setCondition(_applyCond);
			return f;
		}
		catch(IllegalAccessException e)
		{
			e.printStackTrace();
			return null;
		}
		catch(InstantiationException e)
		{
			e.printStackTrace();
			return null;
		}
		catch(InvocationTargetException e)
		{
			e.printStackTrace();
			return null;
		}

	}
}
