package L2jFT.game.skills.skillclasses;

import javolution.util.FastList;
import L2jFT.game.ai.CtrlEvent;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Playable;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.templates.L2Weapon;
import L2jFT.game.templates.StatsSet;
import L2jFT.util.Rnd;

public class Aggression extends L2Skill
{
	private final boolean _unaggring;
	private final boolean _cancelTarget;

	public Aggression(StatsSet set)
	{
		super(set);
		_unaggring = set.getBool("unaggroing", false);
		_cancelTarget = set.getBool("cancelTarget", false);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		int effect = _effectPoint;
		if(!isHandler() && activeChar.unChargeShots(isMagic()))
			effect *= 2;
		L2Weapon weapon = activeChar.getActiveWeaponItem();
		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();

			if(!(target instanceof L2NpcInstance || target instanceof L2Playable))
				continue;

			if(!_unaggring && target instanceof L2Playable)
			{

				int chance = 50 + effect / 60;

				byte mLevel = getMagicLevel() == 0 ? activeChar.getLevel() : getMagicLevel();
				double levelmod = (double) mLevel / target.getLevel();

				if(levelmod > 1)
					levelmod = (levelmod - 1) * 2 + 1;
				else if(levelmod < 1)
					levelmod = 1 - (1 - levelmod) * 2;

				chance *= Math.max(levelmod, 0);

				if(!Rnd.chance(chance))
					continue;

			}

			target.getAI().notifyEvent(CtrlEvent.EVT_AGGRESSION, activeChar, (_unaggring ? -effect : effect));

			if(_cancelTarget && Rnd.chance(80))
				target.setTarget(null);

			if(weapon != null)
				weapon.getEffect(false, activeChar, target, isOffensive());

			getEffects(activeChar, target, getActivateRate() > 0);
		}
	}
}
