package L2jFT.game.skills.skillclasses;

import javolution.util.FastList;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill;
import static L2jFT.game.model.L2Zone.ZoneType.OlympiadStadia;
import static L2jFT.game.model.L2Zone.ZoneType.Siege;
import static L2jFT.game.model.L2Zone.ZoneType.no_restart;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.templates.StatsSet;

import java.util.ArrayList;

public class Call extends L2Skill
{
	final boolean _party;

	public Call(StatsSet set)
	{
		super(set);
		_party = set.getBool("party", false);
	}

	@Override
	public boolean checkCondition(L2Character activeChar, L2Character target, boolean forceUse, boolean dontMove, boolean first)
	{
		if(activeChar.isPlayer)
		{
			if(_party && ((L2Player) activeChar).getParty() == null)
				return false;

			// "Нельзя вызывать персонажей в/из зоны свободного PvP"
			// "в зоны осад"
			// "на Олимпийский стадион"
			// "в зоны определенных рейд-боссов и эпик-боссов"

			if(activeChar.isInZoneBattle() || activeChar.isInZone(Siege) || activeChar.isInZoneIncludeZ(no_restart))
				return false;

			if(target == null)
				return false;

			if(target.isInZone(OlympiadStadia))
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_SUMMON_PLAYERS_WHO_ARE_CURRENTLY_PARTICIPATING_IN_THE_GRAND_OLYMPIAD));
				return false;
			}

			if(target.isInZoneBattle() || target.isInZone(Siege) || target.isInZoneIncludeZ(no_restart))
			{
				activeChar.sendPacket(new SystemMessage(SystemMessage.YOUR_TARGET_IS_IN_AN_AREA_WHICH_BLOCKS_SUMMONING));
				return false;
			}

			if(!_party)
			{
				if(!target.isPlayer)
					return false;

				L2Player Ptarget = (L2Player) target;

				// Нельзя призывать торгующих персонажей
				if(Ptarget.getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE || Ptarget.isTransactionInProgress())
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.S1_IS_CURRENTLY_TRADING_OR_OPERATING_A_PRIVATE_STORE_AND_CANNOT_BE_SUMMONED).addString(Ptarget.getName()));
					return false;
				}

				// Нельзя призывать персонажей, которые находятся в режиме PvP
				if(Ptarget.getPvpFlag() != 0)
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.S1_IS_ENGAGED_IN_COMBAT_AND_CANNOT_BE_SUMMONED).addString(Ptarget.getName()));
					return false;
				}

				// Нельзя призывать мертвых персонажей
				if(target.isAlikeDead())
				{
					activeChar.sendPacket(new SystemMessage(SystemMessage.S1_IS_DEAD_AT_THE_MOMENT_AND_CANNOT_BE_SUMMONED).addString(Ptarget.getName()));
					return false;
				}

			}
		}
		return super.checkCondition(activeChar, target, forceUse, dontMove, first);
	}

	@SuppressWarnings("unchecked")
	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		if(!isHandler())
			activeChar.unChargeShots(isMagic());

		if(_party)
		{
			if(((L2Player) activeChar).getParty() == null)
				return;
			ArrayList<L2Player> others = (ArrayList<L2Player>) ((L2Player) activeChar).getParty().getPartyMembers().clone();
			others.remove(activeChar);
			targets.reset();
			targets.addAll(others);
		}

		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();
			if(!(target instanceof L2Player))
				continue;
			L2Player ptarget = (L2Player) target;
			if(target.isDead() || ptarget.getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE || ptarget.isTransactionInProgress() || target.isInZoneBattle() || target.isInZone(Siege) || target.isInZone(no_restart))
				continue;

			int[] coords = { activeChar.getX(), activeChar.getY(), activeChar.getZ() };
			// При использовании скилов Summon Friend и Word of Invitation у призываемых персонажей
			// надо удалить 1 Summoning Crystall из инвентаря 
			if(getId() == 1403 || getId() == 1404)
				ptarget._SummonConsumeCrystall = true;
			ptarget.summonCharacterRequest(activeChar.getName(), coords);
		}
	}
}