package L2jFT.game.skills.skillclasses;

import javolution.util.FastList;
import L2jFT.game.cache.Msg;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.skills.Formulas;
import L2jFT.game.templates.StatsSet;

public class ChargeDmg extends L2Skill
{
	private final short _numCharges; //used to destroy charges
	private final short _condCharges; //used to get how much charges we need

	public ChargeDmg(final StatsSet set)
	{
		super(set);
		_numCharges = set.getShort("num_charges", _level);
		_condCharges = set.getShort("cond_charges", _numCharges);
	}

	@Override
	public boolean checkCondition(final L2Character activeChar, final L2Character target, boolean forceUse, boolean dontMove, boolean first)
	{
		if(!activeChar.isPlayer)
			return false;

		L2Player player = (L2Player) activeChar;

		//check charges
		if(player.getIncreasedForce() < _condCharges || player.getIncreasedForce() < _numCharges)
		{
			player.sendPacket(new SystemMessage(SystemMessage.YOU_CAN_NO_LONGER_INCREASE_YOUR_FORCE).addString(getName()));
			return false;
		}

		return super.checkCondition(activeChar, target, forceUse, dontMove, first);
	}

	@Override
	public void useSkill(final L2Character activeChar, final FastList<L2Character> targets)
	{
		L2Player player = (L2Player) activeChar;

		if(player.getIncreasedForce() < _numCharges || player.getIncreasedForce() < _condCharges)
			return;

		final boolean ss = activeChar.getChargedSoulShot() && !isHandler();
		if(ss)
			activeChar.unChargeShots(false);

		double modifier = 0.3 * player.getIncreasedForce() + 0.7;

		player.setIncreasedForce(player.getIncreasedForce() - _numCharges);

		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();
			if(target.isDead())
				continue;
			
			target.doCounterAttack(this, activeChar);

			if(target.checkReflectSkill(this))
				target = activeChar;

			boolean shld = !getShieldIgnore() && Formulas.calcShldUse(activeChar, target);

			double damage = modifier * Formulas.calcPhysDam(activeChar, target, this, shld, false, false, ss);

			if(damage < 1)
				damage = 1;
			if(target.isPlayer)
				if(shld)
					if(damage == 1)
						target.sendPacket(Msg.YOUR_EXCELLENT_SHIELD_DEFENSE_WAS_A_SUCCESS);
					else
						target.sendPacket(Msg.SHIELD_DEFENSE_HAS_SUCCEEDED);

			target.reduceCurrentHp(damage, activeChar);
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_HIT_FOR_S1_DAMAGE).addNumber((int) damage));
			getEffects(activeChar, target, getActivateRate() > 0);
		}
	}
}