package L2jFT.game.skills.skillclasses;

import javolution.util.FastList;
import L2jFT.Config;
import L2jFT.game.cache.Msg;
import L2jFT.util.Rnd;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.skills.Stats;
import L2jFT.game.templates.L2Weapon;
import L2jFT.game.templates.StatsSet;

public class Continuous extends L2Skill
{
	private int _lethal1;
	private int _lethal2;
	private final String _aborts;
	private final short _numCharges;
	private final short _condCharges;

	public Continuous(StatsSet set)
	{
		super(set);
		_lethal1 = set.getInteger("lethal1", 0);
		_lethal2 = set.getInteger("lethal2", 0);
		_aborts = set.getString("abortType", "");
		_numCharges = set.getShort("num_charges", (short) 0);
		_condCharges = set.getShort("cond_charges", (short) 0);
	}

	@Override
	public boolean checkCondition(final L2Character activeChar, final L2Character target, boolean forceUse, boolean dontMove, boolean first)
	{
		if(activeChar.getIncreasedForce() < _condCharges || activeChar.getIncreasedForce() < _numCharges)
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CAN_NO_LONGER_INCREASE_YOUR_FORCE).addString(getName()));
			return false;
		}

		return super.checkCondition(activeChar, target, forceUse, dontMove, first);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		if(activeChar.getIncreasedForce() < _numCharges || activeChar.getIncreasedForce() < _condCharges)
			return;
		activeChar.setIncreasedForce(activeChar.getIncreasedForce() - _numCharges);

		L2Weapon weapon = activeChar.getActiveWeaponItem();
		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();
			// Player holding a cursed weapon can't be buffed and can't buff
			if(getSkillType() == L2Skill.SkillType.BUFF && target != activeChar)
				if(target == null || target.isCursedWeaponEquipped())
					continue;
				else if(activeChar.isCursedWeaponEquipped())
					continue;

			_lethal1 *= 0.01 * target.calcStat(Stats.DEATH_RECEPTIVE, 100, null, null);
			_lethal2 *= 0.01 * target.calcStat(Stats.DEATH_RECEPTIVE, 100, null, null);

			if(_lethal1 > 0 && Rnd.chance(_lethal1))
			{
				if(target.isPlayer)
				{
					target.reduceCurrentHp(target.getCurrentCp(), activeChar);
					target.sendPacket(Msg.YOU_HAVE_BEEN_STRUCK_BY_THE_INSTANT_KILL_SKILL);
				}
				else if(target instanceof L2NpcInstance && !target.isLethalImmune() && target.getMaxHp() <= 50000)
					target.reduceCurrentHp(target.getCurrentHp() / 2, activeChar);

				activeChar.sendPacket(Msg.INSTANT_KILL);
			}

			if(_lethal2 > 0 && Rnd.chance(_lethal2))
			{
				if(target.isPlayer)
				{
					target.reduceCurrentHp(target.getCurrentHp() + target.getCurrentCp() - 1, activeChar);
					target.sendPacket(Msg.YOU_HAVE_BEEN_STRUCK_BY_THE_INSTANT_KILL_SKILL);
				}
				else if(target instanceof L2NpcInstance && !target.isLethalImmune() && target.getMaxHp() <= 50000)
					target.reduceCurrentHp(target.getCurrentHp(), activeChar);

				activeChar.sendPacket(Msg.INSTANT_KILL);
			}

			if(weapon != null)
				weapon.getEffect(false, activeChar, target, isOffensive());

			if(_aborts.contains("attack"))
				activeChar.abortAttack();

			if(_aborts.contains("cast"))
				activeChar.abortCast();

			getEffects(activeChar, target, getActivateRate() > 0);
		}

		if(isMagic() && _id < 2000 && !isHandler())
			if(!(Config.SAVING_SPS && _skillType == SkillType.BUFF))
				activeChar.unChargeShots(true);
	}
}