package L2jFT.game.skills.skillclasses;

import javolution.util.FastList;
import L2jFT.game.cache.Msg;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.skills.Formulas;
import L2jFT.game.templates.L2Weapon;
import L2jFT.game.templates.StatsSet;

public class Drain extends L2Skill
{
	private float _absorbPart;
	private int _absorbAbs;

	public Drain(StatsSet set)
	{
		super(set);

		_absorbPart = set.getFloat("absorbPart", 0.f);
		_absorbAbs = set.getInteger("absorbAbs", 0);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		int sps = !isHandler() ? activeChar.getChargedSpiritShot() : 0;
		L2Weapon weapon = activeChar.getActiveWeaponItem();

		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();
			if(target.isDead() && _targetType != SkillTargetType.TARGET_CORPSE)
				continue;

			double hp = 0.;

			if(_targetType != SkillTargetType.TARGET_CORPSE)
			{
				if(target.checkReflectSkill(this))
					target = activeChar;
				boolean shield = Formulas.calcShldUse(activeChar, target);
				double damage = Formulas.calcMagicDam(activeChar, target, this, shield, sps);
				double targetCP = target.getCurrentCp();

				// Нельзя восстанавливать HP из CP  
				if(damage > targetCP || !target.isPlayer)
					hp = (damage - targetCP) * _absorbPart;

				if(damage < 1)
					damage = 1;

				if(target.isPlayer)
					if(shield)
						if(damage == 1)
							target.sendPacket(Msg.YOUR_EXCELLENT_SHIELD_DEFENSE_WAS_A_SUCCESS);
						else
							target.sendPacket(Msg.SHIELD_DEFENSE_HAS_SUCCEEDED);

				target.reduceCurrentHp(damage, activeChar);
				activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_HIT_FOR_S1_DAMAGE).addNumber((int) damage));
			}

			hp += _absorbAbs + activeChar.getCurrentHp();

			activeChar.setCurrentHp(hp);

			if(target.isDead() && _targetType == SkillTargetType.TARGET_CORPSE && target instanceof L2NpcInstance)
				((L2NpcInstance) target).endDecayTask();

			getEffects(activeChar, target, getActivateRate() > 0);

			if(weapon != null)
				weapon.getEffect(false, activeChar, target, isOffensive());
		}

		if(sps != 0)
			activeChar.unChargeShots(true);
	}

}
