package L2jFT.game.skills.skillclasses;

import javolution.util.FastList;
import L2jFT.game.cache.Msg;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Effect;
import L2jFT.game.model.L2Effect.EffectType;
import L2jFT.game.model.L2Skill;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.skills.Formulas;
import L2jFT.game.skills.effects.EffectSeed;
import L2jFT.game.templates.L2Weapon;
import L2jFT.game.templates.StatsSet;

public class Elemental extends L2Skill
{
	private short seed[] = new short[3];
	private short seedcount[] = new short[3];

	private final boolean seedAny;
	private int seedCount;

	public Elemental(StatsSet set)
	{
		super(set);

		seedAny = set.getBool("seed_any", false);
		if(seedAny)
		{
			seedCount = set.getInteger("seedCount");
			return;
		}

		for(int i = 0; i < 3; i++)
		{
			seed[i] = set.getShort("seed" + (i + 1), (short) 0);
			seedcount[i] = set.getShort("seedcount" + (i + 1), (short) 1);
		}
	}

	@Override
	public boolean checkCondition(L2Character activeChar, L2Character target, boolean forceUse, boolean dontMove, boolean first)
	{
		if(seedAny)
		{
			int count = 0;
			for(L2Effect e : activeChar.getAllEffects())
				if(e instanceof EffectSeed)
					count += ((EffectSeed) e).getPower();

			if(count < seedCount)
				return false;
		}
		else
			for(int i = 0; i < 3; i++)
				if(seed[i] != 0)
				{
					L2Effect eff = activeChar.getEffectByIndexAndType(seed[i], EffectType.SEED);
					if(eff == null || ((EffectSeed) eff).getPower() < seedcount[i])
						return false;
				}

		return super.checkCondition(activeChar, target, forceUse, dontMove, first);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		int sps = !isHandler() ? activeChar.getChargedSpiritShot() : 0;
		if(sps != 0)
			activeChar.unChargeShots(true);
		L2Weapon weapon = activeChar.getActiveWeaponItem();
		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();
			if(target.isDead())
				continue;
			if(target.checkReflectSkill(this))
				target = activeChar;
			boolean shield = Formulas.calcShldUse(activeChar, target);
			double damage = Formulas.calcMagicDam(activeChar, target, this, shield, sps);

			if(damage < 1)
				damage = 1;
			if(target.isPlayer)
				if(shield)
					if(damage == 1)
						target.sendPacket(Msg.YOUR_EXCELLENT_SHIELD_DEFENSE_WAS_A_SUCCESS);
					else
						target.sendPacket(Msg.SHIELD_DEFENSE_HAS_SUCCEEDED);

			target.reduceCurrentHp(damage, activeChar);
			if(activeChar.isPlayer)
				activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_HIT_FOR_S1_DAMAGE).addNumber((int) damage));
			getEffects(activeChar, target, getActivateRate() > 0);
			if(weapon != null)
				weapon.getEffect(false, activeChar, target, isOffensive());
		}
		activeChar.stopEffects(EffectType.SEED);
	}
}
