package L2jFT.game.skills.skillclasses;

import javolution.util.FastList;
import L2jFT.Config;
import L2jFT.game.GameTimeController;
import L2jFT.game.geodata.GeoData;
import L2jFT.game.instancemanager.ZoneManager;
import L2jFT.util.Rnd;
import L2jFT.util.Util;
import L2jFT.game.model.FishData;
import L2jFT.game.model.Inventory;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.network.serverpackets.ExFishingStart;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.FishTable;
import L2jFT.game.templates.L2Weapon;
import L2jFT.game.templates.L2Weapon.WeaponType;
import L2jFT.game.templates.StatsSet;

import java.util.List;

public class Fishing extends L2Skill
{
	public Fishing(StatsSet set)
	{
		super(set);
	}

	@Override
	public boolean checkCondition(L2Character activeChar, L2Character target, boolean forceUse, boolean dontMove, boolean first)
	{
		L2Player player = (L2Player) activeChar;

		if(player.getSkillLevel(SKILL_FISHING_MASTERY) == -1)
			return false;

		if(player.isFishing())
		{
			if(player.getFishCombat() != null)
				player.getFishCombat().DoDie(false);
			else
				player.endFishing(false);
			//Cancels fishing     
			player.sendPacket(new SystemMessage(SystemMessage.CANCELS_FISHING));
			return false;
		}

		if(player.isInBoat())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANT_FISH_WHILE_YOU_ARE_ON_BOARD));
			return false;
		}

		if(player.getPrivateStoreType() != L2Player.STORE_PRIVATE_NONE)
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CANNOT_FISH_WHILE_USING_A_RECIPE_BOOK_PRIVATE_MANUFACTURE_OR_PRIVATE_STORE));
			return false;
		}

		if(Config.ENABLE_FISHING_CHECK_WATER)
		{
			int rnd = Rnd.get(50) + 150;
			double angle = Util.convertHeadingToDegree(player.getHeading());
			double radian = Math.toRadians(angle - 90);
			double sin = Math.sin(radian);
			double cos = Math.cos(radian);
			int x1 = -(int) (sin * rnd);
			int y1 = (int) (cos * rnd);
			int x = player.getX() + x1;
			int y = player.getY() + y1;
			int z = GeoData.getInstance().getHeight(x, y, player.getZ());
			player.setFishX(x);
			player.setFishY(y);
			player.setFishZ(z);

			if(!ZoneManager.getInstance().checkIfInZoneFishing(x, y, z))
			{
				player.sendPacket(new SystemMessage(SystemMessage.YOU_CANT_FISH_HERE));
				return false;
			}
		}

		L2Weapon weaponItem = player.getActiveWeaponItem();
		if(weaponItem == null || weaponItem.getItemType() != WeaponType.ROD)
		{
			//Fishing poles are not installed
			player.sendPacket(new SystemMessage(SystemMessage.FISHING_POLES_ARE_NOT_INSTALLED));
			return false;
		}

		return super.checkCondition(activeChar, target, forceUse, dontMove, first);
	}

	@Override
	public void useSkill(L2Character caster, FastList<L2Character> targets)
	{
		if(caster == null || !caster.isPlayer)
			return;

		L2Player player = (L2Player) caster;

		L2ItemInstance lure = player.getInventory().getPaperdollItem(Inventory.PAPERDOLL_LHAND);
		if(lure == null)
		{
			//Not enough bait
			player.sendPacket(new SystemMessage(SystemMessage.BAITS_ARE_NOT_PUT_ON_A_HOOK));
			return;
		}

		player.setLure(lure);
		L2ItemInstance lure2 = player.getInventory().destroyItem(player.getInventory().getPaperdollObjectId(Inventory.PAPERDOLL_LHAND), 1, true, false);

		if(lure2 == null || lure2.getCount() == 0)
			player.sendPacket(new SystemMessage(SystemMessage.NOT_ENOUGH_BAIT));

		player.stopMove(null);
		player.setImobilised(true);
		player.setFishing(true);
		player.broadcastUserInfo();
		//Starts fishing
		int lvl = FishTable.getInstance().GetRandomFishLvl(player.getAllEffects(), player.getSkillLevel((short) 1315));
		int group = FishTable.getInstance().GetRandomGroup(lure.getItemId());
		int type = FishTable.getInstance().GetRandomFishType(group, lure.getItemId());
		List<FishData> fishs = FishTable.getInstance().getfish(lvl, type, group);
		if(fishs == null || fishs.size() == 0)
		{
			player.sendMessage("Error - Fishes are not definied", "Ошибка - таблица рыбы пуста");
			player.endFishing(false);
			return;
		}
		int check = Rnd.get(fishs.size());
		FishData _fish = fishs.get(check);

		fishs.clear();
		fishs = null;
		player.sendPacket(new SystemMessage(SystemMessage.STARTS_FISHING));
		ExFishingStart efs = null;
		if(!GameTimeController.getInstance().isNowNight() && lure.isNightLure())
			_fish.setType(-1);
		efs = new ExFishingStart(player, _fish.getType(), player.getFishx(), player.getFishy(), player.getFishz(), lure.isNightLure());
		player.setFish(_fish);
		player.broadcastPacket(efs);
		player.startLookingForFishTask();
	}
}
