package L2jFT.game.skills.skillclasses;

import javolution.util.FastList;
import L2jFT.Config;
import L2jFT.util.Log;
import L2jFT.util.Rnd;
import L2jFT.game.cache.Msg;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.instances.L2MonsterInstance;
import L2jFT.game.network.serverpackets.InventoryUpdate;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.templates.StatsSet;

public class Harvesting extends L2Skill
{
	public Harvesting(StatsSet set)
	{
		super(set);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		if(!activeChar.isPlayer)
			return;

		boolean send_iu = false;
		L2Player player = (L2Player) activeChar;
		InventoryUpdate iu = new InventoryUpdate();

		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();

			if(!target.isMonster)
				continue;

			L2MonsterInstance monster = (L2MonsterInstance) target;

			// Не посеяно
			if(!monster.isSeeded())
			{
				activeChar.sendPacket(Msg.THE_HARVEST_FAILED_BECAUSE_THE_SEED_WAS_NOT_SOWN);
				continue;
			}

			int SuccessRate = Config.MANOR_HARVESTING_BASIC_SUCCESS;
			int diffPlayerTarget = Math.abs(activeChar.getLevel() - monster.getLevel());

			// Штраф, на разницу уровней между мобом и игроком 
			// 5% на каждый уровень при разнице >5 - по умолчанию
			if(diffPlayerTarget > Config.MANOR_DIFF_PLAYER_TARGET)
				SuccessRate -= (diffPlayerTarget - Config.MANOR_DIFF_PLAYER_TARGET) * Config.MANOR_DIFF_PLAYER_TARGET_PENALTY;

			// Минимальный шанс успеха всегда 1%
			if(SuccessRate < 1)
				SuccessRate = 1;

			if(Config.SKILLS_SHOW_CHANCE)
				activeChar.sendMessage("Harvesting chance: " + SuccessRate + "%.", "Шанс сбора: " + SuccessRate + "%.");

			if(!Rnd.chance(SuccessRate))
			{
				activeChar.sendPacket(Msg.THE_HARVEST_HAS_FAILED);
				monster.takeHarvest();
				continue;
			}

			L2ItemInstance[] items = monster.takeHarvest();

			for(L2ItemInstance item : items)
			{
				int itemCount = item.getCount();
				item = player.getInventory().addItem(item);
				Log.LogItem(player, target, Log.HarvesterItem, item);
				if(item.getLastChange() == L2ItemInstance.ADDED)
					iu.addNewItem(item);
				else
					iu.addModifiedItem(item);

				send_iu = true;
				player.sendPacket(new SystemMessage(SystemMessage.S1_HARVESTED_S3_S2_S).addString("You").addNumber(itemCount).addItemName(item.getItemId()));
				if(player.isInParty())
				{
					SystemMessage smsg = new SystemMessage(SystemMessage.S1_HARVESTED_S3_S2_S).addString(player.getName()).addNumber(itemCount).addItemName(item.getItemId());
					player.getParty().broadcastToPartyMembers(player, smsg);
				}
			}
		}

		if(send_iu)
			player.sendPacket(iu);
	}
}