package L2jFT.game.skills.skillclasses;

import javolution.util.FastList;
import L2jFT.game.cache.Msg;
import L2jFT.util.Rnd;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.skills.Formulas;
import L2jFT.game.skills.Stats;
import L2jFT.game.templates.StatsSet;

public class LethalShot extends L2Skill
{
	public final static float INSTANT_CHANCE = 0.025f;
	private int _lethal1;
	private int _lethal2;

	public LethalShot(StatsSet set)
	{
		super(set);
		_lethal1 = set.getInteger("lethal1", 0);
		_lethal2 = set.getInteger("lethal2", 0);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		boolean ss = activeChar.getChargedSoulShot();
		if(ss)
			activeChar.unChargeShots(false);

		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();
			if(target.isDead())
				continue;

			if(target.checkReflectSkill(this))
				target = activeChar;

			boolean shld = Formulas.calcShldUse(activeChar, target);

			double damage = Formulas.calcPhysDam(activeChar, target, this, shld, false, false, ss);

			double CHANCE = INSTANT_CHANCE;

			CHANCE *= 0.01 * target.calcStat(Stats.DEATH_RECEPTIVE, 100, null, null);
			_lethal1 *= 0.01 * target.calcStat(Stats.DEATH_RECEPTIVE, 100, null, null);
			_lethal2 *= 0.01 * target.calcStat(Stats.DEATH_RECEPTIVE, 100, null, null);

			if(!target.isLethalImmune() && CHANCE > Rnd.get())
			{
				damage = target.getCurrentCp() + target.getCurrentHp();
				if(target.isPlayer)
					target.sendPacket(Msg.YOU_HAVE_BEEN_STRUCK_BY_THE_INSTANT_KILL_SKILL);
				activeChar.sendPacket(Msg.INSTANT_KILL);
			}

			if(_lethal1 > 0 && Rnd.chance(_lethal1))
			{
				if(target.isPlayer)
				{
					damage += target.getCurrentCp();
					target.sendPacket(Msg.YOU_HAVE_BEEN_STRUCK_BY_THE_INSTANT_KILL_SKILL);
				}
				else if(target instanceof L2NpcInstance)
					if(target.isLethalImmune() || target.getMaxHp() > 50000)
						damage *= 2;
					else
						damage += target.getCurrentHp() / 2;

				activeChar.sendPacket(Msg.INSTANT_KILL);
			}

			if(_lethal2 > 0 && Rnd.chance(_lethal2))
			{
				if(target.isPlayer)
				{
					damage = target.getCurrentHp() + target.getCurrentCp() - 1 > damage ? target.getCurrentHp() + target.getCurrentCp() - 1 : damage;
					target.sendPacket(Msg.YOU_HAVE_BEEN_STRUCK_BY_THE_INSTANT_KILL_SKILL);
				}
				else if(target instanceof L2NpcInstance)
					if(target.isLethalImmune() || target.getMaxHp() > 50000)
						damage *= 2;
					else
						damage = target.getCurrentHp() - 1 > damage ? target.getCurrentHp() - 1 : damage;

				activeChar.sendPacket(Msg.INSTANT_KILL);
			}

			if(damage < 1)
				damage = 1;

			if(target.isPlayer)
				if(shld)
					if(damage == 1)
						target.sendPacket(Msg.YOUR_EXCELLENT_SHIELD_DEFENSE_WAS_A_SUCCESS);
					else
						target.sendPacket(Msg.SHIELD_DEFENSE_HAS_SUCCEEDED);

			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_HIT_FOR_S1_DAMAGE).addNumber((int) damage));
			target.reduceCurrentHp(damage, activeChar);
			getEffects(activeChar, target, getActivateRate() > 0);
		}
	}
}
