package L2jFT.game.skills.skillclasses;

import javolution.util.FastList;
import L2jFT.game.cache.Msg;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.L2Summon;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.skills.Formulas;
import L2jFT.game.templates.L2Weapon;
import L2jFT.game.templates.StatsSet;
import L2jFT.util.Rnd;

public class MDam extends L2Skill
{
	public static final double DEATHLINK_MULTIPLIER = 1.8;

	private final boolean _deathlink;
	private final boolean _cancelTarget;

	public MDam(StatsSet set)
	{
		super(set);
		_deathlink = set.getBool("deathlink", false);
		_cancelTarget = set.getBool("cancelTarget", false);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		double deathlinkMult = DEATHLINK_MULTIPLIER * (1. - activeChar.getCurrentHp() / activeChar.getMaxHp());
		int sps = !isHandler() ? activeChar.getChargedSpiritShot() : 0;
		L2Weapon weapon = activeChar.getActiveWeaponItem();

		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();
			if(target.isDead())
				continue;

			if(target.checkReflectSkill(this))
				target = activeChar;

			if(_cancelTarget && Rnd.chance(80))
				target.setTarget(null);

			boolean shield = Formulas.calcShldUse(activeChar, target);
			double damage = Formulas.calcMagicDam(activeChar, target, this, shield, sps);

			if(_deathlink)
				damage *= deathlinkMult;

			if(damage < 1)
				damage = 1;

			if(target.isPlayer)
				if(shield)
					if(damage == 1)
						target.sendPacket(Msg.YOUR_EXCELLENT_SHIELD_DEFENSE_WAS_A_SUCCESS);
					else
						target.sendPacket(Msg.SHIELD_DEFENSE_HAS_SUCCEEDED);

			if(activeChar.isPlayer)
				activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_HIT_FOR_S1_DAMAGE).addNumber((int) damage));
			else if(activeChar instanceof L2Summon)
				((L2Summon) activeChar).sendDamageMsg((int) damage);

			target.reduceCurrentHp(damage, activeChar);

			getEffects(activeChar, target, getActivateRate() > 0);
			if(weapon != null)
				weapon.getEffect(false, activeChar, target, isOffensive());
		}
		if(sps != 0)
			activeChar.unChargeShots(true);

		if(isSuicideAttack())
			activeChar.doDie(null);
	}
}
