package L2jFT.game.skills.skillclasses;

import javolution.util.FastList;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Effect;
import L2jFT.game.model.L2Skill;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.templates.L2Weapon;
import L2jFT.game.templates.StatsSet;

public class NegateEffects extends L2Skill
{
	private final SkillType[] _negateEffects;
	private final float _negatePower;
	private final short _numCharges;
	private final short _condCharges;

	public NegateEffects(StatsSet set)
	{
		super(set);

		String[] negateEffectsString = set.getString("negateEffects", "").split(" ");
		_negateEffects = new SkillType[negateEffectsString.length];
		for(int i = 0; i < negateEffectsString.length; i++)
			_negateEffects[i] = Enum.valueOf(SkillType.class, negateEffectsString[i]);
		_negatePower = set.getFloat("negatePower", 0.f);
		_numCharges = set.getShort("num_charges", (short) 0);
		_condCharges = set.getShort("cond_charges", (short) 0);
	}

	@Override
	public boolean checkCondition(final L2Character activeChar, final L2Character target, boolean forceUse, boolean dontMove, boolean first)
	{
		if(activeChar.getIncreasedForce() < _condCharges || activeChar.getIncreasedForce() < _numCharges)
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_CAN_NO_LONGER_INCREASE_YOUR_FORCE).addString(getName()));
			return false;
		}

		return super.checkCondition(activeChar, target, forceUse, dontMove, first);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		if(activeChar.getIncreasedForce() < _numCharges || activeChar.getIncreasedForce() < _condCharges)
			return;
		activeChar.setIncreasedForce(activeChar.getIncreasedForce() - _numCharges);

		if(!isHandler())
			activeChar.unChargeShots(isMagic());
		L2Weapon weapon = activeChar.getActiveWeaponItem();
		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();
			for(SkillType stat : _negateEffects)
				if(stat == SkillType.POISON || stat == SkillType.BLEED)
					negateEffectAtPower(target, stat, _negatePower);
				else
					negateEffect(target, stat);
			if(weapon != null)
				weapon.getEffect(false, activeChar, target, isOffensive());
			getEffects(activeChar, target, getActivateRate() > 0);
		}
	}

	private void negateEffect(L2Character target, SkillType type)
	{
		for(L2Effect e : target.getAllEffects())
		{
			L2Skill skill = e.getSkill();
			if(skill.getSkillType() == type && skill.isCancelable())
				e.exit();
		}
	}

	private void negateEffectAtPower(L2Character target, SkillType type, double power)
	{
		for(L2Effect e : target.getAllEffects())
		{
			L2Skill skill = e.getSkill();
			if(skill.getSkillType() == type && skill.isCancelable() && skill.getPower() <= power)
				e.exit();
		}
	}
}