package L2jFT.game.skills.skillclasses;

import javolution.util.FastList;
import L2jFT.game.cache.Msg;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.L2Summon;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.skills.Formulas;
import L2jFT.game.templates.StatsSet;
import L2jFT.util.Rnd;

public class PDam extends L2Skill
{
	public static final double DEATHLINK_PDAM_MULTIPLIER = 1.8;

	private final boolean _deathlink;
	private final boolean _cancelTarget;
	private final boolean _skillInterrupt;

	public PDam(StatsSet set)
	{
		super(set);
		_deathlink = set.getBool("deathlink", false);
		_cancelTarget = set.getBool("cancelTarget", false);
		_skillInterrupt = set.getBool("skillInterrupt", false);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		boolean ss = activeChar.getChargedSoulShot() && !isHandler();
		if(ss)
			activeChar.unChargeShots(false);

		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();
			if(target.isDead())
				continue;
			
			target.doCounterAttack(this, activeChar);

			if(target.checkReflectSkill(this))
				target = activeChar;

			boolean shld = !getShieldIgnore() && Formulas.calcShldUse(activeChar, target);

			double damage = Formulas.calcPhysDam(activeChar, target, this, shld, false, false, ss);

			if(_deathlink)
				damage *= DEATHLINK_PDAM_MULTIPLIER * (1. - activeChar.getCurrentHp() / activeChar.getMaxHp());
			if(activeChar.isPlayer)
				activeChar.sendPacket(new SystemMessage(SystemMessage.YOU_HIT_FOR_S1_DAMAGE).addNumber((int) damage));
			else if(activeChar instanceof L2Summon)
				((L2Summon) activeChar).sendDamageMsg((int) damage);

			if(damage < 1)
				damage = 1;

			if(target.isPlayer)
				if(shld)
					if(damage == 1)
						target.sendPacket(Msg.YOUR_EXCELLENT_SHIELD_DEFENSE_WAS_A_SUCCESS);
					else
						target.sendPacket(Msg.SHIELD_DEFENSE_HAS_SUCCEEDED);

			target.reduceCurrentHp(damage, activeChar);

			if(_cancelTarget && Rnd.chance(80))
				target.setTarget(null);

			if(_skillInterrupt)
			{
				if(target.getCastingSkill() != null && !target.getCastingSkill().isMagic())
					target.abortCast();
				target.abortAttack();
			}

			if(!_id.equals(L2Skill.EVADE_SHOT))
				getEffects(activeChar, target, getActivateRate() > 0);

			if(isSuicideAttack())
				activeChar.doDie(null);
		}
	}
}