package L2jFT.game.skills.skillclasses;

import javolution.util.FastList;
import L2jFT.game.cache.Msg;
import L2jFT.game.instancemanager.TownManager;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Playable;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.entity.Castle;
import L2jFT.game.model.instances.L2PetInstance;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.templates.StatsSet;

public class Resurrect extends L2Skill
{
	private final boolean _canPet;

	public Resurrect(StatsSet set)
	{
		super(set);
		_canPet = set.getBool("canPet", false);
	}

	@Override
	public boolean checkCondition(final L2Character activeChar, final L2Character target, boolean forceUse, boolean dontMove, boolean first)
	{
		if(!activeChar.isPlayer)
			return false;

		L2Player player = (L2Player) activeChar;

		Castle castle = TownManager.getInstance().getClosestTown(player).getCastle();
		if(castle != null && castle.getSiege().isInProgress())
		{
			activeChar.sendPacket(new SystemMessage(SystemMessage.IT_IS_IMPOSSIBLE_TO_BE_RESSURECTED_IN_BATTLEFIELDS_WHERE_SIEGE_WARS_ARE_IN_PROCESS));
			return false;
		}

		if(_targetType == SkillTargetType.TARGET_ONE)
		{
			if(target == null || !target.isDead())
				return false;

			if(!(target instanceof L2Playable))
			{
				player.sendPacket(Msg.TARGET_IS_INCORRECT);
				return false;
			}

			if(target.isPet)
			{
				if(target.getOwner() == null)
				{
					player.sendPacket(Msg.TARGET_IS_INCORRECT);
					return false;
				}
				if(target.getOwner().isReviveRequested())
				{
					if(target.getOwner().isRevivingPet())
						activeChar.sendPacket(new SystemMessage(SystemMessage.BETTER_RESURRECTION_HAS_BEEN_ALREADY_PROPOSED));
					else
						activeChar.sendPacket(new SystemMessage(SystemMessage.SINCE_THE_MASTER_WAS_IN_THE_PROCESS_OF_BEING_RESURRECTED_THE_ATTEMPT_TO_RESURRECT_THE_PET_HAS_BEEN_CANCELLED));
					return false;
				}
				if(!(_canPet || _targetType == SkillTargetType.TARGET_PET))
				{
					player.sendPacket(Msg.TARGET_IS_INCORRECT);
					return false;
				}
			}
			else if(target.isPlayer)
			{
				if(((L2Player) target).isReviveRequested())
				{
					if(((L2Player) target).isRevivingPet())
						activeChar.sendPacket(new SystemMessage(SystemMessage.WHILE_A_PET_IS_ATTEMPTING_TO_RESURRECT_IT_CANNOT_HELP_IN_RESURRECTING_ITS_MASTER)); // While a pet is attempting to resurrect, it cannot help in resurrecting its master.
					else
						activeChar.sendPacket(new SystemMessage(SystemMessage.BETTER_RESURRECTION_HAS_BEEN_ALREADY_PROPOSED)); // Resurrection is already been proposed.
					return false;
				}
				if(_targetType == SkillTargetType.TARGET_PET)
				{
					player.sendPacket(Msg.TARGET_IS_INCORRECT);
					return false;
				}
				// Check to see if the player is in a festival.
				if(((L2Player) target).isFestivalParticipant())
				{
					player.sendMessage("You may not resurrect participants in a festival.", "Вам нельзя воскрешать игроков на Фестивале");
					return false;
				}
			}
		}

		return super.checkCondition(activeChar, target, forceUse, dontMove, first);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		double percent = _power;

		if(_id <= 2014)
		{
			int wit = activeChar.getWIT();
			percent = 0.14 * wit * wit - 0.67 * wit - 43.;

			percent = _power * (1. + percent / 100.);

			if(percent > _power + 20)
				percent = _power + 20;
		}

		if(!isHandler())
			activeChar.unChargeShots(true);

		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();

			if(target.getOwner() == null)
				continue;

			if(target.isPet && _canPet)
			{
				if(target.getOwner() == activeChar)
					((L2PetInstance) target).doRevive(percent);
				else
					target.getOwner().reviveRequest((L2Player) activeChar, percent, true);
			}
			else if(target.isPlayer)
			{
				if(_targetType == SkillTargetType.TARGET_PET)
					continue;

				L2Player targetPlayer = (L2Player) target;

				if(targetPlayer.isReviveRequested())
					continue;

				if(targetPlayer.isFestivalParticipant())
					continue;

				targetPlayer.reviveRequest((L2Player) activeChar, percent, false);
			}
			else
				continue;

			getEffects(activeChar, target, getActivateRate() > 0);
		}
	}
}
