package L2jFT.game.skills.skillclasses;

import javolution.util.FastList;
import L2jFT.game.idfactory.IdFactory;
import L2jFT.game.instancemanager.SiegeManager;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2SiegeClan;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.L2Zone.ZoneType;
import L2jFT.game.model.entity.siege.Siege;
import L2jFT.game.model.instances.L2SiegeFlagInstance;
import L2jFT.game.skills.Stats;
import L2jFT.game.skills.funcs.FuncFactory;
import L2jFT.game.tables.NpcTable;
import L2jFT.game.templates.StatsSet;

import java.util.logging.Logger;

public class SiegeFlag extends L2Skill
{
	protected static Logger _log = Logger.getLogger(SiegeFlag.class.getName());
	private final boolean _advanced;
	private final double _advancedMult;

	public SiegeFlag(StatsSet set)
	{
		super(set);
		_advanced = set.getBool("advancedFlag", false);
		_advancedMult = set.getDouble("advancedMultiplier", 1.);
	}

	@Override
	public boolean checkCondition(L2Character activeChar, L2Character target, boolean forceUse, boolean dontMove, boolean first)
	{
		if(!activeChar.isPlayer)
			return false;
		if(!super.checkCondition(activeChar, target, forceUse, dontMove, first))
			return false;

		L2Player player = (L2Player) activeChar;
		if(player.getClan() == null || !player.isClanLeader())
		{
			_log.warning(player.toFullString() + " has " + toString() + ", but he isn't in a clan leader.");
			return false;
		}

		Siege siege = SiegeManager.getInstance().getSiege(activeChar);
		if(siege == null || !checkIfOkToPlaceFlag(player, siege))
			return false;

		return true;
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		L2Player player = (L2Player) activeChar;

		Siege siege = SiegeManager.getInstance().getSiege(activeChar);

		if(siege == null || !checkIfOkToPlaceFlag(player, siege))
			return;

		L2SiegeFlagInstance flag = new L2SiegeFlagInstance(player, IdFactory.getInstance().getNextId(), NpcTable.getTemplate(35062));

		if(_advanced)
			try
			{
				flag.addStatFunc(FuncFactory.createFunc("Mul", Stats.MAX_HP, 0x50, _advancedMult, flag));
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}

		flag.setCurrentHpMp(flag.getMaxHp(), flag.getMaxMp());
		flag.setHeading(player.getHeading());

		// Ставим флаг перед чаром
		int x = (int) (player.getX() + 100 * Math.cos(player.headingToRadians(player.getHeading() - 32768)));
		int y = (int) (player.getY() + 100 * Math.sin(player.headingToRadians(player.getHeading() - 32768)));
		flag.spawnMe(x, y, player.getZ() + 50);

		L2SiegeClan clan = siege.getAttackerClan(player.getClan());
		if(clan != null)
			clan.setFlag(flag);
	}

	public boolean checkIfOkToPlaceFlag(L2Player activeChar, Siege siege)
	{
		if(siege == null || !activeChar.isInZone(ZoneType.Siege))
			activeChar.sendMessage("You must be on siege field to place a flag.");
		else if(!siege.isInProgress())
			activeChar.sendMessage("You can only place a flag during a siege.");
		else if(siege.getAttackerClan(activeChar.getClan()) == null)
			activeChar.sendMessage("You must be an attacker to place a flag.");
		else if(activeChar.getClan() == null || !activeChar.isClanLeader())
			activeChar.sendMessage("You must be a clan leader to place a flag.");
		else
			return true;
		return false;
	}
}
