package L2jFT.game.skills.skillclasses;

import javolution.util.FastList;
import L2jFT.game.geodata.GeoData;
import L2jFT.game.idfactory.IdFactory;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.L2World;
import L2jFT.game.model.base.Experience;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.model.instances.L2SummonInstance;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.NpcTable;
import L2jFT.game.templates.L2NpcTemplate;
import L2jFT.game.templates.StatsSet;
import L2jFT.util.CharPosition;
import L2jFT.util.Rnd;

public class Summon extends L2Skill
{
	private final short _npcId;
	private final float _expPenalty;
	private final boolean _isCubic;
	private final int _itemConsumeIdInTime;
	private final int _itemConsumeCountInTime;
	private final int _itemConsumeDelay;
	private final int _lifeTime;

	public Summon(StatsSet set)
	{
		super(set);

		_npcId = set.getShort("npcId", (short) 0);
		_expPenalty = set.getFloat("expPenalty", 0.f);
		_isCubic = set.getBool("isCubic", false);
		_itemConsumeIdInTime = set.getInteger("itemConsumeIdInTime", 0);
		_itemConsumeCountInTime = set.getInteger("itemConsumeCountInTime", 0);
		_itemConsumeDelay = set.getInteger("itemConsumeDelay", 250) * 1000;
		_lifeTime = set.getInteger("lifeTime", 1200) * 1000;
	}

	@Override
	public boolean checkCondition(L2Character activeChar, L2Character target, boolean forceUse, boolean dontMove, boolean first)
	{
		if(activeChar.isPlayer)
		{
			L2Player player = (L2Player) activeChar;
			if(!_isCubic)
			{
				if(player.getPet() != null)
				{
					player.sendPacket(new SystemMessage(SystemMessage.YOU_ALREADY_HAVE_A_PET));
					return false;
				}
			}
			else if(_targetType == SkillTargetType.TARGET_SELF)
			{
				int mastery = player.getSkillLevel(L2Skill.SKILL_CUBIC_MASTERY);
				if(mastery < 0)
					mastery = 0;
				int count = player.getCubics().size();
				if(count > mastery && player.getCubic(_npcId) == null)
					return false;
			}
		}
		return super.checkCondition(activeChar, target, forceUse, dontMove, first);
	}

	@Override
	public void useSkill(L2Character caster, FastList<L2Character> targets)
	{
		if(_npcId == 0)
		{
			caster.sendMessage("Summon skill " + getId() + " not described yet", "");
			return;
		}

		if(!isHandler())
			caster.unChargeShots(isMagic());

		if(_isCubic)
		{
			for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
			{
				L2Character targ = n.getValue();
				if(!targ.isPlayer)
					continue;

				L2Player target = (L2Player) targ;
				int mastery = target.getSkillLevel(L2Skill.SKILL_CUBIC_MASTERY);
				if(mastery < 0)
					mastery = 0;
				if(target.getCubics().size() > mastery && target.getCubic(_npcId) == null)
				{
					target.sendPacket(new SystemMessage(SystemMessage.CUBIC_SUMMONING_FAILED));
					continue;
				}
				target.addCubic(_npcId, _level, _lifeTime);
				target.broadcastUserInfo();

				getEffects(caster, target, getActivateRate() > 0);
			}
			return;
		}

		if(!caster.isPlayer)
			return;

		L2Player activeChar = (L2Player) caster;

		// Удаление трупа, если идет суммон из трупа.
		if(_targetType == SkillTargetType.TARGET_CORPSE)
			for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
			{
				L2Character target = n.getValue();
				if(target != null && target.isDead() && target instanceof L2NpcInstance)
					((L2NpcInstance) target).endDecayTask();
			}

		if(activeChar.getPet() != null)
			return;

		L2NpcTemplate summonTemplate = NpcTable.getTemplate(_npcId);
		L2SummonInstance summon = new L2SummonInstance(IdFactory.getInstance().getNextId(), summonTemplate, activeChar, _lifeTime, _itemConsumeIdInTime, _itemConsumeCountInTime, _itemConsumeDelay);

		summon.setTitle(activeChar.getName());

		summon.setExpPenalty(_expPenalty);
		if(summon.getLevel() >= Experience.LEVEL.length)
		{
			summon.setExp(Experience.LEVEL[Experience.LEVEL.length - 1]);
			_log.warning("Summon (" + summon.getName() + ") NpcID: " + summon.getNpcId() + " has a level above " + (Experience.LEVEL.length - 1) + ". Please rectify.");
		}
		else
			summon.setExp(Experience.LEVEL[(summon.getLevel() % Experience.LEVEL.length)]);

		summon.setCurrentHp(summon.getMaxHp());
		summon.setCurrentMp(summon.getMaxMp());
		summon.setHeading(activeChar.getHeading());
		summon.setRunning();
		activeChar.setPet(summon);

		L2World.addObject(summon);

		// Пытаемся найти координаты с разбросом, если не получается - берем без разброса 
		int[] coords = new int[3];
		for(int i = 0; i < 100; i++)
		{
			CharPosition cp = Rnd.coordsRandomize(activeChar, 150);
			if(GeoData.getInstance().canMoveToCoord(cp.x, cp.y, cp.z, activeChar.getX(), activeChar.getY(), activeChar.getZ()))
			{
				coords[0] = cp.x;
				coords[1] = cp.y;
				coords[2] = cp.z;
				break;
			}
		}
		if(coords[0] == 0)
		{
			coords[0] = activeChar.getX();
			coords[1] = activeChar.getY();
			coords[2] = activeChar.getZ();
		}
		summon.spawnMe(coords[0], coords[1], coords[2]);

		summon.setFollowStatus(true);
		summon.setShowSummonAnimation(false); // addVisibleObject created the info packets with summon animation

		// if someone comes into range now, the animation shouldn't show any more

		summon.broadcastPetInfo();

		//summon.startConsume();
	}

	@Override
	public boolean isOffensive()
	{
		return _targetType == SkillTargetType.TARGET_CORPSE;
	}
}
