package L2jFT.game.skills.skillclasses;

import javolution.util.FastList;
import L2jFT.util.Rnd;
import L2jFT.game.model.Inventory;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.instances.L2PetInstance;
import L2jFT.game.network.serverpackets.InventoryUpdate;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.tables.ItemTable;
import L2jFT.game.templates.StatsSet;

public class SummonItem extends L2Skill
{
	private final short _itemId;
	private final int _minId;
	private final int _maxId;
	private final int _minCount;
	private final int _maxCount;

	public SummonItem(final StatsSet set)
	{
		super(set);

		_itemId = set.getShort("SummonItemId", (short) 0);
		_minId = set.getInteger("SummonMinId", 0);
		_maxId = set.getInteger("SummonMaxId", _minId);
		_minCount = set.getInteger("SummonMinCount");
		_maxCount = set.getInteger("SummonMaxCount", _minCount);
	}

	@Override
	public void useSkill(final L2Character activeChar, final FastList<L2Character> targets)
	{
		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();
			Inventory inventory;
			if(target.isPlayer)
				inventory = ((L2Player) target).getInventory();
			else if(target.isPet)
				inventory = ((L2PetInstance) target).getInventory();
			else
				continue;

			L2ItemInstance item;
			if(_minId > 0)
				item = ItemTable.getInstance().createItem((short) Rnd.get(_minId, _maxId), activeChar.getObjectId(), getId(), "SkillSummonItem");
			else
				item = ItemTable.getInstance().createItem(_itemId, activeChar.getObjectId(), getId(), "SkillSummonItem");

			final int count = Rnd.get(_minCount, _maxCount);
			item.setCount(count);
			activeChar.sendPacket(new SystemMessage(SystemMessage.EARNED_S2_S1_s).addItemName(item.getItemId()).addNumber(count));
			item = inventory.addItem(item);
			activeChar.sendPacket(new InventoryUpdate().addItem(item));
			activeChar.sendChanges();
			getEffects(activeChar, target, getActivateRate() > 0);
		}
	}
}