package L2jFT.game.skills.skillclasses;

import javolution.util.FastList;
import L2jFT.game.instancemanager.CastleManager;
import L2jFT.game.instancemanager.SiegeManager;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2SiegeClan;
import L2jFT.game.model.L2Skill;
import L2jFT.game.model.entity.Castle;
import L2jFT.game.model.entity.siege.Siege;
import L2jFT.game.model.instances.L2ArtefactInstance;
import L2jFT.game.network.serverpackets.SystemMessage;
import L2jFT.game.templates.StatsSet;

public class TakeCastle extends L2Skill
{
	@Override
	public boolean checkCondition(L2Character activeChar, L2Character target, boolean forceUse, boolean dontMove, boolean first)
	{
		if(!super.checkCondition(activeChar, target, forceUse, dontMove, first) && checkIfOkToCastSealOfRule(activeChar, target))
			return false;

		if(first)
		{
			Siege s = SiegeManager.getInstance().getSiege(activeChar);
			if(s != null)
				for(L2SiegeClan sc : s.getDefenderClans().values())
				{
					L2Clan clan = sc.getClan();
					if(clan != null)
						for(L2Player player : clan.getOnlineMembers(""))
							if(player != null)
								player.sendPacket(new SystemMessage(SystemMessage.THE_OPPONENT_CLAN_HAS_BEGUN_TO_ENGRAVE_THE_RULER));
				}
		}
		
		return true;
	}

	public TakeCastle(StatsSet set)
	{
		super(set);
	}

	@Override
	public void useSkill(L2Character activeChar, FastList<L2Character> targets)
	{
		for(FastList.Node<L2Character> n = targets.head(), end = targets.tail(); (n = n.getNext()) != end;)
		{
			L2Character target = n.getValue();
			if(!(target instanceof L2ArtefactInstance))
				continue;
			L2Player player = (L2Player) activeChar;
			Castle castle = CastleManager.getInstance().getCastleByObject(player);
			if(castle.getSiege().isInProgress())
			{
				Siege.announceToPlayer(new SystemMessage(SystemMessage.CLAN_S1_HAS_SUCCEEDED_IN_ENGRAVING_THE_RULER).addString(player.getClan().getName()), false);
				castle.Engrave(player.getClan(), target.getObjectId());
			}
		}
	}

	public boolean checkIfOkToCastSealOfRule(L2Character activeChar, L2Character target)
	{
		if(activeChar == null || !activeChar.isPlayer)
			return false;

		L2Player player = (L2Player) activeChar;
		if(player.getClan() == null || !player.isClanLeader())
			return false;

		Siege siege = SiegeManager.getInstance().getSiege(activeChar);
		if(siege == null)
			return false;
		if(!siege.isInProgress())
			return false;
		if(!player.isInRange(target, 200))
			return false;
		if(siege.getAttackerClan(player.getClan()) == null)
			return false;

		return true;
	}
}
