package L2jFT.game.tables;

import javolution.util.FastMap;
import L2jFT.database.DatabaseUtils;
import L2jFT.database.FiltredPreparedStatement;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.database.ThreadConnection;
import L2jFT.game.model.L2TeleportLocation;

import java.sql.ResultSet;
import java.util.Map;
import java.util.logging.Logger;

/**
 * This class ...
 *
 * @version $Revision: 1.3.2.2.2.3 $ $Date: 2005/03/27 15:29:18 $
 */
public class ClanHallTeleportsTable
{
	private static Logger _log = Logger.getLogger(ClanHallTeleportsTable.class.getName());

	private static ClanHallTeleportsTable _instance;

	private Map<Integer, L2TeleportLocation> _teleports;

	public static ClanHallTeleportsTable getInstance()
	{
		if(_instance == null)
			_instance = new ClanHallTeleportsTable();
		return _instance;
	}

	private ClanHallTeleportsTable()
	{
		reloadAll();
	}

	public void reloadAll()
	{
		_teleports = new FastMap<Integer, L2TeleportLocation>();

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT Description, id, loc_x, loc_y, loc_z, price, fornoble FROM teleport");
			rset = statement.executeQuery();
			L2TeleportLocation teleport;

			while(rset.next())
			{
				teleport = new L2TeleportLocation();

				teleport.setTeleId(rset.getInt("id"));
				teleport.setLocX(rset.getInt("loc_x"));
				teleport.setLocY(rset.getInt("loc_y"));
				teleport.setLocZ(rset.getInt("loc_z"));
				teleport.setPrice(rset.getInt("price"));
				_teleports.put(teleport.getTeleId(), teleport);
			}

			_log.config("TeleportLocationTable: Loaded " + _teleports.size() + " Teleport Location Templates.");
		}
		catch(Exception e)
		{
			_log.warning("error while creating teleport table " + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	/**
	 * @param template id
	 * @return
	 */
	public L2TeleportLocation getTemplate(int id)
	{
		return _teleports.get(id);
	}
}
