package L2jFT.game.tables;

import L2jFT.Config;
import L2jFT.database.DatabaseUtils;
import L2jFT.database.FiltredPreparedStatement;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.database.ThreadConnection;
import L2jFT.game.handler.IScriptHandler;
import L2jFT.game.handler.ScriptHandler;
import L2jFT.game.instancemanager.CastleManager;
import L2jFT.game.instancemanager.ClanHallManager;
import L2jFT.game.instancemanager.SiegeManager;
import L2jFT.game.instancemanager.TownManager;
import L2jFT.util.CharPosition;
import L2jFT.util.Location;
import L2jFT.util.Rnd;
import L2jFT.game.model.L2Character;
import L2jFT.game.model.L2Clan;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Zone;
import L2jFT.game.model.L2Zone.ZoneType;
import L2jFT.game.model.base.Race;
import L2jFT.game.model.entity.siege.Siege;
import L2jFT.game.model.instances.L2NpcInstance;

import java.sql.ResultSet;
import java.util.logging.Logger;

@SuppressWarnings( { "nls", "unqualified-field-access", "boxing" })
public class MapRegionTable
{
	private final static Logger _log = Logger.getLogger(MapRegionTable.class.getName());

	private static MapRegionTable _instance;

	private final int[][] _regions = new int[19][21];

	public static enum TeleportWhereType
	{
		Castle,
		ClanHall,
		ClosestTown,
		SecondClosestTown,
		SiegeFlag,
		MapLoc
	}

	public static MapRegionTable getInstance()
	{
		if(_instance == null)
			_instance = new MapRegionTable();
		return _instance;
	}

	private MapRegionTable()
	{
		int count2 = 0;

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT region, sec0, sec1, sec2, sec3, sec4, sec5, sec6, sec7, sec8, sec9, sec10 FROM mapregion");
			rset = statement.executeQuery();
			int region;
			while(rset.next())
			{
				region = rset.getInt(1);

				for(int j = 0; j < 11; j++)
				{
					_regions[j][region] = rset.getInt(j + 2);
					count2++;
				}
			}

			if(Config.DEBUG)
				_log.fine(count2 + " mapregion loaded");
		}
		catch(Exception e)
		{
			_log.warning("error while creating map region data: " + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
	}

	public final int getMapRegion(int posX, int posY)
	{
		int tileX = (posX >> 15) + 4; // + centerTileX;
		int tileY = (posY >> 15) + 10; // + centerTileY;
		return _regions[tileX][tileY];
	}

	public int getClosestTownNumber(L2Character activeChar)
	{
		return getMapRegion(activeChar.getX(), activeChar.getY());
	}

	public String getClosestTownName(L2Character activeChar)
	{
		int nearestTownId = getMapRegion(activeChar.getX(), activeChar.getY());
		String nearestTown;

		switch(nearestTownId)
		{
			case 0:
				nearestTown = "Talking Island Village";
				break;
			case 1:
				nearestTown = "Elven Village";
				break;
			case 2:
				nearestTown = "Dark Elven Village";
				break;
			case 3:
				nearestTown = "Orc Village";
				break;
			case 4:
				nearestTown = "Dwarven Village";
				break;
			case 5:
				nearestTown = "Town of Gludio";
				break;
			case 6:
				nearestTown = "Gludin Village";
				break;
			case 7:
				nearestTown = "Town of Dion";
				break;
			case 8:
				nearestTown = "Giran Castle Town";
				break;
			case 9:
				nearestTown = "Town of Oren";
				break;
			case 10:
				nearestTown = "Town of Aden";
				break;
			case 11:
				nearestTown = "Hunters Village";
				break;
			case 12:
				nearestTown = "Giran Harbor";
				break;
			case 13:
				nearestTown = "Heine";
				break;
			case 14:
				nearestTown = "Rune Township";
				break;
			case 15:
				nearestTown = "Town of Goddard";
				break;
			case 16:
				nearestTown = "Town of Schuttgart";
				break;
			case 17:
				nearestTown = "dino_peace";
				break;
			default:
				nearestTown = "Town of Aden";
				break;
		}

		return nearestTown;
	}

	public int getAreaCastle(L2Character activeChar)
	{
		int area = getClosestTownNumber(activeChar);
		int castle;
		switch(area)
		{
			case 0:
				castle = 1;
				break;//Talking Island Village 
			case 1:
				castle = 4;
				break; //Elven Village 
			case 2:
				castle = 4;
				break; //Dark Elven Village 
			case 3:
				castle = 9;
				break; //Orc Village 
			case 4:
				castle = 9;
				break; //Dwarven Village 
			case 5:
				castle = 1;
				break; //Town of Gludio 
			case 6:
				castle = 1;
				break; //Gludin Village 
			case 7:
				castle = 2;
				break; //Town of Dion 
			case 8:
				castle = 3;
				break; //Town of Giran 
			case 9:
				castle = 4;
				break; //Town of Oren 
			case 10:
				castle = 5;
				break; //Town of Aden 
			case 11:
				castle = 5;
				break; //Hunters Village 
			case 12:
				castle = 3;
				break; //Giran Harbor 
			case 13:
				castle = 6;
				break; //Heine 
			case 14:
				castle = 8;
				break; //Rune Township 
			case 15:
				castle = 7;
				break; //Town of Goddard 
			case 16:
				castle = 9;
				break; //Town of Shuttgart  
			case 17:
				castle = 8;
				break; //Primeval Isle Wharf
			default:
				castle = 5;
				break; //Town of Aden 
		}
		return castle;
	}

	public Location getTeleToLocation(L2Character activeChar, TeleportWhereType teleportWhere)
	{
		L2Zone region = activeChar.getZone(ZoneType.region);

		L2Player player = activeChar.getPlayer();

		if(player != null)
		{
			IScriptHandler handler = ScriptHandler.getInstance().getScriptHandler();
			if(handler != null)
			{
				Location loc = handler.onEscape(player);
				if(loc != null)
					return loc;
			}

			/**
			 if(player.getKarma() > 1)
			 {
			 // PK > 5 with karma -> Floran Village
			 if(player.getPkKills() > 5 || region == null)
			 return new Location(17817, 170079, -3530);
			 // PK <= 5 with karma -> out town
			 return region.getPKSpawn();
			 }
			 */

			L2Clan clan = player.getClan();

			if(clan != null)
			{
				// If teleport to clan hall
				if(teleportWhere == TeleportWhereType.ClanHall && clan.getHasHideout() != 0)
					return ClanHallManager.getInstance().getClanHall(clan.getHasHideout()).getZone().getSpawn();

				// If teleport to castle
				if(teleportWhere == TeleportWhereType.Castle && clan.getHasCastle() != 0)
					return CastleManager.getInstance().getCastleByIndex(clan.getHasCastle()).getZone().getSpawn();

				//if teleport to map_loc
				/*
				 if(teleportWhere == TeleportWhereType.MapLoc)
				 return ZoneManager.getInstance().getZoneByIndex(ZoneType.region, getMapRegion(player.getX(), player.getY())).getSpawn();
				 */

				// Checking if in siege
				Siege siege = SiegeManager.getInstance().getSiege(activeChar);
				if(siege != null && siege.isInProgress())
				{
					if(teleportWhere == TeleportWhereType.SiegeFlag)
					{
						// Check if player's clan is attacker
						L2NpcInstance flag = siege.getFlag(player.getClan());
						if(flag != null)
						{
							// Спаун рядом с флагом
							CharPosition pos = Rnd.coordsRandomize(flag.getX(), flag.getY(), flag.getZ(), 0, 49, 51);
							return new Location(pos.x, pos.y, flag.getZ());
						}
						return TownManager.getInstance().getClosestTown(activeChar).getSpawn();
					}

					// Check if player's clan is defender
					if(siege.getDefenderClan(player.getClan()) != null && siege.getCastle() != null && siege.getCastle().getZone() != null)
						return player.getKarma() > 1 ? siege.getCastle().getZone().getPKSpawn() : siege.getCastle().getZone().getSpawn();
					return player.getKarma() > 1 ? TownManager.getInstance().getClosestTown(activeChar).getPKSpawn() : TownManager.getInstance().getClosestTown(activeChar).getSpawn();
				}
			}

			// Светлые эльфы не могут воскрешаться в городе темных
			if(player.getRace() == Race.Elf && TownManager.getInstance().getClosestTown(activeChar).getTownId() == 1)
				return player.getKarma() > 1 ? TownManager.getInstance().getTown(3).getPKSpawn() : TownManager.getInstance().getTown(3).getSpawn();

			// Темные эльфы не могут воскрешаться в городе светлых
			if(player.getRace() == Race.DarkElf && TownManager.getInstance().getClosestTown(activeChar).getTownId() == 3)
				return player.getKarma() > 1 ? TownManager.getInstance().getTown(1).getPKSpawn() : TownManager.getInstance().getTown(1).getSpawn();

			L2Zone battle = activeChar.getZone(ZoneType.battle_zone);

			// If in battle zone
			if(battle != null && battle.getRestartPoints() != null)
				return player.getKarma() > 1 ? battle.getPKSpawn() : battle.getSpawn();

			if(activeChar.isInZone(ZoneType.peace_zone) && activeChar.getZone(ZoneType.peace_zone).getRestartPoints() != null)
				return player.getKarma() > 1 ? activeChar.getZone(ZoneType.peace_zone).getPKSpawn() : activeChar.getZone(ZoneType.peace_zone).getSpawn();

			if(region != null)
				return player.getKarma() > 1 ? region.getPKSpawn() : region.getSpawn();

			// Заглушка, телепорт старым методом по mapregion. пспользовать только до тех пор, пока не опишем зоны типа region
			return player.getKarma() > 1 ? TownManager.getInstance().getClosestTown(activeChar).getPKSpawn() : TownManager.getInstance().getClosestTown(activeChar).getSpawn();

			// Если не найдена точка телепортации - отправляем во флоран
			//return new Location(17817, 170079, -3530);
		}

		return TownManager.getInstance().getClosestTown(activeChar).getSpawn();
	}
}
