package L2jFT.game.tables;

import L2jFT.Config;
import L2jFT.database.DatabaseUtils;
import L2jFT.database.FiltredPreparedStatement;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.database.ThreadConnection;
import L2jFT.extensions.scripts.Scripts;
import L2jFT.game.instancemanager.DayNightSpawnManager;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Spawn;
import L2jFT.game.model.L2World;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.instances.L2MonsterInstance;
import L2jFT.game.templates.L2NpcTemplate;
import L2jFT.util.Rnd;

import java.sql.ResultSet;
import java.util.HashMap;
import java.util.HashSet;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Logger;

@SuppressWarnings( { "nls", "unqualified-field-access", "boxing" })
public class SpawnTable
{
	private static final Logger _log = Logger.getLogger(SpawnTable.class.getName());

	private static SpawnTable _instance;

	private HashMap<Integer, L2Spawn> _spawntable;
	private int _npcSpawnCount;
	private int _spawnCount;

	private int _highestId;
	private int _spawnId;

	public static SpawnTable getInstance()
	{
		if(_instance == null)
			new SpawnTable();
		return _instance;
	}

	private SpawnTable()
	{
		_instance = this;
		if(!Config.DONTLOADSPAWN)
			fillSpawnTable();
		else
			_log.info("Spawn Correctly Disabled");
	}

	public HashMap<Integer, L2Spawn> getSpawnTable()
	{
		return _spawntable;
	}

	private void fillSpawnTable()
	{
		L2World.deleteVisibleNpcSpawns();
		GmListTable.broadcastMessageToGMs("NPC Unspawn completed!");

		_spawntable = new HashMap<Integer, L2Spawn>();
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;

		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT count, npc_templateid, locx, locy, locz, heading, respawn_delay, loc_id, respawn_time, periodOfDay FROM `spawnlist` ORDER by npc_templateid");
			rset = statement.executeQuery();

			L2Spawn spawnDat;
			L2NpcTemplate template1;
			_npcSpawnCount = 0;
			_spawnCount = 0;
			_spawnId = 0;

			while(rset.next())
			{
				template1 = NpcTable.getTemplate(rset.getInt("npc_templateid"));
				if(template1 != null)
				{
					if(template1.type.equalsIgnoreCase("L2SiegeGuard"))
					{
						// Don't spawn
					}
					else if(template1.type.equalsIgnoreCase("L2RaidBoss"))
					{
						// Don't spawn raidboss
					}
					else if(template1.type.equalsIgnoreCase("L2Boss"))
					{
						// Don't spawn bosses
					}
					else if(Config.ALLOW_CLASS_MASTERS_LIST.isEmpty() && template1.type.equals("L2ClassMaster"))
					{
						// Dont' spawn class masters
					}
					else
					{
						spawnDat = new L2Spawn(template1);
						spawnDat.setId(_spawnId);
						_spawnId++;
						spawnDat.setAmount(rset.getInt("count") * (Config.ALT_DOUBLE_SPAWN ? 2 : 1));
						spawnDat.setLocx(rset.getInt("locx"));
						spawnDat.setLocy(rset.getInt("locy"));
						spawnDat.setLocz(rset.getInt("locz"));
						spawnDat.setHeading(rset.getInt("heading"));
						spawnDat.setRespawnDelay(rset.getInt("respawn_delay"));
						spawnDat.setRespawnTime(rset.getInt("respawn_time"));
						spawnDat.setLocation(rset.getInt("loc_id"));

						switch(rset.getInt("periodOfDay"))
						{
							case 0: // default 
								_npcSpawnCount += spawnDat.init();
								_spawntable.put(spawnDat.getId(), spawnDat);
								break;
							case 1: // Day 
								DayNightSpawnManager.getInstance().addDayMob(spawnDat);
								break;
							case 2: // Night 
								DayNightSpawnManager.getInstance().addNightMob(spawnDat);
								break;
						}
						_spawnCount++;

						if(_npcSpawnCount % 1000 == 0)
							_log.info("Spawned " + _npcSpawnCount + " npc");

						if(spawnDat.getId() > _highestId)
							_highestId = spawnDat.getId();
					}
				}
				else
					_log.warning("mob data for id:" + rset.getInt("npc_templateid") + " missing in npc table");
			}
			DayNightSpawnManager.getInstance().notifyChangeMode();
		}
		catch(Exception e1)
		{
			// problem with initializing spawn, go to next one
			_log.warning("spawn couldnt be initialized:" + e1);
			e1.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
		_log.config("SpawnTable: Loaded " + _spawnCount + " Npc Spawn Locations. Total NPCs: " + _npcSpawnCount);
		if(Config.DEBUG)
			_log.fine("Spawning completed, total number of NPCs in the world: " + _npcSpawnCount);
		loadInventory();
		Scripts.getInstance().callOnLoad();
	}

	public L2Spawn getTemplate(int id)
	{
		return _spawntable.get(id);
	}

	public void addNewSpawn(L2Spawn spawn, boolean storeInDb, L2Player editor)
	{
		if(Config.DONTLOADSPAWN)
			return;
		_highestId++;
		spawn.setId(_highestId);
		_spawntable.put(new Integer(spawn.getId()), spawn);

		if(!storeInDb)
			return;

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			if(editor == null)
				statement = con.prepareStatement("INSERT INTO `spawnlist` (location,count,npc_templateid,locx,locy,locz,heading,respawn_delay,loc_id, respawn_time) values(?,?,?,?,?,?,?,?,?,?)");
			else
			{
				statement = con.prepareStatement("INSERT INTO `spawnlist` (location,count,npc_templateid,locx,locy,locz,heading,respawn_delay,loc_id, respawn_time,last_editor) values(?,?,?,?,?,?,?,?,?,?,?)");
				statement.setString(11, editor.getName());
			}

			statement.setString(1, "");// spawn.getLocation());
			statement.setInt(2, spawn.getAmount());
			statement.setInt(3, spawn.getNpcid());
			statement.setInt(4, spawn.getLocx());
			statement.setInt(5, spawn.getLocy());
			statement.setInt(6, spawn.getLocz());
			statement.setInt(7, spawn.getHeading());
			statement.setInt(8, spawn.getRespawnDelay());
			statement.setInt(9, spawn.getLocation());
			statement.setInt(10, spawn.getRespawnTime());
			statement.execute();
		}
		catch(Exception e1)
		{
			// problem with storing spawn
			_log.warning("spawn couldnt be stored in db:" + e1);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	public void deleteSpawn(L2Spawn spawn, boolean updateDb)
	{
		if(_spawntable.remove(new Integer(spawn.getId())) != null && updateDb)
		{
			ThreadConnection con = null;
			FiltredPreparedStatement statement = null;
			try
			{
				con = L2DatabaseFactory.getInstance().getConnection();
				statement = con.prepareStatement("DELETE FROM `spawnlist` WHERE `npc_templateid`=? AND `locx`=? AND `locy`=? AND `locz`=? AND `loc_id`=?");
				statement.setInt(1, spawn.getNpcid());
				statement.setInt(2, spawn.getLocx());
				statement.setInt(3, spawn.getLocy());
				statement.setInt(4, spawn.getLocz());
				statement.setInt(5, spawn.getLocation());
				statement.execute();
			}
			catch(Exception e1)
			{
				// problem with deleting spawn
				_log.warning("spawn couldnt be deleted in db:" + e1);
			}
			finally
			{
				DatabaseUtils.closeDatabaseCS(con, statement);
			}
		}
	}

	//just wrapper
	public void reloadAll()
	{
		fillSpawnTable();
	}

	public void loadInventory()
	{
		ConcurrentHashMap<Integer, L2Object> world = L2World.getAllVisibleObjectsHashmap();
		int count = 0;
		HashSet<L2MonsterInstance> temp = null;
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT owner_id, object_id, item_id, count, enchant_level FROM items WHERE loc = 'MONSTER'");
			rset = statement.executeQuery();

			while(rset.next())
			{
				count++;
				temp = new HashSet<L2MonsterInstance>();
				int id = rset.getInt("owner_id");
				// TODO: снести этот бред, ибо так все вещи будут у одного моба, а должны быть у рандомного
				for(L2Object tmp : world.values())
					if(tmp.isMonster && ((L2MonsterInstance) tmp).getNpcId() == id)
						temp.add((L2MonsterInstance) tmp);
				if(temp.size() > 0)
				{
					L2MonsterInstance monster = (L2MonsterInstance) temp.toArray()[Rnd.get(temp.size())];
					L2ItemInstance item = L2ItemInstance.restoreFromDb(rset.getInt("object_id"));
					monster.giveItem(item, false);
				}
			}
		}
		catch(Exception e1)
		{
			e1.printStackTrace();
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}
		_log.info("Monsters inventory loaded, items: " + count);
	}
}
