package L2jFT.game.tables;

import L2jFT.database.DatabaseUtils;
import L2jFT.database.FiltredPreparedStatement;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.database.ThreadConnection;
import L2jFT.game.TradeController;
import L2jFT.game.model.L2Territory;
import L2jFT.game.model.L2World;

import java.sql.ResultSet;
import java.util.HashMap;
import java.util.logging.Logger;

public class TerritoryTable
{
	private static Logger _log = Logger.getLogger(TradeController.class.getName());
	private static final TerritoryTable _instance = new TerritoryTable();
	private static HashMap<String, L2Territory> _locations;

	public static TerritoryTable getInstance()
	{
		return _instance;
	}

	private TerritoryTable()
	{
		reloadData();
	}

	public L2Territory getLocation(int terr)
	{
		String name = "sql_terr_" + terr;
		L2Territory t = _locations.get(name);
		if(t == null)
			_log.warning("Error territory[49] " + name);
		return t;
	}

	public int[] getRandomPoint(int terr)
	{
		String name = "sql_terr_" + terr;
		L2Territory t = _locations.get(name);
		if(t == null)
		{
			_log.warning("Error territory[49] " + name);
			return new int[3];
		}
		return t.getRandomPoint();
	}

	public int getMinZ(int terr)
	{
		String name = "sql_terr_" + terr;
		L2Territory t = _locations.get(name);
		if(t == null)
		{
			_log.warning("Error territory[61] " + name);
			return 0;
		}
		return t.getZmin();
	}

	public int getMaxZ(int terr)
	{
		String name = "sql_terr_" + terr;
		L2Territory t = _locations.get(name);
		if(t == null)
		{
			_log.warning("Error territory[73] " + name);
			return 0;
		}
		return t.getZmax();
	}

	public void reloadData()
	{
		_locations = new HashMap<String, L2Territory>();
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT loc_id, loc_x, loc_y, loc_zmin, loc_zmax FROM `locations`");
			rset = statement.executeQuery();
			while(rset.next())
			{
				String terr = "sql_terr_" + rset.getString("loc_id");

				if(_locations.get(terr) == null)
				{
					L2Territory t = new L2Territory(terr);
					_locations.put(terr, t);
				}
				_locations.get(terr).add(rset.getInt("loc_x"), rset.getInt("loc_y"), rset.getInt("loc_zmin"), rset.getInt("loc_zmax"));
			}
		}
		catch(Exception e1)
		{
			//problem with initializing spawn, go to next one
			_log.warning("locations couldnt be initialized:" + e1);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}

		_log.config("TerritoryTable: Loaded " + _locations.size() + " locations");
	}

	public void registerZones()
	{
		int registered = 0;
		for(L2Territory terr : _locations.values())
			if(terr.isWorldTerritory())
			{
				L2World.addTerritory(terr);
				registered++;
			}

		_log.config("TerritoryTable: Added " + registered + " locations to L2World");
	}

	public HashMap<String, L2Territory> getLocations()
	{
		return _locations;
	}
}
