package L2jFT.game.taskmanager;

import javolution.util.FastMap;
import L2jFT.Config;
import L2jFT.game.ThreadPoolManager;
import L2jFT.game.model.L2Character;

import java.util.Map;
import java.util.NoSuchElementException;

public class DecayTaskManager
{
	Map<L2Character, Long> _decayTasks = new FastMap<L2Character, Long>().setShared(true);

	public static DecayTaskManager _instance;

	public DecayTaskManager()
	{
		ThreadPoolManager.getInstance().scheduleAiAtFixedRate(new DecayScheduler(), 10000, 5000);
	}

	public static DecayTaskManager getInstance()
	{
		if(_instance == null)
			_instance = new DecayTaskManager();

		return _instance;
	}

	public void addDecayTask(L2Character actor)
	{
		_decayTasks.put(actor, System.currentTimeMillis());
	}

	public void addDecayTask(L2Character actor, int interval)
	{
		_decayTasks.put(actor, System.currentTimeMillis() + interval);
	}

	public void cancelDecayTask(L2Character actor)
	{
		try
		{
			_decayTasks.remove(actor);
		}
		catch(NoSuchElementException e)
		{}
	}

	public class DecayScheduler implements Runnable
	{
		public void run()
		{
			try
			{
				Long current = System.currentTimeMillis();
				if(_decayTasks != null)
					for(L2Character actor : _decayTasks.keySet())
						if(current - _decayTasks.get(actor) > Math.min(8500, Config.CORPSE_MOB_TIME))
						{
							actor.onDecay();
							_decayTasks.remove(actor);
						}
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
		}
	}

	@Override
	public String toString()
	{
		StringBuffer sb = new StringBuffer("============= DecayTask Manager Report ============\n\r");
		sb.append("Tasks count: ").append(_decayTasks.size()).append("\n\r");
		sb.append("Tasks dump:\n\r");

		long current = System.currentTimeMillis();
		for(L2Character actor : _decayTasks.keySet())
		{
			sb.append("Class/Name: ").append(actor.getClass().getSimpleName()).append('/').append(actor.getName());
			sb.append(" decay timer: ").append(current - _decayTasks.get(actor)).append("\n\r");
		}

		return sb.toString();
	}
}
