package L2jFT.game.webserver;

import java.util.TreeMap;

import L2jFT.Config;
import L2jFT.game.Shutdown;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;
import L2jFT.game.tables.GmListTable;

/**
 * Это затычка для минимальной функциональности вебсервера
 * @author Abaddon
 */
abstract class PageParser
{
	public static String parse(String s)
	{
		// Количество игроков в мире
		if(s.contains("%online%"))
			s = s.replaceAll("%online%", String.valueOf(L2World.getAllPlayersCount()));

		// Количество ботов в мире
		if(s.contains("%bots%"))
			s = s.replaceAll("%bots%", String.valueOf(L2World.getAllBotCount()));

		// Количество ГМов
		if(s.contains("%gms%"))
			s = s.replaceAll("%gms%", String.valueOf(GmListTable.getAllGMs().size()));

		// Список имен игроков онлайн
		if(s.contains("%players_list%"))
		{
			String playersList = "";
			final int max_cols = 4; // Количество столбцов для списка онлайн игроков
			int cols = 0;
			playersList += "<table id=\"tbl-pl-list\">";

			TreeMap<String, L2Player> temp = new TreeMap<String, L2Player>(L2World.getAllPlayersHashmap());

			for(L2Player player : temp.values())
			{
				if(cols == 0)
					playersList += "<tr>";
				playersList += "<td>" + player.getName();
				// Маркируем ботов
				if(player.isBot())
					playersList += " <small><font color=\"red\">[Bot]</font></small>";
				playersList += "</td>";
				if(++cols == max_cols)
				{
					cols = 0;
					playersList += "</tr>";
				}
			}
			
			if(cols > 0)
				playersList += "</tr>";
			
			playersList += "</table>";
			s = s.replaceAll("%players_list%", playersList);
		}

		// Информация о текущей ревизии сервера
		if(s.contains("%revision%"))
			s = s.replaceAll("%revision%", Config.SERVER_VERSION);

		// Информация о состоянии памяти
		if(s.contains("%memory%"))
		{
			long freeMem = (Runtime.getRuntime().maxMemory() - Runtime.getRuntime().totalMemory() + Runtime.getRuntime().freeMemory()) / 1024 / 1024;
			long totalMem = Runtime.getRuntime().maxMemory() / 1024 / 1024;
			String memory = freeMem + " Mb / " + totalMem + " Mb";
			s = s.replaceAll("%memory%", memory);
		}

		// Время до рестарта
		int mtc = Shutdown.getInstance().get_seconds();
		if(mtc > 0)
		{
			String ret = "";
			int numDays = mtc / 86400;
			mtc -= numDays * 86400;
			int numHours = mtc / 3600;
			mtc -= numHours * 3600;
			int numMins = mtc / 60;
			mtc -= numMins * 60;
			int numSeconds = mtc;
			if(numDays > 0)
				ret += numDays + "d ";
			if(numHours > 0)
				ret += numHours + "h ";
			if(numMins > 0)
				ret += numMins + "m ";
			if(numSeconds > 0)
				ret += numSeconds + "s";
			s = s.replaceAll("%countdown%", ret);
		}
		else
			s = s.replaceAll("%countdown%", "Restart task not launched");

		return s;
	}
}