/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2, or (at your option) any later version. This
 * program is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 * http://www.gnu.org/copyleft/gpl.html
 */
package L2jFT.login;

import L2jFT.Config;
import L2jFT.Server;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.game.GameServer;
import L2jFT.status.Status;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.GeneralSecurityException;
import java.sql.SQLException;
import java.util.logging.LogManager;
import java.util.logging.Logger;
import java.net.InetAddress;

import com.l2jserver.mmocore.network.SelectorServerConfig;
import com.l2jserver.mmocore.network.SelectorThread;

/**
 *
 * @author KenM
 */
public class L2LoginServer
{
	public static final int PROTOCOL_REV = 0x0102;

	private static L2LoginServer _instance;
	private Logger _log = Logger.getLogger(L2LoginServer.class.getName());
	private GameServerListener _gameServerListener;
	private SelectorThread<L2LoginClient> _selectorThread;
	public static Status statusServer;
	public LoginController loginController;
	@SuppressWarnings("unused")
	private final Shutdown _shutdownHandler;

	//private GameServer gameServer;

	public static void main(String[] args)
	{
		_instance = new L2LoginServer();
	}

	public static L2LoginServer getInstance()
	{
		return _instance;
	}

	public L2LoginServer()
	{
		Server.SERVER_MODE = Server.MODE_LOGINSERVER;
		//      Local Constants
		final String LOG_FOLDER = "log"; // Name of folder for log file
		final String LOG_NAME = "./config/log.ini"; // Name of log file

		/*** Main ***/
		// Create log folder
		File logFolder = new File("./", LOG_FOLDER);
		logFolder.mkdir();

		// Create input stream for log file -- or store file data into memory
		InputStream is = null;
		try
		{
			is = new FileInputStream(new File(LOG_NAME));
			LogManager.getLogManager().readConfiguration(is);
			is.close();
		}
		catch(IOException e)
		{
			e.printStackTrace();
		}
		finally
		{
			try
			{
				if(is != null)
					is.close();
			}
			catch(IOException e)
			{
				e.printStackTrace();
			}
		}

		// Load Config
		Config.load();
		if(Config.COMBO_MODE)
		{
			Server.SERVER_MODE = Server.MODE_COMBOSERVER;
			Config.load();
		}

		// Prepare Database
		try
		{
			L2DatabaseFactory.getInstance();
		}
		catch(SQLException e)
		{
			_log.severe("FATAL: Failed initializing database. Reason: " + e.getMessage());
			if(Config.LOGIN_DEBUG)
				e.printStackTrace();
			System.exit(1);
		}

		try
		{
			LoginController.load();
		}
		catch(GeneralSecurityException e)
		{
			_log.severe("FATAL: Failed initializing LoginController. Reason: " + e.getMessage());
			if(Config.LOGIN_DEBUG)
				e.printStackTrace();
			System.exit(1);
		}

		try
		{
			GameServerTable.load();
		}
		catch(GeneralSecurityException e)
		{
			_log.severe("FATAL: Failed to load GameServerTable. Reason: " + e.getMessage());
			if(Config.LOGIN_DEBUG)
				e.printStackTrace();
			System.exit(1);
		}
		catch(SQLException e)
		{
			_log.severe("FATAL: Failed to load GameServerTable. Reason: " + e.getMessage());
			if(Config.LOGIN_DEBUG)
				e.printStackTrace();
			System.exit(1);
		}

		//this.loadBanFile();

		InetAddress ad = null;
		try
		{
			ad = InetAddress.getByName(Config.LOGIN_HOST);
		}
		catch(Exception e)
		{
			// Игнорим
		}

		SelectorServerConfig ssc = new SelectorServerConfig(ad, Config.PORT_LOGIN);
		L2LoginPacketHandler loginPacketHandler = new L2LoginPacketHandler();
		SelectorHelper sh = new SelectorHelper();
		try
		{
			_selectorThread = new SelectorThread<L2LoginClient>(ssc, loginPacketHandler, sh, sh);
			_selectorThread.setAcceptFilter(sh);
		}
		catch(IOException e)
		{
			_log.severe("FATAL: Failed to open Selector. Reason: " + e.getMessage());
			if(Config.LOGIN_DEBUG)
				e.printStackTrace();
			System.exit(1);
		}

		try
		{
			_gameServerListener = new GameServerListener();
			_gameServerListener.start();
			_log.info("Listening for GameServers on " + Config.GAME_SERVER_LOGIN_HOST + ":" + Config.GAME_SERVER_LOGIN_PORT);
		}
		catch(IOException e)
		{
			_log.severe("FATAL: Failed to start the Game Server Listener. Reason: " + e.getMessage());
			if(Config.LOGIN_DEBUG)
				e.printStackTrace();
			System.exit(1);
		}

		if(Config.IS_LOGIN_TELNET_ENABLED)
			try
			{
				statusServer = new Status(Server.MODE_LOGINSERVER);
				statusServer.start();
			}
			catch(IOException e)
			{
				_log.severe("Failed to start the Telnet Server. Reason: " + e.getMessage());
				if(Config.LOGIN_DEBUG)
					e.printStackTrace();
			}
		else
			_log.info("LoginServer Telnet server is currently disabled.");

		try
		{
			_selectorThread.openServerSocket();
		}
		catch(IOException e)
		{
			_log.severe("FATAL: Failed to open server socket. Reason: " + e.getMessage());
			if(Config.LOGIN_DEBUG)
				e.printStackTrace();
			System.exit(1);
		}
		_selectorThread.start();
		_log.info("Login Server ready on port " + Config.PORT_LOGIN);
		_log.info(IpManager.getInstance().getBannedCount() + " banned IPs defined");

		if(Config.COMBO_MODE)
			try
			{
				GameServer.checkFreePorts();

				if(Config.IS_TELNET_ENABLED)
				{
					Status _statusServer = new Status(Server.MODE_GAMESERVER);
					_statusServer.start();
				}
				else
					_log.info("GameServer Telnet server is currently disabled.");
				new GameServer();
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}

		_shutdownHandler = Shutdown.getInstance();
		Shutdown.getInstance().startShutdownH(Config.LRESTART_TIME, true);

		System.gc();
		// maxMemory is the upper limit the jvm can use, totalMemory the size of the current allocation pool, freeMemory the unused memory in the allocation pool
		long freeMem = (Runtime.getRuntime().maxMemory() - Runtime.getRuntime().totalMemory() + Runtime.getRuntime().freeMemory()) / 1024 / 1024;
		long totalMem = Runtime.getRuntime().maxMemory() / 1024 / 1024;
		_log.info("Free memory " + freeMem + " Mb of " + totalMem + " Mb");
	}

	public GameServerListener getGameServerListener()
	{
		return _gameServerListener;
	}

	public void shutdown(boolean restart)
	{
		Runtime.getRuntime().exit(restart ? 2 : 0);
	}

	public boolean unblockIp(String ipAddress)
	{
		return loginController.ipBlocked(ipAddress);
	}

	public boolean setPassword(String account, String password)
	{
		return loginController.setPassword(account, password);
	}
}
