package L2jFT.login.clientpackets;

import L2jFT.Config;
import L2jFT.login.GameServerTable.GameServerInfo;
import L2jFT.login.L2LoginClient;
import L2jFT.login.L2LoginClient.LoginClientState;
import L2jFT.login.LoginController;
import L2jFT.login.LoginController.State;
import L2jFT.login.LoginController.Status;
import L2jFT.login.serverpackets.AccountKicked.AccountKickedReason;
import L2jFT.login.serverpackets.LoginFail.LoginFailReason;
import L2jFT.login.serverpackets.AccountKicked;
import L2jFT.login.serverpackets.LoginOk;
import L2jFT.login.serverpackets.ServerList;

import javax.crypto.Cipher;
import java.security.GeneralSecurityException;

/**
 * Format: x
 * 0 (a leading null)
 * x: the rsa encrypted block with the login an password
 */
public class RequestAuthLogin extends L2LoginClientPacket
{
	private byte[] _raw = new byte[128];

	private String _user;
	private String _password;
	private int _ncotp;

	/**
	 * @return
	 */
	public String getPassword()
	{
		return _password;
	}

	/**
	 * @return
	 */
	public String getUser()
	{
		return _user;
	}

	public int getOneTimePassword()
	{
		return _ncotp;
	}

	@Override
	public boolean readImpl()
	{
		if(getAvaliableBytes() >= 128)
		{
			readB(_raw);
			return true;
		}
		return false;
	}

	@Override
	public void run()
	{
		byte[] decrypted = null;
		try
		{
			Cipher rsaCipher = Cipher.getInstance("RSA/ECB/nopadding");
			rsaCipher.init(Cipher.DECRYPT_MODE, getClient().getRSAPrivateKey());
			decrypted = rsaCipher.doFinal(_raw, 0x00, 0x80);
		}
		catch(GeneralSecurityException e)
		{
			e.printStackTrace();
			return;
		}

		_user = new String(decrypted, 0x5E, 14).trim();
		_user = _user.toLowerCase();
		_password = new String(decrypted, 0x6C, 16).trim();
		_ncotp = decrypted[0x7c];
		_ncotp |= decrypted[0x7d] << 8;
		_ncotp |= decrypted[0x7e] << 16;
		_ncotp |= decrypted[0x7f] << 24;

		LoginController lc = LoginController.getInstance();
		L2LoginClient client = getClient();

		Status status = lc.tryAuthLogin(_user, _password, getClient());
		if(status._state == State.VALID)
		{
			client.setAccount(_user);
			client.setState(LoginClientState.AUTHED_LOGIN);
			client.setSessionKey(lc.assignSessionKeyToClient(_user, client));
			client.setBonus(status._bonus);
			if(Config.SHOW_LICENCE)
				client.sendPacket(new LoginOk(getClient().getSessionKey()));
			else
				getClient().sendPacket(new ServerList(getClient()));
		}
		else
		{
			L2LoginClient oldClient;
			GameServerInfo gsi;
			if(status._state == State.WRONG)
				client.close(LoginFailReason.REASON_USER_OR_PASS_WRONG);
			else if(status._state == State.BANNED)
				client.close(new AccountKicked(AccountKickedReason.REASON_PERMANENTLY_BANNED));
			else if(status._state == State.IN_USE && (oldClient = lc.getAuthedClient(_user)) != null)
			{
				// kick the other client
				gsi = lc.getAccountOnGameServer(_user);
				if(gsi != null)
					gsi.getGameServerThread().kickPlayer(oldClient.getAccount());
				oldClient.close(LoginFailReason.REASON_ACCOUNT_IN_USE);
				if(LoginController.getInstance().isAccountInLoginServer(oldClient.getAccount()))
				{
					oldClient = LoginController.getInstance().removeAuthedLoginClient(oldClient.getAccount());
					oldClient.close(LoginFailReason.REASON_ACCOUNT_IN_USE);
				}
			}
			else if(status._state == State.IN_USE && (gsi = lc.getAccountOnGameServer(_user)) != null)
			{
				client.close(LoginFailReason.REASON_ACCOUNT_IN_USE);

				// kick from there
				if(gsi.isAuthed())
					gsi.getGameServerThread().kickPlayer(_user);
				if(LoginController.getInstance().isAccountInLoginServer(_user))
				{
					oldClient = LoginController.getInstance().removeAuthedLoginClient(_user);
					oldClient.close(LoginFailReason.REASON_ACCOUNT_IN_USE);
				}
			}
		}
	}
}