package L2jFT.status;

import bsh.EvalError;
import bsh.Interpreter;
import bsh.util.JConsole;
import L2jFT.Config;
import L2jFT.extensions.scripts.Scripts;
import L2jFT.game.Announcements;
import L2jFT.game.GameTimeController;
import L2jFT.game.LoginServerThread;
import L2jFT.game.Shutdown;
import L2jFT.game.ThreadPoolManager;
import L2jFT.game.network.clientpackets.Say2;
import L2jFT.game.idfactory.IdFactory;
import L2jFT.game.instancemanager.QuestManager;
import L2jFT.game.network.loginserverpackets.BanIPList.BannedIp;
import L2jFT.game.model.L2Multisell;
import L2jFT.game.model.L2Object;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2Summon;
import L2jFT.game.model.L2World;
import L2jFT.game.model.instances.L2DoorInstance;
import L2jFT.game.model.instances.L2ItemInstance;
import L2jFT.game.model.instances.L2MonsterInstance;
import L2jFT.game.model.instances.L2NpcInstance;
import L2jFT.game.model.quest.Quest;
import L2jFT.game.network.serverpackets.CreatureSay;
import L2jFT.game.tables.GmListTable;
import L2jFT.game.tables.NpcTable;
import L2jFT.game.tables.SkillTable;
import L2jFT.game.taskmanager.DecayTaskManager;
import org.apache.bsf.BSFException;
import org.apache.bsf.BSFManager;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.Socket;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.logging.Logger;

public class GameStatusThread extends Thread
{

	private static final Logger _log = Logger.getLogger(GameStatusThread.class.getName());

	public int log_chat = 0;
	public int log_tell = 0;
	public GameStatusThread next;

	private Socket _csocket;

	private PrintWriter _print;
	private BufferedReader _read;

	private int _uptime;

	private void telnetOutput(int type, String text)
	{
		if(type == 1)
			_log.fine("GSTELNET | " + text);
		else if(type == 2)
			_log.fine("GSTELNET | " + text);
		else if(type == 3)
			_log.fine(text);
		else if(type == 4)
			_log.fine(text);
		else
			_log.fine("GSTELNET | " + text);
	}

	private boolean isValidIP(Socket client)
	{
		boolean result = false;
		InetAddress ClientIP = client.getInetAddress();

		// convert IP to String, and compare with list
		String clientStringIP = ClientIP.getHostAddress();

		telnetOutput(1, "Connection from: " + clientStringIP);

		// read and loop thru list of IPs, compare with newIP
		try
		{
			Properties telnetSettings = new Properties();
			InputStream telnetIS = new FileInputStream(new File(Config.TELNET_FILE));
			telnetSettings.load(telnetIS);
			telnetIS.close();

			String HostList = telnetSettings.getProperty("ListOfHosts", "127.0.0.1,localhost");

			// compare
			String ipToCompare;
			for(String ip : HostList.split(","))
				if(!result)
				{
					ipToCompare = InetAddress.getByName(ip).getHostAddress();
					if(clientStringIP.equals(ipToCompare))
						result = true;
				}
		}
		catch(IOException e)
		{
			telnetOutput(1, "Error: " + e);
		}

		return result;
	}

	public GameStatusThread(Socket client, int uptime, String StatusPW) throws IOException
	{
		_csocket = client;
		_uptime = uptime;

		_print = new PrintWriter(_csocket.getOutputStream());
		_read = new BufferedReader(new InputStreamReader(_csocket.getInputStream()));

		if(isValidIP(client))
		{
			telnetOutput(1, client.getInetAddress().getHostAddress() + " accepted.");
			_print.println("Welcome To The L2 GS Telnet Session.");
			if(StatusPW != null && !StatusPW.equalsIgnoreCase(""))
			{
				_print.println("Please Insert Your Password!");
				_print.print("Password: ");
				_print.flush();
				String tmpLine = _read.readLine();
				if(tmpLine == null)
				{
					_print.println("Error.");
					_print.println("Disconnected...");
					_print.flush();
					_csocket.close();
				}
				else if(tmpLine.compareTo(StatusPW) != 0)
				{
					_print.println("Incorrect Password!");
					_print.println("Disconnected...");
					_print.flush();
					_csocket.close();
				}
				else
				{
					_print.println("Password Correct!");
					_print.println("[L2GS]");
					_print.print("");
					_print.flush();
					start();
				}
			}
			else
				start();
		}
		else
		{
			telnetOutput(1, "Connection attempt from " + client.getInetAddress().getHostAddress() + " rejected.");
			_csocket.close();
		}
	}

	@Override
	public void run()
	{
		next = Status.telnetlist;
		Status.telnetlist = this;

		String _usrCommand = "";
		try
		{
			while(_usrCommand.compareTo("quit") != 0 && _usrCommand.compareTo("exit") != 0)
			{
				_usrCommand = _read.readLine();
				if(_usrCommand == null)
				{
					_csocket.close();
					break;
				}

				if(_usrCommand.equals("status") || _usrCommand.equals("s"))
				{
					int playerCount, objectCount;
					int max = LoginServerThread.getInstance().getMaxPlayer();

					playerCount = L2World.getAllPlayers().size();
					objectCount = L2World.getAllObjectsCount();

					int itemCount = 0;
					int itemVoidCount = 0;
					int monsterCount = 0;
					int minionCount = 0;
					int minionsGroupCount = 0;
					int npcCount = 0;
					int guardCount = 0;
					int charCount = 0;
					int pcCount = 0;
					int doorCount = 0;
					int summonCount = 0;
					int AICount = 0;
					int extendedAICount = 0;
					int summonAICount = 0;

					for(L2Object obj : L2World.getAllObjects().values())
					{
						if(obj.isCharacter)
						{
							charCount++;
							if(obj.hasAI())
							{
								AICount++;
								if(obj.isNpc)
									extendedAICount++;
								else if(obj instanceof L2Summon)
									summonAICount++;
							}
						}
						else if(obj instanceof L2ItemInstance)
							if(((L2ItemInstance) obj).getLocation() == L2ItemInstance.ItemLocation.VOID)
								itemVoidCount++;
							else
								itemCount++;

						if(obj.isMonster)
						{
							monsterCount++;
							minionCount += ((L2MonsterInstance) obj).getTotalSpawnedMinionsInstances();
							minionsGroupCount += ((L2MonsterInstance) obj).getTotalSpawnedMinionsGroups();
						}

						if(obj instanceof L2NpcInstance)
							npcCount++;
						else if(obj.isPlayer)
							pcCount++;
						else if(obj instanceof L2Summon)
							summonCount++;
						else if(obj instanceof L2DoorInstance)
							doorCount++;
					}
					_print.println("Server Status: ");
					_print.println(" +.............. Players: " + playerCount + "/" + max);
					_print.println(" +.............. Summons: " + summonCount);
					_print.println(" +............. Monsters: " + monsterCount);
					_print.println(" +.............. Minions: " + minionCount);
					_print.println(" +........ Minion Groups: " + minionsGroupCount);
					_print.println(" +.................. Npc: " + npcCount);
					_print.println(" +................... GM: " + GetOnlineGMS());
					_print.println(" +.............. Objects: " + objectCount);
					_print.println(" +............... All AI: " + AICount);
					_print.println(" +........... ExtendedAI: " + extendedAICount);
					_print.println(" +........... L2SummonAI: " + summonAICount);
					_print.println(" +..........Ground Items: " + itemVoidCount);
					_print.println(" +...........Owned Items: " + itemCount);
					_print.println(" +........ L2CastleGuard: " + guardCount);
					_print.println(" +............... L2Door: " + doorCount);
					_print.println(" +............... L2Char: " + charCount);
					_print.println(" +. InGame Time / Uptime: " + GameTime() + " / " + GetUptime(_uptime));
					_print.println(" +.. Shutdown_sec / mode: " + Shutdown.getInstance().get_seconds() + " / " + Shutdown.getInstance().get_mode());
					_print.println(" +.. Log_chat / log_tell: " + log_chat + " / " + log_tell);
					_print.println(" +....... Active Regions: " + L2World.getActiveRegions().size());
					_print.println(" +.............. Threads: " + Thread.activeCount());
					_print.println(" +............. RAM Used: " + (Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory()) / 1024 / 1024);
					_print.flush();
				}
				else if(_usrCommand.equals("database"))
				{
					_print.println("Database Usage Status: ");
					_print.println("+... Players operation: ");
					_print.println("-->  Update characters: " + L2World.getUpdatePlayerBase());
					_print.println("+..... Items operation: ");
					_print.println("-->      Insert: " + L2World.getInsertItemCount());
					_print.println("-->      Delete: " + L2World.getDeleteItemCount());
					_print.println("-->      Update: " + L2World.getUpdateItemCount());
					_print.println("--> Lazy Update: " + L2World.getLazyUpdateItem());
					_print.println("+... Lazy items update: " + Config.LAZY_ITEM_UPDATE);
					_print.println("+... Released ObjectId: " + IdFactory.getInstance().getReleasedCount());
					_print.flush();
				}
				else if(_usrCommand.equals("lazyitems"))
				{
					Config.LAZY_ITEM_UPDATE = !Config.LAZY_ITEM_UPDATE;

					_print.println("Lazy items update set to: " + Config.LAZY_ITEM_UPDATE);
					_print.flush();
				}
				else if(_usrCommand.equals("performance") || _usrCommand.equals("p"))
				{
					for(String line : ThreadPoolManager.getInstance().getStats())
						_print.println(line);
					_print.flush();
				}
				else if(_usrCommand.equals("p packets"))
				{
					_print.println(ThreadPoolManager.getInstance().getPacketStats());
					_print.flush();
				}
				else if(_usrCommand.equals("p general"))
				{
					_print.println(ThreadPoolManager.getInstance().getGeneralPoolStats());
					_print.flush();
				}
				else if(_usrCommand.equals("p ai"))
				{
					_print.println(ThreadPoolManager.getInstance().getAIPoolStats());
					_print.flush();
				}
				else if(_usrCommand.startsWith("announce ") || _usrCommand.startsWith("! "))
					try
					{
						_usrCommand = _usrCommand.substring(9);
						Announcements.getInstance().announceToAll(_usrCommand);
						_print.println("Announcement Sent!");
					}
					catch(StringIndexOutOfBoundsException e)
					{
						_print.println("Please Enter Some Text To Announce!");
					}
				else if(_usrCommand.startsWith("msg "))
					try
					{
						String val = _usrCommand.substring(4);
						StringTokenizer st = new StringTokenizer(val);
						String name = st.nextToken();
						String message = val.substring(name.length() + 1);
						L2Player reciever = L2World.getPlayer(name);
						CreatureSay cs = new CreatureSay(0, Say2.TELL, "Server Admin", message);
						if(reciever != null)
						{
							reciever.sendPacket(cs);
							_print.println("Telnet Priv->" + name + ": " + message);
							_print.println("Message Sent!");
						}
						else
							_print.println("Unable To Find Username: " + name);
					}
					catch(StringIndexOutOfBoundsException e)
					{
						_print.println("Please Enter Some Text!");
					}
				else if(_usrCommand.startsWith("gmchat "))
					try
					{
						_usrCommand = _usrCommand.substring(7);
						CreatureSay cs = new CreatureSay(0, 9, "Telnet GM Broadcast from " + _csocket.getInetAddress().getHostAddress(), _usrCommand);
						GmListTable.broadcastToGMs(cs);
						_print.println("Your Message Has Been Sent To " + GetOnlineGMS() + " GM(s).");
					}
					catch(StringIndexOutOfBoundsException e)
					{
						_print.println("Please Enter Some Text To Announce!");
					}
				else if(_usrCommand.equals("gmlist"))
				{
					int igm = 0;
					String gmList = "";

					for(L2Player player : GmListTable.getAllGMs())
					{
						gmList = gmList + ", " + player.getName();
						igm++;
					}
					_print.println("There are currently " + igm + " GM(s) online...");
					if(!gmList.equals(""))
						_print.println(gmList);
				}
				/*else if(_usrCommand.startsWith("unblock"))
				 {
				 try
				 {
				 _usrCommand = _usrCommand.substring(8);
				 if(LoginServer.getInstance().unblockIp(_usrCommand))
				 {
				 _log.warning("IP removed via TELNET by host: " + _csocket.getInetAddress().getHostAddress());
				 _print.println("The IP " + _usrCommand + " has been removed from the hack protection list!");
				 }
				 else
				 {
				 _print.println("IP " + _usrCommand + " not found in hack protection list...");
				 }
				 //TODO: with packet
				 }
				 catch (StringIndexOutOfBoundsException e)
				 {
				 _print.println("Please Enter the IP to Unblock!");
				 }
				 }*/
				else if(_usrCommand.startsWith("whois "))
					try
					{
						_usrCommand = _usrCommand.substring(6);
						L2Player player = L2World.getPlayer(_usrCommand);
						if(player != null)
						{
							_print.println("Name:" + player.getName());
							_print.println("Account:" + player.getAccountName());
							_print.println("IP:" + player.getNetConnection().getIpAddr());
						}
						else
							_print.println("No such player online!");
					}
					catch(StringIndexOutOfBoundsException e)
					{
						_print.println("Please enter player name to kick");
					}
				else if(_usrCommand.equals("baniplist"))
					try
					{
						ArrayList<BannedIp> baniplist;
						LoginServerThread.getInstance().RequestBanIPList();
						while(LoginServerThread.getInstance().isRequested())
						{}
						baniplist = LoginServerThread.getInstance().getBanIPList();
						if(baniplist != null && baniplist.size() > 0)
						{
							_print.println("Ban IP List:");
							for(BannedIp temp : baniplist)
								_print.println("Ip:" + temp.ip + " banned by " + temp.admin);
							_print.flush();
						}
						else
							_print.println("No banned ips.");
					}
					catch(StringIndexOutOfBoundsException e)
					{
						_print.println("Please enter ip to ban");
					}
				else if(_usrCommand.startsWith("banip "))
					try
					{
						_usrCommand = _usrCommand.substring(6);
						LoginServerThread.getInstance().sendBanIP(_usrCommand, "Telnet: " + _csocket.getInetAddress().getHostAddress());
					}
					catch(StringIndexOutOfBoundsException e)
					{
						_print.println("Please enter ip to unban");
					}
				else if(_usrCommand.startsWith("unbanip "))
					try
					{
						_usrCommand = _usrCommand.substring(8);
						LoginServerThread.getInstance().sendUnbanIP(_usrCommand);
					}
					catch(StringIndexOutOfBoundsException e)
					{
						_print.println("Please enter ip to unban");
					}
				else if(_usrCommand.startsWith("kick "))
					try
					{
						_usrCommand = _usrCommand.substring(5);
						L2Player player = L2World.getPlayer(_usrCommand);
						if(player != null)
						{
							player.sendMessage("You are kicked by gm", "");
							player.kick(false);
							_print.println("Player kicked");
						}
					}
					catch(StringIndexOutOfBoundsException e)
					{
						_print.println("Please enter player name to kick");
					}
				else if(_usrCommand.startsWith("enemy "))
					try
					{
						_usrCommand = _usrCommand.substring(6);
						L2Player player = L2World.getPlayer(_usrCommand);
						if(player != null)
							for(L2NpcInstance enemy : player.getEnemyList())
								_print.println("--> " + enemy.getName() + " <--");
						else
							_print.println("Player is offline.");
					}
					catch(StringIndexOutOfBoundsException e)
					{
						_print.println("Please enter player name.");
					}
				else if(_usrCommand.startsWith("quest_reload "))
					try
					{
						Object result;
						File file;
						String questName, questDir;

						_usrCommand = _usrCommand.substring(13);
						questName = _usrCommand; // directory is default command value
						questDir = "quests"; // set default quest directory

						file = new File(Config.DATAPACK_ROOT, "data/jscript/" + questDir + "/_" + questName + "/__init__.py");
						if(!file.exists())
							try
							{
								int questId = Integer.parseInt(_usrCommand);

								Quest q = QuestManager.getInstance().getQuest(questId);
								if(q != null)
								{
									questName = q.getName();
									questDir = q.getDescr().toLowerCase();
									file = new File(Config.DATAPACK_ROOT, "data/jscript/" + questDir + "/_" + questName + "/__init__.py");
									if(!file.exists())
									{
										questDir = "quests";
										file = new File(Config.DATAPACK_ROOT, "data/jscript/" + questDir + "/_" + questName + "/__init__.py");
									}
								}
								else
									_print.println("Quest with ID " + questId + " not found");
							}
							catch(NumberFormatException e)
							{}
						if(file.exists())
						{
							try
							{
								BSFManager bsf = new BSFManager();
								bsf.exec("jython", "admin", 0, 0, "from data.jscript." + questDir + " import _" + questName + ";reload(_" + questName + ")");
								result = questName + " reload complete.";
							}
							catch(BSFException e)
							{
								result = e;
							}
							_print.println("Jython: " + result);
						}
						else
							_print.println("Wrong path for quest " + questName);
					}
					catch(StringIndexOutOfBoundsException e)
					{
						_print.println("Please enter Quest name.");
					}
				else if(_usrCommand.startsWith("debug"))
				{
					StringTokenizer st = new StringTokenizer(_usrCommand.substring(6));
					try
					{
						String dbg = st.nextToken();

						if(dbg.equals("decay"))
							_print.print(DecayTaskManager.getInstance().toString());
					}
					catch(Exception e)
					{}
				}
				else if(_usrCommand.startsWith("skill_reload "))
				{
					SkillTable.getInstance().reload();
					_print.println("Skills reloaded.");
				}
				else if(_usrCommand.startsWith("npc_reload "))
				{
					NpcTable.getInstance().reloadAllNpc();
					_print.println("NPC's reloaded.");
				}
				else if(_usrCommand.startsWith("multisell_reload "))
				{
					L2Multisell.getInstance().reload();
					_print.println("Multisell reloaded.");
				}
				else if(_usrCommand.startsWith("gmaccess_reload "))
				{
					Config.loadGMAccess();
					for(L2Player player : L2World.getAllPlayers())
						if(!Config.EVERYBODY_HAS_ADMIN_RIGHTS)
							player.setPlayerAccess(Config.gmlist.get(player.getObjectId()));
						else
							player.setPlayerAccess(Config.gmlist.get(new Integer(0)));
					_print.println("GMAccess reloaded.");
				}
				else if(_usrCommand.startsWith("scripts_reload") || _usrCommand.startsWith("sreload"))
				{
					Scripts.getInstance().reload();
					_print.println("Scripts reloaded.");
				}
				else if(_usrCommand.startsWith("shutdown "))
					try
					{
						if(_usrCommand.substring(9).equals("NOW"))
						{
							_log.warning("Shutting down via TELNET by host: " + _csocket.getInetAddress().getHostAddress());
							_print.println("Shutting down...");
							System.exit(-1);
						}

						int val = Integer.parseInt(_usrCommand.substring(9));
						Shutdown.getInstance().startTelnetShutdown(_csocket.getInetAddress().getHostAddress(), val, false);
						_print.println("Server Will Shutdown In " + val + " Seconds!");
						_print.println("Type \"abort\" To Abort Shutdown!");
					}
					catch(StringIndexOutOfBoundsException e)
					{
						_print.println("Please Enter * amount of seconds to shutdown!");
					}
					catch(Exception NumberFormatException)
					{
						_print.println("Numbers Only!");
					}
				else if(_usrCommand.startsWith("restart "))
					try
					{
						int val = Integer.parseInt(_usrCommand.substring(8));
						Shutdown.getInstance().startTelnetShutdown(_csocket.getInetAddress().getHostAddress(), val, true);
						_print.println("Server Will Restart In " + val + " Seconds!");
						_print.println("Type \"abort\" To Abort Restart!");
					}
					catch(StringIndexOutOfBoundsException e)
					{
						_print.println("Please Enter * amount of seconds to restart!");
					}
					catch(Exception NumberFormatException)
					{
						_print.println("Numbers Only!");
					}
				else if(_usrCommand.startsWith("jconsole "))
				{
					try
					{
						JConsole bshConsole = new JConsole();
						Interpreter bsh = new Interpreter(bshConsole);
						bsh.eval("bsh.system.shutdownOnExit=false");
						bsh.eval("desktop()");
						Thread t = new Thread(bsh);
						t.start();
					}
					catch(EvalError e)
					{
						System.err.println("Couldn't start java console: " + e);
					}
					_print.println("OK! - try to start java console");
				}
				else if(_usrCommand.equals("abort") || _usrCommand.equals("a"))
				{
					Shutdown.getInstance().Telnetabort(_csocket.getInetAddress().getHostAddress());
					_print.println("OK! - Shutdown/Restart Aborted.");
				}
				else if(_usrCommand.startsWith("log_chat"))
				{
					if(log_chat == 1)
					{
						log_chat = 0;
						_print.println("Log_chat is turned off");
					}
					else
					{
						log_chat = 1;
						_print.println("Log_chat is turned on.");
					}
				}
				else if(_usrCommand.equals("log_tell"))
				{
					if(log_tell == 1)
					{
						log_tell = 0;
						_print.println("Log_tell is turned off.");
					}
					else
					{
						log_tell = 1;
						_print.println("Log_tell is turned on.");
					}

				}
				else if(_usrCommand.startsWith("tell"))
				{
					GameStatusThread temp = Status.telnetlist;
					int tcounter = 1;
					if(temp == null)
						write("telnetlist is null!");
					while(temp != null)
					{
						temp.write("Telnet" + tcounter + ":" + _usrCommand.substring(4));
						temp = temp.next;
						tcounter++;
					}
				}
				else if(_usrCommand.startsWith("gc"))
				{
					try
					{
						System.gc();
						Thread.sleep(1000L);
						System.gc();
						Thread.sleep(1000L);
						System.gc();
					}
					catch(Exception e)
					{}
					_print.println("OK! - garbage collector called.");
				}
				else if(_usrCommand.equals("ver") || _usrCommand.equals("version"))
					_print.println("L2J Rebellion server. Running " + Config.SERVER_VERSION + "/" + Config.SERVER_BUILD_DATE);
				else if(_usrCommand.equals("quit") || _usrCommand.equals("exit"))
				{ /* Do Nothing :p - Just here to save us from the "Command Not Understood" Text */}
				else if(_usrCommand.length() == 0)
				{ /* Do Nothing Again - Same reason as the quit part */}

				_print.print("");
				_print.flush();
			}
			if(!_csocket.isClosed())
			{
				_print.println("Bye Bye!");
				_print.flush();
				_csocket.close();
			}
			telnetOutput(1, "Connection from " + _csocket.getInetAddress().getHostAddress() + " was closed by client.");
		}
		catch(IOException e)
		{
			e.printStackTrace();
		}
		//      begin clean telnetlist
		if(this == Status.telnetlist)
			Status.telnetlist = next;
		else
		{
			GameStatusThread temp = Status.telnetlist;
			while(temp.next != this)
				temp = temp.next;
			temp.next = next;
		}
		// finished clean telnetlist

	}

	private int GetOnlineGMS()
	{
		return GmListTable.getAllGMs().size();
	}

	private String GetUptime(int _uptime)
	{
		int uptime = (int) System.currentTimeMillis() - _uptime;
		uptime = uptime / 1000;
		int h = uptime / 3600;
		int m = (uptime - h * 3600) / 60;
		int s = uptime - h * 3600 - m * 60;
		return h + "hrs " + m + "mins " + s + "secs";
	}

	public void write(String _text)
	{
		_print.println(_text);
		_print.flush();
	}

	private String GameTime()
	{
		int t = GameTimeController.getInstance().getGameTime();
		int h = t / 60;
		int m = t % 60;
		SimpleDateFormat format = new SimpleDateFormat("H:mm");
		Calendar cal = Calendar.getInstance();
		cal.set(Calendar.HOUR_OF_DAY, h);
		cal.set(Calendar.MINUTE, m);
		return format.format(cal.getTime());
	}
}
