package L2jFT.util;

import L2jFT.database.DatabaseUtils;
import L2jFT.database.FiltredPreparedStatement;
import L2jFT.database.L2DatabaseFactory;
import L2jFT.database.ThreadConnection;
import L2jFT.game.model.L2Player;
import L2jFT.game.model.L2World;

import java.sql.ResultSet;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.Logger;

public final class AutoBan
{
	private static Logger _log = Logger.getLogger(AutoBan.class.getName());

	public static boolean isBanned(int ObjectId)
	{
		boolean res = false;

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();

			statement = con.prepareStatement("SELECT MAX(endban) AS endban FROM bans WHERE obj_Id=?");
			statement.setInt(1, ObjectId);
			rset = statement.executeQuery();

			while(rset.next())
			{
				Long endban = rset.getLong("endban") * 1000;
				res = endban > System.currentTimeMillis();
			}
		}
		catch(Exception e)
		{
			_log.warning("Could not restore ban data: " + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}

		return res;
	}

	public static int GetIDbyName(String name)
	{
		int res = 0;

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		ResultSet rset = null;
		try
		{
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("SELECT obj_Id FROM characters WHERE char_name=?");
			statement.setString(1, name);
			rset = statement.executeQuery();

			while(rset.next())
				res = rset.getInt("obj_Id");
		}
		catch(Exception e)
		{
			_log.warning("Could not find char: " + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCSR(con, statement, rset);
		}

		return res;
	}

	public static void Banned(L2Player actor, int period, String msg, String GM)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			Long srok = System.currentTimeMillis() + period * 24 * 3600 * 1000;
			String date = (new SimpleDateFormat("yy.MM.dd H:mm:ss")).format(new Date());
			String enddate = (new SimpleDateFormat("yy.MM.dd H:mm:ss")).format(new Date(srok));
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("INSERT INTO bans (account_name, obj_id, baned, unban, reason, GM, endban) VALUES(?,?,?,?,?,?,?)");
			statement.setString(1, actor.getAccountName());
			statement.setInt(2, actor.getObjectId());
			statement.setString(3, date);
			statement.setString(4, enddate);
			statement.setString(5, msg);
			statement.setString(6, GM);
			statement.setLong(7, srok / 1000);
			statement.execute();
		}
		catch(Exception e)
		{
			_log.warning("could not store bans data:" + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	//offline
	public static boolean Banned(String actor, int acc_level, int period, String msg, String GM)
	{
		boolean res;
		int obj_id = GetIDbyName(actor);
		res = obj_id > 0;
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		if(res)
			try
			{
				con = L2DatabaseFactory.getInstance().getConnection();
				statement = con.prepareStatement("UPDATE characters SET accesslevel=? WHERE obj_Id=?");
				statement.setInt(1, acc_level);
				statement.setInt(2, obj_id);
				statement.execute();
				DatabaseUtils.closeStatement(statement);

				Long srok = System.currentTimeMillis() + period * 24 * 3600 * 1000;
				String date = (new SimpleDateFormat("yy.MM.dd H:mm:ss")).format(new Date());
				String enddate = (new SimpleDateFormat("yy.MM.dd H:mm:ss")).format(new Date(srok));

				statement = con.prepareStatement("INSERT INTO bans (obj_id, baned, unban, reason, GM, endban) VALUES(?,?,?,?,?,?)");
				statement.setInt(1, obj_id);
				statement.setString(2, date);
				statement.setString(3, enddate);
				statement.setString(4, msg);
				statement.setString(5, GM);
				statement.setLong(6, srok / 1000);
				statement.execute();
			}
			catch(Exception e)
			{
				e.printStackTrace();
				_log.warning("could not store bans data:" + e);
				res = false;
			}
			finally
			{
				DatabaseUtils.closeDatabaseCS(con, statement);
			}
		return res;
	}

	public static void Karma(L2Player actor, int karma, String msg, String GM)
	{
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		try
		{
			String date = (new SimpleDateFormat("yy.MM.dd H:mm:ss")).format(new Date());
			msg = "Add karma(" + karma + ") " + msg;
			con = L2DatabaseFactory.getInstance().getConnection();
			statement = con.prepareStatement("INSERT INTO bans (account_name, obj_id, baned, reason, GM) VALUES(?,?,?,?,?)");
			statement.setString(1, actor.getAccountName());
			statement.setInt(2, actor.getObjectId());
			statement.setString(3, date);
			statement.setString(4, msg);
			statement.setString(5, GM);
			statement.execute();
		}
		catch(Exception e)
		{
			_log.warning("could not store bans data:" + e);
		}
		finally
		{
			DatabaseUtils.closeDatabaseCS(con, statement);
		}
	}

	//offline
	public static boolean Karma(String actor, int karma, String msg, String GM)
	{
		boolean res;
		int obj_id = GetIDbyName(actor);
		res = obj_id > 0;
		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;

		if(res)
			try
			{
				con = L2DatabaseFactory.getInstance().getConnection();

				statement = con.prepareStatement("update characters set karma=karma + ? where obj_Id=?");
				statement.setInt(1, karma);
				statement.setInt(2, obj_id);
				statement.execute();
				DatabaseUtils.closeDatabaseCS(con, statement);

				String date = (new SimpleDateFormat("yy.MM.dd H:mm:ss")).format(new Date());
				msg = "Add karma(" + karma + ") " + msg;
				statement = con.prepareStatement("INSERT INTO bans (obj_id, baned, reason, GM) VALUES(?,?,?,?)");
				statement.setInt(1, obj_id);
				statement.setString(2, date);
				statement.setString(3, msg);
				statement.setString(4, GM);
				statement.execute();
			}
			catch(Exception e)
			{
				_log.warning("could not store bans data:" + e);
				res = false;
			}
			finally
			{
				DatabaseUtils.closeDatabaseCS(con, statement);
			}
		return res;
	}

	public static void Banned(L2Player actor, int period, String msg)
	{
		Banned(actor, period, msg, "AutoBan");
	}

	public static boolean ChatBan(String actor, int period, String msg, String GM)
	{
		boolean res = true;
		long NoChannel = period * 60000;
		L2Player plyr = L2World.getPlayer(actor);

		ThreadConnection con = null;
		FiltredPreparedStatement statement = null;
		if(plyr != null)
		{
			plyr.sendMessage("You chat banned by GM " + GM + " for " + period + " min.", "Ваш чат забанил ГМ " + GM + " на " + period + " минут.");
			plyr.updateNoChannel(NoChannel);
		}
		else
			try
			{
				con = L2DatabaseFactory.getInstance().getConnection();
				statement = con.prepareStatement("UPDATE characters SET nochannel = ? WHERE obj_Id=?");
				statement.setLong(1, NoChannel > 0 ? NoChannel / 1000 : NoChannel);
				statement.setInt(2, GetIDbyName(actor));
				statement.executeUpdate();
			}
			catch(Exception e)
			{
				res = false;
				_log.warning("Could not activate nochannel:" + e);
			}
			finally
			{
				DatabaseUtils.closeDatabaseCS(con, statement);
			}

		if(res)
			try
			{
				con = L2DatabaseFactory.getInstance().getConnection();

				String date = (new SimpleDateFormat("yy.MM.dd H:mm:ss")).format(new Date(System.currentTimeMillis()));
				String enddate = (new SimpleDateFormat("yy.MM.dd H:mm:ss")).format(new Date(System.currentTimeMillis() + NoChannel));

				statement = con.prepareStatement("INSERT INTO bans (obj_id, baned, unban, reason, GM) VALUES(?,?,?,?,?)");
				statement.setInt(1, GetIDbyName(actor));
				statement.setString(2, date);
				statement.setString(3, enddate);
				statement.setString(4, msg);
				statement.setString(5, GM);
				statement.execute();
			}
			catch(Exception e)
			{
				_log.warning("could not store bans data:" + e);
				res = false;
			}
			finally
			{
				DatabaseUtils.closeDatabaseCS(con, statement);
			}
		return res;
	}
}
